#ifndef __MCCONIC_DEFS_H
#define __MCCONIC_DEFS_H

#include <MC/MC_Fit.H>
#include <MC/MC_Point.H>
#include <MC/MC_Transform.H>

class MCConic : public MCFit {

protected:

    MCTransform m_Transform;;
    Array< double > A;

    int CalculateFit(const Vertex &in_vertex);
    
public:

    /**@name Polynomial Fit Data */
    //@{
    MCConic(const PMeshLite &in_mesh, const int in_nRings );
    ///
    virtual ~MCConic();
    ///
    double FunctionValue(const R3Pt &in_point);
    ///
    int Evaluate(const R2Pt &in_uvPoint, R3Pt &out_xyzPoint);
    ///
    int Evaluate(const R2Pt &in_uvPoint, R3Vec &out_normal, 
                 R3Vec &out_dU, R3Vec &out_dV, 
                 R3Vec &out_dUU, R3Vec &out_dUV, R3Vec &out_dVV );
    ///
    int Curvature(const Vertex &in_vertex, MCPoint &cpt);
    ///
    R3Vec getBasis1(const R3Vec &N) const;
    ///
    R3Vec getGradient(const R3Pt &in_point) const;
    ///
    double getGradientMag(const R3Pt &in_point) const;
    ///
    int ProjectLocalX(R3Pt &point) const;
    ///
    int ProjectLocalY(R3Pt &point) const;
    ///
    int ProjectLocalZ(R3Pt &point) const;
    ///
    int EvaluateN(const R3Pt X, R3Pt &Xnew) const;
    ///
    int Project(const R3Pt &X, const R3Vec &NN, R3Pt &Xnew ) const;
    ///
    void MakeMesh(const int nu, const int nv,
                  const double scalefactor, const int flip,
                  PMeshLite &out_mesh) const;
    ///
    void dump(FILE *out);
    ///
    //@}
};

#endif
