#ifndef __MCCurvatureMap_DEFS_H
#define __MCCurvatureMap_DEFS_H

#include <WINSystemDefines.H>
#include <utils/Pm_MeshLite.H>
#include <MC/MC_Data.H>
#include <MC/MC_Distribution.H>

/** MCCurvatureMap 
Array of curvature values representing curvature as a function of distance
from a point
*/

class MCCurvatureMap {

protected:
    int m_iNumVertices;
    int m_iNumData;
    int m_iMaxRings;
    SArray<MCDistribution> m_aMap;
    SArray<int> m_aRingCount;
    char *m_sMeshDescription;
    //SArray<char *>m_dataDescriptions;

public:
    /**@name Mesh Curvature Data */
    //@{

    MCCurvatureMap();
    ///
    virtual ~MCCurvatureMap();
    ///
    void set_description( const char *desc );
	///
	const char *Description() const { return m_sMeshDescription; }
    ///
    //void set_dataDescription(const int index, const char *desc);
    ///
    void Clear();
    ///
    int Init( const int in_nv, const int in_nd, const int in_maxRings );
    ///
	int Copy(MCCurvatureMap &cmap);
	///
	int Extend(MCCurvatureMap &cmap, const int size);
	///
	int Truncate(MCCurvatureMap &cmap, const int size);
	///
	int ExtractSubset(MCCurvatureMap &cmap, Array<int> indices);
	///
    int ExtractFeature(Array<int> &features, const int index,
		               MCCurvatureMap &cmap);
	///
	int NumVertices(void) const { return m_iNumVertices; }
    ///
    int NumData(void) const { return m_iNumData; }
    ///
    int MaxRings(void) const { return m_iMaxRings; }
    ///
    int NumRings(const int index) const;
    ///
    int Create(const char *meshfile, 
		       MCData &data, 
			   const int maxRings);
    ///
    int Create2(const char *meshfile, 
		        MCData &data, 
			    const int maxRings);
    ///
    int Create(const char *meshfile, 
		       MCData &data, 
			   const double maxRadius);
    ///
    int Create2(const char *meshfile, 
		        MCData &data, 
			    const double maxRadius);
    ///
    int Create(const char *meshfile, 
		       const char *fanDataFile, 
			   const int maxRings, const int isBinary = 1, const int verbose = 0);
    ///
    int WriteBinary(const char *outfile) const;
    ///
    int Write(const char *outfile) const;
    ///
    int ReadBinary(const char *infile);
    ///
    int Read(const char *infile);
    ///
	const MCDistribution& Distribution(const int index) const;
	///
	int Replace(const int index, const MCDistribution &dist);
	///
    int Value(MCDistribution::MC_DataType dataType, const int index, 
              const int iRing, double &value) const;
	///
	int InterpolateBary(const int i1, const double b1, 
		                const int i2, const double b2, 
		                const int i3, const double b3, 
		                double &kg, double &kbar);
    ///
	int Smooth(const int nTerm);
    ///
	int RadiusStats(double &Rmin, double &Rmax, double &Rmean, double &Rstddev, double &Rmedian);
	///
	int Filter(const double sigma);
    ///
	int Filter(const int npts);
	///
	int ComputeFeatureDistributions(const int type, 
		                            Array<int> &features, 
								    Array<MCDistribution *> &signatures);
	///
	int ComputeListOfDistributions(Array< Array<int> > &points, 
								   Array<MCDistribution *> &signatures);
	///
	int ComputeFeatureDeviations(const double radius,
		                         Array<int> &features,
								 Array<MCDistribution *> &signatures,
								 Array<double> &deviation);
	///
	int ComputeDeviations(const double radius,
		                  Array< Array<int> > &indexLists,
						  Array<MCDistribution *> &signatures,
						  Array<double> &deviation);
	///
    int ComputeScale(ArrayMesh &mesh, MCCurvatureMap &cmap, 
					 Array<int> &corrFace, Array<double> &corrB1, 
					 Array<double> &corrB2, Array<double> &corrB3, 
					 const double radius,
		             MCDistribution::MC_MapMetric method,
					 const int areaWt, Array<double> &scale, 
					 const double scaleLimit = 5.0, const int maxIter = 100);
	///
    int ComputeScale(ArrayMesh &mesh, MCCurvatureMap &cmap, 
					 Array<int> &corrFace, Array<double> &corrB1, 
					 Array<double> &corrB2, Array<double> &corrB3, 
					 const double radius, const int index,
		             MCDistribution::MC_MapMetric method,
					 const int areaWt, double &scale,
					 const double scaleLimit = 5.0, const int maxIter = 100);
	///
    void dump(FILE *out) const;
    //@}

};

#endif
