#ifndef __MCDISCRETEOPERATOR_DEFS_H
#define __MCDISCRETEOPERATOR_DEFS_H

#include <MC/MC_Fit.H>
#include <MC/MC_Point.H>
#include <utils/Pm_MeshLite.H>

/**  \class MCDiscreteOperator MC_DiscreteOperator.H MC/MC_DiscreteOperator.H 
\brief Calculate all of the discrete methods

  Desbrun's (DCO), Desbrun's with a fix for obtuse angles (ModDCO), 
  Goldfeather and Interrante's cubic fit with points and normals, assumes normals given (ANCe),
  Goldfeather and Interrante's cibic fit with points and normals, calculates normals first (ANC),
  Taubin's Integral Eigen value (finding the eigen values of the shape operator) IEV,
  Hameiri and Shimshoni's fix of Taubin's method (HST),
  Differential of the normal Gauss map (DGM)

*/

class MCDiscreteOperator : public MCFit {

public:
    typedef enum { DCO=0, ModDCO, ANCe, ANC, IEV, HST, DGM } MC_DiscreteType;

protected:

    int CalculateFit(const Vertex &in_vertex);
    MC_DiscreteType m_DiscreteType;
    
public:

    /**@name DiscreteOperator Fit Data */
    //@{
    MCDiscreteOperator( const PMeshLite &in_mesh,
                        const MC_DiscreteType &in_discreteType );
    ///
    virtual ~MCDiscreteOperator();
    ///
    int Curvature(const Vertex &in_vertex, MCPoint &out_cpt);
    ///
    int calcDCO(const Vertex &in_vertex, MCPoint &out_cpt);
	///
	int calcDCOdirections(const R3Pt &in_P, const R3Vec &in_N,
						  const Array<Vertex> &in_aov,
						  R3Vec &out_e1, R3Vec &out_e2);
    ///
    int calcANC(const Vertex &in_vertex, MCPoint &out_point);
    ///
    int calcIEV(const Vertex &in_vertex, MCPoint &out_point);
    ///
    int calcHST(const Vertex &in_vertex, MCPoint &out_point);
    ///
	int calcDGM(const Vertex &in_vertex, MCPoint &out_point);
	///
    //const Vertex& GetVertex();
    ///
    void dump(FILE *out);
    ///
    //@}
};

#endif
