#ifndef __MCDISTANCEGRID_DEFS_H
#define __MCDISTANCEGRID_DEFS_H

#include <WINSystemDefines.H>
#include <utils/Pm_MeshLite.H>
#include <MC/MC_CurvatureMap.H>
#include <MC/MC_Distribution.H>
#include <MC/tdgFanData.H>

class MCDistanceGrid {

public:
    typedef enum { CMAP=0, FANDATA, UNKNOWN } MC_GridType;

protected:
    char *m_sMeshTitle;
    char *m_sMetricTitle;

	
	MC_GridType m_GridType;
	Array<int> points;
    SArray< SArray<double> > grid;
    MCCurvatureMap *m_pCmap;
	SArray<tdgFanData> *m_pFanData;
    int m_iRings;
	int m_iInteg;
	int m_iAreaWt;

protected:
    /**@name Mesh Curvature Data */
    //@{
    MCDistanceGrid( const MCDistanceGrid &in_dist );
    ///
    MCDistanceGrid &operator=(const MCDistanceGrid &in_dist);
    //@}

public:
    /**@name Mesh Curvature Data */
    //@{

    MCDistanceGrid();
    ///
    virtual ~MCDistanceGrid();
	///
	const MCCurvatureMap* GetMapPointer() const { return m_pCmap; }
    ///
    const MCCurvatureMap& GetMap() const { return *m_pCmap; }
    ///
    const char * MeshTitle() const { return m_sMeshTitle; }
    ///
    void MeshTitle(const char *title);
    ///
    const char * MetricTitle() const { return m_sMetricTitle; }
    ///
    void MetricTitle(const char *title);
    ///
    int Create( MCCurvatureMap &cmap, const Array<int> &pts,
                const MCDistribution::MC_MapMetric mapMetric,
                const int nRings, const int integ = 0,
				const int areaWt = 0);
    ///
    int Create( SArray<tdgFanData> &fdat, const Array<int> &pts,
                const MCDistribution::MC_MapMetric mapMetric,
                const int nRings, const int integ = 0,
				const int areaWt = 0);
    ///
    int Update(const MCDistribution::MC_MapMetric metric,
               const int nRing, const int integ,
			   const int areaWt = 0);
    ///
    int Size(void) const { return points.num(); }
    ///
    int NumRings(void) const { return m_iRings; }
	///
	int GetPointIndex(const int index) const;
    ///
    double Value(const int i, const int j) const;
    ///
    double Max() const;
    ///
    double Min() const;
    /// 
	int MeanVariance(const int i0, const int i1,
		             const int j0, const int j1,
					 double &mean, double &variance,
					 int &count);
	///
	int tTest(const char *filename, const int groupSize);
	///
    void dump(FILE *out) const;
    //@}
};

#endif
