#ifndef __MCDISTRIBUTION_DEFS_H
#define __MCDISTRIBUTION_DEFS_H

#include <WINSystemDefines.H>
#include <utils/Pm_MeshLite.H>
#include <MC/MC_Data.H>

/** MCDistribution 
Array of curvature values representing curvature as a function of distance
from a point
*/

class MCCurvatureMap;
class tdgCurve2D;
class ArrayMesh;

class MCDistribution {

protected:
    //PMeshLite *m_pMesh;
    char *m_sDescription;
    //MCData *m_pMeshCurvature;

    int m_iPointIndex;
    int m_iNumRings;
    Array< double > Kg;
    Array< double > Kbar;
    Array< double > KgMax;
    Array< double > KbarMax;
    Array< double > KgMin;
    Array< double > KbarMin;
    Array< double > StdDevKg;
    Array< double > StdDevKbar;
    Array< double > A;
    Array< double > R;

protected:
    /**@name Mesh Curvature Data */
    //@{
    MCDistribution( const MCDistribution &in_dist );
    ///
    MCDistribution &operator=(const MCDistribution &in_dist);
    //@}

public:
    /**@name Mesh Curvature Data */
    //@{

    typedef enum { NONE=-1, RMS_GAUSS=0, RMS_MEAN, RMS_BOTH, RMS_MOD,
                            LOG_GAUSS,   LOG_MEAN, LOG_BOTH, LOG_MOD,
                            RMS_SDGAUSS, RMS_SDMEAN, 
							RMS_GAUSS_MIN, RMS_GAUSS_MAX, RMS_GAUSS_MINMAX,
							LOG_GAUSS_MIN, LOG_GAUSS_MAX, LOG_GAUSS_MINMAX,
                            RMS_MEAN_MIN, RMS_MEAN_MAX, RMS_MEAN_MINMAX,
							LOG_MEAN_MIN, LOG_MEAN_MAX, LOG_MEAN_MINMAX,
                            RMS_BOTH_MIN, RMS_BOTH_MAX, RMS_BOTH_MINMAX,
							LOG_BOTH_MIN, LOG_BOTH_MAX, LOG_BOTH_MINMAX,
                            RMS_MOD_MIN, RMS_MOD_MAX, RMS_MOD_MINMAX,
							LOG_MOD_MIN, LOG_MOD_MAX, LOG_MOD_MINMAX,
                            NUM_METHODS } MC_MapMetric;

    typedef enum { AREA=0, RADIUS, KG, KBAR, KGSD, KBARSD, 
                   KGMIN, KGMAX, KBARMIN, KBARMAX } MC_DataType;

    MCDistribution();
    ///
    virtual ~MCDistribution();
    ///
    void set_description( const char *desc );
    ///
    const char * Description() const { return m_sDescription; }
    ///
    //const PMeshLite* Mesh() const { return m_pMesh; }
    ///
    //const MCData* Data() const { return m_pMeshCurvature; }
    ///
    void Clear();
    ///
	void SetPointIndex(const int index) { m_iPointIndex = index; }
	///
    //int Init( const PMeshLite *mesh, MCData &data, 
    //          const Array< double > &area,
    //          int pointIndex, int maxRings);
    int Create( MCData &data, const Array< double > &area,
                const int pointIndex, const int maxRings);
    ///
	int Create( MCData &data, ArrayMesh &amesh,
                const Array< double > &area,
                const int pointIndex, const int maxRings);
	///
	int Create( MCData &data, ArrayMesh &amesh,
                const Array< double > &area,
                const int pointIndex, const int maxRings,
				int *&queue, int *&counts, 
				int *&flags, int *&list,
				int *&kp, int *&ki, int *&in, int *&ij,
				int *&ji, int *&jk, float *&xc);
	///
	int Create( MCData &data, ArrayMesh &amesh,
                const Array< double > &area,
                const int pointIndex, const double maxRadius,
				int *&queue, int *&counts, 
				int *&flags, int *&list,
				int *&kp, int *&ki, int *&in, int *&ij,
				int *&ji, int *&jk, float *&xc);
	///
    int CreateFromFan(const Array< int > &spokeSize,
                      const Array<R3Pt> &xyz,
                      const Array<double> &K1,
                      const Array<double> &K2,
                      const Array<int> &value,
                      const int pointIndex, 
                      const int maxRings);
    ///
    MCDistribution &operator+=(const MCDistribution &in_dist);
	///
	MCDistribution& operator*=(const double scale);
	///
	MCDistribution& operator/=(const double scale);
	///
	MCDistribution& Add(MCDistribution &in_dist,
						const int trim,
						double &Rmin);
	///
	int Copy(const MCDistribution &in_dist);
	///
	int Average(const MCCurvatureMap &cmap,
				const Array<int> &points);
	///
	int WeightedAverage(const MCCurvatureMap &cmap,
				        const Array<int> &points, 
						const Array<double> &weights);
	///
	MCDistribution& Add(const MCDistribution &in_dist, 
		                const int trim, double &Rmin);
	///
    int Size(void) const { return m_iNumRings; }
    ///
	double Radius(void) const { return R[m_iNumRings-1]; }
	///
    int getPoint(int iRing, double &out_Kg, double &out_Kbar, 
                 double &out_StdDevKg, double &out_StdDevKbar, 
                 double &out_A, double &out_R, 
                 double &out_KgMax, double &out_KbarMax, 
                 double &out_KgMin, double &out_KbarMin) const;
	///
    int AddPoint(const double in_Kg,       const double in_Kbar, 
                 const double in_StdDevKg, const double in_StdDevKbar, 
                 const double in_A,        const double in_R, 
                 const double in_KgMax,    const double in_KgMin, 
                 const double in_KbarMax,  const double in_KbarMin);
    ///
    int AddPoint(const int index, const int iRing, 
                 double *dbuf, const int nData );
    ///
    static int ProcessPoint(const int metric, const int statistic,
		                    const int isLog, const int isAreaWt,
							const double kg, const double kbar, 
                            const double kgSD, const double kbarSD, 
                            const double area, const double radius, 
                            const double kgMax, const double kbarMax, 
                            const double kgMin, const double kbarMin,
							double &gc, double &mc, double &gx, double &mx, double &r);
	///
	static void InterpolatePoint(const int extra,
		                         const double r0, const double gc0, const double mc0, 
		                                          const double gx0, const double mx0,
								 const double r1, const double gc1, const double mc1, 
		                                          const double gx1, const double mx1,
								 const double r,  double &gc, double &mc, 
		                                          double &gx, double &mx);
	///
	static double Integrate(const int extra,   const int metric,
		                    const double r0,   const double r1, 
		                    const double gcA0, const double mcA0, 
		                    const double gxA0, const double mxA0,
						    const double gcA1, const double mcA1, 
		                    const double gxA1, const double mxA1,
						    const double gcB0, const double mcB0, 
		                    const double gxB0, const double mxB0,
						    const double gcB1, const double mcB1, 
		                    const double gxB1, const double mxB1);
	///
	int Integrate(const int extra,
		          const double r0,   const double r1, 
		          const double gcA0, const double mcA0, 
		          const double gxA0, const double mxA0,
				  const double gcA1, const double mcA1, 
		          const double gxA1, const double mxA1,
			      double &kGauss,    double &kMean) const;
	///
	int Difference(const int extra,
		           const double r0,    const double r1, 
		           const double gcA0,  const double mcA0, 
		           const double gxA0,  const double mxA0,
				   const double gcA1,  const double mcA1, 
		           const double gxA1,  const double mxA1,
				   const double gcB0,  const double mcB0, 
		           const double gxB0,  const double mxB0,
				   const double gcB1,  const double mcB1, 
		           const double gxB1,  const double mxB1,
				   double &deltaGauss, double &deltaMean) const;
	///
    int Value(MC_DataType dataType, 
              const int iRing, double &value ) const;
    ///
    int WriteMap(FILE *out) const;
    ///
    static const char * Metric(MC_MapMetric method);
    ///
	int CreateCurve(MCDistribution::MC_DataType indep, 
					MCDistribution::MC_DataType dep,
					const int maxPoints, const int id,
					tdgCurve2D &curve ) const;
	///
	int MaxPos(MCDistribution::MC_DataType type, double &r) const;
	///
	int MaxNeg(MCDistribution::MC_DataType type, double &r) const;
	///
	int MaxMonoPosInc(MCDistribution::MC_DataType type, double &r) const;
	///
	int MaxMonoPosDec(MCDistribution::MC_DataType type, double &r) const;
	///
	int MaxMonoNegInc(MCDistribution::MC_DataType type, double &r) const;
	///
	int MaxMonoNegDec(MCDistribution::MC_DataType type, double &r) const;
	///
	int MaxMonotonic(int &iDirG, int &iNg, double &dRg, double &dAg,
					 int &iDirM, int &iNm, double &dRm, double &dAm) const;
	///
	int FindZeroGauss(double &r, int &dir) const;
	///
    int FindZeroMean(double &r, int &dir) const;
	///
    void dump(FILE *out) const;
    ///
    double Compare(const MCDistribution &dist, MC_MapMetric method, 
                   const int maxRings, const int areaWt) const;
	///
    //double AreaWtCompare(const MCDistribution &dist, MC_MapMetric method, 
    //                     const int maxRings) const;
	///
    double CompareIntegral(const MCDistribution &dist, MC_MapMetric method, 
                           const int maxRings, const int areaWt) const;
	///
    double Compare(const MCDistribution &dist, MC_MapMetric method, 
                   const double Rcomp, const int areaWt) const;
	///
    double Compare(const MCDistribution &dist, MC_MapMetric method, 
                   const double Rcomp, const double scale,
				   const int areaWt) const;
	///
	int Delta(const MCDistribution &dist, 
              MC_MapMetric method, 
              const double Rcomp,
              const int areaWt,
			  double &deltaKgauss,
			  double &deltaKmean) const;
	///
	int Integrate(const double Rcomp,
                  MC_MapMetric method, 
                  const int areaWt,
				  double &sumGauss,
				  double &sumMean) const;
	///
	int GetScale(const MCDistribution &dist, 
                 MC_MapMetric method, 
                 const int nRing,
                 const int areaWt,
				 const int type,
				 float &alphaG, float &alphaM, 
				 float &offsetG, float &offsetM) const;
	///
    double CompareScale(const MCDistribution &dist, 
                        MC_MapMetric method, 
                        const int maxRings,
                        const int areaWt,
				        const int scaleType,
						const int scaleX,
						const int offset,
						float &alphaG, float &alphaM, 
				        float &offsetG, float &offsetM) const;
	///
    int ComputeScale(const MCDistribution &dist, 
		             MC_MapMetric method, 
                     const double Rcomp, 
					 const int areaWt,
					 double &scale,
					 const double scaleLimit = 5.0,
					 const int maxIter = 100) const;
    ///
    static MC_MapMetric Compose(const int metric, const int stat,
		                        const int function);
	///
	static void Decompose(const MC_MapMetric mm, int &metric,
		                  int &statistic, int &function);
	///
	int Smooth(const int nTerm);
	///
	int Filter(const double sigma);
	///
	int Flatness(const double r,
		         double &min_km, double &max_km,
				 double &min_kg, double &max_kg,
				 double &avg_km, double &avg_kg) const;
	///
	int Analyze(const double &r, 
		        int &pos_km,      int &pos_kg, 
		        int &dir_km,      int &dir_kg, 
		        double &km,       double &kg,
		        double &dist1_km, double &dist1_kg, 
				double &dist2_km, double &dist2_kg, 
				double &distM_km, double &distM_kg, 
				double &area1_km, double &area1_kg, 
				double &area2_km, double &area2_kg, 
				double &areaM_km, double &areaM_kg, 
				double &areaR_km, double &areaR_kg, 
				double &abs_R_km, double &abs_R_kg, 
				double &delta_km) const; 
	//@}

};

#endif
