#ifndef __MCFIT_DEFS_H
#define __MCFIT_DEFS_H

#include <WINSystemDefines.H>
#include <utils/Pm_MeshLite.H>
#include <MC/MC_MeshSubset.H>
#include <MC/MC_Param.H>
#include <MC/MC_Point.H>

/**  \class MCFit MC_Fit.H MC/MC_Fit.H 
\brief Base class for actually calculating the curvatures

  four basic types, polynomial fit to a disk in the mesh, 
  radial basis function fit over parameterized disk,
  conic fit over mesh piece (centered at the origin)
  discrete calculations (e.g., Desbrun's, Taubins)
*/
class MCFit {
public:
    typedef enum { POLYNOMIAL=0, RBF, CONIC, DISCRETE } MC_FitType;


protected:
    const PMeshLite &m_pMesh;
    int m_iRings;
    const MCParam::MC_ParamType m_ParamType;
    const MC_FitType m_FitType;
    MCParam m_Param;
    
    R3Vec m_VecOffset;
    double m_dDia;
    WINbool valid;

public:
    /**@name Fit Curvature Data */
    //@{
    MCFit(const PMeshLite  & in_mesh, 
          const MC_FitType & in_fitType,
          const MCParam::MC_ParamType &in_paramType,
          const int in_nRings = 1);
    ///
    MCFit(MCFit &in_fit);
    ///
    MCFit();
    ///
    virtual ~MCFit();
    ///
    const R3Vec& Nref() const { return m_Param.Nref(); }
    ///
    const R3Vec& Offset() { return m_VecOffset; }
    ///
    const MCParam& Param() { return m_Param; }
    ///
    int nRings() { return m_iRings; }
    ///
    virtual int Curvature(const Vertex &in_vertex, MCPoint &out_cpt) = 0;
    /// Calls Curvature2
    virtual void Curvature(const R3Vec &Ds, const R3Vec &Dt, 
                           const R3Vec &Dss, const R3Vec &Dst, const R3Vec &Dtt,
                           double &out_dArea, MCPoint &out_data ) const;
    ///
	void Curvature2(const R3Vec &Ds, const R3Vec &Dt, 
                    const R3Vec &Dss, const R3Vec &Dst, const R3Vec &Dtt,
                    double &out_dArea, MCPoint &out_data ) const;
    ///
    virtual int CalculateFit(const Vertex &in_vertex) = 0;
    ///
    virtual void dump(FILE *out) = 0;
    //@}

};

#endif
