#ifndef __MCFLATTENEDSURFACE_DEFS_H
#define __MCFLATTENEDSURFACE_DEFS_H

#include <WINSystemDefines.H>
#include <MC/MC_Point.H>
#include <MC/MC_Data.H>
#include <utils/Pm_MeshLite.H>

/** MCFlattenedSurface 
Flattened mesh with curvature data
*/
class MCFlattenedSurface {
protected:
    MCData &m_pCurvature;
    PMeshLite m_FlatMesh;

public:
    /**@name Mesh Curvature Data */
    //@{
    MCFlattenedSurface(MCData &in_data);
    ///
	MCFlattenedSurface(MCData &in_data, MCParam::MC_ParamType in_type,
					   const double in_lambda, const double in_mu);
	///
    virtual ~MCFlattenedSurface();
    ///
    PMeshLite& Mesh() { return m_FlatMesh; }
    ///
    //PMeshLite* Mesh3D();
    ///
    MCData& Data();
    ///
    void Init(const int method = -1);
    ///
    void Init(PMeshLite &in_flatMesh, MCData &in_data);
    ///
    int AreaCorrect(int method = 0, double BBfactor = 1.2, 
                    double SZfactor = 1.0);
    ///
    R3Pt ComputeCentroid(void);
    ///
    void AngleBins( int method, const R3Pt &centroid, int nBins, 
                               Array<double> &Ksum );
    ///
    void BinPoints( int method, int nBins, 
                    int iBin1, int iBin2, int iBin3, int iBin4,
                    R3Pt &P1, R3Pt &P2, R3Pt &P3, R3Pt &P4 );
    ///
    void AlignUV(const R3Pt &center, const R3Pt &newCenter, 
                 const double angle, const int flip);
    ///
    int OrientUV( MCFlattenedSurface &ref, int metric, 
                  int nBins, int sep, FILE *out = NULL );
    ///
    void CurvatureAlign( MCFlattenedSurface &ref, 
                         int metric, int nPts, int nSep,
                         int flip = 0, int flex = 1 );
    ///
    void WriteMeshData( char *filename );
    ///
    void dump(FILE *out);
    ///
    void test(void);
    //@}

};

#endif
