#ifndef __MCNRING_DEFS_H
#define __MCNRING_DEFS_H

#include <WINSystemDefines.H>
#include <MC/MC_Point.H>
#include <utils/Pm_MeshLite.H>

/** MCMeshSubset 
Vertices and faces for an N-ring of a mesh
*/
class MCMeshSubset {
public:
    typedef enum { VORONOI=0, BARYCENTRIC, MIXED, MODIFIED } MC_AreaType;
    typedef enum { AVERAGE=0, AREA, ANGLE, COTANGENT, CENTROID, MAX99 } MC_NormalType;

protected:
    const PMeshLite &m_pMesh;
    Vertex m_Vertex;
    int m_iRings;
    R3Vec m_Nref;
    Array< Vertex > m_aopVs;
    Array< Face > m_aopFs;
    Array< int > m_BoundaryVertexIndices;
    Array< Edge > m_BoundaryEdges;
    WINbool valid;
    Array< Face > m_aof;
    

public:
    /**@name Fit Curvature Data */
    //@{
    MCMeshSubset(const PMeshLite &in_mesh, 
                 const int in_nRings);
    ///
    MCMeshSubset(MCMeshSubset &in_ring);
    ///
    MCMeshSubset();
    ///
    virtual ~MCMeshSubset();
    ///
    const PMeshLite& Mesh() { return m_pMesh; }
    ///
    int ExtractMesh(PMeshLite &new_mesh);
    ///
    void SetBoundary();
    ///
    int ParameterizeBoundary(Array<int> &out_Order, 
                             Array<R2Pt> &out_aopUVs );
    ///
    int BoundaryExtremePoints(int &index1, int &index2 );
    ///
    int ExtremePoints(int &index1, int &index2 );
    ///
    WINbool IsBoundary(Vertex &v);
    ///
    int nRings() { return m_iRings; }
    ///
    const R3Vec& Nref() const { return m_Nref; }
    ///
    void Create(const Vertex &in_vertex, const int in_nRings = 1);
    ///
    void CreateFull();
    ///
    Array<Vertex>& Vertices() { return m_aopVs; }
    ///
    Array<Face>& Faces() { return m_aopFs; }
    ///
    Array<Face>& VertexFaces(const Vertex &v);
    ///
    double VertexArea(const Vertex &v, MC_AreaType in_AreaType);
    ///
    int FaceAngles(const Face &in_face, const Vertex &in_vertex,
                   int &out_ThetaId, double &out_Theta, double &out_t, 
                   int &out_AlphaId, double &out_Alpha, double &out_a, 
                   int &out_BetaId, double &out_Beta, double &out_b ) const;
    ///
    int AlphaBeta( const Face &in_face, const Vertex &in_vertex,
                   double &out_theta, double &out_alpha, double &out_beta,
                   double &d1, double &d2, double &d3 ) const;
    ///
    int AlphaBeta( const Vertex &in_vertex, const Edge &in_edge,
                   double &out_theta1, double &out_theta2, 
                   double &out_alpha, double &out_beta,
                   double &out_delta, double &out_gamma ) const;
    ///
    double CotAlphaBeta( const Vertex &v1, const Vertex &v2 ) const;
    ///
    double CotGammaDelta( const Vertex &v1, const Vertex &v2 ) const;
    ///
    void FaceAngles( const Face &in_face, Array<double> &out_angles ) const;
    ///
    int CotAngles( const Face &in_face, const Vertex &in_vertex,
                   int &out_ThetaId, double &out_CotTheta, double &out_t, 
                   int &out_AlphaId, double &out_CotAlpha, double &out_a, 
                   int &out_BetaId, double &out_CotBeta, double &out_b ) const;
	///
	int Centroid(const Face &in_face, const Vertex &in_vertex,
                 R3Pt &out_Centroid, double &out_CentroidDist) const;
	///
	int CentroidSum(const Vertex &in_vertex, double &out_Sum) const;
	///
    int MaxWeight(const Face &in_face, const Vertex &in_vertex,
                  double &out_Weight) const;
	///
    inline double Distance( const R3Pt &p1, const R3Pt &p2 ) const { return Length( p1 - p2 ); }
    ///
    inline double Distance( const Vertex &v1, const Vertex &v2 ) const { return Length( v1->GetLoc() - v2->GetLoc() ); }
    ///
    ///
    int CalculateNormal(const Vertex &in_vertex,
                        const MC_NormalType &in_type,
                        R3Vec &out_normal ) const;
    ///
    void FaceNormal(const Face &in_face, R3Vec &out_normal) const;
    ///
    double FaceArea(const Face &in_face) const;
    ///
    void dump(FILE *out);
    //@}

};

#endif
