#ifndef MCMESHUTILS_HEADERS_H
#define MCMESHUTILS_HEADERS_H

#include <WINSystemDefines.H>
#include <utils/Pm_MeshLite.H>
#include <MC/MC_Data.H>
#include <direct.h>
#include <stdio.h>
#include <errno.h>

extern char g_strDataHome[];

inline char *MyDataHome(const char *in_str, char *in_dir = NULL) 
{
    static char str[512];
    static char dir[512];

    // Make the directory name
    if ( in_dir == NULL || strcmp(in_dir,"") == 0 ) {
        strncpy(dir, g_strDataHome, 512);
    } else {
        sprintf(dir, "%s%s/", g_strDataHome, in_dir);
    }

    // Create the directory if it does not exist
    int ier = _mkdir(dir);
    if( ier == -1 && errno != EEXIST ) {
        printf("Error %d in _mkdir\n", errno);
    }

    // Make the full file name
    sprintf(str, "%s%s", dir, in_str);
    return str;
}

void MCMeshRange(const PMeshLite &in_mesh, R3Pt &Xmin, R3Pt &Xmax );

void MCFixMeshNormals(PMeshLite &in_mesh);

int MCCheckNormalConsistency(PMeshLite &in_mesh);

void MCFlipMeshNormals(PMeshLite &in_mesh);

void MCFindNormalSign(const PMeshLite &in_mesh, 
				      int &IPos, int &INeg, 
				      int &BPos, int &BNeg);

void MCFindNormalSign(const PMeshLite &in_mesh, int &Isign, int &Bsign );

int MCAnalyzeMesh(const PMeshLite &in_mesh, 
				  int &out_iFaceOrient, 
				  int &out_iNormOrient);

int MCFindPointId(const PMeshLite &in_mesh, const int dir, const double tol,
                  int &out_indexId );

int MCFlagBoundedRegion(const PMeshLite &in_mesh, const Vertex &in_vertex, 
                        const int dir, const double depth, 
                        Array<Face> &out_aopFs );

int MCTrimMesh(const PMeshLite &in_mesh, const int trimType,
               const double trimFraction, Array<Face> &out_aopFs );

int MCWriteMeshData(const char *filename, const PMeshLite &in_mesh,
                    const MCData &in_data, Array<R2Pt> *in_uv = NULL);

int MCWriteMeshData(const char *filename, const PMeshLite &in_mesh,
                    const MCData &in_data, const PMeshLite &in_UVmesh);

int MCWriteMeshWithScalar(const char *filename, const PMeshLite &mesh,
                        const Array<double> &scalar1,
                        const Array<double> &scalar2 );

int MCWriteMeshWithScalar(const char *filename, const PMeshLite &mesh,
                          const Array<int> &scalar1,
                          const Array<int> &scalar2 );

int MCWriteGarlandMesh(const char *filename, const PMeshLite &in_mesh);

int MCScalarStats(const Array<int> &scalar, int &nValues, 
				  Array<int> &Values, Array<int> &groupSizes);

int MCStretchStats(const PMeshLite &in_mesh1, const PMeshLite &in_mesh2,
				   double &out_area1, double &out_area2,
				   double &out_scaleFactor, double &out_stretchEfficiency);

int MCReadMeshWithScalar(const char *filename, PMeshLite &mesh,
                         Array<double> &scalar1,
                         Array<double> &scalar2 );

int MCReadMeshWithScalar(const char *filename, PMeshLite &mesh,
                         Array<int> &scalar1,
                         Array<int> &scalar2 );

int MCWriteGridData(const char *filename, int ni, int nj, 
                    Array<R2Pt> grid, Array<double> data);

void InterpolateToGrid( Array <R2Pt> &grid,
                        Array <double> &data,
                        int ni, int nj, 
                        double umin, double umax,
                        double vmin, double vmax,
                        PMeshLite &mesh,
                        Array <Face> &faces,
                        Array <double> &faceData,
                        double max_error );

void FindCell( Array <R2Pt> &uv, int ni, int nj, double x,  double y, 
               int &iCell, int &jCell, double &s, double &t, double &error );

void FindCell( double x,  double y,  int ni, int nj,
               double umin, double umax, double vmin, double vmax,
               int &iCell, int &jCell, double &s, double &t );

int GridInterp( int iCell, int jCell, double s,  double t, 
                double umin, double umax, double vmin, double vmax,
                int ni, int nj, double &x, double &y );

int GridInterp( const Array <R2Pt> &grid, int ni, int nj,
                int iCell, int jCell, double s,  double t, 
                double &x, double &y );

void BoundingBox( const Array <Vertex> &aov, 
                  R3Pt &BBmin, R3Pt &BBmax );

double Area( const R2Pt &p1, const R2Pt &p2, const R2Pt &p3 );

double Area( double x1, double y1,
             double x2, double y2, 
             double x3, double y3 );

void Barycentric( const R2Pt &p1, const R2Pt &p2, 
                  const R2Pt &p3, const R2Pt &p,
                  double &u, double &v, double &w,
                  double &error );

double LaplacianSmoothing( Array <R2Pt> &uv, const Array <R2Pt> &s,
                           int nu, int nv, int max_it, double tol );

void NormalizeUVCoords(PMeshLite &mesh, int nDir=1);

void NormalizeUVCoords(Array <Vertex> &aoVs, int nDir=1);

int AlignBins( Array<double> &Kbin1, Array<double> &Kbin2, 
               double &angle, int &flip );

int SelectBins( Array<double> &Kbin1, Array<double> &Kbin2, 
                int &iBin1, int &iBin2, int &iBin3, int &iBin4 );

void TransMatrix(R3Pt &B1, R3Pt &B2, R3Pt &B3, R3Pt &B4, 
                 R3Pt &A1, R3Pt &A2, R3Pt &A3, R3Pt &A4, 
                 int method, R3Pt &t1, R3Pt &t2, Array<double> &matrix);

void Transform(PMeshLite &mesh, R3Pt &P1, R3Pt &P2, Array<double> &matrix);

R3Pt Transform(R3Pt &in_P, R3Pt &T1, R3Pt &T2, Array<double> &matrix);

int FindMaxPoints( PMeshLite &mesh, Array<double> &func, int nPts, int sep,
                   Array<R3Pt> &pts, Array<double> &val, Array<int> &indices );

int BestCorrespondence( Array<R3Pt> &PA, Array<double> &VA, Array<int> &IA,
                        Array<R3Pt> &PB, Array<double> &VB, Array<int> &IB,
                        Array<int> &CA, Array<int> &CB );

R3Pt Centroid( Array<R3Pt> &pts );

int MCWrite4PointMesh(const char *filename, const R3Pt &P1, const R3Pt &P2, 
                      const R3Pt &P3, const R3Pt &P4 );

void DumpMatrix(char *filename, char *desc, 
                R3Pt &T1, R3Pt &T2, Array<double> &m);

void RampColor(const double val,
               const double valmin, const double valmax,
               const int type, double &red,
               double &green, double &blue );

int MCReadFanData(FILE *in, Array<int> &spokeSize,
			                Array<R3Pt> &xyz, 
			                Array<double> &K1, 
						    Array<double> &K2,
							Array<int> &valid);

int MCReadFanDataBinary(FILE *in, Array<int> &spokeSize,
			                      Array<R3Pt> &xyz, 
			                      Array<double> &K1, 
						          Array<double> &K2,
							      Array<int> &valid);

int MCWriteFan(FILE *out, Array<int> &spokeSize,
			              Array<R3Pt> &xyz, 
			              Array<double> &K1, 
						  Array<double> &K2,
						  Array<int> &valid);

int MCWriteFanBinary(FILE *out, Array<int> &spokeSize,
			                    Array<R3Pt> &xyz, 
			                    Array<double> &K1, 
						        Array<double> &K2,
						        Array<int> &valid);

int MCWriteFanMesh(const char *filename,
				   Array<int> &spokeSize,
			       Array<R3Pt> &xyz, 
			       Array<double> &K1, 
				   Array<double> &K2,
				   Array<int> &valid);

int MCMakeFanMesh(const char *inputFile,
	 			  const char *outputFile,
				  const int index = 0);

double MCGetLength(const Array<Vertex> &aov);

#endif