#ifndef __MCPARAM_DEFS_H
#define __MCPARAM_DEFS_H

#include <WINSystemDefines.H>
#include <utils/Pm_MeshLite.H>
#include <MC/MC_Point.H>
#include <MC/MC_MeshSubset.H>

/**  \class MCParam MC_Param.H MC/MC_Param.H 
\brief Parameterize a piece of a mesh

  Desbrun's (DCO), Desbrun's with a fix for obtuse angles (ModDCO), 
  Goldfeather and Interrante's cubic fit with points and normals, assumes normals given (ANCe),
  Goldfeather and Interrante's cibic fit with points and normals, calculates normals first (ANC),
  Taubin's Integral Eigen value (finding the eigen values of the shape operator) IEV,
  Hameiri and Shimshoni's fix of Taubin's method (HST),
  Differential of the normal Gauss map (DGM)

*/
class MCParam : public MCMeshSubset {
public:
    typedef enum { NONE=-1, PLANAR, CIRCULAR, NATURAL } MC_ParamType;

protected:
    const MC_ParamType m_ParamType;
    double m_lambda, m_mu, m_UVdiameter;
    R2Pt m_PtUV, m_UVmin, m_UVmax;
    R3Pt m_PtXYZ;
    Array< R2Pt > m_aUVs;

public:
    /**@name Fit Curvature Data */
    //@{
    MCParam(const PMeshLite &in_mesh, 
            const int in_nRings,
            const MC_ParamType &in_paramType);
    ///
    MCParam(MCParam &in_param);
    ///
    MCParam();
    ///
    virtual ~MCParam();
    ///
    double Lambda() const  { return m_lambda; }
    ///
    double Mu() const { return m_mu; }
    ///
    const R2Pt& PtUV() const { return m_PtUV; }
    ///
    const R3Pt& PtXYZ() const { return m_PtXYZ; }
    ///
    const R2Pt& UVmin() const { return m_UVmin; }
    ///
    const R2Pt& UVmax() const { return m_UVmax; }
    ///
    double UVdiam() const { return m_UVdiameter; }
    ///
    Array< R2Pt >& UV() { return m_aUVs; }
    ///
    void Set(const double in_lambda,
             const double in_mu);
    ///
    int CalculateUV(const Vertex &in_vertex, const int in_nRings);
    ///
    int CalculateUVplanar();
    ///
    int CalculateUVcircular(const double in_lambda, const double in_mu);
    ///
    int CalculateUVnatural(const double in_lambda, const double in_mu);
    ///
    void SetUVRange();
    ///
    void NormalizeUVCoords(const int nDir = 2);
    ///
    void ScaleUVCoords(const double scale);
    ///
    void MakeMesh(PMeshLite &out_mesh) const;
    ///
    void dump(FILE *out) const;
    //@}

};

#endif
