#ifndef __MCPOINT_DEFS_H
#define __MCPOINT_DEFS_H

#include <WINSystemDefines.H>
#include <utils/Rn_Defs.H>

/**  \class MCPoint MC_Point.H MC/MC_Point.H 
\brief Stores curvature values at a point
Includes: Index id of point, k1,k2, gaussian (k1 k2), mean ( k1+k2 / 2 ).

For some methods that actually calculate the curvature directions as well, stores
 the surface normal, the two tangent plane vecs representing min and max directions, and
 the same two vecs in the parameterized space.
*/
class MCPoint {
protected:
    int      m_iId;
    double   m_dK1, m_dK2, m_dKg, m_dKbar;
    R3Vec    m_vNormal, m_vKmax, m_vKmin;
    R2Vec    m_vKmaxPar, m_vKminPar;

public:
    /**@name Mesh Curvature Point */
    //@{
    MCPoint(const int id=0, const double k1=0.0, const double k2=0.0);
    ///
    MCPoint( const MCPoint &in_pt );
    ///
    virtual ~MCPoint();
    ///
    MCPoint &operator=(const MCPoint &in_pt);
    ///
    void setId(const int id);
    ///
    void setMaxMin(const double k1, const double k2);
    ///
    void setKgKbar(const double kg, const double kbar);
    ///
    void setNormal(const R3Vec &N);
    ///
    void setMaxMinVec(const R3Vec &KmaxVec, const R3Vec &KminVec);
    ///
    void setMaxMinVecPar(const R2Vec &KmaxVec, const R2Vec &KminVec);
    ///
    int Id() const { return m_iId; }
    ///
    double K1() const { return m_dK1; }
    ///
    double K2() const { return m_dK2; }
    ///
    double Kg() const { return m_dKg; }
    ///
    double Kbar() const { return m_dKbar; }
    ///
    const R3Vec& Normal() const { return m_vNormal; }
    ///
    const R3Vec& KmaxVec() const { return m_vKmax; }
    ///
    const R3Vec& KminVec() const { return m_vKmin; }
    ///
    const R2Vec& KmaxVecPar() const { return m_vKmaxPar; }
    ///
    const R2Vec& KminVecPar() const { return m_vKminPar; }
    ///
    void dump(FILE *out);
    //@}

};

#endif
