#ifndef __MCPOLYNOMIAL_DEFS_H
#define __MCPOLYNOMIAL_DEFS_H

#include <MC/MC_Fit.H>
#include <MC/MC_Point.H>

/**  \class MCPolynomial MC_RPolynomial.H MC/MC_Polynomial.H 
\brief Fit a polynomial to the points

  Parameterization can be done by 1) projecting points onto the tangent plane 
  2) using a circular parameterization (flatten by angle) 
  3) using Desbrun's natural parameterization to flatten the mesh first

  Polynomial is of form a0 + a1 s + a2 t + a3 s^2 + a4 s t + a5 t^2
*/
class MCPolynomial : public MCFit {

protected:

    Array< R3Pt > A; // polynomial coeficients

    int CalculateFit(const Vertex &in_vertex);
    
public:

    /**@name Polynomial Fit Data */
    //@{
    MCPolynomial(const PMeshLite &in_mesh,
                 const int in_nRings, 
                 const MCParam::MC_ParamType &in_paramType );
    ///
    virtual ~MCPolynomial();
    ///
    int Evaluate(const R2Pt &in_uvPoint, R3Pt &out_xyzPoint);
    ///
    int Evaluate(const R2Pt &in_uvPoint, R3Vec &out_normal, 
                 R3Vec &out_dU, R3Vec &out_dV, 
                 R3Vec &out_dUU, R3Vec &out_dUV, R3Vec &out_dVV );
    ///
    int Curvature(const Vertex &in_vertex, MCPoint &cpt);
    ///
    void dump(FILE *out);
    ///
    //@}
};

#endif
