#ifndef __MCRADIALBASIS_DEFS_H
#define __MCRADIALBASIS_DEFS_H

#include <MC/MC_Fit.H>
#include <MC/MC_Point.H>
#include <fitting/Fit_RbfGauss.H>

/**  \class MCRadialBasisFunction MC_RadialBasisFunction.H MC/MC_RadialBasisFunction.H 
\brief Calculate the radial basis function method

  Uses a Gaussian radial basis function. The RBF is a 3D function defined over a 2D plane,
  so the mesh is locally flattened to create the parameterization, then the RBF is fit to the (x,y,z) function.
  rbf(u,v) = (x,y,z) (not defined as a height field)

  We use Gaussians because the basis functions are well-defined at 0.
*/

class MCRadialBasisFunction : public MCFit {

protected:

    FITRbfGauss *rbf;

    int CalculateFit(const Vertex &in_vertex);
    
public:

    /**@name Polynomial Fit Data */
    //@{
    MCRadialBasisFunction(const PMeshLite &in_mesh,
                          const int in_nRings, 
                          const MCParam::MC_ParamType &in_paramType );
    ///
    virtual ~MCRadialBasisFunction();
    ///
    int Evaluate(const R2Pt &in_uvPoint, R3Pt &out_xyzPoint);
    ///
    int Evaluate(const R2Pt &in_uvPoint, R3Vec &out_normal, 
                 R3Vec &out_dU, R3Vec &out_dV, 
                 R3Vec &out_dUU, R3Vec &out_dUV, R3Vec &out_dVV );
    ///
    int Curvature(const Vertex &in_vertex, MCPoint &cpt);
    ///
    void dump(FILE *out);
    ///
    //@}
};

#endif
