
#ifndef ALIGN_POINTS_DEF
#define ALIGN_POINTS_DEF

/** \defgroup FittingAlign  Find the transformation which best aligns the two sets of points
    \ingroup FittingElems

   This is from a standard paper on the subject, whose name I can't
   remember. First it aligns the centroids, then the rotation, and
   finally, the scale. <br>

   This was originally written in C so it could call Fortran, hence
   the C style of passing data around. Could be converted to C++ since
   we now have FITTINGsvd, but I'm too lazy. <br>

  Also see src/utils/TriMesh. This is an iterative closest point implementation from
  Princeton; it
  really should live in ExternalLibraries, but Tim put it there.

<br>   Files: 
- include/fitting/FITTools_Align_points.H
- src/fitting/FITTools/FITTools_Align_points.cpp

   */
//@{

/// Format of an input point
typedef struct {
    double pt[3];
    double confidence; // number from 0-1
} FITToolsAlign_point;

/** Format of the output.

    Rotation around x,y,z axis, translation, and scale.
    */
typedef struct {
    double rotation[3][3];
    double translation[3];
    double scale;
} FITToolsAlign_result;


/** Find the best alignment of the two sets of points.

    Two sets of points must be in 1-1 corresponance. <br>

    @param out_oResult The rotations, translation, and scale if specified.
    @param in_iNPts Number of points in each of the arrays.
    @param in_apts1 First set of points
    @param in_apts2 Second set of points
    @param in_bDoScale Do the scale calculation as well. Default is TRUE.
    @return Accuracy of fit.
    */
double FITToolsAlign_points(FITToolsAlign_result  & out_oResult,
                            int                     in_iNPts, 
                            FITToolsAlign_point   * in_apts1, 
                            FITToolsAlign_point   * in_apts2,
                            WINbool                 in_bDoScale = TRUE );
//@}
#endif
