// FITTools_FitEllipse.h: interface for the FITTools_FitEllipse class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_FITTOOLS_FITELLIPSE_H__EA145FF2_16F8_11D5_8CFF_00010270C860__INCLUDED_)
#define AFX_FITTOOLS_FITELLIPSE_H__EA145FF2_16F8_11D5_8CFF_00010270C860__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <fitting/FITTools_Least_squares.H>
#include <utils/Rn_Ellipse.H>

/** \class FITTools_FitEllipse FITTools_FitEllipse.H fitting/FITTools_FitEllipse
    \brief Different methods for fitting an ellipse to a set of points.
    \ingroup FittingElems

  Some of these work better than others, but it depends on what the data looks like */
class FITTools_FitEllipse : public FITTools_LS  
{
protected:
    /// Error of fit found by evaluating each point in the implicit equation
    double    m_dPercErr;
    /// The actual ellipse
    R2Ellipse m_ellipse;

    /// Do an SVD
    R2Vec FindAxis( const Array<R2Pt> & ) const;
public:
    /// Not in Euclidean distance but in eliptical
    double Error() const { return m_dPercErr; }
    ///
    const R2Ellipse &Ellipse() const { return m_ellipse; }

    /**@name These can be called statically */
    //@{
    /// Fit an implicit equation, explicitly limiting the solution to a circle. Good for partial data.
    static R2Ellipse FitImplicitCircle( const Array<R2Pt> & );
    /// Fit an implicit equation, explicitly limiting the solution to ellipses/circles. Good for partial data.
    static R2Ellipse FitImplicit( const Array<R2Pt> & );
    /// Do PCA and use the eigenvectors and values to set the axis angles and sizes
    static R2Ellipse FitSVD( const Array<R2Pt> & );
    /// Pure implicit fit. May not give an ellipse if partial/poorly formed data
    static R2Ellipse FitImplicitDirect( const Array<R2Pt> &, const WINbool in_bUseTangents );
    //@}

    /// Uses SVD
	FITTools_FitEllipse( const Array<R2Pt> & );
    ///
    virtual ~FITTools_FitEllipse() {}

    static WINbool Test();
};

#endif // !defined(AFX_FITTOOLS_FITELLIPSE_H__EA145FF2_16F8_11D5_8CFF_00010270C860__INCLUDED_)
