//////////////////////////////////////////////////////////////////////

template<class PtType, class VecType>
inline
double FITRbfTC<PtType, VecType>::Area( const R2Pt &in_pt ) const
{
    PtType pt;
    VecType vecDs, vecDt;
    AllEvalSub( in_pt, pt, vecDs, vecDt );

    const double dE = Dot( vecDs, vecDs );
    const double dF = Dot( vecDs, vecDt );
    const double dG = Dot( vecDt, vecDt );

    return sqrt( dE * dG - 2 * dF );
}

template<class PtType, class VecType>
inline
VecType FITRbfTC<PtType, VecType>::SrfNormal( const R2Pt &in_pt ) const
{
    PtType pt;
    VecType vecDs, vecDt;
    AllEvalSub( in_pt, pt, vecDs, vecDt );

    return Cross(vecDs, vecDt);
}
template<class PtType, class VecType>
inline
void FITRbfTC<PtType, VecType>::Curvature( const R2Pt &in_pt,
                                           double &out_dCrv1, double &out_dCrv2, 
                                           R2Vec &out_vec1, R2Vec &out_vec2, 
                                           R3Vec &out_vecMin, R3Vec &out_vecMax, 
							               R3Vec &out_vecNorm ) const
{
    R3Vec vecDs, vecDt, vecDss, vecDst, vecDtt;
    R3Pt pt;

    AllEval( in_pt, pt, vecDs, vecDt, vecDss, vecDtt, vecDst );

    double dArea;
    ::Curvature( pt, vecDs, vecDt, vecDss, vecDtt, vecDst, dArea, out_dCrv1, out_dCrv2, out_vec1, out_vec2, out_vecMin, out_vecMax, out_vecNorm );
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

template<class PtType, class VecType>
inline
FITRbfTC<PtType, VecType>::FITRbfTC<PtType, VecType>()
{
    RNZero( m_aptPoly[0] );
    RNZero( m_aptPoly[1] );
    RNZero( m_aptPoly[2] );
}

template<class PtType, class VecType>
inline
FITRbfTC<PtType, VecType>::~FITRbfTC()
{

}

template<class PtType, class VecType>
inline
void FITRbfTC<PtType, VecType>::Write( ofstream &out ) const
{
	out.precision(24);
    out << m_aptLambda.num() << " ";
    for ( int i = 0; i < m_aptLambda.num(); i++ ) {
        m_aptLambda[i].Write(out);
    }
    out << m_aptRad.num() << " ";
    for ( FORINT i = 0; i < m_aptRad.num(); i++ )
        m_aptRad[i].Write(out);

    for ( FORINT i = 0; i < 3; i++ )
        m_aptPoly[i].Write(out);
}

template<class PtType, class VecType>
inline
void FITRbfTC<PtType, VecType>::WriteBinary( ofstream &out ) const
{
    int iN = m_aptLambda.num();
    out.write( (const char *) &iN, sizeof(int) );
    for ( int i = 0; i < m_aptLambda.num(); i++ ) {
        out.write( (const char *) & m_aptLambda[i][0], sizeof( m_aptLambda[i][0] ) * m_aptLambda[i].Dim() );
    }
    iN = m_aptRad.num();
    out.write( (const char *) &iN, sizeof(int) );
    for ( FORINT i = 0; i < m_aptRad.num(); i++ )
        out.write( (const char *) & m_aptRad[i][0], sizeof( m_aptRad[i][0] ) * m_aptRad[i].Dim() );

    for ( FORINT i = 0; i < 3; i++ )
        out.write( (const char *) & m_aptPoly[i][0], sizeof( m_aptPoly[i][0] ) * m_aptPoly[i].Dim() );
}

template<class PtType, class VecType>
inline
void FITRbfTC<PtType, VecType>::Read( ifstream &in ) 
{
    int iN = 0;
    in >> iN;
    m_aptLambda.need(iN);
    for ( int i = 0; i < m_aptLambda.num(); i++ ) {
        m_aptLambda[i].Read(in);
    }

    in >> iN;
    m_aptRad.need(iN);
    for ( FORINT i = 0; i < m_aptRad.num(); i++ )
        m_aptRad[i].Read(in);

    for ( FORINT i = 0; i < 3; i++ )
        m_aptPoly[i].Read(in);
}

template<class PtType, class VecType>
inline
void FITRbfTC<PtType, VecType>::ReadBinary( ifstream &in ) 
{
    int iN = 0;
    in.read( (char *) &iN, sizeof(int) );
    m_aptLambda.need(iN);
    for ( int i = 0; i < m_aptLambda.num(); i++ ) {
        in.read( (char *) &m_aptLambda[i], m_aptLambda[i].Dim() * sizeof(m_aptLambda[i][0]) );
    }

    in >> iN;
    in.read( (char *) &iN, sizeof(int) );
    for ( FORINT i = 0; i < m_aptRad.num(); i++ )
        in.read( (char *) &m_aptRad[i], m_aptRad[i].Dim() * sizeof(m_aptRad[i][0]) );

    for ( FORINT i = 0; i < 3; i++ )
        in.read( (char *) &m_aptPoly[i], m_aptPoly[i].Dim() * sizeof(m_aptPoly[i][0]) );
}

template<class PtType, class VecType>
inline
WINbool FITRbfTC<PtType, VecType>::operator==( const FITRbfTC<PtType, VecType> &in_func ) const
{
    if ( m_aptLambda.num() != in_func.m_aptLambda.num() )
        return FALSE;

    for ( int i = 0; i < 3; i++ ) {
        if ( !ApproxEqual( m_aptPoly[i], in_func.m_aptPoly[i] ) )
            return FALSE;
    }
    for ( FORINT i = 0; i < m_aptLambda.num(); i++ ) {
        if ( !ApproxEqual( m_aptLambda[i], in_func.m_aptLambda[i] ) )
            return FALSE;

        if ( !ApproxEqual( m_aptRad[i], in_func.m_aptRad[i] ) )
            return FALSE;
    }
    return TRUE;
}

///
template<class PtType, class VecType>
inline
FITRbfTC<PtType, VecType> &FITRbfTC<PtType, VecType>::operator=( const FITRbfTC<PtType, VecType> &in_func )
{
    m_aptLambda = in_func.m_aptLambda;
    m_aptRad = in_func.m_aptRad;
    for ( int i = 0; i < 3; i++ )
        m_aptPoly[i] = in_func.m_aptPoly[i];

    return *this;
}

/// Double check the derivatives against their linear counter parts
template<class PtType, class VecType>
inline
void FITRbfTC<PtType, VecType>::Test( ) const
{
    const R2Pt ptMin = GEOMMin<R2Pt>( m_aptRad );
    const R2Pt ptMax = GEOMMax<R2Pt>( m_aptRad );

    const int iDivs = 4;
    const double dX = (ptMax[0] - ptMin[0]) / (iDivs + 1.0);
    const double dY = (ptMax[1] - ptMin[1]) / (iDivs + 1.0);
    const double dStep = WINmin( dX, dY ) * 1e-9;
    const double dAcc = 1e-4;

    PtType ptCheck;
    VecType vecCheckDs, vecCheckDt, vecCheckDss, vecCheckDst, vecCheckDtt;
    for ( int i = 0; i < iDivs; i++ ) {
        for ( int j = 0; j < iDivs; j++ ) {
            const R2Pt pt( ptMin[0] + dX * i, ptMin[1] + dY * j );
            const PtType ptBlend = Eval( pt );
            const VecType vecBlend = ptBlend - PtType(0,0,0);
            const VecType vecBlendDs = ( Eval( pt + R2Vec( dStep, 0.0 ) ) - ptBlend ) / dStep;
            const VecType vecBlendDt = ( Eval( pt + R2Vec( 0.0, dStep ) ) - ptBlend ) / dStep;
            const VecType vecBlendDss = ( DerivS( pt + R2Vec( dStep, 0.0 ) ) - DerivS( pt ) ) / dStep;
            const VecType vecBlendDtt = ( DerivT( pt + R2Vec( 0.0, dStep ) ) - DerivT( pt ) ) / dStep;
            const VecType vecBlendDst = ( DerivS( pt + R2Vec( 0.0, dStep ) ) - DerivS( pt ) ) / dStep;

            vecCheckDs = DerivS( pt );
            ASSERT( ApproxEqual( vecCheckDs, vecBlendDs, dAcc ) );
            vecCheckDt = DerivT( pt );
            ASSERT( ApproxEqual( vecCheckDt, vecBlendDt, dAcc ) );
            vecCheckDss = DerivSS( pt );
            ASSERT( ApproxEqual( vecCheckDss, vecBlendDss, dAcc ) );
            vecCheckDtt = DerivTT( pt );
            ASSERT( ApproxEqual( vecCheckDtt, vecBlendDtt, dAcc ) );
            vecCheckDst = DerivST( pt );
            ASSERT( ApproxEqual( vecCheckDst, vecBlendDst, dAcc ) );

            AllEvalSub( pt, ptCheck, vecCheckDs, vecCheckDt );
            ASSERT( ApproxEqual( ptCheck, ptBlend, dAcc ) );
            ASSERT( ApproxEqual( vecCheckDs, vecBlendDs, dAcc ) );
            ASSERT( ApproxEqual( vecCheckDt, vecBlendDt, dAcc ) );

            AllEval( pt, ptCheck, vecCheckDs, vecCheckDt, vecCheckDss, vecCheckDtt, vecCheckDst );
            ASSERT( ApproxEqual( ptCheck, ptBlend, dAcc ) );
            ASSERT( ApproxEqual( vecCheckDs, vecBlendDs, dAcc ) );
            ASSERT( ApproxEqual( vecCheckDt, vecBlendDt, dAcc ) );
            ASSERT( ApproxEqual( vecCheckDss, vecBlendDss, dAcc ) );
            ASSERT( ApproxEqual( vecCheckDtt, vecBlendDtt, dAcc ) );
            ASSERT( ApproxEqual( vecCheckDst, vecBlendDst, dAcc ) );
        }
    }
}


