// Fit_RBFSpline.H Spline spline
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_FitRbfSplineTC_H__C622E9A3_1A9D_422D_BAF4_50E33C53C211__INCLUDED_)
#define AFX_FitRbfSplineTC_H__C622E9A3_1A9D_422D_BAF4_50E33C53C211__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <utils/Rn_Defs.H>
#include <utils/Mesh_Array.H>
#include <utils/Crv_Basis.H>
#include <fitting/Fit_RBF.H>

/** \class FITRbfSplineTC Fit_RbfSpline.H fitting/Fit_RbfSpline.H 
     \ingroup FittingElems
     \brief Radial Basis function fitting with spline basis function kernel

 * Kernel: An n-degree b-spline basis function
 */
template< class PtType, class VecType >
class FITRbfSplineTC : public FITRbfTC<PtType,VecType>
{
private:
    Array< double >  m_adSigmaSq;
    CRVBasis         m_crv; ///< basis function

public:
    ///
    void SetContinuity( const int in_iC );
    /** \brief Set the input locations and corresponding output points
      * Set all the points at once. f( in_aptParam[i] ) = in_aptData[i]
      @param in_aptParam The s,t points
      @param in_aptData The data points to interpolate */
    void Set( const Array< R2Pt > &in_aptParam, const Array< PtType > &in_aptData );

    ///
    PtType Eval( const R2Pt &in_pt ) const;

    ///
    VecType DerivS( const R2Pt &in_pt ) const;
    ///
    VecType DerivT( const R2Pt &in_pt ) const;
    ///
    VecType DerivSS( const R2Pt &in_pt ) const;
    ///
    VecType DerivTT( const R2Pt &in_pt ) const;
    ///
    VecType DerivST( const R2Pt &in_pt ) const;
    ///
    void AllEval( const R2Pt &in_pt,
                  PtType &out_pt,
                  VecType &out_vecDs, VecType &out_vecDt,
                  VecType &out_vecDss, VecType &out_vecDtt,
                  VecType &out_vecDst ) const;
    ///
    void AllEvalSub( const R2Pt &in_pt,
                     PtType &out_pt,
                     VecType &out_vecDs, VecType &out_vecDt ) const;

    ///
    FITRbfSplineTC<PtType,VecType> &operator=( const FITRbfSplineTC<PtType,VecType> &in_func );
    ///
    WINbool operator==( const FITRbfSplineTC<PtType,VecType> &in_func ) const;

    ///
    FITRbfSplineTC( const FITRbfSplineTC<PtType,VecType> &in_func ) { *this = in_func; }
    ///
    FITRbfSplineTC() { SetContinuity(2); }
    ///
    virtual ~FITRbfSplineTC() {}

    void Write( ofstream &out ) const;
    void Read( ifstream &in );
};


#include "Fit_RbfSpline_i.H"

typedef FITRbfSplineTC<R3Pt, R3Vec> FITRbfSpline;

#endif // !defined(AFX_FitRbfSplineTC_H__C622E9A3_1A9D_422D_BAF4_50E33C53C211__INCLUDED_)


