// Fit_RBFThinPlate.H ThinPlate spline
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_FITRBFThinPlate_H__C622E9A3_1A9D_422D_BAF4_50E33C53C211__INCLUDED_)
#define AFX_FITRBFThinPlate_H__C622E9A3_1A9D_422D_BAF4_50E33C53C211__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <utils/Rn_Defs.H>
#include <utils/Mesh_Array.H>
#include <fitting/Fit_RBF.H>

/** \class FITRbfThinPlateTC Fit_RbfThinPlate.H fitting/Fit_RbfThinPlate.H 
     \ingroup FittingElems
     \brief Radial Basis function fitting with Thin plate spline kernel

 * Thin plate spline
 * kernel: f(r) = (r / sigma)^2 log( r/sigma )
 */
template< class PtType, class VecType >
class FITRbfThinPlateTC : public FITRbfTC< PtType,VecType >
{
protected:
    Array< double >    m_adSigmaSq;

public:
    /** \brief Set the input locations and corresponding output points
      * Set all the points at once. f( in_aptParam[i] ) = in_aptData[i]
      @param in_aptParam The s,t points
      @param in_aptData The data points to interpolate */
    void Set( const Array< R2Pt > &in_aptParam, const Array< PtType > &in_aptData );

    ///
    PtType Eval( const R2Pt &in_pt ) const;

    ///
    VecType DerivS( const R2Pt &in_pt ) const;
    ///
    VecType DerivT( const R2Pt &in_pt ) const;
    ///
    VecType DerivSS( const R2Pt &in_pt ) const;
    ///
    VecType DerivTT( const R2Pt &in_pt ) const;
    ///
    VecType DerivST( const R2Pt &in_pt ) const;
    ///
    void AllEval( const R2Pt &in_pt,
                  PtType &out_pt,
                  VecType &out_vecDs, VecType &out_vecDt,
                  VecType &out_vecDss, VecType &out_vecDtt,
                  VecType &out_vecDst ) const;
    ///
    void AllEvalSub( const R2Pt &in_pt,
                     PtType &out_pt,
                     VecType &out_vecDs, VecType &out_vecDt ) const;

    ///
    FITRbfThinPlateTC<PtType,VecType> &operator=( const FITRbfThinPlateTC<PtType,VecType> &in_func );

    ///
    FITRbfThinPlateTC<PtType,VecType>( const FITRbfThinPlateTC<PtType,VecType> &in_func ) { *this = in_func; }

    ///
    FITRbfThinPlateTC<PtType,VecType>() {}
    ///
    virtual ~FITRbfThinPlateTC<PtType,VecType>() {}

    ///
    void Write( ofstream &out ) const;
    ///
    void Read( ifstream &in );
};

#include "Fit_RbfThinPlate_i.H"

typedef FITRbfThinPlateTC<R3Pt, R3Vec> FITRbfThinPlate;

#endif // !defined(AFX_FITRbfThinPlate_H__C622E9A3_1A9D_422D_BAF4_50E33C53C211__INCLUDED_)

