template <class Pt, class Vec>
inline
Pt CRVHermiteTC<Pt,Vec>::operator()( const double in_dT )
{
    Pt ptRet;

    for ( int i = 0; i < ptRet.Dim(); i++ )
        ptRet[i] = 0.0;

    if ( m_apt.num() == 0 ) {
        return ptRet;
    } else if ( m_apt.num() == 1 ) {
        return m_apt[0];
    }

    const double dDiv = 1.0 / (m_apt.num() - 1.0);
    const int iSeg = (int) ( in_dT / dDiv );
    const int iSegNext = WINmin( iSeg + 1, m_apt.num() - 1 );
    //const double dT = in_dT - iSeg * dDiv;
	const double dT = in_dT/dDiv - iSeg;	// <rts> scales dT to be (0.0, 1.0) instead of (0.0, dDiv)

    const double dTSq = dT * dT;
    const double dTCub = dTSq * dT;

    const double dH1 =  2.0 * dTCub - 3.0 * dTSq + 1.0;
    const double dH2 = -2.0 * dTCub + 3.0 * dTSq;
    const double dH3 =        dTCub - 2.0 * dTSq + dT;
    const double dH4 =        dTCub -       dTSq;

    for ( FORINT i = 0; i < ptRet.Dim(); i++ ) {
        ptRet[i] = dH1 * m_apt[iSeg][i] + dH2 * m_apt[iSegNext][i] +
                   dH3 * m_avec[iSeg][i] + dH4 * m_avec[iSegNext][i];
    }

    return ptRet;
}

template <class Pt, class Vec>
inline
Vec CRVHermiteTC<Pt,Vec>::Deriv( const double in_dT )
{
    Vec vecRet;

    if ( m_apt.num() == 0 ) {
        for ( int i = 0; i < vecRet.Dim(); i++ )
            vecRet[i] = 0.0;

        return vecRet;
    } else if ( m_apt.num() == 1 ) {
        return m_avec[0];
    }

    const double dDiv = 1.0 / (m_apt.num() - 1.0);
    const int iSeg = (int) ( in_dT / dDiv );
    const int iSegNext = WINmin( iSeg + 1, m_apt.num() - 1 );
    //const double dT = in_dT - iSeg * dDiv;
	const double dT = in_dT/dDiv - iSeg;	// <rts> scales dT to be (0.0, 1.0) instead of (0.0, dDiv)

    const double dTSq = dT * dT;

    const double dH1 =  6.0 * dTSq - 6.0 * dT;
    const double dH2 = -6.0 * dTSq + 6.0 * dT;
    const double dH3 =  3.0 * dTSq - 4.0 * dT + 1.0;
    const double dH4 =  3.0 * dTSq - 2.0 * dT;

    for ( int i = 0; i < vecRet.Dim(); i++ ) {
        vecRet[i] = dH1 * m_apt[iSeg][i] + dH2 * m_apt[iSegNext][i] +
                    dH3 * m_avec[iSeg][i] + dH4 * m_avec[iSegNext][i];
    }

    return vecRet;
}

template <class Pt, class Vec>
inline
Vec CRVHermiteTC<Pt,Vec>::SecondDeriv( const double in_dT )
{
    Vec vecRet;
    if ( m_apt.num() == 0 ) {
        return vecRet;
    } else if ( m_apt.num() == 1 ) {
        return m_avec[0];
    }

    const double dDiv = 1.0 / (m_apt.num() - 1.0);
    const int iSeg = (int) ( in_dT / dDiv );
    const int iSegNext = WINmin( iSeg + 1, m_apt.num() - 1 );
    //const double dT = in_dT - iSeg * dDiv;
	const double dT = in_dT/dDiv - iSeg;	// <rts> scales dT to be (0.0, 1.0) instead of (0.0, dDiv)

    const double dH1 =  12.0 * dT - 6.0;
    const double dH2 = -12.0 * dT + 6.0;
    const double dH3 =   6.0 * dT - 4.0;
    const double dH4 =   6.0 * dT - 2.0;

    for ( int i = 0; i < vecRet.Dim(); i++ ) {
        vecRet[i] = dH1 * m_apt[iSeg][i] + dH2 * m_apt[iSegNext][i] +
                    dH3 * m_avec[iSeg][i] + dH4 * m_avec[iSegNext][i];
    }

    return vecRet;
}
