template<class InPoint, class OutPoint, class OutVector>
inline
int 
CRVNurbTC<InPoint, OutPoint, OutVector>::InDim() const 
{
    return -1;
}

template<class InPoint, class OutPoint, class OutVector>
inline
int 
CRVNurbTC<InPoint, OutPoint, OutVector>::OutDim() const 
{
    return -1;
}

template<>
inline
int
CRVNurbTC<R3Pt, R3Pt, R3Vec>::InDim() const 
{
    return 3;
}

template<>
inline
int
CRVNurbTC<R3Pt, R3Pt, R3Vec>::OutDim() const 
{
    return 3;
}



template<>
inline
int
CRVNurbTC<R2Pt, R2Pt, R2Vec>::InDim() const 
{
    return 2;
}

template<>
inline
int
CRVNurbTC<R2Pt, R2Pt, R2Vec>::OutDim() const 
{
    return 2;
}


template<>
inline
int
CRVNurbTC<R1Pt, R1Pt, R1Vec>::InDim() const 
{
    return 1;
}

template<>
inline
int
CRVNurbTC<R1Pt, R1Pt, R1Vec>::OutDim() const 
{
    return 1;
}


/* ------------------------------------------------------------
 * DESCR   :	Evaluate a point
 * DETAILS :    Multiply basis functions by control points, 
 *              divide by sum of weights 
 * ------------------------------------------------------------ */
template<class InPoint, class OutPoint, class OutVector>
inline
OutPoint
CRVNurbTC<InPoint, OutPoint, OutVector>::operator()(double in_dT)
const
{
   InPoint out_ptSum;
   RNZero(out_ptSum);
   
   Array<double> adBasis;
   Array<int>    aiIndx;
   
   EvalAllBasis( in_dT, aiIndx, adBasis );

   for (int i = 0; i < aiIndx.num(); i++) {
       // ugly fix for where basis functions don't sum to one
      double b = adBasis[ i ];
      if (!RNIsZero(b)) {
         for (int j = 0; j < InDim(); j++)
           out_ptSum[j] += b * Pt( aiIndx[i] )[j];
      }
   }

   return out_ptSum;
}


template<class InPoint, class OutPoint, class OutVector>
inline
double CRVNurbTC<InPoint, OutPoint, OutVector>::Integrate_segment(int in_iSeg) const
{
   return Integrate_interval(Knot(in_iSeg), Knot(in_iSeg +1));
}


template<class InPoint, class OutPoint, class OutVector>
inline
OutPoint CRVNurbTC<InPoint, OutPoint, OutVector>::Perc(double in_dPerc) const 
{ 
   return (*this)(Perc_t(in_dPerc));
}

template<class InPoint, class OutPoint, class OutVector>
inline
OutPoint CRVNurbTC<InPoint, OutPoint, OutVector>::PercSegment(int in_iSeg,
                                                              double in_dPerc) const 
{ 
   return (*this)(Segment_perc_t(in_iSeg, in_dPerc));
}

// Approximate length of the curve
template<class InPoint, class OutPoint, class OutVector>
inline
double CRVNurbTC<InPoint, OutPoint, OutVector>::Length() const
{
   return Length(Start_t(), End_t());
}
   
template<class InPoint, class OutPoint, class OutVector>
inline
OutPoint    
CRVNurbTC<InPoint, OutPoint, OutVector>::Closest_pt(const OutPoint &in_pt,
                                                    int in_iNDivs) const
{
   return (*this)(Closest_t(in_pt, in_iNDivs));
}


/* ------------------------------------------------------------
 * DESCR   :	Copy the curve
 * ------------------------------------------------------------ */
template<class InPoint, class OutPoint, class OutVector>
inline
CRVNurbTC<InPoint, OutPoint, OutVector> &
CRVNurbTC<InPoint, OutPoint, OutVector>
::operator=(const CRVNurbTC<InPoint, OutPoint, OutVector> &in_crv)
{
   CRVKnot_vec::operator=(in_crv);
   m_apts = in_crv.m_apts;

//   Check();
   return *this;
}

/* ------------------------------------------------------------
 * DESCR   :	compare the curves
 * ------------------------------------------------------------ */
template<class InPoint, class OutPoint, class OutVector>
inline
WINbool 
CRVNurbTC<InPoint, OutPoint, OutVector>                            
::operator==(const CRVNurbTC<InPoint, OutPoint, OutVector> &in_crv) const
{
   if (CRVKnot_vec::operator==(in_crv) == FALSE) return FALSE;

   for (int i = 0; i < Num_pts(); i++)
     if (ApproxEqual(Pt(i), in_crv.Pt(i)) == FALSE) return FALSE;

   return TRUE;
}


/* --------------- protected Routines -------------- */







template<class InPoint, class OutPoint, class OutVector>
inline
WINbool CRVNurbTC<InPoint, OutPoint, OutVector>::Add_points(int in_iNPts, 
                                                            int in_iWhere)
{
   WINbool out_bRes = Add_knots(in_iNPts, in_iWhere);
   
   m_apts.need(Num_pts());
   int i;
   for (i = Num_pts()-1; i >= WINmax(0, in_iWhere + in_iNPts); i--)
     m_apts[i] = m_apts[i-in_iNPts];

   for (i = in_iWhere; i < in_iWhere + in_iNPts; i++)
     RNZero(m_apts[i]);
   
   //Check();
   return out_bRes;
}


template<class InPoint, class OutPoint, class OutVector>
inline
WINbool CRVNurbTC<InPoint, OutPoint, OutVector>::Remove_points(int in_iNPts, 
                                                               int in_iWhere)
{
   WINbool out_bRes = Remove_knots(in_iNPts, in_iWhere);

   for (int i = in_iWhere; i < m_apts.num() - in_iNPts; i++)
     m_apts[i] = m_apts[i+in_iNPts];

   m_apts.sub(in_iNPts);

   //Check();
   return out_bRes;
}


/* ------------------------------------------------------------
 * DESCR   :	Return a curve clipped off the 
 *               beginning and ending points
 * DETAILS :     knots at start_s and end_s must exist
 * ------------------------------------------------------------ */
template<class InPoint, class OutPoint, class OutVector>
inline
CRVNurbTC<InPoint, OutPoint, OutVector>
CRVNurbTC<InPoint, OutPoint, OutVector>::Clip_crv(double in_dStart,
                                                  double in_dEnd) const
{
   CRVNurbTC<InPoint, OutPoint, OutVector> out_crvClipped(*this);

   int start_knot = 0, last_knot = Num_knots() - 1;

   // See if these knots exist yet
   for (int i = 0; i < out_crvClipped.Num_knots(); i++) {
      if (RNApproxEqual(out_crvClipped.Knot(i), in_dStart))
        start_knot = WINmax(0, i - Order() - 1);
      if (RNApproxEqual(out_crvClipped.Knot(i), in_dEnd))
        last_knot = WINmin(i+Order()-1, out_crvClipped.Num_knots() - 1);
   }
   
   int num_to_remove = out_crvClipped.Num_knots() - last_knot - start_knot - 1;

   // Remove points at the beginning
   out_crvClipped.Remove_points(start_knot, 0);
   out_crvClipped.Remove_points(num_to_remove,
                                out_crvClipped.Num_knots() - num_to_remove);

   return out_crvClipped;
}

/* ------------------------------------------------------------
 * DESCR   :	Integrate a single segment
 * ------------------------------------------------------------ */
template<class InPoint, class OutPoint, class OutVector>
inline
double
CRVNurbTC<InPoint, OutPoint, OutVector>::Integrate_interval(double in_dT0,
                                                          double in_dT1) const
{
   if (RNApproxEqual(in_dT0, in_dT1)) return 0;
   
   const int n_pts = (20 * (Order()+1));
   const double t_div = (in_dT1 - in_dT0) / (double) n_pts;

   double out_dSum = 0;

   OutPoint last_pt((*this)(in_dT0));
   OutPoint cur_pt;
   for (int j = 1; j < n_pts+1; j++) {
      cur_pt = (*this)(in_dT0 + j * t_div);
      out_dSum += ::Length(last_pt - cur_pt);
      last_pt = cur_pt;
   }
   
   return out_dSum;
}


/* ------------- Public Routines ----------------------------- */


/* -------------------------------------------------
 * DESCR   :	Evaluate a derivative
 * DETAILS :    Multiply derivative of basis functions by control points,
 *               divide by sum of weights 
 * ------------------------------------------------------------- */
template<class InPoint, class OutPoint, class OutVector>
inline
OutVector
CRVNurbTC<InPoint, OutPoint, OutVector>::Derivative(double in_dT) const
{
   OutVector out_vecSum;
   RNZero( out_vecSum );

   // Evaluate basis function/control point pairs
   int f_k = Floor_knot(in_dT);
   int s_i = WINmax(0, f_k - (Order()+1));
   int e_i = WINmin(f_k+1, Num_pts());
   for (int i = s_i; i < e_i; i++) {
      double b = Basis(i).Derivative(in_dT);
      if (!RNIsZero(b)) {
         for (int j = 0; j < InDim(); j++)
           out_vecSum[j] += b * m_apts[i][j];
      }
   }

   // Automaticall Divide by out_vecSum[dim] if necessary
   return out_vecSum;
}

/* -------------------------------------------------
 * DESCR   :	Evaluate a second derivative
 * DETAILS :    Multiply derivative of basis functions by control points,
 *               divide by sum of weights 
 * ------------------------------------------------------------- */
template<class InPoint, class OutPoint, class OutVector>
inline
OutVector
CRVNurbTC<InPoint, OutPoint, OutVector>::SecondDerivative(double in_dT) const
{
   OutVector out_vecSum;
   RNZero( out_vecSum );

   // Evaluate basis function/control point pairs
   int f_k = Floor_knot(in_dT);
   int s_i = WINmax(0, f_k - (Order()+1));
   int e_i = WINmin(f_k+1, Num_pts());
   for (int i = s_i; i < e_i; i++) {
      double b = Basis(i).SecondDerivative(in_dT);
      if (!RNIsZero(b)) {
         for (int j = 0; j < InDim(); j++)
           out_vecSum[j] += b * m_apts[i][j];
      }
//      TRACE(" %d %f ", i, b);
   }
//   TRACE("\n");

   // Automaticall Divide by out_vecSum[dim] if necessary
   return out_vecSum;
}


/* ------------------------------------------------------------
 * DESCR   :	length of curve, integral from in_dT0 to in_dT1
 * ------------------------------------------------------------ */
template<class InPoint, class OutPoint, class OutVector>
inline
double
CRVNurbTC<InPoint, OutPoint, OutVector>::Length(double in_dT0,
                                                double in_dT1) const
{
   const double t0 = WINmax(((in_dT0 < in_dT1) ? in_dT0 : in_dT1), Start_t());
   const double t1 = WINmin(((in_dT0 < in_dT1) ? in_dT1 : in_dT0), End_t());

   const int start_i = Floor_knot(t0);
   const int end_i = Ceil_knot(t1);

   double out_dSum = 0;
   if (start_i == end_i) {
      out_dSum = Integrate_interval(t0, t1);
   } else if (end_i < start_i) {
      return 0;
   } else {
      out_dSum = Integrate_interval(t0, Knot(start_i+1));

      for (int i = start_i+1; i < end_i; i++)
         out_dSum += Integrate_segment(i);

      out_dSum += Integrate_interval(Knot(end_i), t1);
   }

   return out_dSum;
}

/* ------------------------------------------------------------
 * DESCR   :	Find the closest point on the curve to the click
 * ------------------------------------------------------------ */
template<class InPoint, class OutPoint, class OutVector>
inline
double
CRVNurbTC<InPoint, OutPoint, OutVector>::Closest_t(const OutPoint &in_pt,
                                                 int in_iNDivs) const
{
   in_iNDivs = WINmax(in_iNDivs, Order() + 2);
   double out_dBestGuess = End_t();

   double dist = LengthSq((*this)(out_dBestGuess) - in_pt);
   const double div = 1.0 / (double) in_iNDivs;
   for (int i = 0; i < Num_segments(); i++) {
      for (int j = 0; j < in_iNDivs; j++) {
         double d = LengthSq(PercSegment(i, j * div) - in_pt);
         if (d < dist) {
            out_dBestGuess = Segment_perc_t(i, j * div);
            dist = d;
         }
      }
   }
   
   return out_dBestGuess;
}


/* -------------------------- Private Routines ---------------- */
template <>
inline
void  CRVNurbTC<R3Pt, R3Pt, R3Vec>::Frame(double in_dT, 
                R3Vec &out_vecT, R3Vec &out_vecM, R3Vec &out_vecB) const
{
    static R3Vec vecZero(0,0,0);

    out_vecT = Derivative( in_dT );
    R3Vec vec = SecondDerivative( in_dT );
    /*
    TRACE(" %f %f %f  2nd %f %f %f\n", 
        out_vecT[0], out_vecT[1], out_vecT[2],
        vec[0], vec[1], vec[2]);
        */
    out_vecB = Cross( out_vecT, vec );
    if (ApproxEqual( out_vecB, vecZero ) ) {
        for (int i = 0; i < out_vecT.Dim(); i++) {
            if (! RNIsZero( out_vecT[i] ) ) {
                out_vecB[i] = out_vecT[ (i+1) % out_vecT.Dim() ];
                out_vecB[ (i+1) % out_vecT.Dim() ] = out_vecT[i];
                break;
            }
        }
    }

    double dLengthT = ::Length( out_vecT );
    double dLengthB = ::Length( out_vecB );

    if ( ! RNIsZero( dLengthT ) ) 
        out_vecT /= dLengthT;

    if ( ! RNIsZero( dLengthB ) ) 
        out_vecB /= dLengthB;

    out_vecM = UnitSafe( Cross( out_vecT, out_vecB ) );
}

template<>
inline
void  CRVNurbTC<R2Pt, R2Pt, R2Vec>::Frame(double in_dT, 
                R2Vec &out_vecT, R2Vec &out_vecM, R2Vec &out_vecB) const
{
    out_vecT = Derivative( in_dT );
    out_vecB = SecondDerivative( in_dT );
    /*
    TRACE(" %f %f %f  2nd %f %f %f\n", 
        out_vecT[0], out_vecT[1], out_vecT[2],
        vec[0], vec[1], vec[2]);
        */
    out_vecT = UnitSafe( out_vecT );
    out_vecB = UnitSafe( out_vecB );
    out_vecM = R2Vec(0,0);
}

template<>
inline
void  CRVNurbTC<R1Pt, R1Pt, R1Vec>::Frame(double in_dT, 
                R1Vec &out_vecT, R1Vec &out_vecM, R1Vec &out_vecB) const
{
    out_vecT = Derivative( in_dT );
    out_vecB = SecondDerivative( in_dT );
    /*
    TRACE(" %f %f %f  2nd %f %f %f\n", 
        out_vecT[0], out_vecT[1], out_vecT[2],
        vec[0], vec[1], vec[2]);
        */
    out_vecT = UnitSafe( out_vecT );
    out_vecB = UnitSafe( out_vecB );
    out_vecM = R1Vec(0);
}



/* ------------------------------------------------------------
 * DESCR   :	Return a nurb curve of the segment
 * DETAILS :    Adds in knots at the appropriate points, then clips the curve
 * ------------------------------------------------------------ */
template<class InPoint, class OutPoint, class OutVector>
inline
CRVNurbTC<InPoint, OutPoint, OutVector>
CRVNurbTC<InPoint, OutPoint, OutVector>::Sub_crv(double in_dStart,
                                                 double in_dEnd
                                                 ) const 
{
   if (in_dStart < Start_t() || in_dStart > End_t())
     TRACE("WARNING: sub crv left split not between %f and %f\n",
           Start_t(), End_t());
   
   if (in_dEnd < Start_t() || in_dEnd > End_t())
     TRACE("WARNING sub crv right split not between %f and %f\n",
           Start_t(), End_t());
   
   // See if it has these t values
   WINbool bHas_start = FALSE, bHas_end = FALSE;
   for (int i = 0; i < Num_knots(); i++) {
      if (RNApproxEqual(Knot(i), in_dStart))
         bHas_start = TRUE;
      if (RNApproxEqual(Knot(i), in_dEnd))
         bHas_end = TRUE;
   }

   // Has t values - just return the clipped curve
   if (bHas_start == TRUE && bHas_end == TRUE)
      return Clip_crv(in_dStart, in_dEnd);

   // Need to refine the curve, than clip the refined curve
   Array<double> new_knots;
   
   if (bHas_start == FALSE) new_knots += in_dStart;
   if (bHas_end == FALSE) new_knots += in_dEnd;

   //   CRVNurbTC<InPoint, OutPoint, OutVector> clpd = this->Refine(new_knots);
   //   return clpd.Clip_crv(in_dStart, in_dEnd);
   return *this;
}

/* ------------------------------------------------------------
 *   :	Make a new curve from this one, adding in the given knots
 * ------------------------------------------------------------ */
template<class InPoint, class OutPoint, class OutVector>
inline
CRVNurbTC<InPoint, OutPoint, OutVector>
CRVNurbTC<InPoint, OutPoint, OutVector>::Refine(double  in_dNewKnot,
                                                WINbool in_bAllowDups
   ) const
{
   Array<double> adNewknots(1);
   adNewknots[0] = in_dNewKnot;
   return Refine(adNewknots, in_bAllowDups);
}


template<class InPoint, class OutPoint, class OutVector>
inline
Array<double> CRVNurbTC<InPoint, OutPoint, OutVector>::Even_spacing(int in_iNDivs) const
{
   Array<double> adDists;
   Array<double> adDistTs;
   Array<double> adTs;

   ASSERT(in_iNDivs > 1);

   const int n_cuts = 100 * in_iNDivs;
   const double delta_t = 1.0 / (n_cuts - 1.0);
   OutPoint last_pt(Start_crv()), next_pt(End_crv());

   adDists += 0;
   adDistTs += Start_t();
   int i;
   for ( i = 1; i < n_cuts; i++) {
      adDistTs += Perc_t(i * delta_t);
      next_pt = (*this)(adDistTs.last());
      adDists += adDists.last() + ::Length(next_pt - last_pt);
      last_pt = next_pt;
   }
   
   const double delta_d = adDists.last() / (in_iNDivs - 1.0);
   int last_d = 0;
   for (i = 0; i < in_iNDivs; i++) {
      double targ_d = i * delta_d;
      for (; last_d < adDists.num(); last_d++) {
         if (targ_d <= adDists[last_d]) {
            break;
         }
      }
      if (last_d == adDists.num()) last_d--;
      
      if (last_d == 0) adTs += adDistTs[0];
      else {
         double div = adDists[last_d] - adDists[last_d-1];
         double blend = 0.0;
         if (fabs(div) < 1e-30) blend = 0.5;
         else blend = (targ_d - adDists[last_d-1]) / div;
         adTs += (1.0-blend) * adDistTs[last_d-1] + blend * adDistTs[last_d];
      }
   }
   return adTs;
}

template<class InPoint, class OutPoint, class OutVector>
inline
CRVNurbTC<InPoint, OutPoint, OutVector> CRVNurbTC<InPoint, OutPoint, OutVector>::Derivative() const
{
    CRVNurbTC<InPoint, OutPoint, OutVector> out_crv( *this );

    out_crv.CRVKnot_vec::operator=( CRVKnot_vec::Derivative());
    return out_crv;
}


/* -------------------------------------------------------------------------
 * DESCR   :	Refine at knots
 * ------------------------------------------------------------------------- */
template<class InPoint, class OutPoint, class OutVector>
inline
CRVNurbTC<InPoint, OutPoint, OutVector>
CRVNurbTC<InPoint, OutPoint, OutVector>::Refine(const Array<double> &in_adNewKnots,
                                                WINbool in_bAllowDups) const
{
   Array<double> pos_knots;
   if (in_bAllowDups == FALSE) {
      int last_k = 0;
      for (int i = 0; i < in_adNewKnots.num(); i++) {
         WINbool is_unique = TRUE;
         for (int j = last_k; j < Num_knots(); j++) {
            if (RNIsZero(in_adNewKnots[i] - Knot(j))) {
               last_k = j;
               is_unique = FALSE;
            }
            if (Knot(j) > in_adNewKnots[i]) {
               last_k = j;
               break;
            }
         }
         if (is_unique == TRUE) 
           pos_knots += in_adNewKnots[i];
      }
      if (pos_knots.num() == 0) return *this;
   }       
   
   Array<double> new_knots((in_bAllowDups==TRUE) ? in_adNewKnots : pos_knots);
   
   // Make blank curve with space for new knots
   int       iNPts = Num_pts() + new_knots.num();
   CRVSpace  cspace( Order() );
   CRVNurbTC<InPoint, OutPoint, OutVector> refnd_crv( cspace, iNPts );
   
   // New knot vec (old knot vec with new_knots inserted)
   Array<double> knot_vec;
   
   int j = 0;
   int i = 0;
   for (int k = 0; k < refnd_crv.Num_knots(); k++) {
      if      (i == Num_knots())        knot_vec += new_knots[j++];
      else if (j == new_knots.num())    knot_vec += Knot(i++);
      else if (Knot(i) < new_knots[j])  knot_vec += Knot(i++);
      else                              knot_vec += new_knots[j++];
   }

   // Coefficients for new control points
   CRVNurb_alpha alpha(*this, Knots(), knot_vec);

   // Give it the new knot vec
   refnd_crv.Set_knots(knot_vec, -1, knot_vec.num());

   // Calculate locations of points
   for (j = 0; j < refnd_crv.Num_pts(); j++) {
      InPoint &pt = refnd_crv.Pt(j);
      int k;
      for (k = 0; k < pt.Dim(); k++) pt[k] = 0;
      
      for (i = 0; i < Num_pts(); i++) {
         double a = alpha(i, j);
         if (a > RNEpsilon_d)
           for (k = 0; k < pt.Dim(); k++) {
              pt[k] += a * Pt(i)[k];
           }
      }
   }
   return refnd_crv;
}


/* ------------------------------------------------------------
 * DESCR   :	Make a copy of the given curve
 * ------------------------------------------------------------ */
template<class InPoint, class OutPoint, class OutVector>
inline
CRVNurbTC<InPoint, OutPoint, OutVector> 
::CRVNurbTC(const CRVNurbTC<InPoint, OutPoint, OutVector> &in_crv) 
: CRVKnot_vec(in_crv)
{
   m_apts = in_crv.m_apts;
}

/* ------------------------------------------------------------
 * DESCR   :	Make a curve with the given # of points
 * ------------------------------------------------------------ */


template<class InPoint, class OutPoint, class OutVector>
inline
CRVNurbTC<InPoint, OutPoint, OutVector>::CRVNurbTC(ifstream &in, WINbool bIs_binary) 
{
    if (bIs_binary == FALSE) {
        Read( in );
    } else {
        Read_binary( in );
    }
}


template<class InPoint, class OutPoint, class OutVector>
inline
CRVNurbTC<InPoint, OutPoint, OutVector>::CRVNurbTC(const CRVSpace &in_crvspace,
                                                   int in_iNPts) 
: CRVKnot_vec(in_crvspace, in_iNPts)
{
    m_apts.need(in_iNPts);
    for (int i = 0; i < Num_pts(); i++)
        RNZero(m_apts[i]);
}

template<class InPoint, class OutPoint, class OutVector>
inline
CRVNurbTC<InPoint, OutPoint, OutVector>::CRVNurbTC() 
: CRVKnot_vec(CRVSpace(2), 10)
{
    m_apts.need( 10 );
    for (int i = 0; i < Num_pts(); i++)
        RNZero(m_apts[i]);
}


/* ------------------------------------------------------------
 * DESCR   :	Print the knot vector and the points
 * ------------------------------------------------------------ */
template<class InPoint, class OutPoint, class OutVector>
inline
void
CRVNurbTC<InPoint, OutPoint, OutVector>::Print(WINbool in_bDoBasis) const 
{

    cout << "Order " << Order() << " Dimension " << OutDim() << " from " <<
        InDim() << " Num pts " << Num_pts() << " knot vector " << "\n";
 
    CRVKnot_vec::PrintKnotVec(in_bDoBasis);
    
    for (int i = 0; i < m_apts.num(); i++) {
        cout << m_apts[i] << "\n";
    }
   
}

/* ------------------------------------------------------------
 * DESCR   :	Write out and read in
 * DETAILS
 *    order
 *    dimension (of points+1)
 *    num of points
 *    num of data allocd
 *    knot dimension
 *    knot (RNvec::write)
 *    pts 0..n (RNpt::write)
 * ------------------------------------------------------------ */
template<class InPoint, class OutPoint, class OutVector>
inline
WINbool
CRVNurbTC<InPoint, OutPoint, OutVector>::Read(ifstream &in)
{
   WINbool out_bRes = CRVKnot_vec::ReadKnotVec(in);

   m_apts.need(Num_basis());

   for (int i = 0; i < m_apts.num(); i++) 
     out_bRes = (out_bRes && m_apts[i].Read(in)) ? TRUE : FALSE ;

   ASSERT(out_bRes);
   return out_bRes;
}

template<class InPoint, class OutPoint, class OutVector>
inline
void
CRVNurbTC<InPoint, OutPoint, OutVector>::Write(ofstream &out)
const 
{
   CRVKnot_vec::WriteKnotVec(out);
   
   for (int i = 0; i < m_apts.num(); i++) {
       out << m_apts[i];
       out << "\n";
   }
}

template<class InPoint, class OutPoint, class OutVector>
inline
WINbool
CRVNurbTC<InPoint, OutPoint, OutVector>::Read_binary(ifstream &in)
{
    WINbool bRes = CRVKnot_vec::Read_binary(in);
    
    if ( UTILSIoReadBinaryPtVec( m_apts, in, (double) 0.0 ) == FALSE )
        bRes = FALSE;

    ASSERT(bRes);
    return bRes;
}


template<class InPoint, class OutPoint, class OutVector>
inline
void
CRVNurbTC<InPoint, OutPoint, OutVector>::Write_binary(ofstream &out) const 
{
    CRVKnot_vec::Write_binary(out);
    
    VERIFY( UTILSIoWriteBinaryPtVec( m_apts, out, (double) 0.0 ) );
}

template<class InPoint, class OutPoint, class OutVector>
inline
WINbool 
CRVNurbTC<InPoint, OutPoint, OutVector>::Check( WINbool in_bDoKnots) const
{

    if (in_bDoKnots) {
        CRVKnot_vec::CheckKnotVec(TRUE);
        int i;
        for (i = 0; i < CRVKnot_vec::Num_knots()-1; i++) 
            ASSERT(Knot(i+1) >= Knot(i));
        
        for (i = 0; i < Num_pts(); i++) {
            for (int j = 0; j < Order()+3; j++)
                ASSERT(Knot(i+j) == Basis(i).Knot(j));
        }
    }

    return TRUE;
}

template<class InPoint, class OutPoint, class OutVector>
inline WINbool
CRVNurbTC<InPoint, OutPoint, OutVector>::Test()
{
    return TRUE;
}

