template<class Point, class Vector, class LineSeg>
inline
void CRVNurb_sampledTC<Point, Vector, LineSeg>::Samples(Array<Point> &in_apts) const
{
   if (m_aSamples.num() == 0 || m_aSamples.last().N() == 0) {
      in_apts.clearcompletely();
      return;
   }

   // return samples in lowest layer
   in_apts.need(m_aSamples.last().N());

   for (int i = 0; i < m_aSamples.last().N(); i++) {
      in_apts[i] = m_aSamples.last()[i];
   }
}

template<class Point, class Vector, class LineSeg>
inline
void CRVNurb_sampledTC<Point, Vector, LineSeg>::Samples(Array<Point> &in_apts,
                                                     double in_dSt, 
                                                     double in_dEt) const
{
   if (m_aSamples.num() == 0 || m_aSamples.last().N() == 0) {
      in_apts.clearcompletely();
      return;
   }
   in_apts.need(0);

   const CRVSamplesTC<Point, Vector, LineSeg> &smpls = m_aSamples.last();
   for (int i = 0; i < smpls.N(); i++) {
      if (smpls.T(i) < in_dSt) continue;
      if (smpls.T(i) > in_dEt) break;
      in_apts += smpls[i];
   }
}

template<class Point, class Vector, class LineSeg>
inline
void CRVNurb_sampledTC<Point, Vector, LineSeg>::Resample(double in_dAccuracy, 
                                                         double in_dMaxLen)
{
    m_aSamples.need(1);
    double tot_len = 0;
    double err = m_aSamples[0].Sample((*this), (this->Order() + 2));
    double maxLen = m_aSamples[0].Max_len(tot_len);
    int n_divs = 2;
    
    // Keep adding in more levels until we're close
    int last_ditch_count = 0;
    while (err > in_dAccuracy || maxLen > in_dMaxLen) {
        m_aSamples.need(m_aSamples.num() + 1);
        err = m_aSamples.last().Sample((*this), 
                                       m_aSamples[m_aSamples.num() - 2], 
                                       n_divs);
        maxLen = m_aSamples.last().Max_len(tot_len);
        if (m_aSamples.num() % 2 == 0) n_divs++;
        if (last_ditch_count++ > 100) ASSERT(FALSE);
    }
    //   TRACE("Sample depth %d\n", m_aSamples.num());
}


template<class Point, class Vector, class LineSeg>
inline
Point CRVNurb_sampledTC<Point, Vector, LineSeg>::Closest(const Point &in_pt, 
                                                         double &out_dT) const
{
    int intrvl = m_aSamples[0].Closest_interval(in_pt);
    for (int i = 1; i < m_aSamples.num() - 1; i++) {
        intrvl = m_aSamples[i].Closest_interval(in_pt, intrvl);
    }
    return m_aSamples.last().Closest(in_pt, intrvl, out_dT);
}

template<class Point, class Vector, class LineSeg>
inline
Point CRVNurb_sampledTC<Point, Vector, LineSeg>::Closest(const Point &in_pt, 
                                                         double &out_dT,
                                                         const double in_dT1,
                                                         const double in_dT2 ) const
{
    int intrvl = m_aSamples[0].Closest_intervalClipped(in_pt, in_dT1, in_dT2);
    for (int i = 1; i < m_aSamples.num() - 1; i++) {
        intrvl = m_aSamples[i].Closest_interval(in_pt, intrvl );
    }
    return m_aSamples.last().Closest(in_pt, intrvl, out_dT);
}


template<class Point, class Vector, class LineSeg>
inline
CRVNurbTC<Point, Point, Vector> &CRVNurb_sampledTC<Point, Vector, LineSeg >
::operator=(const CRVNurbTC<Point, Point, Vector> &crv) 
{ 
    CRVNurbTC<Point, Point, Vector>::operator=(crv); 
    return *this; 
}

template<class Point, class Vector, class LineSeg>
inline
CRVNurb_sampledTC<Point, Vector, LineSeg > &CRVNurb_sampledTC<Point, Vector, LineSeg >
::operator=(const CRVNurb_sampledTC<Point, Vector, LineSeg > &crv) 
{ 
   CRVNurbTC<Point, Point, Vector>::operator=(crv); 
   m_aSamples = crv.m_aSamples;

   return *this; 
}

template<class Point, class Vector, class LineSeg>
inline
CRVNurb_sampledTC<Point, Vector, LineSeg>
:: CRVNurb_sampledTC(const CRVSpace &in_crvspace, 
                     int             in_iNPts)
    : CRVNurbTC<Point, Point, Vector>(in_crvspace, in_iNPts) 
{ 
}


template<class Point, class Vector, class LineSeg>
inline
CRVNurb_sampledTC<Point, Vector, LineSeg>
:: CRVNurb_sampledTC(const CRVNurbTC<Point, Point, Vector> &in_crv, 
                     double in_dAcc, 
                     double in_dMax_len) 
    : CRVNurbTC<Point, Point, Vector>(in_crv) 
{ 
   Resample(in_dAcc, in_dMax_len); 
}


template<class Point, class Vector, class LineSeg>
inline
CRVNurb_sampledTC<Point, Vector, LineSeg>
::CRVNurb_sampledTC(const CRVNurb_sampledTC<Point, Vector, LineSeg> & in_crv)
{
    (*this) = in_crv;
}

template<class Point, class Vector, class LineSeg>
inline
void CRVNurb_sampledTC<Point, Vector, LineSeg>::ReadCrv(ifstream &in, 
                                                        double acc, 
                                                        double maxLen ) 
{ 
    this->Read( in ); 
   
    Resample( acc, maxLen ); 
}


template<class Point, class Vector, class LineSeg>
inline
WINbool
CRVNurb_sampledTC<Point, Vector, LineSeg>::test()
{
    return TRUE;
}
