template<class Point, class Vector, class LineSeg>
inline
int CRVSamplesTC<Point, Vector, LineSeg>::Closest_interval(const Point &in_pt, 
                                                             int in_iPrevInterval) const
{
    return Closest_intervalMe(in_iPrevInterval * m_iNDivs, 
                              (in_iPrevInterval +1) * m_iNDivs, in_pt);
}

template<class Point, class Vector, class LineSeg>
inline
Point 
CRVSamplesTC<Point, Vector, LineSeg>::Closest(const Point &in_pt, 
                                              int in_iPrevInterval) const
{
   double t = 0;
   return Closest(in_pt, in_iPrevInterval, t);
}




template<class Point, class Vector, class LineSeg>
inline
CRVSamplesTC<Point, Vector, LineSeg>::CRVSamplesTC(const CRVSamplesTC<Point, Vector, LineSeg> & in_samples)
{
    (*this) = in_samples;
}

template<class Point, class Vector, class LineSeg>
inline
CRVSamplesTC<Point, Vector, LineSeg> &CRVSamplesTC<Point, Vector, LineSeg>::
operator=(const CRVSamplesTC<Point, Vector, LineSeg> & in_samples)
{
    m_iNDivs = in_samples.m_iNDivs;
    m_aptSamples = in_samples.m_aptSamples;
    m_adTs = in_samples.m_adTs;

    return *this;
}

template<class Point, class Vector, class LineSeg>
inline
double CRVSamplesTC<Point, Vector, LineSeg>::Calc_err(const CRVNurbTC<Point, Point, Vector> &in_crv)
{
   double err = 0;
   const int n_err_divs = 20;
   const double dDiv = 1.0 / (double) (n_err_divs - 1);
   int count = 0;
   for (int i = 0; i < in_crv.Num_segments(); i++) {
      for (int j = 0; j < n_err_divs; j++) {
         Point pt = in_crv.PercSegment(i, j * dDiv);
         Point cpt = Closest(pt);
         err += Length(pt - cpt);
         count++;
      }
   }
   return err / count;
}

template<class Point, class Vector, class LineSeg>
inline
double CRVSamplesTC<Point, Vector, LineSeg>::Max_len(double &out_dTotLen) const
{
    out_dTotLen = 0;
    double len = 0;
    for (int i = 0; i < m_aptSamples.num()-1; i++) {
        double l = Length(m_aptSamples[i+1] - m_aptSamples[i]);
        len = WINmax(len, l);
        out_dTotLen += l;
    }
    return len;
}

template<class Point, class Vector, class LineSeg>
inline
Point 
CRVSamplesTC<Point, Vector, LineSeg>::Closest(const Point &in_pt) const
{
   LineSeg seg;
   int which = Closest_interval(in_pt);
   seg.P1() = m_aptSamples[which];
   seg.P2() = m_aptSamples[which+1];

   Point ret_pt =  seg.Project(in_pt);
   return ret_pt;
}

template<class Point, class Vector, class LineSeg>
inline
Point 
CRVSamplesTC<Point, Vector, LineSeg>::Closest(const Point &in_pt, 
                                              int in_iPrevInterval, 
                                              double &out_dT) const
{
   LineSeg seg;
   int which = Closest_interval(in_pt, in_iPrevInterval);
   seg.P1() = m_aptSamples[which];
   seg.P2() = m_aptSamples[which+1];
   Point out_pt;
   double dist_to_seg;
   seg.FindPtOnSeg(in_pt, out_pt, out_dT, dist_to_seg);
   out_dT = (1.0 - out_dT) * m_adTs[which] + out_dT * m_adTs[which+1];
   return out_pt;
}

template<class Point, class Vector, class LineSeg>
inline
double CRVSamplesTC<Point, Vector, LineSeg>::Sample(const CRVNurbTC<Point, Point, Vector> &in_crv, 
                                                 int in_iNSamples)
{
   ASSERT(in_iNSamples > 0);

   m_iNDivs = 1;
   const double dDiv = 1.0 / (in_iNSamples);
   m_aptSamples.need(in_iNSamples * in_crv.Num_segments() + 1);
   m_adTs.need(m_aptSamples.num());
   int count = 0;
   for (int i = 0; i < in_crv.Num_segments(); i++) {
      for (int j = 0; j < in_iNSamples; j++) {
         m_adTs[count] = in_crv.Segment_perc_t(i, j * dDiv);
         m_aptSamples[count] = in_crv(m_adTs[count]);
         count++;
      }
   }
   ASSERT(count == m_aptSamples.num() - 1);
   m_aptSamples.last() = in_crv.End_crv();
   m_adTs.last() = in_crv.End_t();

/*
   for (i = 0; i < m_aptSamples.num(); i++) {
      TRACE("%f  ", m_adTs[i]);
      for (int j = 0; j < m_aptSamples[i].Dim(); j++)
         TRACE("%f ", m_aptSamples[i][j]);
      TRACE("\n");
   }
   TRACE("\n");
*/
   return Calc_err(in_crv);
}

template<class Point, class Vector, class LineSeg>
inline
double 
CRVSamplesTC<Point, Vector, LineSeg>::Sample(const CRVNurbTC<Point, Point, Vector> &in_crv, 
                                         const CRVSamplesTC<Point, Vector, LineSeg> &in_aptPrevSamples, 
                                         int in_iDivs)
{
   ASSERT(in_iDivs > 0);
   m_iNDivs = in_iDivs;

   m_aptSamples.need((in_aptPrevSamples.N()-1) * in_iDivs + 1);
   m_adTs.need(m_aptSamples.num());
   int count = 0;
   for (int i = 0; i < in_aptPrevSamples.N()-1; i++) {
      double width = in_aptPrevSamples.T(i+1) - in_aptPrevSamples.T(i);
      double div = width / (double) m_iNDivs;
      for (int j = 0; j < m_iNDivs; j++) {
         m_adTs[count] = in_aptPrevSamples.T(i) + j * div;
         m_aptSamples[count] = in_crv(m_adTs[count]);
         count++;
      }
   }
   ASSERT(count == m_aptSamples.num() - 1);
   m_aptSamples.last() = in_crv.End_crv();
   m_adTs.last() = in_crv.End_t();

   /*
   for (i = 0; i < m_aptSamples.num(); i++)
      TRACE("%f   %f %f\n", m_adTs[i], m_aptSamples[i][0], m_aptSamples[i][1]);
   TRACE("\n");
   */
   double d = Calc_err( in_crv );
   return d;
}

template<class Point, class Vector, class LineSeg>
inline
int CRVSamplesTC<Point, Vector, LineSeg>::Closest_intervalMe(int in_iStart, 
                                                             int in_iEnd, 
                                                             const Point &in_pt) const
{
   double min_d = 1e30;
   int min_p = -1;
   for (int i = in_iStart; i < in_iEnd; i++) {
      double d = LengthSq(m_aptSamples[i] - in_pt);
      if (d < min_d) {
         min_d = d;
         min_p = i; 
      }
   }
   if (min_p == 0) return min_p;
   if (min_p == m_aptSamples.num()-1) return min_p-1;
   
   LineSeg seg1, seg2;
   seg1.P1() = m_aptSamples[min_p - 1];
   seg1.P2() = m_aptSamples[min_p];
   seg2.P1() = m_aptSamples[min_p];
   seg2.P2() = m_aptSamples[min_p + 1];

   if (seg1.Dist_to_seg(in_pt) < seg2.Dist_to_seg(in_pt))
     return min_p - 1;

   return min_p;
}

template<class Point, class Vector, class LineSeg>
inline
int CRVSamplesTC<Point, Vector, LineSeg>::Closest_intervalClipped(const Point &in_pt,
                                                                  const double in_dT1,
                                                                  const double in_dT2 ) const
{ 
    int iStart = 0;
    int iEnd = m_aptSamples.num();
    for ( int i = 0; i < m_aptSamples.num(); i++ ) {
        if ( m_adTs[i] < in_dT1 )
            iStart = i;
        if ( m_adTs[i] < in_dT2 )
            iEnd = i;
    }
    return Closest_intervalMe(iStart, iEnd, in_pt);
}   

template<class Point, class Vector, class LineSeg>
inline
int CRVSamplesTC<Point, Vector, LineSeg>::Closest_interval(const Point &in_pt ) const
{ 
    return Closest_intervalMe(0, m_aptSamples.num(), in_pt);
}   
