#ifndef __PM_EDGE_DEFS_H
#define __PM_EDGE_DEFS_H

/** An Edge.

    This class contains an Edge pointer. <br>

    Note: To get adjacent faces, and vertices, use the routines in PickMesh. <br>

    No important data in this class.

<br><br> Files:
<dd><li> include/utils/PM_Edge.H
<dd><li> utils/pickmesh/PMEdge.cpp
  */
class PMEdge  : public UTILSElement {
protected:
    Edge   m_eEdge;

    friend const Edge & PMGetEdge( const PMEdge &pmE ) { return pmE.m_eEdge; }
    friend       Edge & PMGetEdge(       PMEdge &pmE ) { return pmE.m_eEdge; }

    friend void PMSetEdgeMap( PMMesh &in_pm, int iIndx1, int iIndx2, int iId );

    int  m_iVertId1, m_iVertId2;

public:
    ///
    WINbool operator==( const PMEdge & ) const;
    /// I don't recommend calling this.
    PMEdge &operator= ( const PMEdge &);

    int         VertId1() const { return m_iVertId1; }
    int         VertId2() const { return m_iVertId2; }

    void        SetIdFromEdge(PMMesh &) ;
    void        SetEdgeFromId(PMMesh &) ;

    /// Don't call. Edges will be made (and deleted) as needed when faces are made.
    PMEdge( const PMEdge & );
    /// Don't call. Edges will be made (and deleted) as needed when faces are made.
    PMEdge( );
    /// Don't call. Edges will be deleted when their adjacent faces go away.
    virtual ~PMEdge( );

    /**@name Read and write. 

       When deriving, overload in derived class, then call this
       Read/Write to make sure that PMEdges's data gets read/written
       ok. */
    //@{
    ///
    virtual void Read  (ifstream &in);
    ///
    virtual void Write (ofstream &out) const;
    ///
    virtual void ReadBinary(ifstream &in);
    ///
    virtual void WriteBinary(ofstream &out) const;
    //@}
};

#endif
