#ifndef __PM_FACE_DEFS_H
#define __PM_FACE_DEFS_H

/** A Face.

    This class contains a Face pointer. <br>

    Note: To get adjacent edges and vertices, use the routines in PickMesh. <br>

    Data in this class: <br>

    A note on the normal data: These will be computed when
    PickMesh::compute_normals_and_colors is called. The above function
    is called after a Read or before a Draw. If you've been mucking
    with the mesh, call after you're done to reset the normals. <br>

<dd><li> R3Vec_f the normal of the face (only valid if PickMesh::compute_normals_and_colors has been called)
<dd><li> TextureMap Which texture map in PickMesh to use.
<dd><li> PMFaceType  For manifolds, whether the face is inside a vertex chart, lies across an edge, or in the inside of a face.  Used to figure out which texture map coordinates to use.
<dd><li> OwnerId User defined. For manifolds, the owning vertex, edge, or face chart.  

<br><br> Files:
<dd><li> include/utils/Pm_Face.H
<dd><li> utils/pickmesh/PMFace.cpp
*/
class PMFace  : public UTILSElement {
protected:
    Face    m_fFace;

    int        m_iFaceId;
    R3Vec      m_vecNorm;
    PMFaceType m_tFace;
    int        m_iTextureMap;
    int        m_iOwnerId;

    friend const Face & PMGetFace( const PMFace &pmF ) { return pmF.m_fFace; }
    friend       Face & PMGetFace(       PMFace &pmF ) { return pmF.m_fFace; }
    friend void PMSetFaceMap( PMMesh &in_pm, int iIndx, int iId );

public:
    int             FaceId()     const { return m_iFaceId; }
    /// Only valid if PMMesh::compute_normals_and_colors has been called.
    const R3Vec   & Norm()       const { return m_vecNorm; }
    /// Only valid if PMMesh::compute_normals_and_colors has been called.
          R3Vec   & Norm()             { return m_vecNorm; }
    ///
    PMFaceType      Type()       const { return m_tFace; }
    ///
    PMFaceType    & Type()             { return m_tFace; }
    ///
    int             TextureMap() const { return m_iTextureMap; }
    ///
    int           & TextureMap()       { return m_iTextureMap; }
    ///
    int             OwnerId()    const { return m_iOwnerId; }
    ///
    int           & OwnerId()          { return m_iOwnerId; }

    ///
    WINbool  operator==( const PMFace & ) const;
    /// Don't call.
    PMFace  &operator= ( const PMFace & );

    void        SetIdFromFace(PMMesh &) ;
    void        SetFaceFromId(PMMesh &) ;

    /// Don't call. Make faces using PMMesh::AddFace.
    PMFace( const PMFace & );
    /// Don't call. Make faces using PMMesh::AddFace.
    PMFace( );
    /// Don't call. Delete faces using PMMesh::RemoveFace
    virtual ~PMFace( );

    /**@name Read and write. 

       When deriving, overload in derived class, then call this
       Read/Write to make sure that PMEdges's data gets read/written
       ok. */
    //@{
    ///
    virtual void Read  (ifstream &in);
    ///
    virtual void Write (ofstream &out) const;
    ///
    virtual void ReadBinary(ifstream &in);
    ///
    virtual void WriteBinary(ofstream &out) const;
    //@}
};


#endif
