#ifndef _PMESHFEATURE_DEFS_H
#define _PMESHFEATURE_DEFS_H

#include <utils/Rn_Defs.H>
#include <utils/Pm_MeshLite.H>

/** \class PMeshFeatire Pm_MeshFeature.H utils/Pm_MeshFeature.H
  \ingroup Meshes
  \brief Find features on a mesh

   Given a mesh, find vertices/points that are "interesting". Sort of like sift features
    

<br><br> Files:
- include/utils/Pm_MeshFeature.H
- utils/MeshFeature/PMeshFeature.cpp */
class PMeshFeatures
{
private:
    const WINbool   m_bTrace;
    int             m_iNLevels;
    double          m_dAvgEdgeLen;
    Array< double > m_adScores;
    Array< pair< Vertex, int > > m_apairFeatures;

    const int m_iPowerDist;
    const WINbool m_bNormalizeArea;

    /// Calculate the scores
    void SiftScoresSetLevelsGauss( const PMeshLite &in_mesh ) ;
    void SiftScoresGaussFiltPosPos( const PMeshLite &in_mesh ) ;
    void SiftScoresGaussFiltPosCrv( const PMeshLite &in_mesh ) ;
    void SiftScoresGaussFiltCrv( const PMeshLite &in_mesh ) ;
    /// Calculate the scores
    void SiftScores( const PMeshLite &in_mesh ) ;
    /// Calculate the features after doing the scores
    void SiftMinMax( const PMeshLite &in_mesh );


    PMeshFeatures( const PMeshFeatures & );
    PMeshFeatures & operator=( const PMeshFeatures & ) const;
    bool operator==( const PMeshFeatures & ) const;
public:
    /**@name Acces */
    //@{
    /// Number of levels used
    int NLevels() const { return m_iNLevels; }
    /// Number of features
    int NFeatures() const { return m_apairFeatures.num(); }
    /// Actual feature
    const pair< Vertex, int > &Feature( const int in_i ) const { return m_apairFeatures[in_i]; }
    //@}

    /// Calc scores and features
    void CalcSift( const PMeshLite &in_mesh );

    PMeshFeatures() ;
    ~PMeshFeatures() { }

    ///
    void WriteBinary( const char *in_str ) const;
    ///
    void ReadBinary( const char *in_str );
};

#endif