/*************
 * Element routines
 *************/

inline
const PMVertex  & PMMesh::FindVertex  ( const Vertex in_v ) const
{
    return m_aoVerts[ m_aiMapVert[ m_mesh.vertexid( in_v ) ] ];
}

inline
PMVertex  & PMMesh::FindVertex  ( const Vertex in_v )
{
    return m_aoVerts[ m_aiMapVert[ m_mesh.vertexid( in_v ) ] ];
}

inline
const PMEdge    & PMMesh::FindEdge    ( const Edge in_e ) const 
{
    int v1 = m_mesh.vertexid( m_mesh.vertex1(in_e) );
    int v2 = m_mesh.vertexid( m_mesh.vertex2(in_e) );
   
    int iIndx = m_aaiMapEdgeIds[ WINmin(v1, v2) ].index( WINmax(v1,v2) );
    return m_aoEdges[ m_aaiMapEdge[ WINmin(v1, v2) ][iIndx] ];
}

inline
PMEdge    & PMMesh::FindEdge    ( const Edge in_e )
{
    int v1 = m_mesh.vertexid( m_mesh.vertex1(in_e) );
    int v2 = m_mesh.vertexid( m_mesh.vertex2(in_e) );
   
    int iIndx = m_aaiMapEdgeIds[ WINmin(v1, v2) ].index( WINmax(v1,v2) );
    return m_aoEdges[ m_aaiMapEdge[ WINmin(v1, v2) ][iIndx] ];
}

inline
const PMFace    & PMMesh::FindFace    ( const Face in_f ) const 
{
    return m_aoFaces[ m_aiMapFace[ m_mesh.faceid( in_f ) ] ];
}

inline
PMFace    & PMMesh::FindFace    ( const Face in_f )
{
    return m_aoFaces[ m_aiMapFace[ m_mesh.faceid( in_f ) ] ];
}




/*************
 * Changing topology routines
 *************/

inline
void PMMesh::delete_vertex( PMVertex &in_v )
{
    m_mesh.destroyVertex( PMGetVertex(in_v) );
}

inline
void PMMesh::add_vertex(PMVertex &in_v)
{
    PMGetVertex(in_v) = m_mesh.createVertex();
    PMSetMesh(in_v, &m_mesh);
    in_v.SetIdFromVert( *this );
    in_v.SetColor( m_colDefault );
}

/*************
 * All mesh elements access routines
 *************/
inline
int PMMesh::NumVertices() const
{
    return m_mesh.numVertices();
}

inline
int PMMesh::NumEdges()    const
{
    int iNEdges = 0;
    ForMeshEdge( m_mesh, e ) {
        iNEdges++;
        ASSERT(e);
    }EndFor;

    return iNEdges;
}

inline
int PMMesh::NumFaces()    const
{
    return m_mesh.numFaces();
}


/*************
 * All mesh element element access routines
 *************/
inline PMEdge * PMMesh::EdgeFromVertices   ( const PMVertex &in_v1,
                                               const PMVertex &in_v2 ) 
{
    Edge e = m_mesh.queryEdge( PMGetVertex( in_v1 ), PMGetVertex( in_v2 ) );
    if ( e )
        return &FindEdge( e );
    return NULL;
}

inline const PMEdge * PMMesh::CEdgeFromVertices ( const PMVertex &in_v1,
                                                    const PMVertex &in_v2 ) const
{
    Edge e = m_mesh.queryEdge( PMGetVertex( in_v1 ), PMGetVertex( in_v2 ) );
    if ( e )
        return &FindEdge( e );
    return NULL;
}


inline
Array<const PMVertex *>   PMMesh::CVertexVertices  ( const PMVertex &in_v ) const
{
    Array<const PMVertex *> out_vs;
    ForVertexVertex( m_mesh, PMGetVertex(in_v), ov ) {
        out_vs += &FindVertex( ov );
    } EndFor;

    return out_vs;
}

inline
Array<PMVertex *>   PMMesh::VertexVertices  ( const PMVertex &in_v ) 
{
    Array<PMVertex *> out_vs;
    ForVertexVertex( m_mesh, PMGetVertex(in_v), ov ) {
        out_vs += &FindVertex( ov );
    } EndFor;

    return out_vs;
}

inline
Array<const PMEdge *>     PMMesh::CVertexEdges     ( const PMVertex &in_v ) const
{
    Array<const PMEdge *> out_es;

    Vertex v = PMGetVertex(in_v);
    ForVertexEdge( m_mesh, v, oe ) {
        out_es += &FindEdge( oe );
    } EndFor;

    return out_es;
}

inline
Array<PMEdge *>     PMMesh::VertexEdges     ( const PMVertex &in_v ) 
{
    Array<PMEdge *> out_es;

    Vertex v = PMGetVertex(in_v);
    ForVertexEdge( m_mesh, v, oe ) {
        out_es += &FindEdge( oe );
    } EndFor;

    return out_es;
}

inline
Array<const PMFace *>     PMMesh::CVertexFaces     ( const PMVertex &in_v ) const
{
    Array<const PMFace *> out_fs;
    ForVertexFace( m_mesh, PMGetVertex(in_v), of ) {
        out_fs += &FindFace( of );
    } EndFor;

    return out_fs;
}

inline
Array<PMFace *>     PMMesh::VertexFaces     ( const PMVertex &in_v ) 
{
    Array<PMFace *> out_fs;
    ForVertexFace( m_mesh, PMGetVertex(in_v), of ) {
        out_fs += &FindFace( of );
    } EndFor;

    return out_fs;
}

inline
Array<const PMVertex *> PMMesh::CVertexVerticesOrdered ( const PMVertex &in_v ) const
{
    Array<const PMVertex *> out_vs;
    ForVertexCcwVertex( m_mesh, PMGetVertex(in_v), ov ) {
        out_vs += &FindVertex( ov );
    } EndFor;

    return out_vs;
}

inline
Array<PMVertex *> PMMesh::VertexVerticesOrdered ( const PMVertex &in_v ) 
{
    Array<PMVertex *> out_vs;
    ForVertexCcwVertex( m_mesh, PMGetVertex(in_v), ov ) {
        out_vs += &FindVertex( ov );
    } EndFor;

    return out_vs;
}


inline
Array<const PMFace *>     PMMesh::CVertexFacesOrdered     ( const PMVertex &in_v ) const
{
    Array<const PMFace *> out_fs;
    ForVertexCcwFace( m_mesh, PMGetVertex(in_v), of ) {
        out_fs += &FindFace( of );
    } EndFor;

    return out_fs;
}

inline
Array<PMFace *>     PMMesh::VertexFacesOrdered     ( const PMVertex &in_v ) 
{
    Array<PMFace *> out_fs;
    ForVertexCcwFace( m_mesh, PMGetVertex(in_v), of ) {
        out_fs += &FindFace( of );
    } EndFor;

    return out_fs;
}

inline
const PMVertex &          PMMesh::CEdgeVertex1 ( const PMEdge &in_e  ) const
{
    return FindVertex( m_mesh.vertex1( PMGetEdge( in_e ) ) );
}

inline
PMVertex &          PMMesh::EdgeVertex1 ( const PMEdge &in_e  ) 
{
    return FindVertex( m_mesh.vertex1( PMGetEdge( in_e ) ) );
}

inline
const PMVertex &          PMMesh::CEdgeVertex2 ( const PMEdge &in_e  ) const
{
    return FindVertex( m_mesh.vertex2( PMGetEdge( in_e ) ) );
}

inline
PMVertex &          PMMesh::EdgeVertex2 ( const PMEdge &in_e  ) 
{
    return FindVertex( m_mesh.vertex2( PMGetEdge( in_e ) ) );
}

inline
const PMFace *            PMMesh::CEdgeFace1   ( const PMEdge &in_e  ) const
{
    Face f = m_mesh.face1( PMGetEdge( in_e ) );
    if ( f == NULL )
        return NULL;

    return &FindFace( f );
}

inline
PMFace *            PMMesh::EdgeFace1   ( const PMEdge &in_e  ) 
{
    Face f = m_mesh.face1( PMGetEdge( in_e ) );
    if ( f == NULL || (int) f == 0xfdfdfdfd )
        return NULL;

    return &FindFace( f );
}

inline
const PMFace *            PMMesh::CEdgeFace2   ( const PMEdge &in_e  ) const
{
    Face f = m_mesh.face2( PMGetEdge( in_e ) );
    if ( f == NULL )
        return NULL;

    return &FindFace( f );
}

inline
PMFace *            PMMesh::EdgeFace2   ( const PMEdge &in_e  ) 
{
    Face f = m_mesh.face2( PMGetEdge( in_e ) );
    if ( f == NULL )
        return NULL;

    return &FindFace( f );
}


inline
Array<const PMVertex *>   PMMesh::CFaceVertices    ( const PMFace &in_f )   const
{
    Array<const PMVertex *> out_vs;
    ForFaceVertex( m_mesh, PMGetFace(in_f), ov ) {
        out_vs += &FindVertex( ov );
    } EndFor;

    return out_vs;
}

inline
Array<PMVertex *>   PMMesh::FaceVertices    ( const PMFace &in_f )   
{
    Array<PMVertex *> out_vs;
    ForFaceVertex( m_mesh, PMGetFace(in_f), ov ) {
        out_vs += &FindVertex( ov );
    } EndFor;

    return out_vs;
}

inline
Array<const PMEdge *>     PMMesh::CFaceEdges       ( const PMFace &in_f )   const
{
    Array<const PMEdge *> out_es;
    ForFaceEdge( m_mesh, PMGetFace(in_f), oe ) {
        out_es += &FindEdge( oe );
    } EndFor;

    return out_es;
}

inline
Array<PMEdge *>     PMMesh::FaceEdges       ( const PMFace &in_f )   
{
    Array<PMEdge *> out_es;
    ForFaceEdge( m_mesh, PMGetFace(in_f), oe ) {
        out_es += &FindEdge( oe );
    } EndFor;

    return out_es;
}

/**************
 * Boundary routines
 **************/
inline

WINbool PMMesh::IsBoundary( const PMVertex &in_pmV ) const 

{

    return in_pmV.IsBoundary();

}



inline

WINbool PMMesh::IsMultipleBoundary( const PMVertex &in_pmV ) const 

{

    return ( m_mesh.isNice( PMGetVertex( in_pmV ) ) ) ? FALSE : TRUE;

}



inline
WINbool PMMesh::IsBoundary( const PMEdge &in_pmE )   const 
{
    return m_mesh.isBoundary( PMGetEdge( in_pmE ) ) ? TRUE : FALSE;
}


inline int PMMesh::NSides( const PMFace &in_f ) const 
{
    return m_mesh.numVertices( PMGetFace( in_f ) );
}
