/*************
 * Element routines
 *  May be overridden if PMVertex/PMEdge/PMFace want to be overridden
 *************/
inline
PMVertex & PMMesh::NewVertex()
{
    return m_aoVerts.AddElement( new PMVertex );
}

inline
PMEdge   & PMMesh::NewEdge()
{
    return m_aoEdges.AddElement( new PMEdge );
}

inline
PMFace   & PMMesh::NewFace()
{
    return m_aoFaces.AddElement( new PMFace );
}

inline
void PMMesh::KillVertex( PMVertex *io_v )
{
    m_aoVerts.UnIdElement( *io_v );
    m_aopDeadVs += io_v;
}

inline
void PMMesh::KillEdge  ( PMEdge   *io_e )
{
    m_aoEdges.UnIdElement( *io_e );
    m_aopDeadEs += io_e;
}

inline
void PMMesh::KillFace  ( PMFace   *io_f )
{
    m_aoFaces.UnIdElement( *io_f );
    m_aopDeadFs += io_f;
}

inline
int PMMesh::CAllMeshVertices ( Array<const PMVertex *> &out_vs ) const
{
    return m_aoVerts.CGetAll( out_vs );
}

inline
int PMMesh::AllMeshVertices ( Array<PMVertex *> &out_vs ) 
{
    return m_aoVerts.GetAll( out_vs );
}

inline
int PMMesh::CAllMeshEdges    ( Array<const PMEdge *> &out_es ) const
{
    return m_aoEdges.CGetAll( out_es );
}

inline
int PMMesh::AllMeshEdges    ( Array<PMEdge *> &out_es ) 
{
    return m_aoEdges.GetAll( out_es );
}

inline
int PMMesh::CAllMeshFaces    ( Array<const PMFace *> &out_fs) const
{
    return m_aoFaces.CGetAll( out_fs );
}

inline
int PMMesh::AllMeshFaces    ( Array<PMFace *> &out_fs) 
{
    return m_aoFaces.GetAll( out_fs );
}


inline
void PMSetVertMap( PMMesh &in_pm, int iIndx, int iId )
{
    in_pm.m_aiMapVert.needSet( iIndx+1, -1 );
    in_pm.m_aiMapVert[iIndx] = iId;
}

inline 
void PMSetEdgeMap( PMMesh &in_pm, int iIndx1, int iIndx2, int iId )
{
    in_pm.m_aaiMapEdgeIds.need ( WINmax(iIndx1+1,in_pm.m_aaiMapEdgeIds.num()) );
    in_pm.m_aaiMapEdge.need    ( WINmax(iIndx1+1,in_pm.m_aaiMapEdge.num()) );
    int iIndx = in_pm.m_aaiMapEdgeIds[iIndx1].index( iIndx2 );
    if ( iIndx == -1 ) {
        in_pm.m_aaiMapEdgeIds[iIndx1] += iIndx2;
        in_pm.m_aaiMapEdge[iIndx1] += iId;
    } else {
        in_pm.m_aaiMapEdge[iIndx1][iIndx] = iId;
    }
}

inline
void PMSetFaceMap( PMMesh &in_pm, int iIndx, int iId )
{
    in_pm.m_aiMapFace.needSet( iIndx+1, -1 );
    in_pm.m_aiMapFace[iIndx] = iId;
}


/*************
 * Changing topology routines
 *************/
inline
void PMMesh::RemoveVertex( PMVertex &io_pm )
{
    io_pm.OwnerId() = -1;

    RemoveAdjacentFaces( io_pm );

    // Remove the vertex from the Gmesh/bmesh
    delete_vertex( io_pm );

    // Remove from the appropriate UTILSElement_array
    m_aoVerts.RemoveElement( io_pm );
    // Actually kill it
    KillVertex( &io_pm );

    m_bHasChanged = TRUE;
}

inline
PMVertex &PMMesh::AddVertex()
{
    // Get vertex from UTILSElement_array
    PMVertex &nv = NewVertex();

    // Fill in with underlying mesh data
    add_vertex( nv );

    m_bHasChanged = TRUE;

    return nv;
}

inline
PMVertex &PMMesh::AddVertex(const R3Pt &in_pt)
{
    // Get vertex from UTILSElement_array
    PMVertex &nv = NewVertex();

    // Fill in with underlying mesh data
    add_vertex( nv );

    nv.SetLoc(in_pt);

    m_bHasChanged = TRUE;

    return nv;
}

inline
void PMMesh::RemoveFace( PMFace &io_f )
{
    io_f.OwnerId() = -1;
    io_f.TextureMap() = -1;
    io_f.Type() = PM_NONE;

    // Remove Face from underlying mesh
    //  also removes PMEdges
    delete_face( io_f );

    // Remove Face from UTILSElement_array
    m_aoFaces.RemoveElement( io_f );
    // Actually kill it
    KillFace( &io_f );

    m_bHasChanged = TRUE;
}


inline

void PMMesh::RemoveEdge( PMEdge &io_e )

{

    if ( io_e.UTILSValid() == FALSE )

        return;



    PMFace *opF1 = EdgeFace1( io_e );

    PMFace *opF2 = EdgeFace2( io_e );



    if ( opF1 != NULL )

        RemoveFace( *opF1 );



    if ( opF2 != NULL )

        RemoveFace( *opF2 );

}


/**************
 * Texture map routines
 **************/

inline
const PMTextureMap &PMMesh::TextureMap( int in_iIndx ) const 
{
    return *m_aopTextureMaps[ in_iIndx ];
}

inline
PMTextureMap &PMMesh::TextureMap( int in_iIndx ) 
{
    return *m_aopTextureMaps[ in_iIndx ];
}

inline
PMTextureMap &PMMesh::AddTextureMap()
{
    m_aopTextureMaps.add(1);
    m_aopTextureMaps.last() = new PMTextureMap( m_aopTextureMaps.num()-1 );
    return *m_aopTextureMaps.last();
}

inline
void PMMesh::RemoveAllTextureMaps()
{
    for (int i = 0; i < m_aopTextureMaps.num(); i++)
        delete m_aopTextureMaps[i];

    m_aopTextureMaps.clearcompletely();
}


inline void PMMesh::RemoveTextureMap( const int in_iWhich )

{

    delete m_aopTextureMaps[in_iWhich];

    for ( int i = in_iWhich; i < m_aopTextureMaps.num()-1; i++ )

        m_aopTextureMaps[i] = m_aopTextureMaps[i+1];



    m_aopTextureMaps.sub(1);

}


/**************
 * Constructor/destructors
 **************/

inline PMMesh::~PMMesh() 
{
    Clear();

    for (int i = 0; i < m_aopTextureMaps.num(); i++)
        delete m_aopTextureMaps[i];

    for (FORINT i = 0; i < m_aopDeadVs.num(); i++)
        delete m_aopDeadVs[i];

    for (FORINT i = 0; i < m_aopDeadEs.num(); i++)
        delete m_aopDeadEs[i];

    for (FORINT i = 0; i < m_aopDeadFs.num(); i++)
        delete m_aopDeadFs[i];

    m_aopDeadVs.clearcompletely();
    m_aopDeadEs.clearcompletely();
    m_aopDeadFs.clearcompletely();
}



