#ifndef _SUBDIV_PMSUBDIV_DEFS_H
#define _SUBDIV_PMSUBDIV_DEFS_H

#include <utils/Pm_Mesh.H>

class PMSubdivVertex;

/** A subdivision point.
<br><br> Files:
<dd><li> include/utils/Pm_SubdivPt.H
<dd><li> src/utils/pickmesh/PMSubdivPt.cpp
 */
class PMSubdivPt {
protected:
    R3Pt   m_pt;

    Array< double > m_adVertPerc;
    Array< int >    m_aiVertId;

    PMSubdivVertex *m_opSubVert;

public:
    /**@name Subdivision point */
    //@{
    /// How many vertices in linear combination
    int    NBlend()            const { return m_aiVertId.num(); }
    /// Vertex id 
    int    Vert(int in_i)      const { return m_aiVertId[in_i]; }
    ///
    const Array<int> &Ids() const { return m_aiVertId; }
    /// How much of vertex i
    double VertBlend(int in_i) const { return m_adVertPerc[in_i]; }

    /// Return the calculated point
    const R3Pt &SubdivPt( ) const { return m_pt; }
    /// Calculate the value of the subdivision point using the blend values
    const R3Pt &CalcPt( const PMMesh &in_oTopMesh );
    //@}

    /**@name Alternative to Add, much faster at cost of space */
    //@{
    ///
    void Expand(int in_i);
    ///
    void AddExpanded( const PMSubdivPt &, double in_dScl );
    ///
    void Compress();
    //@}

    /**@name The sub vertex */
    //@{
    ///
    PMSubdivVertex *&SubVert() { return m_opSubVert; }
    ///
    const PMSubdivVertex *SubVert() const { return m_opSubVert; }
    ///
    void Renumber( const Array<int> &in_aiMap );

    //@}

    /**@name Constructing */
    //@{
    ///
    void Clear();
    ///
    void Add( int in_iId, double in_dBlend );
    ///
    void Set( const R3Pt &in_pt ) { m_pt = in_pt; }
    ///
    void Set( const Array<int> &in_iId, const Array<double> &in_ad );
    ///
    void Add( const Array<int> &in_iId, const Array<double> &in_ad, double in_dBlend );
    ///
    void Add( const PMSubdivPt &, double in_dBlend );
    ///
    void Scale(  double in_dScl );
    ///
    void Map( const Array<int> &in_aiMapIds );
    ///
    WINbool CheckSum() const;
    ///
    PMSubdivPt &operator=( const PMSubdivPt & );
    //@}

    ///
    PMSubdivPt() : m_pt(0,0,0), m_opSubVert(NULL) {}
    ///
    virtual ~PMSubdivPt() {}

    /**@name Read and write. 

       Labeled funny so the virtual table doesn't get confused with the UTILSElement
       read/write.
       ok. */
    //@{
    ///
    virtual void WriteSPt(ofstream &out) const ;
    ///
    virtual void ReadSPt(ifstream &in);
    ///
    virtual void ReadBinarySPt(ifstream &in);
    ///
    virtual void WriteBinarySPt(ofstream &out) const;
    //@}
    
};

#endif

