#ifndef _S4_QUATERNIAN_H_DEFS

#define _S4_QUATERNIAN_H_DEFS



#include <WINSystemDefines.H>

#include <utils/Rn_Affine4.H>



class S4Quaternian {

protected:

    R4Vec m_vec;



public:

    /**@name Access */

    //@{

    ///

    double Angle() const { return m_vec[0]; }

    ///

    R3Vec  Axis() const { return R3Vec(m_vec[1], m_vec[2], m_vec[3]); }

    ///

    double X() const { return m_vec[1]; }

    ///

    double Y() const { return m_vec[2]; }

    ///

    double Z() const { return m_vec[3]; }

    ///

    double Length() const { return ::Length( m_vec ); }

    //@}



    /**@name Converting */

    //@{

    R3Matrix Convert() const;

    S4Quaternian operator *(const S4Quaternian &q_) const;

    //@}



    /**@name Setting */

    //@{

    /// 

    void SetAngle( const double in_dAng ) { m_vec[0] = in_dAng; }

    ///

    void SetAxis( const R3Vec &in_vec );

    ///

    void SetAngleAxis( const double in_dAng, const R3Vec &in_vec ) { SetAngle( in_dAng ); SetAxis( in_vec ); }

    ///

    void Set( const R3Vec &in_vec1, const R3Vec &in_vec2 );

    //@}



    ///

    S4Quaternian(const double in_dAng, const R3Vec &in_vec) { SetAngleAxis( in_dAng, in_vec ); }

    ///

    S4Quaternian() : m_vec(0, 0, 1, 0) { }

    ///

    ~S4Quaternian() {}



};



inline ostream& operator<<( ostream &out, const S4Quaternian &in_quat )

{

    out << in_quat.Angle() << " " << in_quat.Axis();



    return out;

}



inline istream& operator>>( istream &in, S4Quaternian &out_quat )

{

    double dAng;

    R3Vec vecAxis;

    in >> dAng >> vecAxis;



    out_quat.SetAngleAxis( dAng, vecAxis );



    return in;

}





inline void S4Quaternian::SetAxis( const R3Vec &in_vec ) 

{

    m_vec[1] = in_vec[0];

    m_vec[2] = in_vec[1];

    m_vec[3] = in_vec[2];

}



inline R3Matrix S4Quaternian::Convert() const

{

    const double qw = m_vec[0];

    const double qx = m_vec[1];

    const double qy = m_vec[2];

    const double qz = m_vec[3];



    const double q_len = 0.5 * ::Length( m_vec );



	R3Matrix Q = R3Matrix::Identity();



	Q(0,0) = 2.0 * ( q_len - qy * qy - qz * qz);

    Q(0,1) = 2.0 * ( qx * qy + qw * qz );

    Q(0,2) = 2.0 * ( qx * qz - qw * qw );

    Q(0,3) = 0.0;



    Q(1,0) = 2.0 * ( qx * qy - qw * qz );

	Q(1,1) = 2.0 * ( q_len - qx * qx - qz * qz );

    Q(1,2) = 2.0 * ( qw * qx + qy * qz );

    Q(1,3) = 0.0;



    Q(2,0) = 2.0 * ( qw * qy + qx * qz );

    Q(2,1) = 2.0 * ( qy * qz - qw * qx );

    Q(2,2) = 2.0 * ( q_len  - qx * qx - qz * qz );

    Q(2,3) = 0.0;

    Q(3,0) = 0.0;

	Q(3,1) = 0.0;

    Q(3,2) = 0.0;

	Q(3,3) = q_len;



    return Q;

}





inline

S4Quaternian S4Quaternian::operator *(const S4Quaternian &in_q) const

{

    const R3Vec vecAxis1 = in_q.Axis();

    const R3Vec vecAxis2 = Axis();



    const double dAng1 = in_q.Angle();

    const double dAng2 = Angle();



	const R3Vec vec = vecAxis2 * dAng2

		    	    + vecAxis1 * dAng1 

			        + Cross( vecAxis1, vecAxis2 );



	const double dAng =  dAng1 * dAng2 - Dot( vecAxis1, vecAxis2 );



    return S4Quaternian( dAng, vec );

}



inline

void S4Quaternian::Set( const R3Vec &in_vec1, const R3Vec &in_vec2 )

{

    

}





#endif