#ifndef _SRF_SPATCH_DEFS_H_
#define  _SRF_SPATCH_DEFS_H_

#include <utils/Rn_Defs.H>
#include <utils/Rn_Polygon.H>

class PMeshLite;

class SRFSPatch {
protected:
    // Data that is the same for any patch of the same number of sides, depth

    const Array< Array<int> > * m_aaipIndex;
    const Array<int>          * m_aipDFact;

    R2BoxedPolygon m_poly;
    int m_iDepth;

    Array<R3Pt> m_aptCtrl;

public:
    /**@name Access */
    //@{
    ///
    int NSides() const { return m_poly.Num_pts(); }
    ///
    int Depth() const { return m_iDepth; }
    ///
    int NPts() const { return m_aptCtrl.num(); }
    ///
    const R3Pt &Pt( const int in_i ) const { return m_aptCtrl[in_i]; }
    ///
    const Array<R3Pt> &Pts() const { return m_aptCtrl; }
    ///
    const R2BoxedPolygon &Domain() const { return m_poly; }
    ///
    const Array< Array<int> > &Index() const { return *m_aaipIndex; }
    ///
    const Array< int > &Fact() const { return *m_aipDFact; }
    ///
    int Fact( const int in_i ) const { return (*m_aipDFact)[in_i]; }
    //@}

    /**@name Editing */
    //@{
    ///
    R3Pt &Pt( const int in_i ) { return m_aptCtrl[in_i]; }
    ///
    void SetDomain( const R2Pt &in_pt, const double in_dR, const double in_dRotate = 0.0 );
    ///
    void SetDomain( const R2Polygon &in_poly );
    //@}

    /**@name Evaluation */
    //@{
    ///
    Array<R2Pt> Samples( const int in_iNPerSide, const int in_iNRadial, Array<R4Pt_i> &out_aiptFaces, Array<R3Pt_i> &out_aiptFaces2 ) const;
    ///
    R3Pt operator()( const R2Pt &in_pt ) const;
    ///
    void Basis( const R2Pt &in_pt, Array<double> &out_adBasis ) const;
    ///
    Array<double> Basis( const R2Pt &in_pt,const Array<int> &in_aiWhichPt ) const;
    ///
    /** Return the indices and values of the non-zero second derivative basis functions for $(u, v)$.
        The out_* arrays are all the same length.
        @param out_adValues B_u(in_dU) * B_v(in_dV)
        */
    void BasisDerivative( const int in_iWhich, const R2Pt &in_pt, 
                          Array<double> & out_adValues ) const;
    ///
    Array<double> BasisDerivative( const int in_iWhich, const R2Pt &in_pt,const Array<int> &in_aiWhichPt ) const;
    /** Return the indices and values of the non-zero derivative basis functions for $(u, v)$.
        The out_* arrays are all the same length.
        @param out_adValues B_u(in_dU) * B_v(in_dV)
        */
    void BasisSecondDerivative( const R2Pt_i &in_iWhich, const R2Pt &in_pt, 
                                Array<double> & out_adValues ) const;
    ///
    Array<double> BasisSecondDerivative( const R2Pt_i &in_iWhich, const R2Pt &in_pt, const Array<int> &in_aiWhichPt ) const;
    ///
    R3Vec Derivative( const int in_iWhich, const R2Pt & in_ptUV ) const;
    ///
    R3Vec SecondDerivative( const R2Pt_i &in_iptWhich, const R2Pt & in_ptUV ) const;
    ///
    void AllEval( const R2Pt &in_pt,
                  R3Pt &out_pt,
                  R3Vec &out_vecDs, R3Vec &out_vecDt,
                  R3Vec &out_vecDss, R3Vec &out_vecDtt,
                  R3Vec &out_vecDst ) const;
    ///
    void AllEval( const R2Pt &in_pt, const Array<int> &in_aiWhichPt,
                  Array<double> &out_ad,
                  Array<double> &out_adDs,  Array<double> &out_adDt,
                  Array<double> &out_adDss, Array<double> &out_adDtt,
                  Array<double> &out_adDst ) const;
    ///
    void AllEval( const R2Pt &in_pt,
                  R3Pt &out_pt,
                  R3Vec &out_vecDs, R3Vec &out_vecDt ) const;
    ///
    void AllEval( const R2Pt &in_pt, const Array<int> &in_aiWhichPt,
                  Array<double> &out_ad,
                  Array<double> &out_adDs,  Array<double> &out_adDt ) const;
    ///
    R3Vec Normal( const R2Pt &in_p ) const;
    //@}
    
    /**@name constructor */
    //@{
    ///
    SRFSPatch &operator=( const SRFSPatch & );
    ///
    WINbool operator==( const SRFSPatch & ) const;
    ///
    SRFSPatch( const int in_iNSides = 4, const int in_iDegree = 2 );
    ///
    SRFSPatch( const R2Polygon &in_poly, const int in_iDegree = 2 );
    ///
    SRFSPatch( const SRFSPatch & );
    ///
    ~SRFSPatch( ) { }
    //@}

    /**@name read/write */
    //@{
    ///
    void Read( ifstream &in );
    ///
    void Write( ofstream &out ) const;
    ///
    void ReadBinary( ifstream &in );
    ///
    void WriteBinary( ofstream &out ) const;
    //@}

    ///
    void TestDeriv(const int in_iNDivs) const;
    ///
    void Srf( PMeshLite &in_mesh, const int in_iNDivs );
};

#endif
