// STATDistribution.h: interface for the STATDistribution class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_STATDISTRIBUTION_H__B2A3F93C_D1C7_43CC_9865_FFFEE93636CA__INCLUDED_)
#define AFX_STATDISTRIBUTION_H__B2A3F93C_D1C7_43CC_9865_FFFEE93636CA__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <utils/Mesh_Array.H>

/** STATDistribution
 * A distribution of objects
 */
class STATDistribution  
{
protected:
    int        m_iNBoxes, m_iNTotal;
    Array<int> m_aiNInBox, m_aiNToLeft, m_aiSample;

    double     m_dScl;
    int        m_iMiddle, m_iWidth;

    STATDistribution &operator=( const STATDistribution & );
    WINbool operator==( const STATDistribution & );
    STATDistribution( const STATDistribution & );
public:
    /**@name Access */
    //@{
    ///
    int NBoxes() const { return m_iNBoxes; }
    ///
    int Sample() const;
    ///
    int SampleGaussian() const;
    ///
    int SampleObject() const;
    ///
    double PercInBox( const int in_iBox ) const;
    ///
    int    NumberInBox( const int in_iBox ) const { return m_aiNInBox[in_iBox]; }
    //@}

    /**@name Creation */
    //@{
    //
    void SetGaussian( const double in_dScl, const int in_iMiddle, const int in_iWidth );
    ///
    void SetNBoxes( const int in_iN );
    ///
    void ClearBoxes();
    ///
    void AddBox( const int in_iSample );
    ///
    void AddObject( const int in_iObject );
    //@}

    ///
	STATDistribution();
    ///
	virtual ~STATDistribution();

    /**@name Read/write */
    //@{
    ///
    void Read( ifstream &in );
    ///
    void Write( ofstream &out ) const;
    //@}

    static WINbool Test();
};

#endif // !defined(AFX_STATDISTRIBUTION_H__B2A3F93C_D1C7_43CC_9865_FFFEE93636CA__INCLUDED_)

