// STATKMeans.h: interface for the STATKMeans class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_STATKMEANS_H__CB50FF20_5ABC_4004_BFFB_B6DB7E9D38F7__INCLUDED_)
#define AFX_STATKMEANS_H__CB50FF20_5ABC_4004_BFFB_B6DB7E9D38F7__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <utils/Mesh_Array.H>
#include <ANN/Ann.h>

/* KMeans finder */
class STATKMeans  
{
protected:
    double                    m_dMoveThreshold;
    double                    m_dAvgDiff; // Maximum average difference between data and center
    Array<double>             m_adCenterWidth;
    const int                 m_iNeighSize;
    int                       m_iNData, m_iSize, m_iNCenters;

    /* KD tree */
    ANNpointArray	 m_aAnnPtsData;		// data points
    ANNpointArray  	 m_aAnnPtsCenters;	// Centers
    ANNpointArray  	 m_aAnnPtsRange;	// Range around center
    Array<int>       m_aiNeigh;			// near neighbor indices
    Array<double>    m_adNeighDist;		// near neighbor distances
    ANNkd_tree		*m_KDTree;		    // search structure

    Array<int>          m_aiDataCenter, m_aiCount, m_aiFree;
    Array< Array<int> > m_aaiCenter;

    ofstream        *m_strmOut;

    void AvgDists ( ) ;
    void CalculateMeans( ) ;

    void BuildTree();
    int  ClosestInTree( const int in_i ) ;

    void AssignRandom();
    // return number reclassified
    int  ReClassify();

    int CheckDiff();

    WINbool Iterate( const int in_iNTimes );

    STATKMeans &operator=( const STATKMeans & );
    WINbool operator==( const STATKMeans & );
    STATKMeans( const STATKMeans & );

    void IncreaseSize( const int in_iN );
    void DecreaseSize( const int in_iN );
    void DeleteData();
    void SetRange();

public:
    /**@name Access */
    //@{
    ///
    double AvgDiff() const { return m_dAvgDiff; }
    ///
    int Size() const { return m_iSize; }
    ///
    int NData() const { return m_iNData; }
    ///
    int NCenters( ) const { return m_iNCenters; }
    ///
    int NInCenter( const int in_iCenter ) const { return m_aaiCenter[in_iCenter].num(); }
    ///
    int DataInCenter( const int in_iCenter, const int in_iIndx ) const { return m_aaiCenter[in_iCenter][in_iIndx]; }
    ///
    int AssignedCenter( const int in_iDataId ) const;
    ///
    const double *CenterValue( const int in_iCenter ) const;
    ///
    const double *Range( const int in_iCenter ) const;
    ///
    double CenterAvgDiff( const int in_iCenter ) const { return m_adCenterWidth[ in_iCenter ]; }
    //@}

    /**@name Create */
    //@{
    ///
    void SetOutput( ofstream *strmOut ) { m_strmOut = strmOut; }
    ///
    void SetAvgDiff( const double in_d ) { m_dAvgDiff = in_d; }
    ///
    void SetMoveThreshold( const double in_d ) { m_dMoveThreshold = WINminmax(in_d, 0.0, 1.0); }
    ///
    void SetNCenters( const int in_i, const int in_iSize );
    ///
    void SetData( const SArray<const Array<double> *> &in_p);
    ///
    WINbool Solve( const int in_iTryTimes );
    ///
    int Query(  Array<double> &);
    //@}

    ///
	STATKMeans();
    ///
	virtual ~STATKMeans();

    ///
    void Read( ifstream &in );
    ///
    void Write( ofstream &out ) const;
};

#endif // !defined(AFX_STATKMEANS_H__CB50FF20_5ABC_4004_BFFB_B6DB7E9D38F7__INCLUDED_)

