#include "StdAfx.H"
#include <MC/MC_Point.H>
#include <MC/MC_Data.H>
#include <MC/MC_AllMetrics.H>
#include <MC/MC_MeshUtils.H>
#include <utils/Pm_MeshLite.H>

MCAllMetrics::MCAllMetrics()
{
    m_pMeshDescription = NULL;
    m_aData.init(MCData::NUM_METHODS);
    for( int i=0; i<MCData::NUM_METHODS; i++ ) {
        m_aData[i].Init(m_Mesh,(MCData::MC_Method )i);
    }
}

MCAllMetrics::~MCAllMetrics()
{
    if( m_pMeshDescription != NULL ) delete [] m_pMeshDescription;
}
    
int 
MCAllMetrics::getPoint(MCData::MC_Method method, const Vertex &in_vertex, 
                       MCPoint &point)
{
    if( method < 0 || method >= m_aData.num() ) return -1;
    return m_aData[method].getPoint(in_vertex, point);
}

MCData& 
MCAllMetrics::getData(MCData::MC_Method method)
{
	return m_aData[method];
}

void
MCAllMetrics::ReadMesh( const char *filename )
{
    ifstream is;
    is.open(filename,ios::in);

    m_Mesh.Read(is);
		
	MCFixMeshNormals(m_Mesh);
}

int
MCAllMetrics::ReadExactCurvature( const char *filename )
{
    return m_aData[MCData::EXACT].ReadMeshCurvature(filename);
}

int 
MCAllMetrics::CalculateCurvature(const Vertex &in_vertex, 
                                 const MCData::MC_Method &method,
                                 MCPoint &cpt)
{
    if( method < 0 || method >= MCData::NUM_METHODS ) return -1;
    return m_aData[method].CalculateCurvature(in_vertex,cpt);
}

int 
MCAllMetrics::CalculateCurvature(const Vertex &in_vertex, 
                                 Array<MCPoint> &out_data)
{
    int i, ier;
    MCPoint cpt;

    out_data.need(0);

    for( i=1; i<MCData::NUM_METHODS; i++ ) {
        ier = CalculateCurvature(in_vertex, (MCData::MC_Method)i, cpt);
        if( ier < 0 ) return ier;
        out_data += cpt;
    }
    return 0;
}

int 
MCAllMetrics::CalculateCurvature(const MCData::MC_Method &method)
{
    if( method < 0 || method >= MCData::NUM_METHODS ) return -1;
    return m_aData[method].CalculateCurvature();
}

int 
MCAllMetrics::CalculateCurvature()
{
    int i, ier;

    for( i=1; i<MCData::NUM_METHODS; i++ ) {
        ier = CalculateCurvature((MCData::MC_Method)i);
        if( ier < 0 ) return ier;
    }
    return 0;
}

void 
MCAllMetrics::dump(FILE *out)
{
    int i;

    for( i=0; i<MCData::NUM_METHODS; i++ ) {
        m_aData[i].dump(out);
    }
}

void 
MCAllMetrics::dumpRanges(FILE *out)
{
    int i;

    for( i=0; i<MCData::NUM_METHODS; i++ ) {
        m_aData[i].dumpRanges(out);
    }
}
