#include "StdAfx.H"
#include <MC/MC_Point.H>
#include <utils/Pm_MeshLite.H>

MCPoint::MCPoint(const int id, const double k1, const double k2)
{
    m_iId = id;
    m_dK1 = k1;
    m_dK2 = k2;
    m_dKg = k1*k2;
    m_dKbar = 0.5*(k1+k2);
    
    m_vNormal = R3Vec(0.0, 0.0, 0.0);
    m_vKmax = R3Vec(0.0, 0.0, 0.0);
    m_vKmin = R3Vec(0.0, 0.0, 0.0);
    m_vKmaxPar = R2Vec(0.0, 0.0);
    m_vKminPar = R2Vec(0.0, 0.0);
}

MCPoint::MCPoint( const MCPoint &in_pt ) 
{
    (*this) = in_pt;
}
    
MCPoint::~MCPoint()
{
}

MCPoint &MCPoint::operator=(const MCPoint &in_pt)
{
    m_iId = in_pt.m_iId;
    m_dK1 = in_pt.m_dK1;
    m_dK2 = in_pt.m_dK2;
    m_dKg = in_pt.m_dKg;
    m_dKbar = in_pt.m_dKbar;
	m_vKmax = in_pt.m_vKmax;
	m_vKmin = in_pt.m_vKmin;
	m_vNormal = in_pt.m_vNormal;
	m_vKmaxPar = in_pt.m_vKmaxPar;
	m_vKminPar = in_pt.m_vKminPar;

    return *this;
}

void MCPoint::setId(const int id)
{
    m_iId = id;
}

void MCPoint::setMaxMin(const double k1, const double k2)
{
    m_dK1 = k1;
    m_dK2 = k2;
    m_dKg = k1*k2;
    m_dKbar = 0.5*(k1+k2);
}

void MCPoint::setKgKbar(const double kg, const double kbar)
{
    double radical = kbar*kbar - kg;
    if( radical <= 0.0 ) {
        //radical = 0.0;
        radical = sqrt(-radical);
    } else {
        radical = sqrt(radical);
    }
    //if( kbar > 0.0 ) {
        m_dK1 = kbar + radical;
        m_dK2 = kbar - radical;
    //} else {
    //    m_dK1 = kbar - radical;
    //    m_dK2 = kbar + radical;
    //}

    m_dKg   = m_dK1*m_dK2;
    m_dKbar = 0.5*(m_dK1+m_dK2);
}

void MCPoint::setNormal(const R3Vec &N)
{
    m_vNormal = N;
}

void MCPoint::setMaxMinVec(const R3Vec &KmaxVec, const R3Vec &KminVec)
{
    m_vKmax = KmaxVec;
    m_vKmin = KminVec;
}

void MCPoint::setMaxMinVecPar(const R2Vec &KmaxVec, const R2Vec &KminVec)
{
    m_vKmaxPar = KmaxVec;
    m_vKminPar = KminVec;
}

void MCPoint::dump(FILE *out)
{
    if( out != NULL ) {
        fprintf(out,"  Vertex Id: %d, k1= %lf, k2= %lf, Kg= %lf, Kbar = %lf\n",
                    m_iId, m_dK1, m_dK2, m_dKg, m_dKbar );
    }
}