#include "StdAfx.H"
#include <stdio.h>
#include <MC/MC_RadialBasisFunction.H>
#include <MC/MC_Param.H>
#include <MC/MC_Fit.H>
#include <MC/MC_MeshSubset.H>
#include <utils/Pm_MeshLite.H>
#include <fitting/FITTools_Least_squares.H>

MCRadialBasisFunction::MCRadialBasisFunction(const PMeshLite &in_mesh,
                                             const int in_nRings, 
                                    const MCParam::MC_ParamType &in_paramType )
: MCFit(in_mesh,RBF,in_paramType,in_nRings)
{
    rbf = NULL;
}

MCRadialBasisFunction::~MCRadialBasisFunction()
{
    if( rbf != NULL ) {
        delete rbf;
        rbf = NULL;
    }
}

int
MCRadialBasisFunction::Evaluate(const R2Pt &in_uvPoint, R3Pt &out_xyzPoint)
{
    if( rbf == NULL ) return -1;

    out_xyzPoint = rbf->Eval( in_uvPoint );

    return 0;
}

/*
int
MCRadialBasisFunction::Evaluate(const R2Pt &in_uvPoint, R3Vec &out_normal, 
                                R3Vec &out_dU, R3Vec &out_dV, 
                                R3Vec &out_dUU, R3Vec &out_dUV, R3Vec &out_dVV )
{
    double u, v, x, y, z;

    if( A.num() < 6 ) return -1;
    
    u = in_uvPoint[0];
    v = in_uvPoint[1];

    // Evaluate df/du 
    x = A[1][0] + 2.0*A[3][0]*u + A[4][0]*v;
    y = A[1][1] + 2.0*A[3][1]*u + A[4][1]*v;
    z = A[1][2] + 2.0*A[3][2]*u + A[4][2]*v;
    if( A.num() >= 9 ) {
        x += 2.0*A[6][0]*u*v + A[7][0]*v*v + 2.0*A[8][0]*u*v*v;
        y += 2.0*A[6][1]*u*v + A[7][1]*v*v + 2.0*A[8][1]*u*v*v;
        z += 2.0*A[6][2]*u*v + A[7][2]*v*v + 2.0*A[8][2]*u*v*v;
    }
    out_dU = R3Vec( x, y, z );

    // Evaluate df/dv 
    x = A[2][0] + A[4][0]*u + 2.0*A[5][0]*v;
    y = A[2][1] + A[4][1]*u + 2.0*A[5][1]*v;
    z = A[2][2] + A[4][2]*u + 2.0*A[5][2]*v;
    if( A.num() >= 9 ) {
        x += A[6][0]*u*u + 2.0*A[7][0]*u*v + 2.0*A[8][0]*u*u*v;
        y += A[6][1]*u*u + 2.0*A[7][1]*u*v + 2.0*A[8][1]*u*u*v;
        z += A[6][2]*u*u + 2.0*A[7][2]*u*v + 2.0*A[8][2]*u*u*v;
    }
    out_dV = R3Vec( x, y, z );

    // Evaluate the normal at (u,v) 
    out_normal = Cross(out_dU,out_dV);
    out_normal.Normalize();

    // Evaluate df/duu 
    x = 2.0*A[3][0];
    y = 2.0*A[3][1];
    z = 2.0*A[3][2];
    if( A.num() >= 9 ) {
        x += 2.0*A[6][0]*v + 2.0*A[8][0]*v*v;
        y += 2.0*A[6][1]*v + 2.0*A[8][1]*v*v;
        z += 2.0*A[6][2]*v + 2.0*A[8][2]*v*v;
    }
    out_dUU = R3Vec( x, y, z );

    // Evaluate df/duv 
    x = A[4][0];
    y = A[4][1];
    z = A[4][2];
    if( A.num() >= 9 ) {
        x += 2.0*A[6][0]*u + 2.0*A[7][0]*v + 4.0*A[8][0]*u*v;
        y += 2.0*A[6][1]*u + 2.0*A[7][1]*v + 4.0*A[8][1]*u*v;
        z += 2.0*A[6][2]*u + 2.0*A[7][2]*v + 4.0*A[8][2]*u*v;
    }
    out_dUV = R3Vec( x, y, z );

    // Evaluate df/dvv 
    x = 2.0*A[5][0];
    y = 2.0*A[5][1];
    z = 2.0*A[5][2];
    if( A.num() >= 9 ) {
        x += 2.0*A[7][0]*u + 2.0*A[8][0]*u*u;
        y += 2.0*A[7][1]*u + 2.0*A[8][1]*u*u;
        z += 2.0*A[7][2]*u + 2.0*A[8][2]*u*u;
    }
    out_dVV = R3Vec( x, y, z );

    return 0;
}
*/
        
int 
MCRadialBasisFunction::Curvature(const Vertex &in_vertex, MCPoint &cpt)
{
    int ier;
    R3Pt P;
    R3Vec NN, Xu, Xv, Xuu, Xuv, Xvv;

    FILE *out = NULL;
    //out = fopen(WINDataHome("test_new.dat"),"a");

    P = in_vertex->GetLoc();
    if( out != NULL ) {
        fprintf(out,"Point(%d) =(%lf,%lf,%lf)\n",
                    in_vertex->IndexId(), P[0], P[1], P[2] );
    }

    //if( !valid || in_vertex != m_Vertex ) {
        ier = CalculateFit(in_vertex);
		if( ier != 0 ) return ier;
    //}

    double tmp;
    R2Vec v2tmp;
    R3Vec v3tmp;
    double k1, k2;
    R2Vec vecMax_2d, vecMin_2d;
    R3Vec vecMax_3d, vecMin_3d, vecNorm;

    rbf->Curvature(m_Param.PtUV(), k1, k2, vecMax_2d, vecMin_2d,
                   vecMax_3d, vecMin_3d, vecNorm );
    if( Dot(vecNorm,m_Param.Nref()) > 0.0 ) {
        k1 = -k1;
        k2 = -k2;
        vecMax_3d *= -1.0;
        vecMin_3d *= -1.0;
        vecNorm *= -1.0;
    }
    if( k2 * k2 > k1 * k1 ) {
        tmp = k1;
        k1 = k2;
        k2 = tmp;
        v2tmp = vecMax_2d;
        vecMax_2d = vecMin_2d;
        vecMin_2d = v2tmp;
        v3tmp = vecMin_3d;
        vecMin_3d = vecMax_3d;
        vecMax_3d = v3tmp;
    }
    cpt.setMaxMin(k1, k2);
    cpt.setMaxMinVecPar(vecMax_2d, vecMin_2d);
    cpt.setMaxMinVec(vecMax_3d, vecMin_3d);

    return 0;
}
    
int 
MCRadialBasisFunction::CalculateFit(const Vertex &in_vertex)
{
    int i, n, ier;
    double spacing, du, dv;
    Array<R2Pt> uv;
    Array<R3Pt> xyz;

    // Create the mesh subset
    m_Param.Create(in_vertex,m_iRings);

    // Compute parametric coordinates
    ier = m_Param.CalculateUV(in_vertex,m_iRings);
    if( ier < 0 ) return ier;

    // Compute the spacing for the Radial basis function
    du = m_Param.UVmax()[0] - m_Param.UVmin()[0];
    dv = m_Param.UVmax()[1] - m_Param.UVmin()[1];
    spacing = WINmax(du, dv) / m_iRings;
    ASSERT(spacing != 0.0);
    m_Param.ScaleUVCoords(1.0/spacing);

    // Load the array of points 
    n = m_Param.Vertices().num();
    xyz.need(0);
    for( i=0; i<n; i++ ) {
        xyz += m_Param.Vertices()[i]->GetLoc();
    }
    
    // Make the Radial basis fit function 
    if( rbf != NULL ) {
        delete rbf;
        rbf = NULL;
    }
    rbf = new FITRbfGauss();
    rbf->Set(m_Param.UV(), xyz);

    return 0;
}

void 
MCRadialBasisFunction::dump(FILE *out)
{
    R3Pt pt;

    if( out != NULL ) {
        fprintf(out,"RBF: %d rings\n", m_iRings );
        fprintf(out,"  Method: %d\n", (int) m_ParamType );
        //for( i=0; i<A.num(); i++ ) {
        //    pt = A[i];
        //    fprintf(out," A[%d]: (%lf, %lf, %lf)\n", i, pt[0], pt[1], pt[2] );
        //}
    }
}
