// test.cpp : Defines routines to test the MeshCurvature class.
//

#include "StdAfx.H"
#include <MC/MC_test.h>
#include <MC/MC_AllMetrics.H>
#include <MC/MC_Polynomial.H>
#include <MC/MC_Conic.H>
#include <MC/MC_DiscreteOperator.H>
#include <MC/MC_Point.H>


void MCtest(const int testNumber)
{
    switch( testNumber ) {
        case 0:
            MCtestReadMesh();
            break;
        case 1:
            MCtestAngles();
            break;
        case 2:
            MCtestArea();
            break;
        case 3:
            MCtestParam();
            break;
        case 4:
            MCtestParam2();
            break;
        case 5:
            MCtestPoly();
            break;
        case 6:
            MCtestPoly1();
            break;
        case 7:
            MCtestPoly2();
            break;
        case 8:
            MCtestPoly3();
            break;
        case 9:
            MCtestDCO();
            break;
        case 10:
            MCtestDCO2();
            break;
        case 11:
            MCtestTorus();
            break;
        case 12:
            MCtestANC();
            break;
        case 13:
            MCtestANC1();
            break;
        case 14:
            MCtestANC2();
            break;
        case 15:
            MCtestIEV();
            break;
        case 16:
            MCtestConic();
            break;
        case 17:
            MCtestConic1();
            break;
        case 18:
            MCtestRBF();
            break;
        case 19:
            MCtestAll();
            break;
        default:
            break;
    }
}

void MCExtractRings(PMeshLite &in_mesh, const Vertex &v,
                    const int nRings, PMeshLite &out_mesh )
{
    Array<Vertex> aopVs;
    Array<Face> aopFs;
    in_mesh.GrowRings(v,nRings,aopVs,aopFs);
    in_mesh.ExtractCopy(aopFs,out_mesh);
}

void MCdump(PMeshLite &mesh, const char *filename)
{
    int i, j, ier;
    R3Pt P;
    FILE *out;
    Array<Vertex> aopVs;
    Array<Face> aopFs;
    Array<Vertex> aov;

    mesh.AllMeshVertices(aopVs);
    mesh.AllMeshFaces(aopFs);

    out = fopen(filename,"w");
    ier = fprintf(out,"nv= %d, nf= %d\n", aopVs.num(), aopFs.num() );
    for( i=0; i<aopVs.num(); i++ ) {
        P = aopVs[i]->GetLoc();
        fprintf(out,"Vertex %d: (%lf, %lf, %lf)\n", i, P[0], P[1], P[2] );
    }
    for( i=0; i<aopFs.num(); i++ ) {
        aov = mesh.FaceVertices(aopFs[i]);
        fprintf(out,"Face %d:", i);
        for( j=0; j<aov.num(); j++ ) {
            fprintf(out," %d", aov[j]->IndexId() );
        }
        fprintf(out,"\n");
    }
    fclose(out);
}

void MCdump(MCAllMetrics &data, const char *filename)
{
    FILE *out;

    out = fopen(filename,"w");
    data.dump(out);
    fclose(out);
}

void MCdumpRanges(MCAllMetrics &data, const char *filename)
{
    FILE *out;

    out = fopen(filename,"w");
    data.dumpRanges(out);
    fclose(out);
}

void MCtestReadMesh()
{
    int iPoint, nRings;
    MCAllMetrics data;
    PMeshLite ringMesh;
    Vertex v;
    ofstream os;

    data.ReadMesh(WINDataHome("TestSrf_low.m"));
    data.ReadExactCurvature(WINDataHome("TestSrf_low.txt"));

    iPoint = 0;
    nRings = 10;

    v = data.Mesh().VertexFromIndexId(iPoint);
    MCExtractRings(data.Mesh(),v,nRings,ringMesh);
    MCdump(ringMesh, WINDataHome("mesh.dat"));
    MCdump(data, WINDataHome("data.dat"));

    os.open(WINDataHome("mesh.m"),ios::out);
    ringMesh.Write(os);
}

void MCtestAngles()
{
    double pi, theta, theta2, alpha, beta, delta, gamma, t, a, b;
    PMeshLite mesh;
    MCMeshSubset ring(mesh,1);
    Array<Vertex> aonv;
    Array<Vertex> aov;
    Array<Edge> aoe;
    Array<Face> aof;
    FILE *out;

    pi = 4.0 * atan(1.0);

    aonv += mesh.AddVertex(R3Pt(0.0,0.0,0.0));
    aonv += mesh.AddVertex(R3Pt(3.0,0.0,0.0));
    aonv += mesh.AddVertex(R3Pt(0.0,5.0,0.0));
    aonv += mesh.AddVertex(R3Pt(-3.0,0.0,0.0));
    aonv += mesh.AddVertex(R3Pt(0.0,-5.0,0.0));

    aov.need(0);
    aov += aonv[0];
    aov += aonv[1];
    aov += aonv[2];
    aof += mesh.AddFace(aov);

    aov.need(0);
    aov += aonv[0];
    aov += aonv[2];
    aov += aonv[3];
    aof += mesh.AddFace(aov);

    aov.need(0);
    aov += aonv[0];
    aov += aonv[3];
    aov += aonv[4];
    aof += mesh.AddFace(aov);

    aov.need(0);
    aov += aonv[0];
    aov += aonv[4];
    aov += aonv[1];
    aof += mesh.AddFace(aov);

    ring.Create(aonv[0]);

    out = fopen(WINDataHome("test.out"),"w");

    ring.AlphaBeta(aof[0],aonv[0],theta,alpha,beta,t,a,b);
    fprintf(out,"Pt 1: theta = %lf, alpha = %lf, beta = %lf\n",
                theta*180.0/pi, alpha*180.0/pi, beta*180.0/pi );

    ring.AlphaBeta(aof[0],aonv[1],theta,alpha,beta,t,a,b);
    fprintf(out,"Pt 2: theta = %lf, alpha = %lf, beta = %lf\n",
                theta*180.0/pi, alpha*180.0/pi, beta*180.0/pi );

    ring.AlphaBeta(aof[0],aonv[2],theta,alpha,beta,t,a,b);
    fprintf(out,"Pt 3: theta = %lf, alpha = %lf, beta = %lf\n",
                theta*180.0/pi, alpha*180.0/pi, beta*180.0/pi );

    aoe = mesh.VertexEdges(aonv[0]);
    ring.AlphaBeta( aonv[0], aoe[0], theta, theta2, alpha, beta, delta, gamma );
    fprintf(out,"Edge 0: theta = %lf, alpha = %lf, delta = %lf\n",
                theta*180.0/pi, alpha*180.0/pi, delta*180.0/pi );
    fprintf(out,"Edge 0: theta = %lf, beta = %lf, gamma = %lf\n",
                theta2*180.0/pi, beta*180.0/pi, gamma*180.0/pi );

    fclose(out);
}

void MCtestArea()
{
    double area;
    PMeshLite mesh;
    MCMeshSubset ring(mesh,1);
    Array<Vertex> aonv;
    Array<Vertex> aov;
    Array<Face> aof;
    Vertex v;

    aonv += mesh.AddVertex(R3Pt( 0.0, 0.0, 0.0));
    aonv += mesh.AddVertex(R3Pt( 2.0, 4.0, 0.0));
    aonv += mesh.AddVertex(R3Pt(-2.0, 4.0, 0.0));
    aonv += mesh.AddVertex(R3Pt(-4.0, 0.0, 0.0));
    aonv += mesh.AddVertex(R3Pt(-2.0,-4.0, 0.0));
    aonv += mesh.AddVertex(R3Pt( 2.0,-4.0, 0.0));
    aonv += mesh.AddVertex(R3Pt( 4.0, 0.0, 0.0));
    aonv += mesh.AddVertex(R3Pt( 4.0,-4.0, 0.0));
    aonv += mesh.AddVertex(R3Pt( 4.0,-8.0, 0.0));
    aonv += mesh.AddVertex(R3Pt( 6.0,-4.0, 0.0));
    aonv += mesh.AddVertex(R3Pt( 0.0, 7.0, 0.0));
    aonv += mesh.AddVertex(R3Pt( 0.0, 5.0, 0.0));
    
    aov.need(0);
    aov += aonv[0];
    aov += aonv[1];
    aov += aonv[2];
    aof += mesh.AddFace(aov);

    aov.need(0);
    aov += aonv[0];
    aov += aonv[2];
    aov += aonv[3];
    aof += mesh.AddFace(aov);

    aov.need(0);
    aov += aonv[0];
    aov += aonv[3];
    aov += aonv[4];
    aof += mesh.AddFace(aov);

    aov.need(0);
    aov += aonv[0];
    aov += aonv[4];
    aov += aonv[5];
    aof += mesh.AddFace(aov);

    aov.need(0);
    aov += aonv[0];
    aov += aonv[5];
    aov += aonv[6];
    aof += mesh.AddFace(aov);

    aov.need(0);
    aov += aonv[0];
    aov += aonv[6];
    aov += aonv[1];
    aof += mesh.AddFace(aov);

    aov.need(0);
    aov += aonv[7];
    aov += aonv[6];
    aov += aonv[5];
    aof += mesh.AddFace(aov);

    aov.need(0);
    aov += aonv[7];
    aov += aonv[5];
    aov += aonv[8];
    aof += mesh.AddFace(aov);

    aov.need(0);
    aov += aonv[7];
    aov += aonv[8];
    aov += aonv[9];
    aof += mesh.AddFace(aov);

    aov.need(0);
    aov += aonv[7];
    aov += aonv[9];
    aov += aonv[6];
    aof += mesh.AddFace(aov);

    aov.need(0);
    aov += aonv[11];
    aov += aonv[2];
    aov += aonv[1];
    aof += mesh.AddFace(aov);

    aov.need(0);
    aov += aonv[11];
    aov += aonv[1];
    aov += aonv[10];
    aof += mesh.AddFace(aov);

    aov.need(0);
    aov += aonv[11];
    aov += aonv[10];
    aov += aonv[2];
    aof += mesh.AddFace(aov);

    v = aonv[0];
    ring.Create(v);
    area = ring.VertexArea(v,MCMeshSubset::BARYCENTRIC);
    area = ring.VertexArea(v,MCMeshSubset::VORONOI);
    area = ring.VertexArea(v,MCMeshSubset::MIXED);
    area = ring.VertexArea(v,MCMeshSubset::MODIFIED);

    v = aonv[7];
    ring.Create(v);
    area = ring.VertexArea(v,MCMeshSubset::BARYCENTRIC);
    area = ring.VertexArea(v,MCMeshSubset::VORONOI);
    area = ring.VertexArea(v,MCMeshSubset::MIXED);
    area = ring.VertexArea(v,MCMeshSubset::MODIFIED);

    v = aonv[11];
    ring.Create(v);
    area = ring.VertexArea(v,MCMeshSubset::BARYCENTRIC);
    area = ring.VertexArea(v,MCMeshSubset::VORONOI);
    area = ring.VertexArea(v,MCMeshSubset::MIXED);
    area = ring.VertexArea(v,MCMeshSubset::MODIFIED);   
}

void MCtestParam()
{
    int iPoint, nRings;
    MCAllMetrics data;
    PMeshLite new_mesh;
    Vertex v;
    MCPoint cpt1, cpt2, cpt3, cpt4, cpt5, cpt6, cpt7, cpt8, cpt9;
    ofstream os;
    FILE *out;

    data.ReadMesh(WINDataHome("Sphere.m"));
    
    iPoint = 0;
    v = data.Mesh().VertexFromIndexId(iPoint);
    
    out = fopen(WINDataHome("poly.dat"),"w");

    nRings = 1;
    MCPolynomial poly1(data.Mesh(),nRings,MCParam::PLANAR);
    cpt1 = poly1.Curvature(v);
    cpt1.dump(out);

    poly1.Param().MakeMesh(new_mesh);
    os.open(WINDataHome("param1.m"),ios::out);
    new_mesh.Write(os);
    os.close();

    nRings = 2;
    MCPolynomial poly2(data.Mesh(),nRings,MCParam::PLANAR);
    cpt2 = poly2.Curvature(v);
    cpt2.dump(out);

    poly2.Param().MakeMesh(new_mesh);
    os.open(WINDataHome("param2.m"),ios::out);
    new_mesh.Write(os);
    os.close();

    nRings = 3;
    MCPolynomial poly3(data.Mesh(),nRings,MCParam::PLANAR);
    cpt3 = poly3.Curvature(v);
    cpt3.dump(out);

    poly3.Param().MakeMesh(new_mesh);
    os.open(WINDataHome("param3.m"),ios::out);
    new_mesh.Write(os);
    os.close();

    nRings = 1;
    MCPolynomial poly4(data.Mesh(),nRings,MCParam::NATURAL);
    cpt4 = poly4.Curvature(v);
    cpt4.dump(out);

    poly4.Param().MakeMesh(new_mesh);
    os.open(WINDataHome("param4.m"),ios::out);
    new_mesh.Write(os);
    os.close();

    nRings = 2;
    MCPolynomial poly5(data.Mesh(),nRings,MCParam::NATURAL);
    cpt5 = poly5.Curvature(v);
    cpt5.dump(out);

    poly5.Param().MakeMesh(new_mesh);
    os.open(WINDataHome("param5.m"),ios::out);
    new_mesh.Write(os);
    os.close();

    nRings = 3;
    MCPolynomial poly6(data.Mesh(),nRings,MCParam::NATURAL);
    cpt6 = poly6.Curvature(v);
    cpt6.dump(out);

    poly6.Param().MakeMesh(new_mesh);
    os.open(WINDataHome("param6.m"),ios::out);
    new_mesh.Write(os);
    os.close();

    nRings = 1;
    MCPolynomial poly7(data.Mesh(),nRings,MCParam::CIRCULAR);
    cpt7 = poly7.Curvature(v);
    cpt7.dump(out);

    poly7.Param().MakeMesh(new_mesh);
    os.open(WINDataHome("param7.m"),ios::out);
    new_mesh.Write(os);
    os.close();

    nRings = 2;
    MCPolynomial poly8(data.Mesh(),nRings,MCParam::CIRCULAR);
    cpt8 = poly8.Curvature(v);
    cpt8.dump(out);

    poly8.Param().MakeMesh(new_mesh);
    os.open(WINDataHome("param8.m"),ios::out);
    new_mesh.Write(os);
    os.close();

    nRings = 3;
    MCPolynomial poly9(data.Mesh(),nRings,MCParam::CIRCULAR);
    cpt9 = poly9.Curvature(v);
    cpt9.dump(out);

    poly9.Param().MakeMesh(new_mesh);
    os.open(WINDataHome("param9.m"),ios::out);
    new_mesh.Write(os);
    os.close();

    fclose(out);
}

void MCtestParam2()
{
    int iPoint, nRings;
    MCAllMetrics data;
    PMeshLite new_mesh;
    PMeshLite ringMesh;
    Vertex v;
    MCPoint cpt1, cpt2, cpt3;
    ofstream os;
    FILE *out;

    data.ReadMesh(WINDataHome("TestSrf_low.m"));
    data.ReadExactCurvature(WINDataHome("TestSrf_low.txt"));

    iPoint = 0;
    nRings = 10;

    v = data.Mesh().VertexFromIndexId(iPoint);
    out = fopen(WINDataHome("poly.dat"),"w");

    MCPolynomial poly1(data.Mesh(),nRings,MCParam::PLANAR);
    cpt1 = poly1.Curvature(v);
    cpt1.dump(out);

    poly1.Param().MakeMesh(new_mesh);
    os.open(WINDataHome("param1a.m"),ios::out);
    new_mesh.Write(os);
    os.close();

    MCPolynomial poly2(data.Mesh(),nRings,MCParam::NATURAL);
    cpt2 = poly2.Curvature(v);
    cpt2.dump(out);

    poly2.Param().MakeMesh(new_mesh);
    os.open(WINDataHome("param2a.m"),ios::out);
    new_mesh.Write(os);
    os.close();

    MCPolynomial poly3(data.Mesh(),nRings,MCParam::CIRCULAR);
    cpt3 = poly3.Curvature(v);
    cpt3.dump(out);

    poly3.Param().MakeMesh(new_mesh);
    os.open(WINDataHome("param3a.m"),ios::out);
    new_mesh.Write(os);
    os.close();

    fclose(out);
}

void MCtestPoly()
{
    MCAllMetrics data;
    
    data.ReadMesh(WINDataHome("Sphere.m"));

    data.CalculateCurvature(MCData::FIT1P);
    data.CalculateCurvature(MCData::FIT1N);
    data.CalculateCurvature(MCData::FIT2P);
    data.CalculateCurvature(MCData::FIT2N);
    data.CalculateCurvature(MCData::FIT3P);
    data.CalculateCurvature(MCData::FIT3N);

    MCdump(data, WINDataHome("polyCurvature.dat"));
}

void MCtestPoly1()
{
    MCAllMetrics data;
    int iPoint, nRings;
    Vertex v;
    PMeshLite new_mesh;
    MCPoint cpt;
    Array<Vertex> aov;
    Array<Face> aof;
    ofstream os;
    FILE *out;
    
    data.ReadMesh(WINDataHome("Sphere.m"));

    out = fopen(WINDataHome("pointCurvature.dat"),"a");
    fprintf(out,"Polynomial Fit Method on Sphere Mesh\n");

    iPoint = 0;
    nRings = 2;

    v = data.Mesh().VertexFromIndexId(iPoint);
    fprintf(out,"Point %d\n", iPoint);

    data.Mesh().GrowRings(v,nRings,aov,aof);
    data.Mesh().ExtractCopy(aof,new_mesh);
    os.open(WINDataHome("ring.m"),ios::out);
    new_mesh.Write(os);
    os.close();
    
    //MCPolynomial poly(data.Mesh(),nRings,MCParam::PLANAR);
    //MCPolynomial poly(data.Mesh(),nRings,MCParam::CIRCULAR);
    MCPolynomial poly(data.Mesh(),nRings,MCParam::NATURAL);
    
    cpt = poly.Curvature(v);
    cpt.dump(out);

    poly.dump(out);
    poly.Param().dump(out);
    
    poly.Param().MakeMesh(new_mesh);
    os.open(WINDataHome("param.m"),ios::out);
    new_mesh.Write(os);
    os.close();
    
    fclose(out);
}

void MCtestPoly2()
{
    MCAllMetrics data;
    int iPoint, nRings;
    Vertex v;
    PMeshLite new_mesh;
    MCPoint cpt1, cpt2, cpt3, cpt4, cpt5, cpt6, cpt7, cpt8;
    Array<Vertex> aov;
    Array<Face> aof;
    ofstream os;
    FILE *out;
    
    data.ReadMesh(WINDataHome("Sphere.m"));

    out = fopen(WINDataHome("pointCurvature.dat"),"a");
    fprintf(out,"Polynomial Fit Method on Sphere Mesh\n");

    iPoint = 0;
    v = data.Mesh().VertexFromIndexId(iPoint);
    fprintf(out,"Point %d\n", iPoint);

    nRings = 2;

    data.Mesh().GrowRings(v,nRings,aov,aof);
    data.Mesh().ExtractCopy(aof,new_mesh);
    MCPolynomial poly4(new_mesh,nRings,MCParam::NATURAL);
    //v = new_mesh.VertexFromIndexId(iPoint);
    v = new_mesh.VertexFromIndexId(0);

    //MCPolynomial poly4(data.Mesh(),nRings,MCParam::PLANAR);
    //MCPolynomial poly4(data.Mesh(),nRings,MCParam::NATURAL);
    cpt4 = poly4.Curvature(v);
    cpt4.dump(out);

    poly4.dump(out);
    poly4.Param().dump(out);
    //poly4.Param().MakeMesh(new_mesh);
    os.open(WINDataHome("param4a.m"),ios::out);
    new_mesh.Write(os);
    os.close();
    
    iPoint = 1;
    v = data.Mesh().VertexFromIndexId(iPoint);
    fprintf(out,"Point %d\n", iPoint);

    nRings = 2;

    data.Mesh().GrowRings(v,nRings,aov,aof);
    data.Mesh().ExtractCopy(aof,new_mesh);
    MCPolynomial poly8(new_mesh,nRings,MCParam::NATURAL);
    //v = new_mesh.VertexFromIndexId(iPoint);
    v = new_mesh.VertexFromIndexId(0);

    //MCPolynomial poly8(data.Mesh(),nRings,MCParam::PLANAR);
    //MCPolynomial poly8(data.Mesh(),nRings,MCParam::NATURAL);
    cpt8 = poly8.Curvature(v);
    cpt8.dump(out);

    poly8.dump(out);
    poly8.Param().dump(out);
    //poly8.Param().MakeMesh(new_mesh);
    os.open(WINDataHome("param8a.m"),ios::out);
    new_mesh.Write(os);
    os.close();

    fclose(out);
}

void MCtestPoly3()
{
    MCAllMetrics data;
    int iPoint, nRings;
    Vertex v;
    PMeshLite new_mesh;
    MCPoint cpt1, cpt2, cpt3, cpt4, cpt5, cpt6, cpt7, cpt8;
    Array<Vertex> aov;
    Array<Face> aof;
    ofstream os;
    FILE *out;
    
    data.ReadMesh(WINDataHome("Sphere.m"));

    out = fopen(WINDataHome("pointCurvature.dat"),"w");
    fprintf(out,"Polynomial Fit Method on Sphere Mesh\n");

    nRings = 1;
    iPoint = 0;
    v = data.Mesh().VertexFromIndexId(iPoint);
    fprintf(out,"Point %d\n", iPoint);

    //MCPolynomial poly1(data.Mesh(),nRings,MCParam::PLANAR);
    //MCPolynomial poly1(data.Mesh(),nRings,MCParam::CIRCULAR);
    MCPolynomial poly1(data.Mesh(),nRings,MCParam::NATURAL);
    cpt1 = poly1.Curvature(v);
    cpt1.dump(out);
    poly1.dump(out);
    poly1.Param().dump(out);
    
    data.Mesh().GrowRings(v,nRings,aov,aof);
    data.Mesh().ExtractCopy(aof,new_mesh);
    os.open(WINDataHome("ring1.m"),ios::out);
    new_mesh.Write(os);
    os.close();
    
    poly1.Param().MakeMesh(new_mesh);
    os.open(WINDataHome("param1.m"),ios::out);
    new_mesh.Write(os);
    os.close();

    iPoint = 1;
    v = data.Mesh().VertexFromIndexId(iPoint);
    fprintf(out,"Point %d\n", iPoint);

    //MCPolynomial poly2(data.Mesh(),nRings,MCParam::PLANAR);
    //MCPolynomial poly2(data.Mesh(),nRings,MCParam::CIRCULAR);
    MCPolynomial poly2(data.Mesh(),nRings,MCParam::NATURAL);
    cpt2 = poly2.Curvature(v);
    cpt2.dump(out);
    poly2.dump(out);
    poly2.Param().dump(out);
    
    data.Mesh().GrowRings(v,nRings,aov,aof);
    data.Mesh().ExtractCopy(aof,new_mesh);
    os.open(WINDataHome("ring2.m"),ios::out);
    new_mesh.Write(os);
    os.close();
    
    poly2.Param().MakeMesh(new_mesh);
    os.open(WINDataHome("param2.m"),ios::out);
    new_mesh.Write(os);
    os.close();

    fclose(out);
}

void MCtestDCO()
{
    MCAllMetrics data;
    
    data.ReadMesh(WINDataHome("Sphere.m"));

    data.CalculateCurvature(MCData::DCO);

    MCdump(data, WINDataHome("DCOCurvature.dat"));
}

void MCtestDCO2()
{
    MCAllMetrics data;
    int iPoint;
    Vertex v;
    PMeshLite new_mesh;
    MCPoint cpt1, cpt2;
    FILE *out;
    
    data.ReadMesh(WINDataHome("Sphere.m"));

    out = fopen(WINDataHome("pointCurvature.dat"),"a");
    fprintf(out,"DCO Method on Sphere Mesh\n");

    iPoint = 0;
    v = data.Mesh().VertexFromIndexId(iPoint);
    fprintf(out,"Point %d\n", iPoint);

    MCDiscreteOperator dco1(data.Mesh(),MCDiscreteOperator::DCO);
    cpt1 = dco1.Curvature(v);
    cpt1.dump(out);

    iPoint = 1;
    v = data.Mesh().VertexFromIndexId(iPoint);
    fprintf(out,"Point %d\n", iPoint);

    MCDiscreteOperator dco2(data.Mesh(),MCDiscreteOperator::DCO);
    cpt2 = dco2.Curvature(v);
    cpt2.dump(out);
    
    fclose(out);
}

void MCtestANC()
{
    MCAllMetrics data;
    
    data.ReadMesh(WINDataHome("Sphere.m"));

    data.CalculateCurvature(MCData::ANCE);
    data.CalculateCurvature(MCData::ANC);

    MCdump(data, WINDataHome("ANCCurvature.dat"));
}

void MCtestANC1()
{
    MCAllMetrics data;
    
    data.ReadMesh(WINDataHome("Sphere.m"));

    data.CalculateCurvature(MCData::ANCE);

    MCdump(data, WINDataHome("ANCCurvature.dat"));
}

void MCtestANC2()
{
    MCAllMetrics data;
    
    data.ReadMesh(WINDataHome("Sphere.m"));

    data.CalculateCurvature(MCData::ANC);

    MCdump(data, WINDataHome("ANCCurvature.dat"));
}

void MCtestIEV()
{
    MCAllMetrics data;
    
    data.ReadMesh(WINDataHome("Sphere.m"));

    data.CalculateCurvature(MCData::IEV);

    MCdump(data, WINDataHome("IEVCurvature.dat"));
}

void MCtestConic()
{
    MCAllMetrics data;
    
    data.ReadMesh(WINDataHome("Sphere.m"));

    data.CalculateCurvature(MCData::CON1);
    data.CalculateCurvature(MCData::CON2);
    data.CalculateCurvature(MCData::CON3);

    MCdump(data, WINDataHome("conicCurvature.dat"));
}

void MCtestConic1()
{
    MCAllMetrics data;
    int iPoint, nRings;
    Vertex v;
    PMeshLite new_mesh;
    MCPoint cpt;
    Array<Vertex> aov;
    Array<Face> aof;
    ofstream os;
    FILE *out;
    
    out = fopen(WINDataHome("pointCurvature.dat"),"a");
    fprintf(out,"Conic Fit Method on Sphere Mesh\n");

    data.ReadMesh(WINDataHome("Sphere.m"));

    nRings = 2;
    iPoint = 0;
    v = data.Mesh().VertexFromIndexId(iPoint);
    fprintf(out,"Point %d\n", iPoint);

    data.Mesh().GrowRings(v,nRings,aov,aof);
    data.Mesh().ExtractCopy(aof,new_mesh);
    os.open(WINDataHome("conic1a.m"),ios::out);
    new_mesh.Write(os);
    os.close();

    MCConic conic1(data.Mesh(),nRings);
    cpt = conic1.Curvature(v);
    cpt.dump(out);
    conic1.dump(out);
    conic1.Param().dump(out);

    conic1.MakeMesh(11, 11, 1.0, 1, new_mesh);
    os.open(WINDataHome("conic1b.m"),ios::out);
    new_mesh.Write(os);
    os.close();

    fclose(out);
}

void MCtestRBF()
{
    MCAllMetrics data;
    
    data.ReadMesh(WINDataHome("Sphere.m"));

    data.CalculateCurvature(MCData::RBF1P);
    data.CalculateCurvature(MCData::RBF2P);
    data.CalculateCurvature(MCData::RBF3P);
    data.CalculateCurvature(MCData::RBF1N);
    data.CalculateCurvature(MCData::RBF2N);
    data.CalculateCurvature(MCData::RBF3N);

    MCdump(data, WINDataHome("rbfCurvature.dat"));
}

void MCtestAll()
{
    MCAllMetrics data;
    
    data.ReadMesh(WINDataHome("Sphere.m"));

    data.CalculateCurvature();

    MCdump(data, WINDataHome("testCurvature.dat"));
}

void MCtestTorus()
{
    MCAllMetrics data;
    
    data.ReadMesh(WINDataHome("Torus.m"));

    /*
    data.CalculateCurvature(MCData::FIT1P);
    data.CalculateCurvature(MCData::FIT1N);
    data.CalculateCurvature(MCData::FIT2P);
    data.CalculateCurvature(MCData::FIT2N);
    data.CalculateCurvature(MCData::FIT3P);
    data.CalculateCurvature(MCData::FIT3N);
    */

    data.CalculateCurvature();

    MCdumpRanges(data, WINDataHome("torusCurvature.dat"));
    //MCdump(data, WINDataHome("torusCurvature.dat"));
}

