#include "StdAfx.H"

#include <fitting/FITTools_Align_points.H>
#include <fitting/FITTools_SVD.H>
//#include <cmath>

#define DOT(p,q) ((p)[0]*(q)[0]+(p)[1]*(q)[1]+(p)[2]*(q)[2])
#define LENGTH(p) (sqrt(DOT(p,p)))
static void cross( double *res, 
                   double *a, 
                   double *b )
{
   res[0] = a[1] * b[2] - a[2] * b[1];
   res[1] = - ( a[0] * b[2] - a[2] * b[0]);
   res[2] = a[0] * b[1] - a[1] * b[0];
}

static void centroid(double *c, int n, FITToolsAlign_point *pts)
{
    c[0] = c[1] = c[2] = 0;
    
    for (int i = 0; i < n; i++) {
        c[0] += pts[i].pt[0];
        c[1] += pts[i].pt[1];
        c[2] += pts[i].pt[2];
    }
    
    c[0] /= (double) n;
    c[1] /= (double) n;
    c[2] /= (double) n;
}

static void move( double *c, 
                  int     n, 
                  FITToolsAlign_point *pts, 
                  FITToolsAlign_point *npts )
{
    for (int i = 0; i < n; i++) {
        npts[i].pt[0] = pts[i].pt[0] - c[0];
        npts[i].pt[1] = pts[i].pt[1] - c[1];
        npts[i].pt[2] = pts[i].pt[2] - c[2];
    }
    
}

static void make_frame( double             frame[3][3],
                        FITToolsAlign_point * pts)
{
    for (int j = 0; j < 3; j++) 
        frame[0][j] = pts[1].pt[j] - pts[0].pt[j];
    
    double len = LENGTH(frame[0]);
    if (len < 1e-30 && len > -1e-30) len = 1.0;
    for (FORINT j = 0; j < 3; j++) frame[0][j] = frame[0][j] / len;
    
    for (FORINT j = 0; j < 3; j++) 
        frame[1][j] = pts[2].pt[j] - pts[0].pt[j];
    
    double dot = DOT(frame[1], frame[0]);
    
    for (FORINT j = 0; j < 3; j++) 
        frame[1][j] = pts[2].pt[j] - pts[0].pt[j] - dot * frame[0][j];
    
    len = LENGTH(frame[1]);
    if (len < 1e-30 && len > -1e-30) len = 1.0;
    for (FORINT j = 0; j < 3; j++) frame[1][j] /= len;
    
    cross(frame[2], frame[0], frame[1]);
}

#define MINDX(i,j) ((i)*3+(j))
#define X 0
#define Y 1
#define Z 2
static void fill_M( double *M, 
                    int n, 
                    FITToolsAlign_point *pts_r, 
                    FITToolsAlign_point *pts_l)
{
    for (int i = 0; i < 9; i++) M[i] = 0;
    
    double *pr, *pl;
    for (FORINT i = 0; i < n; i++) {
        pl = pts_l[i].pt;
        pr = pts_r[i].pt;
        
        M[MINDX(X,X)] += pl[X] * pr[X];
        M[MINDX(Y,Y)] += pl[Y] * pr[Y];
        M[MINDX(Z,Z)] += pl[Z] * pr[Z];
        
        M[MINDX(X,Y)] += pl[X] * pr[Y];
        M[MINDX(X,Z)] += pl[X] * pr[Z];
        
        M[MINDX(Y,X)] += pl[Y] * pr[X];
        M[MINDX(Y,Z)] += pl[Y] * pr[Z];
        
        M[MINDX(Z,X)] += pl[Z] * pr[X];
        M[MINDX(Z,Y)] += pl[Z] * pr[Y];
    }
}

#define NINDX(i,j) ((i)*4+(j))
static void fill_N(double *N, double *M)
{
    for (int i = 0; i < 16; i++) N[i] = 0.0;
    
    N[NINDX(0,0)] = M[MINDX(0,0)] + M[MINDX(1,1)] + M[MINDX(2,2)]; 
    N[NINDX(1,1)] = M[MINDX(0,0)] - M[MINDX(1,1)] - M[MINDX(2,2)]; 
    N[NINDX(2,2)] =-M[MINDX(0,0)] + M[MINDX(1,1)] - M[MINDX(2,2)]; 
    N[NINDX(3,3)] =-M[MINDX(0,0)] - M[MINDX(1,1)] + M[MINDX(2,2)]; 
    
    N[NINDX(0,1)] = M[MINDX(Y,Z)] - M[MINDX(Z,Y)];
    N[NINDX(0,2)] = M[MINDX(Z,X)] - M[MINDX(X,Z)];
    N[NINDX(0,3)] = M[MINDX(X,Y)] - M[MINDX(Y,X)];
    
    N[NINDX(1,0)] = M[MINDX(Y,Z)] - M[MINDX(Z,Y)];
    N[NINDX(1,2)] = M[MINDX(X,Y)] + M[MINDX(Y,X)];
    N[NINDX(1,3)] = M[MINDX(Z,X)] + M[MINDX(X,Z)];
    
    N[NINDX(2,0)] = M[MINDX(Z,X)] - M[MINDX(X,Z)];
    N[NINDX(2,1)] = M[MINDX(X,Y)] + M[MINDX(Y,X)];
    N[NINDX(2,3)] = M[MINDX(Y,Z)] + M[MINDX(Z,Y)];
    
    N[NINDX(3,0)] = M[MINDX(X,Y)] - M[MINDX(Y,X)];
    N[NINDX(3,1)] = M[MINDX(Z,X)] + M[MINDX(X,Z)];
    N[NINDX(3,2)] = M[MINDX(Y,Z)] + M[MINDX(Z,Y)];
}

static void make_unit_quat(double *unit_quat, 
                           int indx, 
                           double *u)
{
    for (int i = 0; i < 4; i++)
        unit_quat[i] = u[NINDX(i, indx)];
}

static void set_rotation(FITToolsAlign_result &res,
                         double *q)
{
    double sqrs[4];
    for (int i = 0; i < 4; i++) 
        sqrs[i] = q[i] * q[i];
    
    double q0 = q[0];
    double qx = q[1];
    double qy = q[2];
    double qz = q[3];
    
    res.rotation[0][0] = (sqrs[0] + sqrs[1] - sqrs[2] - sqrs[3]);
    res.rotation[1][1] = (sqrs[0] - sqrs[1] + sqrs[2] - sqrs[3]);
    res.rotation[2][2] = (sqrs[0] - sqrs[1] - sqrs[2] + sqrs[3]);
    
    res.rotation[0][1] = 2 * (qx * qy - q0 * qz);
    res.rotation[0][2] = 2 * (qx * qz + q0 * qy);
    
    res.rotation[1][0] = 2 * (qx * qy + q0 * qz);
    res.rotation[1][2] = 2 * (qy * qz - q0 * qx);
    
    res.rotation[2][0] = 2 * (qz * qx - q0 * qy);
    res.rotation[2][1] = 2 * (qz * qy + q0 * qx);
}

static void rotate(double *res, double mat[3][3], double *vec)
{
    for (int i = 0; i < 3; i++) res[i] = 0.0;
    
    for (FORINT i = 0; i < 3; i++) {
        for (int j = 0; j < 3; j++) {
            res[i] += mat[i][j] * vec[j];
        }
    }
}

static void set_scale(FITToolsAlign_result &res, int n,
                      FITToolsAlign_point *pts_r, FITToolsAlign_point *pts_l)
{
    double pts_r_len = 0, pts_l_len = 0, pts_rl_len = 0;
    double rot[3];
    for (int i = 0; i < n; i++) {
        pts_l_len += DOT(pts_l[i].pt, pts_l[i].pt);
        rotate(rot, res.rotation, pts_l[i].pt);
        pts_rl_len += DOT(pts_r[i].pt, rot);
    }
    if (pts_l_len > 1e-30 || pts_l_len < -1e-30)
        res.scale = pts_rl_len / pts_l_len;
    else
        res.scale = 1.0;
}

static void set_translation(FITToolsAlign_result &res, 
                            double *c_r, 
                            double *c_l)
{
    double rot[3];
    rotate(rot, res.rotation, c_l);
    
    for (int i = 0; i < 3; i++)
        res.translation[i] = c_r[i] - res.scale * rot[i];
}

double FITToolsAlign_points(FITToolsAlign_result &result,
                            int n, 
                            FITToolsAlign_point *pts_r, FITToolsAlign_point *pts_l,
                            WINbool do_scale)
{
    if (n == 0) return -1.0;
    
    static double centroid_r[3], centroid_l[3];
    centroid(centroid_r, n, pts_r);
    centroid(centroid_l, n, pts_l);
    
    move(centroid_r, n, pts_r, pts_r);
    move(centroid_l, n, pts_l, pts_l);
    
    static double M[9];
    fill_M(M, n, pts_r, pts_l);
    
    static double N[16];
    fill_N(N, M);
    
    static double sigma[4], u[16], vt[16];
    FITToolsSVD::SVD(N, 4, 4, u, sigma, vt);
    
    static double unit_quat[4];
    make_unit_quat(unit_quat, 0, u);
    
    set_rotation(result, unit_quat);
    
    if (do_scale)
        set_scale(result, n, pts_r, pts_l);
    else
        result.scale = 1.0;

    set_translation(result, centroid_r, centroid_l);
    
    return 0;
}

