/* Copyright 1997, Microsoft.  All Rights Reserved. */





/* -------------------------------------------------------------------------

 *

 *               Solve a linear programming problem

 *

 * ------------------------------------------------------------------------- */



#include "StdAfx.H"

#include <Util_Linear_programming.h>

#include <IMSL_decls.h>



/* -----------------------  Private Methods  ------------------------------- */





/* ----------------------- Protected Methods ------------------------------- */





/* -----------------------  Public Methods   ------------------------------- */



/* -------------------------------------------------------------------------

 * DESCR   :	Pick the appropriate thing to do

 * ------------------------------------------------------------------------- */

double

Linear_program::solve()

{

   IMSL_linearProgram(n_constraints, n_variables,

                      constraints, 

                      bl, bu, cis, irtype,

                      xlb, xub, obj, solution);

   return 0.0;

}





void Linear_program::reset()

{

   obj = 0.0;

   for (int i = 0; i < n_constraints; i++) {

      cis[i] = 1.0;

      bl[i] = -1.0e30;

      bu[i] = 1.0e30;

      irtype[i] = 3;

   }

   for (i = 0; i < n_variables; i++) {

      xlb[i] = -1.0e30;

      xub[i] = 1.0e30;

   }

   const int size = sizeof(double) / sizeof(char);

   memset(constraints, 0, size * (n_constraints * n_variables));

}



/* -------------------------------------------------------------------------

 * DESCR   :	Create the matrices that will Ae needed

 * ------------------------------------------------------------------------- */

Linear_program::Linear_program(

   const int a,       // number of constraints

   const int n       // number of variables

   ) : n_constraints(a), n_variables(n)

{

   constraints = new double [a * n];

   solution = new double [n];

   cis = new double [a];

   bl = new double [a];

   bu = new double [a];

   xlb = new double [n];

   xub = new double [n];

   irtype = new long [a];



   reset();

}



/* -------------------------------------------------------------------------

 * DESCR   :	Zap everything

 * ------------------------------------------------------------------------- */

Linear_program::~Linear_program()

{

   delete constraints;

   delete solution;

   delete cis;

   delete bl;

   delete bu;

   delete xlb;

   delete xub;

   delete irtype;

}

