/* ---------------------------------------------------------------


 *


 *               Solve A linear programming problem


 * ---------------------------------------------------------- */





#ifndef LINEAR_PROG_DEFS


#define LINEAR_PROG_DEFS





#include <Rn_Defs.h>





/* ----------------------- Classes ----------------------------- */








/* -------------------------------------------------------------


 * DESCR :     


*    Minimize sum_i c_i x_i


*    Subject to bl <= Ax <= bu,  xlb <= x <= xub


 * 		


 * ------------------------------------------------------------ */


class Linear_program  {


 private:


   // These should be const, but fortran wants long * <sigh>


   long n_constraints;   // a


   long n_variables; // n


   


   double *constraints;


   double *solution;


   double *cis, obj;


   double *bl, *bu, *xlb, *xub;


   long *irtype;


   


 public:


   void reset();





   int N_constraints() { return n_constraints; }


   int N_variables() { return n_variables; }


   


   // Solve both absolute and desired


   double solve();


   


   double &Constraints(const int r, const int c);


   double &AXub(const int c);


   double &AXlb(const int c);


   double &Xub(const int r);


   double &Xlb(const int r);


   double &Cis(const int r);





   void Set_irtype_equal(const int r);


   void Set_irtype_lessthan_bu(const int r);


   void Set_irtype_greaterthan_bl(const int r);


   void Set_irtype_between(const int r);





   void Set_irtype_equal();


   void Set_irtype_lessthan_bu();


   void Set_irtype_greaterthan_bl();


   void Set_irtype_between();





   double Solution(const int r);


   double Objective_value() { return obj; }





   // If you want just least-sqrs constraints (axn)


   Linear_program (const int a, const int n);


   ~Linear_program ();


};





inline double &Linear_program::Constraints(const int r, const int c)


{ 


   ASSERT(r >= 0 && r < n_constraints && c >= 0 && c < n_variables);


   return constraints[c * n_constraints + r];


}


inline double &Linear_program::AXub(const int r)


{


   ASSERT(r >= 0 && r < n_constraints);


   return bu[r];


}


inline double &Linear_program::AXlb(const int r)


{


   ASSERT(r >= 0 && r < n_constraints);


   return bl[r];


}


inline double &Linear_program::Xub(const int r)


{


   ASSERT(r >= 0 && r < n_variables);


   return xub[r];


}


inline double &Linear_program::Xlb(const int r)


{


   ASSERT(r >= 0 && r < n_variables);


   return xlb[r];


}


inline double &Linear_program::Cis(const int c)


{


   ASSERT(c >= 0 && c < n_variables);


   return cis[c];


}





inline void Linear_program::Set_irtype_equal(const int r)


{


   ASSERT(r >= 0 && r < n_constraints);


   irtype[r] = 0;


}


inline void Linear_program::Set_irtype_lessthan_bu(const int r)


{


   ASSERT(r >= 0 && r < n_constraints);


   irtype[r] = 1;


}


inline void Linear_program::Set_irtype_greaterthan_bl(const int r)


{


   ASSERT(r >= 0 && r < n_constraints);


   irtype[r] = 2;


}


inline void Linear_program::Set_irtype_between(const int r)


{


   ASSERT(r >= 0 && r < n_constraints);


   irtype[r] = 3;


}





inline void Linear_program::Set_irtype_equal()


{ for (int i = 0; i < n_constraints; i++) irtype[i] = 0; }


inline void Linear_program::Set_irtype_lessthan_bu()


{ for (int i = 0; i < n_constraints; i++) irtype[i] = 1; }


inline void Linear_program::Set_irtype_greaterthan_bl()


{ for (int i = 0; i < n_constraints; i++) irtype[i] = 2; }


inline void Linear_program::Set_irtype_between()


{ for (int i = 0; i < n_constraints; i++) irtype[i] = 3; }





inline double Linear_program::Solution(const int r)


{


   ASSERT(r >= 0 && r < n_variables);


   return solution[r];


}


/* -------------------------- Private Globals  ----------------------------- */





#endif


