#include "StdAfx.H"
#include <utils/Crv_NurbTC.H>
#include <fitting/FITTools_Least_squares.H>


char g_strDataHome[] = "D:\\\\manifolds\\data\\";


void BadLS1()
{
    CRVNub1TC crv( CRVSpace(2), 6 );
    ofstream  out_cmd( WINDataHome( "Paper/LSBad1a_cmd.txt" ), ios::out );
    ofstream  out_pts1( WINDataHome( "Paper/LSBad1_pts1.txt" ), ios::out );
    ofstream  out_pts2( WINDataHome( "Paper/LSBad1_pts2.txt" ), ios::out );
    ofstream  out_pts3( WINDataHome( "Paper/LSBad1_pts3.txt" ), ios::out );
    ofstream  out_cpts1( WINDataHome( "Paper/LSBad1_cntrlpts1.txt" ), ios::out );
    ofstream  out_cpts2( WINDataHome( "Paper/LSBad1_cntrlpts2.txt" ), ios::out );
    ofstream  out_cpts3( WINDataHome( "Paper/LSBad1_cntrlpts3.txt" ), ios::out );
    ofstream  out_crv1( WINDataHome( "Paper/LSBad1_crv1.txt" ), ios::out );
    ofstream  out_crv2( WINDataHome( "Paper/LSBad1_crv2.txt" ), ios::out );
    ofstream  out_crv3( WINDataHome( "Paper/LSBad1_crv3.txt" ), ios::out );

    FITTools_LS oLS1( 17, 6, 1 );
    FITTools_LS oLS2( 17, 6, 1 );
    FITTools_LS oLS3( 17 + crv.Num_pts()-2, 6, 1 );
    Array<double>      adBasis;
    Array<int>         aiIds;
    const double       dDiv = 1.0 / 16.0;

    ASSERT( out_pts1.good() );
    ASSERT( out_crv1.good() );
    ASSERT( out_crv2.good() );

    for ( int i = 0; i < 2; i++ ) {
        crv.Pt(i) = i * dDiv;
        crv.Pt( 5 - i ) = i * dDiv;
    }

    out_cmd << "set terminal fig\n";
    out_cmd << "set output \"" << WINDataHome("Paper/LSBad1a.fig") << "\"\n";
    out_cmd << "plot \"";
    out_cmd << WINDataHome("Paper/LSBad1_cntrlpts1.txt") << "\" ps 2, \"";
    out_cmd << WINDataHome("Paper/LSBad1_crv1.txt") << "\" with lines lw 2, \"";
    out_cmd << WINDataHome("Paper/LSBad1_pts1.txt") << "\" ps 2 \n";
    out_cmd << "set terminal x11\n";
    out_cmd << "plot \"";
    out_cmd << WINDataHome("Paper/LSBad1_cntrlpts1.txt") << "\" ps 2, \"";
    out_cmd << WINDataHome("Paper/LSBad1_crv1.txt") << "\" with lines lw 2, \"";
    out_cmd << WINDataHome("Paper/LSBad1_pts1.txt") << "\" ps 2 \n";
    out_cmd.close();

    out_cmd.open( WINDataHome("Paper/LSBad1b_cmd.txt"), ios::out );
    out_cmd << "set terminal fig\n";
    out_cmd << "set output \"" << WINDataHome("Paper/LSBad1b.fig") << "\"\n";
    out_cmd << "plot \"";
    out_cmd << WINDataHome("Paper/LSBad1_cntrlpts2.txt") << "\" ps 2, \"";
    out_cmd << WINDataHome("Paper/LSBad1_crv2.txt") << "\" with lines lw 2, \"";
    out_cmd << WINDataHome("Paper/LSBad1_pts2.txt") << "\" ps 2 \n";
    out_cmd << "set terminal x11\n";
    out_cmd << "plot \"";
    out_cmd << WINDataHome("Paper/LSBad1_cntrlpts2.txt") << "\" ps 2, \"";
    out_cmd << WINDataHome("Paper/LSBad1_crv2.txt") << "\" with lines lw 2, \"";
    out_cmd << WINDataHome("Paper/LSBad1_pts2.txt") << "\" ps 2 \n";
    out_cmd.close();

    out_cmd.open( WINDataHome("Paper/LSBad1c_cmd.txt"), ios::out );
    out_cmd << "set terminal fig\n";
    out_cmd << "set output \"" << WINDataHome("Paper/LSBad1c.fig") << "\"\n";
    out_cmd << "plot \"";
    out_cmd << WINDataHome("Paper/LSBad1_cntrlpts3.txt") << "\" ps 2, \"";
    out_cmd << WINDataHome("Paper/LSBad1_crv3.txt") << "\" with lines lw 2, \"";
    out_cmd << WINDataHome("Paper/LSBad1_pts3.txt") << "\" ps 2 \n";
    out_cmd << "set terminal x11\n";
    out_cmd << "plot \"";
    out_cmd << WINDataHome("Paper/LSBad1_cntrlpts3.txt") << "\" ps 2, \"";
    out_cmd << WINDataHome("Paper/LSBad1_crv3.txt") << "\" with lines lw 2, \"";
    out_cmd << WINDataHome("Paper/LSBad1_pts3.txt") << "\" ps 2 \n";
    out_cmd.close();

    out_pts1 << "-1.0 -0.1\n";
    out_pts1 << "4.0 0.5\n";

    out_pts2 << "-1.0 -0.1\n";
    out_pts2 << "4.0 0.5\n";

    out_pts3 << "-1.0 -0.1\n";
    out_pts3 << "4.0 0.5\n";

    for ( FORINT i = 0; i < 17; i++ ) {
        double dT = crv.Perc_t( dDiv * i );
        crv.EvalAllBasis( dT, aiIds, adBasis );

        for (int j = 0; j < adBasis.num(); j++) {
            oLS1.Desired_clapack(i, aiIds[j]) = adBasis[j];
            oLS2.Desired_clapack(i, aiIds[j]) = adBasis[j];
            oLS3.Desired_clapack(i, aiIds[j]) = adBasis[j];
        }

        if ( i == 8 ) {
            oLS1.Rhs_clapack(i, 0) = 0.2;
            oLS3.Rhs_clapack(i, 0) = 0.2;
            oLS2.Rhs_clapack(i, 0) = -crv(dT)[0];
            out_pts1 << dT << " 0.2\n";
            out_pts2 << dT << " " << -crv(dT)[0] << " \n";
            out_pts3 << dT << " 0.2\n";
        } else {
            oLS1.Rhs_clapack(i, 0) = -crv(dT)[0];
            oLS2.Rhs_clapack(i, 0) = -crv(dT)[0];
            oLS3.Rhs_clapack(i, 0) = -crv(dT)[0];
            out_pts1 << dT << " " << -crv(dT)[0] << "\n";
            out_pts2 << dT << " " << -crv(dT)[0] << "\n";
            out_pts3 << dT << " " << -crv(dT)[0] << "\n";
        }
    }

    for ( FORINT i = 1; i < crv.Num_pts() - 1; i++ ) {
        oLS3.Desired_clapack(i + 16, i-1) = -0.5;
        oLS3.Desired_clapack(i + 16, i  ) =  1.0;
        oLS3.Desired_clapack(i + 16, i+1) = -0.5;
    }

    out_pts1.close();
    out_pts2.close();
    out_pts3.close();

    oLS1.Print_clapack(cout, FALSE);
    oLS1.Solve_clapack();
    oLS2.Solve_clapack();
    oLS3.Solve_clapack();

    for ( FORINT i = 0; i < crv.Num_pts(); i++ ) {
        crv.Pt(i)[0] = oLS1.Solution_clapack(i, 0);
        out_cpts1 << crv.Perc_t(i * (1.0 / 5.0)) << " " 
                     << crv.Pt(i)[0] << "\n";
    }
    out_cpts1.close();

    const double dCrvDiv = 1.0 / 49.0;
    for ( FORINT i = 0; i < 50; i++ ) {
        double dT = crv.Perc_t( dCrvDiv * i );
        out_crv1 << dT << " " << crv(dT) << "\n";
    }
    out_crv1.close();


    for ( FORINT i = 0; i < crv.Num_pts(); i++ ) {
        crv.Pt(i)[0] = oLS2.Solution_clapack(i, 0);
        out_cpts2 << crv.Perc_t(i * (1.0 / 5.0)) << " " 
                     << crv.Pt(i)[0] << "\n";
    }
    out_cpts2.close();

    for ( FORINT i = 0; i < 50; i++ ) {
        double dT = crv.Perc_t( dCrvDiv * i );
        out_crv2 << dT << " " << crv(dT) << "\n";
    }
    out_crv2.close();

    for ( FORINT i = 0; i < crv.Num_pts(); i++ ) {
        crv.Pt(i)[0] = oLS3.Solution_clapack(i, 0);
        out_cpts3 << crv.Perc_t(i * (1.0 / 5.0)) << " " 
                     << crv.Pt(i)[0] << "\n";
    }
    out_cpts3.close();

    for ( FORINT i = 0; i < 50; i++ ) {
        double dT = crv.Perc_t( dCrvDiv * i );
        out_crv3 << dT << " " << crv(dT) << "\n";
    }
    out_crv3.close();
}

static double s_adBadLS2[] =
{ 0.0, 0.02, 0.04, 0.06, 0.08, 0.1, 0.9, 0.92, 0.94, 0.96, 0.98, 1.0};

void BadLS2a()
{
    CRVNub1TC crv( CRVSpace(1), 7 );
    ofstream  out_cmd( WINDataHome( "Paper/LSBad2a_cmd.txt" ), ios::out );
    ofstream  out_pts( WINDataHome( "Paper/LSBad2_pts1.txt" ), ios::out );
    ofstream  out_cntrlpts( WINDataHome( "Paper/LSBad2_cntrlpts1.txt" ), ios::out );
    ofstream  out_crv( WINDataHome( "Paper/LSBad2_crv1.txt" ), ios::out );

    FITTools_LS oLS( 12, 6, 1 );
    Array<double>      adBasis;
    Array<int>         aiIds;

    ASSERT( out_pts.good() );
    ASSERT( out_crv.good() );

    out_cmd << "set terminal fig\n";
    out_cmd << "set output \"" << WINDataHome("Paper/LSBad2a.fig") << "\"\n";
    out_cmd << "plot \"";
    out_cmd << WINDataHome("Paper/LSBad2_crv1.txt") << "\" with lines lw 2, \"";
    out_cmd << WINDataHome("Paper/LSBad2_cntrlpts1.txt") << "\" ps 2, \"";
    out_cmd << WINDataHome("Paper/LSBad2_pts1.txt") << "\" ps 2\n";
    out_cmd << "set terminal x11\n";
    out_cmd << "plot \"";
    out_cmd << WINDataHome("Paper/LSBad2_crv1.txt") << "\" with lines lw 2, \"";
    out_cmd << WINDataHome("Paper/LSBad2_cntrlpts1.txt") << "\" ps 2, \"";
    out_cmd << WINDataHome("Paper/LSBad2_pts1.txt") << "\" ps 2\n";
    out_cmd.close();

    out_pts << "-0.5 -0.1\n";
    out_pts << "5.5 2.0\n";

    for ( int i = 0; i < 12; i++ ) {
        double dT = crv.Perc_t( s_adBadLS2[i] );
        crv.EvalAllBasis( dT, aiIds, adBasis );

        for (int j = 0; j < adBasis.num(); j++) {
            if ( aiIds[j] > 3 )
                oLS.Desired_clapack(i, aiIds[j]-1) = adBasis[j];
            else
                oLS.Desired_clapack(i, aiIds[j]) = adBasis[j];
        }

        if ( i < 6 ) 
            oLS.Rhs_clapack(i, 0) = 0.1 * i;
        else
            oLS.Rhs_clapack(i, 0) = 0.1 * (11 - i);
        out_pts << dT << " " << oLS.Rhs_clapack(i, 0) << "\n";
    }
    out_pts.close();

    oLS.Print_clapack(cout, FALSE);
    oLS.Solve_clapack();

    for ( FORINT i = 0; i < 3; i++ ) {
        crv.Pt(i)[0] = oLS.Solution_clapack(i, 0);
    }
    crv.Pt(3) = 0;
    for ( FORINT i = 4; i < 7; i++ ) {
        crv.Pt(i)[0] = oLS.Solution_clapack(i-1, 0);
    }
    for ( FORINT i = 0; i < 7; i++ ) {
        out_cntrlpts << crv.Perc_t(i * (1.0 / 6.0)) << " " 
                     << crv.Pt(i)[0] << "\n";
    }
    out_cntrlpts.close();

    const double dCrvDiv = 1.0 / 49.0;
    for ( FORINT i = 0; i < 50; i++ ) {
        double dT = crv.Perc_t( dCrvDiv * i );
        out_crv << dT << " " << crv(dT) << "\n";
    }
    out_crv.close();
}

void BadLS2b()
{
    CRVNub1TC crv( CRVSpace(1), 7 );
    ofstream  out_cmd( WINDataHome( "Paper/LSBad2b_cmd.txt" ), ios::out );
    ofstream  out_pts( WINDataHome( "Paper/LSBad2_pts2.txt" ), ios::out );
    ofstream  out_extrapts( WINDataHome( "Paper/LSBad2_extrapts2.txt" ), ios::out );
    ofstream  out_cntrlpts( WINDataHome( "Paper/LSBad2_cntrlpts2.txt" ), ios::out );
    ofstream  out_crv( WINDataHome( "Paper/LSBad2_crv2.txt" ), ios::out );

    FITTools_LS oLS( 12 + 7, 7, 1 );
    Array<double>      adBasis;
    Array<int>         aiIds;

    ASSERT( out_pts.good() );
    ASSERT( out_crv.good() );

    out_cmd << "set terminal fig\n";
    out_cmd << "set output \"" << WINDataHome("Paper/LSBad2b.fig") << "\"\n";
    out_cmd << "plot \"";
    out_cmd << WINDataHome("Paper/LSBad2_crv2.txt") << "\" with lines lw 2, \"";
    out_cmd << WINDataHome("Paper/LSBad2_cntrlpts2.txt") << "\" ps 2, \"";
    out_cmd << WINDataHome("Paper/LSBad2_pts2.txt") << "\" ps 2, \"";
    out_cmd << WINDataHome("Paper/LSBad2_extrapts2.txt") << "\" ps 2\n";
    out_cmd << "set terminal x11\n";
    out_cmd << "plot \"";
    out_cmd << WINDataHome("Paper/LSBad2_crv2.txt") << "\" with lines lw 2, \"";
    out_cmd << WINDataHome("Paper/LSBad2_cntrlpts2.txt") << "\" ps 2, \"";
    out_cmd << WINDataHome("Paper/LSBad2_pts2.txt") << "\" ps 2, \"";
    out_cmd << WINDataHome("Paper/LSBad2_extrapts2.txt") << "\" ps 2\n";
    out_cmd.close();

    out_pts << "-0.5 -0.1\n";
    out_pts << "5.5 2.0\n";

    for ( int i = 0; i < 12; i++ ) {
        double dT = crv.Perc_t( s_adBadLS2[i] );
        crv.EvalAllBasis( dT, aiIds, adBasis );

        for (int j = 0; j < adBasis.num(); j++) {
            oLS.Desired_clapack(i, aiIds[j]) = adBasis[j];
        }

        if ( i < 6 ) 
            oLS.Rhs_clapack(i, 0) = 0.1 * i;
        else
            oLS.Rhs_clapack(i, 0) = 0.1 * (11 - i);
        out_pts << dT << " " << oLS.Rhs_clapack(i, 0) << "\n";
    }
    out_pts.close();

    for ( FORINT i = 0; i < 7; i++ ) {
        double dT = crv.Perc_t( i * (1.0 / 6.0) );
        crv.EvalAllBasis( dT, aiIds, adBasis );

        for (int j = 0; j < adBasis.num(); j++) {
            oLS.Desired_clapack(i+12, aiIds[j]) = adBasis[j];
        }

        if ( i < 2 ) 
            oLS.Rhs_clapack(i+12, 0) = 0.55 * i;
        else if ( i < 5 )
            oLS.Rhs_clapack(i+12, 0) = 0.55;
        else 
            oLS.Rhs_clapack(i+12, 0) = 0.55 * (6 - i);
        out_extrapts << dT << " " << oLS.Rhs_clapack(i+12, 0) << "\n";
    }
    out_extrapts.close();

    oLS.Print_clapack(cout, FALSE);
    oLS.Solve_clapack();

    for ( FORINT i = 0; i < 7; i++ ) {
        crv.Pt(i)[0] = oLS.Solution_clapack(i, 0);
    }
    for ( FORINT i = 0; i < 7; i++ ) {
        out_cntrlpts << crv.Perc_t(i * (1.0 / 6.0)) << " " 
                     << crv.Pt(i)[0] << "\n";
    }
    out_cntrlpts.close();

    const double dCrvDiv = 1.0 / 49.0;
    for ( FORINT i = 0; i < 50; i++ ) {
        double dT = crv.Perc_t( dCrvDiv * i );
        out_crv << dT << " " << crv(dT) << "\n";
    }
    out_crv.close();
}


static double s_adBadLS3[] =
{ 0.0, 0.0, 0.0, 0.0, 0.02, 0.0, 0.0};

static double s_adBadLS3Ts[] =
{ 0.0, 0.2, 0.4, 0.6, 0.62, 0.8, 1.0};

void BadLS3()
{
    CRVNub1TC crv( CRVSpace(1), 8 );
    ofstream  out_cmd1( WINDataHome( "Paper/LSBad3a_cmd.txt" ), ios::out );
    ofstream  out_cpts1( WINDataHome( "Paper/LSBad3_cpts1.txt" ), ios::out );
    ofstream  out_pts1( WINDataHome( "Paper/LSBad3_pts1.txt" ), ios::out );
    ofstream  out_crv1( WINDataHome( "Paper/LSBad3_crv1.txt" ), ios::out );
    ofstream  out_cmd2( WINDataHome( "Paper/LSBad3b_cmd.txt" ), ios::out );
    ofstream  out_cpts2( WINDataHome( "Paper/LSBad3_cpts2.txt" ), ios::out );
    ofstream  out_pts2( WINDataHome( "Paper/LSBad3_pts2.txt" ), ios::out );
    ofstream  out_crv2( WINDataHome( "Paper/LSBad3_crv2.txt" ), ios::out );

    FITTools_LS oLS1( 7, 8, 1 );
    FITTools_LS oLS2( 7 + crv.Num_pts() - 2, 8, 1 );
    Array<double>      adBasis;
    Array<int>         aiIds;

    ASSERT( out_pts1.good() );
    ASSERT( out_crv1.good() );

    out_cmd1 << "set terminal fig\n";
    out_cmd1 << "set output \"" << WINDataHome("Paper/LSBad3a.fig") << "\"\n";
    out_cmd1 << "plot \"";
    out_cmd1 << WINDataHome("Paper/LSBad3_cpts1.txt") << "\" ps 2, \"";
    out_cmd1 << WINDataHome("Paper/LSBad3_crv1.txt") << "\" with lines lw 2, \"";
    out_cmd1 << WINDataHome("Paper/LSBad3_pts1.txt") << "\" ps 2\n";
    out_cmd1 << "set terminal x11\n";
    out_cmd1 << "plot \"";
    out_cmd1 << WINDataHome("Paper/LSBad3_cpts1.txt") << "\" ps 2, \"";
    out_cmd1 << WINDataHome("Paper/LSBad3_crv1.txt") << "\" with lines lw 2, \"";
    out_cmd1 << WINDataHome("Paper/LSBad3_pts1.txt") << "\" ps 2\n";
    out_cmd1.close();

    out_cmd2 << "set terminal fig\n";
    out_cmd2 << "set output \"" << WINDataHome("Paper/LSBad3b.fig") << "\"\n";
    out_cmd2 << "plot \"";
    out_cmd2 << WINDataHome("Paper/LSBad3_cpts2.txt") << "\" ps 2, \"";
    out_cmd2 << WINDataHome("Paper/LSBad3_crv2.txt") << "\" with lines lw 2, \"";
    out_cmd2 << WINDataHome("Paper/LSBad3_pts2.txt") << "\" ps 2\n";
    out_cmd2 << "set terminal x11\n";
    out_cmd2 << "plot \"";
    out_cmd2 << WINDataHome("Paper/LSBad3_cpts2.txt") << "\" ps 2, \"";
    out_cmd2 << WINDataHome("Paper/LSBad3_crv2.txt") << "\" with lines lw 2, \"";
    out_cmd2 << WINDataHome("Paper/LSBad3_pts2.txt") << "\" ps 2\n";
    out_cmd2.close();

    out_pts1 << "-0.5 -1.1\n";
    out_pts1 << "6.5 2.0\n";

    out_pts2 << "-0.5 -1.1\n";
    out_pts2 << "6.5 2.0\n";

    for ( int i = 0; i < 7; i++ ) {
        double dT = crv.Perc_t( s_adBadLS3Ts[i] );
        crv.EvalAllBasis( dT, aiIds, adBasis );
        
        for (int j = 0; j < adBasis.num(); j++) {
            oLS1.Desired_clapack(i, aiIds[j]) = adBasis[j];
            oLS2.Desired_clapack(i, aiIds[j]) = adBasis[j];
        }
        
        oLS1.Rhs_clapack(i, 0) = s_adBadLS3[i];
        oLS2.Rhs_clapack(i, 0) = s_adBadLS3[i];
        out_pts1 << dT << " " << oLS1.Rhs_clapack(i, 0) << "\n";
        out_pts2 << dT << " " << oLS2.Rhs_clapack(i, 0) << "\n";
    }
    out_pts1.close();
    out_pts2.close();

    for ( FORINT i = 1; i < crv.Num_pts() - 1; i++ ) {
        oLS2.Desired_clapack(i + 6, i-1) = -0.5;
        oLS2.Desired_clapack(i + 6, i  ) =  1.0;
        oLS2.Desired_clapack(i + 6, i+1) = -0.5;
    }


    oLS1.Print_clapack(cout, FALSE);
    oLS1.Solve_clapack();
    oLS2.Solve_clapack();

    for ( FORINT i = 0; i < crv.Num_pts(); i++ ) {
        crv.Pt(i)[0] = oLS1.Solution_clapack(i, 0);
        out_cpts1 << crv.Perc_t(i * (1.0 / 8.0)) << " " 
                     << crv.Pt(i)[0] << "\n";
    }
    out_cpts1.close();

    const double dCrvDiv = 1.0 / 49.0;
    for ( FORINT i = 0; i < 50; i++ ) {
        double dT = crv.Perc_t( dCrvDiv * i );
        out_crv1 << dT << " " << crv(dT) << "\n";
    }
    out_crv1.close();

    for ( FORINT i = 0; i < crv.Num_pts(); i++ ) {
        crv.Pt(i)[0] = oLS2.Solution_clapack(i, 0);
        out_cpts2 << crv.Perc_t(i * (1.0 / 8.0)) << " " 
                     << crv.Pt(i)[0] << "\n";
    }
    out_cpts2.close();

    for ( FORINT i = 0; i < 50; i++ ) {
        double dT = crv.Perc_t( dCrvDiv * i );
        out_crv2 << dT << " " << crv(dT) << "\n";
    }
    out_crv2.close();
}
