// CurvatureCaseStudy.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "CurvatureProfiles.H"

char g_strDataHome[] = "c://graphics/cmg/data/";

int main(int argc, char* argv[])
{

    if ( argc == 1 || (argc > 1 && atoi(argv[1]) > 2) ) {
        printf("Arguments:\n");
        printf("0 : Tests a single point\n");
        printf("1 : Runs the spherical, ellipsoid, and saddle texts\n");
        printf("2 meshname method outputfile : Calculates the curvature for a mesh using the given method (0..22)\n");
        printf("Method: \n");
        for ( int i = 1; i < MCData::NUM_METHODS; i++ )
            printf("%d %s\n", i, MCData::Description( (MCData::MC_Method) (i) ) );
        exit(0);
    }
    // Set Mode: 0=Test one mesh
    //           1=Run profile suite

    const int mode = atoi(argv[1]);
    char strMesh[1024];
    PMeshLite mesh;
    MCData data;
    int iMethod = 1;
    FILE *out = stdout;

    switch(mode) {
        case 0:
            // Test for a single mesh
            printf("Testing a single point\n");
            NoiseTest(1,6,1.0,1.0,0.5,5.0,5.0,1,0.0,0.0,1,-0.1,-0.1,1,
                      0.0,0.0,1,0.0,0.0,1,0.0,0.0,1);
            //NoiseTest(1,5,1.0,1.0,1.0,5.0,5.0,1,0.0,0.0,1,0.0,0.0,1,
            //          0.0,0.0,1,0.0,0.0,1,0.0,0.0,1);
            //NoiseTest(1,3,1.0,1.0,0.5,5.0,5.0,1,0.0,0.0,1,0.0,0.0,1,
            //          0.0,0.0,1,0.0,0.0,1,0.0,0.0,1);
            break;
        case 1:
            // Run the sphere profile
            printf("Running the sphere profile\n");
            SphereProfile(1.0);

            // Run the ellipsoid profile
            printf("Running the ellipsoid profile\n");
            EllipsoidProfile(1.0,1.0,0.5);

            // Run the saddle profile
            printf("Running the saddle profile\n");
            SaddleProfile(1.0,1.0,0.5);
            break;
        case 2:
            if ( argc == 2 ) {
                printf("Mesh name ?\n");
                cin >> strMesh; 
                mesh.Read( strMesh );
            } else {
                mesh.Read( argv[2] );
                printf("Calculating for mesh %s\n", argv[2] );
            }
            mesh.MakeTriangular();

            if ( argc == 3 ) {
                printf("Method: \n");
                for ( int i = 1; i < MCData::NUM_METHODS; i++ )
                    printf("%d %s\n", i, MCData::Description( (MCData::MC_Method) (i) ) );
                cin >> iMethod;
            } else {
                iMethod = atoi( argv[3] );
                printf("Method %s\n", MCData::Description( (MCData::MC_Method) (iMethod) ) );
            }

            data.Init( mesh, (MCData::MC_Method) (iMethod) );
            data.CalculateCurvature();

            if ( argc == 5 ) {
                out = fopen( argv[4], "w" );
            }
            data.dump( out );
            if ( argc == 5 ) {
                printf("Output to %s\n", argv[4] );
                fclose(out);
            }
            break;
        default:
            printf("Must be one of 0,1, or 2\n");
            break;
    }

    // Done
    printf("--- Done --- \n");

    return 0;
}

