#include "stdafx.h"
#include <math.h>
#include "CurvatureProfiles.H"
#include "PMeshLiteUtils.h"
#include <MC/MC_Data.H>
#include <MC/MC_Point.H>
#include <MC/MC_MeshUtils.H>

/***************** SphereProfile ***********************************
  Compute impact of noise on a sphere for a range of errors
  *******************************************************************/
void SphereProfile( const double radius )
{
    int n;

    for( n=3; n<9; n++ ) {
        NoiseTest(0,n,radius,1.0,1.0,1.0,4.0,4,0.0,0.0,1,0.0,0.0,1,
                  0.0,0.0,1,0.0,0.0,1,0.0,0.0,1);
        NoiseTest(0,n,radius,1.0,1.0,5.0,30.0,6,0.0,0.0,1,0.0,0.0,1,
                  0.0,0.0,1,0.0,0.0,1,0.0,0.0,1);
    }
    NoiseTest(0,6,radius,1.0,1.0,5.0,10.0,2,-0.1,0.1,6,0.0,0.0,1,
              0.0,0.0,1,0.0,0.0,1,0.0,0.0,1);
    NoiseTest(0,6,radius,1.0,1.0,5.0,10.0,2,0.0,0.0,1,-0.1,0.1,6,
              0.0,0.0,1,0.0,0.0,1,0.0,0.0,1);
    NoiseTest(0,6,radius,1.0,1.0,5.0,10.0,2,0.0,0.0,1,0.0,0.0,1,
              -4.0,-1.0,4,0.0,0.0,1,0.0,0.0,1);
    NoiseTest(0,6,radius,1.0,1.0,5.0,10.0,2,0.0,0.0,1,0.0,0.0,1,
              1.0,4.0,4,0.0,0.0,1,0.0,0.0,1);
    NoiseTest(0,6,radius,1.0,1.0,5.0,10.0,2,0.0,0.0,1,0.0,0.0,1,
              0.0,0.0,1,1.0,4.0,4,0.0,0.0,1);
    NoiseTest(0,6,radius,1.0,1.0,5.0,10.0,2,0.0,0.0,1,0.0,0.0,1,
              0.0,0.0,1,0.0,0.0,1,5.0,45.0,9);
}

/***************** EllipsoidProfile ***********************************
  Compute impact of noise on a ellipsoid for a range of errors
  *******************************************************************/
void EllipsoidProfile( const double radius, const double a, const double b )
{
    int n;

    for( n=3; n<9; n++ ) {
        NoiseTest(1,n,radius,a,b,1.0,4.0,4,0.0,0.0,1,0.0,0.0,1,
                  0.0,0.0,1,0.0,0.0,1,0.0,0.0,1);
        NoiseTest(1,n,radius,a,b,5.0,30.0,6,0.0,0.0,1,0.0,0.0,1,
                  0.0,0.0,1,0.0,0.0,1,0.0,0.0,1);
    }
    NoiseTest(1,6,radius,a,b,5.0,10.0,2,-0.1,0.1,6,0.0,0.0,1,
              0.0,0.0,1,0.0,0.0,1,0.0,0.0,1);
    NoiseTest(1,6,radius,a,b,5.0,10.0,2,0.0,0.0,1,-0.1,0.1,6,
              0.0,0.0,1,0.0,0.0,1,0.0,0.0,1);
    NoiseTest(1,6,radius,a,b,5.0,10.0,2,0.0,0.0,1,0.0,0.0,1,
              -4.0,-1.0,4,0.0,0.0,1,0.0,0.0,1);
    NoiseTest(1,6,radius,a,b,5.0,10.0,2,0.0,0.0,1,0.0,0.0,1,
              1.0,4.0,4,0.0,0.0,1,0.0,0.0,1);
    NoiseTest(1,6,radius,a,b,5.0,10.0,2,0.0,0.0,1,0.0,0.0,1,
              0.0,0.0,1,1.0,4.0,4,0.0,0.0,1);
    NoiseTest(1,6,radius,a,b,5.0,10.0,2,0.0,0.0,1,0.0,0.0,1,
              0.0,0.0,1,0.0,0.0,1,5.0,45.0,9);
}

/***************** SaddleProfile ***********************************
  Compute impact of noise on a saddle for a range of errors
  *******************************************************************/
void SaddleProfile( const double radius, const double a, const double b )
{
    int n;

    for( n=3; n<9; n++ ) {
        NoiseTest(2,n,radius,a,b,1.0,4.0,4,0.0,0.0,1,0.0,0.0,1,
                  0.0,0.0,1,0.0,0.0,1,0.0,0.0,1);
        NoiseTest(2,n,radius,a,b,5.0,30.0,6,0.0,0.0,1,0.0,0.0,1,
                  0.0,0.0,1,0.0,0.0,1,0.0,0.0,1);
    }
    NoiseTest(2,6,radius,a,b,5.0,10.0,2,-0.1,0.1,6,0.0,0.0,1,
              0.0,0.0,1,0.0,0.0,1,0.0,0.0,1);
    NoiseTest(2,6,radius,a,b,5.0,10.0,2,0.0,0.0,1,-0.1,0.1,6,
              0.0,0.0,1,0.0,0.0,1,0.0,0.0,1);
    NoiseTest(2,6,radius,a,b,5.0,10.0,2,0.0,0.0,1,0.0,0.0,1,
              -4.0,-1.0,4,0.0,0.0,1,0.0,0.0,1);
    NoiseTest(2,6,radius,a,b,5.0,10.0,2,0.0,0.0,1,0.0,0.0,1,
              1.0,4.0,4,0.0,0.0,1,0.0,0.0,1);
    NoiseTest(2,6,radius,a,b,5.0,10.0,2,0.0,0.0,1,0.0,0.0,1,
              0.0,0.0,1,1.0,4.0,4,0.0,0.0,1);
    NoiseTest(2,6,radius,a,b,5.0,10.0,2,0.0,0.0,1,0.0,0.0,1,
              0.0,0.0,1,0.0,0.0,1,5.0,45.0,9);
}

/***************** NoiseTest ***********************************
  Test for the impact of noise on a generic shape
  *******************************************************************/
void NoiseTest( const int shape, const int valence,
                const double radius, const double in_a, const double in_b,  
                const double phi1, const double phi2, const int nphi,
                const double dr_e1, const double dr_e2, const int ndr_e, 
                const double dr_c1, const double dr_c2, const int ndr_c,
                const double dphi_e1, const double dphi_e2, const int ndphi_e, 
                const double dphi_c1, const double dphi_c2, const int ndphi_c, 
                const double dtheta1, const double dtheta2, const int ndtheta,
                const int nRings )
{
    PMeshLite mesh;
    Array< MCData > data;
    Array< MCPoint > points;
    Array< WINbool > valid;
    double PI;
    WINbool ok;

    int n;
    int i0, i1, i2, i3, i4, i5;
    double r, a, b, phi, dr_e, dr_c, dphi_e, dphi_c, dtheta;
    double r_phi, r_dphi_e, r_dphi_c, r_dtheta;

    if( shape < 0 || shape > 2 ) return;     /* Check for valid shape */

    /* --- Initialize data array --- */

    InitializeData( mesh, data );

    a = in_a;
    b = in_b;
    PI = 4.0*atan(1.0);
    r = radius;
    n = valence;

    for( i0=0; i0<nphi; i0++ ) {
    for( i1=0; i1<ndr_e; i1++ ) {
    for( i2=0; i2<ndr_c; i2++ ) {
    for( i3=0; i3<ndphi_e; i3++ ) {
    for( i4=0; i4<ndphi_c; i4++ ) {
    for( i5=0; i5<ndtheta; i5++ ) {

        phi = phi1;
        if( nphi > 1 ) phi = ( (nphi-(i0+1)) * phi1 + i0 * phi2 )
                                                   / (nphi-1);

        dr_e = dr_e1;
        if( ndr_e > 1 ) dr_e = ( (ndr_e-(i1+1)) * dr_e1 + i1 * dr_e2 )
                                                   / (ndr_e-1);

        dr_c = dr_c1;
        if( ndr_c > 1 ) dr_c = ( (ndr_c-(i2+1)) * dr_c1 + i2 * dr_c2 )
                                                   / (ndr_c-1);

        dphi_e = dphi_e1;
        if( ndphi_e > 1 ) dphi_e = ( (ndphi_e-(i3+1)) * dphi_e1
                                        + i3 * dphi_e2 ) / (ndphi_e-1);

        dphi_c = dphi_c1;
        if( ndphi_c > 1 ) dphi_c = ( (ndphi_c-(i4+1)) * dphi_c1
                                        + i4 * dphi_c2 ) / (ndphi_c-1);

        dtheta = dtheta1;
        if( ndtheta > 1 ) dtheta = ( (ndtheta-(i5+1)) * dtheta1
                                        + i5 * dtheta2 ) / (ndtheta-1);

        /* --- Make the mesh --- */

        r_phi    = phi    * PI / 180.0;
        r_dphi_e = dphi_e * PI / 180.0;
        r_dphi_c = dphi_c * PI / 180.0;
        r_dtheta = dtheta * PI / 180.0;
        
        switch(shape) {
           case 0:
               ok = MakeSphericalNRing( mesh, r, n, r_phi, dr_e, dr_c, 
                                        r_dphi_e, r_dphi_c, r_dtheta, nRings );
               a = 1.0;
               b = 1.0;
			   break;
           case 1:
               ok = MakeEllipsoidNRing( mesh, r, a, b, n, r_phi, dr_e,  dr_c, 
                                        r_dphi_e, r_dphi_c, r_dtheta, nRings );
               break;
            case 2:
               ok = MakeSaddleNRing( mesh, r, a, b, n, r_phi, dr_e,  dr_c,
                                     r_dphi_e, r_dphi_c, r_dtheta, nRings );
               break;
        }

        if( !ok ) continue;

        /* --- Calculate metrics --- */

        CalculateCurvatures( data, points, valid );

        /* --- Output mesh parameters --- */

        WriteCurvatureData( shape, r, n, a, b,
                            phi, dr_e, dr_c, dphi_e, dphi_c, dtheta,
                            data, points, valid );

        WriteVectorErrorData( shape, r, n, a, b,
                              phi, dr_e, dr_c, dphi_e, dphi_c, dtheta,
                              data, points, valid );

    } /* End of i5 loop */
    } /* End of i4 loop */
    } /* End of i3 loop */
    } /* End of i2 loop */
    } /* End of i1 loop */
    } /* End of i0 loop */

    points.clearcompletely();
    valid.clearcompletely();
}

void InitializeData( PMeshLite &mesh, Array<MCData> &data )
{
	int n = 20;
    data.init(n);
    int i = 0;
    data[i++].Init(mesh,MCData::ANCE);
    data[i++].Init(mesh,MCData::ANC);
    data[i++].Init(mesh,MCData::IEV);
    data[i++].Init(mesh,MCData::DCO);
    data[i++].Init(mesh,MCData::MDCO);
    data[i++].Init(mesh,MCData::FIT1P);
    data[i++].Init(mesh,MCData::FIT1N);
    data[i++].Init(mesh,MCData::FIT2P);
    data[i++].Init(mesh,MCData::FIT2N);
    data[i++].Init(mesh,MCData::FIT3P);
    data[i++].Init(mesh,MCData::FIT3N);
    data[i++].Init(mesh,MCData::CON1);
    data[i++].Init(mesh,MCData::CON2);
    data[i++].Init(mesh,MCData::CON3);
    data[i++].Init(mesh,MCData::RBF1P);
    data[i++].Init(mesh,MCData::RBF1N);
    data[i++].Init(mesh,MCData::RBF2P);
    data[i++].Init(mesh,MCData::RBF2N);
    data[i++].Init(mesh,MCData::RBF3P);
    data[i++].Init(mesh,MCData::RBF3N);
}

void CalculateCurvatures( Array<MCData> &data, Array<MCPoint> &points,
                          Array<WINbool> &valid )
{
    Vertex v;
    MCPoint p;

    points.need(0);
    valid.need(0);

    for( int i=0; i<data.num(); i++ ) {
        v = data[i].Mesh()->VertexFromIndexId(0);
        
        if( data[i].CalculateCurvature(v, p) == 0 ) {
            valid += TRUE;
        } else {
            valid += FALSE;
        }
        points += p;
    }
}

void WriteCurvatureData( const int shape, 
                         const double radius, const int valence,
                         const double a, const double b,  
                         const double phi, const double dr_e, 
                         const double dr_c, const double dphi_e, 
                         const double dphi_c, const double dtheta,
                         Array<MCData> &data, Array<MCPoint> &points,
                         Array<WINbool> &valid )
{
    double k1, k2, kg, kbar;
    MCData::MC_Method method;
    FILE *out = NULL;

    for( int i=0; i<data.num(); i++ ) {
        method = data[i].GetMethod();
        switch(method) {
            case MCData::EXACT:
                out = fopen(MyDataHome("EXACT.txt","CurvatureCaseStudy"),"a");
                //out = fopen(WINDataHome("EXACT.txt"),"a");
                break;
            case MCData::DCO:
                out = fopen(MyDataHome("DCO.txt","CurvatureCaseStudy"),"a");
                //out = fopen(WINDataHome("DCO.txt"),"a");
                break;
            case MCData::MDCO:
                out = fopen(MyDataHome("MDCO.txt","CurvatureCaseStudy"),"a");
                //out = fopen(WINDataHome("MDCO.txt"),"a");
                break;
            case MCData::ANCE:
                out = fopen(MyDataHome("ANCE.txt","CurvatureCaseStudy"),"a");
                //out = fopen(WINDataHome("ANCE.txt"),"a");
                break;
            case MCData::ANC:
                out = fopen(MyDataHome("ANC.txt","CurvatureCaseStudy"),"a");
                //out = fopen(WINDataHome("ANC.txt"),"a");
                break;
            case MCData::IEV:
                out = fopen(MyDataHome("IEV.txt","CurvatureCaseStudy"),"a");
                //out = fopen(WINDataHome("IEV.txt"),"a");
                break;
            case MCData::FIT1P:
                out = fopen(MyDataHome("FIT1P.txt","CurvatureCaseStudy"),"a");
                //out = fopen(WINDataHome("FIT1P.txt"),"a");
                break;
            case MCData::FIT2P:
                out = fopen(MyDataHome("FIT2P.txt","CurvatureCaseStudy"),"a");
                //out = fopen(WINDataHome("FIT2P.txt"),"a");
                break;
            case MCData::FIT3P:
                out = fopen(MyDataHome("FIT3P.txt","CurvatureCaseStudy"),"a");
                //out = fopen(WINDataHome("FIT3P.txt"),"a");
                break;
            case MCData::FIT1N:
                out = fopen(MyDataHome("FIT1N.txt","CurvatureCaseStudy"),"a");
                //out = fopen(WINDataHome("FIT1N.txt"),"a");
                break;
            case MCData::FIT2N:
                out = fopen(MyDataHome("FIT2N.txt","CurvatureCaseStudy"),"a");
                //out = fopen(WINDataHome("FIT2N.txt"),"a");
                break;
            case MCData::FIT3N:
                out = fopen(MyDataHome("FIT3N.txt","CurvatureCaseStudy"),"a");
                //out = fopen(WINDataHome("FIT3N.txt"),"a");
                break;
            case MCData::RBF1P:
                out = fopen(MyDataHome("RBF1P.txt","CurvatureCaseStudy"),"a");
                //out = fopen(WINDataHome("RBF1P.txt"),"a");
                break;
            case MCData::RBF2P:
                out = fopen(MyDataHome("RBF2P.txt","CurvatureCaseStudy"),"a");
                //out = fopen(WINDataHome("RBF2P.txt"),"a");
                break;
            case MCData::RBF3P:
                out = fopen(MyDataHome("RBF3P.txt","CurvatureCaseStudy"),"a");
                //out = fopen(WINDataHome("RBF3P.txt"),"a");
                break;
            case MCData::RBF1N:
                out = fopen(MyDataHome("RBF1N.txt","CurvatureCaseStudy"),"a");
                //out = fopen(WINDataHome("RBF1N.txt"),"a");
                break;
            case MCData::RBF2N:
                out = fopen(MyDataHome("RBF2N.txt","CurvatureCaseStudy"),"a");
                //out = fopen(WINDataHome("RBF2N.txt"),"a");
                break;
            case MCData::RBF3N:
                out = fopen(MyDataHome("RBF3N.txt","CurvatureCaseStudy"),"a");
                //out = fopen(WINDataHome("RBF3N.txt"),"a");
                break;
            case MCData::CON1:
                out = fopen(MyDataHome("CON1.txt","CurvatureCaseStudy"),"a");
                //out = fopen(WINDataHome("CON1.txt"),"a");
                break;
            case MCData::CON2:
                out = fopen(MyDataHome("CON2.txt","CurvatureCaseStudy"),"a");
                //out = fopen(WINDataHome("CON2.txt"),"a");
                break;
            case MCData::CON3:
                out = fopen(MyDataHome("CON3.txt","CurvatureCaseStudy"),"a");
                //out = fopen(WINDataHome("CON3.txt"),"a");
                break;
            default:
                out = NULL;
                break;
        }

        if( out != NULL ) {
            kbar = points[i].Kbar();
            kg = points[i].Kg();
            k1 = points[i].K1();
            k2 = points[i].K2();
            fprintf(out,"%d ", shape);
            switch( shape ) {
                case 0:
                    fprintf(out, "%8.4lf %2d %8.4lf %8.4lf ",
                                 radius, valence, a, b );
                    fprintf(out, "%7.3lf %8.4lf %8.4lf %7.3lf %7.3lf %7.3lf",
                                 phi, dr_e, dr_c, dphi_e, dphi_c, dtheta );
                    fprintf(out, "%10.5lf %10.5lf %10.5lf %10.5lf\n",
                                 kbar, kg, k1, k2 );
                    break;
                case 1:
                case 2:
                    fprintf(out, "%8.4lf %2d %8.4lf %8.4lf ",
                                 radius, valence, a, b );
                    fprintf(out, "%7.3lf %8.4lf %8.4lf %7.3lf %7.3lf %7.3lf",
                               phi, dr_e, dr_c, dphi_e, dphi_c, dtheta );
                    fprintf(out, "%10.5lf %10.5lf %10.5lf %10.5lf\n",
                                 kbar, kg, k1, k2 );
                    break;
                default:
                    break;
            }
            fclose(out);
        }
    }
}

void WriteVectorErrorData( const int shape, 
                           const double radius, const int valence,
                           const double a, const double b,  
                           const double phi, const double dr_e, 
                           const double dr_c, const double dphi_e, 
                           const double dphi_c, const double dtheta,
                           Array<MCData> &data, Array<MCPoint> &points,
                           Array<WINbool> &valid )
{
    MCData::MC_Method method;
    FILE *out = NULL;

	double PI, alpha, k1_exact, k2_exact, kg_exact, kb_exact;
	double d0, d1, d2, de0, de1, de2, N_err, e1_err, e2_err, cang, tmp;
	R3Vec N, N_exact;
	R3Vec e1, e1_exact;
	R3Vec e2, e2_exact;

	PI = 4.0*atan(1.0);
	alpha = 0.25*PI;

	// Compute exact values
	switch(shape) {
        case 0:
            k1_exact = 1.0/radius;
            k2_exact = 1.0/radius;
            break;
        case 1:
			if(a*a < b*b) {
				k1_exact = radius/(a*a);
				k2_exact = radius/(b*b);
			} else {
				k1_exact = radius/(b*b);
				k2_exact = radius/(a*a);
			}
	        break;
        case 2:
            if( a*a < b*b ) {
                k1_exact =  1.0 / a;
                k2_exact = -1.0 / b;
			} else {
                k1_exact = -1.0 / b;
                k2_exact =  1.0 / a;
			}
            break;
    }
	kg_exact = k1_exact * k2_exact;
    kb_exact = 0.5*(k1_exact + k2_exact);
	N_exact  = R3Vec(0.0, 0.0, 1.0);
    e1_exact = R3Vec( cos(alpha), -sin(alpha), 0.0);
    e2_exact = R3Vec( sin(alpha),  cos(alpha), 0.0);

	de0 = sqrt(N_exact[0]*N_exact[0] + 
		       N_exact[1]*N_exact[1] + 
			   N_exact[2]*N_exact[2]);
	de1 = sqrt(e1_exact[0]*e1_exact[0] + 
		       e1_exact[1]*e1_exact[1] + 
			   e1_exact[2]*e1_exact[2]);
	de2 = sqrt(e2_exact[0]*e2_exact[0] + 
		       e2_exact[1]*e2_exact[1] + 
			   e2_exact[2]*e2_exact[2]);

    for( int i=0; i<data.num(); i++ ) {
        method = data[i].GetMethod();
        switch(method) {
            case MCData::EXACT:
                out = fopen(MyDataHome("EXACT_Verr.txt","CurvatureCaseStudy"),"a");
                break;
            case MCData::DCO:
                out = fopen(MyDataHome("DCO_Verr.txt","CurvatureCaseStudy"),"a");
                break;
            case MCData::MDCO:
                out = fopen(MyDataHome("MDCO_Verr.txt","CurvatureCaseStudy"),"a");
                break;
            case MCData::ANCE:
                out = fopen(MyDataHome("ANCE_Verr.txt","CurvatureCaseStudy"),"a");
                break;
            case MCData::ANC:
                out = fopen(MyDataHome("ANC_Verr.txt","CurvatureCaseStudy"),"a");
                break;
            case MCData::IEV:
                out = fopen(MyDataHome("IEV_Verr.txt","CurvatureCaseStudy"),"a");
                break;
            case MCData::FIT1P:
                out = fopen(MyDataHome("FIT1P_Verr.txt","CurvatureCaseStudy"),"a");
                break;
            case MCData::FIT2P:
                out = fopen(MyDataHome("FIT2P_Verr.txt","CurvatureCaseStudy"),"a");
                break;
            case MCData::FIT3P:
                out = fopen(MyDataHome("FIT3P_Verr.txt","CurvatureCaseStudy"),"a");
                break;
            case MCData::FIT1N:
                out = fopen(MyDataHome("FIT1N_Verr.txt","CurvatureCaseStudy"),"a");
                break;
            case MCData::FIT2N:
                out = fopen(MyDataHome("FIT2N_Verr.txt","CurvatureCaseStudy"),"a");
                break;
            case MCData::FIT3N:
                out = fopen(MyDataHome("FIT3N_Verr.txt","CurvatureCaseStudy"),"a");
                break;
            case MCData::RBF1P:
                out = fopen(MyDataHome("RBF1P_Verr.txt","CurvatureCaseStudy"),"a");
                break;
            case MCData::RBF2P:
                out = fopen(MyDataHome("RBF2P_Verr.txt","CurvatureCaseStudy"),"a");
                break;
            case MCData::RBF3P:
                out = fopen(MyDataHome("RBF3P_Verr.txt","CurvatureCaseStudy"),"a");
                break;
            case MCData::RBF1N:
                out = fopen(MyDataHome("RBF1N_Verr.txt","CurvatureCaseStudy"),"a");
                break;
            case MCData::RBF2N:
                out = fopen(MyDataHome("RBF2N_Verr.txt","CurvatureCaseStudy"),"a");
                break;
            case MCData::RBF3N:
                out = fopen(MyDataHome("RBF3N_Verr.txt","CurvatureCaseStudy"),"a");
                break;
            case MCData::CON1:
                out = fopen(MyDataHome("CON1_Verr.txt","CurvatureCaseStudy"),"a");
                break;
            case MCData::CON2:
                out = fopen(MyDataHome("CON2_Verr.txt","CurvatureCaseStudy"),"a");
                break;
            case MCData::CON3:
                out = fopen(MyDataHome("CON3_Verr.txt","CurvatureCaseStudy"),"a");
                break;
            default:
                out = NULL;
                break;
        }

        if( out != NULL ) {
			N = points[i].Normal();
			e1 = points[i].KmaxVec();
            e2 = points[i].KminVec();
            d0 = sqrt(N[0]*N[0] + N[1]*N[1] + N[2]*N[2]);
            d1 = sqrt(e1[0]*e1[0] + e1[1]*e1[1] + e1[2]*e1[2]);
            d2 = sqrt(e2[0]*e2[0] + e2[1]*e2[1] + e2[2]*e2[2]);

			N_err = N_exact[0]*N[0] + N_exact[1]*N[1] + N_exact[2]*N[2];
			if( de0 == 0.0 || d0 == 0.0 ) {
				N_err = 999.0;
			} else {
				cang = N_err/(de0*d0);
				cang = (cang < -1.0) ? -1.0 : (cang > 1.0) ? 1.0 : cang;
				N_err = acos(cang)*180.0/PI;
			}

			e1_err = e1_exact[0]*e1[0] + e1_exact[1]*e1[1] + e1_exact[2]*e1[2];
			if( de1 == 0.0 || d1 == 0.0 ) {
				e1_err = 999.0;
			} else {
				if( e1_err < 0.0 ) e1_err = -e1_err;
				cang = e1_err/(de1*d1);
				cang = (cang < -1.0) ? -1.0 : (cang > 1.0) ? 1.0 : cang;
				e1_err = acos(cang)*180.0/PI;
			}

			e2_err = e2_exact[0]*e2[0] + e2_exact[1]*e2[1] + e2_exact[2]*e2[2];
			if( de2 == 0.0 || d2 == 0.0 ) {
				e2_err = 999.0;
			} else {
				if( e2_err < 0.0 ) e2_err = -e2_err;
				cang = e2_err/(de2*d2);
				cang = (cang < -1.0) ? -1.0 : (cang > 1.0) ? 1.0 : cang;
				e2_err = acos(cang)*180.0/PI;
			}

			if( e1_err > 45.0 && e2_err > 45.00 ) {
				tmp = e1_err;
				e1_err = e2_err - 90.0;
				e2_err = tmp - 90.0;
			}
			
			fprintf(out,"%d ", shape);
            switch( shape ) {
                case 0:
                    fprintf(out, "%8.4lf %2d %8.4lf %8.4lf ",
                                 radius, valence, a, b );
                    fprintf(out, "%7.3lf %8.4lf %8.4lf %7.3lf %7.3lf %7.3lf",
                                 phi, dr_e, dr_c, dphi_e, dphi_c, dtheta );
                    fprintf(out, "%10.5lf %10.5lf %10.5lf\n",
                                 N_err, e1_err, e2_err );
                    break;
                case 1:
                case 2:
                    fprintf(out, "%8.4lf %2d %8.4lf %8.4lf ",
                                 radius, valence, a, b );
                    fprintf(out, "%7.3lf %8.4lf %8.4lf %7.3lf %7.3lf %7.3lf",
                               phi, dr_e, dr_c, dphi_e, dphi_c, dtheta );
                    fprintf(out, "%10.5lf %10.5lf %10.5lf\n",
                                 N_err, e1_err, e2_err );
                    break;
                default:
                    break;
            }
            fclose(out);
        }
    }
}
