#include "StdAfx.H"
#include <utils/Rn_Defs.H>
#include <utils/Pm_MeshLite.H>
#include <map>

Face   
PMeshLite::FaceFromVertices( const Vertex in_v1, const Vertex in_v2, const Vertex in_v3 ) const
{
    Edge e1 = queryEdge( in_v1, in_v2 );
    Edge e2 = queryEdge( in_v2, in_v3 );
    Edge e3 = queryEdge( in_v3, in_v1 );

    if ( e1 == NULL || e2 == NULL || e3 == NULL ) return NULL;

    Face f11 = face1( e1 );
    Face f12 = face2( e1 );
    Face f21 = face1( e2 );
    Face f22 = face2( e2 );
    Face f31 = face1( e3 );
    Face f32 = face2( e3 );

    if ( f11 == f21 && f11 == f31 ) return f11;
    if ( f11 == f21 && f11 == f32 ) return f11;
    if ( f11 == f22 && f11 == f31 ) return f11;
    if ( f11 == f22 && f11 == f32 ) return f11;

    if ( f12 == f21 && f12 == f31 ) return f12;
    if ( f12 == f21 && f12 == f32 ) return f12;
    if ( f12 == f22 && f12 == f31 ) return f12;
    if ( f12 == f22 && f12 == f32 ) return f12;

    return NULL;
}

Face   
PMeshLite::FaceFromVertices( const Vertex in_v1, const Vertex in_v2 ) const
{
    Edge e1 = queryEdge( in_v1, in_v2 );

    if ( e1 == NULL ) return NULL;

    Face f1 = face1( e1 );
    Face f2 = face2( e1 );

    Array<Vertex> aopVF1;
    Array<Vertex> aopVF2;
    if ( f1 ) aopVF1 = FaceVertices(f1);
    if ( f2 ) aopVF2 = FaceVertices(f2);

    for ( int iV = 0; iV < aopVF1.num(); iV++ )
        if ( aopVF1[iV] == in_v1 && aopVF1.wrap(iV+1) == in_v2 )
            return f1;

    for ( FORINT iV = 0; iV < aopVF2.num(); iV++ )
        if ( aopVF2[iV] == in_v1 && aopVF2.wrap(iV+1) == in_v2 )
            return f2;

    return NULL;
}

void PMeshLite::Color( const UTILSColor &in_col )
{
    ForMeshVertex( *this, v ) {
        v->SetColor( in_col );
    } EndFor;

    SetDefaultColor(in_col);
}

Face PMeshLite::AddFace( const Array<Vertex> &in_aopVs, const int id  )
{
    Face f = NULL;
    if(id == -1)
        f = createFace( in_aopVs );
    else f = createFaceI( id, in_aopVs );

    m_bHasChanged = TRUE;

    return f;
}

void PMeshLite::AddFacesFixOrientation( const Array< Array<Vertex> > &in_aavface )
{
    boost::dynamic_bitset<> abVisited( in_aavface.num() );
    boost::dynamic_bitset<> abBad( in_aavface.num() );
    Array<Vertex> av;

    abVisited.set();
    abBad.reset();

    if (in_aavface.num() == 0 )
        return;

    int iAddedTotal = 0;
    if ( IsOkToAdd( in_aavface[0] ) ) {
        AddFace( in_aavface[0] );
        iAddedTotal++;
    } else {
        TRACE("PMeshLite: Bad first face\n");
        return;
    }
    abVisited.reset(0);

    int iFirst = abVisited.find_first();
    while ( iFirst != -1 ) {
        int iAdded = 0;
        for ( int iNext = iFirst; iNext != -1; iNext = abVisited.find_next(iNext) ) {
            for ( int iV = 0; iV < in_aavface[iNext].num(); iV++ ) {
                if ( EdgeFromVertices( in_aavface[iNext][iV], in_aavface[iNext].wrap(iV+1) ) ) {
                    abVisited.reset(iNext);
                    if ( IsOkToAdd( in_aavface[iNext] ) ) {
                        AddFace( in_aavface[iNext] );
                        iAdded++;
                    } else {
                        av = in_aavface[iNext];
                        av.reverse();
                        if ( IsOkToAdd( av ) ) {
                            AddFace( av );
                            iAdded++;
                        } else {
                            abBad.set( iNext );
                        }
                    }
                    break;
                }
            }
        }
        iFirst = abVisited.find_first();
        if ( iAdded == 0 && iFirst != -1 ) {
            abVisited.reset(iFirst);
            if ( IsOkToAdd( in_aavface[iFirst] ) )
                AddFace( in_aavface[iFirst] );
            iFirst = abVisited.find_next(iFirst);
        }
        iAddedTotal += iAdded;
    }

    if ( abBad.count() == 0 || iAddedTotal == 0 ) 
        return;

    Array<Vertex> avDup;
    Array<int> aiMap( NumVertices() );
    aiMap.fill(-1);

    Array< Array<Vertex> > aav( abBad.count() );
    int iCount = 0;
    for ( int iNext = abBad.find_first(); iNext != -1; iNext = abBad.find_next(iNext) ) {
        aav[iCount].need( in_aavface[iNext].num() );
        for ( int iV = 0; iV < aav[iCount].num(); iV++ ) {
            const int iId = in_aavface[iNext][iV]->IndexId();
            if ( aiMap[ iId ] != -1 ) {
                aav[iCount][iV] = avDup[ aiMap[iId] ];
            } else if ( Degree( in_aavface[iNext][iV] ) == 0 ) {
                aav[iCount][iV] = in_aavface[iNext][iV];
            } else {
                aiMap[ iId ] = avDup.num();
                avDup += AddVertex( in_aavface[iNext][iV]->GetLoc(), UTILSColor::RED );
                aav[iCount][iV] = avDup[ aiMap[iId] ];
            }
        }
        iCount++;
    }
    ASSERT( iCount == abBad.count() );
    TRACE("Bad faces %d added %d vertices\n", abBad.count(), avDup.num() );
    AddFacesFixOrientation( aav );
}


void PMeshLite::RemoveAdjacentFaces( Vertex in_v )
{
    Array<Face> aopFaces = VertexFaces( in_v );

    for (int i = 0; i < aopFaces.num(); i++) {
        RemoveFace(aopFaces[i]);
    }

    m_bHasChanged = TRUE;
}

void PMeshLite::RemoveFaces( Array<Face> &in_aopF )
{
    for (int i = 0; i < in_aopF.num(); i++)
        RemoveFace( in_aopF[i] );
}

/// Also removes adjacent edges and faces
void PMeshLite::RemoveVertices( Array<Vertex> &in_aopV )
{
    for (int i = 0; i < in_aopV.num(); i++)
        RemoveVertex( in_aopV[i] );
}


void PMeshLite::RemoveDupVertices( double in_dPerc){
    Array<double> adEdgeWeights(NumEdges());
    double sumEdge = 0.0;
    ForMeshEdge( Mesh(), e ) {
		adEdgeWeights[e->IndexId()] = GetEdgeLength(e);
        sumEdge += adEdgeWeights[e->IndexId()];
    } EndFor;
    double threshold = (sumEdge/(double)NumEdges())*in_dPerc;

    Array<Edge> edgesToCollapse(0);
    ForMeshEdge( Mesh(), e ) {
        if(adEdgeWeights[e->IndexId()] < threshold && IsOkToCollapse(e)){  
            edgesToCollapse += e;
        }
    } EndFor;
    for(int i = 0; i< edgesToCollapse.num(); i++){
        if(Valid(edgesToCollapse[i]) && IsOkToCollapse(edgesToCollapse[i])){  
            EdgeCollapse(edgesToCollapse[i]);
        }
    }
    Renumber();
}

void PMeshLite::Clear() 
{
    clear();
}


void PMeshLite::Renumber(  )
{
    renumber( );
}

/// Centroid of a face
R3Pt PMeshLite::GetPointInFace( Face                  in_oFace,
                                const Array<double> & in_ptPerc ) const
{
    Array< Vertex > aopVs = FaceVertices( in_oFace );
    
    ASSERT( aopVs.num() == in_ptPerc.num() );

    R3Pt ptOut(0,0,0);
    
    for (int i = 0; i < aopVs.num(); i++) {
        const R3Pt &pt = point( aopVs[i] );
        for (int j = 0; j < 3; j++)
            ptOut[j] += in_ptPerc[i] * pt[j];
    }
    
    return ptOut;
}


/// Centroid of a face
R3Pt PMeshLite::GetFaceCentroid( Face in_oFace ) const
{
    Array< Vertex > aopVs = FaceVertices( in_oFace );
    
    R3Pt ptOut(0,0,0);
    
    const double dPerc = 1.0 / (double) aopVs.num();
    for (int i = 0; i < aopVs.num(); i++) {
        const R3Pt &pt = point( aopVs[i] );
        for (int j = 0; j < 3; j++)
            ptOut[j] += dPerc * pt[j];
    }
    
    return ptOut;
}

/// Point along an edge
R3Pt PMeshLite::GetPointOnEdge( Edge    in_oEdge, 
                                double  in_dPerc ) const
{
    return Lerp( point( vertex1(in_oEdge) ), point( vertex2(in_oEdge) ), in_dPerc );
}

void  PMeshLite::FaceVertexEdge ( Face           in_oFace,
                                  Array<Vertex> &out_aopVs,
                                  Array<Edge>   &out_aopEs )   const
{
    out_aopVs = FaceVertices( in_oFace );
    static Array<Edge> aopEs;
    aopEs = FaceEdges( in_oFace );

    out_aopEs.need( aopEs.num() );
    for ( int i = 0; i < out_aopEs.num(); i++ ) {
        out_aopEs[i] = aopEs[(i+1)%aopEs.num()];
    }
    for ( FORINT  i = 0; i < out_aopVs.num(); i++ ) {
        Vertex v1 = EdgeVertex1( out_aopEs[i] );
        Vertex v2 = EdgeVertex2( out_aopEs[i] );

        ASSERT( ( out_aopVs[i] == v1 && out_aopVs.wrap(i+1) == v2 ) ||
                ( out_aopVs[i] == v2 && out_aopVs.wrap(i+1) == v1 ) );
    }
}


void PMeshLite::Regularize( double in_dPerc )
{
    Array<Vertex> aopVs;
    Array<int>    aiN;
    Array<Face >  aopFs;
    
    AllMeshFaces( aopFs );
    AllMeshVertices( aopVs );

	R3Pt ptScl, ptCenter;
	ptCenter = GetCenter( ptScl );

	Array<Vertex> aopAlignVs;
	for ( int i = 0; i < aopVs.num(); i++ ) {

		for (int iD  = 0; iD < 3; iD++) {
			const double dComp = point(aopVs[i])[iD];
			const double dClip = in_dPerc * ptScl[iD];
			aopAlignVs.clearcompletely();

			for (int j = i+1; j < aopVs.num(); j++) {
				R3Pt pt = point(aopVs[j]);
				const double dDist = fabs( pt[iD] - dComp );
				if ( dDist < dClip ) {
					pt[iD] = dComp;
					setPoint(aopVs[j], pt ) ;
				}
			}
		}
	}
}

int 
PMeshLite::AlphaBeta( const Face &in_face, const Vertex &in_vertex,
                         double &out_theta, 
                         double &out_alpha, double &out_beta,
                         double &t, double &a, double &b ) const
{
    int id;
    Array< Vertex > aov;
    R3Pt T, A, B;

    aov = FaceVertices(in_face);
    id = -1;
    if( in_vertex->IndexId() == aov[0]->IndexId() ) id = 0;
    if( in_vertex->IndexId() == aov[1]->IndexId() ) id = 1;
    if( in_vertex->IndexId() == aov[2]->IndexId() ) id = 2;
    if( id == -1 ) return id;

    T = aov[id]->GetLoc();
    A = aov[(id+1)%3]->GetLoc();
    B = aov[(id+2)%3]->GetLoc();

    t = Length(A-B);
    a = Length(B-T);
    b = Length(T-A);

    out_alpha = acos( ( b*b + t*t - a*a ) / ( 2.0 * b * t ) );
    out_beta  = acos( ( t*t + a*a - b*b ) / ( 2.0 * t * a ) );
    out_theta = acos( ( a*a + b*b - t*t ) / ( 2.0 * a * b ) );
    
    return 0;
}

int 
PMeshLite::AlphaBeta( const Vertex &in_vertex, const Edge &in_edge,
                         double &out_theta1, double &out_theta2, 
                         double &out_alpha, double &out_beta,
                         double &out_delta, double &out_gamma ) const
{
    int i;
    Face f1, f2;
    Vertex opposite;
    Array< Vertex > aov;
    Array<double> angles;
    R3Pt P, Q, R;

    opposite = EdgeVertex1(in_edge);
    if( in_vertex == opposite ) opposite = EdgeVertex2(in_edge);

	out_theta1 = 0.0;
	out_theta2 = 0.0;
	out_alpha  = 0.0;
	out_beta   = 0.0;
	out_delta  = 0.0;
	out_gamma  = 0.0;

    f1 = EdgeFace1(in_edge);
    f2 = EdgeFace2(in_edge);

	if( f1 != NULL ) {
		angles.need(0);
		FaceAngles(f1,angles);
		ASSERT( angles.num() == 3 );

		aov = FaceVertices(f1);
		for( i=0; i<aov.num(); i++ ) {
		    if(aov[i] == in_vertex ) {
			    out_theta1 = angles[i];
		    } else {
		        if( aov[i] == opposite ) {
		            out_delta = angles[i];
		        } else {
		            out_alpha = angles[i];
		        }
		    }
		}
	}

	if( f2 != NULL ) {
		angles.need(0);
		FaceAngles(f2,angles);
		ASSERT( angles.num() == 3 );

		aov = FaceVertices(f2);
		for( i=0; i<aov.num(); i++ ) {
		    if(aov[i] == in_vertex ) {
		        out_theta2 = angles[i];
		    } else {
		        if( aov[i] == opposite ) {
		            out_gamma = angles[i];
		       } else {
		          out_beta = angles[i];
		      }
		   }
		}
	}
    return 0;
}

double 
PMeshLite::CotAlphaBeta( const Vertex &v1, const Vertex &v2 ) const
{
    double theta1, theta2, alpha, beta, delta, gamma, 
           tan_alpha, tan_beta, result;

    const Edge e = EdgeFromVertices(v1,v2);
    if( e == NULL )  {
        ASSERT(FALSE);
        return 0.0;
    }
    result = 0.0;

    const int ier = AlphaBeta(v1, e, theta1, theta2, alpha, beta, delta, gamma);

	if( EdgeFace1(e) != NULL ) {
		// Compute the alpha component 
		tan_alpha = tan(alpha);
        if ( ! RNIsZero( tan_alpha ) ) {
		    result += 1.0 / tan_alpha;
        }
	}
    
	if( EdgeFace2(e) != NULL ) {
		// Compute the beta component 
		tan_beta = tan(beta);
        if ( !RNIsZero( tan_beta ) ) {
            result += 1.0 / tan_beta;
        }
	}

    return result;
}

double 
PMeshLite::CotGammaDelta( const Vertex &v1, const Vertex &v2 ) const
{
    int ier;
    double theta1, theta2, alpha, beta, delta, gamma, 
           tan_delta, tan_gamma, result;
    Edge e;

    e = EdgeFromVertices(v1,v2);
    if( e == NULL ) return 0.0;
    result = 0.0;

    FILE *out = NULL;
    //out = fopen(WINDataHome("edges.dat"),"a");
    if( out != NULL ) {
        fprintf(out,"Result: %d to %d\n", 
                    EdgeVertex1(e)->IndexId(),
                    EdgeVertex2(e)->IndexId() );
        fclose(out);
        out = NULL;
    }

    ier = AlphaBeta(v1, e, theta1, theta2, alpha, beta, delta, gamma);

	if( EdgeFace2(e) != NULL ) {
		// Compute the gamma component 
		tan_gamma = tan(gamma);
		if( tan_gamma != 0.0 ) result += 1.0 / tan_gamma;
	}
    
	if( EdgeFace2(e) != NULL ) {
		// Compute the delta component 
		tan_delta = tan(delta);
		if( tan_delta != 0.0 ) result += 1.0 / tan_delta;
	}

    return result;
}

int 
PMeshLite::FaceAngles(const Face &in_face, const Vertex &in_vertex,
                         int &out_ThetaId, double &out_Theta, double &out_t, 
                         int &out_AlphaId, double &out_Alpha, double &out_a, 
                         int &out_BetaId, double &out_Beta, double &out_b ) const
{
    int i, n, id;
    Array< Vertex > aov;
    R3Pt P, Q, R;

	if( in_face == NULL ) return -1;

    aov = FaceVertices(in_face);

    n = aov.num();
    id = -1;
    for( i=0; i<n; i++ ) {
        if( aov[i] == in_vertex ) {
            id = i;
            break;
        }
    }
    if( id == -1 ) return -1;

    out_ThetaId = aov[i]->IndexId();
    P = aov[id]->GetLoc();
    Q = aov[(id+1)%n]->GetLoc();
    R = aov[(id+2)%n]->GetLoc();

    out_t = Length(Q - R);
    out_a = Length(R - P);
    out_b = Length(P - Q);

    out_Theta = acos( ( out_a*out_a + out_b*out_b - out_t*out_t ) / 
                      ( 2.0 * out_a * out_b ) );
    out_Alpha = acos( ( out_t*out_t + out_b*out_b - out_a*out_a ) / 
                      ( 2.0 * out_t * out_b ) );
    out_Beta  = acos( ( out_a*out_a + out_t*out_t - out_b*out_b ) / 
                      ( 2.0 * out_a * out_t ) );

    out_ThetaId = aov[id]->IndexId();
    out_AlphaId = aov[(id+1)%n]->IndexId();
    out_BetaId  = aov[(id+2)%n]->IndexId();

    return 0;
}
    
void PMeshLite::FaceAngles( const Face &in_face, Array<double> &out_adAngles ) const
{
    const Array< Vertex > aov = FaceVertices(in_face);
    out_adAngles.need( aov.num() );

    // In case something goes wrong
    const double dAng = M_PI / (double) out_adAngles.num();
    for ( int i = 0; i < out_adAngles.num(); i++ )
        out_adAngles[i] = dAng;

    Array< double > adDistances( aov.num() );
    WINbool bIsZero = FALSE;
    for ( FORINT i = 0; i < aov.num(); i++ ) {
        adDistances[i] = Length( aov[i]->GetLoc() - aov.wrap(i+1)->GetLoc() );
        if ( RNIsZero( adDistances[i] ) )
            bIsZero = TRUE;
    }
    if ( bIsZero == TRUE ) {
        ASSERT( aov.num() == 3 ); // This is only the right thing to do if this is a triangle
        return; 
    }

    double dSumAllAngles = 0.0;
    for ( FORINT i = 0; i < aov.num(); i++ ) {
        const double dDenom = 2.0 * adDistances[i] * adDistances.wrap(i-1);  // Two adjacent sides
        const double dSumAngles = ( adDistances[i] * adDistances[i] + adDistances.wrap(i-1) * adDistances.wrap(i-1) - adDistances.wrap(i+1)*adDistances.wrap(i+1) ) / dDenom;
        out_adAngles[i] = acos( WINminmax( dSumAngles, -1.0, 1.0 ) );

        dSumAllAngles += out_adAngles[i];
    }

    if ( aov.num() == 3 )
        ASSERT( RNApproxEqual( dSumAllAngles, M_PI, 1e-6 ) );
}

int
PMeshLite::CotAngles( const Face &in_face, const Vertex &in_vertex,
                         int &out_ThetaId, double &out_CotTheta, double &out_t, 
                         int &out_AlphaId, double &out_CotAlpha, double &out_a, 
                         int &out_BetaId, double &out_CotBeta, double &out_b ) const
{
    int ier;
    double theta, alpha, beta, TanTheta, TanAlpha, TanBeta;

    ier = FaceAngles(in_face, in_vertex, 
                     out_ThetaId, theta, out_t, 
                     out_AlphaId, alpha, out_a,
                     out_BetaId, beta, out_b);
    if( ier != 0 ) return ier;

    out_CotTheta = 0.0;
    TanTheta = tan(theta);
    if( TanTheta != 0.0 ) out_CotTheta = 1.0/TanTheta;

    out_CotAlpha = 0.0;
    TanAlpha = tan(alpha);
    if( TanAlpha != 0.0 ) out_CotAlpha = 1.0/TanAlpha;

    out_CotBeta = 0.0;
    TanBeta = tan(beta);
    if( TanBeta != 0.0 ) out_CotBeta = 1.0/TanBeta;

    return 0;
}



void PMeshLite::Center( const R3Pt &in_ptCenter, const double in_dRadius )
{
    R3Pt ptScl;
    const R3Pt ptCenter = GetCenter( ptScl );

    const double dWidth = WINmax( ptScl[0], WINmax( ptScl[1], ptScl[2] ) );

    const double dScl = (RNIsZero(dWidth)) ? 1.0 : (2.0 * in_dRadius) / dWidth;

    R4Matrix oFrame = R4Matrix::Scaling( dScl ) * R4Matrix::Translation( R3Pt(0,0,0) - ptCenter );

    Transform( oFrame );

}

Vertex PMeshLite::GetExtreme() const
{
    R3Pt ptScale, ptCenter;
    ptCenter = GetCenter( ptScale );

    double dMaxDist = 0.0;
    Vertex vRet = NULL;

    ForMeshVertex( Mesh(), v ) {
        if ( LengthSq( v->GetLoc() - ptCenter ) > dMaxDist ) {
            vRet = v;
            dMaxDist = LengthSq( v->GetLoc() - ptCenter );
        }
    }EndFor;

    return vRet;
}

void PMeshLite::FollowBoundary( Vertex          in_v, 
                                Array<Vertex> & out_aopVs ) const
{
    ASSERT( IsBoundary( in_v ) );

    out_aopVs.clearcompletely();
    out_aopVs += in_v;

    Array<Vertex> aopVs;
    WINbool bFound = FALSE;
    do {
        aopVs = VertexVertices( out_aopVs.last() );

        bFound = FALSE;
        for (int i = 0; i < aopVs.num(); i++) {
            // Previous vertex
            if ( aopVs[i] == out_aopVs.wrap( out_aopVs.num() - 2 ) )
                continue;

            if ( IsBoundary( aopVs[i] ) ) {
                Edge opE = EdgeFromVertices( out_aopVs.last(), aopVs[i] );
                ASSERT( opE );
                if ( EdgeFace1(opE) == NULL || EdgeFace2(opE) == NULL ) {
                    bFound = TRUE;
                    if ( out_aopVs.num() > 2 && aopVs[i] == in_v )
                        return;

                    // Shouldn't happen with a clean boundary (no multiple edges)
                    if ( out_aopVs.contains(aopVs[i]) ) {
                        TRACE("Possible boundary problem at %d to %d, duplicate %d\n",
                               out_aopVs.last()->IndexId(),
                               aopVs[i]->IndexId(),
                               out_aopVs.index( aopVs[i] ) );
                    } else {
                        out_aopVs += aopVs[i];
                        break;
                    }
                }
            }
        }
    } while ( bFound == TRUE );

}

