#include "StdAfx.H"

#include <utils/Pm_MeshLite.H>
#include <utils/Mesh_PolygonSpatial.H>


WINbool PMeshLite::IsOkToAdd( const Array<Vertex> &in_avs )
{
    if ( !legalCreateFace(in_avs) )
        return FALSE;
    return TRUE;
}


void PMeshLite::Compute_normals( Array<Vertex> & in_aopVs, 
                                 Array<Face>   & in_aopFs)
{
    R3Vec vecCross;
    Array<Vertex> aopV;

    for (int i = 0; i < in_aopFs.num(); i++) {
        vertices( in_aopFs[i], aopV );
        vecCross = Cross( aopV[2]->GetLoc() - aopV[1]->GetLoc(), aopV[0]->GetLoc() - aopV[1]->GetLoc() );

        in_aopFs[i]->SetNorm( UnitSafe( vecCross ) );
    }

    for (FORINT i = 0; i < in_aopVs.num(); i++) {
        R3Vec vec(0,0,0);
        
        Array<Face> aopFaces = VertexFaces( in_aopVs[i] );
        double dCount = 0.0;
        for (int j = 0; j < aopFaces.num(); j++) {
            vec += norm(aopFaces[j]);
            dCount += 1.0;
        } 
        if ( dCount > 0.0 )
            vec = vec * ( 1.0 / dCount );

        setNorm( in_aopVs[i], vec );
    }

    m_bHasChanged = FALSE;
}

void PMeshLite::Compute_normals( )
{
    Array<Face> aopFaces;
    AllMeshFaces( aopFaces );

    Array<Vertex> aopVerts;
    AllMeshVertices( aopVerts );

    Compute_normals( aopVerts, aopFaces );
}


void PMeshLite::Compute_face_normals( )
{
    R3Vec vecCross;
    Array<Vertex> aopV;
    ForMeshFace( (*this), f ) {
        vertices( f, aopV );
        vecCross = Cross( aopV[2]->GetLoc() - aopV[1]->GetLoc(), aopV[0]->GetLoc() - aopV[1]->GetLoc() );

        f->SetNorm( UnitSafe( vecCross ) );
    } EndFor;
}





PMeshLite &PMeshLite::operator=(const PMeshLite &in_oPick)
{
    Clear();

    copy( in_oPick );
    m_bHasChanged = TRUE;

    return *this;
}

R3Pt PMeshLite::GetCenter( R3Pt &out_ptScale ) const
{
	R3Pt ptMin(1e30, 1e30, 1e30);
	R3Pt ptMax(-1e30, -1e30, -1e30);

	ForMeshVertex( *this, v ) {
		const R3Pt &p = point(v);
		for (int j = 0; j < 3; j++) {
			ptMin[j] = WINmin(ptMin[j], (double) p[j]);
			ptMax[j] = WINmax(ptMax[j], (double) p[j]);
		}
	} EndFor;

	R3Pt ptCenter;
	for (int j = 0; j < 3; j++) {
		out_ptScale[j] = ptMax[j] - ptMin[j];
        ptCenter[j] = (RNIsZero(out_ptScale[j]) ? 0.0 : (0.5 * (ptMax[j] + ptMin[j])) );
	}
	return ptCenter;
}

void PMeshLite::Flip()
{
    Array< Array<Vertex> > aaFVs;
    Array< Face >          aF;
    ForMeshFace( *this, f ) {
        aaFVs.add(1);
        vertices(f, aaFVs.last());
        aaFVs.last().reverse();
        aF += f;
    }EndFor;

    for (int i = 0; i < aF.num(); i++)
        RemoveFace( aF[i] );

    Array< Vertex > aVs;
    for ( FORINT i = 0; i < aaFVs.num(); i++ ) {
        aVs.need( aaFVs[i].num() );
        for (int j = 0; j < aVs.num(); j++)
            aVs[j] = aaFVs[i][j];

        AddFace( aVs );
    }
}

Array<R2Contour>  
PMeshLite::Contours( const R3Vec &in_vecPlaneNormal,
                     double       in_dStartZ,
                     double       in_dWidth ) const
{ 
    Array<R2Contour> aoContours;
    R3Vec vecNorm( UnitSafe( in_vecPlaneNormal ) );

    GMesh oMesh;
    ASSERT( ! RNIsZero( Length( vecNorm ) ) );

    R3Matrix mat;
    R3Matrix::MatrixVecToVec( mat, vecNorm, R3Vec(0,0,1) );

    // Line up the mesh so the desired axis is the z-axis
    oMesh.copy( *this );
    oMesh.transform( mat );
    R3Vec vecCheck = mat * vecNorm;

    double dZMin = 1e30, dZMax = -1e30;
    ForMeshVertex( oMesh, v ) {
        const R3Pt &p = oMesh.point( v );
        dZMin = WINmin( p[2], dZMin );
        dZMax = WINmax( p[2], dZMax );
    } EndFor;

    // Starting slice
    double dZStart = in_dStartZ;
    while( dZStart > dZMin )  dZStart -= in_dWidth;
    while( dZStart < dZMin )  dZStart += in_dWidth;

    // jiggle any points that lie exactly on one of the contour lines
    double dZ = dZStart;
    while ( dZ < dZMax ) {
        ForMeshVertex( oMesh, v ) {
            R3Pt p = oMesh.point( v );
            if ( RNApproxEqual( p[2], dZ ) ) {
                p[2] += in_dWidth * 0.00001;
                oMesh.setPoint( v, p );
            }
        } EndFor;
        dZ += in_dWidth;
    }

    dZ = dZStart;
    Array<Edge>  aoEdge;
    Array<Face>  aoFace1, aoFace2;
    Array<R3Pt> apPts;

    while ( dZ < dZMax ) {
        aoEdge.need(0);
        apPts.need(0);
        aoFace1.need(0);
        aoFace2.need(0);
        // Find all edges that cross the current slice line
        ForMeshEdge( oMesh, e ) {
            const R3Pt &p1 = oMesh.point( oMesh.vertex1( e ) );
            const R3Pt &p2 = oMesh.point( oMesh.vertex2( e ) );

            if ( p1[2] > dZ && p2[2] < dZ ) {
                aoEdge += e;
                apPts += Lerp( p1, p2, (dZ - p2[2]) / (p1[2] - p2[2]) );
                aoFace1 += oMesh.face1( e );
                aoFace2 += oMesh.face2( e );
            } else if ( p1[2] < dZ && p2[2] > dZ ) {
                aoEdge += e;
                apPts += Lerp( p2, p1, (dZ - p1[2]) / (p2[2] - p1[2]) );
                aoFace1 += oMesh.face1( e );
                aoFace2 += oMesh.face2( e );
            }
        } EndFor;

        // new contour
        aoContours.add( 1 );
        aoContours.last().SetPlane( vecNorm, dZ );
        aoContours.last().Width() = in_dWidth;
        while ( aoEdge.num() ) {
            Array< R3Pt > aptAdd;

            // Grab an edge
            aptAdd += apPts.last();
            Edge e = aoEdge.last();
            Face f1 = aoFace1.last();
            Face f2 = aoFace2.last();
            apPts.sub(1);
            aoEdge.sub(1);
            aoFace1.sub(1);
            aoFace2.sub(1);
            
            // walk around the contour, chasing faces
            do {
                WINbool bIsFound = FALSE;
                for (int j = 0; j < aoFace1.num(); j++) {
                    if ( aoFace1[j] == f2 || aoFace2[j] == f2) {
                        aptAdd += apPts[j];
                        if ( aoFace1[j] == f2 ) {
                            f2 = aoFace2[j];
                        } else {
                            f2 = aoFace1[j];
                        }

                        apPts[j] = apPts.last();
                        aoEdge[j] = aoEdge.last();
                        aoFace1[j] = aoFace1.last();
                        aoFace2[j] = aoFace2.last();
                        apPts.sub(1);
                        aoEdge.sub(1);
                        aoFace1.sub(1);
                        aoFace2.sub(1);
                        bIsFound = TRUE;
                        break;
                    }
                }
                if ( bIsFound == FALSE ) {
                    cerr << "Warning: Boundary\n";
                }

            } while ( f2 != f1 );
            
            R2Polygon oPoly( aptAdd.num() );
            for (int i = 0; i < aptAdd.num(); i++) {
                oPoly[i] = R2Pt( aptAdd[i][0], aptAdd[i][1] );
            }
            aoContours.last().Add( oPoly );
        }
        
        dZ += in_dWidth;
    }
    
    return aoContours;
}

/**************
 * Constructor/destructors
 *   GMesh doesn't have anything special but BMesh does
 **************/

PMeshLite::PMeshLite() : 
m_colDefault( 0.8, 0.2, 0.5 )
{
}

void PMeshLite::VertexEdgeFace( Vertex         in_v,
                                Array<Edge > & out_aopEs,
                                Array<Face > & out_aopFs ) const
{
    out_aopEs.clearcompletely();
    out_aopFs.clearcompletely();

    ASSERT(isNice( in_v ) );

    out_aopEs += mostClwEdge( in_v );
    if ( out_aopEs[0] == NULL ) return;

    do {
        out_aopEs += ccwEdge( in_v, out_aopEs.last() );
    } while ( out_aopEs.last() && (out_aopEs.last() != out_aopEs[0]) );

    out_aopEs.sub(1);

    ForVertexCcwFace( *this, in_v, f ) {
        out_aopFs += f;
    } EndFor;

    WINbool bFound = FALSE;

    for ( int i = 0; i < out_aopFs.num(); i++) {
        int iE1 = -1, iE2 = -1, iE = 0;
        ForFaceEdge( *this, out_aopFs[0], e ) {
            if ( e == out_aopEs[i] ) {
                iE2 = iE;
            }
            if ( e == out_aopEs.wrap(i+1) ) {
                iE1 = iE;
            }
            iE++;
        } EndFor;
        if ( iE1 != -1 && iE2 != -1 ) {
            ASSERT( iE2 - iE1 == 1 || iE2 == 0 && iE1 == iE-1 );
            bFound = TRUE;
        }
    }
    ASSERT( bFound == TRUE );
}

void PMeshLite::Transform( const R4Matrix & in_oFrame )
{
    transform(in_oFrame);
    Compute_normals();
}



