#include "StdAfx.H"
#include <vector>
#include <utils/Pm_MeshLite.H>
#include <utils/PMeshLite_Ply.H>

///
void PMeshLite::Write( const char *in_str ) const
{
    ofstream out;
    WINWriteBinary( in_str, out );

    const char *strEnd = strrchr( in_str, '.' );
    if ( !strcmp( strEnd, ".m" ) || !strcmp( strEnd, ".M") )
        Write( out );
    else if ( !strcmp( strEnd, ".iv" ) || !strcmp( strEnd, ".IV") )
        WriteInventor(out);
    else if ( !strcmp( strEnd, ".ply" ) || !strcmp( strEnd, ".Ply") || !strcmp( strEnd, ".PLY") )
        PLYWritePMeshLiteToPly( *this, in_str );
    else if ( !strcmp( strEnd, ".off" ) || !strcmp( strEnd, ".Off") || !strcmp( strEnd, ".OFF") ) {
        cerr << "Err: no .off file write yet\n";
        ASSERT(FALSE);
    } else if ( !strcmp( strEnd, ".bm" ) || !strcmp( strEnd, ".BM") || !strcmp( strEnd, ".Bm") )
        WriteBinary( out );
    else
        ASSERT(FALSE);
    out.close();
}

///
void PMeshLite::Read( const char *in_str, const WINbool in_bReverse ) 
{

    ifstream in;
    WINReadBinary( in_str, in );

    const char *strEnd = strrchr( in_str, '.' );
    if ( !strcmp( strEnd, ".m" ) || !strcmp( strEnd, ".M") )
        Read(in, in_bReverse);
    else if ( !strcmp( strEnd, ".iv" ) || !strcmp( strEnd, ".IV") )
        ReadInventor(in);
    else if ( !strcmp( strEnd, ".ply" ) || !strcmp( strEnd, ".Ply") || !strcmp( strEnd, ".PLY") )
        PLYReadPlyToPMeshLite( in_str, *this, in_bReverse );
    else if ( !strcmp( strEnd, ".off" ) || !strcmp( strEnd, ".Off") || !strcmp( strEnd, ".OFF") )
        ReadOFF( in, in_bReverse );
    else if ( !strcmp( strEnd, ".obj" ) || !strcmp( strEnd, ".Obj") || !strcmp( strEnd, ".OBJ") )
        ReadObj( in, in_bReverse );
    else if ( !strcmp( strEnd, ".bm" ) || !strcmp( strEnd, ".BM") || !strcmp( strEnd, ".Bm") )
        ReadBinary( in, in_bReverse );
    else
        ASSERT(FALSE);

    in.close();
}

void PMeshLite::Write(ofstream &out) const
{
    out << "PMeshLite\n";

    Array<Vertex> aopV;
    AllMeshVertices( aopV );

    out << aopV.num() << "\n";
    for (int i = 0; i < aopV.num(); i++) 
        aopV[i]->GetColor().Write(out);

    out << "Color ";
    m_colDefault.Write(out);

    out << "\nStartMesh\n";
    write(out);
    out << "EndMesh\n";
}

void PMeshLite::WriteBinary(ofstream &out) const
{
    Array<Vertex> aopV;
    AllMeshVertices( aopV );

    const int iNV = aopV.num();
    UTILSColor col;
    out.write( ( const char * ) &iNV, sizeof(int) );
    out.write( ( const char * ) m_colDefault.Data(), 3 * sizeof(float) );
    float fR, fG, fB;
    for ( int i = 0; i < aopV.num(); i++ ) {
        const int iVId = aopV[i]->IndexId();
        out.write( ( const char * ) &iVId, sizeof(int) );

        out.write( ( const char * ) &aopV[i]->GetLoc()[0], 3 * sizeof(double) );
        out.write( ( const char * ) &aopV[i]->GetNorm()[0], 3 * sizeof(double) );
        col = aopV[i]->GetColor();
        fR = (float) col[0];
        fG = (float) col[1];
        fB = (float) col[2];
        out.write( ( const char * ) &fR, sizeof(float) );
        out.write( ( const char * ) &fG, sizeof(float) );
        out.write( ( const char * ) &fB, sizeof(float) );
    }

    Array<Face> aopF;
    AllMeshFaces( aopF );

    const int iNF = aopF.num();
    out.write( ( const char * ) &iNF, sizeof(int) );

    for ( int iF = 0; iF < aopF.num(); iF++ ) {

        out.write( ( const char * ) &aopF[iF]->GetNorm()[0], 3 * sizeof(double) );
        const Array<Vertex> avF = FaceVertices( aopF[iF] );
        const int iFN = avF.num();
        out.write( ( const char * ) &iFN, sizeof(int) );
        for ( int iFV = 0; iFV < avF.num(); iFV++ ) {
            const int iVId = avF[iFV]->IndexId();
            out.write( ( const char * ) & iVId, sizeof(int) );
        }
    }
}

void PMeshLite::ReadBinary(ifstream &in, const WINbool in_bReverse )
{
    Clear();

    Array<Vertex> aopV;
    Array<int> aiId;

    int iNV = 0;
    in.read( ( char * ) &iNV, sizeof(int) );
    in.read( ( char * ) &m_colDefault[0], 3 * sizeof(float) );
    ASSERT( iNV >= 0 );

    R3Pt pt;
    R3Vec vec;
    UTILSColor col;
    aopV.need( iNV );
    aiId.need( iNV );
    aiId.memfill(0);
    aopV.memfill(0);
    float fR, fG, fB;
    for ( int i = 0; i < aopV.num(); i++ ) {
        in.read( ( char * ) &aiId[i], sizeof(int) );

        in.read( ( char * ) &pt[0], 3 * sizeof(double) );
        in.read( ( char * ) &vec[0], 3 * sizeof(double) );
        in.read( ( char * ) &fR, sizeof(float) );
        in.read( ( char * ) &fG, sizeof(float) );
        in.read( ( char * ) &fB, sizeof(float) );
        aopV[i] = AddVertex( pt, vec, UTILSColor( fR, fG, fB ) );
    }
    ASSERT( in.good( ) );

    Array<Face> aopF;

    int iNF = 0;
    in.read( ( char * ) &iNF, sizeof(int) );
    ASSERT( iNF >= 0 );

    aopF.need( iNF );
    Array<Vertex> avF;
    for ( int iF = 0; iF < aopF.num(); iF++ ) {

        in.read( ( char * ) &vec[0], 3 * sizeof(double) );
        int iFN = 0;
        in.read( ( char * ) &iFN, sizeof(int) );
        avF.need( iFN );
        for ( int iFV = 0; iFV < avF.num(); iFV++ ) {
            int iVId = 0;
            in.read( ( char * ) & iVId, sizeof(int) );
            if ( iVId < aiId.num() && iVId == aiId[ iVId ] ) {
                avF[iFV] = aopV[ iVId ];
            } else {
                for ( int i = 0; i < aiId.num(); i++ ) {
                    if ( aiId[i] == iVId ) {
                        avF[iFV] = aopV[i];
                        break;
                    }
                }
            }
        }
        Face f = AddFace( avF );
        f->SetNorm(vec);
    }
    ASSERT( in.good( ) );
}

void PMeshLite::Read(ifstream &in, const WINbool in_bReverse )
{
    Clear();

    char str[255];
    in >> str;
    if ( strcmp(str, "PMeshLite") ) {
        in.seekg(0l);
        read(in );
        renumber();
        Array<Vertex> aopV;
        AllMeshVertices( aopV );
        for ( int i = 0; i < aopV.num(); i++ )
            aopV[i]->SetColor( m_colDefault );
    } else {
        int iN;
        in >> iN;

        Array<UTILSColor> acol(iN);

        for (int i = 0; i < iN; i++) {
            acol[i].Read(in);
        }
        ASSERT(in.good());
        ASSERT( !in.eof());

        in >> str;
        if ( !strncmp( str, "Color", 5 ) ) {
            ASSERT( in.good() );
            m_colDefault.Read(in);
            ASSERT( in.good() );
            in >> str;
            ASSERT( in.good() );
        }
        in.ignore(10, '\n');

        ASSERT( !strncmp(str, "StartMesh", 9) );
        ASSERT( in.good() );
        read(in);

        Array<Vertex> aopV;
        AllMeshVertices( aopV );
        for (FORINT i = 0; i < acol.num(); i++) {
            aopV[i]->SetColor( acol[i] );
        }
        for (FORINT i = acol.num(); i < aopV.num(); i++) {
            aopV[i]->SetColor( m_colDefault );
        }
    }

    if ( in_bReverse ) {
        cerr << "Flipping\n";
        Flip();
    }

    Compute_normals();
    m_bHasChanged = TRUE;

}

void PMeshLite::ReadOFF( ifstream &in, const WINbool in_bReverse )
{
    char str[1024];
    in >> str;
    ASSERT( !strcmp(str, "OFF") );
    int iNV = 0, iNF = 0, iN = 3;

    Clear();

    in >> iNV >> iNF >> iN;
    in.getline( str, 1024, '\n' );
    while ( in.peek() == '#' || in.peek() == '/n' ) {
        in.getline( str, 1024, '\n' );
    } 
    ASSERT( in.good() );

    R3Pt pt;
    Array<Vertex> av(iNV);
    for ( int i = 0; i < iNV; i++ ) {
        in >> pt[0] >> pt[1] >> pt[2];
        av[i] = AddVertex( pt );
    }
    ASSERT( in.good() );

    Array< Array<Vertex> > aavF(iNF);
    Array<Vertex> avF(3);
    aavF.need(0);
    for ( FORINT i = 0; i < iNF; i++ ) {
        in >> iN;
        // For Tamal Dey's files
        if ( iN == 1 ) {
            in.getline( str, 1024, '\n' );
            continue;
        }
        avF.need(iN);
        for ( int j = 0; j < avF.num(); j++ ) {
            in >> iN;
            avF[j] = av[ iN ];
        }
        aavF += avF;

        in.getline( str, 1024, '\n' );
    }
    ASSERT( in.good() );

    AddFacesFixOrientation( aavF );
    Array<Vertex> avDangling;
    ForMeshVertex( Mesh(), v ) {
        if ( Degree(v) == 0 )
            avDangling += v;
    } EndFor;

    for ( FORINT i = 0; i < avDangling.num(); i++ )
        RemoveVertex( avDangling[i] );

    if ( avDangling.num() )
        Renumber();

    if ( in_bReverse ) {
        cerr << "Flipping\n";
        Flip();
    }

    TRACE("Added %d vertices, %d of %d faces\n", iNV, NumFaces(), iNF );

    Compute_normals();
    m_bHasChanged = TRUE;
}

void PMeshLite::WriteInventor( const char *in_strFName,
                               const char  * in_strHeader ) const
{
    ofstream out;
    WINWriteBinary( in_strFName, out );
    WriteInventor( out, in_strHeader );
    out.close();
}

void PMeshLite::WriteInventor( ofstream    & out,
                               const char  * in_strHeader ) const
{
    char str[512];
    ifstream in;

    if ( in_strHeader ) {
        in.open( in_strHeader );
        if ( !in.good() ) {
            cerr << "PMMesh::Input file name bad " << in_strHeader << "\n";
            ASSERT(FALSE);
            return;
        }
        in.get(str, 512);
        in.ignore( 1, '\n' );
        while (  strstr( str, "Coord" ) == NULL ) {
            out << str << "\n";

            in.get(str, 512);
            in.ignore( 1, '\n' );
        }
        out << str << "\n";
    } else {
        out << "#Inventor V1.0 ascii\n\n";

        out << "Separator {\n";
        out << "    Coordinate3 { point [\n";
    }
    
    Array<Vertex> aopVs;
    AllMeshVertices( aopVs );
    
    for ( int i = 0; i < aopVs.num(); i++ ) {
        out << "		"  << point(aopVs[i])[0];
        out << "	" << point(aopVs[i])[1];
        out << "	" << point(aopVs[i])[2] << ",\n";
    }
    out << "		]\n";
    out << "	}\n";

    /*
    out << "	Normal { vector [\n";
    for (FORINT i = 0; i < aopVs.num(); i++ ) {
        out << "		"  << aopVs[i]->GetNorm()[0];
        out << "	" << aopVs[i]->GetNorm()[1];
        out << "	" << aopVs[i]->GetNorm()[2] << ",\n";
    }
    out << "		]\n";
    out << "	}\n";
    out << "	NormalBinding {\n";
    out << "	value PER_VERTEX_INDEXED\n";
    out << "}\n";


    out << "	Material { diffuseColor [\n";
    for (FORINT i = 0; i < aopVs.num(); i++ ) {
        out << "		"  << aopVs[i]->GetColor()[0];
        out << "	" << aopVs[i]->GetColor()[1];
        out << "	" << aopVs[i]->GetColor()[2] << ",\n";
    }
    out << "		]\n";
    out << "	}\n";

    out << "MaterialBinding {\n";
    out << "	value PER_VERTEX_INDEXED\n";
    out << "}\n";
*/ 
    out << "	IndexedFaceSet {\n";
    out << "coordIndex  [\n";
    Array<Face> aopFs;
    AllMeshFaces( aopFs );
    
    for (FORINT i = 0; i < aopFs.num(); i++ ) {
        aopVs = FaceVertices( aopFs[i] );
        
        out << "		";
        for (int j = 0; j < aopVs.num(); j++)
            out  << vertexid(aopVs[j]) - 1 << ", ";
        
        out << " 	-1, \n";
    }
    out << " 	]\n";
    out << "}\n";
    out << "}\n";
    out << "# End of file.\n";
    
}

void PMeshLite::ReadInventor( ifstream &in )
{
    Clear();

    typedef enum {
        TOP_LEVEL = 0,
        BRACKET,
        SQUARE_BRACKET,
        SEPERATOR,
        VERTEX_LOC,
        VERTEX_NORMAL,
        FACES,
        MATERIAL,
        UNKNOWN_LEVEL
    } TypeLevel;

    typedef enum {
        POINT,
        VECTOR,
        FACE_LIST,
        UNKNOWN_DATA
    } TypeData;

    char       str[255], name[255];

    R3Pt pt;
    R3Vec vec;
    Array<int> aiList;
    int iPtIndex = 0, iVecIndex = 0;
    int iBracket = 0;
    int iSquareBracket = 0;

    Array<R3Pt> apt;
    Array<R3Vec> avec;
    Array<UTILSColor> acol;
    Array<Vertex> av, avF(3);

    TypeLevel tData = UNKNOWN_LEVEL;
    TypeData tDataType = UNKNOWN_DATA;
    ASSERT(in.good());
    int iIndex = -1;
    while ( !in.eof() ) {
        in >> name;

        if ( name[0] == '#' ) {
            in.getline( str, 255, '\n' );
        } else if ( name[0] == '{' ) {
            iBracket++;
        } else if ( name[0] == '[' ) {
            iSquareBracket++;
        } else if ( name[0] == '}' ) {
            iBracket--;
            ASSERT( iBracket >= 0 );

            tData = UNKNOWN_LEVEL;
            tDataType = UNKNOWN_DATA;
        } else if ( name[0] == ']' ) {
            iSquareBracket--;
            ASSERT( iSquareBracket >= 0 );

            tData = UNKNOWN_LEVEL;
            tDataType = UNKNOWN_DATA;
        } else if ( !strcmp( name, "Separator") ) {
            tData =  SEPERATOR;

        } else if ( !strcmp( name, "Normal") ) {
            tData = VERTEX_NORMAL;

        } else if ( !strcmp( name, "Coordinate3") ) {
            tData = VERTEX_LOC;

        } else if ( !strcmp( name, "IndexedFaceSet") ) {
            tData = FACES;

        } else if ( !strcmp( name, "point") ) {
            tDataType = POINT;

        } else if ( !strcmp( name, "vector") ) {
            tDataType = VECTOR;

        } else if ( !strcmp( name, "coordIndex") ) {
            tDataType = FACE_LIST;

        } else if ( strcspn( name, "0123456789-" ) == 0 ) { 
            switch (tDataType) {
            case POINT :
                if ( name[ strlen(name) - 1 ] == ',' )
                    name[ strlen(name) - 1 ] = ' ';
                pt[ iPtIndex ] = atof( name );
                iPtIndex = (iPtIndex+1) %3;

                if ( tData == VERTEX_LOC && iPtIndex == 0 )
                    apt += pt;
                break;

            case VECTOR :
                if ( name[ strlen(name) - 1 ] == ',' )
                    name[ strlen(name) - 1 ] = ' ';
                vec[ iVecIndex ] = atof( name );
                iVecIndex = (iVecIndex+1) %3;

                if ( tData == VERTEX_NORMAL && iVecIndex == 0 )
                    avec += vec;
                break;

            case FACE_LIST :
                if ( NumVertices() == 0 ) {
                    av.need( apt.num() );
                    for ( int i = 0; i < apt.num(); i++ ) {
                        const UTILSColor col = (acol.num() == apt.num()) ? acol[i] : m_colDefault;
                        if ( avec.num() == apt.num() )
                            av[i] = AddVertex( apt[i], avec[i], col );
                        else
                            av[i] = AddVertex( apt[i], col );
                    }
                }

                if ( name[ strlen(name) - 1 ] == ',' )
                    name[ strlen(name) - 1 ] = ' ';
                if ( atoi( name ) == -1 ) {
                    avF.need( aiList.num() );
                    for ( int i = 0; i < avF.num(); i++ )
                        avF[i] = av[ aiList[i] ];

                    if ( IsOkToAdd( avF ) ) {
                        AddFace( avF );
                    } else {
                        TRACE("Bad Face ");
                        for ( int i = 0; i < aiList.num(); i++ )
                            TRACE("%d ", aiList[i]);
                        TRACE("\n");
                    }
                    aiList.need(0);
                } else {
                    aiList += atoi( name );
                }
                break;
            }
        } else {
            TRACE("Unknown token %s\n", name);
        }
    }

    ASSERT( iBracket == 0 );
    ASSERT( iSquareBracket == 0 );
    if ( avec.num() == 0 )
        Compute_normals();
    else
       Compute_face_normals();
    cerr << "Added " << NumVertices() << " vertices and " << NumFaces() << " faces \n";
}

static void StripWhiteSpace( char *&io_str )
{
    int iCount = strlen( io_str );
    while ( io_str[0] == ' ' || io_str[0] == '\t' || io_str[0] == '\r' ) {
        io_str = &io_str[1];

        iCount = iCount - 1;
        if ( iCount == 0 )
            break;
    }
}

static void StripWhiteSpaceEnd( char *&io_str )
{
    for ( int i = strlen( io_str ) - 1; i >= 0; i++ ) {
        if ( io_str[i] == ' ' || io_str[i] == '\t' || io_str[i] == '\r' ) {
            io_str[i] = '\0';
        } else {
            return;
        }
    }
}

static char s_strWS[3];

static R3Pt ReadPt( char *in_str ) 
{
    R3Pt pt(0,0,0);
    char *strPt = in_str;

    for (int i = 0; i < 3; i++ ) {
        StripWhiteSpace( strPt );
        pt[i] = atof( strPt );
        strPt = strpbrk( strPt, s_strWS );
    }
    return pt;
}

static R3Vec ReadNorm( char *in_str ) 
{
    R3Vec vec(0,0,0);
    char *strPt = in_str;

    for (int i = 0; i < 3; i++ ) {
        StripWhiteSpace( strPt );
        vec[i] = atof( strPt );
        strPt = strpbrk( strPt, s_strWS );
    }
    return vec;
}


void PMeshLite::ReadObj( ifstream &in, const WINbool in_bReverse )
{
    s_strWS[0] = ' ';
    s_strWS[1] = '\t';
    s_strWS[2] = '\r';

    char strLine[4096];
    char *strLineStrip;
    WINbool bVerbose = TRUE, bTraceTex = FALSE, bTraceMtl = FALSE, bTraceUseMtl = FALSE, bTraceG = FALSE, bTraceO = FALSE;

    Array<R3Pt> aptVs;
    Array<R3Vec> avecNorm;
    Array<Vertex> avAdd;

    int iOffset = 0;
    while ( !in.eof() ) {
        in.getline( strLine, sizeof( strLine ) );

        if ( strlen( strLine ) > 4094 )
            ASSERT(FALSE);

        if ( strlen( strLine ) == 0 )
            continue;

        strLineStrip = strLine;
        StripWhiteSpace( strLineStrip );

        if ( strLineStrip[0] == '#' ) {
            //Comment
            continue;
        } else if ( strstr( strLineStrip, "mtllib" ) ) {
            if ( bVerbose && bTraceMtl == FALSE ) {
                bTraceMtl = TRUE;
                TRACE("Skipping Material library\n");
            }

        } else if ( strstr( strLineStrip, "usemtl" ) ) {
            if ( bVerbose && bTraceUseMtl == FALSE ) {
                bTraceUseMtl = TRUE;
                TRACE("Skipping use Material\n");
            }

        } else if ( strLineStrip[0] == 'g' || strLineStrip[0] == 'G'  ) {
            if ( bVerbose && bTraceG == FALSE ) {
                TRACE("Skipping g param\n");
                bTraceG = TRUE;
            }

        } else if (  strLineStrip[0] == 'o' || strLineStrip[0] == 'O' ) {
            if ( bVerbose && bTraceO == FALSE ) {
                TRACE("Skipping o param\n");
                bTraceO = TRUE;
            }

        } else if ( strstr( strLineStrip, "vt" ) || strstr( strLineStrip, "VT" ) ) {
            if ( bVerbose && bTraceTex == FALSE ) {
                TRACE("Skipping Texture coords\n");
                bTraceTex = TRUE;
            }

        } else if ( strstr( strLineStrip, "vn" ) || strstr( strLineStrip, "VN" ) ) {
            avecNorm += UnitSafe( ReadNorm( &strLineStrip[2] ) );

        } else if ( strLineStrip[0] == 'v' || strLineStrip[0] == 'V' ) {
            aptVs += ReadPt( &strLineStrip[2] );
            if ( avAdd.num() ) {
                aptVs.need(0);
                avecNorm.need(0);
                avAdd.need(0);
            }
        } else if ( strLineStrip[0] == 'f' || strLineStrip[0] == 'F' ) {

            if ( aptVs.num() != avAdd.num() ) {
                if ( bVerbose == TRUE )
                    TRACE("Vs %d\n", aptVs.num() );

                avAdd.need( aptVs.num() );
                for ( int i = 0; i < aptVs.num(); i++ ) {
                    avAdd[i] = AddVertex( aptVs[i] );
                }
            }

            Array<Vertex> avF;

            char *strPt = &strLineStrip[1];

            StripWhiteSpace( strPt );
            while ( strPt && strlen(strPt) > 0 ) {
                int iVIndx = -1, iTIndx = -1, iNIndx = -1;

                char *strSave = strPt;
                strPt = strpbrk( strPt, s_strWS );
                if ( strPt != NULL )
                    StripWhiteSpace( strPt );

                if (strstr( strSave, "//" ) ) {
                    char *strNum = strtok( strSave, "/");
                    iVIndx = atoi( strNum );
                    char *strNext = strtok( NULL, s_strWS );
                    strNext = &strNext[1];
                    if ( strNext ) {
                        iNIndx = atoi( strNext );

                    }
                } else if ( strstr( strSave, "/" ) ) {
                    char *strNext = strpbrk( strSave, "/" );
                    strNext[0] = ' ';
                    strNext = &strNext[1];

                    iVIndx = atoi( strSave );
                    char *strNextNext = strpbrk( strNext, "/ " );
                    if ( strNextNext != NULL && strNextNext[0] == '/' ) {
                        strNextNext[0] = ' ';
                        strNextNext = &strNextNext[1];
                        iTIndx = atoi( strNext );
                        iNIndx = atoi( strNextNext );
                    } else {
                        iTIndx = atoi( strNext );
                    }
                } else {
                    iVIndx = atoi( strSave );
                }

                avF += VertexFromId( iVIndx - iOffset );

                if ( iNIndx != -1 ) {
                    avF.last()->SetNorm( avecNorm[ iNIndx-1 ] );
                }
            }

            Face f = NULL;
            if ( IsOkToAdd( avF ) ) {
                f = AddFace(avF);
            } else {
				if ( bVerbose==TRUE )
					TRACE("Bad face\n");
            }
        } else if ( strLineStrip[0] == 's' || strLineStrip[0] == 'S' ) {
			if (bVerbose==TRUE)
				TRACE("Ignoring smoothness parameter\n");
        } else {
			if (bVerbose==TRUE)
				TRACE("Unknown line %s\n", strLine);
        }
    }
    if ( in_bReverse ) {
        cerr << "Flipping\n";
        Flip();
    }


}



