#include "StdAfx.H"

#include <utils/Pm_MeshLite.H>
#include <boost/dynamic_bitset/dynamic_bitset.hpp>
#include <utils/Utils_Dots.H>

inline void CalcMaskFunc( Array<double> &out_ad ) 
{
    for ( int i = 3; i < out_ad.num(); i++ )
        out_ad[i] = (1.0 / (double) i) * ( 5.0 / 8.0 - pow(3.0 / 8.0 - 0.25 * cos( 2.0 * M_PI / (double) i ), 2) );
}

// Note: these are symmetric around the center vertex; hence, only store one number
const int ciMaxExact = 10;
static double s_aadLoopExactPos[10][2] =
{ 
    {1,0},
    {1,0},
    {1,0},
    {1,0},
    {1,0},
    {1,0},
    {1,0},
    {1,0},
};
static double s_aadLoopExactDs[10][2] =
{ 
    {1,0},
    {1,0},
    {1,0},
    {1,0},
    {1,0},
    {1,0},
    {1,0},
    {1,0},
};
static double s_aadLoopExactDt[10][2] =
{ 
    {1,0},
    {1,0},
    {1,0},
    {1,0},
    {1,0},
    {1,0},
    {1,0},
    {1,0},
};
void PMeshLite::SubdivideLoop( const int in_iNTimes,
						              PMeshLite & out_mesh,
                               const WINbool in_bExactPts,
                               const WINbool in_bExactNorms ) const
{

	ASSERT( in_iNTimes > 0 );

    PMeshLite meshTemp;
	const PMeshLite* meshFrom = this;
    PMeshLite *meshTo = &meshTemp;

    Array<double> adAlpha(15);
    CalcMaskFunc( adAlpha );

    Array<Vertex> avNew, avF, avFNew(3), avAll;
    Array<Edge> aeF, aeAll;
    Array<Face> afNew;

    R3Pt ptNew(0,0,0);
	for(int k = 0 ; k < in_iNTimes; ++k) {
        // Final one should be output mesh
        if(k == in_iNTimes-1) {
			meshTo = &out_mesh;
		}

        meshTo->Clear();

        // Pre-alloc data
        avNew.need( meshFrom->NumVertices() + meshFrom->NumEdges() );
        afNew.need( 4 * meshFrom->NumFaces() );
        const int iVOffset = meshFrom->NumVertices();

        meshFrom->AllMeshVertices( avAll );
        for ( int iV = 0; iV < avAll.num(); iV++ ) {
            const int iDegree = meshFrom->Mesh().degree( avAll[iV] );

            if ( iDegree >= adAlpha.num() ) {
                adAlpha.need( iDegree + 5 );
                CalcMaskFunc( adAlpha );
            }

            const double &dAlpha = adAlpha[ iDegree ];
            const double dMe = 1.0 - iDegree * dAlpha;
            const R3Pt &ptMe = avAll[iV]->GetLoc();

            ptNew = R3Pt( dMe * ptMe[0], dMe * ptMe[1], dMe * ptMe[2] );

            ForVertexVertex( meshFrom->Mesh(), avAll[iV], vv ) {
                const R3Pt &ptRing = vv->GetLoc();
                ptNew[0] += dAlpha * ptRing[0];
                ptNew[1] += dAlpha * ptRing[1];
                ptNew[2] += dAlpha * ptRing[2];
            }EndFor;

            avNew[ avAll[iV]->IndexId() ] = meshTo->AddVertex( ptNew );
            ASSERT( avAll[iV]->IndexId() == iV );
        }

        meshFrom->AllMeshEdges(aeAll);
        for ( int iE = 0; iE < aeAll.num(); iE++ ) {
            const R3Pt &ptV1 = meshFrom->EdgeVertex1(aeAll[iE])->GetLoc();
            const R3Pt &ptV2 = meshFrom->EdgeVertex2(aeAll[iE])->GetLoc();
            const R3Pt &ptOV1 = meshFrom->SideVertex1(aeAll[iE])->GetLoc();
            const R3Pt &ptOV2 = meshFrom->SideVertex2(aeAll[iE])->GetLoc();

            ptNew = R3Pt( 3.0 / 8.0 * ( ptV1[0] + ptV2[0] ) + 1.0 / 8.0 * ( ptOV1[0] + ptOV2[0] ),
                          3.0 / 8.0 * ( ptV1[1] + ptV2[1] ) + 1.0 / 8.0 * ( ptOV1[1] + ptOV2[1] ),
                          3.0 / 8.0 * ( ptV1[2] + ptV2[2] ) + 1.0 / 8.0 * ( ptOV1[2] + ptOV2[2] ) );
            avNew[ iVOffset + iE ] = meshTo->AddVertex( ptNew );
            ASSERT( iE == aeAll[iE]->IndexId() );
        }


        ForMeshFace( meshFrom->Mesh(), f ) {
			meshFrom->FaceVertexEdge(f, avF, aeF );

			// add new triangles
			for(int j = 0; j < avF.num(); ++j){

				avFNew[0] = avNew[iVOffset + aeF[j]->IndexId()];
				avFNew[1] = avNew[ avF.wrap(j+1)->IndexId() ];
				avFNew[2] = avNew[iVOffset + aeF.wrap(j+1)->IndexId()];

				ASSERT( meshTo->IsOkToAdd( avFNew ) );
				meshTo->AddFace( avFNew );				
            }

			avFNew[0] = avNew[iVOffset + aeF[0]->IndexId()];
			avFNew[1] = avNew[iVOffset + aeF[1]->IndexId()];
			avFNew[2] = avNew[iVOffset + aeF[2]->IndexId()];

            // Add middle triangle
			ASSERT( meshTo->IsOkToAdd( avFNew ) );
            meshTo->AddFace( avFNew );
		}EndFor;

        // Swap input/output mesh
		meshFrom = meshTo;
	}

    if ( in_bExactPts == TRUE || in_bExactNorms == TRUE ) {
        Array<R3Pt> apt( NumVertices() );
        ForMeshVertex( Mesh(), v ) {
            apt[v->IndexId()] = v->GetLoc();
        }EndFor;

        ForMeshVertex( out_mesh.Mesh(), v ) {
            const int iDeg = out_mesh.degree(v);
            if ( iDeg >= 3 + ciMaxExact || iDeg < 3 ) {
                TRACE("Warning: Loop vertex %d,%d greater than %d degree, not doing exact\n", v->IndexId(), iDeg, 3 + ciMaxExact );

                continue;
            }

            const double *adPos =  s_aadLoopExactPos[iDeg - 3];
            const double *adDs =  s_aadLoopExactDs[iDeg - 3];
            const double *adDt =  s_aadLoopExactDt[iDeg - 3];
            const R3Pt &ptMe = apt[v->IndexId()];
            ptNew = R3Pt( adPos[0] * ptMe[0], adPos[0] * ptMe[1], adPos[0] * ptMe[2]);
            R3Vec vecDs = R3Vec( adDs[0] * ptMe[0], adDs[0]  * ptMe[1], adDs[0]  * ptMe[2]);
            R3Vec vecDt = R3Vec( adDt[0] * ptMe[0], adDt[0]  * ptMe[1], adDt[0]  * ptMe[2]);

            ForVertexVertex( out_mesh.Mesh(), v, vv ) {
                const R3Pt &pt = apt[ vv->IndexId() ];

                for ( int j = 0; j < 3; j++ ) {
                    ptNew[j] += adPos[1] * pt[j];
                    vecDs[j] += adDs[1] * pt[j];
                    vecDt[j] += adDt[1] * pt[j];
                }
            }EndFor;

            if ( in_bExactPts == TRUE )
                v->SetLoc( ptNew );
            if ( in_bExactNorms == TRUE )
                v->SetNorm( UnitSafe( Cross( vecDs, vecDt ) ) );
        }EndFor;
    }

    if ( in_bExactNorms == FALSE )
	    out_mesh.Compute_normals();
    else
        out_mesh.Compute_face_normals();
}

void PMeshLite::SubdivideCC( const int in_iNTimes,
						     PMeshLite & out_mesh ) const
{

	ASSERT( in_iNTimes > 0 );

    PMeshLite meshTemp;
	const PMeshLite* meshFrom = this;
    PMeshLite *meshTo = &meshTemp;

    Array<Vertex> avNew, avF, avFNew(4), avAll, avV;
    Array<Edge> aeF, aeV, aeAll;
    Array<Face> afV, afAll;

    R3Pt ptNew(0,0,0);
    const double dAlpha = 0.3, dBeta = 0.3;
    Array<double> adBlend(3);
	for(int k = 0 ; k < in_iNTimes; ++k) {
        // Final one should be output mesh
        if(k == in_iNTimes-1) {
			meshTo = &out_mesh;
		}

        meshTo->Clear();

        // Pre-alloc data
        avNew.need( meshFrom->NumVertices() + meshFrom->NumEdges() + meshFrom->NumFaces() );
        const int iVOffset = meshFrom->NumVertices();
        const int iEOffset = meshFrom->NumVertices() + meshFrom->NumEdges();

        meshFrom->AllMeshVertices( avAll );
        meshFrom->AllMeshEdges( aeAll );
        meshFrom->AllMeshFaces( afAll );

        for ( int iV = 0; iV < avAll.num(); iV++ ) {
            avNew[ avAll[iV]->IndexId() ] = meshTo->AddVertex( avAll[iV]->GetLoc() );
            ASSERT( avAll[iV]->IndexId() == iV );
        }

        for ( int iE = 0; iE < aeAll.num(); iE++ ) {
            avNew[ iVOffset + iE ] = meshTo->AddVertex( meshFrom->GetPointOnEdge( aeAll[iE], 0.5 ) );
            ASSERT( iE == aeAll[iE]->IndexId() );
        }

        for ( int iF = 0; iF < afAll.num(); iF++ ) {
            avNew[ iEOffset + iF ] = meshTo->AddVertex( meshFrom->GetFaceCentroid( afAll[iF] ) );
            ASSERT( iF == afAll[iF]->IndexId() ); 
        }

        /* Make new faces */
        for ( int i = 0; i < afAll.num(); i++ ) {
            meshFrom->FaceVertexEdge( afAll[i], avF, aeF );
            for ( int j = 0; j < avF.num(); j++ ) {
                avFNew[0] = avNew[ avF[j]->IndexId() ];
                avFNew[1] = avNew[ iVOffset + aeF[j]->IndexId() ];
                avFNew[2] = avNew[ iEOffset + afAll[i]->IndexId() ];
                avFNew[3] = avNew[ iVOffset + aeF.wrap(j-1)->IndexId() ];

                meshTo->AddFace( avFNew );
            }
        }

        for ( FORINT i = 0; i < avAll.num(); i++ ) {
            if ( meshFrom->IsBoundary( avAll[i] ) )
                continue;

            avV = meshFrom->VertexVertices( avAll[i] );
            aeV = meshFrom->VertexEdges( avAll[i] );
            afV = meshFrom->VertexFaces( avAll[i] );

            if ( aeV.num() == 0 )
                continue;

            R3Pt ptE(0.0, 0.0, 0.0), ptF(0.0, 0.0, 0.0);
            for ( int iE = 0; iE < aeV.num(); iE++ ) {
                for ( int j = 0; j < 3; j++ )
                    ptE[j] += avNew[ iVOffset + aeV[iE]->IndexId() ]->GetLoc()[j];
            }

            for ( int iF = 0; iF < afV.num(); iF++ ) {
                for ( int j = 0; j < 3; j++ )
                    ptF[j] += avNew[ iEOffset + afV[iF]->IndexId() ]->GetLoc()[j];

            }

            R3Pt ptV = avAll[i]->GetLoc();
            for ( int j = 0; j < 3; j++ ) {
                ptE[j] /= (double) aeV.num();
                ptF[j] /= (double) afV.num();

                ptV[j] = (1.0 - dAlpha - dBeta) * ptV[j] + dAlpha * ptE[j] + dBeta * ptF[j];
            }
            avNew[ avAll[i]->IndexId() ]->SetLoc( ptV ); 
        }

        for ( FORINT i = 0; i < aeAll.num(); i++ ) {
            const Face f1 = meshFrom->EdgeFace1( aeAll[i] );
            const Face f2 = meshFrom->EdgeFace2( aeAll[i] );
            if ( f1 == NULL || f2 == NULL )
                continue;

            const R3Pt ptFace = Lerp( avNew[ iEOffset + f1->IndexId() ]->GetLoc(),
                                      avNew[ iEOffset + f2->IndexId() ]->GetLoc(), 0.5 );

            avNew[ iVOffset + i ]->SetLoc( Lerp( avNew[ iVOffset + i ]->GetLoc(), ptFace, 0.5 ) );            
        }
        // Swap input/output mesh
		meshFrom = meshTo;
	}
    out_mesh.Compute_normals();
}

void PMeshLite::AdjustLocationsCC( PMeshLite &out_mesh, 
                                   const Array<Vertex> & in_avMe, 
                                   Array<Vertex>       & out_avNextLevel ) const
{
    Array<Vertex> aopVNew;

    Array<Edge> aeMove;
    Array<Face> afMove;
    Array<Vertex> avMove;

    Grow( in_avMe, aeMove, afMove );
    Grow( afMove, avMove, aeMove );
    out_mesh.AllMeshVertices( aopVNew );

    out_avNextLevel.need(0);

    const int iVOffset = 0;
    const int iEOffset = NumVertices();
    const int iFOffset = iEOffset + NumEdges();

    for ( int i = 0; i < afMove.num(); i++ ) {
        aopVNew[ iFOffset + afMove[i]->IndexId() ]->SetLoc( GetFaceCentroid( afMove[i] ) );
        out_avNextLevel += aopVNew[ iFOffset + afMove[i]->IndexId() ];
    }

    const double dAlpha = 0.3, dBeta = 0.3;
    for ( FORINT i = 0; i < avMove.num(); i++ ) {
        R3Pt ptVAvg(0,0,0);

        if ( IsBoundary( avMove[i] ) ) {
            ptVAvg = avMove[i]->GetLoc();
        } else {
            R3Pt ptEAvg(0,0,0), ptFAvg(0,0,0);
            Array<Edge> avE = VertexEdges( avMove[i] );
            Array<Face> avF = VertexFaces( avMove[i] );
            const double dDiv = 1.0 / (double) avE.num();

            for ( int j = 0; j < avE.num(); j++ ) {
                const R3Pt  ptE = Lerp( EdgeVertex1( avE[j] )->GetLoc(),EdgeVertex2( avE[j] )->GetLoc(), 0.5 );
                const R3Pt &ptF = aopVNew[ iFOffset + avF[j]->IndexId() ]->GetLoc();
                for ( int k = 0; k < 3; k++ ) {
                    ptEAvg[k] += ptE[k] * dDiv;
                    ptFAvg[k] += ptF[k] * dDiv;
                }
            }

            for ( int k = 0; k < 3; k++ ) {
                ptVAvg[k] = (1.0 - dAlpha - dBeta ) * avMove[i]->GetLoc()[k] + dAlpha * ptEAvg[k] + dBeta * ptFAvg[k];
            }
        }
        aopVNew[ iVOffset + avMove[i]->IndexId() ]->SetLoc( ptVAvg );

        out_avNextLevel += aopVNew[ iVOffset + avMove[i]->IndexId() ];
    }

    const int iFAdd = NumFaces();
    for ( FORINT i = 0; i < aeMove.num(); i++ ) {
        const Face f1 = EdgeFace1( aeMove[i] );
        const Face f2 = EdgeFace2( aeMove[i] );

        if ( f1 && f2 ) {
            aopVNew[ iEOffset + aeMove[i]->IndexId() ]->SetLoc( Lerp( Lerp( EdgeVertex1( aeMove[i] )->GetLoc(),
                                                                            EdgeVertex2( aeMove[i] )->GetLoc(), 0.5 ),
                                                                      Lerp( aopVNew[ iFOffset + f1->IndexId() ]->GetLoc(),
                                                                            aopVNew[ iFOffset + f2->IndexId() ]->GetLoc(), 0.5 ), 0.5 ) );
        } else {
            aopVNew[ iEOffset + aeMove[i]->IndexId() ]->SetLoc( Lerp( EdgeVertex1( aeMove[i] )->GetLoc(),
                                                                      EdgeVertex2( aeMove[i] )->GetLoc(), 0.5 ) );
        }
        out_avNextLevel += aopVNew[ iEOffset + aeMove[i]->IndexId() ];
    }

}

void PMeshLite::InfluencesCC( const PMeshLite     & out_mesh, 
                              const Array<Vertex> & in_avMe, 
                                    Array<Vertex> & out_avNextLevel ) const
{
    Array<Vertex> aopVNew;

    Array<Edge> aeMove;
    Array<Face> afMove;
    Array<Vertex> avMove;

    Grow( in_avMe, aeMove, afMove );
    Grow( afMove, avMove, aeMove );
    out_mesh.AllMeshVertices( aopVNew );

    out_avNextLevel.need(0);

    const int iVOffset = 0;
    const int iEOffset = NumVertices();
    const int iFOffset = iEOffset + NumEdges();

    for ( int i = 0; i < afMove.num(); i++ ) {
        out_avNextLevel += aopVNew[ iFOffset + afMove[i]->IndexId() ];
    }

    for ( FORINT i = 0; i < aeMove.num(); i++ ) {
        out_avNextLevel += aopVNew[ iEOffset + aeMove[i]->IndexId() ];
    }

    for ( FORINT i = 0; i < avMove.num(); i++ ) {
        out_avNextLevel += aopVNew[ iVOffset + avMove[i]->IndexId() ];
    }
}

void PMeshLite::InfluenceCC( const PMeshLite       & in_meshChild, 
                             const Vertex            in_vChild, 
                                   Array<Vertex>   & out_avParent, 
                                   Array<double>   & out_adInfluence ) const
{
    const int iVOffset = 0;
    const int iEOffset = NumVertices();
    const int iFOffset = iEOffset + NumEdges();
    const double dAlpha = 0.3, dBeta = 0.3;

    if ( in_vChild->IndexId() >= iFOffset ) {
        out_avParent = FaceVertices( FaceFromIndexId( in_vChild->IndexId() - iFOffset ) );
        out_adInfluence.need( out_avParent.num() );
        const double dPerc = 1.0 / out_avParent.num();
        for ( int i = 0; i < out_adInfluence.num(); i++ )
            out_adInfluence[i] = dPerc;

    } else if ( in_vChild->IndexId() >= iEOffset ) {
        const Edge e = EdgeFromIndexId( in_vChild->IndexId() - iEOffset );

        const Face f1 = EdgeFace1( e );
        const Face f2 = EdgeFace2( e );

        out_avParent.need(2);
        out_adInfluence.need(2);
        out_avParent[0] = EdgeVertex1( e );
        out_avParent[1] = EdgeVertex2( e );
        out_adInfluence[0] = 0.5;
        out_adInfluence[1] = 0.5;

        if ( f1 && f2 ) {
            Array<Vertex> avGetF;
            Array<double> adGetF;
            out_adInfluence[0] = 0.25;
            out_adInfluence[1] = 0.25;
            InfluenceCC( in_meshChild, in_meshChild.VertexFromIndexId( iFOffset + f1->IndexId() ), avGetF, adGetF );
            for ( int i = 0; i < avGetF.num(); i++ ) {
                const int iIndx = out_avParent.index( avGetF[i] );
                if ( iIndx == -1 ) {
                    out_avParent += avGetF[i];
                    out_adInfluence += 0.25 * adGetF[i];
                } else {
                    out_adInfluence[iIndx] += 0.25 * adGetF[i];
                }
            }

            InfluenceCC( in_meshChild, in_meshChild.VertexFromIndexId( iFOffset + f2->IndexId() ), avGetF, adGetF );
            for ( FORINT i = 0; i < avGetF.num(); i++ ) {
                const int iIndx = out_avParent.index( avGetF[i] );
                if ( iIndx == -1 ) {
                    out_avParent += avGetF[i];
                    out_adInfluence += 0.25 * adGetF[i];
                } else {
                    out_adInfluence[iIndx] += 0.25 * adGetF[i];
                }
            }

        }
    } else {
        const Vertex v = VertexFromIndexId( in_vChild->IndexId() - iVOffset );

        out_avParent.need(1);
        out_adInfluence.need(1);
        out_avParent[0] = v;
        out_adInfluence[0] = (1.0 - dAlpha - dBeta);

        Array<Edge> ae = VertexEdges( v );
        Array<Face> af = VertexFaces( v );

        if ( ae.num() == af.num() && ae.num() != 0 ) {
            Array<Vertex> avGetF;
            Array<double> adGetF;
            const double dPerc = 1.0 / af.num();
            for ( int iE = 0; iE < ae.num(); iE++ ) {
                const int iIndx1 = out_avParent.index( EdgeVertex1( ae[iE] ) );
                if ( iIndx1 == -1 ) {
                    out_avParent += EdgeVertex1( ae[iE] );
                    out_adInfluence += dPerc * dAlpha * 0.5;
                } else {
                    out_adInfluence[iIndx1] += dPerc * dAlpha * 0.5;
                }
                const int iIndx2 = out_avParent.index( EdgeVertex2( ae[iE] ) );
                if ( iIndx2 == -1 ) {
                    out_avParent += EdgeVertex2( ae[iE] );
                    out_adInfluence += dPerc * dAlpha * 0.5;
                } else {
                    out_adInfluence[iIndx2] += dPerc * dAlpha * 0.5;
                }
            }
            for ( int iF = 0; iF < af.num(); iF++ ) {
                InfluenceCC( in_meshChild, in_meshChild.VertexFromIndexId( iFOffset + af[iF]->IndexId() ), avGetF, adGetF );
                for ( int i = 0; i < avGetF.num(); i++ ) {
                    const int iIndx = out_avParent.index( avGetF[i] );
                    if ( iIndx == -1 ) {
                        out_avParent += avGetF[i];
                        out_adInfluence += dPerc * dBeta * adGetF[i];
                    } else {
                        out_adInfluence[iIndx] += dPerc * dBeta * adGetF[i];
                    }
                }
            }
        } else {
            out_adInfluence[0] = 1.0;
        }
    }

    double dSum = 0.0;
    for ( int i = 0; i < out_adInfluence.num(); i++ )
        dSum += out_adInfluence[i];

    ASSERT( RNApproxEqual( dSum, 1.0, 1e-6 ) );
    ASSERT( out_adInfluence.num() == out_avParent.num() );
}

void PMeshLite::SubFacesCC( const PMeshLite   & in_meshNextLevel, 
                            const Array<Face> & in_af, 
                                  Array<Face> & out_afNextLevel ) const
{
    Array<Vertex> avCenter, aopVNew;

    in_meshNextLevel.AllMeshVertices( aopVNew );

    out_afNextLevel.need(0);

    const int iFOffset = NumVertices() + NumEdges();
    for ( int i = 0; i < in_af.num(); i++ ) {
        Vertex vCenter = aopVNew[ iFOffset + in_af[i]->IndexId() ];

        Array<Face> afAdj = in_meshNextLevel.VertexFaces( vCenter );
        out_afNextLevel.add( afAdj );
    }
}

void PMeshLite::TestSubdivCC() 
{
    PMeshLite mesh, meshSubdiv;

    mesh.Read( WINDataHome("meshes/table.m") );
    mesh.SubdivideCC( 1, meshSubdiv );
    meshSubdiv.Write( WINDataHome("test/tableOut.m") );

    Array<Vertex> avParent, avChild;
    Array<double> adInf;
    UTILSDotArray dots;
    ForMeshVertex( meshSubdiv.Mesh(), v ) {
        mesh.InfluenceCC( meshSubdiv, v, avParent, adInf );
        R3Pt ptSum(0,0,0);
        double dSum = 0.0;
        for ( int i = 0; i < avParent.num(); i++ ) {
            for ( int j = 0; j < 3; j++ )
                ptSum[j] += avParent[i]->GetLoc()[j] * adInf[i];
            dSum += adInf[i];
        }
        ASSERT( RNApproxEqual( dSum, 1.0 ) );
        if ( v->IndexId() < mesh.NumVertices() )
            dots.Add( ptSum, UTILSColor::RED );
        else if ( v->IndexId() < mesh.NumVertices() + mesh.NumEdges() )
            dots.Add( ptSum, UTILSColor::GREEN );
        else
            dots.Add( ptSum, UTILSColor::BLUE );

    }EndFor;
    dots.Write( WINDataHome("test/tableOrig.txt") );

    avParent.need(1);
    avParent[0] = mesh.VertexFromIndexId(0);
    mesh.InfluencesCC( meshSubdiv, avParent, avChild );

    for ( int i = 0; i < avChild.num(); i++ ) 
        dots.AddPair( avParent[0]->GetLoc(), avChild[i]->GetLoc(), UTILSColor::GREY );
    dots.Write( WINDataHome("test/table.txt") );
    
    avParent[0]->SetLoc( avParent[0]->GetLoc() + R3Vec(0.2, 0.1, 0.0 ) );
    mesh.AdjustLocationsCC( meshSubdiv, avParent, avChild );

    dots.RemoveAll();
    for ( FORINT i = 0; i < avChild.num(); i++ ) {
        if ( avChild[i]->IndexId() < mesh.NumVertices() )
            dots.AddPair( avParent[0]->GetLoc(), avChild[i]->GetLoc(), UTILSColor::YELLOW );
        else
            dots.AddPair( avParent[0]->GetLoc(), avChild[i]->GetLoc(), UTILSColor(1,0,1) );
    }

    meshSubdiv.Write( WINDataHome("test/tableMove.m") );
    mesh.SubdivideCC( 1, meshSubdiv );
    meshSubdiv.Write( WINDataHome("test/tableMove2.m") );
    dots.Write( WINDataHome("test/table.txt") );

    Array<Face> af(1), afNext;
    af[0] = mesh.FaceFromIndexId( 3 );
    mesh.SubFacesCC( meshSubdiv, af, afNext );
    dots.RemoveAll();
    for ( FORINT i = 0; i < afNext.num(); i++ ) 
        dots.AddPair( mesh.GetFaceCentroid( af[0] ), meshSubdiv.GetFaceCentroid( afNext[i] ), UTILSColor::BLUE );
    dots.Write( WINDataHome("test/table.txt") );
}