/* Copyright 1991, Brown Computer Graphics Group.  All Rights Reserved. */

/* -------------------------------------------------------------------------
   Manipulations of a point in euclidean space
   ------------------------------------------------------------------------- */

#include "StdAfx.H"

#include <utils/Rn_Line_seg.H>


/* -------------------------- protected Routines --------------------------- */

/* -------------------------- Private Routines ----------------------------- */

/* -------------------------- public routines  ----------------------------- */

template<class Point, class Vector>
class RNLine_segTC {
public:
    
    static WINbool FindPtOnSeg( const Point & m_pt1,
                                const Point & m_pt2,
                                const Point & in_p, 
                                Point       & out_ptClosest, 
                                double      & out_t_on_seg, 
                                double      & out_d_to_seg,
                                const WINbool       out_bRes) ;
};

#ifndef RN_EPSILON
#define RN_EPSILON 1e-15
#endif

template<class Point, class Vector>
inline
WINbool 
RNLine_segTC<Point, Vector>::FindPtOnSeg( const Point & m_pt1,
                                          const Point & m_pt2,
                                          const Point & in_p, 
                                          Point       & out_ptClosest, 
                                          double      & out_t_on_seg, 
                                          double      & out_d_to_seg,
                                          const WINbool  in_bRes) 
{

    WINbool out_bRes = in_bRes;


    double len = Length( m_pt1 - m_pt2 );
    
    if ( RNIsZero(len) ) {
        if ( in_p == m_pt1 ) out_t_on_seg = 0;
        else                 out_t_on_seg = -1; /// will be clipped to 0
    } else {
        out_t_on_seg = out_t_on_seg / len;
    }
    
    if (out_t_on_seg < RN_EPSILON) {
        out_t_on_seg = 0.0;
        out_bRes = FALSE;
    }
    
    if (out_t_on_seg > 1.0 + RN_EPSILON) {
        out_t_on_seg = 1.0;
        out_bRes = FALSE;
    }
    
    out_ptClosest = m_pt1 + (m_pt2 - m_pt1) * (out_t_on_seg);
    out_d_to_seg = Length(out_ptClosest - in_p);
    
    return out_bRes;
}

WINbool
R3Line_seg::FindPtOnSeg( const R3Pt & in_p, 
                         R3Pt       & out_ptClosest, 
                         double     & out_t_on_seg, 
                         double     & out_d_to_seg) const
{

    if ( ApproxEqual( P1(), P2() ) ) {

        out_t_on_seg = 0.5;

        out_ptClosest = P1();

        out_d_to_seg = ::Length( in_p - P1() );

        return (RNIsZero( out_d_to_seg )) ? TRUE : FALSE;

    }


    R3Line l1(P1(), P2() - P1());
    
    WINbool out_bRes = 
        l1.FindPtOnLine(in_p, out_ptClosest, out_t_on_seg, out_d_to_seg);

    return 
        RNLine_segTC<R3Pt, R3Vec>::FindPtOnSeg(P1(), P2(),
                                               in_p, out_ptClosest,
                                               out_t_on_seg,
                                               out_d_to_seg,
                                               out_bRes);
}
