#include "StdAfx.H"
#include <utils/S1_CircleAngleTC.H>

void S1CircleAngleTC<double>::Test()
{
    for ( double dTT = -4.0 * M_PI; dTT < 6.0 * M_PI; dTT += 2.0 * M_PI ) {
        S1CircleAngleTC<double> ang( dTT + 0.25 );
        ASSERT( ang.Angle() == 0.25 );
    }

    for ( double dT = -2.0 * M_PI; dT < 4.0 * M_PI; dT += 0.1 ) {
        S1CircleAngleTC<double> angPrev( dT - 0.1 );
        S1CircleAngleTC<double> angNext( dT + 0.3 );
        S1CircleAngleTC<double> angIn( dT ), angOut( dT - 0.2 );
        ASSERT( angIn.Inside( angPrev, angNext ) );
        ASSERT( angOut.Inside( angPrev, angNext ) == FALSE );
        ASSERT( angIn.Inside( angNext, angPrev) );
        ASSERT( angOut.Inside( angNext, angPrev) == FALSE );

        S1CircleAngleTC<double> ang = Lerp( angPrev, angNext, 0.25 );
        ASSERT( ApproxEqual( ang, angIn ) );

        ang = Lerp( angNext, angPrev, 0.75 );
        ASSERT( ApproxEqual( ang, angIn ) );

        const R2Vec vecTangent = angIn.Tangent( angNext );
        const R2Pt pt = angIn.Circle();
        ASSERT( RNApproxEqual( pt[0],  vecTangent[1] ) );
        ASSERT( RNApproxEqual( pt[1], -vecTangent[0] ) );

        const R2Vec vecTangentRev = angIn.Tangent( angPrev );
        ASSERT( RNApproxEqual( pt[0], -vecTangentRev[1] ) );
        ASSERT( RNApproxEqual( pt[1],  vecTangentRev[0] ) );

        // Counter clockwise is increasing angle
        ASSERT( angPrev.Clockwise( angNext ) == FALSE );
        ASSERT( angNext.Clockwise( angPrev ) == TRUE );
    }

}
