#include "StdAfx.H"



#include <WINSystemDefines.H>

#include <utils/Rn_Affine1.H>
#include <utils/Rn_Affine2.H>
#include <utils/Rn_Affine3.H>
#include <utils/Rn_Polygon.H>
#include <utils/Rn_Polynomial.H>
#include <utils/Rn_Line.H>
#include <utils/Rn_Line_seg.H>

void TestR1()
{
    ofstream out;
    ifstream in;

    R1PointTC<double>    pt1a(3.0), pt1b = 3.0;
    R1VectorTC<double>   vec1a, vec1b;
    R1CoVectorTC<double> coVec1a, coVec1b;
    R1CoVectorTC<float>  coVecf1a, coVecf1b;

    vec1a = pt1a - pt1b;
    vec1a = vec1a + vec1b;
    pt1a = pt1b + vec1a;

    VERIFY( pt1a[0] == 3.0 );
    VERIFY( pt1a == pt1b );
    VERIFY( pt1a != pt1b );
    VERIFY( pt1a < pt1b );
    VERIFY( pt1a <= pt1b );
    VERIFY( ApproxEqual( pt1a, pt1b ) );

    pt1a.Write(out);
    pt1a.Print();
    pt1a.Read(in);
    in >> pt1a[0];
    out << pt1a[0];

    VERIFY( vec1a[0] == 3.0 );
    VERIFY( vec1a == vec1b );
    VERIFY( vec1a != vec1b );
    VERIFY( ApproxEqual( vec1a, vec1b ) );

    vec1a.Write(out);
    vec1a.Print();
    vec1a.Read(in);
    in >> vec1a[0];
    out << vec1a[0];


    VERIFY( coVec1a[0] == 3.0 );
    VERIFY( coVec1a == coVec1b );
    VERIFY( coVec1a != coVec1b );
    VERIFY( ApproxEqual( coVec1a, coVec1b ) );
    VERIFY( ApproxEqual( coVecf1a, coVecf1b ) );

    coVec1a.Write(out);
    coVec1a.Print();
    coVec1a.Read(in);
    in >> coVec1a[0];
    out << coVec1a[0];

}

void TestR2()
{
    ofstream out;
    ifstream in;

    R2PointTC<double>    pt2a(3.0, 4.0), pt2b;
    R2VectorTC<double>   vec2a, vec2b;
    R2CoVectorTC<double> coVec2a, coVec2b;
    R2CoVectorTC<float>  coVecf2a, coVecf2b;
    R2MatrixTC<double>   matrix2a, matrix2b;
    R2MatrixTC<float>    matrixf2a, matrixf2b;

    VERIFY( matrix2a == matrix2b );
    VERIFY( matrix2a != matrix2b );
    VERIFY( ApproxEqual( matrix2a, matrix2b ) );
    VERIFY( ApproxEqual( matrixf2a, matrixf2b ) );

    pt2b = pt2a;

    vec2a = pt2a - pt2b;
    vec2a = vec2a + vec2b;
    pt2a = pt2b + vec2a;

    VERIFY( pt2a[0] == 3.0 );
    VERIFY( pt2a == pt2b );
    VERIFY( pt2a != pt2b );
    VERIFY( pt2a < pt2b );
    VERIFY( pt2a <= pt2b );
    VERIFY( ApproxEqual( pt2a, pt2b ) );

    pt2a.Write(out);
    pt2a.Print();
    pt2a.Read(in);
    in >> pt2a;
    out << pt2a;
    in >> vec2a;
    out << vec2a;
    in >> coVec2a;
    out << coVec2a;

    VERIFY( vec2a[0] == 3.0 );
    VERIFY( vec2a == vec2b );
    VERIFY( vec2a != vec2b );
    VERIFY( ApproxEqual( vec2a, vec2b ) );

    vec2a.Write(out);
    vec2a.Print();
    vec2a.Read(in);
    in >> vec2a[0];
    out << vec2a[0];


    VERIFY( coVec2a[0] == 3.0 );
    VERIFY( coVec2a == coVec2b );
    VERIFY( coVec2a != coVec2b );
    VERIFY( ApproxEqual( coVec2a, coVec2b ) );
    VERIFY( ApproxEqual( coVecf2a, coVecf2b ) );

    coVec2a.Write(out);
    coVec2a.Print();
    coVec2a.Read(in);
    in >> coVec2a[0];
    out << coVec2a[0];

}

void TestR3()
{
    ofstream out;
    ifstream in;

    R3PointTC<double>    pt3a(3.0, 4.0, 5.0), pt3b;
    R3VectorTC<double>   vec3a, vec3b;
    R3CoVectorTC<double> coVec3a, coVec3b;
    R3CoVectorTC<float>  coVecf3a, coVecf3b;
    R3MatrixTC<double>   matrix3a, matrix3b;
    R3MatrixTC<float>    matrixf3a, matrixf3b;

    VERIFY( matrix3a == matrix3b );
    VERIFY( matrix3a != matrix3b );
    VERIFY( ApproxEqual( matrix3a, matrix3b ) );
    VERIFY( ApproxEqual( matrixf3a, matrixf3b ) );

    pt3b = pt3a;

    vec3a = pt3a - pt3b;
    vec3a = vec3a + vec3b;
    pt3a = pt3b + vec3a;

    VERIFY( pt3a[0] == 3.0 );
    VERIFY( pt3a == pt3b );
    VERIFY( pt3a != pt3b );
    VERIFY( pt3a < pt3b );
    VERIFY( pt3a <= pt3b );
    VERIFY( ApproxEqual( pt3a, pt3b ) );

    pt3a.Write(out);
    pt3a.Print();
    pt3a.Read(in);
    in >> pt3a;
    out << pt3a;
    in >> vec3a;
    out << vec3a;
    in >> coVec3a;
    out << coVec3a;

    VERIFY( vec3a[0] == 3.0 );
    VERIFY( vec3a == vec3b );
    VERIFY( vec3a != vec3b );
    VERIFY( ApproxEqual( vec3a, vec3b ) );

    vec3a.Write(out);
    vec3a.Print();
    vec3a.Read(in);
    in >> vec3a[0];
    out << vec3a[0];


    VERIFY( coVec3a[0] == 3.0 );
    VERIFY( coVec3a == coVec3b );
    VERIFY( coVec3a != coVec3b );
    VERIFY( ApproxEqual( coVec3a, coVec3b ) );
    VERIFY( ApproxEqual( coVecf3a, coVecf3b ) );

    coVec3a.Write(out);
    coVec3a.Print();
    coVec3a.Read(in);
    in >> coVec3a[0];
    out << coVec3a[0];

}

int
main(int argc, char ** argv )
{
    RNPolynomial oPolynomial;
    R3Pt pt3;
    R3Pt_f pt3f;
    R3Pt_i pt3i(0,1,2);

    R2Pt pt2;
    R2Pt_f pt2f;
    R2Pt_i pt2i;

    R1Pt pt1;
    R1Pt_f pt1f;
    R1Pt_i pt1i;


    R3Vec vec3;
    R3Vec_f vec3f;
    R3Vec_i vec3i;

    R2Vec vec2;
    R2Vec_f vec2f;
    R2Vec_i vec2i;

    R1Vec vec1;
    R1Vec_f vec1f;
    R1Vec_i vec1i;

    
    R2Line dl2;
    R3Line dl3;
    
    dl2.Test();
    dl3.Test();
    
    R2Line_seg lineseg2;
    R3Line_seg lineseg3;

    lineseg2.Test();
    lineseg3.Test();
    
    R2Polygon oPoly2;
    R3Polygon oPoly3;
    
    oPoly2.Test();
    oPoly3.Test();

    RNPolynomial oPoly;
    oPoly.Test();

    R2Line_seg::IsPerpendicularSeg(lineseg2, lineseg2);
    R2Line_seg::IsParallelSeg(lineseg2,lineseg2);

    double dDistAlong = 5.2;
    R3Vec ptPlaneNorm = Unit( R3Vec( 0.5, 0.2, 0.6 ) );
    R3Pt ptPlaneCenter = R3Pt(0,0,0) + ptPlaneNorm * dDistAlong;

    R3Pt ptTry(0.8, -0.2, 0.5);
    R3Pt ptClosest = RNClosestPointToPlane( ptPlaneNorm, dDistAlong, ptTry );

    double dDotProj = Dot( Unit(ptTry - ptClosest), ptPlaneNorm );
    double dDotInPlane = Dot( ptClosest - ptPlaneCenter, ptPlaneNorm );

    ASSERT( RNIsZero( 1.0 - fabs( dDotProj ) ) );
    ASSERT( RNIsZero( dDotInPlane, 1e-14 ) );

    R3Line_seg oLineSeg( ptTry, R3Pt(10.2, 20.6, 30.1) );

    WINbool bRes = RNIntersectPlaneSegment( ptPlaneNorm, dDistAlong, oLineSeg, ptClosest );
    ASSERT( bRes == TRUE );

    dDotInPlane = Dot( ptClosest - ptPlaneCenter, ptPlaneNorm );
    ASSERT( RNIsZero( dDotInPlane ) );
    return 0;
}
