// This may look like C code, but it is really -*- C++ -*-
// Copyright (c) 1993 Hugues H. Hoppe; All rights reserved.

#include <utils/Mesh_Hh.H>

#include <utils/Mesh_Bbox.H>


Bbox::Bbox(const R3Pt& pmin, const R3Pt& pmax)
{
	p[0]=pmin;
	p[1]=pmax;
}

void Bbox::clear()
{
	const float big=1e30f;
	p[0]=R3Pt(big,big,big);
	p[1]=R3Pt(-big,-big,-big);
}

void Bbox::infinite()
{
	const float big=1e30f;
	p[0]=R3Pt(-big,-big,-big);
	p[1]=R3Pt(big,big,big);
}

void Bbox::takeunion(const Bbox& bb)
{
	for (int c=0;c<3;c++) {
		p[0][c]=WINmin(p[0][c],bb[0][c]);
		p[1][c]=WINmax(p[1][c],bb[1][c]);
	}
}

void Bbox::takeunion(const R3Pt& pp)
{
	for (int c=0;c<3;c++) {
		p[0][c]=WINmin( p[0][c], pp[c] );
		p[1][c]=WINmax( p[1][c], pp[c] );
	}
}

void Bbox::intersect(const Bbox& bb)
{
	for (int c=0;c<3;c++) {
		p[0][c]=WINmax(p[0][c],bb[0][c]);
		p[1][c]=WINmin(p[1][c],bb[1][c]);
	}
}

int Bbox::inside(const Bbox& bb) const
{
	for (int c=0;c<3;c++) {
		if (p[0][c]<bb[0][c]) return 0;
		if (p[1][c]>bb[1][c]) return 0;
	}
	return 1;
}

R4Matrix Bbox::getFrameToCube() const
{
	R3Vec di=p[1]-p[0];
	double maxdi=0;
    int c = 0;


	for (c=0;c<3;c++) if (di[c]>maxdi) maxdi=di[c];
	MESHassertx(maxdi);
	R3Vec center;
	for (c=0;c<3;c++) center[c]=(1.0-di[c]/maxdi)*.5;
	center[2]=0;	// objects lie at bottom of cube
	return (R4Matrix::Translation( R3Pt(0,0,0) - p[0] ) *
		    R4Matrix::Scaling(1.0/maxdi, 1.0/maxdi, 1.0/maxdi, 1.0 )*
		    R4Matrix::Translation( center ) );
}

R4Matrix Bbox::getFrameToSmallCube() const
{
	R4Matrix f=getFrameToCube();
	f *= R4Matrix::Scaling(.8f) * R4Matrix::Translation( R3Vec(0.1,0.1,0.1) );
	ForIndex(i,3) {
		if (fabs(f(i,i)-1.)<.05) f(i,i)=1.0;
		if (fabs(f(3,i))<0.05) f(3,i)=0.;
	} EndFor;
	return f;
}


