// This may look like C code, but it is really -*- C++ -*-
// Copyright (c) 1993 Hugues H. Hoppe; All rights reserved.

#include <utils/Mesh_Hh.H>
#include <ctype.h>
#include <utils/Mesh_GMesh.H>
#include <utils/Mesh_Array.H>
#include <utils/Mesh_Set.H>

#pragma warning( disable : 4291 )

const int BUFLEN=2000;

GMesh::GMesh() : ios(0), nextOBJVert(1), nextOBJFace(1) { }

GMesh::~GMesh() { }

static int s_cuspv=MVertex::allocateFlag();
const int& GMesh::CUSPV=s_cuspv;

static int s_sharpe=MEdge::allocateFlag();
const int& GMesh::SHARPE=s_sharpe;

void GMesh::copy(const GMesh& m)
{
    Mesh::copy(m);
    ForMeshVertex(m,v) {
        Vertex vn=idvertex(m.vertexid(v));
        setString(vn,m.string(v));
        setPoint(vn,m.point(v));
        setNorm(vn,m.norm(v));
        vn->SetColor( v->GetColor() );
    } EndFor;
    ForMeshFace(m,f) {
        Face fn=idface(m.faceid(f));
        setString(fn,m.string(f));
        setNorm(fn,m.norm(f));
        ForFaceCorner(m,f,c) {
            if (!m.string(c)) continue;
            Vertex v=m.corner_vertex(c);
            Vertex vn=idvertex(m.vertexid(v));
            Corner cn=corner(vn,fn);
            setString(cn,m.string(c));
        } EndFor;
    } EndFor;
    ForMeshEdge(m,e) {
        if (!m.string(e)) continue;
        Edge en=edge(idvertex(m.vertexid(m.vertex1(e))),
                     idvertex(m.vertexid(m.vertex2(e))));
        setString(en,m.string(e));
    } EndFor;
}

void GMesh::merge(const GMesh& mo, MMap<Vertex,Vertex>* pmvvn)
{
    MMap<Vertex,Vertex>* amvvn=pmvvn;
    if (!pmvvn) amvvn = new MMap<Vertex,Vertex>;
    MMap<Vertex,Vertex>& mvvn=*amvvn;
    ForMeshVertex(mo,vo) {
        Vertex vn=createVertex(); mvvn.enter(vo,vn);
        modFlag(vn,GMesh::ALL,mo.flag(vo,GMesh::ALL));
        setString(vn,mo.string(vo));
        setPoint(vn,mo.point(vo));
    } EndFor;
    Array<Vertex> van;
    ForMeshFace(mo,fo) {
        van.init(0);
        ForFaceVertex(mo,fo,vo) { van+=mvvn.get(vo); } EndFor;
        Face fn=createFace(van);;
        modFlag(fn,GMesh::ALL,mo.flag(fo,GMesh::ALL));
        setString(fn,mo.string(fo));
        ForFaceCorner(mo,fo,co) {
            if (!mo.string(co)) continue;
            Vertex vo=mo.corner_vertex(co);
            Vertex vn=mvvn.get(vo);
            Corner cn=corner(vn,fn);
            setString(cn,mo.string(co));
        } EndFor;
    } EndFor;
    ForMeshEdge(mo,eo) {
        Edge en=edge(mvvn.get(mo.vertex1(eo)),
                     mvvn.get(mo.vertex2(eo)));
        modFlag(en,GMesh::ALL,mo.flag(eo,GMesh::ALL));
        setString(en,mo.string(eo));
    } EndFor;
    if (!pmvvn) delete amvvn;
}

void GMesh::setPoint(Vertex v, const R3Pt& pp)
{
    v->point=pp;
    if (ios) *ios << hform("MVertex %d  %g %g %g\n",
                           vertexid(v),pp[0],pp[1],pp[2]);
}

void GMesh::setNorm(Vertex v, const R3Vec & vec)
{
    v->norm=vec;
}

void GMesh::setNorm(Face f, const R3Vec & vec)
{
    f->norm=vec;
}

void GMesh::polygon(Face f,  R3Polygon& poly) const
{
    poly.Clear();
    ForFaceVertex(*this,f,v) { poly += point(v); } EndFor;

}

double GMesh::length2(Edge e) const
{
    return LengthSq(point(vertex1(e)) - point(vertex2(e)));
}

double GMesh::length(Edge e) const
{
    return Length(point(vertex1(e)) - point(vertex2(e)));
}

void GMesh::transform(const R4Matrix& frame)
{
    ForMeshVertex(*this,v) { setPoint(v, frame * point(v)); } EndFor;
}

// STRING

StringKeyIter::StringKeyIter(const char* pstr) : str(pstr), s(pstr) { }

StringKeyIter::~StringKeyIter() { }

inline int str_key_nchars(const char* s)
{
    // return strcspn(s," =");
    register const char* p;
    for (p=s;;) {
        register char c=*p;
        if (c==' ' || c=='=' || c==0) break;
        p++;
    }
    return p-s;
}

inline const char* str_last_non_space(const char* s)
{
    // const char* sr=strchr(s,' ');
    // return sr ? sr-1 : s+strlen(s)-1;
    register const char* p;
    for (p=s;;) {
        register char c=*p;
        if (c==' ' || c==0) break;
        p++;
    }
    return p-1;
}

inline const char* str_chr(const char* s, char ch)
{
    for (register const char* p=s;;) {
        register char c=*p;
        if (c==ch) return p;
        if (!c) return 0;
        p++;
    }
}

int StringKeyIter::next(const char*& kb, int& kl, const char*& vb, int& vl)
{
    if (!s || !*s) return 0;
    int nch=str_key_nchars(s);
    if (MESHassertw(nch)) { SHOWN(str); SHOWN(s); return 0; }
    kb=s; kl=nch;
    if (s[nch]!='=') {
        MESHassertx(s[nch]==0 || s[nch]==' ');
        vb=kb+kl; vl=0;         // null string ""
        s+=nch; if (s[0]==' ') s++;
        return 1;
    }
    char ch=s[nch+1];
    const char* send=(ch=='('?MESHassertv(str_chr(s+nch+2,')')):
                      ch=='"'?MESHassertv(str_chr(s+nch+2,'"')):
                      ch>='0'&&ch<='9' || ch>='A'&&ch<='z' ?
                      str_last_non_space(s+nch+2) : 0);
    if (MESHassertw(send)) { SHOWN(str); SHOWN(s+nch+1); return 0; }
    vb=s+nch+1; vl=send-(s+nch);
    s=send+1; if (s[0]==' ') s++;
    return 1;
}

const char* GMesh::string_key(const char* str, const char* key)
{
    int keyl=strlen(key);
    ForStringKeyValue(str,kb,kl,vb,vl) {
        int found=!strncmp(kb,key,kl) && kl==keyl;
        if (!found) continue;
        char* sret=(char*)hform(""); // un-const
        strncat(sret,vb,vl);
        return sret;
    } EndFor;
    return 0;
}

const char* GMesh::corner_key(Corner c, const char* key) const
{
    const char* s1=string_key(string(c),key);
    const char* s2=string_key(string(corner_vertex(c)),key);
    if (s1 && s2) MESHWarning("Have both vertex and corner info");
    if (!s1) s1=s2;
    return s1;
}

const char* GMesh::string_update(const char* str, const char* key,
                                 const char* val)
{
    char* sret=(char*)hform(""); // un-const!
    int replaced=0;
    int keyl=strlen(key);
    ForStringKeyValue(str,kb,kl,vb,vl) {
        int found=!strncmp(kb,key,kl) && kl==keyl;
        if (found) {
            if (MESHassertw(!replaced++)) {
                SHOWN(str); SHOWN(kb); SHOWN(kl); SHOWN(vb); SHOWN(vl);
                SHOWN(key); SHOWN(val); MESHassertnever("");
            }
            if (val) {
                if (sret[0]) strcat(sret," ");
                strcat(sret,key);
                if (*val) { strcat(sret,"="); strcat(sret,val); }
            }
        } else {
            if (sret[0]) strcat(sret," ");
            strncat(sret,kb,kl);
            if (vl) {
                strcat(sret,"=");
                strncat(sret,vb,vl);
            }
        }
    } EndFor;
    if (!replaced) {
        if (val) {
            if (sret[0]) strcat(sret," ");
            strcat(sret,key);
            if (*val) { strcat(sret,"="); strcat(sret,val); }
        }
    }
    return sret[0] ? sret : 0;
}

void GMesh::updateString(Vertex v, const char* key, const char* val)
{
    setString(v,string_update(string(v),key,val));
}

void GMesh::updateString(Face f, const char* key, const char* val)
{
    setString(f,string_update(string(f),key,val));
}

void GMesh::updateString(Edge e, const char* key, const char* val)
{
    setString(e,string_update(string(e),key,val));
}

void GMesh::updateString(Corner c, const char* key, const char* val)
{
    setString(c,string_update(string(c),key,val));
}

// I/O

void GMesh::read(istream& is)
{
    nextOBJVert=1;
    nextOBJFace=1;

    char str[BUFLEN];
    
    edgenum = 1;

    if ( !is.good() )
        is.clear();

    while ( !is.eof() ) {
//#if defined(WIN32)
        is >> ws;
//#endif
	
        is.get(str,sizeof(str));
        if ( !strncmp(str, "EndMesh", 7) ) 
            break;
        //MESHassertw1(is.peek()=='\n');
        is.ignore(9999,'\n');
        readline(str);
    }
    if (sdebug>=1) OK();
}

void GMesh::readline(char* str)
{
    if (str[0]=='#') return;
    char* sinfo=(char*)str_chr(str,'{');

    if (sinfo) {
        *sinfo++=0;
        char* s=(char*)str_chr(sinfo,'}');
        if (MESHassertw1(s)) sinfo=0; else *s=0;
    }
    if (str[0]=='V' && !strncmp(str,"Vertex ",7)) {
        R3Pt p; int vi;
        MESHassertx(sscanf(str,"Vertex %d %lf %lf %lf",&vi,&p[0],&p[1],&p[2])==4);
        Vertex v=createVertexI(vi); setPoint(v,p);
        if (sinfo) {
            setString(v,sinfo);
            if (string_key(sinfo,"cusp")) modFlag(v,CUSPV,1);
        }
    } else if (str[0]=='F' && !strncmp(str,"Face ",5)) {
        static Array<Vertex> va; va.init(0);
        char* s=str+4;
        int fi=-1;
        for (;;) {
            while (*s && isspace(*s)) s++;
            if (!*s) break;
            char* beg=s;
            while (*s && isdigit(*s)) s++;
            if (MESHassertw1(!*s || isspace(*s))) continue;
            int j=atoi(beg);
            if (fi<0) { fi=j; continue; }
            va+=idvertex(j);
        }
        if (MESHassertw1(va.num()>=3)) return;
        if (MESHassertw1(legalCreateFace(va))) 
           return;
        Face f=fi?createFaceI(fi,va):createFace(va);
        if (sinfo) setString(f,sinfo);
    } else if (str[0]=='C' && !strncmp(str,"Corner ",7)) {
        int vi; int fi;
        MESHassertx(sscanf(str,"Corner %d %d",&vi,&fi)==2);
        Vertex v=idvertex(vi);
        // Face f=idface(fi);
        Face f=id_retrieve_face(fi);
        if (!f) {
            MESHWarning("Corner does not exist");
        } else {
            Corner c=corner(v,f);
            if (sinfo) setString(c,sinfo);
        }
    } else if (str[0]=='E' && !strncmp(str,"Edge ",5)) {
        int vi1, vi2;
        MESHassertx(sscanf(str,"Edge %d %d",&vi1,&vi2)==2);
        Edge e=queryEdge(idvertex(vi1),idvertex(vi2));
        if (!e) MESHWarning("GMesh::read(): Did not find edge in mesh");
        if (e && sinfo) {
            setString(e,sinfo);
            modFlag(e,SHARPE,string_key(sinfo,"sharp")?1:0);
        }
    } else if (str[0]=='v' && str[1]==' ') /*read an OBJ vertex*/ {
        R3Pt p; int vi=nextOBJVert++;
        MESHassertx(sscanf(str,"v %lf %lf %lf",&p[0],&p[1],&p[2])==3);
        Vertex v=createVertexI(vi); setPoint(v,p);
        if (sinfo) {
            setString(v,sinfo);
            if (string_key(sinfo,"cusp")) modFlag(v,CUSPV,1);
        }        
    } else if (str[0]=='f' && str[1]==' ') /*read an OBJ face*/ {
        static Array<Vertex> va; va.init(0);
        char* s=str+2;
        int fi=nextOBJFace++;
        for (;;) {
            while (*s && isspace(*s)) s++;
            if (!*s) break;
            char* beg=s;
            while (*s && isdigit(*s)) s++;
            while (*s && ( isdigit(*s) || *s=='/' )){
                *s = ' ';
                s++;
            }
            if (MESHassertw1(!*s || isspace(*s))) continue;
            int j=atoi(beg);
            va+=idvertex(j);
        }
        if (MESHassertw1(va.num()>=3)) return;
        if (MESHassertw1(legalCreateFace(va))){
           cout << "illegal line: " << str << endl;
           return;
        }
        Face f=fi?createFaceI(fi,va):createFace(va);
        if (sinfo) setString(f,sinfo);
    } else if (str[0]=='v' && str[1]=='t' && str[2]==' ') /*discard an OBJ vt line*/ {
        //do nothing; simply discard line
    } else if (str[0]=='v' && str[1]=='n' && str[2]==' ') /*Read an OBJ normal*/ {
        R3Vec n;
        MESHassertx(sscanf(str,"vn %lf %lf %lf",&n[0],&n[1],&n[2])==3);
        //do nothing; simply discard line
    } else if (!strncmp(str,"MVertex ",8)) {
        R3Pt p; int vi;
        MESHassertx(sscanf(str,"MVertex %d %g %g %g",&vi,&p[0],&p[1],&p[2])==4);
        Vertex v=idvertex(vi);
        setPoint(v,p);
        if (sinfo) setString(v,sinfo);
    } else if (!strncmp(str,"CVertex ",8)) {
        (void)createVertexI(atoi(str+8));
    } else if (!strncmp(str,"DVertex ",8)) {
        destroyVertex(idvertex(atoi(str+8)));
    } else if (!strncmp(str,"DFace ",6)) {
        destroyFace(idface(atoi(str+6)));
    } else if (!strncmp(str,"Ecol ",5)) {
        int vi1, vi2;
        MESHassertx(sscanf(str,"Ecol %d %d",&vi1,&vi2)==2);
        // collapseEdge(orderedEdge(idvertex(vi1),idvertex(vi2)));
        Vertex vs=idvertex(vi1), vt=idvertex(vi2);
        collapseEdgeVertex(edge(vs,vt),vs);
    } else if (!strncmp(str,"Eswa ",5)) {
        int vi1, vi2;
        MESHassertx(sscanf(str,"Eswa %d %d",&vi1,&vi2)==2);
        MESHassertx(swapEdge(orderedEdge(idvertex(vi1),idvertex(vi2))));
    } else if (!strncmp(str,"Espl ",5)) {
        int vi1, vi2, vi3;
        MESHassertx(sscanf(str,"Espl %d %d %d",&vi1,&vi2,&vi3)==3);
        splitEdge(orderedEdge(idvertex(vi1),idvertex(vi2)),vi3);
    } else if (!strncmp(str,"Vspl ",5)) {
        int vi, vs1i, vs2i, vni;
        MESHassertx(sscanf(str,"Vspl %d %d %d %d",&vi,&vs1i,&vs2i,&vni)==4);
        splitVertex(idvertex(vi),idvertex(vs1i),vs2i?idvertex(vs2i):0,vni);
    } else {
        if (MESHWarning("GMesh::read: cannot parse line")) SHOWN(str);
    }
}

static int strprefix(const char* s, const char* p)
{
    register char pc;
    for (;;) {
        pc=*p;
        if (!pc || *s!=pc) break;
        s++; p++;
    }
    return !pc;
}

int GMesh::recognizeLine(const char* s)
{
    static const char* sar[]={"Vertex ", "Face ", "Corner ", "Edge ",
                              "MVertex ", "CVertex ", "DVertex ", "DFace ",
                              "Ecol ", "Eswa ", "Espl ", "Vspl"};
    ForIndex(i,12) {
        if (strprefix(s,sar[i])) return 1;
    } EndFor;
    return 0;
}

void GMesh::write(ostream& os) const
{
    ForMeshOrderedVertex(*this,v) {
        const R3Pt& p=point(v);
        os << "Vertex " << vertexid(v) << "  " <<
            p[0] << " " << p[1] << " " << p[2];
        const char* sinfo=string(v);
        if (sinfo) os << " {" << sinfo << "}";
        os << "\n";
        MESHassertx(os);
    } EndFor;
    ForMeshOrderedFace(*this,f) {
        os << "Face " << faceid(f) << " "; 
        ForFaceVertex(*this,f,v) {
            os << " " << vertexid(v);
        } EndFor;
        const char* sinfo=string(f);
        if (sinfo) os << " {" << sinfo << "}";
        os << "\n";
        MESHassertx(os);
    } EndFor;
    ForMeshEdge(*this,e) {
        const char* sinfo=string(e);
        if (!sinfo) continue;
        os << "Edge " << vertexid(vertex1(e)) << " " <<
            vertexid(vertex2(e)) << " {" << sinfo << "}\n";
        MESHassertx(os);
    } EndFor;
    ForMeshOrderedFace(*this,f) {
        ForFaceCorner(*this,f,c) {
            const char* sinfo=string(c);
            if (!sinfo) continue;
            os << "Corner " << vertexid(corner_vertex(c)) << " " <<
                faceid(f) << " {" << sinfo << "}\n";
            MESHassertx(os);
        } EndFor;
    } EndFor;
    os << flush;
}

ostream* GMesh::recordChanges(ostream* pos)
{
    ostream* oos=ios; ios=pos; return oos;
}

Vertex GMesh::createVertexI(int id)
{
    if (ios) *ios << "CVertex " << id << '\n';
    return Mesh::createVertexI(id);
}

// Override Mesh members
void GMesh::destroyVertex(Vertex v)
{
    if (ios) *ios << "DVertex " << vertexid(v) << '\n';
    Mesh::destroyVertex(v);
}

Face GMesh::createFaceI(int id, const Array<Vertex>& va)
{
    Face f=Mesh::createFaceI(id,va);
    if (ios) {
        *ios << "Face " << id << ' ';
        ForIndex(i,va.num()) { *ios << ' ' << vertexid(va[i]); } EndFor;
        *ios << '\n';
    }
    return f;
}

void GMesh::destroyFace(Face f)
{
    if (ios) *ios << "DFace " << faceid(f) << '\n';
    Mesh::destroyFace(f);
}

void GMesh::collapseEdgeVertex(Edge e, Vertex vs)
{
    if (sdebug>=1) valid(e);
    ostream* tos=ios; ios=0;
    Vertex vt=oppVertex(vs,e);
    int ids=vertexid(vs), idt=vertexid(vt);
    // Compute geometry for unified vertex (vs)
    int isbs=isBoundary(vs), isbt=isBoundary(vt), sumb=isbs+isbt;
    R3Pt p=(sumb==0 || sumb==2 ? Lerp(point(vs),point(vt), 0.5) :
             isbs ? point(vs) : point(vt));
    Set<Vertex> vsharp;
    ForVertexEdge(*this,vt,ee) {
        if (ee!=e && flag(ee,SHARPE)) vsharp.enter(oppVertex(vt,ee));
    } EndFor;
    Mesh::collapseEdgeVertex(e,vs); // vs kept
    setPoint(vs,p);             // (ios==0)
    ForSet(vsharp,Vertex,v) {
        Edge e=edge(vs,v);
        if (!isBoundary(e)) modFlag(e,SHARPE,1);
    } EndFor;
    ForVertexEdge(*this,vs,ee) {
        // test (vs,vo1), (vs,vo2) to remove SHARPE if now boundary
        if (isBoundary(ee) && flag(ee,SHARPE)) modFlag(ee,SHARPE,0);
    } EndFor;
    if (tos) ios=tos, *ios << "Ecol " << ids << ' ' << idt << '\n';
}

void GMesh::collapseEdge(Edge e)
{
    collapseEdgeVertex(e,vertex1(e));
}

Vertex GMesh::splitEdge(Edge e, int id)
{
    if (sdebug>=1) valid(e);
    ostream* tos=ios; ios=0;
    Vertex v1=vertex1(e), v2=vertex2(e);
    Face f1=face1(e), f2=face2(e);
    Vertex vo1=sideVertex1(e), vo2=sideVertex2(e);
    int fle=flag(e,SHARPE);
    const char* fstring1= string(f1);
    const char* fstring2= f2 ? string(f2) : 0;
    Vertex vn=Mesh::splitEdge(e,id);
    modFlag(edge(v1,vn),SHARPE,fle),modFlag(edge(v2,vn),SHARPE,fle);
    if (fstring1) {
        ForEdgeFace(*this,edge(vn,vo1),f) { setString(f,fstring1); } EndFor;
    }
    if (fstring2) {
        ForEdgeFace(*this,edge(vn,vo2),f) { setString(f,fstring2); } EndFor;
    }
    setPoint(vn, Lerp(point(v1),point(v2), 0.5)); // (ios==0)
    if (tos) ios=tos, *ios << "Espl " << vertexid(v1) << ' ' <<
        vertexid(v2) << ' ' << vertexid(vn) << '\n';
    return vn;
}

Edge GMesh::swapEdge(Edge e)
{
    if (sdebug>=1) valid(e);
    ostream* tos=ios; ios=0;
    Vertex v1=vertex1(e), v2=vertex2(e);
    Face f1=face1(e), f2=face2(e);
    const char* fstring=0;
    if (string(f1) && string(f2) && !strcmp(string(f1),string(f2)))
        fstring=newString(string(f1));
    Edge ne=Mesh::swapEdge(e);
    if (fstring) {
        ForEdgeFace(*this,ne,f) { setString(f,fstring); } EndFor;
        delete[] (char*)fstring;
    }
    if (tos) ios=tos, *ios << "Eswa " << vertexid(v1) << ' ' <<
        vertexid(v2) << '\n';
    return ne;
}

Vertex GMesh::splitVertex(Vertex v1, Vertex vs1, Vertex vs2, int v2i)
{
    ostream* tos=ios; ios=0;
    Vertex vn=Mesh::splitVertex(v1,vs1,vs2,v2i);
    NEST {
        Edge e;
        if (1) {
            e=queryEdge(v1,vs1); if (e) setString(e,0),modFlag(e,ALL,0);
        }
        if (vs2) {
            e=queryEdge(v1,vs2); if (e) setString(e,0),modFlag(e,ALL,0);
        }
    }
    if (tos) ios=tos, *ios << "Vspl " << vertexid(v1) << ' ' <<
        vertexid(vs1) << ' ' << (vs2?vertexid(vs2):0) << ' ' << v2i << '\n';
    return vn;
}

void GMesh::center_split_face(Face f, Vertex v)
{
    const char* fstring=newString(string(f));
    MMap<Vertex,char*> mvs;
    ForFaceCorner(*this,f,c) {
        Vertex v=corner_vertex(c);
        if (string(c)) mvs.enter(v,newString(string(c)));
    } EndFor;
    Mesh::center_split_face(f,v);

    if (fstring) {
        ForVertexFace(*this,v,fn) { setString(fn,fstring); } EndFor;
        delete[] (char*)fstring;
    }
    if (mvs.num()) {
        ForVertexFace(*this,v,fn) {
            ForFaceCorner(*this,fn,cc) {
                Vertex vv=corner_vertex(cc);
                if (vv==v) continue;
                setString(cc,mvs.retrieve(vv)); // could be 0
            } EndFor;
        } EndFor;
        ForMapValue(mvs,Vertex,char*,s) { delete s; } EndFor;
    }
}


Vertex GMesh::center_split_face(Face f)
{
    static R3Polygon poly; 
    
    polygon(f,poly);

    const char* fstring=newString(string(f));
    MMap<Vertex,char*> mvs;
    ForFaceCorner(*this,f,c) {
        Vertex v=corner_vertex(c);
        if (string(c)) mvs.enter(v,newString(string(c)));
    } EndFor;
    R3Vec scol(0,0,0); int havecol=1;
    R3Vec snor(0,0,0); int havenor=1;
    R3Vec suvw(0,0,0); int haveuvw=1;
    ForFaceCorner(*this,f,c) {
        Vertex v=corner_vertex(c);
        R3Vec n; const char* s;
        if ((s=corner_key(c,"rgb")) &&
             !MESHassertw(sscanf(s,"( %g %g %g )",&n[0],&n[1],&n[2])==3)) scol+=n;
        else havecol=0;
        if ((s=corner_key(c,"normal")) &&
             !MESHassertw(sscanf(s,"( %g %g %g )",&n[0],&n[1],&n[2])==3)) snor+=n;
        else havenor=0;
        if ((s=corner_key(c,"uv")) &&
             !MESHassertw(sscanf(s,"( %g %g )",&n[0],&n[1])==2)) suvw+=n;
        else haveuvw=0;
    } EndFor;
    scol/=poly.Num_pts(); snor/=poly.Num_pts(); suvw/=poly.Num_pts();
    Vertex vn=Mesh::center_split_face(f);
    setPoint(vn,poly.Centroid());
    if (havecol)
        updateString(vn,"rgb",hform("(%f %f %f)",scol[0],scol[1],scol[2]));
    if (havenor)
        updateString(vn,"normal",hform("(%f %f %f)",snor[0],snor[1],snor[2]));
    if (haveuvw)
        updateString(vn,"uv",hform("(%f %f)",suvw[0],suvw[1]));
    if (fstring) {
        ForVertexFace(*this,vn,fn) { setString(fn,fstring); } EndFor;
        delete[] (char*)fstring;
    }
    if (mvs.num()) {
        ForVertexFace(*this,vn,fn) {
            ForFaceCorner(*this,fn,cc) {
                Vertex vv=corner_vertex(cc);
                if (vv==vn) continue;
                setString(cc,mvs.retrieve(vv)); // could be 0
            } EndFor;
        } EndFor;
        ForMapValue(mvs,Vertex,char*,s) { delete s; } EndFor;
    }
    return vn;
}

Edge GMesh::split_face(Face f, Vertex v1, Vertex v2)
{
    const char* fstring=newString(string(f));
    Edge en=Mesh::split_face(f,v1,v2);
    if (fstring) {
        ForEdgeFace(*this,en,f) { setString(f,fstring); } EndFor;
        delete[] (char*)fstring;
    }
    return en;
}

Face GMesh::coalesce_faces(Edge e)
{
    Face f1=face1(e), f2=face2(e);
    const char* fstring=0;
    if (string(f1) && string(f2) && !strcmp(string(f1),string(f2)))
        fstring=newString(string(f1));
    Face fn=Mesh::coalesce_faces(e);
    setString(fn,fstring);
    delete[] (char*)fstring;
    return fn;
}

Vertex GMesh::insert_vertex_on_edge(Edge e)
{
    // lose strings
    Vertex v1=vertex1(e), v2=vertex2(e);
    Vertex vn=Mesh::insert_vertex_on_edge(e);
    setPoint(vn,Lerp(point(v1),point(v2), 0.5));
    return vn;
}

Edge GMesh::remove_vertex_between_edges(Vertex vr)
{
    // lose strings
    return Mesh::remove_vertex_between_edges(vr);
}

int GMesh::substituteFaceVertex(Face f, Vertex vold, Vertex vnew)
{
	if (!Mesh::substituteFaceVertex(f,vold,vnew)) return 0;
	if (ios) *ios << "SubFV " << faceid(f) << ' ' <<
		vertexid(vold) << ' ' << vertexid(vnew) << '\n';
	return 1;
}
