// This may look like C code, but it is really -*- C++ -*-
// Copyright (c) 1992 Hugues H. Hoppe; All rights reserved.

#include <utils/Mesh_Hh.H>
#include <utils/Mesh_PolygonSpatial.H>
#include <utils/Mesh_Bbox.H>


static double polygonapproxdistance2(const R3Pt& p, Univ id)
{
	R3Polygon& poly=*Conv<R3Polygon*>::d(id);

    const double dDist = poly.LBDistSq( p );
	return dDist;
}

static double polygondistance2(const R3Pt& p, Univ id)
{
	R3Polygon& poly=*Conv<R3Polygon*>::d(id);
	MESHassertx(poly.Num_pts()==3);

    const double dDist = poly.DistSq( p );

    return dDist;
}

PolygonSpatial::PolygonSpatial(int pgn)
: ObjectSpatial(pgn,polygonapproxdistance2,polygondistance2) { }

PolygonSpatial::~PolygonSpatial() { }


class Senter {
public:
	const R3Polygon* polyorigp;
	R3Polygon poly;
	Bbox bbox;
};
static Senter senter;

static WINbool intersectBbox(const R3Polygon &in_poly, const Bbox& bb, R3Polygon &out_poly)
{
	R3Polygon poly = in_poly, polyTemp;

    poly.IntersectHyperplane(bb[0], R3Vec(+1,+0,+0), polyTemp ); 
    if ( polyTemp.Num_pts() == 0 ) return FALSE;

	polyTemp.IntersectHyperplane(bb[1], R3Vec(-1,+0,+0), poly); 
    if ( poly.Num_pts() == 0 ) return FALSE;

	poly.IntersectHyperplane(bb[0], R3Vec(+0,+1,+0), polyTemp);
    if ( polyTemp.Num_pts() == 0 ) return FALSE;

	polyTemp.IntersectHyperplane(bb[1], R3Vec(+0,-1,+0), poly); 
    if ( poly.Num_pts() == 0 ) return FALSE;

	poly.IntersectHyperplane(bb[0], R3Vec(+0,+0,+1), polyTemp); 
    if ( polyTemp.Num_pts() == 0 ) return FALSE;

	polyTemp.IntersectHyperplane(bb[1], R3Vec(+0,+0,-1), poly); 
    if ( poly.Num_pts() == 0 ) return FALSE;

    out_poly = poly;

	return TRUE;
}


static WINbool polygoninbbox(const Bbox& bb)
{
    for (int c=0;c<3;c++) {
		if (senter.bbox[0][c]>bb[1][c] ||
		    senter.bbox[1][c]<bb[0][c]) return FALSE;
    }
/*
    MeshPolygon poly(3);
    for ( int i = 0; i < 3; i++ )
        poly[i] = (*senter.polyorigp)[i];

    poly.intersectBbox(bb );
    */
    WINbool bInter = intersectBbox( *senter.polyorigp, senter.bbox, senter.poly );
    /*
    for ( FORINT i = 0; i < 3; i++ ) {
        ASSERT( ApproxEqual( poly[i], senter.poly[i] ) );
    }
    */

    return bInter;
}

static void getbbox(const R3Polygon &in_poly, Bbox& bb) 
{
	bb.clear();
	for (int i = 0; i < in_poly.Num_pts();i++) 
        bb.takeunion(in_poly[i]);
}

void PolygonSpatial::enter(const R3Polygon &poly)
{
	ASSERT(poly.Num_pts()==3);
	senter.polyorigp = &poly;
	senter.poly = *senter.polyorigp;
	getbbox(senter.poly, senter.bbox);
	ObjectSpatial::enter(Conv<const R3Polygon*>::e( &poly),
			             poly[0],polygoninbbox);
}


static struct Ssinter {
	const R3Pt* p1;
	const R3Pt* p2;
	R3Vec vray;
	R3Pt pint;
	int foundint;
	const R3Polygon* poly;
	double fmin;
} sinter;

static int testpolygonwithray(Univ id)
{
	const R3Polygon* poly=Conv<R3Polygon*>::d(id);
	R3Pt pint;
	if (!poly->IntersectSegment( R3Line_seg(*sinter.p1, *sinter.p2 ), pint )) return 0;
	double f=Dot(pint - *sinter.p1, sinter.vray);
	if (!sinter.foundint || f<sinter.fmin)
		sinter.fmin=f,sinter.pint=pint,sinter.poly=poly;
	sinter.foundint=1;
	return 1;
}

int PolygonSpatial::firstAlongSegment(const R3Pt& p1, const R3Pt& p2,
				                      const R3Polygon*& poly, R3Pt& pint) const
{
	sinter.p1=&p1;
	sinter.p2=&p2;
	sinter.vray=p2-p1;
	sinter.foundint=0;
	searchsegment(p1,p2,testpolygonwithray);
	poly=sinter.poly;
	pint=sinter.pint;
	return sinter.foundint;
}

