/** \defgroup Meshes Mesh classes

   Classes beginning with PMeshLite, GMesh, and Mesh. GMesh and Mesh
  are Hugues Hoppe's meshes. Also Vertex, Face, and Edge. Helper
  classes include Array, PQueue, BMap.<br>

 Libraries needed: wsock32.lib kernel32.lib

 You will also need plyDB.lib (read and write ply files, found in ExternalLibraries).

The Mesh class is a winged-edge implementation of a complex; i.e.,
  vertices (indices), edges (two vertices), and faces (ordered list of
  vertices). GMesh builds on Mesh to add geometry. PMeshLite is a
  further wrapper around GMesh that also adds some additional
  functionality, like growing disks.

The mesh code is rather old and looks a lot like C. Many of the helper
  classes (e.g., Array, Queue) can now be found in the standard
  template library; this class has it's own versions because, well,
  the STL didn't exist back then. The mesh code has a lot of 
  macros which let you iterate over the mesh elements; these are the
  fastest way to access element information. PMeshLite has methods
  which return Arrays; the two approaches provide the same basic
  functionallity.

Because this is C++ and not Modula, the vertex, edge, and face classes
  actually contain geometric information. So this info (locations,
  ids, normals) can be gotten directly from the classes. However,
  information like adjacency can only be extracted from the mesh
  itself. Hence all of the adjacency information is contained in
  functions on PMeshLite, or  macros that take a mesh
  (GMesh). Note that you can get a GMesh reference from PMeshlite
  using the method Mesh().

Do not allocate vertices/edges/faces yourself - get them from the mesh
  class. There is no add edge; this is because edges are added and
  deleted when faces are. Note that Vertex, Edge, and Face are all
  typedef'd pointers to the real classes, MVertex, MEdge, MFace. There
  is also a half-edge data structure used internally; you can't get to
  it. The edge class has v1, v2, but do not assume the order means
  anything, it just depends on which face was added into the mesh first. 

There is a very funky mechanism for adding additional data (such as
  u,v coordinates) to the mesh. The data is stored in a string, and
  methods are supplied to pull data out/put data into the string. 

You should not assume that the vertices/edges/faces are numbered sequentially unless you have called Renumber().

Each vertex/edge/face has got an Id() and an IndexId(). Id() is equal
  to IndexId() + 1; this is because the zero label is used to indicate
  an invalid element. Sigh. Use IndexId().

Files in utils:
  - Mesh_*
  - GMesh_*
  - PMeshLite_*

Files in OpenGL:
  - OGLMeshes_DrawingVariables.H Wrapper around drawing variables
  - OGLMeshes_Texture.H Texture mapping class
  - OGLMeshes_Shadow.H Shadow maps
  - OGLMeshes_Scene.H Wrapper around groups of meshes/scenes

Source file directories:
  - src/utils/mesh (Mesh, GMesh, and helper functions)
  - src/utils/PMeshLite (PMeshLite)
  - src/OpenGL/OGLMeshes (OGLMeshes classes)

<br>
*/
//@{
//@}

/** \defgroup MeshesClosest Closest point, intersection
   \ingroup Meshes

 Intersecting a ray with a mesh, finding the closest point, getting which
 face was clicked on */
//@{
//@}

/** \defgroup MeshesDraw Drawing routines for meshes and scenes, shadow and texture mapping
   \ingroup Meshes

Draw some or all of the elements of a mesh. */
//@{
//@}

/** \defgroup MeshesHelper Helper classes (e.g., Arrays, queues)
   \ingroup Meshes

   Most of these have standard template library equivalents. */
//@{
//@}

/** \defgroup MeshesHash Hash table/mapping functions 
   \ingroup MeshesHelper

  Most of the mesh elements are stored in hash tables*/
//@{
//@}

/** \defgroup MeshesQueue  Re-sizable priority queues.
  \ingroup MeshesHelper

  These queues were primarily used for mesh simplification, But also show up to
  make some search-type queries faster.  */
//@{
//@}


