#include <utils/Mesh_Hh.H>
#include "StdAfx.H"
#include <utils/Srf_Draw.H>

short SRFDraw::m_iSrfNum = 0;

void SRFDraw::Set_vert(GMesh &io_mesh, Vertex in_v, 
                        const R2Pt &in_pt)
{

    R2Pt ptUV( in_pt );
    m_srf.Clip_uv( ptUV );

   int iVid = io_mesh.vertexid(in_v);
   if (iVid >= m_aSrfUvs.num())
     m_aSrfUvs.need(iVid + 10);

   m_aSrfUvs[iVid].m_dU = ptUV[0];
   m_aSrfUvs[iVid].m_dV = ptUV[1];
   m_aSrfUvs[iVid].m_iSrfName = m_iSrfName;
   
   io_mesh.setPoint(in_v, Srf()(ptUV));

   char strSrfUV[255];
   sprintf(strSrfUV, "(%lf %lf %d)", ptUV[0], ptUV[1], m_iSrfName);
   io_mesh.updateString(in_v, "Srfuv", strSrfUV);
}


Vertex SRFDraw::New_vert(const R2Pt &in_pt)
{
   Vertex out_vert = m_mesh.createVertex();
   Set_vert(m_mesh, out_vert, in_pt);
   return out_vert;
}


SRFDraw::SRFDraw(const SRFNub3TC &in_srf, 
                 double in_dEps
                 ) : m_srf(in_srf), m_dEpsFit(in_dEps), m_crvTrim(CRVSpace(0), 5)
{ 
   m_iSrfName = m_iSrfNum++;

   m_crvTrim.Pt(0) = R2Pt(Srf().Start_u(), Srf().Start_v());
   m_crvTrim.Pt(1) = R2Pt(Srf().Start_u(), Srf().End_v());
   m_crvTrim.Pt(2) = R2Pt(Srf().End_u(), Srf().End_v());
   m_crvTrim.Pt(3) = R2Pt(Srf().End_u(), Srf().Start_v());
   m_crvTrim.Pt(4) = R2Pt(Srf().Start_u(), Srf().Start_v());
   Make_corners();

   Start_check_sample();
}


SRFDraw::SRFDraw(const SRFNub3TC &in_srf, double in_dEps,
                 const CRVNub2TC &in_crvE0, const CRVNub2TC &in_crvE1, 
                 const CRVNub2TC &in_crvE2, const CRVNub2TC &in_crvE3, 
                 CRVNub3TC &out_crvDraw)
                 : m_srf(in_srf), m_dEpsFit(in_dEps), 
                 m_crvTrim((CRVSpace) in_crvE0, 
                          in_crvE0.Num_pts() + in_crvE1.Num_pts() + 
                          in_crvE2.Num_pts() + in_crvE3.Num_pts() - 3 * 3)
{ 
   m_iSrfName = m_iSrfNum++;

   Make_trim(in_crvE0, in_crvE1, in_crvE2, in_crvE3);
   Make_corners();

   Start_check_sample();
   Clip(out_crvDraw);
}

double SRFDraw::Fit_edge( const R2Pt &in_pt1, const R2Pt &in_pt2)
{
   static R3Pt P1, P2, midP, avgP;
   static const R3Pt ptZero(0,0,0);

   if (in_pt1[0] >= Srf().End_u() && in_pt2[0] >= Srf().End_u()) return 0;
   if (in_pt1[1] >= Srf().End_v() && in_pt2[1] >= Srf().End_v()) return 0;

   P1 = Srf()(in_pt1);
   P2 = Srf()(in_pt2);
   midP = Srf()( Lerp( in_pt1, in_pt2, 0.5 ) );
   avgP = Lerp( P1, P2, 0.5 );
   return Length(avgP - midP);
}


void SRFDraw::Checker_fit_depth(DepthGrid &io_depthGrid,
                                double in_dSu, double in_dEu,
                                double in_dSv, double in_dEv, int in_iDepth)
{
   bool bSplit = (in_iDepth < 2) ? TRUE : FALSE;
   double dEps = 1e-6;
   if (RNApproxEqual(in_dSu, Srf().Start_u(), dEps)) in_dSu = Srf().Start_u();
   if (RNApproxEqual(in_dSv, Srf().Start_v(), dEps)) in_dSv = Srf().Start_v();
   if (RNApproxEqual(in_dEu, Srf().End_u(), dEps)) in_dEu = Srf().End_u();
   if (RNApproxEqual(in_dEv, Srf().End_v(), dEps)) in_dEv = Srf().End_v();

//   for (int i = 0; i < in_iDepth; i++)
//      TRACE("  ");
//   TRACE("Sample %f %f %f %f  \n", in_dSu, in_dEu, in_dSv, in_dEv);
   Array<double> adFit;
   adFit += Fit_edge( R2Pt( in_dSu, in_dSv ),  R2Pt( in_dEu, in_dEv) );
   adFit += Fit_edge( R2Pt( in_dSu, in_dSv ),  R2Pt( in_dSu, in_dEv) );
   adFit += Fit_edge( R2Pt( in_dSu, in_dSv ),  R2Pt( in_dEu, in_dSv) );
   adFit += Fit_edge( R2Pt( in_dEu, in_dEv ),  R2Pt( in_dSu, in_dEv) );
   adFit += Fit_edge( R2Pt( in_dEu, in_dEv ),  R2Pt( in_dEu, in_dSv) );
   
   double dFitSum = 0;
   for (int i = 0; i < adFit.num(); i++) {
      if (adFit[i] > 2.0 * m_dEpsFit) bSplit = TRUE;
      dFitSum += adFit[i];
   }
   if (dFitSum / 5.0 > m_dEpsFit) bSplit = TRUE;

   // Check along trim curve, unless outside srf edge
   // (The grid space may be stretched/doubled)
   bool bOver = FALSE;
   if (in_dSu > Srf().End_u() && in_dEu > Srf().End_u()) bOver = TRUE;
   if (in_dSv > Srf().End_v() && in_dEv > Srf().End_v()) bOver = TRUE;
   if (bSplit == FALSE && bOver == FALSE) {
      R3Polygon poly_srf(4);
      poly_srf[0] = Srf()(R2Pt( in_dSu, in_dSv ) );
      poly_srf[1] = Srf()(R2Pt( in_dSu, in_dEv ) );
      poly_srf[2] = Srf()(R2Pt( in_dEu, in_dEv ) );
      poly_srf[3] = Srf()(R2Pt( in_dEu, in_dSv ) );
      
      Array<double> us, vs;
      for (int i = 0; i < m_crvTrim.Num_pts(); i++) {
         const R2Pt &pt = m_crvTrim.Pt(i);
         if (pt[0] > in_dSu && pt[0] < in_dEu  && pt[1] > in_dSv && pt[1] < in_dEv) {
            us += pt[0];
            vs += pt[1];
         }
      }
      double min_d = 1e30;
      for (FORINT i = 0; i < us.num(); i++) {
         R3Pt srf_pt = Srf()( R2Pt( us[i], vs[i] ));
         min_d = 1e30;
         for (int j = 0; j < poly_srf.Num_pts(); j++) {
            double d = Length(poly_srf[j] - srf_pt);
            min_d = WINmin(d, min_d);
         }
         if (min_d > 4.0 * m_dEpsFit) {
            bSplit = TRUE;
            break;
         }
      }
   }
   // We can stop now
   if (bSplit == FALSE) {
      io_depthGrid.need(2);
      io_depthGrid[0].need(2);
      io_depthGrid[1].need(2);
      
      io_depthGrid[0][0] = in_iDepth;

      return;
   }

   /*   c1   c2
    *   c3   c4
    */
   DepthGrid depthGridCi[4];
   Checker_fit_depth(depthGridCi[0], 
                     in_dSu,  0.5 * (in_dSu + in_dEu), 
                     in_dSv,  0.5 * (in_dSv + in_dEv), 
                     in_iDepth+1);
   Checker_fit_depth(depthGridCi[1], 
                     0.5 * (in_dSu + in_dEu), in_dEu, 
                     in_dSv,                  0.5 * (in_dSv + in_dEv), 
                     in_iDepth+1);
   Checker_fit_depth(depthGridCi[2], 
                     in_dSu,                  0.5 * (in_dSu + in_dEu), 
                     0.5 * (in_dSv + in_dEv), in_dEv, 
                     in_iDepth+1);
   Checker_fit_depth(depthGridCi[3], 
                     0.5 * (in_dSu + in_dEu), in_dEu, 
                     0.5 * (in_dSv + in_dEv), in_dEv, 
                     in_iDepth+1);

   int iMaxDiv = 0;
   for (FORINT i = 0; i < 4; i++) iMaxDiv = WINmax(iMaxDiv, depthGridCi[i].num());
   
   int iDiv = iMaxDiv + iMaxDiv-1;
   io_depthGrid.need(iDiv);
   for ( FORINT i = 0; i < io_depthGrid.num(); i++) {
      io_depthGrid[i].need(iDiv);
      for (int j = 0; j < io_depthGrid[i].num(); j++)
        io_depthGrid[i][j] = FALSE;
   }

   for (FORINT i = 0; i < 4; i++) {
      int si = (i % 2) ? iMaxDiv-1 : 0;
      int sj = (i / 2) ? iMaxDiv-1 : 0;

      int skip = 1;
      int n_tot = (depthGridCi[i].num()-1) * skip;
      while (iMaxDiv-1 != n_tot) {
         skip *= 2;
         n_tot = (depthGridCi[i].num()-1) * skip;
      }
      for (int x = 0; x < depthGridCi[i].num()-1; x++) {
         for (int y = 0; y < depthGridCi[i][x].num()-1; y ++) {
            for (int j = x * skip; j < (x+1) * skip; j++)
               for (int k = y * skip; k < (y+1) * skip; k++)
                  io_depthGrid[si + j][sj + k] = depthGridCi[i][x][y];
         }
      }
   }
   if (io_depthGrid.num() < 0) 
      for (int i = 0; i < io_depthGrid.num()-1; i++) {
         for (int d = 0; d < in_iDepth; d++)
            TRACE("  ");
         for (int j = 0; j < io_depthGrid[i].num()-1; j++) {
            if (io_depthGrid[i][j] < 10)
              TRACE(" %d ", io_depthGrid[i][j]);
            else         
              TRACE("%d ", io_depthGrid[i][j]);
         }
         TRACE("\n");
      }
   
}


bool SRFDraw::Checker_adjust_depth(const DepthGrid &in_depthGrid, 
                                   DepthGrid &out_depthGridNew, 
                                   int in_dSu, int in_dEu, int in_dSv, int in_dEv, 
                                   int in_iDepth)
{
   bool bSplit = FALSE;
   bool out_bChanged = FALSE;

//   for (int i = 0; i < in_iDepth; i++)
//     TRACE("  ");
//   TRACE("Split %d %d %d %d  \n", in_dSu, in_dEu, in_dSv, in_dEv);
   int i;
   
   if (in_depthGrid[in_dSu][in_dSv] > in_iDepth) {
      bSplit = TRUE;
   } else {
      if (in_dSv != 0)
        for (i = in_dSu; i < in_dEu; i++) 
          if (in_depthGrid[i][in_dSv-1] > in_iDepth + 1) bSplit = TRUE;
      if (in_dEv != in_depthGrid.num())
        for (i = in_dSu; i < in_dEu; i++) 
          if (in_depthGrid[i][in_dEv] > in_iDepth + 1) bSplit = TRUE;
      if (in_dSu != 0)
        for (i = in_dSv; i < in_dEv; i++) 
          if (in_depthGrid[in_dSu-1][i] > in_iDepth + 1) bSplit = TRUE;
      if (in_dEu != in_depthGrid.num())
        for (i = in_dSv; i < in_dEv; i++) 
          if (in_depthGrid[in_dEu][i] > in_iDepth + 1) bSplit = TRUE;
      
      if (bSplit == TRUE) 
        out_bChanged = TRUE;
   }

   if (bSplit == FALSE) {
      out_depthGridNew.need(2);
      out_depthGridNew[0].need(2);
      out_depthGridNew[1].need(2);
      
      out_depthGridNew[0][0] = in_iDepth;

      return out_bChanged;
   }

   /*   c1   c2
    *   c3   c4
    */
   DepthGrid depthGridCi[4];
   bool bChanged[4];
   bChanged[0] = Checker_adjust_depth(in_depthGrid, depthGridCi[0], 
                                      in_dSu, (int) (0.5 * (in_dSu + in_dEu) ), 
                                      in_dSv, (int) (0.5 * (in_dSv + in_dEv) ), 
                                      in_iDepth+1);
   bChanged[1] = Checker_adjust_depth(in_depthGrid, depthGridCi[1], 
                                      (int) (0.5 * (in_dSu + in_dEu) ), in_dEu, 
                                      in_dSv, (int) (0.5 * (in_dSv + in_dEv) ), 
                                      in_iDepth+1);
   bChanged[2] = Checker_adjust_depth(in_depthGrid, depthGridCi[2], 
                                      in_dSu, (int) (0.5 * (in_dSu + in_dEu) ), 
                                      (int) (0.5 * (in_dSv + in_dEv)), in_dEv , 
                                      in_iDepth+1);
   bChanged[3] = Checker_adjust_depth(in_depthGrid, depthGridCi[3], 
                                      (int) (0.5 * (in_dSu + in_dEu) ), in_dEu, 
                                      (int) (0.5 * (in_dSv + in_dEv) ), in_dEv, 
                                      in_iDepth+1);

   int iMaxDiv = 0;
   for (i = 0; i < 4; i++) {
      iMaxDiv = WINmax(iMaxDiv, depthGridCi[i].num());
      if (bChanged[i] == TRUE)
        out_bChanged = TRUE;
   }

   int iDiv = iMaxDiv + iMaxDiv-1;
   out_depthGridNew.need(iDiv);
   for (i = 0; i < out_depthGridNew.num(); i++) {
      out_depthGridNew[i].need(iDiv);
      for (int j = 0; j < out_depthGridNew[i].num(); j++)
        out_depthGridNew[i][j] = FALSE;
   }

   for (i = 0; i < 4; i++) {
      int si = (i % 2) ? iMaxDiv-1 : 0;
      int sj = (i / 2) ? iMaxDiv-1 : 0;

      int skip = 1;
      int n_tot = (depthGridCi[i].num()-1) * skip;
      while (iMaxDiv-1 != n_tot) {
         skip *= 2;
         n_tot = (depthGridCi[i].num()-1) * skip;
      }
      for (int x = 0; x < depthGridCi[i].num()-1; x++) {
         for (int y = 0; y < depthGridCi[i][x].num()-1; y ++) {
            for (int j = x * skip; j < (x+1) * skip; j++)
               for (int k = y * skip; k < (y+1) * skip; k++)
                  out_depthGridNew[si + j][sj + k] = depthGridCi[i][x][y];
         }
      }
   }
   if (out_depthGridNew.num() < 0)
      for (i = 0; i < out_depthGridNew.num()-1; i++) {
         for (int d = 0; d < in_iDepth; d++)
            TRACE("  ");
         for (int j = 0; j < out_depthGridNew[i].num()-1; j++) {
            if (out_depthGridNew[i][j] < 10)
              TRACE(" %d ", out_depthGridNew[i][j]);
            else         
              TRACE("%d ", out_depthGridNew[i][j]);
         }
         TRACE("\n");
      }
   return out_bChanged;
}


void
SRFDraw::Checker_sample(const DepthGrid &in_depthGrid,
                        BoolGrid &io_boolGrid,
                        int in_dSu, int in_dEu, int in_dSv, int in_dEv, 
                        int in_iDepth)
{
   bool bSplit  =  (in_depthGrid[in_dSu][in_dSv] > in_iDepth) ? TRUE : FALSE;
   if (bSplit == FALSE) {
      io_boolGrid.need(2);
      io_boolGrid[0].need(2);
      io_boolGrid[1].need(2);

      io_boolGrid[0][0] = TRUE;
      io_boolGrid[1][0] = TRUE;
      io_boolGrid[1][1] = TRUE;
      io_boolGrid[0][1] = TRUE;

      return;
   }

   /*   c1   c2
    *   c3   c4
    */
   BoolGrid depthGridCi[4];
   Checker_sample(in_depthGrid, depthGridCi[0], in_dSu, (int) (0.5 * (in_dSu + in_dEu)), in_dSv, (int) (0.5 * (in_dSv + in_dEv)), in_iDepth+1);
   Checker_sample(in_depthGrid, depthGridCi[1], (int) (0.5 * (in_dSu + in_dEu)), in_dEu, in_dSv,(int) ( 0.5 * (in_dSv + in_dEv)), in_iDepth+1);
   Checker_sample(in_depthGrid, depthGridCi[2], in_dSu, (int) (0.5 * (in_dSu + in_dEu)), (int) (0.5 * (in_dSv + in_dEv)), in_dEv, in_iDepth+1);
   Checker_sample(in_depthGrid, depthGridCi[3], (int) (0.5 * (in_dSu + in_dEu)), in_dEu, (int) (0.5 * (in_dSv + in_dEv)), in_dEv, in_iDepth+1);

   int iMaxDiv = 0;
   for (int i = 0; i < 4; i++) iMaxDiv = WINmax(iMaxDiv, depthGridCi[i].num());

   int iDiv = iMaxDiv + iMaxDiv-1;
   io_boolGrid.need(iDiv);
   for (FORINT i = 0; i < io_boolGrid.num(); i++) {
      io_boolGrid[i].need(iDiv);
      for (int j = 0; j < io_boolGrid[i].num(); j++)
        io_boolGrid[i][j] = FALSE;
   }

   for (FORINT i = 0; i < 4; i++) {
      int si = (i % 2) ? iMaxDiv-1 : 0;
      int sj = (i / 2) ? iMaxDiv-1 : 0;

      int skip = 1;
      int n_tot = (depthGridCi[i].num()-1) * skip;
      while (iMaxDiv-1 != n_tot) {
         skip *= 2;
         n_tot = (depthGridCi[i].num()-1) * skip;
      }
      for (int x = 0; x < depthGridCi[i].num(); x++) {
         for (int y = 0; y < depthGridCi[i][x].num(); y ++) {
            if (depthGridCi[i][x][y] == TRUE) {
               io_boolGrid[si + x * skip][sj + y * skip] = TRUE;
            }
         }
      }
   }
   if (io_boolGrid.num() < 0) 
      for (int i = 0; i < io_boolGrid.num(); i++) {
         for (int d = 0; d < in_iDepth; d++)
            TRACE("  ");
         for (int j = 0; j < io_boolGrid[i].num(); j++) {
            if (io_boolGrid[i][j]) TRACE("1 ");
            else              TRACE("0 ");
         }
         TRACE("\n");
      }

   return;
}


void SRFDraw::Write_mesh()
{
    char fname[512];
    if (m_iSrfIndex != -1) {
        ofstream out;
        sprintf(fname, "%sTest/centered_patch_no_trim%d.m", WINDataHome("//"), m_iSrfIndex);
        WINWriteBinary( fname, out );

        if (out.good()) m_mesh.write(out);
        out.close();
    }
}


int SRFDraw::m_iSrfIndex = -1;

void SRFDraw::Start_check_sample()
{
   double su = Srf().Start_u();
   double eu = Srf().End_u();
   double sv = Srf().Start_v();
   double ev = Srf().End_v();
   double div_u = eu - su;
   double div_v = ev - sv;
   m_iDoubled_u = 1;
   m_iDoubled_v = 1;
   while (div_u / div_v > 1.5) {
      ev = ev + div_v;
      div_v *= 2;
      m_iDoubled_v++;
   }
   while (div_v / (div_u) > 1.5) {
      eu = eu + div_u;
      div_u *= 2;
      m_iDoubled_u++;
   }
   TRACE("Doubled u %d Doubled v %d\n", m_iDoubled_u, m_iDoubled_v);
   DepthGrid depthGrid; 
   Checker_fit_depth(depthGrid, su, eu, sv, ev, 1);
   
   DepthGrid depthGridNew; 
   int n=0;
   bool bRes = Checker_adjust_depth(depthGrid, depthGridNew, 
                                    0, depthGrid.num() -1, 
                                    0, depthGrid.num() - 1, 
                                    1);
   while (bRes == TRUE) {
      TRACE("Round %d\n", n++);
      depthGrid = depthGridNew;
      bRes = Checker_adjust_depth(depthGrid, depthGridNew,  
                                  0, depthGrid.num() -1, 
                                  0, depthGrid.num() - 1, 
                                  1);
   }
   
   BoolGrid boolGrid;
   Checker_sample(depthGridNew, boolGrid,
                  0, depthGrid.num() -1, 0, depthGrid.num() - 1, 1);
   Fill_faces(boolGrid);
   Write_mesh();
}


void SRFDraw::Fill_square(int in_iSi, int in_iSj, int in_iStep)
{
   for (int i = in_iSi+1; i < in_iSi + in_iStep; i++) {
      for (int j = in_iSj+1; j < in_iSj + in_iStep; j++) {
         if (m_VertGrid[i][j]) return;
      }
   }

   Array<Vertex> avertEdge;
   
   for (FORINT i = in_iSi; i < in_iSi + in_iStep; i++)
     if (m_VertGrid[i][in_iSj]) 
       avertEdge += m_VertGrid[i][in_iSj];
   
   for (FORINT i = in_iSj; i < in_iSj + in_iStep; i++)
     if (m_VertGrid[in_iSi+in_iStep][i]) 
       avertEdge += m_VertGrid[in_iSi + in_iStep][i];

   for (FORINT i = in_iSi+in_iStep; i > in_iSi; i--)
     if (m_VertGrid[i][in_iSj+in_iStep]) 
       avertEdge += m_VertGrid[i][in_iSj+in_iStep];
   
   for (FORINT i = in_iSj+in_iStep; i > in_iSj; i--)
     if (m_VertGrid[in_iSi][i]) 
       avertEdge += m_VertGrid[in_iSi][i];

   Fill(avertEdge);
}


void SRFDraw::Fill(Array<Vertex> &in_avertEdges)
{
   Array<Vertex> avertFace(3);

   const double dEps = 1e-7;
   Array<int> aiCorner;
   Array<bool> abIsCorner(in_avertEdges.num());
   for (int i = 0; i < in_avertEdges.num(); i++) {
//      TRACE("%f %f   ", meshUV(in_avertEdges[i]).u, meshUV(in_avertEdges[i]).v);
      abIsCorner[i] = FALSE;

      Vertex v1 = in_avertEdges.wrap(i-1);
      Vertex v2 = in_avertEdges.wrap(i);
      Vertex v3 = in_avertEdges.wrap(i+1);
//      TRACE("%d %d %d  ", mesh.vertexid(v1), 
//            m_mesh.vertexid(v2), m_mesh.vertexid(v3) );
      double d1 = fabs(SrfU(v1) - SrfU(v2));
      double d2 = fabs(SrfU(v2) - SrfU(v3));
//      TRACE("%f %f  ", d1, d2);
      if (d1 < dEps && d2 < dEps) {
//         TRACE("\n");
         continue;
      }
      d1 = fabs(SrfV(v1) - SrfV(v2));
      d2 = fabs(SrfV(v2) - SrfV(v3));
//      TRACE("%f %f  ", d1, d2);
      if (d1 < dEps && d2 < dEps) {
//         TRACE("\n");
         continue;
      }

      aiCorner += i;
      abIsCorner[i] = TRUE;
//      TRACE("  corner\n");
   }
   if (in_avertEdges.num() == 3) {
      avertFace[0] = in_avertEdges[0];
      avertFace[1] = in_avertEdges[1];
      avertFace[2] = in_avertEdges[2];
      m_mesh.createFace(avertFace);
   } else if (in_avertEdges.num() == 4) {
      int si = 0;
      for (int i = 0; i < 4; i++)
         if (abIsCorner[i] == FALSE)
            si = i;

      avertFace[0] = in_avertEdges[si];
      avertFace[1] = in_avertEdges[(si+1)%4];
      avertFace[2] = in_avertEdges[(si+2)%4];
      m_mesh.createFace(avertFace);

      avertFace[0] = in_avertEdges[(si)%4];
      avertFace[1] = in_avertEdges[(si+2)%4];
      avertFace[2] = in_avertEdges[(si+3)%4];
      m_mesh.createFace(avertFace);
   } else {

      int iDiv = (in_avertEdges.num() - aiCorner.last()) + aiCorner[0];
      int si = (aiCorner.last() + (iDiv+1) / 2) % in_avertEdges.num();
      int n = (in_avertEdges.num()+1) / 2;
      for (int i = 0; i < aiCorner.num()-1; i++) {
         if (aiCorner.wrap(i+1) - aiCorner[i] > iDiv) {
            iDiv = aiCorner.wrap(i+1) - aiCorner[i];
            si = aiCorner[i] + (iDiv+1) / 2;
         }
      }
//      TRACE("%d\n\n", si);

      Array<Vertex> avertEdge1, avertEdge2;
      for (FORINT i = si; i < si + n; i++) {
         avertEdge1 += in_avertEdges.wrap(i);
      }
      int n2 = in_avertEdges.num() - n + 2; 
      for (FORINT i = si + n - 1; i < si + n - 1 + n2; i++) {
         avertEdge2 += in_avertEdges.wrap(i);
      }

      Fill(avertEdge1);
      Fill(avertEdge2);
   }
}


void SRFDraw::Fill_faces(const BoolGrid &in_boolGrid)
{
   m_VertGrid.need(in_boolGrid.num());
   
   double su = Srf().Start_u();
   double sv = Srf().Start_v();
   double divu = Srf().End_u() - Srf().Start_u();
   double divv = Srf().End_v() - Srf().Start_v();
   divu = divu / (in_boolGrid.num() - 1);
   divv = divv / (in_boolGrid.num() - 1);
   int nu = m_VertGrid.num();
   int nv = m_VertGrid.num();
   for (int i = 1; i < m_iDoubled_u; i++) {
      divu *= 2.0;
      nu = (nu - 1) / 2 + 1;
   }
   for (FORINT i = 1; i < m_iDoubled_v; i++) {
      divv *= 2.0;
      nv = (nv - 1) / 2 + 1;
   }

   for (FORINT i = 0; i < m_VertGrid.num(); i++) {
      m_VertGrid[i].need(m_VertGrid.num());
      if (i >= nu) {
         for (int j = 0; j < m_VertGrid[i].num(); j++)
            m_VertGrid[i][j] = NULL;
         continue;
      }
      for (int j = 0; j < m_VertGrid[i].num(); j++) {
         if (j >= nv) {
            m_VertGrid[i][j] = NULL;
         } else if (in_boolGrid[i][j] == FALSE)
           m_VertGrid[i][j] = NULL;
         else 
           m_VertGrid[i][j] = New_vert(R2Pt( su + i * divu, sv + j * divv ));
      }
   }
   
   for (int iStep = 1; iStep < in_boolGrid.num() - iStep; iStep *= 2) {
      for (int i = 0; i < m_VertGrid.num() - iStep; i+=iStep) {
         for (int j = 0; j < m_VertGrid[i].num() - iStep; j+= iStep) {
            if (m_VertGrid[i][j] && m_VertGrid[i+iStep][j] &&
                m_VertGrid[i+iStep][j+iStep] && m_VertGrid[i][j+iStep]) {
               Fill_square(i, j, iStep);
            }
         }
      }
   }
}

