#include "StdAfx.H"
#include <utils/Srf_Draw.H>

void SRFDraw::Fix_bad()
{
   Array<Vertex> avertBad;
   Array<R3Pt> aptVertStarCenter;

   ForMeshVertex(m_mesh, v) {
      if (m_mesh.isBoundary(v)) {
         continue;
      }

      R2Polygon poly(m_mesh.degree(v));
      Vertex ov = m_mesh.mostClwVertex(v);
      for (int i = 0; i < poly.Num_pts(); i++) {
         poly[i] = R2Pt(SrfU(ov), SrfV(ov));
         ov = m_mesh.clwVertex(v, ov);
      }
      R2Pt ptPolyCenter = poly.Centroid();
      R2Pt ptVert(SrfU(v), SrfV(v));
      R2Vec vecDiff = ptVert - ptPolyCenter;
      ptVert = ptVert + vecDiff * 0.01;
      if (poly.Inside_winding(ptVert) == FALSE) {
        avertBad += v;
        aptVertStarCenter += R3Pt(ptPolyCenter[0], ptPolyCenter[1], 0);
      }
   } EndFor;

   for (int i = 0; i < avertBad.num(); i++) {
      Vertex v = avertBad[i];
      Vertex ov = m_mesh.mostClwVertex(v);

      R2Pt pt(0,0);
      const double div = 1.0 / m_mesh.degree(v);
      for (int j = 0; j < m_mesh.degree(v); j++) {
         pt[0] += SrfU(ov) * div;
         pt[1] += SrfV(ov) * div;
         ov = m_mesh.clwVertex(v, ov);
      }
//      set_vert(m_mesh, v, pt[0], pt[1]);
      Set_vert(m_mesh, v, R2Pt( aptVertStarCenter[i][0], aptVertStarCenter[i][1] ));
//      m_mesh.updateString(v, "rgb", "(0.8 0.8 0)");
   }

}

void SRFDraw::Make_trim(const CRVNub2TC &in_crvE0, const CRVNub2TC &in_crvE1, 
                        const CRVNub2TC &in_crvE2, const CRVNub2TC &in_crvE3)
{
   int count = 0;
   if (Length(in_crvE0.End_crv() - in_crvE1.Start_crv()) < 0.01) {
      for (count = 0; count < in_crvE0.Num_pts(); count++) {
         m_crvTrim.Pt(count) = in_crvE0.Pt(count);
      }
      for (int i = 0; i < in_crvE1.Num_pts() - 3; i++, count++) {
         m_crvTrim.Pt(count) = in_crvE1.Pt(i+3);
      }
      for (FORINT i = 0; i < in_crvE2.Num_pts() - 3; i++, count++) {
         m_crvTrim.Pt(count) = in_crvE2.Pt(i+3);
      }
      for (FORINT i = 0; i < in_crvE3.Num_pts() - 3; i++, count++) {
         m_crvTrim.Pt(count) = in_crvE3.Pt(i+3);
      }
   } else {
      for (count = 0; count < in_crvE0.Num_pts(); count++) {
         m_crvTrim.Pt(count) = in_crvE0.Pt(in_crvE0.Num_pts() - count - 1);
      }
      for (int i = 3; i < in_crvE1.Num_pts(); i++, count++) {
         m_crvTrim.Pt(count) = in_crvE1.Pt(in_crvE1.Num_pts() - i - 1);
      }
      for (FORINT i = 3; i < in_crvE2.Num_pts(); i++, count++) {
         m_crvTrim.Pt(count) = in_crvE2.Pt(in_crvE2.Num_pts() - i - 1);
      }
      for (FORINT i = 3; i < in_crvE3.Num_pts(); i++, count++) {
         m_crvTrim.Pt(count) = in_crvE3.Pt(in_crvE3.Num_pts() - i - 1);
      }
   }
   ASSERT(count == m_crvTrim.Num_pts());
}

R2Polygon Make_boundary(const CRVNub_sampled2TC &in_crv)
{
   Array<R2Pt > samples;
   in_crv.Samples(samples);
   
   Array<int> aiDups;
   double eps = 1e-6;
   for (int i = 0; i < samples.num(); i++) {
      if (LengthSq(samples[i] - samples.wrap(i+1)) < eps)
        aiDups += i;
   }
   R2Polygon poly(samples.num() - aiDups.num());
   int indx = 0;
   for (FORINT i = 0; i < poly.Num_pts(); i++, indx++) {
      while (aiDups.contains(indx)) indx++;
      poly[i] = samples[indx];
   }
   return poly;
}

void SRFDraw::Find_outside(Array<Face> &out_afaceDelete, 
                           const SRFDraw::BoolGrid &boolGridMask)
{
   Array<Face> afaceValid, afaceTryMe, afaceVisited, afaceIsInside;

   double divu = Srf().End_u() - Srf().Start_u();
   double divv = Srf().End_v() - Srf().Start_v();
   divu = divu / (boolGridMask.num() - 1);
   divv = divv / (boolGridMask.num() - 1);
   int iMidMaskU = boolGridMask.num() / 2;
   int iMidMaskV = boolGridMask.num() / 2;

   for (int i = 1; i < m_iDoubled_u; i++) {
       iMidMaskU = (int) ( iMidMaskU / 2.0 );
       divu *= 2.0;
   }
   for (FORINT i = 1; i < m_iDoubled_v; i++) {
       iMidMaskV = (int) ( iMidMaskV / 2.0 );
       divv *= 2.0;
   }

   Array<Vertex> avFace;
   int iMostMid = 0;
   ForMeshFace(m_mesh, f) {
      bool bNope = FALSE;
      bool bIsMid = FALSE;
      m_mesh.vertices(f, avFace);
      for (int i = 0; i < avFace.num(); i++) {
         Vertex v = avFace[i];
         int iU = (int) ( (SrfU(v) - Srf().Start_u()) / divu );
         int iV = (int) ( (SrfV(v) - Srf().Start_v()) / divv );
         if (abs(iU - iMidMaskU) < 5 && abs(iV - iMidMaskV) < 5)
           bIsMid = TRUE;

         if (boolGridMask[iU][iV] == FALSE) {
            bNope = TRUE;
            break;
         }
      }
      if (bNope == TRUE) continue;
      if (bNope == FALSE) {
         afaceValid += f;
         if (bIsMid) iMostMid = afaceValid.num()-1;
      }
   }EndFor;

   afaceTryMe += afaceValid[iMostMid];
   afaceIsInside += afaceValid[iMostMid];
   while (afaceTryMe.num()) {
      Face f = afaceTryMe.last();
      afaceTryMe.sub(1);

      ForFaceFace(m_mesh, f, of) {
         if (afaceTryMe.contains(of)) continue;
         if (afaceVisited.contains(of)) continue;
         if (!afaceValid.contains(of)) continue;

         afaceTryMe += of;
         afaceIsInside += of;
      } EndFor;
      afaceVisited += f;
   } 

   ForMeshFace(m_mesh, f) {
      if (!afaceIsInside.contains(f))
         out_afaceDelete += f;
   }EndFor;
}

void Mark_boundary(Array<Vertex> &out_avertBdry, GMesh &in_mesh)
{
   out_avertBdry.clearcompletely();
   Array<Vertex> avertDelete;
   ForMeshVertex(in_mesh, v) {
      if (in_mesh.degree(v) == 0) {
         avertDelete += v;
      } else if (in_mesh.isBoundary(v))
         out_avertBdry += v;
   } EndFor;

   Array<Face> afaceDelete;
   for (int i = 0; i < out_avertBdry.num(); i++) {
      Vertex v1 = in_mesh.mostClwVertex(out_avertBdry[i]);
      Vertex v2 = in_mesh.mostCcwVertex(out_avertBdry[i]);
      if (!out_avertBdry.contains(v1)) continue;
      if (!out_avertBdry.contains(v2)) continue;
      Edge e = in_mesh.queryEdge(v1, v2);
      if (e) {
         avertDelete += out_avertBdry[i];
         afaceDelete += in_mesh.mostClwFace(out_avertBdry[i]);
         out_avertBdry[i] = out_avertBdry.last();
         out_avertBdry.sub(1);
      } else {
//         in_mesh.updateString(out_avertBdry[i], "rgb", "(1 0 0)");
      }
   }

   for (FORINT i = 0; i < afaceDelete.num(); i++) 
      in_mesh.destroyFace(afaceDelete[i]);

   for (FORINT i = 0; i < avertDelete.num(); i++) 
      in_mesh.destroyVertex(avertDelete[i]);
      
}

void Fix_ts(Array<double> &io_adTs, const CRVNub_sampled2TC &in_crv)
{
   int min_i = io_adTs.min_i(io_adTs);
   int max_i = io_adTs.max_i(io_adTs);

   if ((max_i + 1) % io_adTs.num() != min_i) {
      if (fabs(io_adTs[max_i] - in_crv.End_t()) < 
          fabs(io_adTs[min_i] - in_crv.Start_t())) {
         min_i = (max_i + 1) % io_adTs.num();
      } else {
         max_i = (min_i - 1 + io_adTs.num()) % io_adTs.num();
      }
   }

   Array<int> aiBad;
   for (int i = 0; i < io_adTs.num(); i++) {
      if (i == max_i)
          continue;
      if (io_adTs[i] >= io_adTs.wrap(i+1))
         aiBad += i;
   }
   for (FORINT i = 0; i < aiBad.num(); i++) {
      int si = i;
      int ei = i+1;
      double div = 2.0;
      for (int j = ei; j < aiBad.num(); j++, ei++, div++) {
         if (aiBad[ei] != aiBad[si] + j)
            break;
         div += 1.0;
      }
      int indx1 = aiBad[si]-1;
      int indx2 = aiBad[ei-1] + 1;
      int indx = aiBad[si];
      div = 1.0 / div;
      for (FORINT j = 1; j < ei - si+1; j++, indx++)
         io_adTs.wrap(indx) = (j*div) * io_adTs.wrap(indx1) + (1.0-(j * div)) * io_adTs.wrap(indx2);
   }


}

void
SRFDraw::Sort_boundary(Array<double> &io_adTs,
                       Array<Vertex> &out_avertBdry, 
                       const CRVNub_sampled2TC &in_crv)
{
   Array<Vertex> avertSortBdry;

   avertSortBdry += out_avertBdry[0];
   for (int i = 0; i < out_avertBdry.num()-1; i++) {
      avertSortBdry += m_mesh.mostClwVertex(avertSortBdry.last());
//      m_mesh.updateString(out_avertBdry[i], "rgb", "(0.7 0.7 0.5)");
   }
   out_avertBdry = avertSortBdry;

   io_adTs.need(out_avertBdry.num());
   for (FORINT i = 0; i < out_avertBdry.num(); i++) {
      Vertex v = out_avertBdry[i];
      R2Pt pt(SrfU(v), SrfV(v));

      R2Pt Proj_pt = in_crv.Closest(pt, io_adTs[i]);
   }

   int forward = 0, backward = 0;
   for (FORINT i = 0; i < io_adTs.num(); i++) {
      if (io_adTs.wrap(i+1) > io_adTs[i]) {
         forward++; 
         backward++;
      } if (io_adTs.wrap(i+1) > io_adTs[i]) {
        forward++;
      } else if (io_adTs.wrap(i+1) < io_adTs[i]) {
         backward++;
      }
   }
   if (backward > forward) {
      out_avertBdry.reverse();
      io_adTs.reverse();
   }
   /*
   TRACE("Io_adTs ");
   for (FORINT i = 0; i < io_adTs.num(); i++) {
      if (io_adTs[i] == io_adTs[(i+1)%io_adTs.num()])
         TRACE("!");
      if (io_adTs[i] > io_adTs[(i+1)%io_adTs.num()])
         TRACE("@");
      TRACE("%f ", io_adTs[i]);
   }
*/
} 

int 
SRFDraw::Fix_boundary(Array<Vertex> &out_avertBdry, 
                      const CRVNub_sampled2TC &in_crv, 
                      bool bAll)
{
   Array<double> ts;

   Sort_boundary(ts, out_avertBdry, in_crv);

   int max_i = 0, min_i = 0;
   for (int i = 0; i < ts.num(); i++) {
      if (ts[i] < ts[min_i]) min_i = i;
      if (ts[i] > ts[max_i]) max_i = i;
   }
   int n_fixed = 0;

   double new_t;
   if ((max_i + 1) % ts.num() != min_i) {
      if (fabs(ts[max_i] - in_crv.End_t()) < fabs(ts[min_i] - in_crv.Start_t())) {
         Vertex bv = out_avertBdry[min_i];
         min_i = (max_i + 1) % ts.num();
         double u = SrfU(out_avertBdry[min_i]);
         double v = SrfV(out_avertBdry[min_i]);
         double u1 = SrfU(bv);
         double v1 = SrfV(bv);

         Set_vert(m_mesh, out_avertBdry[min_i], Lerp( R2Pt( u,v ), R2Pt(u1,v1), 0.5 ) );


         R2Pt pt(SrfU(out_avertBdry[min_i]), SrfV(out_avertBdry[min_i]));
         R2Pt proj_pt = in_crv.Closest(pt, new_t);
         if (new_t != ts[min_i]) {
            n_fixed++;
            ts[min_i] = new_t;
         }
      } else {
         Vertex bv = out_avertBdry[max_i];
         max_i = (min_i - 1 + ts.num()) % ts.num();
         double u = SrfU(out_avertBdry[max_i]);
         double v = SrfV(out_avertBdry[max_i]);
         double u1 = SrfU(bv);
         double v1 = SrfV(bv);

         Set_vert(m_mesh, out_avertBdry[max_i], Lerp( R2Pt( u,v ), R2Pt(u1,v1), 0.5 ) );
         R2Pt pt(SrfU(out_avertBdry[max_i]), SrfV(out_avertBdry[max_i]));
         R2Pt proj_pt = in_crv.Closest(pt, new_t);
         if (new_t != ts[max_i]) {
            n_fixed++;
            ts[max_i] = new_t;
         }
      }
   }

   for (FORINT i = 0; i < ts.num(); i++) {
      if (ts[i] < ts[(i+1)%ts.num()]) continue;
      if (i == max_i)continue;

      int indx1 = i;
      int indx2 = (indx1 +2) % ts.num();
      int indx = (indx1 + 1) % ts.num();

//      mesh.updateString(out_avertBdry[indx1], "rgb", "(1 0 0)");
//      mesh.updateString(out_avertBdry[indx2], "rgb", "(1 0 1)");
//      mesh.updateString(out_avertBdry[indx], "rgb", "(1 1 1)");
   }

   for (FORINT i = 0; i < ts.num(); i++) {
      int indx1 = i;
      int indx2 = (indx1 + 1) % ts.num();
      int indx = 0;
      if (i == max_i)continue;
      if (ts[indx1] == ts[indx2] || (bAll && ts[indx1] > ts[indx2])) {
         R2Pt pt(SrfU(out_avertBdry[indx1]), SrfV(out_avertBdry[indx1]));

         double new_t = 0;
         R2Pt proj_pt = in_crv.Closest(pt, new_t);
         double d1 = LengthSq(proj_pt - pt);

         pt = R2Pt(SrfU(out_avertBdry[indx2]), SrfV(out_avertBdry[indx2]));
         proj_pt = in_crv.Closest(pt, new_t);
         double d2 = LengthSq(proj_pt - pt);

         if (bAll) {
            indx = indx2;
            indx2 = (indx2 + 1) % ts.num();
         } else if (d1 < d2) {
            indx = indx2;
            indx2 = (indx2 + 1) % ts.num();
         } else {
            indx = indx1;
            indx1 = (indx + ts.num() - 1) % ts.num();
         }
         double u = SrfU(out_avertBdry[indx]);
         double v = SrfV(out_avertBdry[indx]);
         double u1 = SrfU(out_avertBdry[indx1]);
         double u2 = SrfU(out_avertBdry[indx2]);
         double v1 = SrfV(out_avertBdry[indx1]);
         double v2 = SrfV(out_avertBdry[indx2]);

         Set_vert(m_mesh, out_avertBdry[indx], Lerp( R2Pt( u,v ), R2Pt(u1,v1), 0.5 ) );
         pt = R2Pt(SrfU(out_avertBdry[indx]), SrfV(out_avertBdry[indx]));
         proj_pt = in_crv.Closest(pt, new_t);
         if (new_t != ts[indx]) {
            n_fixed++;
            ts[indx] = new_t;
         }
//         m_mesh.updateString(out_avertBdry[indx], "rgb", "(0.2 0.2 0.7)");
      }
   }
/*
   TRACE("Ts ");
   for (FORINT i = 0; i < ts.num(); i++) {
      if (ts[i] == ts[(i+1)%ts.num()])
         TRACE("!");
      if (ts[i] > ts[(i+1)%ts.num()])
         TRACE("@");
      TRACE("%f ", ts[i]);
   }

   TRACE("\n");
   */
   return n_fixed;
         
}

// Attach faces between the inside, clipped mesh and the 
// vertices (in_avertBdry) along the bdry
void SRFDraw::Fill_boundary(const Array<double> &in_adTs,
                            const Array<Vertex> &in_avertBdry, 
                            const Array<double> &in_adCorners, 
                            const CRVNub_sampled2TC &in_aCrvTrim)
{
   Vertex last_v = NULL, first_v = NULL;
   Vertex v1, v2, v3;
   Array<Vertex> vs(3);
   Edge e = m_mesh.queryEdge(in_avertBdry[0], in_avertBdry[1]);
   ASSERT(e);

   // Reverse the face before adding, y/n
   const bool bReverse =  (m_mesh.face1(e)) ? TRUE : FALSE;
   // For determining itty bitty edges
   const double dCrvDomainLen = in_aCrvTrim.DomainLength() / in_avertBdry.num();

   double last_t = in_adTs[0];
   for (int i = 0; i < in_avertBdry.num(); i++) {
      double t1 = in_adTs[i];
      double t3 = in_adTs.wrap(i+1);
      double t2 = 0.5 * (t1 + t3);
      
      // Wrapped around
      if (t1 > t3) {
         t2 = in_aCrvTrim.Start_t();
      }
      // See if we bracket a corner, in which case, set t2 to be a corner

      WINbool bCorner = FALSE;
      for (int j = 0; j < in_adCorners.num(); j++) {
         if (t1 < in_adCorners[j] && t3 > in_adCorners[j]) {
            t2 = in_adCorners[j];
            bCorner = TRUE;
         } else if (t1 > t3 && (in_adCorners[j] > t1 || in_adCorners[j] < t3)) {
            t2 = in_adCorners[j];
            bCorner = TRUE;
         }
      }

      R2Pt uv1 = in_aCrvTrim(t1);
      R2Pt uv2 = in_aCrvTrim(t2);
      R2Pt uv3 = in_aCrvTrim(t3);

      // See if we should add the usual 3 faces, or if we should collapse
      // the t1,t3 vertices and add just one face
      if (fabs(last_t - t3) < 0.1 * dCrvDomainLen) {
         Srf().Clip_uv(uv1);
         v1 = (i == 0) ? New_vert(uv1) : last_v;
         v2 = v1;
         v3 = (i == in_avertBdry.num()-1) ? first_v : v1;
      } else {
         Srf().Clip_uv(uv1);
         Srf().Clip_uv(uv3);
         Srf().Clip_uv(uv2);
         v1 = (i == 0) ? New_vert(uv1) : last_v;
         v2 = New_vert(uv2);
         v3 = (i == in_avertBdry.num()-1) ? first_v : New_vert(uv3);
         last_t = t3;
      }

//      if (bCorner)
//         m_mesh.updateString(v2, "rgb", "(1 0 1)");


      last_v = v3;
      if (i == 0) first_v = v1;

      const R3Pt &p1 = m_mesh.point(in_avertBdry[i]);
      const R3Pt &p2 = m_mesh.point(in_avertBdry[(i+1)%in_avertBdry.num()]);
      R3Vec vec1 = p1 - p2;
      R3Vec vec2 = (p2 + vec1 * 0.5) - m_mesh.point(v2);
      vec1 = UnitSafe(vec1);
      vec2 = UnitSafe( vec2 );
      // If this triangle is really askew, shift the points 
      if (fabs(Dot(vec1, vec2)) > 0.9) {
         double u1 = SrfU(in_avertBdry[i]);
         double u2 = SrfU(in_avertBdry.wrap(i+2));
         double v1 = SrfV(in_avertBdry[i]);
         double v2 = SrfV(in_avertBdry.wrap(i+2));
         Set_vert(m_mesh, in_avertBdry.wrap(i+1), R2Pt( 0.5 * (u1 + u2), 0.5 * (v1 + v2)) );
      }

      if (v1 == v3) {
         vs[0] = in_avertBdry[i];
         vs[1] = v2;
         vs[2] = in_avertBdry.wrap(i+1);
         if (bReverse) vs.reverse();
         m_mesh.createFace(vs);
      } else if (v1 == v2) {
         vs[0] = in_avertBdry[i];
         vs[1] = v1;
         vs[2] = v3;
         if (bReverse) vs.reverse();
         m_mesh.createFace(vs);

         vs[0] = in_avertBdry[i];
         vs[1] = v3;
         vs[2] = in_avertBdry.wrap(i+1);
         if (bReverse) vs.reverse();
         m_mesh.createFace(vs);
      } else {
         vs[0] = in_avertBdry[i];
         vs[1] = v1;
         vs[2] = v2;
         if (bReverse) vs.reverse();
         m_mesh.createFace(vs);

         vs[0] = in_avertBdry[i];
         vs[1] = v2;
         vs[2] = in_avertBdry.wrap(i+1);
         if (bReverse) vs.reverse();
         m_mesh.createFace(vs);

         vs[0] = v2;
         vs[1] = v3;
         vs[2] = in_avertBdry.wrap(i+1);
         if (bReverse) vs.reverse();
         m_mesh.createFace(vs);
      }
   }
//   m_mesh.updateString(first_v, "rgb", "(1 1 1)");
}

void SRFDraw::Flood_fill(const SRFDraw::BoolGrid &in_boolGridBdry, 
                         SRFDraw::BoolGrid &out_boolGridFillXY)
{
   BoolGrid boolGridFillYX(m_VertGrid.num());
   
   for (int i = 0; i < boolGridFillYX.num(); i++) {
      boolGridFillYX[i].need(m_VertGrid.num());
      for (int j = 0; j < boolGridFillYX[i].num(); j++) {
         boolGridFillYX[i][j] = FALSE;
      }
   }
   
   for (int x = 0; x < out_boolGridFillXY.num(); x++) {
      bool is_inside = FALSE;
      bool in_bdry = in_boolGridBdry[0][x];
      for (int y = 0; y < out_boolGridFillXY.num(); y++) {
         if (in_bdry == TRUE && in_boolGridBdry[y][x] == FALSE) {
            if (is_inside == FALSE) {
               is_inside = TRUE;
            } else {
               is_inside = FALSE;
            }
            
         }
         if (is_inside == TRUE && in_boolGridBdry[y][x] == FALSE)
           boolGridFillYX[y][x] = TRUE; 
         in_bdry = in_boolGridBdry[y][x];
      }
   }

   for (FORINT x = 0; x < out_boolGridFillXY.num(); x++) {
      bool is_inside = FALSE;
      bool in_bdry = in_boolGridBdry[x][0];
      for (int y = 0; y < out_boolGridFillXY.num(); y++) {
         if (in_bdry == TRUE && in_boolGridBdry[x][y] == FALSE) {
            if (is_inside == FALSE) {
               is_inside = TRUE;
            } else {
               is_inside = FALSE;
            }
            
         }
         if (is_inside == TRUE && in_boolGridBdry[x][y] == FALSE)
           out_boolGridFillXY[x][y] = TRUE; 
         in_bdry = in_boolGridBdry[x][y];
      }
   }
   Check_inside(in_boolGridBdry, out_boolGridFillXY, boolGridFillYX);
}

void SRFDraw::Check_inside(const BoolGrid &in_boolGridBdry, 
                           BoolGrid &in_boolGridFillXY, 
                           BoolGrid &in_boolGridFillYX)
{
   const int boolGridNum = in_boolGridBdry.num();

   for (int i = 0; i < boolGridNum; i++) {
      for (int j = 0; j < boolGridNum; j++) {
         if (in_boolGridFillXY[i][j] == in_boolGridFillYX[i][j]) continue;
         
         if (i == 0 || j == 0 || i == boolGridNum-1 || j == boolGridNum-1) {
            in_boolGridFillXY[i][j] = in_boolGridBdry[i][j];
            continue;
         }
         bool bNextBdry = FALSE;
         bool bNextInside = FALSE;
         for (int k = -1; k < 2; k++) {
            for (int l = -1; l < 2; l++) {
               if (in_boolGridFillXY[i+k][j+l] == TRUE && 
                   in_boolGridFillYX[i+k][j+l] == TRUE) bNextInside = TRUE;
               if (in_boolGridBdry[i+k][j+l] == TRUE)   bNextBdry = TRUE;
            }
         }
         if (bNextBdry == FALSE && bNextInside == TRUE) {
            in_boolGridFillXY[i][j] = TRUE;
         } else {
            in_boolGridFillXY[i][j] = FALSE;
         }
      }
   }
}

void SRFDraw::Make_draw_crv(CRVNub_sampled2TC &out_crvSampled,
                            CRVNub3TC &out_crvDraw)
{
    Array<R2Pt > samples;
    
    out_crvSampled.Samples(samples);
    CRVNub3TC _dcrv((CRVSpace) out_crvDraw, samples.num());
    for (int i = 0; i < samples.num(); i++) {
        //      TRACE("%f %f\n", samples[i][0], samples[i][1]);
        _dcrv.Pt(i) =  Srf()(samples[i]);
    }
    out_crvDraw = _dcrv;
    //   TRACE("\n");
}
 
void SRFDraw::Make_bdry_inside_grid(BoolGrid &io_boolGridInside, 
                                    BoolGrid &io_boolGridBdry,
                                    const R2Polygon &poly)
{
   io_boolGridInside.need(m_VertGrid.num());
   io_boolGridBdry.need(m_VertGrid.num());

   for (int i = 0; i < m_VertGrid.num(); i++) {
      io_boolGridInside[i].need(m_VertGrid.num());
      io_boolGridBdry[i].need(m_VertGrid.num());
      for (int j = 0; j < io_boolGridInside[i].num(); j++) {
         io_boolGridInside[i][j] = FALSE;
         io_boolGridBdry[i][j] = FALSE;
      }
   }
   double divu = Srf().End_u() - Srf().Start_u();
   double divv = Srf().End_v() - Srf().Start_v();
   int nu = m_VertGrid.num() - 1;
   int nv = m_VertGrid.num() - 1;
   for (FORINT i = 1; i < m_iDoubled_u; i++) 
     nu = nu / 2;
   for (FORINT i = 1; i < m_iDoubled_v; i++) 
     nv = nv / 2;

   divu = divu / nu;
   divv = divv / nv;

   int last_u = (int) ( (poly[0][0] - Srf().Start_u()) / divu );
   int last_v = (int) ( (poly[0][1] - Srf().Start_v()) / divv );
   for (FORINT i = 1; i < poly.Num_pts(); i++) {
      const R2Pt &pt = poly[i];
      int u = (int) ( (pt[0] - Srf().Start_u()) / divu );
      int v = (int) ( (pt[1] - Srf().Start_v()) / divv );
      int n_steps = WINmax(abs(u - last_u), abs(v - last_v));

      if (n_steps == 0) continue;
      for (int n = 0; n < n_steps; n++) {
         double div = (double) n / (double) n_steps;
         int su = (int) ( ((1.0 - div) * last_u + div * u) + 0.5 );
         int sv = (int) ( ((1.0 - div) * last_v + div * v) + 0.5 );
         //TRACE("%d %d\n", su, sv);
         for (int x = su; x < su+2; x++) {
            if (x < 0 || x >= io_boolGridBdry.num()) continue;
            for (int y = sv; y < sv+2; y++) {
               if (y < 0 || y >= io_boolGridBdry.num()) continue;
               io_boolGridBdry[x][y] = TRUE;
            }
         }
      }
      last_u = u;
      last_v = v;
   }
}

void SRFDraw::Clip(CRVNub3TC &out_crvDraw)
{
   CRVNub_sampled2TC crv_sampled(m_crvTrim, 0.5, 1e30);
   Make_draw_crv(crv_sampled, out_crvDraw);

   Array<Vertex> avertOut, avertClose;
   R2Polygon poly = Make_boundary(crv_sampled);

   BoolGrid inside_grid, bdry_grid;
   Make_bdry_inside_grid(inside_grid, bdry_grid, poly);

   Flood_fill(bdry_grid, inside_grid);
//   check_inside(bdry_grid, inside_grid);
   avertOut.clearcompletely();
   avertClose.clearcompletely();
   for (int i = 0; i < inside_grid.num(); i++)
      for (int j = 0; j < inside_grid.num(); j++)
         if (inside_grid[i][j] == FALSE && m_VertGrid[i][j]) {
            avertOut += m_VertGrid[i][j];
         } else if (bdry_grid[i][j] && m_VertGrid[i][j]) {
            avertClose += m_VertGrid[i][j];
         }
/*
   for (FORINT i = 0; i < avertOut.num(); i++)
      m_mesh.updateString(avertOut[i], "rgb", "(1 0 1)");

   for (FORINT i = 0; i < avertClose.num(); i++)
      m_mesh.updateString(avertClose[i], "rgb", "(0 1 1)");
*/
   Array<Face> afaceDelete;
   Find_outside(afaceDelete, inside_grid);
      Write_mesh();

   for (FORINT i = 0; i < afaceDelete.num(); i++) {
      m_mesh.destroyFace(afaceDelete[i]);
   }
   Array<Vertex> bdry;
   Mark_boundary(bdry, m_mesh);

   Array<double> ts;
   while (Fix_boundary(bdry, crv_sampled, FALSE)) {
      Write_mesh();
   }
   int count = 0;
   while (Fix_boundary(bdry, crv_sampled, TRUE)) {
      Write_mesh();
      count++;
      if (count > 15) break;
   }
   Sort_boundary(ts, bdry, crv_sampled);
   Fix_ts(ts, crv_sampled);

   Fill_boundary(ts, bdry, m_adCorners, crv_sampled);
   Write_mesh();

   Fix_bad();
   m_mesh.renumber();

}

void SRFDraw::Make_corners()
{
   for (int i = 0; i < m_crvTrim.Num_pts() - 3; i++) {
      if (m_crvTrim.Pt(i) == m_crvTrim.Pt(i+1) && m_crvTrim.Pt(i) == m_crvTrim.Pt(i+2)) {
         m_adCorners += m_crvTrim.Knot(i+m_crvTrim.Order() + 1);
         if (m_adCorners.last() <= m_crvTrim.Start_t())
            m_adCorners.last() = m_crvTrim.Start_t() + 1.0e-6;
         if (m_adCorners.last() >= m_crvTrim.End_t())
            m_adCorners.last() = m_crvTrim.End_t() - 1.0e-6;
//         TRACE("%f %f  %f %f\n", m_crvTrim.Pt(i)[0], m_crvTrim.Pt(i)[1], 
//               m_crvTrim(m_adCorners.last())[0], m_crvTrim(m_adCorners.last())[1]);
      }
   }
//   TRACE("\n");
}
