#include "StdAfx.H"
#include <utils/Rn_Polygon.H>
#include <utils/Srf_SPatch.H>
#include <utils/Pm_MeshLite.H>
#include <FAD/fad.H>

static const R2Polygon UnitPolygon( const int in_iNSides )
{
    static Array<R2Polygon> s_apoly;

    if ( in_iNSides >= s_apoly.num() ) {
        int iOld = s_apoly.num();

        s_apoly.add( in_iNSides + 1 - iOld );
        for ( int i = WINmax( iOld, 3 ); i < s_apoly.num(); i++ ) {
            s_apoly[i] = R2Polygon::Unit_polygon( i );
        }
    }

    ASSERT( in_iNSides >= 3 );
    return s_apoly[in_iNSides];
}

typedef Array< Array<int> > IndexType;
typedef Array<int >         FactType;
static Array< Array< IndexType * > > s_aaaaiIndices;
static Array< Array< FactType * > >  s_aaaiFact;
static const Array< Array<int> > &Index( const int in_iNSides, const int in_iDepth )
{


    if ( s_aaaaiIndices.num() <= in_iNSides ) {
        s_aaaaiIndices.add( in_iNSides + 1 - s_aaaaiIndices.num() );
        s_aaaiFact.add( in_iNSides + 1 - s_aaaiFact.num() );
    }

    if ( s_aaaaiIndices[in_iNSides].num() <= in_iDepth ) {
        int iOld = s_aaaaiIndices[in_iNSides].num();
        s_aaaaiIndices[in_iNSides].add( in_iDepth + 1 - iOld );
        s_aaaiFact[in_iNSides].add( in_iDepth + 1 - iOld );

        for ( int i = iOld; i < s_aaaaiIndices[in_iNSides].num(); i++ ) {
            Array< Array<int> > *aaip = new Array< Array<int> >;
            Array< int >        *aipFact = new Array<int>;
            Array< Array<int> > &aai = *aaip;
            Array< int >        &aiFact = *aipFact;
            Array<int> ai( in_iNSides );
            ai.memfill(0);

            s_aaaaiIndices[in_iNSides][i] = aaip;
            s_aaaiFact[in_iNSides][i] = aipFact;

            WINbool bDone = FALSE;
            while ( bDone == FALSE ) {
                int iSum = 0;
                for ( int j = 0; j < ai.num(); j++ ) {
                    iSum += ai[j];
                }
                
                if ( iSum == i ) {
                    if ( !aai.contains( ai ) ) {
                        aai += ai;
                        int iFact = 1;
                        for ( int k = 1; k <= i; k++ )
                            iFact = iFact * k;

                        for ( FORINT k = 0; k < ai.num(); k++ ) {
                            int iDiv = 1;
                            for ( int l = 2; l <= ai[k]; l++ )
                                iDiv = iDiv * l;

                            iFact = iFact / iDiv;
                        }

                        aiFact += iFact;
                    }
                }

                bDone = TRUE;
                for ( FORINT j = 0; j < ai.num(); j++ ) {
                    ai[j] = ai[j] + 1;
                    if ( ai[j] > i ) {
                        ai[j] = 0;
                    } else {
                        bDone = FALSE;
                        break;
                    }
                }
            }
        }
    }

    return *s_aaaaiIndices[in_iNSides][in_iDepth];
}

static double Area( const R2Pt &in_p1, 
             const R2Pt &in_p2, 
             const R2Pt &in_p3 )
{
    return 0.5 * fabs( ( in_p2[0] - in_p1[0] ) * ( in_p3[1] - in_p1[1] ) - ( in_p2[1] - in_p1[1] ) * ( in_p3[0] - in_p1[0] ) );
}

template<class T>
T Area( const R2PointTC<T> &in_p1, 
        const R2Pt &in_p2, 
        const R2Pt &in_p3 )
{
    const T dCross = ( in_p2[0] - in_p1[0] ) * ( in_p3[1] - in_p1[1] ) - ( in_p2[1] - in_p1[1] ) * ( in_p3[0] - in_p1[0] );
    if ( dCross.x() < 0.0 )
        return - 0.5 * dCross;
    return 0.5 * dCross;
}

WINbool IsInside( const R2Polygon &in_poly,
                  const int in_iTri,
                  const R2Pt &in_p )
{
    int aiIndx[3];
    aiIndx[0] = in_iTri;
    aiIndx[1] = (in_iTri + 1) % in_poly.Num_pts();
    aiIndx[2] = (in_iTri + 2) % in_poly.Num_pts();

    for ( int i = 0; i < 3; i++ ) {
        const R2Vec v1 = in_poly[ aiIndx[i] ] - in_p;
        const R2Vec e1 = in_poly[ aiIndx[i] ] - in_poly[ aiIndx[ (i+1)%3 ] ];
        const double dCross = v1[0] * e1[1] - v1[1] * e1[0];

        if ( dCross < 0 )
            return FALSE;
    } 
    return TRUE;
}

WINbool IsInside( const R2Polygon &in_poly,
                  const R2Pt &in_p )
{
    return in_poly.InsideQuick( in_p );
}


template<class T>
WINbool IsInside( const R2Polygon &in_poly,
                  const R2PointTC<T> &in_p )
{
    return IsInside( in_poly, R2Pt( in_p[0].x(), in_p[1].x() ) );
}

/*
 * Area of triangle p,c1,c2 over area of triangle c1,c2,c3
 */
template<class T>
T Alpha( const R2Polygon    & in_poly,
         const int            in_i, 
         const R2PointTC<T> & in_pt ) 
{
    const T dAreaPPiPi1 = Area( in_pt, in_poly.PtWrap(in_i), in_poly.PtWrap(in_i+1) );
    const double dAreaPiPi1Pi2 = Area( in_poly.PtWrap(in_i), in_poly.PtWrap(in_i+1), in_poly.PtWrap(in_i+2) );

    return dAreaPPiPi1 / dAreaPiPi1Pi2;
}

/*
 * a1 a2 .. an, leavling out a i and ai-1
 */
template<class T>
void Pi( const R2Polygon    & in_poly,
         const R2PointTC<T> & in_pt,
         Array<T> &out_adPi ) 
{
    double dSign = 1.0;
    if ( IsInside( in_poly, in_pt ) == FALSE )
        dSign = -1.0;

    Array<T> adAlpha( out_adPi.num() );
    if ( adAlpha.num() == 3 ) {
        const double dAreaPiPi1Pi2 = Area( in_poly.PtWrap(0), in_poly.PtWrap(1), in_poly.PtWrap(2) );
        for ( int i = 0; i < adAlpha.num(); i++ ) {
            adAlpha[i] = dSign * Area( in_pt, in_poly.PtWrap(i), in_poly.PtWrap(i+1) ) / dAreaPiPi1Pi2;
        }
    } else {
        for ( int i = 0; i < adAlpha.num(); i++ ) {
            adAlpha[i] = dSign * Alpha( in_poly, i, in_pt );
        }
    }

    out_adPi.need( adAlpha.num() );

    for ( int i = 0; i < out_adPi.num(); i++ ) {
        out_adPi[i] = 1.0;

        for ( int j = 1; j < out_adPi.num() - 1; j++ ) {
            out_adPi[i] = out_adPi[i] * adAlpha.wrap( i + j );
        }
    }
}

/*
 * pi / sum pi
 */
template<class T>
void Li( const R2Polygon    & in_poly,
         const R2PointTC<T> & in_pt,
         Array<T>           & out_adLi ) 
{
    T dSum = 0.0;

    Array<T> adPi( in_poly.Num_pts() );

    Pi( in_poly, in_pt, adPi );

    if ( in_poly.Num_pts() != 3 ) {
        for ( int i = 0; i < adPi.num(); i++ ) {
            dSum = dSum + adPi[i];
        }
    } else {
        dSum = 1.0;
    }

    for ( int i = 0; i < adPi.num(); i++ ) {
        out_adLi[i] = adPi[i] / dSum;
    }
}

template<class T>
void Basis( const R2Polygon              & in_poly,
            const Array< R2PointTC<T> >  & in_apt,
            const Array< Array<int> >    & in_aaiIndex,
            const Array< int >           & in_aiFact,
            Array< T >                   & out_adBasis )
{

    out_adBasis.need( in_aaiIndex.num() );
    for ( int i = 0; i < out_adBasis.num(); i++ ) {
        Array<T> adLi( in_poly.Num_pts() );
        Li( in_poly, in_apt[i], adLi );

        out_adBasis[i] = in_aiFact[i];
        for ( int d = 0; d < in_aaiIndex[i].num(); d++ ) {
            if ( in_aaiIndex[i][d] > 0 ) {
                const double dDeg = in_aaiIndex[i][d];
                out_adBasis[i] = out_adBasis[i] * pow( adLi[d], dDeg );
            }
        }
    }
}

template<class T>
void Basis( const R2Polygon              & in_poly,
            const R2PointTC<T>           & in_pt,
            const Array< Array<int> >    & in_aaiIndex,
            const Array< int >           & in_aiFact,
            Array< T >                   & out_adBasis )
{
    Array<T> adLi( in_poly.Num_pts() );
    Li( in_poly, in_pt, adLi );

    out_adBasis.need( in_aaiIndex.num() );
    for ( int i = 0; i < out_adBasis.num(); i++ ) {
        out_adBasis[i] = in_aiFact[i];
        for ( int d = 0; d < in_aaiIndex[i].num(); d++ ) {
            if ( in_aaiIndex[i][d] == 1 ) {
                out_adBasis[i] = out_adBasis[i] * adLi[d];
            } else if ( in_aaiIndex[i][d] > 1 ) {
                const int iDeg = in_aaiIndex[i][d];
                out_adBasis[i] = out_adBasis[i] * pow( adLi[d], iDeg );
            }
        }
    }
}


R3Pt SRFSPatch::operator()( const R2Pt &in_pt ) const
{
    Array<double> adBasis;

    ::Basis( Domain(), in_pt, Index(), Fact(), adBasis );

    R3Pt ptSum(0,0,0);
    for ( int i = 0; i < adBasis.num(); i++ ) {
        for ( int j = 0; j < 3; j++ ) 
            ptSum[j] += adBasis[i] * m_aptCtrl[i][j];
    }

    return ptSum;
}

void SRFSPatch::Basis( const R2Pt &in_pt, Array<double> &out_adBasis ) const
{
    ::Basis( Domain(), in_pt, Index(), Fact(), out_adBasis );
}


Array<double> SRFSPatch::Basis( const R2Pt &in_pt,const Array<int> &in_aiWhichPt ) const
{
    R2PointTC< double > pt( in_pt[0], in_pt[1] );

    Array< double > adLi( m_poly.Num_pts() );
    Li( Domain(), pt, adLi );


    Array<double> adSum( in_aiWhichPt.num() );
    for ( int i = 0; i < in_aiWhichPt.num(); i++ ) {
        double dSum = Fact(in_aiWhichPt[i]);
        for ( int d = 0; d < Index()[in_aiWhichPt[i]].num(); d++ ) {
            if ( Index()[in_aiWhichPt[i]][d] > 0 ) {
                const int iDeg = Index()[in_aiWhichPt[i]][d];
                if ( iDeg == 1 ) {
                    dSum = dSum * adLi[d];
                } else { 
                    dSum = dSum * pow( adLi[d], iDeg );
                }
            }
        }
        adSum[i] = dSum;
    }

    return adSum;
}

Array<double> SRFSPatch::BasisDerivative( const int in_iWhich, const R2Pt &in_pt, const Array<int> &in_aiWhichPt ) const
{
    R2PointTC< F<double> > pt( in_pt[0], in_pt[1] );
    pt[in_iWhich].diff(0,1);

    Array< F<double> > adLi( m_poly.Num_pts() );
    Li( Domain(), pt, adLi );


    Array<double> adSum( in_aiWhichPt.num() );
    for ( int i = 0; i < in_aiWhichPt.num(); i++ ) {
        F<double> dSum = Fact(in_aiWhichPt[i]);
        for ( int d = 0; d < Index()[in_aiWhichPt[i]].num(); d++ ) {
            if ( Index()[in_aiWhichPt[i]][d] > 0 ) {
                const int iDeg = Index()[in_aiWhichPt[i]][d];
                if ( iDeg == 1 ) {
                    dSum = dSum * adLi[d];
                } else { 
                    dSum = dSum * pow( adLi[d], iDeg );
                }
            }
        }
        adSum[i] = dSum.d(0);
    }

    return adSum;
}

Array<double> SRFSPatch::BasisSecondDerivative( const R2Pt_i &in_iptWhich, const R2Pt &in_pt, const Array<int> &in_aiWhichPt ) const
{
    R2PointTC< F< F<double> > > pt( in_pt[0], in_pt[1] );
    pt[ in_iptWhich[0] ].diff(0,1);
    pt[ in_iptWhich[1] ].x().diff(0,1);

    Array< F< F<double> > > adLi( m_poly.Num_pts() );
    Li( Domain(), pt, adLi );

    Array<double> adSum( in_aiWhichPt.num() );
    for ( int i = 0; i < in_aiWhichPt.num(); i++ ) {
        F< F<double> > dSum = Fact(in_aiWhichPt[i]);
        for ( int d = 0; d < Index()[in_aiWhichPt[i]].num(); d++ ) {
            if ( Index()[in_aiWhichPt[i]][d] > 0 ) {
                const int iDeg = Index()[in_aiWhichPt[i]][d];
                if ( iDeg == 1 ) {
                    dSum = dSum * adLi[d];
                } else { 
                    dSum = dSum * pow( adLi[d], iDeg );
                }
            }
        }
        adSum[i] = dSum.d(0).d(0);
    }

    return adSum;
}

void SRFSPatch::BasisDerivative( const int       in_iWhich, 
                                 const R2Pt    & in_pt, 
                                 Array<double> & out_adValues ) const
{
	R2PointTC< F<double> > xy(in_pt[0], in_pt[1]);   // Initialize arguments

    xy[ in_iWhich ].diff(0,1);

    Array< F<double> > adBasis;

    ::Basis( Domain(), xy, Index(), Fact(), adBasis );

    out_adValues.need( adBasis.num() );
    for ( int i = 0; i < adBasis.num(); i++ ) {
        out_adValues[i] = adBasis[i].d( 0 );
    }
}


void SRFSPatch::BasisSecondDerivative( const R2Pt_i  & in_iWhich, 
                                       const R2Pt    & in_pt, 
                                       Array<double> & out_adValues ) const
{ 
    // Initialize arguments
    Array< R2PointTC< F<double> > > aptInF( Index().num() );
    Array< R2PointTC< F< F<double> > > > aptIn( Index().num() );
    out_adValues.need( Index().num() );

    R2PointTC< F< F<double> > > ptB( in_pt[0], in_pt[1] );
    ptB[ in_iWhich[0] ].diff(0,1);
    ptB[ in_iWhich[1] ].x().diff(0,1);

    Array< F< F<double> > > adLi( m_poly.Num_pts() );
    Li( Domain(), ptB, adLi );

    for ( int i = 0; i < aptIn.num(); i++ ) {
        F< F<double> > dSum = Fact(i);
        for ( int d = 0; d < Index()[i].num(); d++ ) {
            if ( Index()[i][d] > 0 ) {
                const int iDeg = Index()[i][d];
                if ( iDeg == 1 ) {
                    dSum = dSum * adLi[d];
                } else { 
                    dSum = dSum * pow( adLi[d], iDeg );
                }
            }
        }

        out_adValues[i] = dSum.d(0).d(0);
    }
}

///
R3Vec SRFSPatch::Derivative( const int in_iWhich, const R2Pt & in_ptUV ) const
{
    Array<double> adBasisDeriv;
    BasisDerivative( in_iWhich, in_ptUV, adBasisDeriv );

    R3Vec vecSum(0,0,0);
    for ( int i = 0; i < adBasisDeriv.num(); i++ ) {
        for ( int j = 0; j < 3; j++ ) 
            vecSum[j] += adBasisDeriv[i] * m_aptCtrl[i][j];
    }

    return vecSum;
}

///
R3Vec SRFSPatch::SecondDerivative( const R2Pt_i &in_iptWhich, const R2Pt & in_ptUV ) const
{
    Array<double> adBasisDeriv;
    BasisSecondDerivative( in_iptWhich, in_ptUV, adBasisDeriv );

    R3Vec vecSum(0,0,0);
    for ( int i = 0; i < adBasisDeriv.num(); i++ ) {
        for ( int j = 0; j < 3; j++ ) 
            vecSum[j] += adBasisDeriv[i] * m_aptCtrl[i][j];
    }

    return vecSum;
}

void SRFSPatch::AllEval( const R2Pt &in_pt, const Array<int> &in_aiWhichPt,
                         Array<double> &out_ad,
                         Array<double> &out_adDs,  Array<double> &out_adDt,
                         Array<double> &out_adDss, Array<double> &out_adDtt,
                         Array<double> &out_adDst ) const
{
    R2PointTC< F< F<double> > > ptIn( in_pt[0], in_pt[1] );
    ptIn[0].diff(0,2);
    ptIn[1].diff(1,2);
    ptIn[0].x().diff(0,2);
    ptIn[1].x().diff(1,2);

    Array< F< F<double> > > adLi( m_poly.Num_pts() );
    Li( Domain(), ptIn, adLi );

    out_ad.need( in_aiWhichPt.num() );
    out_adDs.need( in_aiWhichPt.num() );
    out_adDt.need( in_aiWhichPt.num() );
    out_adDss.need( in_aiWhichPt.num() );
    out_adDtt.need( in_aiWhichPt.num() );
    out_adDst.need( in_aiWhichPt.num() );

    for ( int i = 0; i < in_aiWhichPt.num(); i++ ) {
        const int iIndx = in_aiWhichPt[i];
        F< F<double> > dSum = Fact(iIndx);
        for ( int d = 0; d < Index()[iIndx].num(); d++ ) {
            if ( Index()[iIndx][d] > 0 ) {
                const int iDeg = Index()[iIndx][d];
                if ( iDeg == 1 ) {
                    dSum = dSum * adLi[d];
                } else { 
                    dSum = dSum * pow( adLi[d], iDeg );
                }
            }
        }
        out_ad[i] = dSum.x().x();
        out_adDs[i] = dSum.x().d(0);
        out_adDt[i] = dSum.x().d(1);
        out_adDss[i] = dSum.d(0).d(0);
        out_adDtt[i] = dSum.d(1).d(1);
        out_adDst[i] = dSum.d(0).d(1);
    }
}
///

void SRFSPatch::AllEval( const R2Pt &in_pt, const Array<int> &in_aiWhichPt,
                         Array<double> &out_ad,
                         Array<double> &out_adDs,  Array<double> &out_adDt ) const
{
    R2PointTC< F<double> > ptIn( in_pt[0], in_pt[1] );
    ptIn[0].diff(0,2);
    ptIn[1].diff(1,2);

    Array< F<double> > adLi( m_poly.Num_pts() );
    Li( Domain(), ptIn, adLi );

    out_ad.need( in_aiWhichPt.num() );
    out_adDs.need( in_aiWhichPt.num() );
    out_adDt.need( in_aiWhichPt.num() );

    for ( int i = 0; i < in_aiWhichPt.num(); i++ ) {
        const int iIndx = in_aiWhichPt[i];
        F<double> dSum = Fact(iIndx);
        for ( int d = 0; d < Index()[iIndx].num(); d++ ) {
            if ( Index()[iIndx][d] > 0 ) {
                const int iDeg = Index()[iIndx][d];
                if ( iDeg == 1 ) {
                    dSum = dSum * adLi[d];
                } else { 
                    dSum = dSum * pow( adLi[d], iDeg );
                }
            }
        }
        out_ad[i] = dSum.x();
        out_adDs[i] = dSum.d(0);
        out_adDt[i] = dSum.d(1);
    }
}
///
void SRFSPatch::AllEval(  const R2Pt &in_pt,
                          R3Pt &out_pt,
                          R3Vec &out_vecDs, R3Vec &out_vecDt,
                          R3Vec &out_vecDss, R3Vec &out_vecDtt,
                          R3Vec &out_vecDst ) const
{
    // Initialize arguments
    R2PointTC< F< F<double> > > ptIn( in_pt[0], in_pt[1] );
    ptIn[0].diff(0,2);
    ptIn[1].diff(1,2);
    ptIn[0].x().diff(0,2);
    ptIn[1].x().diff(1,2);

    Array< F< F<double> > > adBasis( Index().num() );

    out_pt = R3Pt(0,0,0);
    out_vecDs = out_vecDt = out_vecDss = out_vecDtt = out_vecDst = R3Vec(0,0,0);

    ::Basis( Domain(), ptIn, Index(), Fact(), adBasis );

    out_pt = R3Pt(0,0,0);
    out_vecDs = out_vecDt = R3Vec(0,0,0);
    for ( int i = 0; i < adBasis.num(); i++ ) {
        for ( int j = 0; j < 3; j++ ) {
            out_pt[j] += adBasis[i].x().x() * m_aptCtrl[i][j];
            out_vecDs[j] += adBasis[i].d(0).x() * m_aptCtrl[i][j];
            out_vecDt[j] += adBasis[i].d(1).x() * m_aptCtrl[i][j];
            out_vecDss[j] += adBasis[i].d(0).d(0) * m_aptCtrl[i][j];
            out_vecDtt[j] += adBasis[i].d(1).d(1) * m_aptCtrl[i][j];
            out_vecDst[j] += adBasis[i].d(1).d(0) * m_aptCtrl[i][j];
        }
    }
}

///
void SRFSPatch::AllEval( const R2Pt &in_pt,
                         R3Pt &out_pt,
                         R3Vec &out_vecDs, R3Vec &out_vecDt ) const
{
	R2PointTC< F<double> >   xy(in_pt[0], in_pt[1]);
	xy[0].diff(0,2);                    // Second order wrt. x
	xy[1].diff(1,2);                    // Second order wrt. y

    Array< F<double> > adBasis( Index().num() );

    ::Basis( Domain(), xy, Index(), Fact(), adBasis );

    out_pt = R3Pt(0,0,0);
    out_vecDs = out_vecDt = R3Vec(0,0,0);
    for ( int i = 0; i < adBasis.num(); i++ ) {
        for ( int j = 0; j < 3; j++ ) {
            out_pt[j] += adBasis[i].x() * m_aptCtrl[i][j];
            out_vecDs[j] += adBasis[i].d(0) * m_aptCtrl[i][j];
            out_vecDt[j] += adBasis[i].d(1) * m_aptCtrl[i][j];
        }
    }
}
///
R3Vec SRFSPatch::Normal( const R2Pt &in_pt ) const
{
	R2PointTC< F<double> >   xy(in_pt[0], in_pt[1]);
	xy[0].diff(0,2);                    // Second order wrt. x
	xy[1].diff(1,2);                    // Second order wrt. y

    Array< F<double> > adBasis( Index().num() );

    ::Basis( Domain(), xy, Index(), Fact(), adBasis );

    R3Vec vecDs(0,0,0), vecDt(0,0,0);
    for ( int i = 0; i < adBasis.num(); i++ ) {
        for ( int j = 0; j < 3; j++ ) {
            vecDs[j] += adBasis[i].d(0) * m_aptCtrl[i][j];
            vecDt[j] += adBasis[i].d(1) * m_aptCtrl[i][j];
        }
    }

    return Cross( vecDs, vecDt );
}

SRFSPatch &SRFSPatch::operator=( const SRFSPatch &in_srf )
{
    m_iDepth = in_srf.m_iDepth;
    m_poly = in_srf.m_poly;
    m_aaipIndex = in_srf.m_aaipIndex;
    m_aipDFact = in_srf.m_aipDFact;
    m_aptCtrl = in_srf.m_aptCtrl;

    return *this;
}

///
WINbool SRFSPatch::operator==( const SRFSPatch &in_srf ) const
{
    if ( m_iDepth != in_srf.m_iDepth ) return FALSE;
    if ( ! ( m_poly == in_srf.Domain() ) ) return FALSE;

    for ( int i = 0; i < m_aptCtrl.num(); i++ ) 
        if ( ApproxEqual( m_aptCtrl[i], in_srf.m_aptCtrl[i] ) == FALSE )
            return FALSE;

    return TRUE;
}

///
SRFSPatch::SRFSPatch( const int in_iNSides, const int in_iDegree )
: m_poly( UnitPolygon( in_iNSides ) ),
  m_iDepth( in_iDegree ),
  m_aaipIndex( &::Index( in_iNSides, in_iDegree ) ),
  m_aipDFact( s_aaaiFact[in_iNSides][in_iDegree] )
{
    m_aptCtrl.need( Index().num() );

    SetDomain( R2Pt(0,0), 0.5, 0.0 );

    ASSERT( NSides() > 2 );
    ASSERT( in_iDegree > 0 );
}

SRFSPatch::SRFSPatch( const R2Polygon &in_poly, const int in_iDegree )
: m_poly( in_poly ),
  m_iDepth( in_iDegree ),
  m_aaipIndex( &::Index( in_poly.Num_pts(), in_iDegree ) ),
  m_aipDFact( s_aaaiFact[in_poly.Num_pts()][in_iDegree] )
{
    m_aptCtrl.need( Index().num() );

    SetDomain( m_poly );

    ASSERT( NSides() > 2 );
    ASSERT( in_iDegree > 0 );
}
///
SRFSPatch::SRFSPatch( const SRFSPatch &in_srf )
{
    *this = in_srf;
}

void SRFSPatch::SetDomain( const R2Pt &in_ptCenter, const double in_dRadius, const double in_dRotate )
{
    const double dDiv = (2.0 * M_PI) / (double) m_poly.Num_pts();

    const R2Pt ptV1( cos(dDiv * 0.5), sin(dDiv * 0.5) );
    const R2Pt ptV2( cos(dDiv * 0.5 + dDiv), sin(dDiv * 0.5 + dDiv) );
    const double dLenEdge = Length( Lerp( ptV1, ptV2, 0.5 ) - R2Pt(0,0) );
    const double dScl = in_dRadius / dLenEdge;
    for ( int iR = 0; iR < m_poly.Num_pts(); iR++ ) {
        const double dR = in_dRotate + iR * dDiv;
        m_poly[iR] = in_ptCenter + R2Vec( cos( dR ), sin( dR ) ) * dScl;
    }
    const double dLenAcc = Length( m_poly.Edge(0)(0.5) - in_ptCenter );
    ASSERT( RNApproxEqual( dLenAcc, in_dRadius ) );

    SetDomain( m_poly );
}

void SRFSPatch::SetDomain( const R2Polygon &in_poly )
{
    m_aaipIndex = &::Index( in_poly.Num_pts(), m_iDepth );
    m_aipDFact = s_aaaiFact[in_poly.Num_pts()][m_iDepth];
    m_aptCtrl.need( Index().num() );

    m_poly = in_poly;
    m_poly.Set_dim();

    Array<double> adBasis;
    for ( int i = 0; i < m_aptCtrl.num(); i++ ) {
        m_aptCtrl[i] = R3Pt(0,0,0);

        double dSum = 0.0;
        for ( int j = 0; j < Index()[i].num(); j++ ) {
            for ( int k = 0; k < Index()[i][j]; k++ ) {
                dSum += (double) Index()[i][j] / (double) m_iDepth;
            }
        }
        for ( FORINT j = 0; j < Index()[i].num(); j++ ) {
            for ( int k = 0; k < Index()[i][j]; k++ ) {
                const double dPerc = (double) Index()[i][j] / (double) m_iDepth;
                m_aptCtrl[i][0] += dPerc / dSum * Domain()[j][0];
                m_aptCtrl[i][1] += dPerc / dSum * Domain()[j][1];
            }
        }
        Basis( R2Pt( m_aptCtrl[i][0], m_aptCtrl[i][1] ), adBasis );
        for ( FORINT j = 0; j < adBasis.num(); j++ )
            m_aptCtrl[i][2] = WINmin( m_aptCtrl[i][2], 1.0 - adBasis[j] );
    }
}

///
void SRFSPatch::Read( ifstream &in )
{
    in >> m_iDepth;
    m_poly.Read(in);
    m_aaipIndex = &::Index( NSides(), m_iDepth );
    m_aipDFact = s_aaaiFact[ NSides() ][ m_iDepth ];

    m_aptCtrl.need( Index().num() );
    for ( int i = 0; i < m_aptCtrl.num(); i++ )
        in >> m_aptCtrl[i];
}

void SRFSPatch::ReadBinary( ifstream &in )
{
    in.read( (char *) &m_iDepth, sizeof(int) );
    m_poly.ReadBinary(in);
    m_aaipIndex = &::Index( NSides(), m_iDepth );
    m_aipDFact = s_aaaiFact[ NSides() ][ m_iDepth ];

    m_aptCtrl.need( Index().num() );
    for ( int i = 0; i < m_aptCtrl.num(); i++ )
        in.read( (char *) & m_aptCtrl[i][0], m_aptCtrl[i].Dim() * sizeof(double) );
}
///
void SRFSPatch::Write( ofstream &out ) const
{
    out << m_iDepth << "\n";
    m_poly.Write(out);
    for ( int i = 0; i < m_aptCtrl.num(); i++ )
        out << m_aptCtrl[i];
}
///
void SRFSPatch::WriteBinary( ofstream &out ) const
{
    out.write( (const char *) &m_iDepth, sizeof(int) );
    m_poly.WriteBinary(out);
    for ( int i = 0; i < m_aptCtrl.num(); i++ )
        out.write( (const char *) & m_aptCtrl[i][0], m_aptCtrl[i].Dim() * sizeof(double) );
}

void SRFSPatch::TestDeriv( const int in_iNDivs ) const
{
    const R2BoxedPolygon poly = Domain();

    const double dStepX = ( poly.WMax() - poly.WMin() ) / (double) (in_iNDivs);
    const double dStepY = ( poly.HMax() - poly.HMin() ) / (double) (in_iNDivs);

    const double dStep = WINmin( dStepX, dStepY) * 1e-9;
    double dAcc = 1e-4;

    R3Pt ptCheck;
    for ( double dS = poly.WMin() - dStepX * 0.5; dS <= poly.WMax() + dStepX * 0.5; dS += dStepX ) {
        for ( double dT = poly.HMin() - dStepY * 0.5; dT <= poly.HMax() + dStepY * 0.5; dT += dStepY ) {
            const R3Pt pt = (*this)( R2Pt( dS, dT ) );
            const R3Vec vecDs = ( (*this)( R2Pt( dS + dStep, dT ) ) - pt ) / dStep;
            const R3Vec vecDt = ( (*this)( R2Pt( dS, dT + dStep ) ) - pt ) / dStep;
            const R3Vec vecDss = ( Derivative( 0, R2Pt( dS + dStep, dT ) ) - Derivative( 0, R2Pt( dS, dT ) ) ) / dStep;
            const R3Vec vecDtt = ( Derivative( 1, R2Pt( dS, dT + dStep ) ) - Derivative( 1, R2Pt( dS, dT ) ) ) / dStep;
            const R3Vec vecDst = ( Derivative( 0, R2Pt( dS, dT + dStep ) ) - Derivative( 0, R2Pt( dS, dT ) ) ) / dStep;

            dAcc = WINmax( 1e-4, WINmax( 1e-4 * vecDs[0], 1e-4 * vecDs[1] ) );
            R3Vec vecCheckDs = Derivative( 0, R2Pt( dS, dT ) );
            ASSERT( ApproxEqual( vecCheckDs, vecDs, dAcc ) );

            dAcc = WINmax( 1e-4, WINmax( 1e-4 * vecDt[0], 1e-4 * vecDt[1] ) );
            R3Vec vecCheckDt = Derivative( 1, R2Pt( dS, dT ) );
            ASSERT( ApproxEqual( vecCheckDt, vecDt, dAcc ) );

            dAcc = WINmax( 1e-4, WINmax( 1e-4 * vecDss[0], 1e-4 * vecDss[1] ) );
            R3Vec vecCheckDss = SecondDerivative( R2Pt_i(0,0), R2Pt( dS, dT ) );
            ASSERT( ApproxEqual( vecCheckDss, vecDss, dAcc ) );

            dAcc = WINmax( 1e-4, WINmax( 1e-4 * vecDtt[0], 1e-4 * vecDtt[1] ) );
            R3Vec vecCheckDtt = SecondDerivative( R2Pt_i(1,1), R2Pt( dS, dT ) );
            ASSERT( ApproxEqual( vecCheckDtt, vecDtt, dAcc ) );

            dAcc = WINmax( 1e-4, WINmax( 1e-4 * vecDst[0], 1e-4 * vecDst[1] ) );
            R3Vec vecCheckDst = SecondDerivative( R2Pt_i(0,1), R2Pt( dS, dT ) );
            ASSERT( ApproxEqual( vecCheckDst, vecDst, dAcc ) );

            dAcc = WINmax( 1e-4, WINmax( 1e-4 * vecDst[0], 1e-4 * vecDst[1] ) );
            R3Vec vecCheckDts = SecondDerivative( R2Pt_i(1,0), R2Pt( dS, dT ) );
            ASSERT( ApproxEqual( vecCheckDts, vecDst, dAcc ) );

            AllEval( R2Pt( dS, dT ), ptCheck, vecCheckDs, vecCheckDt );
            ASSERT( ApproxEqual( ptCheck, pt, dAcc ) );
            ASSERT( ApproxEqual( vecCheckDs, vecDs, dAcc ) );
            ASSERT( ApproxEqual( vecCheckDt, vecDt, dAcc ) );

            AllEval( R2Pt( dS, dT ), ptCheck, vecCheckDs, vecCheckDt, vecCheckDss, vecCheckDtt, vecCheckDst );
            ASSERT( ApproxEqual( ptCheck, pt, dAcc ) );
            ASSERT( ApproxEqual( vecCheckDs, vecDs, dAcc ) );
            ASSERT( ApproxEqual( vecCheckDt, vecDt, dAcc ) );
            ASSERT( ApproxEqual( vecCheckDss, vecDss, dAcc ) );
            ASSERT( ApproxEqual( vecCheckDtt, vecDtt, dAcc ) );
            ASSERT( ApproxEqual( vecCheckDst, vecDst, dAcc ) );

            R3Vec vecNormCheck = Normal( R2Pt(dS,dT) );
            R3Vec vecNorm = Cross( vecCheckDs, vecCheckDt );
            ASSERT( ApproxEqual( vecNormCheck, vecNorm, dAcc ) );
        }
    }
}

Array<R2Pt> SRFSPatch::Samples( const int in_iNPerSide, 
                                const int in_iNRadial, 
                                Array<R4Pt_i> &out_aiptFaces, 
                                Array<R3Pt_i> &out_aiptFaces2 ) const
{
    Array<R2Pt> aptSamples;
    const R2Pt ptCenter = Domain().Centroid();
    for ( int iR = 0; iR < in_iNRadial; iR++ ) {
        const double dR = (double) iR / (double) (in_iNRadial);

        for ( int iS = 0; iS < NSides(); iS++ ) {
            for ( int iT = 0; iT < in_iNPerSide; iT++ ) {
                const R2Pt ptEdge( Domain().Edge(iS)( (double) iT / (double) in_iNPerSide ) );
                aptSamples += Lerp( ptEdge, ptCenter, dR );
            }
        }
    }
    aptSamples += ptCenter;

    out_aiptFaces.need( (in_iNRadial - 1) * (NSides() * in_iNPerSide) );
    int iIndx = 0;
    for ( FORINT iR = 0; iR < in_iNRadial - 1; iR++ ) {
        for ( int iS = 0; iS < NSides() * in_iNPerSide; iS++, iIndx++ ) {
            out_aiptFaces[iIndx][0] = iR * NSides() * in_iNPerSide + iS;
            out_aiptFaces[iIndx][1] = (iR+1) * NSides() * in_iNPerSide + iS;
            out_aiptFaces[iIndx][2] = (iR+1) * NSides() * in_iNPerSide + (iS + 1) % (NSides() * in_iNPerSide);
            out_aiptFaces[iIndx][3] = iR * NSides() * in_iNPerSide + (iS + 1) % (NSides() * in_iNPerSide);
        }
    }

    out_aiptFaces2.need( NSides() * in_iNPerSide );
    for ( int iS = 0; iS < NSides() * in_iNPerSide; iS++ ) {
        out_aiptFaces2[iS][2] = (in_iNRadial - 1) * NSides() * in_iNPerSide + iS;
        out_aiptFaces2[iS][1] = (in_iNRadial - 1) * NSides() * in_iNPerSide + (iS+1) % (NSides() * in_iNPerSide);
        out_aiptFaces2[iS][0] = aptSamples.num() - 1;
    }

    return aptSamples;
}


void SRFSPatch::Srf( PMeshLite &in_mesh, const int in_iNDivs )
{
    /// Radial Loop
    in_mesh.Clear();
    Array<Vertex> av;

    Array<R4Pt_i> aiptF1;
    Array<R3Pt_i> aiptF2;
    Array<R2Pt> aptSamples = Samples( in_iNDivs, in_iNDivs, aiptF1, aiptF2 );
    for ( int i = 0; i < aptSamples.num(); i++ ) {

        av += in_mesh.AddVertex( (*this)( aptSamples[i] ), Normal( aptSamples[i] ), UTILSColor(0.5, 0.5, 0.5) );
    }

    Array<Vertex> avF(4);
    for ( FORINT i = 0; i < aiptF1.num(); i++ ) {
        for ( int j = 0; j < 4; j++ )
            avF[j] = av[ aiptF1[i][j] ];

        in_mesh.AddFace( avF );
    }

    avF.need(3);
    for ( FORINT i = 0; i < aiptF2.num(); i++ ) {
        for ( int j = 0; j < 3; j++ )
            avF[j] = av[ aiptF2[i][j] ];

        in_mesh.AddFace( avF );
    }
}

template<class T>
void SRFSPatch_BasisIndividual( const SRFSPatch    & in_patch, 
                                const R2PointTC<T> & in_pt, 
                                const Array<int>   & in_aiWhichPt, 
                                Array<T>           & out_ad ) 
{
    Array<T> adLi( in_patch.Domain().Num_pts() );

    R2PointTC<T> pt( in_pt[0], in_pt[1] );
    Li( in_patch.Domain(), in_pt, adLi );


    out_ad.need( in_aiWhichPt.num() );
    for ( int i = 0; i < in_aiWhichPt.num(); i++ ) {
        out_ad[i] = in_patch.Fact(in_aiWhichPt[i]);
        for ( int d = 0; d < in_patch.Index()[in_aiWhichPt[i]].num(); d++ ) {
            if ( in_patch.Index()[in_aiWhichPt[i]][d] > 0 ) {
                const int iDeg = in_patch.Index()[in_aiWhichPt[i]][d];
                if ( iDeg == 1 ) {
                    out_ad[i] = out_ad[i] * adLi[d];
                } else { 
                    out_ad[i] = out_ad[i] * pow( adLi[d], iDeg );
                }
            }
        }
    }
}

void SRFForceEval()
{
    SRFSPatch spatch;
    Array<int> aiWhich;
    R2PointTC<double> pt(0,0);
    R2PointTC< F<double> > ptF;
    R2PointTC< F< F<double> > > ptFF;

    Array<double> ad;
    SRFSPatch_BasisIndividual( spatch, pt, aiWhich, ad );
    Array< F<double> > ad2;
    SRFSPatch_BasisIndividual( spatch, ptF, aiWhich, ad2 );
    Array< F< F<double> > > ad3;
    SRFSPatch_BasisIndividual( spatch, ptFF, aiWhich, ad3 );
}
