/* ------------------------------------------------------------
  A non uniform rational bspline surface
  Operations implemented:
   ------------------------------------------------------------ */

#include "StdAfx.H"
#include <utils/Vol_NurbTC.H>

/* --------------- protected Routines -------------- */
/* -------------------------- Private Routines ---------------- */


/* ------------- Public Routines ----------------------------- */



template<>
WINbool VOLNurbTC<R2Pt, R2Pt, R2Vec >::Test()
{
   VOLNub3TC srf(R3Pt_i( 1,2,3 ), R3Pt_i(3, 4, 5) );

   for (int u = 0; u < 3; u++) {
       for (int v = 0; v < 4; v++) {
           for ( int w = 0; w < 5; w++ ) {
               srf.Pt( u, v, w ) = R3Pt (u,v, w);
               const int iPtIndx = srf.PtIndx( u, v, w );
               ASSERT( srf.PtUIndx( iPtIndx ) == u );
               ASSERT( srf.PtVIndx( iPtIndx ) == v );
               ASSERT( srf.PtWIndx( iPtIndx ) == w );
           }
       }
   }

   ASSERT(srf.InDim() == 3);
   ASSERT(srf.OutDim() == 3);

   ASSERT(srf.Num_pts() == 3 * 4 * 5);
   R3Pt out_pt = srf( R3Pt(0.5, 0.5, 0.5) );
   ASSERT(out_pt[0] == out_pt[1]);

   R3Pt ptEval;
   out_pt = srf.Closest_pt(R3Pt (2, 2, 2),
                           ptEval);

   VOLNub3TC srf2(srf);
   ASSERT(srf2 == srf);
   srf2.Pt(1, 0, 0) = R3Pt (0.3, 0.2, 0.1);

   ofstream out("RNtest.txt", ios::out);
   srf.Write(out);
   out.close();
   ifstream in("RNtest.txt", ios::in);
   VERIFY(srf2.Read(in));
   in.close();
   VERIFY(srf == srf2);


   in.open("RNtest.txt", ios::in);
   VOLNub3TC srf4(in, (WINbool) FALSE);
   in.close();
   VERIFY(srf == srf4);

   srf2.Print();
   srf2.Pt(1, 2,3) = R3Pt (0.3, 0.2, 0.2);

   srf.InDim();
   srf.OutDim();
   srf.U_kvec();
   srf.V_kvec();
   srf.W_kvec();
   srf.Start_u();
   srf.Start_v();
   srf.Start_w();
   srf.End_u();
   srf.End_v();
   srf.End_w();
   srf.PercU(  0.1 );
   srf.PercV( 0.1 );
   srf.PercW( 0.1 );
   srf.Num_pts();
   srf.NUpts();
   srf.NVpts();
   srf.NWpts();
   srf.PtUIndx( 0 );
   srf.PtVIndx( 0 );
   srf.PtWIndx( 0 );
   srf.Pt( 0,0,0 );
   srf.Pt( 0,0,0 );
   srf.Pt( 0 );
   srf.Pt( 0 );
   Array<double> adValue;
   Array<int> aiIndex;
   srf.Basis( R3Pt(0.0, 0.0, 0.0 ), adValue, aiIndex );
   srf.BasisDerivative( 0, R3Pt(0.0, 0.0, 0.0 ), adValue, aiIndex );
   srf.BasisSecondDerivative( R2Pt_i(0,0), R3Pt(0.0, 0.0, 0.0 ), adValue, aiIndex );
   srf(R3Pt(0.0, 0.0, 0.0 ) );
   srf.Perc(R3Pt(0.0, 0.0, 0.0 ) );

   R3Pt ptOut;
   R3Vec vecOut;
   srf.AllEval( R3Pt(0.0, 0.0, 0.0), ptOut, vecOut, vecOut, vecOut, vecOut, vecOut, vecOut, vecOut, vecOut, vecOut);
   srf.AllEval( R3Pt(0.0, 0.0, 0.0), ptOut, vecOut, vecOut, vecOut );
   R3Pt ptIn;
   srf.Clip_uv( ptIn );
   srf.Closest_pt( ptOut, ptOut );

   srf.SetKnots( R3Pt(0.0, 0.0, 0.0), R3Pt(0.0, 0.0, 0.0), R3Vec(0.0, 0.0, 0.0));
   srf.SetKnots( R3Pt(0.0, 0.0, 0.0), R3Pt(0.0, 0.0, 0.0) );
   Array<double> adBlend;
   srf.SetKnots( adBlend, adBlend, adBlend );

   return FALSE;
}

