#include "StdAfx.H"

#include <utils/Utils_Dots.H>
 
const int ABOVE_FLAG = 1;
const int BELOW_FLAG = 2;
const int LEFT_FLAG = 4;
const int RIGHT_FLAG = 8;
const int LIP_FLAG = 4096;
const int EYE_FLAG = 2 * LIP_FLAG;
const int FLAG = LEFT_FLAG + RIGHT_FLAG + ABOVE_FLAG + BELOW_FLAG;

//------------------------------------------------------------------------------------------

WINbool UTILSDot::Is_compatable( int in_ciGId ) const
{
    for ( int iJ = 0; iJ < 3; iJ++ ) {
        int iDid = ( m_oiGroupId >> ( 4 * iJ ) ) & FLAG;
        int iGid = ( in_ciGId >> ( 4 * iJ ) ) & FLAG;
        if ( iDid & LEFT_FLAG || iGid & LEFT_FLAG ) continue;
        if ( iDid & RIGHT_FLAG || iGid & RIGHT_FLAG ) continue;
        if ( iDid & ABOVE_FLAG && iGid & BELOW_FLAG ) 
            return FALSE;
        if ( iDid & BELOW_FLAG && iGid & ABOVE_FLAG ) 
            return FALSE;
    }
    return TRUE;

}//UTILSDot::Is_compatable

//------------------------------------------------------------------------------------------

WINbool UTILSDot::Is_lip(  ) const
{ 
    return ( m_oiGroupId & LIP_FLAG ) ? TRUE : FALSE;

}//UTILSDot::Is_lip

//------------------------------------------------------------------------------------------

WINbool UTILSDot::Is_eye(  ) const
{ 
    return ( m_oiGroupId & EYE_FLAG ) ? TRUE : FALSE;

}//UTILSDot::Is_eye

//------------------------------------------------------------------------------------------

WINbool UTILSDot::Is_above_lip(  ) const
{ 
    return ( m_oiGroupId & ABOVE_FLAG ) ? TRUE : FALSE;

}//UTILSDot::Is_above_lip

//------------------------------------------------------------------------------------------

WINbool UTILSDot::Is_below_lip(  ) const
{ 
    return ( m_oiGroupId & BELOW_FLAG ) ? TRUE : FALSE;

}//UTILSDot::Is_below_lip

//------------------------------------------------------------------------------------------

UTILSDot & UTILSDot::operator=( const UTILSDot & in_rcoDot )
{
    m_iName          = in_rcoDot.m_iName;
    m_oColor         = in_rcoDot.m_oColor;
    m_oiGroupId      = in_rcoDot.m_oiGroupId;
    m_ptNeutralLoc   = in_rcoDot.m_ptNeutralLoc;
    m_vecCurOffset   = in_rcoDot.m_vecCurOffset;
   
    return *this;

}//UTILSDot::operator=

//------------------------------------------------------------------------------------------

void UTILSDot::Read( ifstream & in_rstrmIn )
{
    char szWiffle[ 255 ];

    in_rstrmIn >> m_iName;
    m_oColor.Read( in_rstrmIn );
    in_rstrmIn >> m_oiGroupId;
    in_rstrmIn >> m_ptNeutralLoc[ 0 ] >> m_ptNeutralLoc[ 1 ] >> m_ptNeutralLoc[ 2 ];

    m_vecCurOffset = R3Vec( 0,0,0 );
    
    in_rstrmIn >> szWiffle;
    if ( !strcmp( szWiffle, "OFFSET" ) ) {
        in_rstrmIn >> m_vecCurOffset[0];
        in_rstrmIn >> m_vecCurOffset[1];
        in_rstrmIn >> m_vecCurOffset[2];
        in_rstrmIn >> szWiffle;
    } 

    ASSERT ( !strcmp( szWiffle, "END" ) );

}//UTILSDot::Read

//------------------------------------------------------------------------------------------

void UTILSDot::Read_locs_only( ifstream & in_rstrmIn )
{
    static int names = 0;

    m_iName = names++;
    m_oColor = UTILSColor::GREY;
    m_oiGroupId = 0;

    in_rstrmIn >> m_iName;
    in_rstrmIn >> m_ptNeutralLoc[ 0 ] >> m_ptNeutralLoc[ 1 ] >> m_ptNeutralLoc[ 2 ];

}//UTILSDot::Read_locs_only

//------------------------------------------------------------------------------------------

void UTILSDot::Write( ofstream & io_strmOut ) const
{
    io_strmOut << m_iName << " ";
    m_oColor.Write( io_strmOut );
    io_strmOut << " " << m_oiGroupId;
    io_strmOut << " " << m_ptNeutralLoc[ 0 ] << " " << 
    m_ptNeutralLoc[ 1 ] << " " << m_ptNeutralLoc[ 2 ] << " ";
   
    io_strmOut << " OFFSET ";
    io_strmOut << " " << m_vecCurOffset[ 0 ] << " ";
    io_strmOut << " " << m_vecCurOffset[ 1 ]; 
    io_strmOut << " " << m_vecCurOffset[ 2 ];
    io_strmOut << " END\n";

}//UTILSDot::Write

//------------------------------------------------------------------------------------------

int UTILSDotArray::Read_locs_only( ifstream & in_rstrmIn )
{
    for ( int iI = 0; iI < N( ); iI++ )
        m_aoDots[ iI ].Read_locs_only( in_rstrmIn );
   
    return m_aoDots.num(  );

}//UTILSDotArray::Read_locs_only

//------------------------------------------------------------------------------------------

int UTILSDotArray::Read( ifstream & in_rstrmIn )
{
    m_aoDots.clearcompletely(  );
   
    int      iN;

    in_rstrmIn >> iN;

    if ( iN < 0 ) {
        m_aoDots.need( -iN );
        return Read_locs_only( in_rstrmIn );
    }
    m_aoDots.need( iN );
    for ( int iI = 0; iI < iN; iI++ )
        m_aoDots[ iI ].Read( in_rstrmIn );
       
    return m_aoDots.num(  );

}//UTILSDotArray::Read

//------------------------------------------------------------------------------------------

void UTILSDotArray::Write( ofstream & io_rstrmOut ) const
{
    io_rstrmOut << m_aoDots.num(  ) << "\n";
    for ( int iI = 0; iI < m_aoDots.num(  ); iI++ ) {
        m_aoDots[ iI ].Write( io_rstrmOut );
    }
    io_rstrmOut << "\n\n";

}//UTILSDotArray::Write

//------------------------------------------------------------------------------------------

void UTILSDotArray::Write_locs_only( ofstream & io_rstrmOut ) const
{
    io_rstrmOut << m_aoDots.num(  ) << "\n";
    for ( int iI = 0; iI < m_aoDots.num(  ); iI++ ) {
        io_rstrmOut << m_aoDots[ iI ].Name(  ) << m_aoDots[ iI ].Loc(  )[ 0 ] << " " 
            << m_aoDots[ iI ].Loc(  )[ 1 ] << " " 
         << m_aoDots[ iI ].Loc(  )[ 2 ] << "\n";
    }
    io_rstrmOut << "\n";

}//UTILSDotArray::Write_locs_only

//------------------------------------------------------------------------------------------

void UTILSDotArray::Set_names(  )
{
    for ( int iI = 0; iI < m_aoDots.num(  ); iI++ ) {
        m_aoDots[ iI ].Set_name( iI );
    }

}//UTILSDotArray::Set_names

//------------------------------------------------------------------------------------------


//------------------------------------------------------------------------------------------

void UTILSDotArray::Order_by_group_ids(  )
{
    UTILSDotArray   oDots;
    Array<WINbool>  abAdded( N(  ) );

    for ( int iI = 0; iI < N(  ); iI++ ) abAdded[ iI ] = FALSE;
   
    for ( FORINT iI = 0; iI < N(  ); iI++ ) {
        if ( abAdded[ iI ] == TRUE ) continue;
        oDots.Add( m_aoDots[ iI ] );
        abAdded[ iI ] = TRUE;
        for ( int iJ = iI+1; iJ < N(  ); iJ++ )
            if ( m_aoDots[ iJ ].GroupId(  ) == m_aoDots[ iI ].GroupId(  ) ) {
                oDots.Add( m_aoDots[ iJ ] );
                abAdded[ iJ ] = TRUE;
            }
        }
    ( *this ) = oDots;
    Set_names(  );

}//UTILSDotArray::Order_by_group_ids

//------------------------------------------------------------------------------------------
///
pair< R3Pt, double > UTILSDotArray::CenterScale() const
{
    R3Pt ptCenter(0,0,0);
    double dScale = 0.0;

    if ( N() == 0 )
        return pair< R3Pt, double >( ptCenter, dScale );

    for ( int i = 0; i < N(); i++ ) {
        for ( int j = 0; j < 3; j++ )
            ptCenter[j] += m_aoDots[i].Loc()[j];
    }
    for ( int j = 0; j < 3; j++ )
        ptCenter[j] /= (double) N();

    for ( FORINT i = 0; i < N(); i++ ) {
        dScale += Length( m_aoDots[i].Loc() - ptCenter );
    }
    dScale /= (double) N();

    return pair< R3Pt, double >( ptCenter, dScale );
}

//------------------------------------------------------------------------------------------

void UTILSDotArray::Center()
{
    const pair< R3Pt, double > centerScale = CenterScale();

    const double dScale = ( RNIsZero( centerScale.second ) ) ? 1.0 : 1.0 / centerScale.second;
    for ( int i = 0; i < N(); i++ ) {
        const R3Vec vecMove = m_aoDots[i].Loc() - centerScale.first;
        m_aoDots[i].Set_point( R3Pt( vecMove[0] * dScale, vecMove[1] * dScale, vecMove[2] * dScale ) );
        m_aoDots[i].Set_offset( R3Vec( m_aoDots[i].Offset()[0] * dScale, m_aoDots[i].Offset()[1] * dScale, m_aoDots[i].Offset()[2] * dScale ) );
    }
}

//------------------------------------------------------------------------------------------

void UTILSDotArray::Move( const R3Matrix & f )
{
    for ( int iI = 0; iI < N(  ); iI++ ) {
        m_aoDots[ iI ].Set_point( f * m_aoDots[ iI ].Loc(  ) );
    }

}//UTILSDotArray::Move



//------------------------------------------------------------------------------------------

double UTILSDotArray::Avg_dist(  ) const
{
    double dAvgDotDistance = 0;
    for ( int iI = 0; iI < m_aoDots.num(  ); iI++ ) {
        double dMin = 1e30, dD = 0;
        for ( int iJ = 0; iJ < m_aoDots.num(  ); iJ++ ) {
            if ( iI == iJ ) continue;
            dD = Length( m_aoDots[ iJ ].Loc(  ) - m_aoDots[ iI ].Loc(  ) );
            dMin = WINmin( dMin, dD );
        }
        dAvgDotDistance += dMin;
    }
    ASSERT( m_aoDots.num(  ) );
    dAvgDotDistance /= ( double ) m_aoDots.num(  );
    return dAvgDotDistance;

}//UTILSDotArray::Avg_dist

//------------------------------------------------------------------------------------------

UTILSDotArray & UTILSDotArray::operator=( const UTILSDotArray & in_rcoDots )
{
    m_aoDots.clearcompletely(  );
    for ( int iI = 0; iI < in_rcoDots.N(  ); iI++ )
        m_aoDots += in_rcoDots[ iI ];

    return *this;

}//UTILSDotArray::operator=

//------------------------------------------------------------------------------------------
void UTILSDotArray::Need( int iN )
{
	m_aoDots.need( iN );
	for (int i = 0; i < iN; i++) {
		m_aoDots[i].Set_name(i);
	}
}

//------------------------------------------------------------------------------------------

UTILSDot & UTILSDotArray::Add( const R3Pt       & in_rcptP, 
                               const UTILSColor & in_rcoColor,
                               int                in_iName )
{
    UTILSDot    oDot; 

    oDot.Set_name( N(  ) );
    oDot.Set_point( in_rcptP ); 
    oDot.Color(  ) = in_rcoColor;
    if ( in_iName != -1 ) oDot.Set_name( in_iName );
    m_aoDots += oDot;
    return m_aoDots.last(  );

}//UTILSDotArray::Add


//------------------------------------------------------------------------------------------

UTILSDot & UTILSDotArray::Add( const R2Pt       & in_rcptP, 
                               const R2Vec      & in_rcvecDir,
                               const UTILSColor & in_rcoColor )
{
    UTILSDot    oDot; 

    oDot.Set_name( N(  ) );
    oDot.Set_point( R3Pt( in_rcptP[0], in_rcptP[1], 0.0 ) ); 
    oDot.Set_offset( R3Vec( in_rcvecDir[0], in_rcvecDir[1], 0.0 ) ); 
    oDot.Color(  ) = in_rcoColor;
    m_aoDots += oDot;
    return m_aoDots.last(  );

}//UTILSDotArray::Add


//------------------------------------------------------------------------------------------

UTILSDot & UTILSDotArray::Add( const R3Pt       & in_rcptP, 
                               const R3Vec      & in_rcvecDir,
                               const UTILSColor & in_rcoColor )
{
    UTILSDot    oDot; 

    oDot.Set_name( N(  ) );
    oDot.Set_point( in_rcptP ); 
    oDot.Set_offset( in_rcvecDir ); 
    oDot.Color(  ) = in_rcoColor;
    m_aoDots += oDot;
    return m_aoDots.last(  );

}//UTILSDotArray::Add



//------------------------------------------------------------------------------------------
UTILSDot & UTILSDotArray::Add( const R2Pt & in_rcptP, const UTILSColor & in_rcoColor )
{
    UTILSDot    oDot; 

    oDot.Set_name( N(  ) );
    oDot.Set_point( in_rcptP ); 
    oDot.Color(  ) = in_rcoColor;
    m_aoDots += oDot;
    return m_aoDots.last(  );

}//UTILSDotArray::Add

//------------------------------------------------------------------------------------------

void UTILSDotArray::AddPair( const R3Pt & in_rcptP1, const R3Pt & in_rcptP2, const UTILSColor & in_rcoColor )
{
    Add( in_rcptP1, in_rcptP2 - in_rcptP1, in_rcoColor );
    Add( in_rcptP2, UTILSColor( 1.0 - in_rcoColor[0], 1.0 - in_rcoColor[1], 1.0 - in_rcoColor[2] ) );

}//UTILSDotArray::Add

//------------------------------------------------------------------------------------------

void UTILSDotArray::AddPair( const R2Pt & in_rcptP1, const R2Pt & in_rcptP2, const UTILSColor & in_rcoColor )
{
    const R2Vec vec = in_rcptP2 - in_rcptP1;
    Add( R3Pt( in_rcptP1[0], in_rcptP1[1], 0.0 ), R3Vec(vec[0], vec[1], 0.0), in_rcoColor );
    Add( R3Pt( in_rcptP2[0], in_rcptP2[1], 0.0 ), UTILSColor( 1.0 - in_rcoColor[0], 1.0 - in_rcoColor[1], 1.0 - in_rcoColor[2] ) );

}//UTILSDotArray::Add

//------------------------------------------------------------------------------------------

void UTILSDotArray::AddPolygon( const Array<R2Pt> &in_apt, const UTILSColor & in_rcoColor )
{
    for ( int i = 0; i < in_apt.num(); i++ )
        Add( in_apt[i], in_apt.wrap(i+1) - in_apt[i], in_rcoColor );

}//UTILSDotArray::Add

//------------------------------------------------------------------------------------------

void UTILSDotArray::AddCurve( const Array<R3Pt> &in_apt, const UTILSColor & in_rcoColor )
{
    for ( int i = 0; i < in_apt.num() - 1; i++ )
        Add( in_apt[i], in_apt[i+1] - in_apt[i], in_rcoColor );
    Add( in_apt.last(), R3Vec(0,0,0), in_rcoColor );

}//UTILSDotArray::Add

//------------------------------------------------------------------------------------------

void UTILSDotArray::AddCurve( const Array<R2Pt> &in_apt, const UTILSColor & in_rcoColor )
{
    for ( int i = 0; i < in_apt.num() - 1; i++ )
        Add( in_apt[i], in_apt[i+1] - in_apt[i], in_rcoColor );
    Add( in_apt.last(), R2Vec(0,0), in_rcoColor );

}//UTILSDotArray::Add

//------------------------------------------------------------------------------------------

void UTILSDotArray::AddPolygon( const Array<R3Pt> &in_apt, const UTILSColor & in_rcoColor )
{
    for ( int i = 0; i < in_apt.num(); i++ )
        Add( in_apt[i], in_apt.wrap(i+1) - in_apt[i], in_rcoColor );

}//UTILSDotArray::Add

//------------------------------------------------------------------------------------------

void UTILSDotArray::RemoveAt( int which ) 
{
    m_aoDots[ which ] = m_aoDots.last(  );
    m_aoDots.sub( 1 );

}//UTILSDotArray::RemoveAt

//------------------------------------------------------------------------------------------

int UTILSDot::m_siNames = 0;

UTILSDotArray::UTILSDotArray(  )
{

}//UTILSDotArray::UTILSDotArray

//------------------------------------------------------------------------------------------

UTILSDotArray::UTILSDotArray( const UTILSDotArray & in_rcoDots )
{ 
    *this = in_rcoDots; 

}//UTILSDotArray::UTILSDotArray

//------------------------------------------------------------------------------------------

int UTILSDotArray::Closest( const R3Pt & in_rcptP )  const
{
    double dMinD = 1e30;
    int iMin = 0;
    for ( int iI = 0; iI < N(  ); iI++ ) {
        double dD = Length( m_aoDots[ iI ].Loc(  ) - in_rcptP );
        if ( dD < dMinD ) {
            iMin = iI;
            dMinD = dD;
        }
    }

    return iMin;  
   
}//UTILSDotArray::Closest

//------------------------------------------------------------------------------------------

void UTILSDotArray::Read( const char* in_rcszName )
{
    ifstream     strmIn;
    
    strmIn.open( in_rcszName, ios::in );
    ASSERT( strmIn.good() );
    Read( strmIn );
    strmIn.close( );

}//UTILSDotArray::Read

//------------------------------------------------------------------------------------------

void UTILSDotArray::Write( const char * in_rcszName ) const
{
    ofstream     strmOut;
    strmOut.open( in_rcszName, ios::out );
    ASSERT( strmOut.good() );
    if ( N() < 30 )
        strmOut.precision(12);
    Write( strmOut );
    strmOut.close( );

}//UTILSDotArray::Write


//------------------------------------------------------------------------------------------

Array <R3Pt> UTILSDotArray::PointsOnly(  ) const
{
    Array <R3Pt> out_pts( N( ) );
    for (int i = 0; i < N( ); i++)
       out_pts[i] = (*this)[i].Loc( );

    return out_pts;

}//UTILSDotArray::PointsOnly


//------------------------------------------------------------------------------------------

