// Strictly for this:
#include "StdAfx.H"
#include "utils/Utils_Io.H"

#ifdef sun
#include <sys/types.h>
#include <netinet/in.h>
#include <inttypes.h>
#endif

#ifdef sgi
#include <sys/types.h>
#include <netinet/in.h>
#include <inttypes.h>
#endif


#ifdef WIN32
#include <winsock2.h>
#endif

static unsigned int s_saiFlags[] = 
{
    1, 2, 4, 8, 16, 32, 64, 128,
    256, 512, 1024, 2048, 4096, 8192, 16384, 32768,
    65536, 131072, 262144, 524288, 1048576, 2097152, 4194304, 8388608,
    16777216, 33554432, 67108864, 134217728,
    268435456, 536870912, 1073741824, 2147483648
};

static int    s_iNDigits = 30;
static double s_dMaxVal = 1.0,  s_dMinVal = -1.0;
static float  s_fMaxVal = 1.0f, s_fMinVal = -1.0f;

void UTILSIo::SetNDigits( int in_iNDigits )
{
    ASSERT( in_iNDigits <= 30 && in_iNDigits > 0 );

    if ( in_iNDigits < 25 )
        cerr << "Small " << in_iNDigits << "\n";

    s_iNDigits = in_iNDigits;
    s_dMaxVal = (double) s_saiFlags[ 30 ] / (double) s_saiFlags[ in_iNDigits ];
    s_dMinVal =  - s_dMaxVal;

    s_fMaxVal = (float) s_saiFlags[ 30 ] / (float) s_saiFlags[ in_iNDigits ];
    s_fMinVal =  - s_fMaxVal;
}

long    
UTILSIo::Convert( int in_i )
{
    return htonl( (long) in_i );
}

long    
UTILSIo::Convert( unsigned int in_i )
{
    return htonl( (long) in_i );
}

long    
UTILSIo::Convert( short in_i )
{
    return htonl( (long) in_i );
}

long    
UTILSIo::Convert( unsigned short in_i )
{
    return htonl( (long) in_i );
}

long    
UTILSIo::Convert( long in_l )
{
    return htonl( in_l );
}


long    
UTILSIo::Convert( float in_f )
{
    long lRes;
    if ( in_f < 0 ) {
        if ( in_f < s_fMinVal ) {
            cerr << "Bad convert " << in_f << " " << s_fMinVal << "\n";
            in_f = s_fMinVal;
        }
        lRes = - ( (long) (-in_f * s_saiFlags[ s_iNDigits ] ) );
        return htonl( lRes );
    }
    if ( in_f > s_fMaxVal ) {
        cerr << "Bad convert " << in_f << " " << s_fMaxVal << "\n";
        in_f = s_fMaxVal;
    }
    lRes = ( (long) (in_f * s_saiFlags[ s_iNDigits ] ) );
    return htonl( lRes );
}

long    
UTILSIo::Convert( double in_d )
{
    long lRes;
    if ( in_d < 0 ) {
        if ( in_d < s_dMinVal ) {
            cerr << "Bad convert " << in_d << " " << s_dMinVal << "\n";
            in_d = s_dMinVal;
        }
        lRes = - ( (long) (-in_d * s_saiFlags[ s_iNDigits ] ) );
        return htonl( lRes );
    }
    if ( in_d > s_dMaxVal ) {
        cerr << "Bad convert " << in_d << " " << s_dMaxVal << "\n";
        in_d = s_dMaxVal;
    }
    lRes = ( (long) (in_d * s_saiFlags[ s_iNDigits ] ) );
    return htonl( lRes );
}

void
UTILSIo::ConvertBack( int &out_i, long in_l )
{
    long lRes = ntohl( in_l );
    out_i = (int) lRes;
}

void
UTILSIo::ConvertBack( unsigned short &out_i, long in_l )
{
    long lRes = ntohl( in_l );
    out_i = (int) lRes;
}

void
UTILSIo::ConvertBack(  short &out_i, long in_l )
{
    long lRes = ntohl( in_l );
    out_i = (int) lRes;
}

void    
UTILSIo::ConvertBack( unsigned int & out_ui, long in_l )
{
    long lRes = ntohl( in_l );
    out_ui = ( unsigned int ) lRes;
}

void    
UTILSIo::ConvertBack( long & out_l, long in_l )
{
    out_l = ntohl( in_l );
}

void
UTILSIo::ConvertBack( float &out_f, long in_l )
{
    long lRes = ntohl( in_l );
    out_f = ( (float) lRes ) / (float) s_saiFlags[ s_iNDigits ];
}

void
UTILSIo::ConvertBack( double &out_d, long in_l )
{
    long lRes = ntohl( in_l );
    out_d = ( (double) lRes ) / (double) s_saiFlags[ s_iNDigits ];
}


void UTILSIo::WriteBinary( float in_f, ofstream &out)
{
    long lRes = Convert( in_f );
    out.write( (char *) &lRes, sizeof(long) );
}

///
WINbool UTILSIo::ReadBinary( float &io_f, ifstream &in)
{
    long lRes;
    in.read( (char *) &lRes, sizeof(long) );
    ConvertBack( io_f, lRes );

    return TRUE;
}

///
void UTILSIo::WriteBinary( double in_d, ofstream &out)
{
    long lRes = Convert( in_d );
    out.write( (char *) &lRes, sizeof(long) );
}

/// 
WINbool UTILSIo::ReadBinary( double &io_d, ifstream &in)
{
    long lRes;
    in.read( (char *) &lRes, sizeof(long) );
    ConvertBack( io_d, lRes );

    return TRUE;
}




/// Find the best value to use then write out the array. ofstream must be opened binary.
WINbool UTILSIo::WriteBinary( const Array<double> & in_ad, 
                              ofstream            & out )
{
    long   alInfo[2];
    long  *alData = new long [ in_ad.num() ];

    alInfo[0] = htonl( (long) in_ad.num() );
    alInfo[1] = htonl( (long) 30 );

    if ( in_ad.num() == 0 ) {
        out.write( (char *) alInfo, 2 * sizeof(long) );
        return TRUE;
    }

    const double dMin = in_ad.min_e( in_ad );
    const double dMax = in_ad.max_e( in_ad );

    double dMaxSize = WINmax( fabs(dMin), fabs(dMax) );

    WINbool bFound = FALSE;
    for ( int i = 30; i >= 1; i--) {
        double dMaxVal = (double) s_saiFlags[ 30 ] / (double) s_saiFlags[ i ];
        if ( dMaxVal >= dMaxSize ) {
            bFound = TRUE;
            SetNDigits( i );
            break;
        }
    }
    if ( bFound == FALSE ) {
        SetNDigits(1);
    }
    
    alInfo[1] = htonl( (long) s_iNDigits );
    for ( FORINT  i = 0; i < in_ad.num(); i++ ) {
        alData[i] = Convert( in_ad[i] );
    }

    out.write( (char *) alInfo, 2 * sizeof(long) );
    out.write( (char *) alData, in_ad.num() * sizeof(long) );
    
    delete [] alData;
    return bFound;
}

/// Find the best value to use then write out the array. ofstream must be opened binary.
WINbool UTILSIo::WriteBinary( const Array<float> & in_af, 
                              ofstream           & out )
{
    long   alInfo[2];
    long  *alData = new long [ in_af.num() ];

    alInfo[0] = htonl( (long) in_af.num() );
    alInfo[1] = htonl( (long) 30 );

    if ( in_af.num() == 0 ) {
        out.write( (char *) alInfo, 2 * sizeof(long) );
        return TRUE;
    }

    float fMin = 1e30f;
    float fMax = -1e30f;

    for (int i = 0; i < in_af.num(); i++) {
        fMin = WINmin( fMin, in_af[i] );
        fMax = WINmax( fMax, in_af[i] );
    }

    fMax = (float)(WINmax( fabs(fMin), fabs(fMax) ) );

    WINbool bFound = FALSE;
    for ( FORINT i = 30; i >= 1; i--) {
        float fMaxVal = (float) s_saiFlags[ 30 ] / (float) s_saiFlags[ i ];
        if ( fMaxVal >= fMax ) {
            bFound = TRUE;
            SetNDigits( i );
            break;
        }
    }
    if ( bFound == FALSE ) {
        SetNDigits(1);
    }
    
    alInfo[0] = htonl( (long) in_af.num() );
    alInfo[1] = htonl( (long) s_iNDigits );
    for ( FORINT  i = 0; i < in_af.num(); i++ ) {
        alData[i] = Convert( in_af[i] );
    }

    out.write( (char *) alInfo, 2 * sizeof(long) );
    out.write( (char *) alData, in_af.num() * sizeof(long) );
    
    delete [] alData;
    return bFound;
}

/// ints and such are no bigger than longs
WINbool UTILSIo::WriteBinary( const Array<int> & in_ai, 
                              ofstream         & out )
{
    long  *alData = new long [ in_ai.num() ];

    for ( int i = 0; i < in_ai.num(); i++ ) {
        alData[i] = Convert( in_ai[i] );
    }

    WriteBinary( in_ai.num(), out );
    out.write( (char *) alData, in_ai.num() * sizeof(long) );
    
    delete [] alData;
    return TRUE;
}

WINbool UTILSIo::WriteBinary( const Array<long> & in_al, 
                              ofstream          & out )
{
    long  *alData = new long [ in_al.num() ];

    for ( int i = 0; i < in_al.num(); i++ ) {
        alData[i] = Convert( in_al[i] );
    }

    WriteBinary( in_al.num(), out );
    out.write( (char *) alData, in_al.num() * sizeof(long) );
    
    delete [] alData;
    return TRUE;
}

WINbool UTILSIo::WriteBinary( const Array<unsigned int> & in_aui, 
                              ofstream                  & out )
{
    long  *alData = new long [ in_aui.num() ];

    for ( int i = 0; i < in_aui.num(); i++ ) {
        alData[i] = Convert( in_aui[i] );
    }

    WriteBinary( in_aui.num(), out );
    out.write( (char *) alData, in_aui.num() * sizeof(long) );
    
    delete [] alData;
    return TRUE;
}

WINbool UTILSIo::WriteBinary( const Array<unsigned short> & in_aui, 
                              ofstream                    & out )
{
    long  *alData = new long [ in_aui.num() ];

    for ( int i = 0; i < in_aui.num(); i++ ) {
        alData[i] = Convert( in_aui[i] );
    }

    WriteBinary( in_aui.num(), out );
    out.write( (char *) alData, in_aui.num() * sizeof(long) );
    
    delete [] alData;
    return TRUE;
}

WINbool UTILSIo::WriteBinary( const Array<short> & in_aui, 
                              ofstream           & out )
{
    long  *alData = new long [ in_aui.num() ];

    for ( int i = 0; i < in_aui.num(); i++ ) {
        alData[i] = Convert( in_aui[i] );
    }

    WriteBinary( in_aui.num(), out );
    out.write( (char *) alData, in_aui.num() * sizeof(long) );
    
    delete [] alData;
    return TRUE;
}

WINbool UTILSIo::WriteBinary(const Array<char> & in_ac, 
                             ofstream          & out)
{
    WriteBinary( in_ac.num(), out );
    if ( in_ac.num() )
        out.write( &in_ac[0], in_ac.num() );

    return TRUE;
}

WINbool UTILSIo::ReadBinary( Array<double> & io_ad, 
                             ifstream      & in)
{
    long    alInfo[2];
    long   *alData = NULL;
    WINbool bRes = TRUE;
    
    in.read( (char *) alInfo, 2 * sizeof(long) );
    if ( in.gcount() != 2 * sizeof(long) ) bRes = FALSE;
    
    int iN = ntohl( alInfo[0] );
    int iNDigits = ntohl( alInfo[1] );
    io_ad.need( iN );
    SetNDigits( iNDigits );
    
    alData = new long [ io_ad.num() ];

    in.read( (char *) alData, io_ad.num() * sizeof(long) );
    if ( in.gcount() != io_ad.num() * sizeof(long) ) bRes = FALSE;
    
    for ( int i = 0; i < io_ad.num(); i++ ) {
        ConvertBack( io_ad[i], alData[i] );
    }

    delete [] alData;
    return bRes;
}

WINbool UTILSIo::ReadBinary( Array<float> & io_af, 
                             ifstream     & in)
{
    long   alInfo[2];
    long  *alData = NULL;
    WINbool bRes = TRUE;
    
    in.read( (char *) alInfo, 2 * sizeof(long) );
    if ( in.gcount() != 2 * sizeof(long) ) bRes = FALSE;
    
    io_af.need( ntohl( alInfo[0] ) );
    SetNDigits( ntohl( alInfo[1] ) );
    
    alData = new long [ io_af.num() ];

    in.read( (char *) alData, io_af.num() * sizeof(long) );
    if ( in.gcount() != io_af.num() * sizeof(long) ) bRes = FALSE;
    
    for ( int i = 0; i < io_af.num(); i++ ) {
         ConvertBack( io_af[i], alData[i] );
    }

    delete [] alData;

    return bRes;
}


WINbool UTILSIo::ReadBinary( Array<int> & io_ai, 
                             ifstream   & in)
{
    int    iN = -1;
    long  *alData = NULL;
    
    WINbool bRes = ReadBinary( iN, in );

    io_ai.need( iN );
    
    alData = new long [ io_ai.num() ];
    in.read( (char *) alData, io_ai.num() * sizeof(long) );
    if ( in.gcount() != io_ai.num() * sizeof(long) ) bRes = FALSE;
    
    for ( int i = 0; i < io_ai.num(); i++ ) {
        ConvertBack( io_ai[i], alData[i] );
    }

    delete [] alData;
    return bRes;
}

WINbool UTILSIo::ReadBinary( Array<long> & io_al, 
                             ifstream    & in)
{
    int    iN = -1;
    long  *alData = NULL;
    
    WINbool bRes = ReadBinary( iN, in );

    io_al.need( iN );
    
    alData = new long [ io_al.num() ];
    in.read( (char *) alData, io_al.num() * sizeof(long) );
    if ( in.gcount() != io_al.num() * sizeof(long) ) bRes = FALSE;
    
    for ( int i = 0; i < io_al.num(); i++ ) {
        ConvertBack( io_al[i], alData[i] );
    }

    delete [] alData;
    return bRes;
}

WINbool UTILSIo::ReadBinary( Array<unsigned int> & io_aui, 
                             ifstream            & in)
{
    int    iN = -1;
    long  *alData = NULL;
    
    WINbool bRes = ReadBinary( iN, in );

    io_aui.need( iN );
    
    alData = new long [ io_aui.num() ];
    in.read( (char *) alData, io_aui.num() * sizeof(long) );
    if ( in.gcount() != io_aui.num() * sizeof(long) ) bRes = FALSE;
    
    for ( int i = 0; i < io_aui.num(); i++ ) {
        ConvertBack( io_aui[i], alData[i] );
    }

    delete [] alData;
    return bRes;
}

WINbool UTILSIo::ReadBinary( Array<unsigned short> & io_aui, 
                             ifstream              & in)
{
    int    iN = -1;
    long  *alData = NULL;
    
    WINbool bRes = ReadBinary( iN, in );

    io_aui.need( iN );
    
    alData = new long [ io_aui.num() ];
    in.read( (char *) alData, io_aui.num() * sizeof(long) );
    if ( in.gcount() != io_aui.num() * sizeof(long) ) bRes = FALSE;
    
    for ( int i = 0; i < io_aui.num(); i++ ) {
        ConvertBack( io_aui[i], alData[i] );
    }

    delete [] alData;
    return bRes;
}

WINbool UTILSIo::ReadBinary( Array<short> & io_aui, 
                             ifstream     & in)
{
    int    iN = -1;
    long  *alData = NULL;
    
    WINbool bRes = ReadBinary( iN, in );

    io_aui.need( iN );
    
    alData = new long [ io_aui.num() ];
    in.read( (char *) alData, io_aui.num() * sizeof(long) );
    if ( in.gcount() != io_aui.num() * sizeof(long) ) bRes = FALSE;
    
    for ( int i = 0; i < io_aui.num(); i++ ) {
        ConvertBack( io_aui[i], alData[i] );
    }

    delete [] alData;
    return bRes;
}

WINbool UTILSIo::ReadBinary( Array<char> & io_ac, 
                             ifstream    & in)
{
    int    iN = -1;
    
    WINbool bRes = ReadBinary( iN, in );

    io_ac.need( iN );

    if ( iN > 0 ) {
        in.read( (char *) &io_ac[0], io_ac.num() );
        if ( in.gcount() != io_ac.num() ) bRes = FALSE;
    }

    return bRes;
}

///
void UTILSIo::WriteBinary( int in_i, ofstream &out)
{
    long l = htonl( (long) in_i );
    out.write( (char *) &l, sizeof(long) );
}

/// 
WINbool UTILSIo::ReadBinary( int & io_i, ifstream &in)
{
    long l;
    in.read( (char *) &l, sizeof(long) );
    io_i = ntohl( l );

    return ( in.gcount() == sizeof(long) ? TRUE : FALSE );
}

///
void UTILSIo::WriteBinary( short in_i, ofstream &out)
{
    long l = htonl( (long) in_i );
    out.write( (char *) &l, sizeof(long) );
}

/// 
WINbool UTILSIo::ReadBinary( short & io_i, ifstream &in)
{
    long l;
    in.read( (char *) &l, sizeof(long) );
    io_i = (short) ntohl( l );

    return ( in.gcount() == sizeof(long) ? TRUE : FALSE );
}

///
void UTILSIo::WriteBinary( unsigned short in_i, ofstream &out)
{
    long l = htonl( (long) in_i );
    out.write( (char *) &l, sizeof(long) );
}

/// 
WINbool UTILSIo::ReadBinary( unsigned short  & io_i, ifstream &in)
{
    long l;
    in.read( (char *) &l, sizeof(long) );
    io_i = (short) ntohl( l );

    return ( in.gcount() == sizeof(long) ? TRUE : FALSE );
}



void 
UTILSIo::TestConversion()
{
    for (int i = 1; i <= 30; i++) {
        double dMaxVal = (double) s_saiFlags[ 30 ] / (double) s_saiFlags[ i ];
        double dMinVal =  - dMaxVal;
        double dResMax = ( ( dMaxVal * s_saiFlags[ i ] ) );
        double dResMin = - ( (-dMinVal * s_saiFlags[ i ] ) );
        long   lResMax = (long) dResMax;
        long   lResMin = (long) dResMin;
        cerr << "min " << dMinVal << " " << dMaxVal << " ";
        cerr << dResMin << " " << dResMax << " " << lResMin << " " << lResMax << "\n";
        cerr << " system " << ntohl( lResMin ) << " " << ntohl( lResMax ) << "\n";
        cerr << " back " << ntohl( htonl( lResMin ) ) << " " << ntohl( htonl( lResMax ) ) << " ";
        cerr <<  ntohl( htonl( lResMin ) ) / (double) s_saiFlags[ i ] << " ";
        cerr <<  ntohl( htonl( lResMax ) ) / (double) s_saiFlags[ i ] << "\n";
    }

    Array<double> adData;
    ifstream in;
    ofstream out;

    adData += 0.1;
    adData += 10.0;
    adData += -11.0;

    for ( FORINT i = 0; i < adData.num(); i++)
        cerr << adData[i] << " " ;
    cerr << "\n";

    WINWriteBinary( WINDataHome("Test/testUTILSWrite.dat"), out );

    out << "MFEmbedding\n";
    VERIFY( WriteBinary( adData, out ) );
    out.close();

    adData.clearcompletely();
    WINReadBinary( WINDataHome("Test/testUTILSWrite.dat"), in );

    char str[255];
    in >> str;
    in.ignore( 5, '\n' );
    cerr << "String " << str << "\n";
    ReadBinary( adData, in );
    in.close();

    for ( FORINT i = 0; i < adData.num(); i++)
        cerr << adData[i] << " " ;
    cerr << "\n";
}

