!*******************************************************************************
!                              INTEL CONFIDENTIAL
!   Copyright(C) 2005 Intel Corporation. All Rights Reserved.
!   The source code contained  or  described herein and all documents related to
!   the source code ("Material") are owned by Intel Corporation or its suppliers
!   or licensors.  Title to the  Material remains with  Intel Corporation or its
!   suppliers and licensors. The Material contains trade secrets and proprietary
!   and  confidential  information of  Intel or its suppliers and licensors. The
!   Material  is  protected  by  worldwide  copyright  and trade secret laws and
!   treaty  provisions. No part of the Material may be used, copied, reproduced,
!   modified, published, uploaded, posted, transmitted, distributed or disclosed
!   in any way without Intel's prior express written permission.
!   No license  under any  patent, copyright, trade secret or other intellectual
!   property right is granted to or conferred upon you by disclosure or delivery
!   of the Materials,  either expressly, by implication, inducement, estoppel or
!   otherwise.  Any  license  under  such  intellectual property  rights must be
!   express and approved by Intel in writing.
!
!*******************************************************************************
!  Content:
!      Intel(R) Math Kernel Library (MKL) F95 interface for LAPACK routines
!*******************************************************************************

MODULE MKL95_PRECISION
	INTEGER, PARAMETER :: SP = KIND(1.0E0)
	INTEGER, PARAMETER :: DP = KIND(1.0D0)
END MODULE MKL95_PRECISION

MODULE MKL95_LAPACK
INTERFACE GETRF
	SUBROUTINE SGETRF_MKL95(A,IPIV,INFO)
		! MKL Fortran77 call:
		! SGETRF(M,N,A,LDA,IPIV,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE SGETRF_MKL95
	SUBROUTINE DGETRF_MKL95(A,IPIV,INFO)
		! MKL Fortran77 call:
		! DGETRF(M,N,A,LDA,IPIV,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE DGETRF_MKL95
	SUBROUTINE CGETRF_MKL95(A,IPIV,INFO)
		! MKL Fortran77 call:
		! CGETRF(M,N,A,LDA,IPIV,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE CGETRF_MKL95
	SUBROUTINE ZGETRF_MKL95(A,IPIV,INFO)
		! MKL Fortran77 call:
		! ZGETRF(M,N,A,LDA,IPIV,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE ZGETRF_MKL95
END INTERFACE GETRF
INTERFACE GBTRF
	SUBROUTINE SGBTRF_MKL95(A,KL,M,IPIV,INFO)
		! MKL Fortran77 call:
		! SGBTRF(M,N,KL,KU,A,LDA,IPIV,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		INTEGER, INTENT(IN), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE SGBTRF_MKL95
	SUBROUTINE DGBTRF_MKL95(A,KL,M,IPIV,INFO)
		! MKL Fortran77 call:
		! DGBTRF(M,N,KL,KU,A,LDA,IPIV,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		INTEGER, INTENT(IN), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE DGBTRF_MKL95
	SUBROUTINE CGBTRF_MKL95(A,KL,M,IPIV,INFO)
		! MKL Fortran77 call:
		! CGBTRF(M,N,KL,KU,A,LDA,IPIV,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		INTEGER, INTENT(IN), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE CGBTRF_MKL95
	SUBROUTINE ZGBTRF_MKL95(A,KL,M,IPIV,INFO)
		! MKL Fortran77 call:
		! ZGBTRF(M,N,KL,KU,A,LDA,IPIV,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		INTEGER, INTENT(IN), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE ZGBTRF_MKL95
END INTERFACE GBTRF
INTERFACE GTTRF
	SUBROUTINE SGTTRF_MKL95(DL,D,DU,DU2,IPIV,INFO)
		! MKL Fortran77 call:
		! SGTTRF(N,DL,D,DU,DU2,IPIV,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: DL(:)
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: DU(:)
		REAL(WP), INTENT(OUT) :: DU2(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE SGTTRF_MKL95
	SUBROUTINE DGTTRF_MKL95(DL,D,DU,DU2,IPIV,INFO)
		! MKL Fortran77 call:
		! DGTTRF(N,DL,D,DU,DU2,IPIV,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: DL(:)
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: DU(:)
		REAL(WP), INTENT(OUT) :: DU2(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE DGTTRF_MKL95
	SUBROUTINE CGTTRF_MKL95(DL,D,DU,DU2,IPIV,INFO)
		! MKL Fortran77 call:
		! CGTTRF(N,DL,D,DU,DU2,IPIV,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: DL(:)
		COMPLEX(WP), INTENT(INOUT) :: D(:)
		COMPLEX(WP), INTENT(INOUT) :: DU(:)
		COMPLEX(WP), INTENT(OUT) :: DU2(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE CGTTRF_MKL95
	SUBROUTINE ZGTTRF_MKL95(DL,D,DU,DU2,IPIV,INFO)
		! MKL Fortran77 call:
		! ZGTTRF(N,DL,D,DU,DU2,IPIV,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: DL(:)
		COMPLEX(WP), INTENT(INOUT) :: D(:)
		COMPLEX(WP), INTENT(INOUT) :: DU(:)
		COMPLEX(WP), INTENT(OUT) :: DU2(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE ZGTTRF_MKL95
END INTERFACE GTTRF
INTERFACE POTRF
	SUBROUTINE SPOTRF_MKL95(A,UPLO,INFO)
		! MKL Fortran77 call:
		! SPOTRF(UPLO,N,A,LDA,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
	END SUBROUTINE SPOTRF_MKL95
	SUBROUTINE DPOTRF_MKL95(A,UPLO,INFO)
		! MKL Fortran77 call:
		! DPOTRF(UPLO,N,A,LDA,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
	END SUBROUTINE DPOTRF_MKL95
	SUBROUTINE CPOTRF_MKL95(A,UPLO,INFO)
		! MKL Fortran77 call:
		! CPOTRF(UPLO,N,A,LDA,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
	END SUBROUTINE CPOTRF_MKL95
	SUBROUTINE ZPOTRF_MKL95(A,UPLO,INFO)
		! MKL Fortran77 call:
		! ZPOTRF(UPLO,N,A,LDA,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
	END SUBROUTINE ZPOTRF_MKL95
END INTERFACE POTRF
INTERFACE PPTRF
	SUBROUTINE SPPTRF_MKL95(A,UPLO,INFO)
		! MKL Fortran77 call:
		! SPPTRF(UPLO,N,A,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
	END SUBROUTINE SPPTRF_MKL95
	SUBROUTINE DPPTRF_MKL95(A,UPLO,INFO)
		! MKL Fortran77 call:
		! DPPTRF(UPLO,N,A,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
	END SUBROUTINE DPPTRF_MKL95
	SUBROUTINE CPPTRF_MKL95(A,UPLO,INFO)
		! MKL Fortran77 call:
		! CPPTRF(UPLO,N,A,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
	END SUBROUTINE CPPTRF_MKL95
	SUBROUTINE ZPPTRF_MKL95(A,UPLO,INFO)
		! MKL Fortran77 call:
		! ZPPTRF(UPLO,N,A,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
	END SUBROUTINE ZPPTRF_MKL95
END INTERFACE PPTRF
INTERFACE PBTRF
	SUBROUTINE SPBTRF_MKL95(A,UPLO,INFO)
		! MKL Fortran77 call:
		! SPBTRF(UPLO,N,KD,A,LDA,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
	END SUBROUTINE SPBTRF_MKL95
	SUBROUTINE DPBTRF_MKL95(A,UPLO,INFO)
		! MKL Fortran77 call:
		! DPBTRF(UPLO,N,KD,A,LDA,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
	END SUBROUTINE DPBTRF_MKL95
	SUBROUTINE CPBTRF_MKL95(A,UPLO,INFO)
		! MKL Fortran77 call:
		! CPBTRF(UPLO,N,KD,A,LDA,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
	END SUBROUTINE CPBTRF_MKL95
	SUBROUTINE ZPBTRF_MKL95(A,UPLO,INFO)
		! MKL Fortran77 call:
		! ZPBTRF(UPLO,N,KD,A,LDA,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
	END SUBROUTINE ZPBTRF_MKL95
END INTERFACE PBTRF
INTERFACE PTTRF
	SUBROUTINE SPTTRF_MKL95(D,E,INFO)
		! MKL Fortran77 call:
		! SPTTRF(N,D,E,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
	END SUBROUTINE SPTTRF_MKL95
	SUBROUTINE DPTTRF_MKL95(D,E,INFO)
		! MKL Fortran77 call:
		! DPTTRF(N,D,E,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
	END SUBROUTINE DPTTRF_MKL95
	SUBROUTINE CPTTRF_MKL95(D,E,INFO)
		! MKL Fortran77 call:
		! CPTTRF(N,D,E,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		COMPLEX(WP), INTENT(INOUT) :: E(:)
	END SUBROUTINE CPTTRF_MKL95
	SUBROUTINE ZPTTRF_MKL95(D,E,INFO)
		! MKL Fortran77 call:
		! ZPTTRF(N,D,E,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		COMPLEX(WP), INTENT(INOUT) :: E(:)
	END SUBROUTINE ZPTTRF_MKL95
END INTERFACE PTTRF
INTERFACE SYTRF
	SUBROUTINE SSYTRF_MKL95(A,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! SSYTRF(UPLO,N,A,LDA,IPIV,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE SSYTRF_MKL95
	SUBROUTINE DSYTRF_MKL95(A,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! DSYTRF(UPLO,N,A,LDA,IPIV,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE DSYTRF_MKL95
	SUBROUTINE CSYTRF_MKL95(A,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! CSYTRF(UPLO,N,A,LDA,IPIV,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE CSYTRF_MKL95
	SUBROUTINE ZSYTRF_MKL95(A,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! ZSYTRF(UPLO,N,A,LDA,IPIV,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE ZSYTRF_MKL95
END INTERFACE SYTRF
INTERFACE HETRF
	SUBROUTINE CHETRF_MKL95(A,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! CHETRF(UPLO,N,A,LDA,IPIV,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE CHETRF_MKL95
	SUBROUTINE ZHETRF_MKL95(A,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! ZHETRF(UPLO,N,A,LDA,IPIV,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE ZHETRF_MKL95
END INTERFACE HETRF
INTERFACE SPTRF
	SUBROUTINE SSPTRF_MKL95(A,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! SSPTRF(UPLO,N,A,IPIV,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE SSPTRF_MKL95
	SUBROUTINE DSPTRF_MKL95(A,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! DSPTRF(UPLO,N,A,IPIV,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE DSPTRF_MKL95
	SUBROUTINE CSPTRF_MKL95(A,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! CSPTRF(UPLO,N,A,IPIV,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE CSPTRF_MKL95
	SUBROUTINE ZSPTRF_MKL95(A,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! ZSPTRF(UPLO,N,A,IPIV,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE ZSPTRF_MKL95
END INTERFACE SPTRF
INTERFACE HPTRF
	SUBROUTINE CHPTRF_MKL95(A,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! CHPTRF(UPLO,N,A,IPIV,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE CHPTRF_MKL95
	SUBROUTINE ZHPTRF_MKL95(A,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! ZHPTRF(UPLO,N,A,IPIV,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE ZHPTRF_MKL95
END INTERFACE HPTRF
INTERFACE GETRS
	SUBROUTINE SGETRS_MKL95(A,IPIV,B,TRANS,INFO)
		! MKL Fortran77 call:
		! SGETRS(TRANS,N,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE SGETRS_MKL95
	SUBROUTINE SGETRS1_MKL95(A,IPIV,B,TRANS,INFO)
		! MKL Fortran77 call:
		! SGETRS(TRANS,N,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		REAL(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE SGETRS1_MKL95
	SUBROUTINE DGETRS_MKL95(A,IPIV,B,TRANS,INFO)
		! MKL Fortran77 call:
		! DGETRS(TRANS,N,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE DGETRS_MKL95
	SUBROUTINE DGETRS1_MKL95(A,IPIV,B,TRANS,INFO)
		! MKL Fortran77 call:
		! DGETRS(TRANS,N,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		REAL(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE DGETRS1_MKL95
	SUBROUTINE CGETRS_MKL95(A,IPIV,B,TRANS,INFO)
		! MKL Fortran77 call:
		! CGETRS(TRANS,N,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE CGETRS_MKL95
	SUBROUTINE CGETRS1_MKL95(A,IPIV,B,TRANS,INFO)
		! MKL Fortran77 call:
		! CGETRS(TRANS,N,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE CGETRS1_MKL95
	SUBROUTINE ZGETRS_MKL95(A,IPIV,B,TRANS,INFO)
		! MKL Fortran77 call:
		! ZGETRS(TRANS,N,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE ZGETRS_MKL95
	SUBROUTINE ZGETRS1_MKL95(A,IPIV,B,TRANS,INFO)
		! MKL Fortran77 call:
		! ZGETRS(TRANS,N,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE ZGETRS1_MKL95
END INTERFACE GETRS
INTERFACE GBTRS
	SUBROUTINE SGBTRS_MKL95(A,B,IPIV,KL,TRANS,INFO)
		! MKL Fortran77 call:
		! SGBTRS(TRANS,N,KL,KU,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE SGBTRS_MKL95
	SUBROUTINE SGBTRS1_MKL95(A,B,IPIV,KL,TRANS,INFO)
		! MKL Fortran77 call:
		! SGBTRS(TRANS,N,KL,KU,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE SGBTRS1_MKL95
	SUBROUTINE DGBTRS_MKL95(A,B,IPIV,KL,TRANS,INFO)
		! MKL Fortran77 call:
		! DGBTRS(TRANS,N,KL,KU,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE DGBTRS_MKL95
	SUBROUTINE DGBTRS1_MKL95(A,B,IPIV,KL,TRANS,INFO)
		! MKL Fortran77 call:
		! DGBTRS(TRANS,N,KL,KU,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE DGBTRS1_MKL95
	SUBROUTINE CGBTRS_MKL95(A,B,IPIV,KL,TRANS,INFO)
		! MKL Fortran77 call:
		! CGBTRS(TRANS,N,KL,KU,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE CGBTRS_MKL95
	SUBROUTINE CGBTRS1_MKL95(A,B,IPIV,KL,TRANS,INFO)
		! MKL Fortran77 call:
		! CGBTRS(TRANS,N,KL,KU,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE CGBTRS1_MKL95
	SUBROUTINE ZGBTRS_MKL95(A,B,IPIV,KL,TRANS,INFO)
		! MKL Fortran77 call:
		! ZGBTRS(TRANS,N,KL,KU,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE ZGBTRS_MKL95
	SUBROUTINE ZGBTRS1_MKL95(A,B,IPIV,KL,TRANS,INFO)
		! MKL Fortran77 call:
		! ZGBTRS(TRANS,N,KL,KU,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE ZGBTRS1_MKL95
END INTERFACE GBTRS
INTERFACE GTTRS
	SUBROUTINE SGTTRS_MKL95(DL,D,DU,DU2,B,IPIV,TRANS,INFO)
		! MKL Fortran77 call:
		! SGTTRS(TRANS,N,NRHS,DL,D,DU,DU2,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: DL(:)
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: DU(:)
		REAL(WP), INTENT(IN) :: DU2(:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE SGTTRS_MKL95
	SUBROUTINE SGTTRS1_MKL95(DL,D,DU,DU2,B,IPIV,TRANS,INFO)
		! MKL Fortran77 call:
		! SGTTRS(TRANS,N,NRHS,DL,D,DU,DU2,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: DL(:)
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: DU(:)
		REAL(WP), INTENT(IN) :: DU2(:)
		REAL(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE SGTTRS1_MKL95
	SUBROUTINE DGTTRS_MKL95(DL,D,DU,DU2,B,IPIV,TRANS,INFO)
		! MKL Fortran77 call:
		! DGTTRS(TRANS,N,NRHS,DL,D,DU,DU2,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: DL(:)
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: DU(:)
		REAL(WP), INTENT(IN) :: DU2(:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE DGTTRS_MKL95
	SUBROUTINE DGTTRS1_MKL95(DL,D,DU,DU2,B,IPIV,TRANS,INFO)
		! MKL Fortran77 call:
		! DGTTRS(TRANS,N,NRHS,DL,D,DU,DU2,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: DL(:)
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: DU(:)
		REAL(WP), INTENT(IN) :: DU2(:)
		REAL(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE DGTTRS1_MKL95
	SUBROUTINE CGTTRS_MKL95(DL,D,DU,DU2,B,IPIV,TRANS,INFO)
		! MKL Fortran77 call:
		! CGTTRS(TRANS,N,NRHS,DL,D,DU,DU2,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: DL(:)
		COMPLEX(WP), INTENT(IN) :: D(:)
		COMPLEX(WP), INTENT(IN) :: DU(:)
		COMPLEX(WP), INTENT(IN) :: DU2(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE CGTTRS_MKL95
	SUBROUTINE CGTTRS1_MKL95(DL,D,DU,DU2,B,IPIV,TRANS,INFO)
		! MKL Fortran77 call:
		! CGTTRS(TRANS,N,NRHS,DL,D,DU,DU2,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: DL(:)
		COMPLEX(WP), INTENT(IN) :: D(:)
		COMPLEX(WP), INTENT(IN) :: DU(:)
		COMPLEX(WP), INTENT(IN) :: DU2(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE CGTTRS1_MKL95
	SUBROUTINE ZGTTRS_MKL95(DL,D,DU,DU2,B,IPIV,TRANS,INFO)
		! MKL Fortran77 call:
		! ZGTTRS(TRANS,N,NRHS,DL,D,DU,DU2,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: DL(:)
		COMPLEX(WP), INTENT(IN) :: D(:)
		COMPLEX(WP), INTENT(IN) :: DU(:)
		COMPLEX(WP), INTENT(IN) :: DU2(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE ZGTTRS_MKL95
	SUBROUTINE ZGTTRS1_MKL95(DL,D,DU,DU2,B,IPIV,TRANS,INFO)
		! MKL Fortran77 call:
		! ZGTTRS(TRANS,N,NRHS,DL,D,DU,DU2,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: DL(:)
		COMPLEX(WP), INTENT(IN) :: D(:)
		COMPLEX(WP), INTENT(IN) :: DU(:)
		COMPLEX(WP), INTENT(IN) :: DU2(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE ZGTTRS1_MKL95
END INTERFACE GTTRS
INTERFACE POTRS
	SUBROUTINE SPOTRS_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! SPOTRS(UPLO,N,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE SPOTRS_MKL95
	SUBROUTINE SPOTRS1_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! SPOTRS(UPLO,N,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE SPOTRS1_MKL95
	SUBROUTINE DPOTRS_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! DPOTRS(UPLO,N,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE DPOTRS_MKL95
	SUBROUTINE DPOTRS1_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! DPOTRS(UPLO,N,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE DPOTRS1_MKL95
	SUBROUTINE CPOTRS_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! CPOTRS(UPLO,N,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE CPOTRS_MKL95
	SUBROUTINE CPOTRS1_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! CPOTRS(UPLO,N,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE CPOTRS1_MKL95
	SUBROUTINE ZPOTRS_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! ZPOTRS(UPLO,N,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE ZPOTRS_MKL95
	SUBROUTINE ZPOTRS1_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! ZPOTRS(UPLO,N,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE ZPOTRS1_MKL95
END INTERFACE POTRS
INTERFACE PPTRS
	SUBROUTINE SPPTRS_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! SPPTRS(UPLO,N,NRHS,A,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE SPPTRS_MKL95
	SUBROUTINE SPPTRS1_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! SPPTRS(UPLO,N,NRHS,A,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE SPPTRS1_MKL95
	SUBROUTINE DPPTRS_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! DPPTRS(UPLO,N,NRHS,A,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE DPPTRS_MKL95
	SUBROUTINE DPPTRS1_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! DPPTRS(UPLO,N,NRHS,A,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE DPPTRS1_MKL95
	SUBROUTINE CPPTRS_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! CPPTRS(UPLO,N,NRHS,A,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE CPPTRS_MKL95
	SUBROUTINE CPPTRS1_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! CPPTRS(UPLO,N,NRHS,A,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE CPPTRS1_MKL95
	SUBROUTINE ZPPTRS_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! ZPPTRS(UPLO,N,NRHS,A,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE ZPPTRS_MKL95
	SUBROUTINE ZPPTRS1_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! ZPPTRS(UPLO,N,NRHS,A,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE ZPPTRS1_MKL95
END INTERFACE PPTRS
INTERFACE PBTRS
	SUBROUTINE SPBTRS_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! SPBTRS(UPLO,N,KD,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE SPBTRS_MKL95
	SUBROUTINE SPBTRS1_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! SPBTRS(UPLO,N,KD,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE SPBTRS1_MKL95
	SUBROUTINE DPBTRS_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! DPBTRS(UPLO,N,KD,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE DPBTRS_MKL95
	SUBROUTINE DPBTRS1_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! DPBTRS(UPLO,N,KD,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE DPBTRS1_MKL95
	SUBROUTINE CPBTRS_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! CPBTRS(UPLO,N,KD,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE CPBTRS_MKL95
	SUBROUTINE CPBTRS1_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! CPBTRS(UPLO,N,KD,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE CPBTRS1_MKL95
	SUBROUTINE ZPBTRS_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! ZPBTRS(UPLO,N,KD,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE ZPBTRS_MKL95
	SUBROUTINE ZPBTRS1_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! ZPBTRS(UPLO,N,KD,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE ZPBTRS1_MKL95
END INTERFACE PBTRS
INTERFACE PTTRS
	SUBROUTINE SPTTRS_MKL95(D,E,B,INFO)
		! MKL Fortran77 call:
		! SPTTRS(N,NRHS,D,E,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: E(:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE SPTTRS_MKL95
	SUBROUTINE SPTTRS1_MKL95(D,E,B,INFO)
		! MKL Fortran77 call:
		! SPTTRS(N,NRHS,D,E,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: E(:)
		REAL(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE SPTTRS1_MKL95
	SUBROUTINE DPTTRS_MKL95(D,E,B,INFO)
		! MKL Fortran77 call:
		! DPTTRS(N,NRHS,D,E,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: E(:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE DPTTRS_MKL95
	SUBROUTINE DPTTRS1_MKL95(D,E,B,INFO)
		! MKL Fortran77 call:
		! DPTTRS(N,NRHS,D,E,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: E(:)
		REAL(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE DPTTRS1_MKL95
	SUBROUTINE CPTTRS_MKL95(D,E,B,UPLO,INFO)
		! MKL Fortran77 call:
		! CPTTRS(UPLO,N,NRHS,D,E,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		COMPLEX(WP), INTENT(IN) :: E(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE CPTTRS_MKL95
	SUBROUTINE CPTTRS1_MKL95(D,E,B,UPLO,INFO)
		! MKL Fortran77 call:
		! CPTTRS(UPLO,N,NRHS,D,E,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		COMPLEX(WP), INTENT(IN) :: E(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE CPTTRS1_MKL95
	SUBROUTINE ZPTTRS_MKL95(D,E,B,UPLO,INFO)
		! MKL Fortran77 call:
		! ZPTTRS(UPLO,N,NRHS,D,E,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		COMPLEX(WP), INTENT(IN) :: E(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE ZPTTRS_MKL95
	SUBROUTINE ZPTTRS1_MKL95(D,E,B,UPLO,INFO)
		! MKL Fortran77 call:
		! ZPTTRS(UPLO,N,NRHS,D,E,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		COMPLEX(WP), INTENT(IN) :: E(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE ZPTTRS1_MKL95
END INTERFACE PTTRS
INTERFACE SYTRS
	SUBROUTINE SSYTRS_MKL95(A,B,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! SSYTRS(UPLO,N,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE SSYTRS_MKL95
	SUBROUTINE SSYTRS1_MKL95(A,B,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! SSYTRS(UPLO,N,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE SSYTRS1_MKL95
	SUBROUTINE DSYTRS_MKL95(A,B,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! DSYTRS(UPLO,N,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE DSYTRS_MKL95
	SUBROUTINE DSYTRS1_MKL95(A,B,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! DSYTRS(UPLO,N,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE DSYTRS1_MKL95
	SUBROUTINE CSYTRS_MKL95(A,B,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! CSYTRS(UPLO,N,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE CSYTRS_MKL95
	SUBROUTINE CSYTRS1_MKL95(A,B,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! CSYTRS(UPLO,N,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE CSYTRS1_MKL95
	SUBROUTINE ZSYTRS_MKL95(A,B,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! ZSYTRS(UPLO,N,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE ZSYTRS_MKL95
	SUBROUTINE ZSYTRS1_MKL95(A,B,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! ZSYTRS(UPLO,N,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE ZSYTRS1_MKL95
END INTERFACE SYTRS
INTERFACE HETRS
	SUBROUTINE CHETRS_MKL95(A,B,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! CHETRS(UPLO,N,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE CHETRS_MKL95
	SUBROUTINE CHETRS1_MKL95(A,B,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! CHETRS(UPLO,N,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE CHETRS1_MKL95
	SUBROUTINE ZHETRS_MKL95(A,B,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! ZHETRS(UPLO,N,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE ZHETRS_MKL95
	SUBROUTINE ZHETRS1_MKL95(A,B,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! ZHETRS(UPLO,N,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE ZHETRS1_MKL95
END INTERFACE HETRS
INTERFACE SPTRS
	SUBROUTINE SSPTRS_MKL95(A,B,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! SSPTRS(UPLO,N,NRHS,A,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE SSPTRS_MKL95
	SUBROUTINE SSPTRS1_MKL95(A,B,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! SSPTRS(UPLO,N,NRHS,A,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE SSPTRS1_MKL95
	SUBROUTINE DSPTRS_MKL95(A,B,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! DSPTRS(UPLO,N,NRHS,A,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE DSPTRS_MKL95
	SUBROUTINE DSPTRS1_MKL95(A,B,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! DSPTRS(UPLO,N,NRHS,A,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE DSPTRS1_MKL95
	SUBROUTINE CSPTRS_MKL95(A,B,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! CSPTRS(UPLO,N,NRHS,A,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE CSPTRS_MKL95
	SUBROUTINE CSPTRS1_MKL95(A,B,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! CSPTRS(UPLO,N,NRHS,A,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE CSPTRS1_MKL95
	SUBROUTINE ZSPTRS_MKL95(A,B,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! ZSPTRS(UPLO,N,NRHS,A,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE ZSPTRS_MKL95
	SUBROUTINE ZSPTRS1_MKL95(A,B,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! ZSPTRS(UPLO,N,NRHS,A,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE ZSPTRS1_MKL95
END INTERFACE SPTRS
INTERFACE HPTRS
	SUBROUTINE CHPTRS_MKL95(A,B,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! CHPTRS(UPLO,N,NRHS,A,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE CHPTRS_MKL95
	SUBROUTINE CHPTRS1_MKL95(A,B,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! CHPTRS(UPLO,N,NRHS,A,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE CHPTRS1_MKL95
	SUBROUTINE ZHPTRS_MKL95(A,B,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! ZHPTRS(UPLO,N,NRHS,A,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE ZHPTRS_MKL95
	SUBROUTINE ZHPTRS1_MKL95(A,B,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! ZHPTRS(UPLO,N,NRHS,A,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE ZHPTRS1_MKL95
END INTERFACE HPTRS
INTERFACE TRTRS
	SUBROUTINE STRTRS_MKL95(A,B,UPLO,TRANS,DIAG,INFO)
		! MKL Fortran77 call:
		! STRTRS(UPLO,TRANS,DIAG,N,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE STRTRS_MKL95
	SUBROUTINE STRTRS1_MKL95(A,B,UPLO,TRANS,DIAG,INFO)
		! MKL Fortran77 call:
		! STRTRS(UPLO,TRANS,DIAG,N,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE STRTRS1_MKL95
	SUBROUTINE DTRTRS_MKL95(A,B,UPLO,TRANS,DIAG,INFO)
		! MKL Fortran77 call:
		! DTRTRS(UPLO,TRANS,DIAG,N,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE DTRTRS_MKL95
	SUBROUTINE DTRTRS1_MKL95(A,B,UPLO,TRANS,DIAG,INFO)
		! MKL Fortran77 call:
		! DTRTRS(UPLO,TRANS,DIAG,N,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE DTRTRS1_MKL95
	SUBROUTINE CTRTRS_MKL95(A,B,UPLO,TRANS,DIAG,INFO)
		! MKL Fortran77 call:
		! CTRTRS(UPLO,TRANS,DIAG,N,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE CTRTRS_MKL95
	SUBROUTINE CTRTRS1_MKL95(A,B,UPLO,TRANS,DIAG,INFO)
		! MKL Fortran77 call:
		! CTRTRS(UPLO,TRANS,DIAG,N,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE CTRTRS1_MKL95
	SUBROUTINE ZTRTRS_MKL95(A,B,UPLO,TRANS,DIAG,INFO)
		! MKL Fortran77 call:
		! ZTRTRS(UPLO,TRANS,DIAG,N,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE ZTRTRS_MKL95
	SUBROUTINE ZTRTRS1_MKL95(A,B,UPLO,TRANS,DIAG,INFO)
		! MKL Fortran77 call:
		! ZTRTRS(UPLO,TRANS,DIAG,N,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE ZTRTRS1_MKL95
END INTERFACE TRTRS
INTERFACE TPTRS
	SUBROUTINE STPTRS_MKL95(A,B,UPLO,TRANS,DIAG,INFO)
		! MKL Fortran77 call:
		! STPTRS(UPLO,TRANS,DIAG,N,NRHS,A,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE STPTRS_MKL95
	SUBROUTINE STPTRS1_MKL95(A,B,UPLO,TRANS,DIAG,INFO)
		! MKL Fortran77 call:
		! STPTRS(UPLO,TRANS,DIAG,N,NRHS,A,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE STPTRS1_MKL95
	SUBROUTINE DTPTRS_MKL95(A,B,UPLO,TRANS,DIAG,INFO)
		! MKL Fortran77 call:
		! DTPTRS(UPLO,TRANS,DIAG,N,NRHS,A,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE DTPTRS_MKL95
	SUBROUTINE DTPTRS1_MKL95(A,B,UPLO,TRANS,DIAG,INFO)
		! MKL Fortran77 call:
		! DTPTRS(UPLO,TRANS,DIAG,N,NRHS,A,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE DTPTRS1_MKL95
	SUBROUTINE CTPTRS_MKL95(A,B,UPLO,TRANS,DIAG,INFO)
		! MKL Fortran77 call:
		! CTPTRS(UPLO,TRANS,DIAG,N,NRHS,A,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE CTPTRS_MKL95
	SUBROUTINE CTPTRS1_MKL95(A,B,UPLO,TRANS,DIAG,INFO)
		! MKL Fortran77 call:
		! CTPTRS(UPLO,TRANS,DIAG,N,NRHS,A,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE CTPTRS1_MKL95
	SUBROUTINE ZTPTRS_MKL95(A,B,UPLO,TRANS,DIAG,INFO)
		! MKL Fortran77 call:
		! ZTPTRS(UPLO,TRANS,DIAG,N,NRHS,A,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE ZTPTRS_MKL95
	SUBROUTINE ZTPTRS1_MKL95(A,B,UPLO,TRANS,DIAG,INFO)
		! MKL Fortran77 call:
		! ZTPTRS(UPLO,TRANS,DIAG,N,NRHS,A,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE ZTPTRS1_MKL95
END INTERFACE TPTRS
INTERFACE TBTRS
	SUBROUTINE STBTRS_MKL95(A,B,UPLO,TRANS,DIAG,INFO)
		! MKL Fortran77 call:
		! STBTRS(UPLO,TRANS,DIAG,N,KD,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE STBTRS_MKL95
	SUBROUTINE STBTRS1_MKL95(A,B,UPLO,TRANS,DIAG,INFO)
		! MKL Fortran77 call:
		! STBTRS(UPLO,TRANS,DIAG,N,KD,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE STBTRS1_MKL95
	SUBROUTINE DTBTRS_MKL95(A,B,UPLO,TRANS,DIAG,INFO)
		! MKL Fortran77 call:
		! DTBTRS(UPLO,TRANS,DIAG,N,KD,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE DTBTRS_MKL95
	SUBROUTINE DTBTRS1_MKL95(A,B,UPLO,TRANS,DIAG,INFO)
		! MKL Fortran77 call:
		! DTBTRS(UPLO,TRANS,DIAG,N,KD,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE DTBTRS1_MKL95
	SUBROUTINE CTBTRS_MKL95(A,B,UPLO,TRANS,DIAG,INFO)
		! MKL Fortran77 call:
		! CTBTRS(UPLO,TRANS,DIAG,N,KD,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE CTBTRS_MKL95
	SUBROUTINE CTBTRS1_MKL95(A,B,UPLO,TRANS,DIAG,INFO)
		! MKL Fortran77 call:
		! CTBTRS(UPLO,TRANS,DIAG,N,KD,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE CTBTRS1_MKL95
	SUBROUTINE ZTBTRS_MKL95(A,B,UPLO,TRANS,DIAG,INFO)
		! MKL Fortran77 call:
		! ZTBTRS(UPLO,TRANS,DIAG,N,KD,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE ZTBTRS_MKL95
	SUBROUTINE ZTBTRS1_MKL95(A,B,UPLO,TRANS,DIAG,INFO)
		! MKL Fortran77 call:
		! ZTBTRS(UPLO,TRANS,DIAG,N,KD,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE ZTBTRS1_MKL95
END INTERFACE TBTRS
INTERFACE GECON
	SUBROUTINE SGECON_MKL95(A,ANORM,RCOND,NORM,INFO)
		! MKL Fortran77 call:
		! SGECON(NORM,N,A,LDA,ANORM,RCOND,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
	END SUBROUTINE SGECON_MKL95
	SUBROUTINE DGECON_MKL95(A,ANORM,RCOND,NORM,INFO)
		! MKL Fortran77 call:
		! DGECON(NORM,N,A,LDA,ANORM,RCOND,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
	END SUBROUTINE DGECON_MKL95
	SUBROUTINE CGECON_MKL95(A,ANORM,RCOND,NORM,INFO)
		! MKL Fortran77 call:
		! CGECON(NORM,N,A,LDA,ANORM,RCOND,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
	END SUBROUTINE CGECON_MKL95
	SUBROUTINE ZGECON_MKL95(A,ANORM,RCOND,NORM,INFO)
		! MKL Fortran77 call:
		! ZGECON(NORM,N,A,LDA,ANORM,RCOND,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
	END SUBROUTINE ZGECON_MKL95
END INTERFACE GECON
INTERFACE GBCON
	SUBROUTINE SGBCON_MKL95(A,IPIV,ANORM,RCOND,KL,NORM,INFO)
		! MKL Fortran77 call:
		! SGBCON(NORM,N,KL,KU,A,LDA,IPIV,ANORM,RCOND,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		INTEGER, INTENT(IN), OPTIONAL :: KL
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE SGBCON_MKL95
	SUBROUTINE DGBCON_MKL95(A,IPIV,ANORM,RCOND,KL,NORM,INFO)
		! MKL Fortran77 call:
		! DGBCON(NORM,N,KL,KU,A,LDA,IPIV,ANORM,RCOND,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		INTEGER, INTENT(IN), OPTIONAL :: KL
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE DGBCON_MKL95
	SUBROUTINE CGBCON_MKL95(A,IPIV,ANORM,RCOND,KL,NORM,INFO)
		! MKL Fortran77 call:
		! CGBCON(NORM,N,KL,KU,A,LDA,IPIV,ANORM,RCOND,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		INTEGER, INTENT(IN), OPTIONAL :: KL
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE CGBCON_MKL95
	SUBROUTINE ZGBCON_MKL95(A,IPIV,ANORM,RCOND,KL,NORM,INFO)
		! MKL Fortran77 call:
		! ZGBCON(NORM,N,KL,KU,A,LDA,IPIV,ANORM,RCOND,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		INTEGER, INTENT(IN), OPTIONAL :: KL
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE ZGBCON_MKL95
END INTERFACE GBCON
INTERFACE GTCON
	SUBROUTINE SGTCON_MKL95(DL,D,DU,DU2,IPIV,ANORM,RCOND,NORM,INFO)
		! MKL Fortran77 call:
		! SGTCON(NORM,N,DL,D,DU,DU2,IPIV,ANORM,RCOND,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: DL(:)
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: DU(:)
		REAL(WP), INTENT(IN) :: DU2(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE SGTCON_MKL95
	SUBROUTINE DGTCON_MKL95(DL,D,DU,DU2,IPIV,ANORM,RCOND,NORM,INFO)
		! MKL Fortran77 call:
		! DGTCON(NORM,N,DL,D,DU,DU2,IPIV,ANORM,RCOND,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: DL(:)
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: DU(:)
		REAL(WP), INTENT(IN) :: DU2(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE DGTCON_MKL95
	SUBROUTINE CGTCON_MKL95(DL,D,DU,DU2,IPIV,ANORM,RCOND,NORM,INFO)
		! MKL Fortran77 call:
		! CGTCON(NORM,N,DL,D,DU,DU2,IPIV,ANORM,RCOND,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: DL(:)
		COMPLEX(WP), INTENT(IN) :: D(:)
		COMPLEX(WP), INTENT(IN) :: DU(:)
		COMPLEX(WP), INTENT(IN) :: DU2(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE CGTCON_MKL95
	SUBROUTINE ZGTCON_MKL95(DL,D,DU,DU2,IPIV,ANORM,RCOND,NORM,INFO)
		! MKL Fortran77 call:
		! ZGTCON(NORM,N,DL,D,DU,DU2,IPIV,ANORM,RCOND,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: DL(:)
		COMPLEX(WP), INTENT(IN) :: D(:)
		COMPLEX(WP), INTENT(IN) :: DU(:)
		COMPLEX(WP), INTENT(IN) :: DU2(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE ZGTCON_MKL95
END INTERFACE GTCON
INTERFACE POCON
	SUBROUTINE SPOCON_MKL95(A,ANORM,RCOND,UPLO,INFO)
		! MKL Fortran77 call:
		! SPOCON(UPLO,N,A,LDA,ANORM,RCOND,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
	END SUBROUTINE SPOCON_MKL95
	SUBROUTINE DPOCON_MKL95(A,ANORM,RCOND,UPLO,INFO)
		! MKL Fortran77 call:
		! DPOCON(UPLO,N,A,LDA,ANORM,RCOND,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
	END SUBROUTINE DPOCON_MKL95
	SUBROUTINE CPOCON_MKL95(A,ANORM,RCOND,UPLO,INFO)
		! MKL Fortran77 call:
		! CPOCON(UPLO,N,A,LDA,ANORM,RCOND,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
	END SUBROUTINE CPOCON_MKL95
	SUBROUTINE ZPOCON_MKL95(A,ANORM,RCOND,UPLO,INFO)
		! MKL Fortran77 call:
		! ZPOCON(UPLO,N,A,LDA,ANORM,RCOND,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
	END SUBROUTINE ZPOCON_MKL95
END INTERFACE POCON
INTERFACE PPCON
	SUBROUTINE SPPCON_MKL95(A,ANORM,RCOND,UPLO,INFO)
		! MKL Fortran77 call:
		! SPPCON(UPLO,N,A,ANORM,RCOND,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
	END SUBROUTINE SPPCON_MKL95
	SUBROUTINE DPPCON_MKL95(A,ANORM,RCOND,UPLO,INFO)
		! MKL Fortran77 call:
		! DPPCON(UPLO,N,A,ANORM,RCOND,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
	END SUBROUTINE DPPCON_MKL95
	SUBROUTINE CPPCON_MKL95(A,ANORM,RCOND,UPLO,INFO)
		! MKL Fortran77 call:
		! CPPCON(UPLO,N,A,ANORM,RCOND,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
	END SUBROUTINE CPPCON_MKL95
	SUBROUTINE ZPPCON_MKL95(A,ANORM,RCOND,UPLO,INFO)
		! MKL Fortran77 call:
		! ZPPCON(UPLO,N,A,ANORM,RCOND,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
	END SUBROUTINE ZPPCON_MKL95
END INTERFACE PPCON
INTERFACE PBCON
	SUBROUTINE SPBCON_MKL95(A,ANORM,RCOND,UPLO,INFO)
		! MKL Fortran77 call:
		! SPBCON(UPLO,N,KD,A,LDA,ANORM,RCOND,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
	END SUBROUTINE SPBCON_MKL95
	SUBROUTINE DPBCON_MKL95(A,ANORM,RCOND,UPLO,INFO)
		! MKL Fortran77 call:
		! DPBCON(UPLO,N,KD,A,LDA,ANORM,RCOND,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
	END SUBROUTINE DPBCON_MKL95
	SUBROUTINE CPBCON_MKL95(A,ANORM,RCOND,UPLO,INFO)
		! MKL Fortran77 call:
		! CPBCON(UPLO,N,KD,A,LDA,ANORM,RCOND,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
	END SUBROUTINE CPBCON_MKL95
	SUBROUTINE ZPBCON_MKL95(A,ANORM,RCOND,UPLO,INFO)
		! MKL Fortran77 call:
		! ZPBCON(UPLO,N,KD,A,LDA,ANORM,RCOND,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
	END SUBROUTINE ZPBCON_MKL95
END INTERFACE PBCON
INTERFACE PTCON
	SUBROUTINE SPTCON_MKL95(D,E,ANORM,RCOND,INFO)
		! MKL Fortran77 call:
		! SPTCON(N,D,E,ANORM,RCOND,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: E(:)
	END SUBROUTINE SPTCON_MKL95
	SUBROUTINE DPTCON_MKL95(D,E,ANORM,RCOND,INFO)
		! MKL Fortran77 call:
		! DPTCON(N,D,E,ANORM,RCOND,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: E(:)
	END SUBROUTINE DPTCON_MKL95
	SUBROUTINE CPTCON_MKL95(D,E,ANORM,RCOND,INFO)
		! MKL Fortran77 call:
		! CPTCON(N,D,E,ANORM,RCOND,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		COMPLEX(WP), INTENT(IN) :: E(:)
	END SUBROUTINE CPTCON_MKL95
	SUBROUTINE ZPTCON_MKL95(D,E,ANORM,RCOND,INFO)
		! MKL Fortran77 call:
		! ZPTCON(N,D,E,ANORM,RCOND,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		COMPLEX(WP), INTENT(IN) :: E(:)
	END SUBROUTINE ZPTCON_MKL95
END INTERFACE PTCON
INTERFACE SYCON
	SUBROUTINE SSYCON_MKL95(A,IPIV,ANORM,RCOND,UPLO,INFO)
		! MKL Fortran77 call:
		! SSYCON(UPLO,N,A,LDA,IPIV,ANORM,RCOND,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE SSYCON_MKL95
	SUBROUTINE DSYCON_MKL95(A,IPIV,ANORM,RCOND,UPLO,INFO)
		! MKL Fortran77 call:
		! DSYCON(UPLO,N,A,LDA,IPIV,ANORM,RCOND,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE DSYCON_MKL95
	SUBROUTINE CSYCON_MKL95(A,IPIV,ANORM,RCOND,UPLO,INFO)
		! MKL Fortran77 call:
		! CSYCON(UPLO,N,A,LDA,IPIV,ANORM,RCOND,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE CSYCON_MKL95
	SUBROUTINE ZSYCON_MKL95(A,IPIV,ANORM,RCOND,UPLO,INFO)
		! MKL Fortran77 call:
		! ZSYCON(UPLO,N,A,LDA,IPIV,ANORM,RCOND,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE ZSYCON_MKL95
END INTERFACE SYCON
INTERFACE HECON
	SUBROUTINE CHECON_MKL95(A,IPIV,ANORM,RCOND,UPLO,INFO)
		! MKL Fortran77 call:
		! CHECON(UPLO,N,A,LDA,IPIV,ANORM,RCOND,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE CHECON_MKL95
	SUBROUTINE ZHECON_MKL95(A,IPIV,ANORM,RCOND,UPLO,INFO)
		! MKL Fortran77 call:
		! ZHECON(UPLO,N,A,LDA,IPIV,ANORM,RCOND,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE ZHECON_MKL95
END INTERFACE HECON
INTERFACE SPCON
	SUBROUTINE SSPCON_MKL95(A,IPIV,ANORM,RCOND,UPLO,INFO)
		! MKL Fortran77 call:
		! SSPCON(UPLO,N,A,IPIV,ANORM,RCOND,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE SSPCON_MKL95
	SUBROUTINE DSPCON_MKL95(A,IPIV,ANORM,RCOND,UPLO,INFO)
		! MKL Fortran77 call:
		! DSPCON(UPLO,N,A,IPIV,ANORM,RCOND,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE DSPCON_MKL95
	SUBROUTINE CSPCON_MKL95(A,IPIV,ANORM,RCOND,UPLO,INFO)
		! MKL Fortran77 call:
		! CSPCON(UPLO,N,A,IPIV,ANORM,RCOND,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE CSPCON_MKL95
	SUBROUTINE ZSPCON_MKL95(A,IPIV,ANORM,RCOND,UPLO,INFO)
		! MKL Fortran77 call:
		! ZSPCON(UPLO,N,A,IPIV,ANORM,RCOND,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE ZSPCON_MKL95
END INTERFACE SPCON
INTERFACE HPCON
	SUBROUTINE CHPCON_MKL95(A,IPIV,ANORM,RCOND,UPLO,INFO)
		! MKL Fortran77 call:
		! CHPCON(UPLO,N,A,IPIV,ANORM,RCOND,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE CHPCON_MKL95
	SUBROUTINE ZHPCON_MKL95(A,IPIV,ANORM,RCOND,UPLO,INFO)
		! MKL Fortran77 call:
		! ZHPCON(UPLO,N,A,IPIV,ANORM,RCOND,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(IN) :: ANORM
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE ZHPCON_MKL95
END INTERFACE HPCON
INTERFACE TRCON
	SUBROUTINE STRCON_MKL95(A,RCOND,UPLO,DIAG,NORM,INFO)
		! MKL Fortran77 call:
		! STRCON(NORM,UPLO,DIAG,N,A,LDA,RCOND,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
	END SUBROUTINE STRCON_MKL95
	SUBROUTINE DTRCON_MKL95(A,RCOND,UPLO,DIAG,NORM,INFO)
		! MKL Fortran77 call:
		! DTRCON(NORM,UPLO,DIAG,N,A,LDA,RCOND,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
	END SUBROUTINE DTRCON_MKL95
	SUBROUTINE CTRCON_MKL95(A,RCOND,UPLO,DIAG,NORM,INFO)
		! MKL Fortran77 call:
		! CTRCON(NORM,UPLO,DIAG,N,A,LDA,RCOND,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
	END SUBROUTINE CTRCON_MKL95
	SUBROUTINE ZTRCON_MKL95(A,RCOND,UPLO,DIAG,NORM,INFO)
		! MKL Fortran77 call:
		! ZTRCON(NORM,UPLO,DIAG,N,A,LDA,RCOND,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
	END SUBROUTINE ZTRCON_MKL95
END INTERFACE TRCON
INTERFACE TPCON
	SUBROUTINE STPCON_MKL95(A,RCOND,UPLO,DIAG,NORM,INFO)
		! MKL Fortran77 call:
		! STPCON(NORM,UPLO,DIAG,N,A,RCOND,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
	END SUBROUTINE STPCON_MKL95
	SUBROUTINE DTPCON_MKL95(A,RCOND,UPLO,DIAG,NORM,INFO)
		! MKL Fortran77 call:
		! DTPCON(NORM,UPLO,DIAG,N,A,RCOND,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
	END SUBROUTINE DTPCON_MKL95
	SUBROUTINE CTPCON_MKL95(A,RCOND,UPLO,DIAG,NORM,INFO)
		! MKL Fortran77 call:
		! CTPCON(NORM,UPLO,DIAG,N,A,RCOND,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
	END SUBROUTINE CTPCON_MKL95
	SUBROUTINE ZTPCON_MKL95(A,RCOND,UPLO,DIAG,NORM,INFO)
		! MKL Fortran77 call:
		! ZTPCON(NORM,UPLO,DIAG,N,A,RCOND,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
	END SUBROUTINE ZTPCON_MKL95
END INTERFACE TPCON
INTERFACE TBCON
	SUBROUTINE STBCON_MKL95(A,RCOND,UPLO,DIAG,NORM,INFO)
		! MKL Fortran77 call:
		! STBCON(NORM,UPLO,DIAG,N,KD,A,LDA,RCOND,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
	END SUBROUTINE STBCON_MKL95
	SUBROUTINE DTBCON_MKL95(A,RCOND,UPLO,DIAG,NORM,INFO)
		! MKL Fortran77 call:
		! DTBCON(NORM,UPLO,DIAG,N,KD,A,LDA,RCOND,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
	END SUBROUTINE DTBCON_MKL95
	SUBROUTINE CTBCON_MKL95(A,RCOND,UPLO,DIAG,NORM,INFO)
		! MKL Fortran77 call:
		! CTBCON(NORM,UPLO,DIAG,N,KD,A,LDA,RCOND,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
	END SUBROUTINE CTBCON_MKL95
	SUBROUTINE ZTBCON_MKL95(A,RCOND,UPLO,DIAG,NORM,INFO)
		! MKL Fortran77 call:
		! ZTBCON(NORM,UPLO,DIAG,N,KD,A,LDA,RCOND,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(OUT) :: RCOND
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: NORM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
	END SUBROUTINE ZTBCON_MKL95
END INTERFACE TBCON
INTERFACE GERFS
	SUBROUTINE SGERFS_MKL95(A,AF,IPIV,B,X,TRANS,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! SGERFS(TRANS,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: AF(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE SGERFS_MKL95
	SUBROUTINE SGERFS1_MKL95(A,AF,IPIV,B,X,TRANS,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! SGERFS(TRANS,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: AF(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE SGERFS1_MKL95
	SUBROUTINE DGERFS_MKL95(A,AF,IPIV,B,X,TRANS,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! DGERFS(TRANS,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: AF(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE DGERFS_MKL95
	SUBROUTINE DGERFS1_MKL95(A,AF,IPIV,B,X,TRANS,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! DGERFS(TRANS,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: AF(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE DGERFS1_MKL95
	SUBROUTINE CGERFS_MKL95(A,AF,IPIV,B,X,TRANS,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! CGERFS(TRANS,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: AF(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE CGERFS_MKL95
	SUBROUTINE CGERFS1_MKL95(A,AF,IPIV,B,X,TRANS,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! CGERFS(TRANS,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: AF(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE CGERFS1_MKL95
	SUBROUTINE ZGERFS_MKL95(A,AF,IPIV,B,X,TRANS,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! ZGERFS(TRANS,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: AF(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE ZGERFS_MKL95
	SUBROUTINE ZGERFS1_MKL95(A,AF,IPIV,B,X,TRANS,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! ZGERFS(TRANS,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: AF(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE ZGERFS1_MKL95
END INTERFACE GERFS
INTERFACE GBRFS
	SUBROUTINE SGBRFS_MKL95(A,AF,IPIV,B,X,KL,TRANS,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! SGBRFS(TRANS,N,KL,KU,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: AF(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE SGBRFS_MKL95
	SUBROUTINE SGBRFS1_MKL95(A,AF,IPIV,B,X,KL,TRANS,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! SGBRFS(TRANS,N,KL,KU,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: AF(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE SGBRFS1_MKL95
	SUBROUTINE DGBRFS_MKL95(A,AF,IPIV,B,X,KL,TRANS,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! DGBRFS(TRANS,N,KL,KU,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: AF(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE DGBRFS_MKL95
	SUBROUTINE DGBRFS1_MKL95(A,AF,IPIV,B,X,KL,TRANS,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! DGBRFS(TRANS,N,KL,KU,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: AF(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE DGBRFS1_MKL95
	SUBROUTINE CGBRFS_MKL95(A,AF,IPIV,B,X,KL,TRANS,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! CGBRFS(TRANS,N,KL,KU,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: AF(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE CGBRFS_MKL95
	SUBROUTINE CGBRFS1_MKL95(A,AF,IPIV,B,X,KL,TRANS,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! CGBRFS(TRANS,N,KL,KU,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: AF(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE CGBRFS1_MKL95
	SUBROUTINE ZGBRFS_MKL95(A,AF,IPIV,B,X,KL,TRANS,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! ZGBRFS(TRANS,N,KL,KU,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: AF(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE ZGBRFS_MKL95
	SUBROUTINE ZGBRFS1_MKL95(A,AF,IPIV,B,X,KL,TRANS,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! ZGBRFS(TRANS,N,KL,KU,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: AF(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE ZGBRFS1_MKL95
END INTERFACE GBRFS
INTERFACE GTRFS
	SUBROUTINE SGTRFS_MKL95(DL,D,DU,DLF,DF,DUF,DU2,IPIV,B,X,TRANS,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! SGTRFS(TRANS,N,NRHS,DL,D,DU,DLF,DF,DUF,DU2,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: DL(:)
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: DU(:)
		REAL(WP), INTENT(IN) :: DLF(:)
		REAL(WP), INTENT(IN) :: DF(:)
		REAL(WP), INTENT(IN) :: DUF(:)
		REAL(WP), INTENT(IN) :: DU2(:)
		INTEGER, INTENT(IN) :: IPIV(:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE SGTRFS_MKL95
	SUBROUTINE SGTRFS1_MKL95(DL,D,DU,DLF,DF,DUF,DU2,IPIV,B,X,TRANS,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! SGTRFS(TRANS,N,NRHS,DL,D,DU,DLF,DF,DUF,DU2,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: DL(:)
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: DU(:)
		REAL(WP), INTENT(IN) :: DLF(:)
		REAL(WP), INTENT(IN) :: DF(:)
		REAL(WP), INTENT(IN) :: DUF(:)
		REAL(WP), INTENT(IN) :: DU2(:)
		INTEGER, INTENT(IN) :: IPIV(:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE SGTRFS1_MKL95
	SUBROUTINE DGTRFS_MKL95(DL,D,DU,DLF,DF,DUF,DU2,IPIV,B,X,TRANS,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! DGTRFS(TRANS,N,NRHS,DL,D,DU,DLF,DF,DUF,DU2,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: DL(:)
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: DU(:)
		REAL(WP), INTENT(IN) :: DLF(:)
		REAL(WP), INTENT(IN) :: DF(:)
		REAL(WP), INTENT(IN) :: DUF(:)
		REAL(WP), INTENT(IN) :: DU2(:)
		INTEGER, INTENT(IN) :: IPIV(:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE DGTRFS_MKL95
	SUBROUTINE DGTRFS1_MKL95(DL,D,DU,DLF,DF,DUF,DU2,IPIV,B,X,TRANS,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! DGTRFS(TRANS,N,NRHS,DL,D,DU,DLF,DF,DUF,DU2,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: DL(:)
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: DU(:)
		REAL(WP), INTENT(IN) :: DLF(:)
		REAL(WP), INTENT(IN) :: DF(:)
		REAL(WP), INTENT(IN) :: DUF(:)
		REAL(WP), INTENT(IN) :: DU2(:)
		INTEGER, INTENT(IN) :: IPIV(:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE DGTRFS1_MKL95
	SUBROUTINE CGTRFS_MKL95(DL,D,DU,DLF,DF,DUF,DU2,IPIV,B,X,TRANS,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! CGTRFS(TRANS,N,NRHS,DL,D,DU,DLF,DF,DUF,DU2,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: DL(:)
		COMPLEX(WP), INTENT(IN) :: D(:)
		COMPLEX(WP), INTENT(IN) :: DU(:)
		COMPLEX(WP), INTENT(IN) :: DLF(:)
		COMPLEX(WP), INTENT(IN) :: DF(:)
		COMPLEX(WP), INTENT(IN) :: DUF(:)
		COMPLEX(WP), INTENT(IN) :: DU2(:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE CGTRFS_MKL95
	SUBROUTINE CGTRFS1_MKL95(DL,D,DU,DLF,DF,DUF,DU2,IPIV,B,X,TRANS,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! CGTRFS(TRANS,N,NRHS,DL,D,DU,DLF,DF,DUF,DU2,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: DL(:)
		COMPLEX(WP), INTENT(IN) :: D(:)
		COMPLEX(WP), INTENT(IN) :: DU(:)
		COMPLEX(WP), INTENT(IN) :: DLF(:)
		COMPLEX(WP), INTENT(IN) :: DF(:)
		COMPLEX(WP), INTENT(IN) :: DUF(:)
		COMPLEX(WP), INTENT(IN) :: DU2(:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE CGTRFS1_MKL95
	SUBROUTINE ZGTRFS_MKL95(DL,D,DU,DLF,DF,DUF,DU2,IPIV,B,X,TRANS,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! ZGTRFS(TRANS,N,NRHS,DL,D,DU,DLF,DF,DUF,DU2,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: DL(:)
		COMPLEX(WP), INTENT(IN) :: D(:)
		COMPLEX(WP), INTENT(IN) :: DU(:)
		COMPLEX(WP), INTENT(IN) :: DLF(:)
		COMPLEX(WP), INTENT(IN) :: DF(:)
		COMPLEX(WP), INTENT(IN) :: DUF(:)
		COMPLEX(WP), INTENT(IN) :: DU2(:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE ZGTRFS_MKL95
	SUBROUTINE ZGTRFS1_MKL95(DL,D,DU,DLF,DF,DUF,DU2,IPIV,B,X,TRANS,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! ZGTRFS(TRANS,N,NRHS,DL,D,DU,DLF,DF,DUF,DU2,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: DL(:)
		COMPLEX(WP), INTENT(IN) :: D(:)
		COMPLEX(WP), INTENT(IN) :: DU(:)
		COMPLEX(WP), INTENT(IN) :: DLF(:)
		COMPLEX(WP), INTENT(IN) :: DF(:)
		COMPLEX(WP), INTENT(IN) :: DUF(:)
		COMPLEX(WP), INTENT(IN) :: DU2(:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE ZGTRFS1_MKL95
END INTERFACE GTRFS
INTERFACE PORFS
	SUBROUTINE SPORFS_MKL95(A,AF,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! SPORFS(UPLO,N,NRHS,A,LDA,AF,LDAF,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: AF(:,:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE SPORFS_MKL95
	SUBROUTINE SPORFS1_MKL95(A,AF,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! SPORFS(UPLO,N,NRHS,A,LDA,AF,LDAF,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: AF(:,:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE SPORFS1_MKL95
	SUBROUTINE DPORFS_MKL95(A,AF,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! DPORFS(UPLO,N,NRHS,A,LDA,AF,LDAF,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: AF(:,:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE DPORFS_MKL95
	SUBROUTINE DPORFS1_MKL95(A,AF,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! DPORFS(UPLO,N,NRHS,A,LDA,AF,LDAF,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: AF(:,:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE DPORFS1_MKL95
	SUBROUTINE CPORFS_MKL95(A,AF,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! CPORFS(UPLO,N,NRHS,A,LDA,AF,LDAF,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: AF(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE CPORFS_MKL95
	SUBROUTINE CPORFS1_MKL95(A,AF,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! CPORFS(UPLO,N,NRHS,A,LDA,AF,LDAF,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: AF(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE CPORFS1_MKL95
	SUBROUTINE ZPORFS_MKL95(A,AF,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! ZPORFS(UPLO,N,NRHS,A,LDA,AF,LDAF,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: AF(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE ZPORFS_MKL95
	SUBROUTINE ZPORFS1_MKL95(A,AF,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! ZPORFS(UPLO,N,NRHS,A,LDA,AF,LDAF,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: AF(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE ZPORFS1_MKL95
END INTERFACE PORFS
INTERFACE PPRFS
	SUBROUTINE SPPRFS_MKL95(A,AF,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! SPPRFS(UPLO,N,NRHS,A,AF,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(IN) :: AF(:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE SPPRFS_MKL95
	SUBROUTINE SPPRFS1_MKL95(A,AF,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! SPPRFS(UPLO,N,NRHS,A,AF,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(IN) :: AF(:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE SPPRFS1_MKL95
	SUBROUTINE DPPRFS_MKL95(A,AF,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! DPPRFS(UPLO,N,NRHS,A,AF,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(IN) :: AF(:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE DPPRFS_MKL95
	SUBROUTINE DPPRFS1_MKL95(A,AF,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! DPPRFS(UPLO,N,NRHS,A,AF,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(IN) :: AF(:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE DPPRFS1_MKL95
	SUBROUTINE CPPRFS_MKL95(A,AF,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! CPPRFS(UPLO,N,NRHS,A,AF,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(IN) :: AF(:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE CPPRFS_MKL95
	SUBROUTINE CPPRFS1_MKL95(A,AF,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! CPPRFS(UPLO,N,NRHS,A,AF,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(IN) :: AF(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE CPPRFS1_MKL95
	SUBROUTINE ZPPRFS_MKL95(A,AF,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! ZPPRFS(UPLO,N,NRHS,A,AF,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(IN) :: AF(:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE ZPPRFS_MKL95
	SUBROUTINE ZPPRFS1_MKL95(A,AF,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! ZPPRFS(UPLO,N,NRHS,A,AF,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(IN) :: AF(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE ZPPRFS1_MKL95
END INTERFACE PPRFS
INTERFACE PBRFS
	SUBROUTINE SPBRFS_MKL95(A,AF,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! SPBRFS(UPLO,N,KD,NRHS,A,LDA,AF,LDAF,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: AF(:,:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE SPBRFS_MKL95
	SUBROUTINE SPBRFS1_MKL95(A,AF,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! SPBRFS(UPLO,N,KD,NRHS,A,LDA,AF,LDAF,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: AF(:,:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE SPBRFS1_MKL95
	SUBROUTINE DPBRFS_MKL95(A,AF,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! DPBRFS(UPLO,N,KD,NRHS,A,LDA,AF,LDAF,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: AF(:,:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE DPBRFS_MKL95
	SUBROUTINE DPBRFS1_MKL95(A,AF,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! DPBRFS(UPLO,N,KD,NRHS,A,LDA,AF,LDAF,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: AF(:,:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE DPBRFS1_MKL95
	SUBROUTINE CPBRFS_MKL95(A,AF,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! CPBRFS(UPLO,N,KD,NRHS,A,LDA,AF,LDAF,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: AF(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE CPBRFS_MKL95
	SUBROUTINE CPBRFS1_MKL95(A,AF,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! CPBRFS(UPLO,N,KD,NRHS,A,LDA,AF,LDAF,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: AF(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE CPBRFS1_MKL95
	SUBROUTINE ZPBRFS_MKL95(A,AF,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! ZPBRFS(UPLO,N,KD,NRHS,A,LDA,AF,LDAF,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: AF(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE ZPBRFS_MKL95
	SUBROUTINE ZPBRFS1_MKL95(A,AF,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! ZPBRFS(UPLO,N,KD,NRHS,A,LDA,AF,LDAF,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: AF(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE ZPBRFS1_MKL95
END INTERFACE PBRFS
INTERFACE PTRFS
	SUBROUTINE SPTRFS_MKL95(D,DF,E,EF,B,X,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! SPTRFS(N,NRHS,D,E,DF,EF,B,LDB,X,LDX,FERR,BERR,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: DF(:)
		REAL(WP), INTENT(IN) :: E(:)
		REAL(WP), INTENT(IN) :: EF(:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE SPTRFS_MKL95
	SUBROUTINE SPTRFS1_MKL95(D,DF,E,EF,B,X,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! SPTRFS(N,NRHS,D,E,DF,EF,B,LDB,X,LDX,FERR,BERR,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: DF(:)
		REAL(WP), INTENT(IN) :: E(:)
		REAL(WP), INTENT(IN) :: EF(:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE SPTRFS1_MKL95
	SUBROUTINE DPTRFS_MKL95(D,DF,E,EF,B,X,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! DPTRFS(N,NRHS,D,E,DF,EF,B,LDB,X,LDX,FERR,BERR,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: DF(:)
		REAL(WP), INTENT(IN) :: E(:)
		REAL(WP), INTENT(IN) :: EF(:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE DPTRFS_MKL95
	SUBROUTINE DPTRFS1_MKL95(D,DF,E,EF,B,X,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! DPTRFS(N,NRHS,D,E,DF,EF,B,LDB,X,LDX,FERR,BERR,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: DF(:)
		REAL(WP), INTENT(IN) :: E(:)
		REAL(WP), INTENT(IN) :: EF(:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE DPTRFS1_MKL95
	SUBROUTINE CPTRFS_MKL95(D,DF,E,EF,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! CPTRFS(UPLO,N,NRHS,D,E,DF,EF,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: DF(:)
		COMPLEX(WP), INTENT(IN) :: E(:)
		COMPLEX(WP), INTENT(IN) :: EF(:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE CPTRFS_MKL95
	SUBROUTINE CPTRFS1_MKL95(D,DF,E,EF,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! CPTRFS(UPLO,N,NRHS,D,E,DF,EF,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: DF(:)
		COMPLEX(WP), INTENT(IN) :: E(:)
		COMPLEX(WP), INTENT(IN) :: EF(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE CPTRFS1_MKL95
	SUBROUTINE ZPTRFS_MKL95(D,DF,E,EF,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! ZPTRFS(UPLO,N,NRHS,D,E,DF,EF,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: DF(:)
		COMPLEX(WP), INTENT(IN) :: E(:)
		COMPLEX(WP), INTENT(IN) :: EF(:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE ZPTRFS_MKL95
	SUBROUTINE ZPTRFS1_MKL95(D,DF,E,EF,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! ZPTRFS(UPLO,N,NRHS,D,E,DF,EF,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: DF(:)
		COMPLEX(WP), INTENT(IN) :: E(:)
		COMPLEX(WP), INTENT(IN) :: EF(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE ZPTRFS1_MKL95
END INTERFACE PTRFS
INTERFACE SYRFS
	SUBROUTINE SSYRFS_MKL95(A,AF,IPIV,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! SSYRFS(UPLO,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: AF(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE SSYRFS_MKL95
	SUBROUTINE SSYRFS1_MKL95(A,AF,IPIV,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! SSYRFS(UPLO,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: AF(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE SSYRFS1_MKL95
	SUBROUTINE DSYRFS_MKL95(A,AF,IPIV,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! DSYRFS(UPLO,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: AF(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE DSYRFS_MKL95
	SUBROUTINE DSYRFS1_MKL95(A,AF,IPIV,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! DSYRFS(UPLO,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: AF(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE DSYRFS1_MKL95
	SUBROUTINE CSYRFS_MKL95(A,AF,IPIV,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! CSYRFS(UPLO,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: AF(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE CSYRFS_MKL95
	SUBROUTINE CSYRFS1_MKL95(A,AF,IPIV,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! CSYRFS(UPLO,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: AF(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE CSYRFS1_MKL95
	SUBROUTINE ZSYRFS_MKL95(A,AF,IPIV,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! ZSYRFS(UPLO,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: AF(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE ZSYRFS_MKL95
	SUBROUTINE ZSYRFS1_MKL95(A,AF,IPIV,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! ZSYRFS(UPLO,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: AF(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE ZSYRFS1_MKL95
END INTERFACE SYRFS
INTERFACE HERFS
	SUBROUTINE CHERFS_MKL95(A,AF,IPIV,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! CHERFS(UPLO,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: AF(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE CHERFS_MKL95
	SUBROUTINE CHERFS1_MKL95(A,AF,IPIV,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! CHERFS(UPLO,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: AF(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE CHERFS1_MKL95
	SUBROUTINE ZHERFS_MKL95(A,AF,IPIV,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! ZHERFS(UPLO,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: AF(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE ZHERFS_MKL95
	SUBROUTINE ZHERFS1_MKL95(A,AF,IPIV,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! ZHERFS(UPLO,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: AF(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE ZHERFS1_MKL95
END INTERFACE HERFS
INTERFACE SPRFS
	SUBROUTINE SSPRFS_MKL95(A,AF,IPIV,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! SSPRFS(UPLO,N,NRHS,A,AF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(IN) :: AF(:)
		INTEGER, INTENT(IN) :: IPIV(:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE SSPRFS_MKL95
	SUBROUTINE SSPRFS1_MKL95(A,AF,IPIV,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! SSPRFS(UPLO,N,NRHS,A,AF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(IN) :: AF(:)
		INTEGER, INTENT(IN) :: IPIV(:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE SSPRFS1_MKL95
	SUBROUTINE DSPRFS_MKL95(A,AF,IPIV,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! DSPRFS(UPLO,N,NRHS,A,AF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(IN) :: AF(:)
		INTEGER, INTENT(IN) :: IPIV(:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE DSPRFS_MKL95
	SUBROUTINE DSPRFS1_MKL95(A,AF,IPIV,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! DSPRFS(UPLO,N,NRHS,A,AF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(IN) :: AF(:)
		INTEGER, INTENT(IN) :: IPIV(:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE DSPRFS1_MKL95
	SUBROUTINE CSPRFS_MKL95(A,AF,IPIV,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! CSPRFS(UPLO,N,NRHS,A,AF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(IN) :: AF(:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE CSPRFS_MKL95
	SUBROUTINE CSPRFS1_MKL95(A,AF,IPIV,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! CSPRFS(UPLO,N,NRHS,A,AF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(IN) :: AF(:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE CSPRFS1_MKL95
	SUBROUTINE ZSPRFS_MKL95(A,AF,IPIV,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! ZSPRFS(UPLO,N,NRHS,A,AF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(IN) :: AF(:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE ZSPRFS_MKL95
	SUBROUTINE ZSPRFS1_MKL95(A,AF,IPIV,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! ZSPRFS(UPLO,N,NRHS,A,AF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(IN) :: AF(:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE ZSPRFS1_MKL95
END INTERFACE SPRFS
INTERFACE HPRFS
	SUBROUTINE CHPRFS_MKL95(A,AF,IPIV,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! CHPRFS(UPLO,N,NRHS,A,AF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(IN) :: AF(:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE CHPRFS_MKL95
	SUBROUTINE CHPRFS1_MKL95(A,AF,IPIV,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! CHPRFS(UPLO,N,NRHS,A,AF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(IN) :: AF(:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE CHPRFS1_MKL95
	SUBROUTINE ZHPRFS_MKL95(A,AF,IPIV,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! ZHPRFS(UPLO,N,NRHS,A,AF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(IN) :: AF(:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE ZHPRFS_MKL95
	SUBROUTINE ZHPRFS1_MKL95(A,AF,IPIV,B,X,UPLO,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! ZHPRFS(UPLO,N,NRHS,A,AF,IPIV,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(IN) :: AF(:)
		INTEGER, INTENT(IN) :: IPIV(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(INOUT) :: X(:)
	END SUBROUTINE ZHPRFS1_MKL95
END INTERFACE HPRFS
INTERFACE TRRFS
	SUBROUTINE STRRFS_MKL95(A,B,X,UPLO,TRANS,DIAG,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! STRRFS(UPLO,TRANS,DIAG,N,NRHS,A,LDA,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(IN) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE STRRFS_MKL95
	SUBROUTINE STRRFS1_MKL95(A,B,X,UPLO,TRANS,DIAG,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! STRRFS(UPLO,TRANS,DIAG,N,NRHS,A,LDA,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(IN) :: X(:)
	END SUBROUTINE STRRFS1_MKL95
	SUBROUTINE DTRRFS_MKL95(A,B,X,UPLO,TRANS,DIAG,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! DTRRFS(UPLO,TRANS,DIAG,N,NRHS,A,LDA,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(IN) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE DTRRFS_MKL95
	SUBROUTINE DTRRFS1_MKL95(A,B,X,UPLO,TRANS,DIAG,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! DTRRFS(UPLO,TRANS,DIAG,N,NRHS,A,LDA,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(IN) :: X(:)
	END SUBROUTINE DTRRFS1_MKL95
	SUBROUTINE CTRRFS_MKL95(A,B,X,UPLO,TRANS,DIAG,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! CTRRFS(UPLO,TRANS,DIAG,N,NRHS,A,LDA,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(IN) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE CTRRFS_MKL95
	SUBROUTINE CTRRFS1_MKL95(A,B,X,UPLO,TRANS,DIAG,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! CTRRFS(UPLO,TRANS,DIAG,N,NRHS,A,LDA,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(IN) :: X(:)
	END SUBROUTINE CTRRFS1_MKL95
	SUBROUTINE ZTRRFS_MKL95(A,B,X,UPLO,TRANS,DIAG,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! ZTRRFS(UPLO,TRANS,DIAG,N,NRHS,A,LDA,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(IN) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE ZTRRFS_MKL95
	SUBROUTINE ZTRRFS1_MKL95(A,B,X,UPLO,TRANS,DIAG,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! ZTRRFS(UPLO,TRANS,DIAG,N,NRHS,A,LDA,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(IN) :: X(:)
	END SUBROUTINE ZTRRFS1_MKL95
END INTERFACE TRRFS
INTERFACE TPRFS
	SUBROUTINE STPRFS_MKL95(A,B,X,UPLO,TRANS,DIAG,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! STPRFS(UPLO,TRANS,DIAG,N,NRHS,A,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(IN) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE STPRFS_MKL95
	SUBROUTINE STPRFS1_MKL95(A,B,X,UPLO,TRANS,DIAG,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! STPRFS(UPLO,TRANS,DIAG,N,NRHS,A,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(IN) :: X(:)
	END SUBROUTINE STPRFS1_MKL95
	SUBROUTINE DTPRFS_MKL95(A,B,X,UPLO,TRANS,DIAG,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! DTPRFS(UPLO,TRANS,DIAG,N,NRHS,A,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(IN) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE DTPRFS_MKL95
	SUBROUTINE DTPRFS1_MKL95(A,B,X,UPLO,TRANS,DIAG,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! DTPRFS(UPLO,TRANS,DIAG,N,NRHS,A,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(IN) :: X(:)
	END SUBROUTINE DTPRFS1_MKL95
	SUBROUTINE CTPRFS_MKL95(A,B,X,UPLO,TRANS,DIAG,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! CTPRFS(UPLO,TRANS,DIAG,N,NRHS,A,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(IN) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE CTPRFS_MKL95
	SUBROUTINE CTPRFS1_MKL95(A,B,X,UPLO,TRANS,DIAG,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! CTPRFS(UPLO,TRANS,DIAG,N,NRHS,A,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(IN) :: X(:)
	END SUBROUTINE CTPRFS1_MKL95
	SUBROUTINE ZTPRFS_MKL95(A,B,X,UPLO,TRANS,DIAG,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! ZTPRFS(UPLO,TRANS,DIAG,N,NRHS,A,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(IN) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE ZTPRFS_MKL95
	SUBROUTINE ZTPRFS1_MKL95(A,B,X,UPLO,TRANS,DIAG,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! ZTPRFS(UPLO,TRANS,DIAG,N,NRHS,A,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(IN) :: X(:)
	END SUBROUTINE ZTPRFS1_MKL95
END INTERFACE TPRFS
INTERFACE TBRFS
	SUBROUTINE STBRFS_MKL95(A,B,X,UPLO,TRANS,DIAG,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! STBRFS(UPLO,TRANS,DIAG,N,KD,NRHS,A,LDA,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(IN) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE STBRFS_MKL95
	SUBROUTINE STBRFS1_MKL95(A,B,X,UPLO,TRANS,DIAG,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! STBRFS(UPLO,TRANS,DIAG,N,KD,NRHS,A,LDA,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(IN) :: X(:)
	END SUBROUTINE STBRFS1_MKL95
	SUBROUTINE DTBRFS_MKL95(A,B,X,UPLO,TRANS,DIAG,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! DTBRFS(UPLO,TRANS,DIAG,N,KD,NRHS,A,LDA,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(IN) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE DTBRFS_MKL95
	SUBROUTINE DTBRFS1_MKL95(A,B,X,UPLO,TRANS,DIAG,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! DTBRFS(UPLO,TRANS,DIAG,N,KD,NRHS,A,LDA,B,LDB,X,LDX,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(IN) :: X(:)
	END SUBROUTINE DTBRFS1_MKL95
	SUBROUTINE CTBRFS_MKL95(A,B,X,UPLO,TRANS,DIAG,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! CTBRFS(UPLO,TRANS,DIAG,N,KD,NRHS,A,LDA,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(IN) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE CTBRFS_MKL95
	SUBROUTINE CTBRFS1_MKL95(A,B,X,UPLO,TRANS,DIAG,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! CTBRFS(UPLO,TRANS,DIAG,N,KD,NRHS,A,LDA,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(IN) :: X(:)
	END SUBROUTINE CTBRFS1_MKL95
	SUBROUTINE ZTBRFS_MKL95(A,B,X,UPLO,TRANS,DIAG,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! ZTBRFS(UPLO,TRANS,DIAG,N,KD,NRHS,A,LDA,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(IN) :: X(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE ZTBRFS_MKL95
	SUBROUTINE ZTBRFS1_MKL95(A,B,X,UPLO,TRANS,DIAG,FERR,BERR,INFO)
		! MKL Fortran77 call:
		! ZTBRFS(UPLO,TRANS,DIAG,N,KD,NRHS,A,LDA,B,LDB,X,LDX,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(IN) :: X(:)
	END SUBROUTINE ZTBRFS1_MKL95
END INTERFACE TBRFS
INTERFACE GETRI
	SUBROUTINE SGETRI_MKL95(A,IPIV,INFO)
		! MKL Fortran77 call:
		! SGETRI(N,A,LDA,IPIV,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE SGETRI_MKL95
	SUBROUTINE DGETRI_MKL95(A,IPIV,INFO)
		! MKL Fortran77 call:
		! DGETRI(N,A,LDA,IPIV,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE DGETRI_MKL95
	SUBROUTINE CGETRI_MKL95(A,IPIV,INFO)
		! MKL Fortran77 call:
		! CGETRI(N,A,LDA,IPIV,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE CGETRI_MKL95
	SUBROUTINE ZGETRI_MKL95(A,IPIV,INFO)
		! MKL Fortran77 call:
		! ZGETRI(N,A,LDA,IPIV,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE ZGETRI_MKL95
END INTERFACE GETRI
INTERFACE POTRI
	SUBROUTINE SPOTRI_MKL95(A,UPLO,INFO)
		! MKL Fortran77 call:
		! SPOTRI(UPLO,N,A,LDA,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
	END SUBROUTINE SPOTRI_MKL95
	SUBROUTINE DPOTRI_MKL95(A,UPLO,INFO)
		! MKL Fortran77 call:
		! DPOTRI(UPLO,N,A,LDA,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
	END SUBROUTINE DPOTRI_MKL95
	SUBROUTINE CPOTRI_MKL95(A,UPLO,INFO)
		! MKL Fortran77 call:
		! CPOTRI(UPLO,N,A,LDA,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
	END SUBROUTINE CPOTRI_MKL95
	SUBROUTINE ZPOTRI_MKL95(A,UPLO,INFO)
		! MKL Fortran77 call:
		! ZPOTRI(UPLO,N,A,LDA,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
	END SUBROUTINE ZPOTRI_MKL95
END INTERFACE POTRI
INTERFACE PPTRI
	SUBROUTINE SPPTRI_MKL95(A,UPLO,INFO)
		! MKL Fortran77 call:
		! SPPTRI(UPLO,N,A,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
	END SUBROUTINE SPPTRI_MKL95
	SUBROUTINE DPPTRI_MKL95(A,UPLO,INFO)
		! MKL Fortran77 call:
		! DPPTRI(UPLO,N,A,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
	END SUBROUTINE DPPTRI_MKL95
	SUBROUTINE CPPTRI_MKL95(A,UPLO,INFO)
		! MKL Fortran77 call:
		! CPPTRI(UPLO,N,A,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
	END SUBROUTINE CPPTRI_MKL95
	SUBROUTINE ZPPTRI_MKL95(A,UPLO,INFO)
		! MKL Fortran77 call:
		! ZPPTRI(UPLO,N,A,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
	END SUBROUTINE ZPPTRI_MKL95
END INTERFACE PPTRI
INTERFACE SYTRI
	SUBROUTINE SSYTRI_MKL95(A,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! SSYTRI(UPLO,N,A,LDA,IPIV,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE SSYTRI_MKL95
	SUBROUTINE DSYTRI_MKL95(A,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! DSYTRI(UPLO,N,A,LDA,IPIV,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE DSYTRI_MKL95
	SUBROUTINE CSYTRI_MKL95(A,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! CSYTRI(UPLO,N,A,LDA,IPIV,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE CSYTRI_MKL95
	SUBROUTINE ZSYTRI_MKL95(A,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! ZSYTRI(UPLO,N,A,LDA,IPIV,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE ZSYTRI_MKL95
END INTERFACE SYTRI
INTERFACE HETRI
	SUBROUTINE CHETRI_MKL95(A,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! CHETRI(UPLO,N,A,LDA,IPIV,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE CHETRI_MKL95
	SUBROUTINE ZHETRI_MKL95(A,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! ZHETRI(UPLO,N,A,LDA,IPIV,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE ZHETRI_MKL95
END INTERFACE HETRI
INTERFACE SPTRI
	SUBROUTINE SSPTRI_MKL95(A,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! SSPTRI(UPLO,N,A,IPIV,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE SSPTRI_MKL95
	SUBROUTINE DSPTRI_MKL95(A,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! DSPTRI(UPLO,N,A,IPIV,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE DSPTRI_MKL95
	SUBROUTINE CSPTRI_MKL95(A,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! CSPTRI(UPLO,N,A,IPIV,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE CSPTRI_MKL95
	SUBROUTINE ZSPTRI_MKL95(A,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! ZSPTRI(UPLO,N,A,IPIV,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE ZSPTRI_MKL95
END INTERFACE SPTRI
INTERFACE HPTRI
	SUBROUTINE CHPTRI_MKL95(A,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! CHPTRI(UPLO,N,A,IPIV,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE CHPTRI_MKL95
	SUBROUTINE ZHPTRI_MKL95(A,IPIV,UPLO,INFO)
		! MKL Fortran77 call:
		! ZHPTRI(UPLO,N,A,IPIV,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		INTEGER, INTENT(IN) :: IPIV(:)
	END SUBROUTINE ZHPTRI_MKL95
END INTERFACE HPTRI
INTERFACE TRTRI
	SUBROUTINE STRTRI_MKL95(A,UPLO,DIAG,INFO)
		! MKL Fortran77 call:
		! STRTRI(UPLO,DIAG,N,A,LDA,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
	END SUBROUTINE STRTRI_MKL95
	SUBROUTINE DTRTRI_MKL95(A,UPLO,DIAG,INFO)
		! MKL Fortran77 call:
		! DTRTRI(UPLO,DIAG,N,A,LDA,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
	END SUBROUTINE DTRTRI_MKL95
	SUBROUTINE CTRTRI_MKL95(A,UPLO,DIAG,INFO)
		! MKL Fortran77 call:
		! CTRTRI(UPLO,DIAG,N,A,LDA,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
	END SUBROUTINE CTRTRI_MKL95
	SUBROUTINE ZTRTRI_MKL95(A,UPLO,DIAG,INFO)
		! MKL Fortran77 call:
		! ZTRTRI(UPLO,DIAG,N,A,LDA,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
	END SUBROUTINE ZTRTRI_MKL95
END INTERFACE TRTRI
INTERFACE TPTRI
	SUBROUTINE STPTRI_MKL95(A,UPLO,DIAG,INFO)
		! MKL Fortran77 call:
		! STPTRI(UPLO,DIAG,N,A,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
	END SUBROUTINE STPTRI_MKL95
	SUBROUTINE DTPTRI_MKL95(A,UPLO,DIAG,INFO)
		! MKL Fortran77 call:
		! DTPTRI(UPLO,DIAG,N,A,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
	END SUBROUTINE DTPTRI_MKL95
	SUBROUTINE CTPTRI_MKL95(A,UPLO,DIAG,INFO)
		! MKL Fortran77 call:
		! CTPTRI(UPLO,DIAG,N,A,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
	END SUBROUTINE CTPTRI_MKL95
	SUBROUTINE ZTPTRI_MKL95(A,UPLO,DIAG,INFO)
		! MKL Fortran77 call:
		! ZTPTRI(UPLO,DIAG,N,A,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: DIAG
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
	END SUBROUTINE ZTPTRI_MKL95
END INTERFACE TPTRI
INTERFACE GEEQU
	SUBROUTINE SGEEQU_MKL95(A,R,C,ROWCND,COLCND,AMAX,INFO)
		! MKL Fortran77 call:
		! SGEEQU(M,N,A,LDA,R,C,ROWCND,COLCND,AMAX,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(OUT), OPTIONAL :: ROWCND
		REAL(WP), INTENT(OUT), OPTIONAL :: COLCND
		REAL(WP), INTENT(OUT), OPTIONAL :: AMAX
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(OUT) :: R(:)
		REAL(WP), INTENT(OUT) :: C(:)
	END SUBROUTINE SGEEQU_MKL95
	SUBROUTINE DGEEQU_MKL95(A,R,C,ROWCND,COLCND,AMAX,INFO)
		! MKL Fortran77 call:
		! DGEEQU(M,N,A,LDA,R,C,ROWCND,COLCND,AMAX,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(OUT), OPTIONAL :: ROWCND
		REAL(WP), INTENT(OUT), OPTIONAL :: COLCND
		REAL(WP), INTENT(OUT), OPTIONAL :: AMAX
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(OUT) :: R(:)
		REAL(WP), INTENT(OUT) :: C(:)
	END SUBROUTINE DGEEQU_MKL95
	SUBROUTINE CGEEQU_MKL95(A,R,C,ROWCND,COLCND,AMAX,INFO)
		! MKL Fortran77 call:
		! CGEEQU(M,N,A,LDA,R,C,ROWCND,COLCND,AMAX,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(OUT), OPTIONAL :: ROWCND
		REAL(WP), INTENT(OUT), OPTIONAL :: COLCND
		REAL(WP), INTENT(OUT), OPTIONAL :: AMAX
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(OUT) :: R(:)
		REAL(WP), INTENT(OUT) :: C(:)
	END SUBROUTINE CGEEQU_MKL95
	SUBROUTINE ZGEEQU_MKL95(A,R,C,ROWCND,COLCND,AMAX,INFO)
		! MKL Fortran77 call:
		! ZGEEQU(M,N,A,LDA,R,C,ROWCND,COLCND,AMAX,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(OUT), OPTIONAL :: ROWCND
		REAL(WP), INTENT(OUT), OPTIONAL :: COLCND
		REAL(WP), INTENT(OUT), OPTIONAL :: AMAX
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(OUT) :: R(:)
		REAL(WP), INTENT(OUT) :: C(:)
	END SUBROUTINE ZGEEQU_MKL95
END INTERFACE GEEQU
INTERFACE GBEQU
	SUBROUTINE SGBEQU_MKL95(A,R,C,KL,ROWCND,COLCND,AMAX,INFO)
		! MKL Fortran77 call:
		! SGBEQU(M,N,KL,KU,A,LDA,R,C,ROWCND,COLCND,AMAX,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		REAL(WP), INTENT(OUT), OPTIONAL :: ROWCND
		REAL(WP), INTENT(OUT), OPTIONAL :: COLCND
		REAL(WP), INTENT(OUT), OPTIONAL :: AMAX
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(OUT) :: R(:)
		REAL(WP), INTENT(OUT) :: C(:)
	END SUBROUTINE SGBEQU_MKL95
	SUBROUTINE DGBEQU_MKL95(A,R,C,KL,ROWCND,COLCND,AMAX,INFO)
		! MKL Fortran77 call:
		! DGBEQU(M,N,KL,KU,A,LDA,R,C,ROWCND,COLCND,AMAX,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		REAL(WP), INTENT(OUT), OPTIONAL :: ROWCND
		REAL(WP), INTENT(OUT), OPTIONAL :: COLCND
		REAL(WP), INTENT(OUT), OPTIONAL :: AMAX
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(OUT) :: R(:)
		REAL(WP), INTENT(OUT) :: C(:)
	END SUBROUTINE DGBEQU_MKL95
	SUBROUTINE CGBEQU_MKL95(A,R,C,KL,ROWCND,COLCND,AMAX,INFO)
		! MKL Fortran77 call:
		! CGBEQU(M,N,KL,KU,A,LDA,R,C,ROWCND,COLCND,AMAX,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		REAL(WP), INTENT(OUT), OPTIONAL :: ROWCND
		REAL(WP), INTENT(OUT), OPTIONAL :: COLCND
		REAL(WP), INTENT(OUT), OPTIONAL :: AMAX
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(OUT) :: R(:)
		REAL(WP), INTENT(OUT) :: C(:)
	END SUBROUTINE CGBEQU_MKL95
	SUBROUTINE ZGBEQU_MKL95(A,R,C,KL,ROWCND,COLCND,AMAX,INFO)
		! MKL Fortran77 call:
		! ZGBEQU(M,N,KL,KU,A,LDA,R,C,ROWCND,COLCND,AMAX,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		REAL(WP), INTENT(OUT), OPTIONAL :: ROWCND
		REAL(WP), INTENT(OUT), OPTIONAL :: COLCND
		REAL(WP), INTENT(OUT), OPTIONAL :: AMAX
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(OUT) :: R(:)
		REAL(WP), INTENT(OUT) :: C(:)
	END SUBROUTINE ZGBEQU_MKL95
END INTERFACE GBEQU
INTERFACE POEQU
	SUBROUTINE SPOEQU_MKL95(A,S,SCOND,AMAX,INFO)
		! MKL Fortran77 call:
		! SPOEQU(N,A,LDA,S,SCOND,AMAX,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(OUT), OPTIONAL :: SCOND
		REAL(WP), INTENT(OUT), OPTIONAL :: AMAX
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(OUT) :: S(:)
	END SUBROUTINE SPOEQU_MKL95
	SUBROUTINE DPOEQU_MKL95(A,S,SCOND,AMAX,INFO)
		! MKL Fortran77 call:
		! DPOEQU(N,A,LDA,S,SCOND,AMAX,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(OUT), OPTIONAL :: SCOND
		REAL(WP), INTENT(OUT), OPTIONAL :: AMAX
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(OUT) :: S(:)
	END SUBROUTINE DPOEQU_MKL95
	SUBROUTINE CPOEQU_MKL95(A,S,SCOND,AMAX,INFO)
		! MKL Fortran77 call:
		! CPOEQU(N,A,LDA,S,SCOND,AMAX,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(OUT), OPTIONAL :: SCOND
		REAL(WP), INTENT(OUT), OPTIONAL :: AMAX
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(OUT) :: S(:)
	END SUBROUTINE CPOEQU_MKL95
	SUBROUTINE ZPOEQU_MKL95(A,S,SCOND,AMAX,INFO)
		! MKL Fortran77 call:
		! ZPOEQU(N,A,LDA,S,SCOND,AMAX,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(OUT), OPTIONAL :: SCOND
		REAL(WP), INTENT(OUT), OPTIONAL :: AMAX
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(OUT) :: S(:)
	END SUBROUTINE ZPOEQU_MKL95
END INTERFACE POEQU
INTERFACE PPEQU
	SUBROUTINE SPPEQU_MKL95(A,S,SCOND,AMAX,UPLO,INFO)
		! MKL Fortran77 call:
		! SPPEQU(UPLO,N,A,S,SCOND,AMAX,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(OUT), OPTIONAL :: SCOND
		REAL(WP), INTENT(OUT), OPTIONAL :: AMAX
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(OUT) :: S(:)
	END SUBROUTINE SPPEQU_MKL95
	SUBROUTINE DPPEQU_MKL95(A,S,SCOND,AMAX,UPLO,INFO)
		! MKL Fortran77 call:
		! DPPEQU(UPLO,N,A,S,SCOND,AMAX,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(OUT), OPTIONAL :: SCOND
		REAL(WP), INTENT(OUT), OPTIONAL :: AMAX
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(OUT) :: S(:)
	END SUBROUTINE DPPEQU_MKL95
	SUBROUTINE CPPEQU_MKL95(A,S,SCOND,AMAX,UPLO,INFO)
		! MKL Fortran77 call:
		! CPPEQU(UPLO,N,A,S,SCOND,AMAX,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(OUT), OPTIONAL :: SCOND
		REAL(WP), INTENT(OUT), OPTIONAL :: AMAX
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(OUT) :: S(:)
	END SUBROUTINE CPPEQU_MKL95
	SUBROUTINE ZPPEQU_MKL95(A,S,SCOND,AMAX,UPLO,INFO)
		! MKL Fortran77 call:
		! ZPPEQU(UPLO,N,A,S,SCOND,AMAX,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(OUT), OPTIONAL :: SCOND
		REAL(WP), INTENT(OUT), OPTIONAL :: AMAX
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(OUT) :: S(:)
	END SUBROUTINE ZPPEQU_MKL95
END INTERFACE PPEQU
INTERFACE PBEQU
	SUBROUTINE SPBEQU_MKL95(A,S,SCOND,AMAX,UPLO,INFO)
		! MKL Fortran77 call:
		! SPBEQU(UPLO,N,KD,A,LDA,S,SCOND,AMAX,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(OUT), OPTIONAL :: SCOND
		REAL(WP), INTENT(OUT), OPTIONAL :: AMAX
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(OUT) :: S(:)
	END SUBROUTINE SPBEQU_MKL95
	SUBROUTINE DPBEQU_MKL95(A,S,SCOND,AMAX,UPLO,INFO)
		! MKL Fortran77 call:
		! DPBEQU(UPLO,N,KD,A,LDA,S,SCOND,AMAX,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(OUT), OPTIONAL :: SCOND
		REAL(WP), INTENT(OUT), OPTIONAL :: AMAX
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(OUT) :: S(:)
	END SUBROUTINE DPBEQU_MKL95
	SUBROUTINE CPBEQU_MKL95(A,S,SCOND,AMAX,UPLO,INFO)
		! MKL Fortran77 call:
		! CPBEQU(UPLO,N,KD,A,LDA,S,SCOND,AMAX,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(OUT), OPTIONAL :: SCOND
		REAL(WP), INTENT(OUT), OPTIONAL :: AMAX
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(OUT) :: S(:)
	END SUBROUTINE CPBEQU_MKL95
	SUBROUTINE ZPBEQU_MKL95(A,S,SCOND,AMAX,UPLO,INFO)
		! MKL Fortran77 call:
		! ZPBEQU(UPLO,N,KD,A,LDA,S,SCOND,AMAX,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(OUT), OPTIONAL :: SCOND
		REAL(WP), INTENT(OUT), OPTIONAL :: AMAX
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(OUT) :: S(:)
	END SUBROUTINE ZPBEQU_MKL95
END INTERFACE PBEQU
INTERFACE GESV
	SUBROUTINE SGESV_MKL95(A,B,IPIV,INFO)
		! MKL Fortran77 call:
		! SGESV(N,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE SGESV_MKL95
	SUBROUTINE SGESV1_MKL95(A,B,IPIV,INFO)
		! MKL Fortran77 call:
		! SGESV(N,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE SGESV1_MKL95
	SUBROUTINE DGESV_MKL95(A,B,IPIV,INFO)
		! MKL Fortran77 call:
		! DGESV(N,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE DGESV_MKL95
	SUBROUTINE DGESV1_MKL95(A,B,IPIV,INFO)
		! MKL Fortran77 call:
		! DGESV(N,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE DGESV1_MKL95
	SUBROUTINE CGESV_MKL95(A,B,IPIV,INFO)
		! MKL Fortran77 call:
		! CGESV(N,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE CGESV_MKL95
	SUBROUTINE CGESV1_MKL95(A,B,IPIV,INFO)
		! MKL Fortran77 call:
		! CGESV(N,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE CGESV1_MKL95
	SUBROUTINE ZGESV_MKL95(A,B,IPIV,INFO)
		! MKL Fortran77 call:
		! ZGESV(N,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE ZGESV_MKL95
	SUBROUTINE ZGESV1_MKL95(A,B,IPIV,INFO)
		! MKL Fortran77 call:
		! ZGESV(N,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE ZGESV1_MKL95
END INTERFACE GESV
INTERFACE GESVX
	SUBROUTINE SGESVX_MKL95(A,B,X,AF,IPIV,FACT,TRANS,EQUED,R,C,FERR,BERR,RCOND,RPVGRW,INFO)
		! MKL Fortran77 call:
		! SGESVX(FACT,TRANS,N,NRHS,A,LDA,AF,LDAF,IPIV,EQUED,R,C,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		REAL(WP), INTENT(OUT), OPTIONAL :: RPVGRW
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: X(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: R(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: C(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE SGESVX_MKL95
	SUBROUTINE SGESVX1_MKL95(A,B,X,AF,IPIV,FACT,TRANS,EQUED,R,C,FERR,BERR,RCOND,RPVGRW,INFO)
		! MKL Fortran77 call:
		! SGESVX(FACT,TRANS,N,NRHS,A,LDA,AF,LDAF,IPIV,EQUED,R,C,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		REAL(WP), INTENT(OUT), OPTIONAL :: RPVGRW
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT) :: X(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: R(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: C(:)
	END SUBROUTINE SGESVX1_MKL95
	SUBROUTINE DGESVX_MKL95(A,B,X,AF,IPIV,FACT,TRANS,EQUED,R,C,FERR,BERR,RCOND,RPVGRW,INFO)
		! MKL Fortran77 call:
		! DGESVX(FACT,TRANS,N,NRHS,A,LDA,AF,LDAF,IPIV,EQUED,R,C,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		REAL(WP), INTENT(OUT), OPTIONAL :: RPVGRW
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: X(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: R(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: C(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE DGESVX_MKL95
	SUBROUTINE DGESVX1_MKL95(A,B,X,AF,IPIV,FACT,TRANS,EQUED,R,C,FERR,BERR,RCOND,RPVGRW,INFO)
		! MKL Fortran77 call:
		! DGESVX(FACT,TRANS,N,NRHS,A,LDA,AF,LDAF,IPIV,EQUED,R,C,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		REAL(WP), INTENT(OUT), OPTIONAL :: RPVGRW
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT) :: X(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: R(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: C(:)
	END SUBROUTINE DGESVX1_MKL95
	SUBROUTINE CGESVX_MKL95(A,B,X,AF,IPIV,FACT,TRANS,EQUED,R,C,FERR,BERR,RCOND,RPVGRW,INFO)
		! MKL Fortran77 call:
		! CGESVX(FACT,TRANS,N,NRHS,A,LDA,AF,LDAF,IPIV,EQUED,R,C,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		REAL(WP), INTENT(OUT), OPTIONAL :: RPVGRW
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: X(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: R(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: C(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE CGESVX_MKL95
	SUBROUTINE CGESVX1_MKL95(A,B,X,AF,IPIV,FACT,TRANS,EQUED,R,C,FERR,BERR,RCOND,RPVGRW,INFO)
		! MKL Fortran77 call:
		! CGESVX(FACT,TRANS,N,NRHS,A,LDA,AF,LDAF,IPIV,EQUED,R,C,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		REAL(WP), INTENT(OUT), OPTIONAL :: RPVGRW
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		COMPLEX(WP), INTENT(OUT) :: X(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: R(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: C(:)
	END SUBROUTINE CGESVX1_MKL95
	SUBROUTINE ZGESVX_MKL95(A,B,X,AF,IPIV,FACT,TRANS,EQUED,R,C,FERR,BERR,RCOND,RPVGRW,INFO)
		! MKL Fortran77 call:
		! ZGESVX(FACT,TRANS,N,NRHS,A,LDA,AF,LDAF,IPIV,EQUED,R,C,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		REAL(WP), INTENT(OUT), OPTIONAL :: RPVGRW
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: X(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: R(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: C(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE ZGESVX_MKL95
	SUBROUTINE ZGESVX1_MKL95(A,B,X,AF,IPIV,FACT,TRANS,EQUED,R,C,FERR,BERR,RCOND,RPVGRW,INFO)
		! MKL Fortran77 call:
		! ZGESVX(FACT,TRANS,N,NRHS,A,LDA,AF,LDAF,IPIV,EQUED,R,C,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		REAL(WP), INTENT(OUT), OPTIONAL :: RPVGRW
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		COMPLEX(WP), INTENT(OUT) :: X(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: R(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: C(:)
	END SUBROUTINE ZGESVX1_MKL95
END INTERFACE GESVX
INTERFACE GBSV
	SUBROUTINE SGBSV_MKL95(A,B,KL,IPIV,INFO)
		! MKL Fortran77 call:
		! SGBSV(N,KL,KU,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE SGBSV_MKL95
	SUBROUTINE SGBSV1_MKL95(A,B,KL,IPIV,INFO)
		! MKL Fortran77 call:
		! SGBSV(N,KL,KU,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE SGBSV1_MKL95
	SUBROUTINE DGBSV_MKL95(A,B,KL,IPIV,INFO)
		! MKL Fortran77 call:
		! DGBSV(N,KL,KU,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE DGBSV_MKL95
	SUBROUTINE DGBSV1_MKL95(A,B,KL,IPIV,INFO)
		! MKL Fortran77 call:
		! DGBSV(N,KL,KU,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE DGBSV1_MKL95
	SUBROUTINE CGBSV_MKL95(A,B,KL,IPIV,INFO)
		! MKL Fortran77 call:
		! CGBSV(N,KL,KU,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE CGBSV_MKL95
	SUBROUTINE CGBSV1_MKL95(A,B,KL,IPIV,INFO)
		! MKL Fortran77 call:
		! CGBSV(N,KL,KU,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE CGBSV1_MKL95
	SUBROUTINE ZGBSV_MKL95(A,B,KL,IPIV,INFO)
		! MKL Fortran77 call:
		! ZGBSV(N,KL,KU,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE ZGBSV_MKL95
	SUBROUTINE ZGBSV1_MKL95(A,B,KL,IPIV,INFO)
		! MKL Fortran77 call:
		! ZGBSV(N,KL,KU,NRHS,A,LDA,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE ZGBSV1_MKL95
END INTERFACE GBSV
INTERFACE GBSVX
	SUBROUTINE SGBSVX_MKL95(A,B,X,KL,AF,IPIV,FACT,TRANS,EQUED,R,C,FERR,BERR,RCOND,RPVGRW,INFO)
		! MKL Fortran77 call:
		! SGBSVX(FACT,TRANS,N,KL,KU,NRHS,A,LDA,AF,LDAF,IPIV,EQUED,R,C,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		REAL(WP), INTENT(OUT), OPTIONAL :: RPVGRW
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: X(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: R(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: C(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE SGBSVX_MKL95
	SUBROUTINE SGBSVX1_MKL95(A,B,X,KL,AF,IPIV,FACT,TRANS,EQUED,R,C,FERR,BERR,RCOND,RPVGRW,INFO)
		! MKL Fortran77 call:
		! SGBSVX(FACT,TRANS,N,KL,KU,NRHS,A,LDA,AF,LDAF,IPIV,EQUED,R,C,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		REAL(WP), INTENT(OUT), OPTIONAL :: RPVGRW
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT) :: X(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: R(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: C(:)
	END SUBROUTINE SGBSVX1_MKL95
	SUBROUTINE DGBSVX_MKL95(A,B,X,KL,AF,IPIV,FACT,TRANS,EQUED,R,C,FERR,BERR,RCOND,RPVGRW,INFO)
		! MKL Fortran77 call:
		! DGBSVX(FACT,TRANS,N,KL,KU,NRHS,A,LDA,AF,LDAF,IPIV,EQUED,R,C,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		REAL(WP), INTENT(OUT), OPTIONAL :: RPVGRW
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: X(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: R(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: C(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE DGBSVX_MKL95
	SUBROUTINE DGBSVX1_MKL95(A,B,X,KL,AF,IPIV,FACT,TRANS,EQUED,R,C,FERR,BERR,RCOND,RPVGRW,INFO)
		! MKL Fortran77 call:
		! DGBSVX(FACT,TRANS,N,KL,KU,NRHS,A,LDA,AF,LDAF,IPIV,EQUED,R,C,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		REAL(WP), INTENT(OUT), OPTIONAL :: RPVGRW
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT) :: X(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: R(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: C(:)
	END SUBROUTINE DGBSVX1_MKL95
	SUBROUTINE CGBSVX_MKL95(A,B,X,KL,AF,IPIV,FACT,TRANS,EQUED,R,C,FERR,BERR,RCOND,RPVGRW,INFO)
		! MKL Fortran77 call:
		! CGBSVX(FACT,TRANS,N,KL,KU,NRHS,A,LDA,AF,LDAF,IPIV,EQUED,R,C,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		REAL(WP), INTENT(OUT), OPTIONAL :: RPVGRW
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: X(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: R(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: C(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE CGBSVX_MKL95
	SUBROUTINE CGBSVX1_MKL95(A,B,X,KL,AF,IPIV,FACT,TRANS,EQUED,R,C,FERR,BERR,RCOND,RPVGRW,INFO)
		! MKL Fortran77 call:
		! CGBSVX(FACT,TRANS,N,KL,KU,NRHS,A,LDA,AF,LDAF,IPIV,EQUED,R,C,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		REAL(WP), INTENT(OUT), OPTIONAL :: RPVGRW
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		COMPLEX(WP), INTENT(OUT) :: X(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: R(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: C(:)
	END SUBROUTINE CGBSVX1_MKL95
	SUBROUTINE ZGBSVX_MKL95(A,B,X,KL,AF,IPIV,FACT,TRANS,EQUED,R,C,FERR,BERR,RCOND,RPVGRW,INFO)
		! MKL Fortran77 call:
		! ZGBSVX(FACT,TRANS,N,KL,KU,NRHS,A,LDA,AF,LDAF,IPIV,EQUED,R,C,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		REAL(WP), INTENT(OUT), OPTIONAL :: RPVGRW
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: X(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: R(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: C(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE ZGBSVX_MKL95
	SUBROUTINE ZGBSVX1_MKL95(A,B,X,KL,AF,IPIV,FACT,TRANS,EQUED,R,C,FERR,BERR,RCOND,RPVGRW,INFO)
		! MKL Fortran77 call:
		! ZGBSVX(FACT,TRANS,N,KL,KU,NRHS,A,LDA,AF,LDAF,IPIV,EQUED,R,C,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		REAL(WP), INTENT(OUT), OPTIONAL :: RPVGRW
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		COMPLEX(WP), INTENT(OUT) :: X(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: R(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: C(:)
	END SUBROUTINE ZGBSVX1_MKL95
END INTERFACE GBSVX
INTERFACE GTSV
	SUBROUTINE SGTSV_MKL95(DL,D,DU,B,INFO)
		! MKL Fortran77 call:
		! SGTSV(N,NRHS,DL,D,DU,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: DL(:)
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: DU(:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE SGTSV_MKL95
	SUBROUTINE SGTSV1_MKL95(DL,D,DU,B,INFO)
		! MKL Fortran77 call:
		! SGTSV(N,NRHS,DL,D,DU,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: DL(:)
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: DU(:)
		REAL(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE SGTSV1_MKL95
	SUBROUTINE DGTSV_MKL95(DL,D,DU,B,INFO)
		! MKL Fortran77 call:
		! DGTSV(N,NRHS,DL,D,DU,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: DL(:)
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: DU(:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE DGTSV_MKL95
	SUBROUTINE DGTSV1_MKL95(DL,D,DU,B,INFO)
		! MKL Fortran77 call:
		! DGTSV(N,NRHS,DL,D,DU,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: DL(:)
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: DU(:)
		REAL(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE DGTSV1_MKL95
	SUBROUTINE CGTSV_MKL95(DL,D,DU,B,INFO)
		! MKL Fortran77 call:
		! CGTSV(N,NRHS,DL,D,DU,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: DL(:)
		COMPLEX(WP), INTENT(INOUT) :: D(:)
		COMPLEX(WP), INTENT(INOUT) :: DU(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE CGTSV_MKL95
	SUBROUTINE CGTSV1_MKL95(DL,D,DU,B,INFO)
		! MKL Fortran77 call:
		! CGTSV(N,NRHS,DL,D,DU,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: DL(:)
		COMPLEX(WP), INTENT(INOUT) :: D(:)
		COMPLEX(WP), INTENT(INOUT) :: DU(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE CGTSV1_MKL95
	SUBROUTINE ZGTSV_MKL95(DL,D,DU,B,INFO)
		! MKL Fortran77 call:
		! ZGTSV(N,NRHS,DL,D,DU,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: DL(:)
		COMPLEX(WP), INTENT(INOUT) :: D(:)
		COMPLEX(WP), INTENT(INOUT) :: DU(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE ZGTSV_MKL95
	SUBROUTINE ZGTSV1_MKL95(DL,D,DU,B,INFO)
		! MKL Fortran77 call:
		! ZGTSV(N,NRHS,DL,D,DU,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: DL(:)
		COMPLEX(WP), INTENT(INOUT) :: D(:)
		COMPLEX(WP), INTENT(INOUT) :: DU(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE ZGTSV1_MKL95
END INTERFACE GTSV
INTERFACE GTSVX
	SUBROUTINE SGTSVX_MKL95(DL,D,DU,B,X,DLF,DF,DUF,DU2,IPIV,FACT,TRANS,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! SGTSVX(FACT,TRANS,N,NRHS,DL,D,DU,DLF,DF,DUF,DU2,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: DL(:)
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: DU(:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(OUT) :: X(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: DLF(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: DF(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: DUF(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: DU2(:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE SGTSVX_MKL95
	SUBROUTINE SGTSVX1_MKL95(DL,D,DU,B,X,DLF,DF,DUF,DU2,IPIV,FACT,TRANS,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! SGTSVX(FACT,TRANS,N,NRHS,DL,D,DU,DLF,DF,DUF,DU2,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: DL(:)
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: DU(:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(OUT) :: X(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: DLF(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: DF(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: DUF(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: DU2(:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE SGTSVX1_MKL95
	SUBROUTINE DGTSVX_MKL95(DL,D,DU,B,X,DLF,DF,DUF,DU2,IPIV,FACT,TRANS,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! DGTSVX(FACT,TRANS,N,NRHS,DL,D,DU,DLF,DF,DUF,DU2,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: DL(:)
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: DU(:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(OUT) :: X(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: DLF(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: DF(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: DUF(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: DU2(:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE DGTSVX_MKL95
	SUBROUTINE DGTSVX1_MKL95(DL,D,DU,B,X,DLF,DF,DUF,DU2,IPIV,FACT,TRANS,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! DGTSVX(FACT,TRANS,N,NRHS,DL,D,DU,DLF,DF,DUF,DU2,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: DL(:)
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: DU(:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(OUT) :: X(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: DLF(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: DF(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: DUF(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: DU2(:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE DGTSVX1_MKL95
	SUBROUTINE CGTSVX_MKL95(DL,D,DU,B,X,DLF,DF,DUF,DU2,IPIV,FACT,TRANS,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! CGTSVX(FACT,TRANS,N,NRHS,DL,D,DU,DLF,DF,DUF,DU2,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: DL(:)
		COMPLEX(WP), INTENT(IN) :: D(:)
		COMPLEX(WP), INTENT(IN) :: DU(:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: X(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: DLF(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: DF(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: DUF(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: DU2(:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE CGTSVX_MKL95
	SUBROUTINE CGTSVX1_MKL95(DL,D,DU,B,X,DLF,DF,DUF,DU2,IPIV,FACT,TRANS,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! CGTSVX(FACT,TRANS,N,NRHS,DL,D,DU,DLF,DF,DUF,DU2,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: DL(:)
		COMPLEX(WP), INTENT(IN) :: D(:)
		COMPLEX(WP), INTENT(IN) :: DU(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(OUT) :: X(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: DLF(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: DF(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: DUF(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: DU2(:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE CGTSVX1_MKL95
	SUBROUTINE ZGTSVX_MKL95(DL,D,DU,B,X,DLF,DF,DUF,DU2,IPIV,FACT,TRANS,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! ZGTSVX(FACT,TRANS,N,NRHS,DL,D,DU,DLF,DF,DUF,DU2,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: DL(:)
		COMPLEX(WP), INTENT(IN) :: D(:)
		COMPLEX(WP), INTENT(IN) :: DU(:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: X(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: DLF(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: DF(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: DUF(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: DU2(:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE ZGTSVX_MKL95
	SUBROUTINE ZGTSVX1_MKL95(DL,D,DU,B,X,DLF,DF,DUF,DU2,IPIV,FACT,TRANS,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! ZGTSVX(FACT,TRANS,N,NRHS,DL,D,DU,DLF,DF,DUF,DU2,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: DL(:)
		COMPLEX(WP), INTENT(IN) :: D(:)
		COMPLEX(WP), INTENT(IN) :: DU(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(OUT) :: X(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: DLF(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: DF(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: DUF(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: DU2(:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE ZGTSVX1_MKL95
END INTERFACE GTSVX
INTERFACE POSV
	SUBROUTINE SPOSV_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! SPOSV(UPLO,N,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE SPOSV_MKL95
	SUBROUTINE SPOSV1_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! SPOSV(UPLO,N,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE SPOSV1_MKL95
	SUBROUTINE DPOSV_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! DPOSV(UPLO,N,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE DPOSV_MKL95
	SUBROUTINE DPOSV1_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! DPOSV(UPLO,N,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE DPOSV1_MKL95
	SUBROUTINE CPOSV_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! CPOSV(UPLO,N,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE CPOSV_MKL95
	SUBROUTINE CPOSV1_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! CPOSV(UPLO,N,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE CPOSV1_MKL95
	SUBROUTINE ZPOSV_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! ZPOSV(UPLO,N,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE ZPOSV_MKL95
	SUBROUTINE ZPOSV1_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! ZPOSV(UPLO,N,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE ZPOSV1_MKL95
END INTERFACE POSV
INTERFACE POSVX
	SUBROUTINE SPOSVX_MKL95(A,B,X,UPLO,AF,FACT,EQUED,S,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! SPOSVX(FACT,UPLO,N,NRHS,A,LDA,AF,LDAF,EQUED,S,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: X(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: S(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE SPOSVX_MKL95
	SUBROUTINE SPOSVX1_MKL95(A,B,X,UPLO,AF,FACT,EQUED,S,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! SPOSVX(FACT,UPLO,N,NRHS,A,LDA,AF,LDAF,EQUED,S,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT) :: X(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: S(:)
	END SUBROUTINE SPOSVX1_MKL95
	SUBROUTINE DPOSVX_MKL95(A,B,X,UPLO,AF,FACT,EQUED,S,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! DPOSVX(FACT,UPLO,N,NRHS,A,LDA,AF,LDAF,EQUED,S,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: X(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: S(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE DPOSVX_MKL95
	SUBROUTINE DPOSVX1_MKL95(A,B,X,UPLO,AF,FACT,EQUED,S,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! DPOSVX(FACT,UPLO,N,NRHS,A,LDA,AF,LDAF,EQUED,S,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT) :: X(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: S(:)
	END SUBROUTINE DPOSVX1_MKL95
	SUBROUTINE CPOSVX_MKL95(A,B,X,UPLO,AF,FACT,EQUED,S,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! CPOSVX(FACT,UPLO,N,NRHS,A,LDA,AF,LDAF,EQUED,S,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: X(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: S(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE CPOSVX_MKL95
	SUBROUTINE CPOSVX1_MKL95(A,B,X,UPLO,AF,FACT,EQUED,S,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! CPOSVX(FACT,UPLO,N,NRHS,A,LDA,AF,LDAF,EQUED,S,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		COMPLEX(WP), INTENT(OUT) :: X(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: S(:)
	END SUBROUTINE CPOSVX1_MKL95
	SUBROUTINE ZPOSVX_MKL95(A,B,X,UPLO,AF,FACT,EQUED,S,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! ZPOSVX(FACT,UPLO,N,NRHS,A,LDA,AF,LDAF,EQUED,S,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: X(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: S(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE ZPOSVX_MKL95
	SUBROUTINE ZPOSVX1_MKL95(A,B,X,UPLO,AF,FACT,EQUED,S,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! ZPOSVX(FACT,UPLO,N,NRHS,A,LDA,AF,LDAF,EQUED,S,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		COMPLEX(WP), INTENT(OUT) :: X(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: S(:)
	END SUBROUTINE ZPOSVX1_MKL95
END INTERFACE POSVX
INTERFACE PPSV
	SUBROUTINE SPPSV_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! SPPSV(UPLO,N,NRHS,A,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE SPPSV_MKL95
	SUBROUTINE SPPSV1_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! SPPSV(UPLO,N,NRHS,A,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE SPPSV1_MKL95
	SUBROUTINE DPPSV_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! DPPSV(UPLO,N,NRHS,A,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE DPPSV_MKL95
	SUBROUTINE DPPSV1_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! DPPSV(UPLO,N,NRHS,A,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE DPPSV1_MKL95
	SUBROUTINE CPPSV_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! CPPSV(UPLO,N,NRHS,A,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE CPPSV_MKL95
	SUBROUTINE CPPSV1_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! CPPSV(UPLO,N,NRHS,A,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE CPPSV1_MKL95
	SUBROUTINE ZPPSV_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! ZPPSV(UPLO,N,NRHS,A,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE ZPPSV_MKL95
	SUBROUTINE ZPPSV1_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! ZPPSV(UPLO,N,NRHS,A,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE ZPPSV1_MKL95
END INTERFACE PPSV
INTERFACE PPSVX
	SUBROUTINE SPPSVX_MKL95(A,B,X,UPLO,AF,FACT,EQUED,S,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! SPPSVX(FACT,UPLO,N,NRHS,A,AF,EQUED,S,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: X(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: S(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE SPPSVX_MKL95
	SUBROUTINE SPPSVX1_MKL95(A,B,X,UPLO,AF,FACT,EQUED,S,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! SPPSVX(FACT,UPLO,N,NRHS,A,AF,EQUED,S,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT) :: X(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: S(:)
	END SUBROUTINE SPPSVX1_MKL95
	SUBROUTINE DPPSVX_MKL95(A,B,X,UPLO,AF,FACT,EQUED,S,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! DPPSVX(FACT,UPLO,N,NRHS,A,AF,EQUED,S,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: X(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: S(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE DPPSVX_MKL95
	SUBROUTINE DPPSVX1_MKL95(A,B,X,UPLO,AF,FACT,EQUED,S,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! DPPSVX(FACT,UPLO,N,NRHS,A,AF,EQUED,S,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT) :: X(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: S(:)
	END SUBROUTINE DPPSVX1_MKL95
	SUBROUTINE CPPSVX_MKL95(A,B,X,UPLO,AF,FACT,EQUED,S,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! CPPSVX(FACT,UPLO,N,NRHS,A,AF,EQUED,S,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: X(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: S(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE CPPSVX_MKL95
	SUBROUTINE CPPSVX1_MKL95(A,B,X,UPLO,AF,FACT,EQUED,S,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! CPPSVX(FACT,UPLO,N,NRHS,A,AF,EQUED,S,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		COMPLEX(WP), INTENT(OUT) :: X(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: S(:)
	END SUBROUTINE CPPSVX1_MKL95
	SUBROUTINE ZPPSVX_MKL95(A,B,X,UPLO,AF,FACT,EQUED,S,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! ZPPSVX(FACT,UPLO,N,NRHS,A,AF,EQUED,S,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: X(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: S(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE ZPPSVX_MKL95
	SUBROUTINE ZPPSVX1_MKL95(A,B,X,UPLO,AF,FACT,EQUED,S,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! ZPPSVX(FACT,UPLO,N,NRHS,A,AF,EQUED,S,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		COMPLEX(WP), INTENT(OUT) :: X(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: S(:)
	END SUBROUTINE ZPPSVX1_MKL95
END INTERFACE PPSVX
INTERFACE PBSV
	SUBROUTINE SPBSV_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! SPBSV(UPLO,N,KD,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE SPBSV_MKL95
	SUBROUTINE SPBSV1_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! SPBSV(UPLO,N,KD,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE SPBSV1_MKL95
	SUBROUTINE DPBSV_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! DPBSV(UPLO,N,KD,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE DPBSV_MKL95
	SUBROUTINE DPBSV1_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! DPBSV(UPLO,N,KD,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE DPBSV1_MKL95
	SUBROUTINE CPBSV_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! CPBSV(UPLO,N,KD,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE CPBSV_MKL95
	SUBROUTINE CPBSV1_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! CPBSV(UPLO,N,KD,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE CPBSV1_MKL95
	SUBROUTINE ZPBSV_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! ZPBSV(UPLO,N,KD,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE ZPBSV_MKL95
	SUBROUTINE ZPBSV1_MKL95(A,B,UPLO,INFO)
		! MKL Fortran77 call:
		! ZPBSV(UPLO,N,KD,NRHS,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE ZPBSV1_MKL95
END INTERFACE PBSV
INTERFACE PBSVX
	SUBROUTINE SPBSVX_MKL95(A,B,X,UPLO,AF,FACT,EQUED,S,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! SPBSVX(FACT,UPLO,N,KD,NRHS,A,LDA,AF,LDAF,EQUED,S,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: X(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: S(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE SPBSVX_MKL95
	SUBROUTINE SPBSVX1_MKL95(A,B,X,UPLO,AF,FACT,EQUED,S,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! SPBSVX(FACT,UPLO,N,KD,NRHS,A,LDA,AF,LDAF,EQUED,S,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT) :: X(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: S(:)
	END SUBROUTINE SPBSVX1_MKL95
	SUBROUTINE DPBSVX_MKL95(A,B,X,UPLO,AF,FACT,EQUED,S,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! DPBSVX(FACT,UPLO,N,KD,NRHS,A,LDA,AF,LDAF,EQUED,S,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: X(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: S(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE DPBSVX_MKL95
	SUBROUTINE DPBSVX1_MKL95(A,B,X,UPLO,AF,FACT,EQUED,S,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! DPBSVX(FACT,UPLO,N,KD,NRHS,A,LDA,AF,LDAF,EQUED,S,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT) :: X(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: S(:)
	END SUBROUTINE DPBSVX1_MKL95
	SUBROUTINE CPBSVX_MKL95(A,B,X,UPLO,AF,FACT,EQUED,S,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! CPBSVX(FACT,UPLO,N,KD,NRHS,A,LDA,AF,LDAF,EQUED,S,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: X(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: S(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE CPBSVX_MKL95
	SUBROUTINE CPBSVX1_MKL95(A,B,X,UPLO,AF,FACT,EQUED,S,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! CPBSVX(FACT,UPLO,N,KD,NRHS,A,LDA,AF,LDAF,EQUED,S,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		COMPLEX(WP), INTENT(OUT) :: X(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: S(:)
	END SUBROUTINE CPBSVX1_MKL95
	SUBROUTINE ZPBSVX_MKL95(A,B,X,UPLO,AF,FACT,EQUED,S,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! ZPBSVX(FACT,UPLO,N,KD,NRHS,A,LDA,AF,LDAF,EQUED,S,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: X(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: S(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE ZPBSVX_MKL95
	SUBROUTINE ZPBSVX1_MKL95(A,B,X,UPLO,AF,FACT,EQUED,S,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! ZPBSVX(FACT,UPLO,N,KD,NRHS,A,LDA,AF,LDAF,EQUED,S,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		CHARACTER(LEN=1), INTENT(INOUT), OPTIONAL :: EQUED
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		COMPLEX(WP), INTENT(OUT) :: X(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: S(:)
	END SUBROUTINE ZPBSVX1_MKL95
END INTERFACE PBSVX
INTERFACE PTSV
	SUBROUTINE SPTSV_MKL95(D,E,B,INFO)
		! MKL Fortran77 call:
		! SPTSV(N,NRHS,D,E,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE SPTSV_MKL95
	SUBROUTINE SPTSV1_MKL95(D,E,B,INFO)
		! MKL Fortran77 call:
		! SPTSV(N,NRHS,D,E,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		REAL(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE SPTSV1_MKL95
	SUBROUTINE DPTSV_MKL95(D,E,B,INFO)
		! MKL Fortran77 call:
		! DPTSV(N,NRHS,D,E,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE DPTSV_MKL95
	SUBROUTINE DPTSV1_MKL95(D,E,B,INFO)
		! MKL Fortran77 call:
		! DPTSV(N,NRHS,D,E,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		REAL(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE DPTSV1_MKL95
	SUBROUTINE CPTSV_MKL95(D,E,B,INFO)
		! MKL Fortran77 call:
		! CPTSV(N,NRHS,D,E,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		COMPLEX(WP), INTENT(INOUT) :: E(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE CPTSV_MKL95
	SUBROUTINE CPTSV1_MKL95(D,E,B,INFO)
		! MKL Fortran77 call:
		! CPTSV(N,NRHS,D,E,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		COMPLEX(WP), INTENT(INOUT) :: E(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE CPTSV1_MKL95
	SUBROUTINE ZPTSV_MKL95(D,E,B,INFO)
		! MKL Fortran77 call:
		! ZPTSV(N,NRHS,D,E,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		COMPLEX(WP), INTENT(INOUT) :: E(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE ZPTSV_MKL95
	SUBROUTINE ZPTSV1_MKL95(D,E,B,INFO)
		! MKL Fortran77 call:
		! ZPTSV(N,NRHS,D,E,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		COMPLEX(WP), INTENT(INOUT) :: E(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE ZPTSV1_MKL95
END INTERFACE PTSV
INTERFACE PTSVX
	SUBROUTINE SPTSVX_MKL95(D,E,B,X,DF,EF,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! SPTSVX(FACT,N,NRHS,D,E,DF,EF,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: E(:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(OUT) :: X(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: DF(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: EF(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE SPTSVX_MKL95
	SUBROUTINE SPTSVX1_MKL95(D,E,B,X,DF,EF,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! SPTSVX(FACT,N,NRHS,D,E,DF,EF,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: E(:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(OUT) :: X(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: DF(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: EF(:)
	END SUBROUTINE SPTSVX1_MKL95
	SUBROUTINE DPTSVX_MKL95(D,E,B,X,DF,EF,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! DPTSVX(FACT,N,NRHS,D,E,DF,EF,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: E(:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(OUT) :: X(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: DF(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: EF(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE DPTSVX_MKL95
	SUBROUTINE DPTSVX1_MKL95(D,E,B,X,DF,EF,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! DPTSVX(FACT,N,NRHS,D,E,DF,EF,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: E(:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(OUT) :: X(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: DF(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: EF(:)
	END SUBROUTINE DPTSVX1_MKL95
	SUBROUTINE CPTSVX_MKL95(D,E,B,X,DF,EF,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! CPTSVX(FACT,N,NRHS,D,E,DF,EF,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		COMPLEX(WP), INTENT(IN) :: E(:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: X(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: DF(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: EF(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE CPTSVX_MKL95
	SUBROUTINE CPTSVX1_MKL95(D,E,B,X,DF,EF,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! CPTSVX(FACT,N,NRHS,D,E,DF,EF,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		COMPLEX(WP), INTENT(IN) :: E(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(OUT) :: X(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: DF(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: EF(:)
	END SUBROUTINE CPTSVX1_MKL95
	SUBROUTINE ZPTSVX_MKL95(D,E,B,X,DF,EF,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! ZPTSVX(FACT,N,NRHS,D,E,DF,EF,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		COMPLEX(WP), INTENT(IN) :: E(:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: X(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: DF(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: EF(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE ZPTSVX_MKL95
	SUBROUTINE ZPTSVX1_MKL95(D,E,B,X,DF,EF,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! ZPTSVX(FACT,N,NRHS,D,E,DF,EF,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		COMPLEX(WP), INTENT(IN) :: E(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(OUT) :: X(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: DF(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: EF(:)
	END SUBROUTINE ZPTSVX1_MKL95
END INTERFACE PTSVX
INTERFACE SYSV
	SUBROUTINE SSYSV_MKL95(A,B,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! SSYSV(UPLO,N,NRHS,A,LDA,IPIV,B,LDB,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE SSYSV_MKL95
	SUBROUTINE SSYSV1_MKL95(A,B,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! SSYSV(UPLO,N,NRHS,A,LDA,IPIV,B,LDB,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE SSYSV1_MKL95
	SUBROUTINE DSYSV_MKL95(A,B,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! DSYSV(UPLO,N,NRHS,A,LDA,IPIV,B,LDB,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE DSYSV_MKL95
	SUBROUTINE DSYSV1_MKL95(A,B,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! DSYSV(UPLO,N,NRHS,A,LDA,IPIV,B,LDB,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE DSYSV1_MKL95
	SUBROUTINE CSYSV_MKL95(A,B,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! CSYSV(UPLO,N,NRHS,A,LDA,IPIV,B,LDB,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE CSYSV_MKL95
	SUBROUTINE CSYSV1_MKL95(A,B,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! CSYSV(UPLO,N,NRHS,A,LDA,IPIV,B,LDB,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE CSYSV1_MKL95
	SUBROUTINE ZSYSV_MKL95(A,B,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! ZSYSV(UPLO,N,NRHS,A,LDA,IPIV,B,LDB,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE ZSYSV_MKL95
	SUBROUTINE ZSYSV1_MKL95(A,B,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! ZSYSV(UPLO,N,NRHS,A,LDA,IPIV,B,LDB,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE ZSYSV1_MKL95
END INTERFACE SYSV
INTERFACE SYSVX
	SUBROUTINE SSYSVX_MKL95(A,B,X,UPLO,AF,IPIV,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! SSYSVX(FACT,UPLO,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,LWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(OUT) :: X(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE SSYSVX_MKL95
	SUBROUTINE SSYSVX1_MKL95(A,B,X,UPLO,AF,IPIV,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! SSYSVX(FACT,UPLO,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,LWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(OUT) :: X(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE SSYSVX1_MKL95
	SUBROUTINE DSYSVX_MKL95(A,B,X,UPLO,AF,IPIV,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! DSYSVX(FACT,UPLO,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,LWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(OUT) :: X(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE DSYSVX_MKL95
	SUBROUTINE DSYSVX1_MKL95(A,B,X,UPLO,AF,IPIV,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! DSYSVX(FACT,UPLO,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,LWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(OUT) :: X(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE DSYSVX1_MKL95
	SUBROUTINE CSYSVX_MKL95(A,B,X,UPLO,AF,IPIV,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! CSYSVX(FACT,UPLO,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: X(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE CSYSVX_MKL95
	SUBROUTINE CSYSVX1_MKL95(A,B,X,UPLO,AF,IPIV,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! CSYSVX(FACT,UPLO,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(OUT) :: X(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE CSYSVX1_MKL95
	SUBROUTINE ZSYSVX_MKL95(A,B,X,UPLO,AF,IPIV,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! ZSYSVX(FACT,UPLO,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: X(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE ZSYSVX_MKL95
	SUBROUTINE ZSYSVX1_MKL95(A,B,X,UPLO,AF,IPIV,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! ZSYSVX(FACT,UPLO,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(OUT) :: X(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE ZSYSVX1_MKL95
END INTERFACE SYSVX
INTERFACE HESVX
	SUBROUTINE CHESVX_MKL95(A,B,X,UPLO,AF,IPIV,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! CHESVX(FACT,UPLO,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: X(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE CHESVX_MKL95
	SUBROUTINE CHESVX1_MKL95(A,B,X,UPLO,AF,IPIV,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! CHESVX(FACT,UPLO,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(OUT) :: X(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE CHESVX1_MKL95
	SUBROUTINE ZHESVX_MKL95(A,B,X,UPLO,AF,IPIV,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! ZHESVX(FACT,UPLO,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: X(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE ZHESVX_MKL95
	SUBROUTINE ZHESVX1_MKL95(A,B,X,UPLO,AF,IPIV,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! ZHESVX(FACT,UPLO,N,NRHS,A,LDA,AF,LDAF,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(OUT) :: X(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE ZHESVX1_MKL95
END INTERFACE HESVX
INTERFACE HESV
	SUBROUTINE CHESV_MKL95(A,B,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! CHESV(UPLO,N,NRHS,A,LDA,IPIV,B,LDB,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE CHESV_MKL95
	SUBROUTINE CHESV1_MKL95(A,B,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! CHESV(UPLO,N,NRHS,A,LDA,IPIV,B,LDB,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE CHESV1_MKL95
	SUBROUTINE ZHESV_MKL95(A,B,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! ZHESV(UPLO,N,NRHS,A,LDA,IPIV,B,LDB,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE ZHESV_MKL95
	SUBROUTINE ZHESV1_MKL95(A,B,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! ZHESV(UPLO,N,NRHS,A,LDA,IPIV,B,LDB,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE ZHESV1_MKL95
END INTERFACE HESV
INTERFACE SPSV
	SUBROUTINE SSPSV_MKL95(A,B,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! SSPSV(UPLO,N,NRHS,A,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE SSPSV_MKL95
	SUBROUTINE SSPSV1_MKL95(A,B,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! SSPSV(UPLO,N,NRHS,A,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE SSPSV1_MKL95
	SUBROUTINE DSPSV_MKL95(A,B,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! DSPSV(UPLO,N,NRHS,A,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE DSPSV_MKL95
	SUBROUTINE DSPSV1_MKL95(A,B,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! DSPSV(UPLO,N,NRHS,A,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE DSPSV1_MKL95
	SUBROUTINE CSPSV_MKL95(A,B,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! CSPSV(UPLO,N,NRHS,A,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE CSPSV_MKL95
	SUBROUTINE CSPSV1_MKL95(A,B,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! CSPSV(UPLO,N,NRHS,A,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE CSPSV1_MKL95
	SUBROUTINE ZSPSV_MKL95(A,B,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! ZSPSV(UPLO,N,NRHS,A,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE ZSPSV_MKL95
	SUBROUTINE ZSPSV1_MKL95(A,B,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! ZSPSV(UPLO,N,NRHS,A,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE ZSPSV1_MKL95
END INTERFACE SPSV
INTERFACE SPSVX
	SUBROUTINE SSPSVX_MKL95(A,B,X,UPLO,AF,IPIV,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! SSPSVX(FACT,UPLO,N,NRHS,A,AF,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(OUT) :: X(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE SSPSVX_MKL95
	SUBROUTINE SSPSVX1_MKL95(A,B,X,UPLO,AF,IPIV,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! SSPSVX(FACT,UPLO,N,NRHS,A,AF,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(OUT) :: X(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE SSPSVX1_MKL95
	SUBROUTINE DSPSVX_MKL95(A,B,X,UPLO,AF,IPIV,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! DSPSVX(FACT,UPLO,N,NRHS,A,AF,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(OUT) :: X(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE DSPSVX_MKL95
	SUBROUTINE DSPSVX1_MKL95(A,B,X,UPLO,AF,IPIV,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! DSPSVX(FACT,UPLO,N,NRHS,A,AF,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(IN) :: B(:)
		REAL(WP), INTENT(OUT) :: X(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE DSPSVX1_MKL95
	SUBROUTINE CSPSVX_MKL95(A,B,X,UPLO,AF,IPIV,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! CSPSVX(FACT,UPLO,N,NRHS,A,AF,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: X(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE CSPSVX_MKL95
	SUBROUTINE CSPSVX1_MKL95(A,B,X,UPLO,AF,IPIV,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! CSPSVX(FACT,UPLO,N,NRHS,A,AF,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(OUT) :: X(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE CSPSVX1_MKL95
	SUBROUTINE ZSPSVX_MKL95(A,B,X,UPLO,AF,IPIV,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! ZSPSVX(FACT,UPLO,N,NRHS,A,AF,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: X(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE ZSPSVX_MKL95
	SUBROUTINE ZSPSVX1_MKL95(A,B,X,UPLO,AF,IPIV,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! ZSPSVX(FACT,UPLO,N,NRHS,A,AF,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(OUT) :: X(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE ZSPSVX1_MKL95
END INTERFACE SPSVX
INTERFACE HPSVX
	SUBROUTINE CHPSVX_MKL95(A,B,X,UPLO,AF,IPIV,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! CHPSVX(FACT,UPLO,N,NRHS,A,AF,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: X(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE CHPSVX_MKL95
	SUBROUTINE CHPSVX1_MKL95(A,B,X,UPLO,AF,IPIV,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! CHPSVX(FACT,UPLO,N,NRHS,A,AF,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(OUT) :: X(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE CHPSVX1_MKL95
	SUBROUTINE ZHPSVX_MKL95(A,B,X,UPLO,AF,IPIV,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! ZHPSVX(FACT,UPLO,N,NRHS,A,AF,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: X(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: FERR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BERR(:)
	END SUBROUTINE ZHPSVX_MKL95
	SUBROUTINE ZHPSVX1_MKL95(A,B,X,UPLO,AF,IPIV,FACT,FERR,BERR,RCOND,INFO)
		! MKL Fortran77 call:
		! ZHPSVX(FACT,UPLO,N,NRHS,A,AF,IPIV,B,LDB,X,LDX,RCOND,FERR,BERR,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: FACT
		REAL(WP), INTENT(OUT), OPTIONAL :: FERR
		REAL(WP), INTENT(OUT), OPTIONAL :: BERR
		REAL(WP), INTENT(OUT), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
		COMPLEX(WP), INTENT(OUT) :: X(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: AF(:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE ZHPSVX1_MKL95
END INTERFACE HPSVX
INTERFACE HPSV
	SUBROUTINE CHPSV_MKL95(A,B,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! CHPSV(UPLO,N,NRHS,A,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE CHPSV_MKL95
	SUBROUTINE CHPSV1_MKL95(A,B,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! CHPSV(UPLO,N,NRHS,A,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE CHPSV1_MKL95
	SUBROUTINE ZHPSV_MKL95(A,B,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! ZHPSV(UPLO,N,NRHS,A,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE ZHPSV_MKL95
	SUBROUTINE ZHPSV1_MKL95(A,B,UPLO,IPIV,INFO)
		! MKL Fortran77 call:
		! ZHPSV(UPLO,N,NRHS,A,IPIV,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IPIV(:)
	END SUBROUTINE ZHPSV1_MKL95
END INTERFACE HPSV
INTERFACE GEQRF
	SUBROUTINE SGEQRF_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! SGEQRF(M,N,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE SGEQRF_MKL95
	SUBROUTINE DGEQRF_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! DGEQRF(M,N,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE DGEQRF_MKL95
	SUBROUTINE CGEQRF_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! CGEQRF(M,N,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE CGEQRF_MKL95
	SUBROUTINE ZGEQRF_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! ZGEQRF(M,N,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE ZGEQRF_MKL95
END INTERFACE GEQRF
INTERFACE GEQPF
	SUBROUTINE SGEQPF_MKL95(A,JPVT,TAU,INFO)
		! MKL Fortran77 call:
		! SGEQPF(M,N,A,LDA,JPVT,TAU,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(INOUT) :: JPVT(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE SGEQPF_MKL95
	SUBROUTINE DGEQPF_MKL95(A,JPVT,TAU,INFO)
		! MKL Fortran77 call:
		! DGEQPF(M,N,A,LDA,JPVT,TAU,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(INOUT) :: JPVT(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE DGEQPF_MKL95
	SUBROUTINE CGEQPF_MKL95(A,JPVT,TAU,INFO)
		! MKL Fortran77 call:
		! CGEQPF(M,N,A,LDA,JPVT,TAU,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(INOUT) :: JPVT(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE CGEQPF_MKL95
	SUBROUTINE ZGEQPF_MKL95(A,JPVT,TAU,INFO)
		! MKL Fortran77 call:
		! ZGEQPF(M,N,A,LDA,JPVT,TAU,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(INOUT) :: JPVT(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE ZGEQPF_MKL95
END INTERFACE GEQPF
INTERFACE GEQP3
	SUBROUTINE SGEQP3_MKL95(A,JPVT,TAU,INFO)
		! MKL Fortran77 call:
		! SGEQP3(M,N,A,LDA,JPVT,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(INOUT) :: JPVT(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE SGEQP3_MKL95
	SUBROUTINE DGEQP3_MKL95(A,JPVT,TAU,INFO)
		! MKL Fortran77 call:
		! DGEQP3(M,N,A,LDA,JPVT,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(INOUT) :: JPVT(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE DGEQP3_MKL95
	SUBROUTINE CGEQP3_MKL95(A,JPVT,TAU,INFO)
		! MKL Fortran77 call:
		! CGEQP3(M,N,A,LDA,JPVT,TAU,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(INOUT) :: JPVT(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE CGEQP3_MKL95
	SUBROUTINE ZGEQP3_MKL95(A,JPVT,TAU,INFO)
		! MKL Fortran77 call:
		! ZGEQP3(M,N,A,LDA,JPVT,TAU,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		INTEGER, INTENT(INOUT) :: JPVT(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE ZGEQP3_MKL95
END INTERFACE GEQP3
INTERFACE ORGQR
	SUBROUTINE SORGQR_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! SORGQR(M,N,K,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
	END SUBROUTINE SORGQR_MKL95
	SUBROUTINE DORGQR_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! DORGQR(M,N,K,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
	END SUBROUTINE DORGQR_MKL95
END INTERFACE ORGQR
INTERFACE ORMQR
	SUBROUTINE SORMQR_MKL95(A,TAU,C,SIDE,TRANS,INFO)
		! MKL Fortran77 call:
		! SORMQR(SIDE,TRANS,M,N,K,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
		REAL(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE SORMQR_MKL95
	SUBROUTINE DORMQR_MKL95(A,TAU,C,SIDE,TRANS,INFO)
		! MKL Fortran77 call:
		! DORMQR(SIDE,TRANS,M,N,K,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
		REAL(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE DORMQR_MKL95
END INTERFACE ORMQR
INTERFACE UNGQR
	SUBROUTINE CUNGQR_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! CUNGQR(M,N,K,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
	END SUBROUTINE CUNGQR_MKL95
	SUBROUTINE ZUNGQR_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! ZUNGQR(M,N,K,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
	END SUBROUTINE ZUNGQR_MKL95
END INTERFACE UNGQR
INTERFACE UNMQR
	SUBROUTINE CUNMQR_MKL95(A,TAU,C,SIDE,TRANS,INFO)
		! MKL Fortran77 call:
		! CUNMQR(SIDE,TRANS,M,N,K,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
		COMPLEX(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE CUNMQR_MKL95
	SUBROUTINE ZUNMQR_MKL95(A,TAU,C,SIDE,TRANS,INFO)
		! MKL Fortran77 call:
		! ZUNMQR(SIDE,TRANS,M,N,K,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
		COMPLEX(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE ZUNMQR_MKL95
END INTERFACE UNMQR
INTERFACE GELQF
	SUBROUTINE SGELQF_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! SGELQF(M,N,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE SGELQF_MKL95
	SUBROUTINE DGELQF_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! DGELQF(M,N,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE DGELQF_MKL95
	SUBROUTINE CGELQF_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! CGELQF(M,N,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE CGELQF_MKL95
	SUBROUTINE ZGELQF_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! ZGELQF(M,N,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE ZGELQF_MKL95
END INTERFACE GELQF
INTERFACE ORGLQ
	SUBROUTINE SORGLQ_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! SORGLQ(M,N,K,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
	END SUBROUTINE SORGLQ_MKL95
	SUBROUTINE DORGLQ_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! DORGLQ(M,N,K,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
	END SUBROUTINE DORGLQ_MKL95
END INTERFACE ORGLQ
INTERFACE ORMLQ
	SUBROUTINE SORMLQ_MKL95(A,TAU,C,SIDE,TRANS,INFO)
		! MKL Fortran77 call:
		! SORMLQ(SIDE,TRANS,M,N,K,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
		REAL(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE SORMLQ_MKL95
	SUBROUTINE DORMLQ_MKL95(A,TAU,C,SIDE,TRANS,INFO)
		! MKL Fortran77 call:
		! DORMLQ(SIDE,TRANS,M,N,K,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
		REAL(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE DORMLQ_MKL95
END INTERFACE ORMLQ
INTERFACE UNGLQ
	SUBROUTINE CUNGLQ_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! CUNGLQ(M,N,K,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
	END SUBROUTINE CUNGLQ_MKL95
	SUBROUTINE ZUNGLQ_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! ZUNGLQ(M,N,K,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
	END SUBROUTINE ZUNGLQ_MKL95
END INTERFACE UNGLQ
INTERFACE UNMLQ
	SUBROUTINE CUNMLQ_MKL95(A,TAU,C,SIDE,TRANS,INFO)
		! MKL Fortran77 call:
		! CUNMLQ(SIDE,TRANS,M,N,K,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
		COMPLEX(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE CUNMLQ_MKL95
	SUBROUTINE ZUNMLQ_MKL95(A,TAU,C,SIDE,TRANS,INFO)
		! MKL Fortran77 call:
		! ZUNMLQ(SIDE,TRANS,M,N,K,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
		COMPLEX(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE ZUNMLQ_MKL95
END INTERFACE UNMLQ
INTERFACE GEQLF
	SUBROUTINE SGEQLF_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! SGEQLF(M,N,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE SGEQLF_MKL95
	SUBROUTINE DGEQLF_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! DGEQLF(M,N,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE DGEQLF_MKL95
	SUBROUTINE CGEQLF_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! CGEQLF(M,N,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE CGEQLF_MKL95
	SUBROUTINE ZGEQLF_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! ZGEQLF(M,N,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE ZGEQLF_MKL95
END INTERFACE GEQLF
INTERFACE ORGQL
	SUBROUTINE SORGQL_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! SORGQL(M,N,K,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
	END SUBROUTINE SORGQL_MKL95
	SUBROUTINE DORGQL_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! DORGQL(M,N,K,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
	END SUBROUTINE DORGQL_MKL95
END INTERFACE ORGQL
INTERFACE UNGQL
	SUBROUTINE CUNGQL_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! CUNGQL(M,N,K,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
	END SUBROUTINE CUNGQL_MKL95
	SUBROUTINE ZUNGQL_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! ZUNGQL(M,N,K,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
	END SUBROUTINE ZUNGQL_MKL95
END INTERFACE UNGQL
INTERFACE ORMQL
	SUBROUTINE SORMQL_MKL95(A,TAU,C,SIDE,TRANS,INFO)
		! MKL Fortran77 call:
		! SORMQL(SIDE,TRANS,M,N,K,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
		REAL(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE SORMQL_MKL95
	SUBROUTINE DORMQL_MKL95(A,TAU,C,SIDE,TRANS,INFO)
		! MKL Fortran77 call:
		! DORMQL(SIDE,TRANS,M,N,K,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
		REAL(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE DORMQL_MKL95
END INTERFACE ORMQL
INTERFACE UNMQL
	SUBROUTINE CUNMQL_MKL95(A,TAU,C,SIDE,TRANS,INFO)
		! MKL Fortran77 call:
		! CUNMQL(SIDE,TRANS,M,N,K,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
		COMPLEX(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE CUNMQL_MKL95
	SUBROUTINE ZUNMQL_MKL95(A,TAU,C,SIDE,TRANS,INFO)
		! MKL Fortran77 call:
		! ZUNMQL(SIDE,TRANS,M,N,K,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
		COMPLEX(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE ZUNMQL_MKL95
END INTERFACE UNMQL
INTERFACE GERQF
	SUBROUTINE SGERQF_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! SGERQF(M,N,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE SGERQF_MKL95
	SUBROUTINE DGERQF_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! DGERQF(M,N,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE DGERQF_MKL95
	SUBROUTINE CGERQF_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! CGERQF(M,N,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE CGERQF_MKL95
	SUBROUTINE ZGERQF_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! ZGERQF(M,N,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE ZGERQF_MKL95
END INTERFACE GERQF
INTERFACE ORGRQ
	SUBROUTINE SORGRQ_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! SORGRQ(M,N,K,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
	END SUBROUTINE SORGRQ_MKL95
	SUBROUTINE DORGRQ_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! DORGRQ(M,N,K,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
	END SUBROUTINE DORGRQ_MKL95
END INTERFACE ORGRQ
INTERFACE UNGRQ
	SUBROUTINE CUNGRQ_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! CUNGRQ(M,N,K,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
	END SUBROUTINE CUNGRQ_MKL95
	SUBROUTINE ZUNGRQ_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! ZUNGRQ(M,N,K,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
	END SUBROUTINE ZUNGRQ_MKL95
END INTERFACE UNGRQ
INTERFACE ORMRQ
	SUBROUTINE SORMRQ_MKL95(A,TAU,C,SIDE,TRANS,INFO)
		! MKL Fortran77 call:
		! SORMRQ(SIDE,TRANS,M,N,K,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
		REAL(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE SORMRQ_MKL95
	SUBROUTINE DORMRQ_MKL95(A,TAU,C,SIDE,TRANS,INFO)
		! MKL Fortran77 call:
		! DORMRQ(SIDE,TRANS,M,N,K,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
		REAL(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE DORMRQ_MKL95
END INTERFACE ORMRQ
INTERFACE UNMRQ
	SUBROUTINE CUNMRQ_MKL95(A,TAU,C,SIDE,TRANS,INFO)
		! MKL Fortran77 call:
		! CUNMRQ(SIDE,TRANS,M,N,K,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
		COMPLEX(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE CUNMRQ_MKL95
	SUBROUTINE ZUNMRQ_MKL95(A,TAU,C,SIDE,TRANS,INFO)
		! MKL Fortran77 call:
		! ZUNMRQ(SIDE,TRANS,M,N,K,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
		COMPLEX(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE ZUNMRQ_MKL95
END INTERFACE UNMRQ
INTERFACE TZRZF
	SUBROUTINE STZRZF_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! STZRZF(M,N,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE STZRZF_MKL95
	SUBROUTINE DTZRZF_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! DTZRZF(M,N,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE DTZRZF_MKL95
	SUBROUTINE CTZRZF_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! CTZRZF(M,N,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE CTZRZF_MKL95
	SUBROUTINE ZTZRZF_MKL95(A,TAU,INFO)
		! MKL Fortran77 call:
		! ZTZRZF(M,N,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE ZTZRZF_MKL95
END INTERFACE TZRZF
INTERFACE ORMRZ
	SUBROUTINE SORMRZ_MKL95(A,TAU,C,L,SIDE,TRANS,INFO)
		! MKL Fortran77 call:
		! SORMRZ(SIDE,TRANS,M,N,K,L,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN) :: L
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
		REAL(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE SORMRZ_MKL95
	SUBROUTINE DORMRZ_MKL95(A,TAU,C,L,SIDE,TRANS,INFO)
		! MKL Fortran77 call:
		! DORMRZ(SIDE,TRANS,M,N,K,L,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN) :: L
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
		REAL(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE DORMRZ_MKL95
END INTERFACE ORMRZ
INTERFACE UNMRZ
	SUBROUTINE CUNMRZ_MKL95(A,TAU,C,L,SIDE,TRANS,INFO)
		! MKL Fortran77 call:
		! CUNMRZ(SIDE,TRANS,M,N,K,L,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN) :: L
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
		COMPLEX(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE CUNMRZ_MKL95
	SUBROUTINE ZUNMRZ_MKL95(A,TAU,C,L,SIDE,TRANS,INFO)
		! MKL Fortran77 call:
		! ZUNMRZ(SIDE,TRANS,M,N,K,L,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN) :: L
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
		COMPLEX(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE ZUNMRZ_MKL95
END INTERFACE UNMRZ
INTERFACE GGQRF
	SUBROUTINE SGGQRF_MKL95(A,B,TAUA,TAUB,INFO)
		! MKL Fortran77 call:
		! SGGQRF(N,M,P,A,LDA,TAUA,B,LDB,TAUB,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: TAUA(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: TAUB(:)
	END SUBROUTINE SGGQRF_MKL95
	SUBROUTINE DGGQRF_MKL95(A,B,TAUA,TAUB,INFO)
		! MKL Fortran77 call:
		! DGGQRF(N,M,P,A,LDA,TAUA,B,LDB,TAUB,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: TAUA(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: TAUB(:)
	END SUBROUTINE DGGQRF_MKL95
	SUBROUTINE CGGQRF_MKL95(A,B,TAUA,TAUB,INFO)
		! MKL Fortran77 call:
		! CGGQRF(N,M,P,A,LDA,TAUA,B,LDB,TAUB,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: TAUA(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: TAUB(:)
	END SUBROUTINE CGGQRF_MKL95
	SUBROUTINE ZGGQRF_MKL95(A,B,TAUA,TAUB,INFO)
		! MKL Fortran77 call:
		! ZGGQRF(N,M,P,A,LDA,TAUA,B,LDB,TAUB,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: TAUA(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: TAUB(:)
	END SUBROUTINE ZGGQRF_MKL95
END INTERFACE GGQRF
INTERFACE GGRQF
	SUBROUTINE SGGRQF_MKL95(A,B,TAUA,TAUB,INFO)
		! MKL Fortran77 call:
		! SGGRQF(M,P,N,A,LDA,TAUA,B,LDB,TAUB,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: TAUA(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: TAUB(:)
	END SUBROUTINE SGGRQF_MKL95
	SUBROUTINE DGGRQF_MKL95(A,B,TAUA,TAUB,INFO)
		! MKL Fortran77 call:
		! DGGRQF(M,P,N,A,LDA,TAUA,B,LDB,TAUB,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: TAUA(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: TAUB(:)
	END SUBROUTINE DGGRQF_MKL95
	SUBROUTINE CGGRQF_MKL95(A,B,TAUA,TAUB,INFO)
		! MKL Fortran77 call:
		! CGGRQF(M,P,N,A,LDA,TAUA,B,LDB,TAUB,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: TAUA(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: TAUB(:)
	END SUBROUTINE CGGRQF_MKL95
	SUBROUTINE ZGGRQF_MKL95(A,B,TAUA,TAUB,INFO)
		! MKL Fortran77 call:
		! ZGGRQF(M,P,N,A,LDA,TAUA,B,LDB,TAUB,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: TAUA(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: TAUB(:)
	END SUBROUTINE ZGGRQF_MKL95
END INTERFACE GGRQF
INTERFACE GEBRD
	SUBROUTINE SGEBRD_MKL95(A,D,E,TAUQ,TAUP,INFO)
		! MKL Fortran77 call:
		! SGEBRD(M,N,A,LDA,D,E,TAUQ,TAUP,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: D(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: E(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: TAUQ(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: TAUP(:)
	END SUBROUTINE SGEBRD_MKL95
	SUBROUTINE DGEBRD_MKL95(A,D,E,TAUQ,TAUP,INFO)
		! MKL Fortran77 call:
		! DGEBRD(M,N,A,LDA,D,E,TAUQ,TAUP,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: D(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: E(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: TAUQ(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: TAUP(:)
	END SUBROUTINE DGEBRD_MKL95
	SUBROUTINE CGEBRD_MKL95(A,D,E,TAUQ,TAUP,INFO)
		! MKL Fortran77 call:
		! CGEBRD(M,N,A,LDA,D,E,TAUQ,TAUP,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: D(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: E(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: TAUQ(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: TAUP(:)
	END SUBROUTINE CGEBRD_MKL95
	SUBROUTINE ZGEBRD_MKL95(A,D,E,TAUQ,TAUP,INFO)
		! MKL Fortran77 call:
		! ZGEBRD(M,N,A,LDA,D,E,TAUQ,TAUP,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: D(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: E(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: TAUQ(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: TAUP(:)
	END SUBROUTINE ZGEBRD_MKL95
END INTERFACE GEBRD
INTERFACE GBBRD
	SUBROUTINE SGBBRD_MKL95(A,C,D,E,Q,PT,KL,M,INFO)
		! MKL Fortran77 call:
		! SGBBRD(VECT,M,N,NCC,KL,KU,A,LDA,D,E,Q,LDQ,PT,LDPT,C,LDC,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		INTEGER, INTENT(IN), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: C(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: D(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: E(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Q(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: PT(:,:)
	END SUBROUTINE SGBBRD_MKL95
	SUBROUTINE DGBBRD_MKL95(A,C,D,E,Q,PT,KL,M,INFO)
		! MKL Fortran77 call:
		! DGBBRD(VECT,M,N,NCC,KL,KU,A,LDA,D,E,Q,LDQ,PT,LDPT,C,LDC,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		INTEGER, INTENT(IN), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: C(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: D(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: E(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Q(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: PT(:,:)
	END SUBROUTINE DGBBRD_MKL95
	SUBROUTINE CGBBRD_MKL95(A,C,D,E,Q,PT,KL,M,INFO)
		! MKL Fortran77 call:
		! CGBBRD(VECT,M,N,NCC,KL,KU,A,LDA,D,E,Q,LDQ,PT,LDPT,C,LDC,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		INTEGER, INTENT(IN), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: C(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: D(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: E(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Q(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: PT(:,:)
	END SUBROUTINE CGBBRD_MKL95
	SUBROUTINE ZGBBRD_MKL95(A,C,D,E,Q,PT,KL,M,INFO)
		! MKL Fortran77 call:
		! ZGBBRD(VECT,M,N,NCC,KL,KU,A,LDA,D,E,Q,LDQ,PT,LDPT,C,LDC,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: KL
		INTEGER, INTENT(IN), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: C(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: D(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: E(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Q(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: PT(:,:)
	END SUBROUTINE ZGBBRD_MKL95
END INTERFACE GBBRD
INTERFACE ORGBR
	SUBROUTINE SORGBR_MKL95(A,TAU,VECT,INFO)
		! MKL Fortran77 call:
		! SORGBR(VECT,M,N,K,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: VECT
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
	END SUBROUTINE SORGBR_MKL95
	SUBROUTINE DORGBR_MKL95(A,TAU,VECT,INFO)
		! MKL Fortran77 call:
		! DORGBR(VECT,M,N,K,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: VECT
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
	END SUBROUTINE DORGBR_MKL95
END INTERFACE ORGBR
INTERFACE ORMBR
	SUBROUTINE SORMBR_MKL95(A,TAU,C,VECT,SIDE,TRANS,INFO)
		! MKL Fortran77 call:
		! SORMBR(VECT,SIDE,TRANS,M,N,K,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: VECT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
		REAL(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE SORMBR_MKL95
	SUBROUTINE DORMBR_MKL95(A,TAU,C,VECT,SIDE,TRANS,INFO)
		! MKL Fortran77 call:
		! DORMBR(VECT,SIDE,TRANS,M,N,K,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: VECT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
		REAL(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE DORMBR_MKL95
END INTERFACE ORMBR
INTERFACE UNGBR
	SUBROUTINE CUNGBR_MKL95(A,TAU,VECT,INFO)
		! MKL Fortran77 call:
		! CUNGBR(VECT,M,N,K,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: VECT
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
	END SUBROUTINE CUNGBR_MKL95
	SUBROUTINE ZUNGBR_MKL95(A,TAU,VECT,INFO)
		! MKL Fortran77 call:
		! ZUNGBR(VECT,M,N,K,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: VECT
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
	END SUBROUTINE ZUNGBR_MKL95
END INTERFACE UNGBR
INTERFACE UNMBR
	SUBROUTINE CUNMBR_MKL95(A,TAU,C,VECT,SIDE,TRANS,INFO)
		! MKL Fortran77 call:
		! CUNMBR(VECT,SIDE,TRANS,M,N,K,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: VECT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
		COMPLEX(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE CUNMBR_MKL95
	SUBROUTINE ZUNMBR_MKL95(A,TAU,C,VECT,SIDE,TRANS,INFO)
		! MKL Fortran77 call:
		! ZUNMBR(VECT,SIDE,TRANS,M,N,K,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: VECT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
		COMPLEX(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE ZUNMBR_MKL95
END INTERFACE UNMBR
INTERFACE RBDSQR
	SUBROUTINE SBDSQR_MKL95(D,E,VT,U,C,UPLO,INFO)
		! MKL Fortran77 call:
		! SBDSQR(UPLO,N,NCVT,NRU,NCC,D,E,VT,LDVT,U,LDU,C,LDC,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: VT(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: U(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: C(:,:)
	END SUBROUTINE SBDSQR_MKL95
	SUBROUTINE DBDSQR_MKL95(D,E,VT,U,C,UPLO,INFO)
		! MKL Fortran77 call:
		! DBDSQR(UPLO,N,NCVT,NRU,NCC,D,E,VT,LDVT,U,LDU,C,LDC,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: VT(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: U(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: C(:,:)
	END SUBROUTINE DBDSQR_MKL95
END INTERFACE RBDSQR
INTERFACE BDSQR
	SUBROUTINE CBDSQR_MKL95(D,E,VT,U,C,UPLO,INFO)
		! MKL Fortran77 call:
		! CBDSQR(UPLO,N,NCVT,NRU,NCC,D,E,VT,LDVT,U,LDU,C,LDC,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: VT(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: U(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: C(:,:)
	END SUBROUTINE CBDSQR_MKL95
	SUBROUTINE ZBDSQR_MKL95(D,E,VT,U,C,UPLO,INFO)
		! MKL Fortran77 call:
		! ZBDSQR(UPLO,N,NCVT,NRU,NCC,D,E,VT,LDVT,U,LDU,C,LDC,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: VT(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: U(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: C(:,:)
	END SUBROUTINE ZBDSQR_MKL95
END INTERFACE BDSQR
INTERFACE BDSDC
	SUBROUTINE SBDSDC_MKL95(D,E,U,VT,Q,IQ,UPLO,INFO)
		! MKL Fortran77 call:
		! SBDSDC(UPLO,COMPQ,N,D,E,U,LDU,VT,LDVT,Q,IQ,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: U(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VT(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Q(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IQ(:)
	END SUBROUTINE SBDSDC_MKL95
	SUBROUTINE DBDSDC_MKL95(D,E,U,VT,Q,IQ,UPLO,INFO)
		! MKL Fortran77 call:
		! DBDSDC(UPLO,COMPQ,N,D,E,U,LDU,VT,LDVT,Q,IQ,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: U(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VT(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Q(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IQ(:)
	END SUBROUTINE DBDSDC_MKL95
END INTERFACE BDSDC
INTERFACE SYTRD
	SUBROUTINE SSYTRD_MKL95(A,TAU,UPLO,INFO)
		! MKL Fortran77 call:
		! SSYTRD(UPLO,N,A,LDA,D,E,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: TAU(:)
	END SUBROUTINE SSYTRD_MKL95
	SUBROUTINE DSYTRD_MKL95(A,TAU,UPLO,INFO)
		! MKL Fortran77 call:
		! DSYTRD(UPLO,N,A,LDA,D,E,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: TAU(:)
	END SUBROUTINE DSYTRD_MKL95
END INTERFACE SYTRD
INTERFACE ORGTR
	SUBROUTINE SORGTR_MKL95(A,TAU,UPLO,INFO)
		! MKL Fortran77 call:
		! SORGTR(UPLO,N,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
	END SUBROUTINE SORGTR_MKL95
	SUBROUTINE DORGTR_MKL95(A,TAU,UPLO,INFO)
		! MKL Fortran77 call:
		! DORGTR(UPLO,N,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
	END SUBROUTINE DORGTR_MKL95
END INTERFACE ORGTR
INTERFACE ORMTR
	SUBROUTINE SORMTR_MKL95(A,TAU,C,SIDE,UPLO,TRANS,INFO)
		! MKL Fortran77 call:
		! SORMTR(SIDE,UPLO,TRANS,M,N,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
		REAL(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE SORMTR_MKL95
	SUBROUTINE DORMTR_MKL95(A,TAU,C,SIDE,UPLO,TRANS,INFO)
		! MKL Fortran77 call:
		! DORMTR(SIDE,UPLO,TRANS,M,N,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
		REAL(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE DORMTR_MKL95
END INTERFACE ORMTR
INTERFACE HETRD
	SUBROUTINE CHETRD_MKL95(A,TAU,UPLO,INFO)
		! MKL Fortran77 call:
		! CHETRD(UPLO,N,A,LDA,D,E,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(OUT) :: TAU(:)
	END SUBROUTINE CHETRD_MKL95
	SUBROUTINE ZHETRD_MKL95(A,TAU,UPLO,INFO)
		! MKL Fortran77 call:
		! ZHETRD(UPLO,N,A,LDA,D,E,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(OUT) :: TAU(:)
	END SUBROUTINE ZHETRD_MKL95
END INTERFACE HETRD
INTERFACE UNGTR
	SUBROUTINE CUNGTR_MKL95(A,TAU,UPLO,INFO)
		! MKL Fortran77 call:
		! CUNGTR(UPLO,N,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
	END SUBROUTINE CUNGTR_MKL95
	SUBROUTINE ZUNGTR_MKL95(A,TAU,UPLO,INFO)
		! MKL Fortran77 call:
		! ZUNGTR(UPLO,N,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
	END SUBROUTINE ZUNGTR_MKL95
END INTERFACE UNGTR
INTERFACE UNMTR
	SUBROUTINE CUNMTR_MKL95(A,TAU,C,SIDE,UPLO,TRANS,INFO)
		! MKL Fortran77 call:
		! CUNMTR(SIDE,UPLO,TRANS,M,N,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
		COMPLEX(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE CUNMTR_MKL95
	SUBROUTINE ZUNMTR_MKL95(A,TAU,C,SIDE,UPLO,TRANS,INFO)
		! MKL Fortran77 call:
		! ZUNMTR(SIDE,UPLO,TRANS,M,N,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
		COMPLEX(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE ZUNMTR_MKL95
END INTERFACE UNMTR
INTERFACE SPTRD
	SUBROUTINE SSPTRD_MKL95(A,TAU,UPLO,INFO)
		! MKL Fortran77 call:
		! SSPTRD(UPLO,N,A,D,E,TAU,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(OUT) :: TAU(:)
	END SUBROUTINE SSPTRD_MKL95
	SUBROUTINE DSPTRD_MKL95(A,TAU,UPLO,INFO)
		! MKL Fortran77 call:
		! DSPTRD(UPLO,N,A,D,E,TAU,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(OUT) :: TAU(:)
	END SUBROUTINE DSPTRD_MKL95
END INTERFACE SPTRD
INTERFACE OPGTR
	SUBROUTINE SOPGTR_MKL95(A,TAU,Q,UPLO,INFO)
		! MKL Fortran77 call:
		! SOPGTR(UPLO,N,A,TAU,Q,LDQ,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(IN) :: TAU(:)
		REAL(WP), INTENT(OUT) :: Q(:,:)
	END SUBROUTINE SOPGTR_MKL95
	SUBROUTINE DOPGTR_MKL95(A,TAU,Q,UPLO,INFO)
		! MKL Fortran77 call:
		! DOPGTR(UPLO,N,A,TAU,Q,LDQ,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(IN) :: TAU(:)
		REAL(WP), INTENT(OUT) :: Q(:,:)
	END SUBROUTINE DOPGTR_MKL95
END INTERFACE OPGTR
INTERFACE OPMTR
	SUBROUTINE SOPMTR_MKL95(A,TAU,C,SIDE,UPLO,TRANS,INFO)
		! MKL Fortran77 call:
		! SOPMTR(SIDE,UPLO,TRANS,M,N,A,TAU,C,LDC,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(IN) :: TAU(:)
		REAL(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE SOPMTR_MKL95
	SUBROUTINE DOPMTR_MKL95(A,TAU,C,SIDE,UPLO,TRANS,INFO)
		! MKL Fortran77 call:
		! DOPMTR(SIDE,UPLO,TRANS,M,N,A,TAU,C,LDC,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:)
		REAL(WP), INTENT(IN) :: TAU(:)
		REAL(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE DOPMTR_MKL95
END INTERFACE OPMTR
INTERFACE HPTRD
	SUBROUTINE CHPTRD_MKL95(A,TAU,UPLO,INFO)
		! MKL Fortran77 call:
		! CHPTRD(UPLO,N,A,D,E,TAU,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		COMPLEX(WP), INTENT(OUT) :: TAU(:)
	END SUBROUTINE CHPTRD_MKL95
	SUBROUTINE ZHPTRD_MKL95(A,TAU,UPLO,INFO)
		! MKL Fortran77 call:
		! ZHPTRD(UPLO,N,A,D,E,TAU,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		COMPLEX(WP), INTENT(OUT) :: TAU(:)
	END SUBROUTINE ZHPTRD_MKL95
END INTERFACE HPTRD
INTERFACE UPGTR
	SUBROUTINE CUPGTR_MKL95(A,TAU,Q,UPLO,INFO)
		! MKL Fortran77 call:
		! CUPGTR(UPLO,N,A,TAU,Q,LDQ,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
		COMPLEX(WP), INTENT(OUT) :: Q(:,:)
	END SUBROUTINE CUPGTR_MKL95
	SUBROUTINE ZUPGTR_MKL95(A,TAU,Q,UPLO,INFO)
		! MKL Fortran77 call:
		! ZUPGTR(UPLO,N,A,TAU,Q,LDQ,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
		COMPLEX(WP), INTENT(OUT) :: Q(:,:)
	END SUBROUTINE ZUPGTR_MKL95
END INTERFACE UPGTR
INTERFACE UPMTR
	SUBROUTINE CUPMTR_MKL95(A,TAU,C,SIDE,UPLO,TRANS,INFO)
		! MKL Fortran77 call:
		! CUPMTR(SIDE,UPLO,TRANS,M,N,A,TAU,C,LDC,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
		COMPLEX(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE CUPMTR_MKL95
	SUBROUTINE ZUPMTR_MKL95(A,TAU,C,SIDE,UPLO,TRANS,INFO)
		! MKL Fortran77 call:
		! ZUPMTR(SIDE,UPLO,TRANS,M,N,A,TAU,C,LDC,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
		COMPLEX(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE ZUPMTR_MKL95
END INTERFACE UPMTR
INTERFACE SBTRD
	SUBROUTINE SSBTRD_MKL95(A,Q,VECT,UPLO,INFO)
		! MKL Fortran77 call:
		! SSBTRD(VECT,UPLO,N,KD,A,LDA,D,E,Q,LDQ,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: VECT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
	END SUBROUTINE SSBTRD_MKL95
	SUBROUTINE DSBTRD_MKL95(A,Q,VECT,UPLO,INFO)
		! MKL Fortran77 call:
		! DSBTRD(VECT,UPLO,N,KD,A,LDA,D,E,Q,LDQ,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: VECT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
	END SUBROUTINE DSBTRD_MKL95
END INTERFACE SBTRD
INTERFACE HBTRD
	SUBROUTINE CHBTRD_MKL95(A,Q,VECT,UPLO,INFO)
		! MKL Fortran77 call:
		! CHBTRD(VECT,UPLO,N,KD,A,LDA,D,E,Q,LDQ,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: VECT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
	END SUBROUTINE CHBTRD_MKL95
	SUBROUTINE ZHBTRD_MKL95(A,Q,VECT,UPLO,INFO)
		! MKL Fortran77 call:
		! ZHBTRD(VECT,UPLO,N,KD,A,LDA,D,E,Q,LDQ,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: VECT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
	END SUBROUTINE ZHBTRD_MKL95
END INTERFACE HBTRD
INTERFACE STERF
	SUBROUTINE SSTERF_MKL95(D,E,INFO)
		! MKL Fortran77 call:
		! SSTERF(N,D,E,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
	END SUBROUTINE SSTERF_MKL95
	SUBROUTINE DSTERF_MKL95(D,E,INFO)
		! MKL Fortran77 call:
		! DSTERF(N,D,E,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
	END SUBROUTINE DSTERF_MKL95
END INTERFACE STERF
INTERFACE RSTEQR
	SUBROUTINE SSTEQR_MKL95(D,E,Z,COMPZ,INFO)
		! MKL Fortran77 call:
		! SSTEQR(COMPZ,N,D,E,Z,LDZ,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPZ
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE SSTEQR_MKL95
	SUBROUTINE DSTEQR_MKL95(D,E,Z,COMPZ,INFO)
		! MKL Fortran77 call:
		! DSTEQR(COMPZ,N,D,E,Z,LDZ,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPZ
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE DSTEQR_MKL95
END INTERFACE RSTEQR
INTERFACE STEQR
	SUBROUTINE CSTEQR_MKL95(D,E,Z,COMPZ,INFO)
		! MKL Fortran77 call:
		! CSTEQR(COMPZ,N,D,E,Z,LDZ,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPZ
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE CSTEQR_MKL95
	SUBROUTINE ZSTEQR_MKL95(D,E,Z,COMPZ,INFO)
		! MKL Fortran77 call:
		! ZSTEQR(COMPZ,N,D,E,Z,LDZ,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPZ
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE ZSTEQR_MKL95
END INTERFACE STEQR
INTERFACE RSTEDC
	SUBROUTINE SSTEDC_MKL95(D,E,Z,COMPZ,INFO)
		! MKL Fortran77 call:
		! SSTEDC(COMPZ,N,D,E,Z,LDZ,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPZ
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE SSTEDC_MKL95
	SUBROUTINE DSTEDC_MKL95(D,E,Z,COMPZ,INFO)
		! MKL Fortran77 call:
		! DSTEDC(COMPZ,N,D,E,Z,LDZ,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPZ
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE DSTEDC_MKL95
END INTERFACE RSTEDC
INTERFACE STEDC
	SUBROUTINE CSTEDC_MKL95(D,E,Z,COMPZ,INFO)
		! MKL Fortran77 call:
		! CSTEDC(COMPZ,N,D,E,Z,LDZ,WORK,LWORK,RWORK,LRWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPZ
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE CSTEDC_MKL95
	SUBROUTINE ZSTEDC_MKL95(D,E,Z,COMPZ,INFO)
		! MKL Fortran77 call:
		! ZSTEDC(COMPZ,N,D,E,Z,LDZ,WORK,LWORK,RWORK,LRWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPZ
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE ZSTEDC_MKL95
END INTERFACE STEDC
INTERFACE RSTEGR
	SUBROUTINE SSTEGR_MKL95(D,E,W,Z,VL,VU,IL,IU,M,ISUPPZ,ABSTOL,INFO)
		! MKL Fortran77 call:
		! SSTEGR(JOBZ,RANGE,N,D,E,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,ISUPPZ,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: ISUPPZ(:)
	END SUBROUTINE SSTEGR_MKL95
	SUBROUTINE DSTEGR_MKL95(D,E,W,Z,VL,VU,IL,IU,M,ISUPPZ,ABSTOL,INFO)
		! MKL Fortran77 call:
		! DSTEGR(JOBZ,RANGE,N,D,E,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,ISUPPZ,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: ISUPPZ(:)
	END SUBROUTINE DSTEGR_MKL95
END INTERFACE RSTEGR
INTERFACE STEGR
	SUBROUTINE CSTEGR_MKL95(D,E,W,Z,VL,VU,IL,IU,M,ISUPPZ,ABSTOL,INFO)
		! MKL Fortran77 call:
		! CSTEGR(JOBZ,RANGE,N,D,E,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,ISUPPZ,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: ISUPPZ(:)
	END SUBROUTINE CSTEGR_MKL95
	SUBROUTINE ZSTEGR_MKL95(D,E,W,Z,VL,VU,IL,IU,M,ISUPPZ,ABSTOL,INFO)
		! MKL Fortran77 call:
		! ZSTEGR(JOBZ,RANGE,N,D,E,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,ISUPPZ,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: ISUPPZ(:)
	END SUBROUTINE ZSTEGR_MKL95
END INTERFACE STEGR
INTERFACE RPTEQR
	SUBROUTINE SPTEQR_MKL95(D,E,Z,COMPZ,INFO)
		! MKL Fortran77 call:
		! SPTEQR(COMPZ,N,D,E,Z,LDZ,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPZ
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE SPTEQR_MKL95
	SUBROUTINE DPTEQR_MKL95(D,E,Z,COMPZ,INFO)
		! MKL Fortran77 call:
		! DPTEQR(COMPZ,N,D,E,Z,LDZ,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPZ
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE DPTEQR_MKL95
END INTERFACE RPTEQR
INTERFACE PTEQR
	SUBROUTINE CPTEQR_MKL95(D,E,Z,COMPZ,INFO)
		! MKL Fortran77 call:
		! CPTEQR(COMPZ,N,D,E,Z,LDZ,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPZ
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE CPTEQR_MKL95
	SUBROUTINE ZPTEQR_MKL95(D,E,Z,COMPZ,INFO)
		! MKL Fortran77 call:
		! ZPTEQR(COMPZ,N,D,E,Z,LDZ,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPZ
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE ZPTEQR_MKL95
END INTERFACE PTEQR
INTERFACE STEBZ
	SUBROUTINE SSTEBZ_MKL95(D,E,M,NSPLIT,W,IBLOCK,ISPLIT,ORDER,VL,VU,IL,IU,ABSTOL,INFO)
		! MKL Fortran77 call:
		! SSTEBZ(RANGE,ORDER,N,VL,VU,IL,IU,ABSTOL,D,E,M,NSPLIT,W,IBLOCK,ISPLIT,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT) :: M
		INTEGER, INTENT(OUT) :: NSPLIT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: ORDER
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: E(:)
		REAL(WP), INTENT(OUT) :: W(:)
		INTEGER, INTENT(OUT) :: IBLOCK(:)
		INTEGER, INTENT(OUT) :: ISPLIT(:)
	END SUBROUTINE SSTEBZ_MKL95
	SUBROUTINE DSTEBZ_MKL95(D,E,M,NSPLIT,W,IBLOCK,ISPLIT,ORDER,VL,VU,IL,IU,ABSTOL,INFO)
		! MKL Fortran77 call:
		! DSTEBZ(RANGE,ORDER,N,VL,VU,IL,IU,ABSTOL,D,E,M,NSPLIT,W,IBLOCK,ISPLIT,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT) :: M
		INTEGER, INTENT(OUT) :: NSPLIT
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: ORDER
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: E(:)
		REAL(WP), INTENT(OUT) :: W(:)
		INTEGER, INTENT(OUT) :: IBLOCK(:)
		INTEGER, INTENT(OUT) :: ISPLIT(:)
	END SUBROUTINE DSTEBZ_MKL95
END INTERFACE STEBZ
INTERFACE STEIN
	SUBROUTINE SSTEIN_MKL95(D,E,W,IBLOCK,ISPLIT,Z,IFAILV,INFO)
		! MKL Fortran77 call:
		! SSTEIN(N,D,E,M,W,IBLOCK,ISPLIT,Z,LDZ,WORK,IWORK,IFAILV,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: E(:)
		REAL(WP), INTENT(IN) :: W(:)
		INTEGER, INTENT(IN) :: IBLOCK(:)
		INTEGER, INTENT(IN) :: ISPLIT(:)
		REAL(WP), INTENT(OUT) :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAILV(:)
	END SUBROUTINE SSTEIN_MKL95
	SUBROUTINE DSTEIN_MKL95(D,E,W,IBLOCK,ISPLIT,Z,IFAILV,INFO)
		! MKL Fortran77 call:
		! DSTEIN(N,D,E,M,W,IBLOCK,ISPLIT,Z,LDZ,WORK,IWORK,IFAILV,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: E(:)
		REAL(WP), INTENT(IN) :: W(:)
		INTEGER, INTENT(IN) :: IBLOCK(:)
		INTEGER, INTENT(IN) :: ISPLIT(:)
		REAL(WP), INTENT(OUT) :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAILV(:)
	END SUBROUTINE DSTEIN_MKL95
	SUBROUTINE CSTEIN_MKL95(D,E,W,IBLOCK,ISPLIT,Z,IFAILV,INFO)
		! MKL Fortran77 call:
		! CSTEIN(N,D,E,M,W,IBLOCK,ISPLIT,Z,LDZ,WORK,IWORK,IFAILV,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: E(:)
		REAL(WP), INTENT(IN) :: W(:)
		INTEGER, INTENT(IN) :: IBLOCK(:)
		INTEGER, INTENT(IN) :: ISPLIT(:)
		COMPLEX(WP), INTENT(OUT) :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAILV(:)
	END SUBROUTINE CSTEIN_MKL95
	SUBROUTINE ZSTEIN_MKL95(D,E,W,IBLOCK,ISPLIT,Z,IFAILV,INFO)
		! MKL Fortran77 call:
		! ZSTEIN(N,D,E,M,W,IBLOCK,ISPLIT,Z,LDZ,WORK,IWORK,IFAILV,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(IN) :: E(:)
		REAL(WP), INTENT(IN) :: W(:)
		INTEGER, INTENT(IN) :: IBLOCK(:)
		INTEGER, INTENT(IN) :: ISPLIT(:)
		COMPLEX(WP), INTENT(OUT) :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAILV(:)
	END SUBROUTINE ZSTEIN_MKL95
END INTERFACE STEIN
INTERFACE DISNA
	SUBROUTINE SDISNA_MKL95(D,SEP,JOB,MINMN,INFO)
		! MKL Fortran77 call:
		! SDISNA(JOB,M,N,D,SEP,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: MINMN
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(OUT) :: SEP(:)
	END SUBROUTINE SDISNA_MKL95
	SUBROUTINE DDISNA_MKL95(D,SEP,JOB,MINMN,INFO)
		! MKL Fortran77 call:
		! DDISNA(JOB,M,N,D,SEP,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: MINMN
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: D(:)
		REAL(WP), INTENT(OUT) :: SEP(:)
	END SUBROUTINE DDISNA_MKL95
END INTERFACE DISNA
INTERFACE SYGST
	SUBROUTINE SSYGST_MKL95(A,B,ITYPE,UPLO,INFO)
		! MKL Fortran77 call:
		! SSYGST(ITYPE,UPLO,N,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(IN) :: B(:,:)
	END SUBROUTINE SSYGST_MKL95
	SUBROUTINE DSYGST_MKL95(A,B,ITYPE,UPLO,INFO)
		! MKL Fortran77 call:
		! DSYGST(ITYPE,UPLO,N,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(IN) :: B(:,:)
	END SUBROUTINE DSYGST_MKL95
END INTERFACE SYGST
INTERFACE HEGST
	SUBROUTINE CHEGST_MKL95(A,B,ITYPE,UPLO,INFO)
		! MKL Fortran77 call:
		! CHEGST(ITYPE,UPLO,N,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
	END SUBROUTINE CHEGST_MKL95
	SUBROUTINE ZHEGST_MKL95(A,B,ITYPE,UPLO,INFO)
		! MKL Fortran77 call:
		! ZHEGST(ITYPE,UPLO,N,A,LDA,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
	END SUBROUTINE ZHEGST_MKL95
END INTERFACE HEGST
INTERFACE SPGST
	SUBROUTINE SSPGST_MKL95(A,B,ITYPE,UPLO,INFO)
		! MKL Fortran77 call:
		! SSPGST(ITYPE,UPLO,N,A,B,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(IN) :: B(:)
	END SUBROUTINE SSPGST_MKL95
	SUBROUTINE DSPGST_MKL95(A,B,ITYPE,UPLO,INFO)
		! MKL Fortran77 call:
		! DSPGST(ITYPE,UPLO,N,A,B,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(IN) :: B(:)
	END SUBROUTINE DSPGST_MKL95
END INTERFACE SPGST
INTERFACE HPGST
	SUBROUTINE CHPGST_MKL95(A,B,ITYPE,UPLO,INFO)
		! MKL Fortran77 call:
		! CHPGST(ITYPE,UPLO,N,A,B,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
	END SUBROUTINE CHPGST_MKL95
	SUBROUTINE ZHPGST_MKL95(A,B,ITYPE,UPLO,INFO)
		! MKL Fortran77 call:
		! ZHPGST(ITYPE,UPLO,N,A,B,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		COMPLEX(WP), INTENT(IN) :: B(:)
	END SUBROUTINE ZHPGST_MKL95
END INTERFACE HPGST
INTERFACE SBGST
	SUBROUTINE SSBGST_MKL95(A,B,X,UPLO,INFO)
		! MKL Fortran77 call:
		! SSBGST(VECT,UPLO,N,KA,KB,A,LDA,B,LDB,X,LDX,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: X(:,:)
	END SUBROUTINE SSBGST_MKL95
	SUBROUTINE DSBGST_MKL95(A,B,X,UPLO,INFO)
		! MKL Fortran77 call:
		! DSBGST(VECT,UPLO,N,KA,KB,A,LDA,B,LDB,X,LDX,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: X(:,:)
	END SUBROUTINE DSBGST_MKL95
END INTERFACE SBGST
INTERFACE HBGST
	SUBROUTINE CHBGST_MKL95(A,B,X,UPLO,INFO)
		! MKL Fortran77 call:
		! CHBGST(VECT,UPLO,N,KA,KB,A,LDA,B,LDB,X,LDX,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: X(:,:)
	END SUBROUTINE CHBGST_MKL95
	SUBROUTINE ZHBGST_MKL95(A,B,X,UPLO,INFO)
		! MKL Fortran77 call:
		! ZHBGST(VECT,UPLO,N,KA,KB,A,LDA,B,LDB,X,LDX,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: X(:,:)
	END SUBROUTINE ZHBGST_MKL95
END INTERFACE HBGST
INTERFACE PBSTF
	SUBROUTINE SPBSTF_MKL95(B,UPLO,INFO)
		! MKL Fortran77 call:
		! SPBSTF(UPLO,N,KB,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE SPBSTF_MKL95
	SUBROUTINE DPBSTF_MKL95(B,UPLO,INFO)
		! MKL Fortran77 call:
		! DPBSTF(UPLO,N,KB,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE DPBSTF_MKL95
	SUBROUTINE CPBSTF_MKL95(B,UPLO,INFO)
		! MKL Fortran77 call:
		! CPBSTF(UPLO,N,KB,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE CPBSTF_MKL95
	SUBROUTINE ZPBSTF_MKL95(B,UPLO,INFO)
		! MKL Fortran77 call:
		! ZPBSTF(UPLO,N,KB,B,LDB,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE ZPBSTF_MKL95
END INTERFACE PBSTF
INTERFACE GEHRD
	SUBROUTINE SGEHRD_MKL95(A,TAU,ILO,IHI,INFO)
		! MKL Fortran77 call:
		! SGEHRD(N,ILO,IHI,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE SGEHRD_MKL95
	SUBROUTINE DGEHRD_MKL95(A,TAU,ILO,IHI,INFO)
		! MKL Fortran77 call:
		! DGEHRD(N,ILO,IHI,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE DGEHRD_MKL95
	SUBROUTINE CGEHRD_MKL95(A,TAU,ILO,IHI,INFO)
		! MKL Fortran77 call:
		! CGEHRD(N,ILO,IHI,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE CGEHRD_MKL95
	SUBROUTINE ZGEHRD_MKL95(A,TAU,ILO,IHI,INFO)
		! MKL Fortran77 call:
		! ZGEHRD(N,ILO,IHI,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: TAU(:)
	END SUBROUTINE ZGEHRD_MKL95
END INTERFACE GEHRD
INTERFACE ORGHR
	SUBROUTINE SORGHR_MKL95(A,TAU,ILO,IHI,INFO)
		! MKL Fortran77 call:
		! SORGHR(N,ILO,IHI,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
	END SUBROUTINE SORGHR_MKL95
	SUBROUTINE DORGHR_MKL95(A,TAU,ILO,IHI,INFO)
		! MKL Fortran77 call:
		! DORGHR(N,ILO,IHI,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
	END SUBROUTINE DORGHR_MKL95
END INTERFACE ORGHR
INTERFACE ORMHR
	SUBROUTINE SORMHR_MKL95(A,TAU,C,ILO,IHI,SIDE,TRANS,INFO)
		! MKL Fortran77 call:
		! SORMHR(SIDE,TRANS,M,N,ILO,IHI,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
		REAL(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE SORMHR_MKL95
	SUBROUTINE DORMHR_MKL95(A,TAU,C,ILO,IHI,SIDE,TRANS,INFO)
		! MKL Fortran77 call:
		! DORMHR(SIDE,TRANS,M,N,ILO,IHI,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: TAU(:)
		REAL(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE DORMHR_MKL95
END INTERFACE ORMHR
INTERFACE UNGHR
	SUBROUTINE CUNGHR_MKL95(A,TAU,ILO,IHI,INFO)
		! MKL Fortran77 call:
		! CUNGHR(N,ILO,IHI,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
	END SUBROUTINE CUNGHR_MKL95
	SUBROUTINE ZUNGHR_MKL95(A,TAU,ILO,IHI,INFO)
		! MKL Fortran77 call:
		! ZUNGHR(N,ILO,IHI,A,LDA,TAU,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
	END SUBROUTINE ZUNGHR_MKL95
END INTERFACE UNGHR
INTERFACE UNMHR
	SUBROUTINE CUNMHR_MKL95(A,TAU,C,ILO,IHI,SIDE,TRANS,INFO)
		! MKL Fortran77 call:
		! CUNMHR(SIDE,TRANS,M,N,ILO,IHI,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
		COMPLEX(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE CUNMHR_MKL95
	SUBROUTINE ZUNMHR_MKL95(A,TAU,C,ILO,IHI,SIDE,TRANS,INFO)
		! MKL Fortran77 call:
		! ZUNMHR(SIDE,TRANS,M,N,ILO,IHI,A,LDA,TAU,C,LDC,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: TAU(:)
		COMPLEX(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE ZUNMHR_MKL95
END INTERFACE UNMHR
INTERFACE GEBAL
	SUBROUTINE SGEBAL_MKL95(A,SCALE,ILO,IHI,JOB,INFO)
		! MKL Fortran77 call:
		! SGEBAL(JOB,N,A,LDA,ILO,IHI,SCALE,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: ILO
		INTEGER, INTENT(OUT), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: SCALE(:)
	END SUBROUTINE SGEBAL_MKL95
	SUBROUTINE DGEBAL_MKL95(A,SCALE,ILO,IHI,JOB,INFO)
		! MKL Fortran77 call:
		! DGEBAL(JOB,N,A,LDA,ILO,IHI,SCALE,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: ILO
		INTEGER, INTENT(OUT), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: SCALE(:)
	END SUBROUTINE DGEBAL_MKL95
	SUBROUTINE CGEBAL_MKL95(A,SCALE,ILO,IHI,JOB,INFO)
		! MKL Fortran77 call:
		! CGEBAL(JOB,N,A,LDA,ILO,IHI,SCALE,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: ILO
		INTEGER, INTENT(OUT), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: SCALE(:)
	END SUBROUTINE CGEBAL_MKL95
	SUBROUTINE ZGEBAL_MKL95(A,SCALE,ILO,IHI,JOB,INFO)
		! MKL Fortran77 call:
		! ZGEBAL(JOB,N,A,LDA,ILO,IHI,SCALE,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: ILO
		INTEGER, INTENT(OUT), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: SCALE(:)
	END SUBROUTINE ZGEBAL_MKL95
END INTERFACE GEBAL
INTERFACE GEBAK
	SUBROUTINE SGEBAK_MKL95(V,SCALE,ILO,IHI,JOB,SIDE,INFO)
		! MKL Fortran77 call:
		! SGEBAK(JOB,SIDE,N,ILO,IHI,SCALE,M,V,LDV,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: V(:,:)
		REAL(WP), INTENT(IN) :: SCALE(:)
	END SUBROUTINE SGEBAK_MKL95
	SUBROUTINE DGEBAK_MKL95(V,SCALE,ILO,IHI,JOB,SIDE,INFO)
		! MKL Fortran77 call:
		! DGEBAK(JOB,SIDE,N,ILO,IHI,SCALE,M,V,LDV,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: V(:,:)
		REAL(WP), INTENT(IN) :: SCALE(:)
	END SUBROUTINE DGEBAK_MKL95
	SUBROUTINE CGEBAK_MKL95(V,SCALE,ILO,IHI,JOB,SIDE,INFO)
		! MKL Fortran77 call:
		! CGEBAK(JOB,SIDE,N,ILO,IHI,SCALE,M,V,LDV,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: V(:,:)
		REAL(WP), INTENT(IN) :: SCALE(:)
	END SUBROUTINE CGEBAK_MKL95
	SUBROUTINE ZGEBAK_MKL95(V,SCALE,ILO,IHI,JOB,SIDE,INFO)
		! MKL Fortran77 call:
		! ZGEBAK(JOB,SIDE,N,ILO,IHI,SCALE,M,V,LDV,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: SIDE
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: V(:,:)
		REAL(WP), INTENT(IN) :: SCALE(:)
	END SUBROUTINE ZGEBAK_MKL95
END INTERFACE GEBAK
INTERFACE HSEQR
	SUBROUTINE SHSEQR_MKL95(H,WR,WI,ILO,IHI,Z,JOB,COMPZ,INFO)
		! MKL Fortran77 call:
		! SHSEQR(JOB,COMPZ,N,ILO,IHI,H,LDH,WR,WI,Z,LDZ,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPZ
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: H(:,:)
		REAL(WP), INTENT(OUT) :: WR(:)
		REAL(WP), INTENT(OUT) :: WI(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE SHSEQR_MKL95
	SUBROUTINE DHSEQR_MKL95(H,WR,WI,ILO,IHI,Z,JOB,COMPZ,INFO)
		! MKL Fortran77 call:
		! DHSEQR(JOB,COMPZ,N,ILO,IHI,H,LDH,WR,WI,Z,LDZ,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPZ
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: H(:,:)
		REAL(WP), INTENT(OUT) :: WR(:)
		REAL(WP), INTENT(OUT) :: WI(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE DHSEQR_MKL95
	SUBROUTINE CHSEQR_MKL95(H,W,ILO,IHI,Z,JOB,COMPZ,INFO)
		! MKL Fortran77 call:
		! CHSEQR(JOB,COMPZ,N,ILO,IHI,H,LDH,W,Z,LDZ,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPZ
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: H(:,:)
		COMPLEX(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE CHSEQR_MKL95
	SUBROUTINE ZHSEQR_MKL95(H,W,ILO,IHI,Z,JOB,COMPZ,INFO)
		! MKL Fortran77 call:
		! ZHSEQR(JOB,COMPZ,N,ILO,IHI,H,LDH,W,Z,LDZ,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPZ
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: H(:,:)
		COMPLEX(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE ZHSEQR_MKL95
END INTERFACE HSEQR
INTERFACE HSEIN
	SUBROUTINE SHSEIN_MKL95(H,WR,WI,SELECT,VL,VR,IFAILL,IFAILR,INITV,EIGSRC,M,INFO)
		! MKL Fortran77 call:
		! SHSEIN(JOB,EIGSRC,INITV,SELECT,N,H,LDH,WR,WI,VL,LDVL,VR,LDVR,MM,M,WORK,IFAILL,IFAILR,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: INITV
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: EIGSRC
		INTEGER, INTENT(OUT), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: H(:,:)
		REAL(WP), INTENT(INOUT) :: WR(:)
		REAL(WP), INTENT(IN) :: WI(:)
		LOGICAL, INTENT(INOUT) :: SELECT(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: VL(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: VR(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAILL(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAILR(:)
	END SUBROUTINE SHSEIN_MKL95
	SUBROUTINE DHSEIN_MKL95(H,WR,WI,SELECT,VL,VR,IFAILL,IFAILR,INITV,EIGSRC,M,INFO)
		! MKL Fortran77 call:
		! DHSEIN(JOB,EIGSRC,INITV,SELECT,N,H,LDH,WR,WI,VL,LDVL,VR,LDVR,MM,M,WORK,IFAILL,IFAILR,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: INITV
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: EIGSRC
		INTEGER, INTENT(OUT), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: H(:,:)
		REAL(WP), INTENT(INOUT) :: WR(:)
		REAL(WP), INTENT(IN) :: WI(:)
		LOGICAL, INTENT(INOUT) :: SELECT(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: VL(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: VR(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAILL(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAILR(:)
	END SUBROUTINE DHSEIN_MKL95
	SUBROUTINE CHSEIN_MKL95(H,W,SELECT,VL,VR,IFAILL,IFAILR,INITV,EIGSRC,M,INFO)
		! MKL Fortran77 call:
		! CHSEIN(JOB,EIGSRC,INITV,SELECT,N,H,LDH,W,VL,LDVL,VR,LDVR,MM,M,WORK,RWORK,IFAILL,IFAILR,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: INITV
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: EIGSRC
		INTEGER, INTENT(OUT), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: H(:,:)
		COMPLEX(WP), INTENT(INOUT) :: W(:)
		LOGICAL, INTENT(IN) :: SELECT(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: VL(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: VR(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAILL(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAILR(:)
	END SUBROUTINE CHSEIN_MKL95
	SUBROUTINE ZHSEIN_MKL95(H,W,SELECT,VL,VR,IFAILL,IFAILR,INITV,EIGSRC,M,INFO)
		! MKL Fortran77 call:
		! ZHSEIN(JOB,EIGSRC,INITV,SELECT,N,H,LDH,W,VL,LDVL,VR,LDVR,MM,M,WORK,RWORK,IFAILL,IFAILR,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: INITV
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: EIGSRC
		INTEGER, INTENT(OUT), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: H(:,:)
		COMPLEX(WP), INTENT(INOUT) :: W(:)
		LOGICAL, INTENT(IN) :: SELECT(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: VL(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: VR(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAILL(:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAILR(:)
	END SUBROUTINE ZHSEIN_MKL95
END INTERFACE HSEIN
INTERFACE TREVC
	SUBROUTINE STREVC_MKL95(T,HOWMNY,SELECT,VL,VR,M,INFO)
		! MKL Fortran77 call:
		! STREVC(SIDE,HOWMNY,SELECT,N,T,LDT,VL,LDVL,VR,LDVR,MM,M,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: HOWMNY
		INTEGER, INTENT(OUT), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: T(:,:)
		LOGICAL, INTENT(INOUT), OPTIONAL, TARGET :: SELECT(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: VL(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: VR(:,:)
	END SUBROUTINE STREVC_MKL95
	SUBROUTINE DTREVC_MKL95(T,HOWMNY,SELECT,VL,VR,M,INFO)
		! MKL Fortran77 call:
		! DTREVC(SIDE,HOWMNY,SELECT,N,T,LDT,VL,LDVL,VR,LDVR,MM,M,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: HOWMNY
		INTEGER, INTENT(OUT), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: T(:,:)
		LOGICAL, INTENT(INOUT), OPTIONAL, TARGET :: SELECT(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: VL(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: VR(:,:)
	END SUBROUTINE DTREVC_MKL95
	SUBROUTINE CTREVC_MKL95(T,HOWMNY,SELECT,VL,VR,M,INFO)
		! MKL Fortran77 call:
		! CTREVC(SIDE,HOWMNY,SELECT,N,T,LDT,VL,LDVL,VR,LDVR,MM,M,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: HOWMNY
		INTEGER, INTENT(OUT), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: T(:,:)
		LOGICAL, INTENT(IN), OPTIONAL, TARGET :: SELECT(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: VL(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: VR(:,:)
	END SUBROUTINE CTREVC_MKL95
	SUBROUTINE ZTREVC_MKL95(T,HOWMNY,SELECT,VL,VR,M,INFO)
		! MKL Fortran77 call:
		! ZTREVC(SIDE,HOWMNY,SELECT,N,T,LDT,VL,LDVL,VR,LDVR,MM,M,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: HOWMNY
		INTEGER, INTENT(OUT), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: T(:,:)
		LOGICAL, INTENT(IN), OPTIONAL, TARGET :: SELECT(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: VL(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: VR(:,:)
	END SUBROUTINE ZTREVC_MKL95
END INTERFACE TREVC
INTERFACE TRSNA
	SUBROUTINE STRSNA_MKL95(T,S,SEP,VL,VR,SELECT,M,INFO)
		! MKL Fortran77 call:
		! STRSNA(JOB,HOWMNY,SELECT,N,T,LDT,VL,LDVL,VR,LDVR,S,SEP,MM,M,WORK,LDWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: T(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: S(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: SEP(:)
		REAL(WP), INTENT(IN), OPTIONAL, TARGET :: VL(:,:)
		REAL(WP), INTENT(IN), OPTIONAL, TARGET :: VR(:,:)
		LOGICAL, INTENT(IN), OPTIONAL, TARGET :: SELECT(:)
	END SUBROUTINE STRSNA_MKL95
	SUBROUTINE DTRSNA_MKL95(T,S,SEP,VL,VR,SELECT,M,INFO)
		! MKL Fortran77 call:
		! DTRSNA(JOB,HOWMNY,SELECT,N,T,LDT,VL,LDVL,VR,LDVR,S,SEP,MM,M,WORK,LDWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: T(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: S(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: SEP(:)
		REAL(WP), INTENT(IN), OPTIONAL, TARGET :: VL(:,:)
		REAL(WP), INTENT(IN), OPTIONAL, TARGET :: VR(:,:)
		LOGICAL, INTENT(IN), OPTIONAL, TARGET :: SELECT(:)
	END SUBROUTINE DTRSNA_MKL95
	SUBROUTINE CTRSNA_MKL95(T,S,SEP,VL,VR,SELECT,M,INFO)
		! MKL Fortran77 call:
		! CTRSNA(JOB,HOWMNY,SELECT,N,T,LDT,VL,LDVL,VR,LDVR,S,SEP,MM,M,WORK,LDWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: T(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: S(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: SEP(:)
		COMPLEX(WP), INTENT(IN), OPTIONAL, TARGET :: VL(:,:)
		COMPLEX(WP), INTENT(IN), OPTIONAL, TARGET :: VR(:,:)
		LOGICAL, INTENT(IN), OPTIONAL, TARGET :: SELECT(:)
	END SUBROUTINE CTRSNA_MKL95
	SUBROUTINE ZTRSNA_MKL95(T,S,SEP,VL,VR,SELECT,M,INFO)
		! MKL Fortran77 call:
		! ZTRSNA(JOB,HOWMNY,SELECT,N,T,LDT,VL,LDVL,VR,LDVR,S,SEP,MM,M,WORK,LDWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: T(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: S(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: SEP(:)
		COMPLEX(WP), INTENT(IN), OPTIONAL, TARGET :: VL(:,:)
		COMPLEX(WP), INTENT(IN), OPTIONAL, TARGET :: VR(:,:)
		LOGICAL, INTENT(IN), OPTIONAL, TARGET :: SELECT(:)
	END SUBROUTINE ZTRSNA_MKL95
END INTERFACE TRSNA
INTERFACE TREXC
	SUBROUTINE STREXC_MKL95(T,IFST,ILST,Q,INFO)
		! MKL Fortran77 call:
		! STREXC(COMPQ,N,T,LDT,Q,LDQ,IFST,ILST,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(INOUT) :: IFST
		INTEGER, INTENT(INOUT) :: ILST
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: T(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
	END SUBROUTINE STREXC_MKL95
	SUBROUTINE DTREXC_MKL95(T,IFST,ILST,Q,INFO)
		! MKL Fortran77 call:
		! DTREXC(COMPQ,N,T,LDT,Q,LDQ,IFST,ILST,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(INOUT) :: IFST
		INTEGER, INTENT(INOUT) :: ILST
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: T(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
	END SUBROUTINE DTREXC_MKL95
	SUBROUTINE CTREXC_MKL95(T,IFST,ILST,Q,INFO)
		! MKL Fortran77 call:
		! CTREXC(COMPQ,N,T,LDT,Q,LDQ,IFST,ILST,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN) :: IFST
		INTEGER, INTENT(IN) :: ILST
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: T(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
	END SUBROUTINE CTREXC_MKL95
	SUBROUTINE ZTREXC_MKL95(T,IFST,ILST,Q,INFO)
		! MKL Fortran77 call:
		! ZTREXC(COMPQ,N,T,LDT,Q,LDQ,IFST,ILST,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN) :: IFST
		INTEGER, INTENT(IN) :: ILST
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: T(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
	END SUBROUTINE ZTREXC_MKL95
END INTERFACE TREXC
INTERFACE TRSEN
	SUBROUTINE STRSEN_MKL95(T,SELECT,WR,WI,M,S,SEP,Q,INFO)
		! MKL Fortran77 call:
		! STRSEN(JOB,COMPQ,SELECT,N,T,LDT,Q,LDQ,WR,WI,M,S,SEP,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(OUT), OPTIONAL :: S
		REAL(WP), INTENT(OUT), OPTIONAL :: SEP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: T(:,:)
		LOGICAL, INTENT(IN) :: SELECT(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: WR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: WI(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
	END SUBROUTINE STRSEN_MKL95
	SUBROUTINE DTRSEN_MKL95(T,SELECT,WR,WI,M,S,SEP,Q,INFO)
		! MKL Fortran77 call:
		! DTRSEN(JOB,COMPQ,SELECT,N,T,LDT,Q,LDQ,WR,WI,M,S,SEP,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(OUT), OPTIONAL :: S
		REAL(WP), INTENT(OUT), OPTIONAL :: SEP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: T(:,:)
		LOGICAL, INTENT(IN) :: SELECT(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: WR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: WI(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
	END SUBROUTINE DTRSEN_MKL95
	SUBROUTINE CTRSEN_MKL95(T,SELECT,W,M,S,SEP,Q,INFO)
		! MKL Fortran77 call:
		! CTRSEN(JOB,COMPQ,SELECT,N,T,LDT,Q,LDQ,W,M,S,SEP,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(OUT), OPTIONAL :: S
		REAL(WP), INTENT(OUT), OPTIONAL :: SEP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: T(:,:)
		LOGICAL, INTENT(IN) :: SELECT(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: W(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
	END SUBROUTINE CTRSEN_MKL95
	SUBROUTINE ZTRSEN_MKL95(T,SELECT,W,M,S,SEP,Q,INFO)
		! MKL Fortran77 call:
		! ZTRSEN(JOB,COMPQ,SELECT,N,T,LDT,Q,LDQ,W,M,S,SEP,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(OUT), OPTIONAL :: S
		REAL(WP), INTENT(OUT), OPTIONAL :: SEP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: T(:,:)
		LOGICAL, INTENT(IN) :: SELECT(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: W(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
	END SUBROUTINE ZTRSEN_MKL95
END INTERFACE TRSEN
INTERFACE TRSYL
	SUBROUTINE STRSYL_MKL95(A,B,C,SCALE,TRANA,TRANB,ISGN,INFO)
		! MKL Fortran77 call:
		! STRSYL(TRANA,TRANB,ISGN,M,N,A,LDA,B,LDB,C,LDC,SCALE,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(OUT) :: SCALE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANA
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANB
		INTEGER, INTENT(IN), OPTIONAL :: ISGN
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE STRSYL_MKL95
	SUBROUTINE DTRSYL_MKL95(A,B,C,SCALE,TRANA,TRANB,ISGN,INFO)
		! MKL Fortran77 call:
		! DTRSYL(TRANA,TRANB,ISGN,M,N,A,LDA,B,LDB,C,LDC,SCALE,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(OUT) :: SCALE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANA
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANB
		INTEGER, INTENT(IN), OPTIONAL :: ISGN
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE DTRSYL_MKL95
	SUBROUTINE CTRSYL_MKL95(A,B,C,SCALE,TRANA,TRANB,ISGN,INFO)
		! MKL Fortran77 call:
		! CTRSYL(TRANA,TRANB,ISGN,M,N,A,LDA,B,LDB,C,LDC,SCALE,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(OUT) :: SCALE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANA
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANB
		INTEGER, INTENT(IN), OPTIONAL :: ISGN
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE CTRSYL_MKL95
	SUBROUTINE ZTRSYL_MKL95(A,B,C,SCALE,TRANA,TRANB,ISGN,INFO)
		! MKL Fortran77 call:
		! ZTRSYL(TRANA,TRANB,ISGN,M,N,A,LDA,B,LDB,C,LDC,SCALE,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(OUT) :: SCALE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANA
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANB
		INTEGER, INTENT(IN), OPTIONAL :: ISGN
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: C(:,:)
	END SUBROUTINE ZTRSYL_MKL95
END INTERFACE TRSYL
INTERFACE GGHRD
	SUBROUTINE SGGHRD_MKL95(A,B,ILO,IHI,Q,Z,COMPQ,COMPZ,INFO)
		! MKL Fortran77 call:
		! SGGHRD(COMPQ,COMPZ,N,ILO,IHI,A,LDA,B,LDB,Q,LDQ,Z,LDZ,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPQ
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPZ
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE SGGHRD_MKL95
	SUBROUTINE DGGHRD_MKL95(A,B,ILO,IHI,Q,Z,COMPQ,COMPZ,INFO)
		! MKL Fortran77 call:
		! DGGHRD(COMPQ,COMPZ,N,ILO,IHI,A,LDA,B,LDB,Q,LDQ,Z,LDZ,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPQ
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPZ
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE DGGHRD_MKL95
	SUBROUTINE CGGHRD_MKL95(A,B,ILO,IHI,Q,Z,COMPQ,COMPZ,INFO)
		! MKL Fortran77 call:
		! CGGHRD(COMPQ,COMPZ,N,ILO,IHI,A,LDA,B,LDB,Q,LDQ,Z,LDZ,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPQ
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPZ
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE CGGHRD_MKL95
	SUBROUTINE ZGGHRD_MKL95(A,B,ILO,IHI,Q,Z,COMPQ,COMPZ,INFO)
		! MKL Fortran77 call:
		! ZGGHRD(COMPQ,COMPZ,N,ILO,IHI,A,LDA,B,LDB,Q,LDQ,Z,LDZ,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPQ
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPZ
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE ZGGHRD_MKL95
END INTERFACE GGHRD
INTERFACE GGBAL
	SUBROUTINE SGGBAL_MKL95(A,B,ILO,IHI,LSCALE,RSCALE,JOB,INFO)
		! MKL Fortran77 call:
		! SGGBAL(JOB,N,A,LDA,B,LDB,ILO,IHI,LSCALE,RSCALE,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: ILO
		INTEGER, INTENT(OUT), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: LSCALE(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RSCALE(:)
	END SUBROUTINE SGGBAL_MKL95
	SUBROUTINE DGGBAL_MKL95(A,B,ILO,IHI,LSCALE,RSCALE,JOB,INFO)
		! MKL Fortran77 call:
		! DGGBAL(JOB,N,A,LDA,B,LDB,ILO,IHI,LSCALE,RSCALE,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: ILO
		INTEGER, INTENT(OUT), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: LSCALE(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RSCALE(:)
	END SUBROUTINE DGGBAL_MKL95
	SUBROUTINE CGGBAL_MKL95(A,B,ILO,IHI,LSCALE,RSCALE,JOB,INFO)
		! MKL Fortran77 call:
		! CGGBAL(JOB,N,A,LDA,B,LDB,ILO,IHI,LSCALE,RSCALE,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: ILO
		INTEGER, INTENT(OUT), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: LSCALE(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RSCALE(:)
	END SUBROUTINE CGGBAL_MKL95
	SUBROUTINE ZGGBAL_MKL95(A,B,ILO,IHI,LSCALE,RSCALE,JOB,INFO)
		! MKL Fortran77 call:
		! ZGGBAL(JOB,N,A,LDA,B,LDB,ILO,IHI,LSCALE,RSCALE,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: ILO
		INTEGER, INTENT(OUT), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: LSCALE(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RSCALE(:)
	END SUBROUTINE ZGGBAL_MKL95
END INTERFACE GGBAL
INTERFACE GGBAK
	SUBROUTINE SGGBAK_MKL95(V,ILO,IHI,LSCALE,RSCALE,JOB,INFO)
		! MKL Fortran77 call:
		! SGGBAK(JOB,SIDE,N,ILO,IHI,LSCALE,RSCALE,M,V,LDV,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: V(:,:)
		REAL(WP), INTENT(IN), OPTIONAL, TARGET :: LSCALE(:)
		REAL(WP), INTENT(IN), OPTIONAL, TARGET :: RSCALE(:)
	END SUBROUTINE SGGBAK_MKL95
	SUBROUTINE DGGBAK_MKL95(V,ILO,IHI,LSCALE,RSCALE,JOB,INFO)
		! MKL Fortran77 call:
		! DGGBAK(JOB,SIDE,N,ILO,IHI,LSCALE,RSCALE,M,V,LDV,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: V(:,:)
		REAL(WP), INTENT(IN), OPTIONAL, TARGET :: LSCALE(:)
		REAL(WP), INTENT(IN), OPTIONAL, TARGET :: RSCALE(:)
	END SUBROUTINE DGGBAK_MKL95
	SUBROUTINE CGGBAK_MKL95(V,ILO,IHI,LSCALE,RSCALE,JOB,INFO)
		! MKL Fortran77 call:
		! CGGBAK(JOB,SIDE,N,ILO,IHI,LSCALE,RSCALE,M,V,LDV,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: V(:,:)
		REAL(WP), INTENT(IN), OPTIONAL, TARGET :: LSCALE(:)
		REAL(WP), INTENT(IN), OPTIONAL, TARGET :: RSCALE(:)
	END SUBROUTINE CGGBAK_MKL95
	SUBROUTINE ZGGBAK_MKL95(V,ILO,IHI,LSCALE,RSCALE,JOB,INFO)
		! MKL Fortran77 call:
		! ZGGBAK(JOB,SIDE,N,ILO,IHI,LSCALE,RSCALE,M,V,LDV,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: V(:,:)
		REAL(WP), INTENT(IN), OPTIONAL, TARGET :: LSCALE(:)
		REAL(WP), INTENT(IN), OPTIONAL, TARGET :: RSCALE(:)
	END SUBROUTINE ZGGBAK_MKL95
END INTERFACE GGBAK
INTERFACE HGEQZ
	SUBROUTINE SHGEQZ_MKL95(H,T,ILO,IHI,ALPHAR,ALPHAI,BETA,Q,Z,JOB,COMPQ,COMPZ,INFO)
		! MKL Fortran77 call:
		! SHGEQZ(JOB,COMPQ,COMPZ,N,ILO,IHI,H,LDH,T,LDT,ALPHAR,ALPHAI,BETA,Q,LDQ,Z,LDZ,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPQ
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPZ
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: H(:,:)
		REAL(WP), INTENT(INOUT) :: T(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: ALPHAR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: ALPHAI(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BETA(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE SHGEQZ_MKL95
	SUBROUTINE DHGEQZ_MKL95(H,T,ILO,IHI,ALPHAR,ALPHAI,BETA,Q,Z,JOB,COMPQ,COMPZ,INFO)
		! MKL Fortran77 call:
		! DHGEQZ(JOB,COMPQ,COMPZ,N,ILO,IHI,H,LDH,T,LDT,ALPHAR,ALPHAI,BETA,Q,LDQ,Z,LDZ,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPQ
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPZ
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: H(:,:)
		REAL(WP), INTENT(INOUT) :: T(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: ALPHAR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: ALPHAI(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BETA(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE DHGEQZ_MKL95
	SUBROUTINE CHGEQZ_MKL95(H,T,ILO,IHI,ALPHA,BETA,Q,Z,JOB,COMPQ,COMPZ,INFO)
		! MKL Fortran77 call:
		! CHGEQZ(JOB,COMPQ,COMPZ,N,ILO,IHI,H,LDH,T,LDT,ALPHA,BETA,Q,LDQ,Z,LDZ,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPQ
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPZ
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: H(:,:)
		COMPLEX(WP), INTENT(INOUT) :: T(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: ALPHA(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: BETA(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE CHGEQZ_MKL95
	SUBROUTINE ZHGEQZ_MKL95(H,T,ILO,IHI,ALPHA,BETA,Q,Z,JOB,COMPQ,COMPZ,INFO)
		! MKL Fortran77 call:
		! ZHGEQZ(JOB,COMPQ,COMPZ,N,ILO,IHI,H,LDH,T,LDT,ALPHA,BETA,Q,LDQ,Z,LDZ,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ILO
		INTEGER, INTENT(IN), OPTIONAL :: IHI
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPQ
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: COMPZ
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: H(:,:)
		COMPLEX(WP), INTENT(INOUT) :: T(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: ALPHA(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: BETA(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE ZHGEQZ_MKL95
END INTERFACE HGEQZ
INTERFACE TGEVC
	SUBROUTINE STGEVC_MKL95(S,P,HOWMNY,SELECT,VL,VR,M,INFO)
		! MKL Fortran77 call:
		! STGEVC(SIDE,HOWMNY,SELECT,N,S,LDS,P,LDP,VL,LDVL,VR,LDVR,MM,M,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: HOWMNY
		INTEGER, INTENT(OUT), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: S(:,:)
		REAL(WP), INTENT(IN) :: P(:,:)
		LOGICAL, INTENT(IN), OPTIONAL, TARGET :: SELECT(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: VL(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: VR(:,:)
	END SUBROUTINE STGEVC_MKL95
	SUBROUTINE DTGEVC_MKL95(S,P,HOWMNY,SELECT,VL,VR,M,INFO)
		! MKL Fortran77 call:
		! DTGEVC(SIDE,HOWMNY,SELECT,N,S,LDS,P,LDP,VL,LDVL,VR,LDVR,MM,M,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: HOWMNY
		INTEGER, INTENT(OUT), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: S(:,:)
		REAL(WP), INTENT(IN) :: P(:,:)
		LOGICAL, INTENT(IN), OPTIONAL, TARGET :: SELECT(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: VL(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: VR(:,:)
	END SUBROUTINE DTGEVC_MKL95
	SUBROUTINE CTGEVC_MKL95(S,P,HOWMNY,SELECT,VL,VR,M,INFO)
		! MKL Fortran77 call:
		! CTGEVC(SIDE,HOWMNY,SELECT,N,S,LDS,P,LDP,VL,LDVL,VR,LDVR,MM,M,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: HOWMNY
		INTEGER, INTENT(OUT), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: S(:,:)
		COMPLEX(WP), INTENT(IN) :: P(:,:)
		LOGICAL, INTENT(IN), OPTIONAL, TARGET :: SELECT(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: VL(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: VR(:,:)
	END SUBROUTINE CTGEVC_MKL95
	SUBROUTINE ZTGEVC_MKL95(S,P,HOWMNY,SELECT,VL,VR,M,INFO)
		! MKL Fortran77 call:
		! ZTGEVC(SIDE,HOWMNY,SELECT,N,S,LDS,P,LDP,VL,LDVL,VR,LDVR,MM,M,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: HOWMNY
		INTEGER, INTENT(OUT), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: S(:,:)
		COMPLEX(WP), INTENT(IN) :: P(:,:)
		LOGICAL, INTENT(IN), OPTIONAL, TARGET :: SELECT(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: VL(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: VR(:,:)
	END SUBROUTINE ZTGEVC_MKL95
END INTERFACE TGEVC
INTERFACE TGEXC
	SUBROUTINE STGEXC_MKL95(A,B,IFST,ILST,Z,Q,INFO)
		! MKL Fortran77 call:
		! STGEXC(WANTQ,WANTZ,N,A,LDA,B,LDB,Q,LDQ,Z,LDZ,IFST,ILST,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(INOUT), OPTIONAL :: IFST
		INTEGER, INTENT(INOUT), OPTIONAL :: ILST
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
	END SUBROUTINE STGEXC_MKL95
	SUBROUTINE DTGEXC_MKL95(A,B,IFST,ILST,Z,Q,INFO)
		! MKL Fortran77 call:
		! DTGEXC(WANTQ,WANTZ,N,A,LDA,B,LDB,Q,LDQ,Z,LDZ,IFST,ILST,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(INOUT), OPTIONAL :: IFST
		INTEGER, INTENT(INOUT), OPTIONAL :: ILST
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
	END SUBROUTINE DTGEXC_MKL95
	SUBROUTINE CTGEXC_MKL95(A,B,IFST,ILST,Z,Q,INFO)
		! MKL Fortran77 call:
		! CTGEXC(WANTQ,WANTZ,N,A,LDA,B,LDB,Q,LDQ,Z,LDZ,IFST,ILST,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(INOUT), OPTIONAL :: IFST
		INTEGER, INTENT(INOUT), OPTIONAL :: ILST
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
	END SUBROUTINE CTGEXC_MKL95
	SUBROUTINE ZTGEXC_MKL95(A,B,IFST,ILST,Z,Q,INFO)
		! MKL Fortran77 call:
		! ZTGEXC(WANTQ,WANTZ,N,A,LDA,B,LDB,Q,LDQ,Z,LDZ,IFST,ILST,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(INOUT), OPTIONAL :: IFST
		INTEGER, INTENT(INOUT), OPTIONAL :: ILST
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
	END SUBROUTINE ZTGEXC_MKL95
END INTERFACE TGEXC
INTERFACE TGSEN
	SUBROUTINE STGSEN_MKL95(A,B,SELECT,ALPHAR,ALPHAI,BETA,IJOB,Q,Z,PL,PR,DIF,M,INFO)
		! MKL Fortran77 call:
		! STGSEN(IJOB,WANTQ,WANTZ,SELECT,N,A,LDA,B,LDB,ALPHAR,ALPHAI,BETA,Q,LDQ,Z,LDZ,M,PL,PR,DIF,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: IJOB
		REAL(WP), INTENT(OUT), OPTIONAL :: PL
		REAL(WP), INTENT(OUT), OPTIONAL :: PR
		INTEGER, INTENT(OUT), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		LOGICAL, INTENT(IN) :: SELECT(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: ALPHAR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: ALPHAI(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BETA(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: DIF(2)
	END SUBROUTINE STGSEN_MKL95
	SUBROUTINE DTGSEN_MKL95(A,B,SELECT,ALPHAR,ALPHAI,BETA,IJOB,Q,Z,PL,PR,DIF,M,INFO)
		! MKL Fortran77 call:
		! DTGSEN(IJOB,WANTQ,WANTZ,SELECT,N,A,LDA,B,LDB,ALPHAR,ALPHAI,BETA,Q,LDQ,Z,LDZ,M,PL,PR,DIF,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: IJOB
		REAL(WP), INTENT(OUT), OPTIONAL :: PL
		REAL(WP), INTENT(OUT), OPTIONAL :: PR
		INTEGER, INTENT(OUT), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		LOGICAL, INTENT(IN) :: SELECT(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: ALPHAR(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: ALPHAI(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BETA(:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: DIF(2)
	END SUBROUTINE DTGSEN_MKL95
	SUBROUTINE CTGSEN_MKL95(A,B,SELECT,ALPHA,BETA,IJOB,Q,Z,PL,PR,DIF,M,INFO)
		! MKL Fortran77 call:
		! CTGSEN(IJOB,WANTQ,WANTZ,SELECT,N,A,LDA,B,LDB,ALPHA,BETA,Q,LDQ,Z,LDZ,M,PL,PR,DIF,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: IJOB
		REAL(WP), INTENT(OUT), OPTIONAL :: PL
		REAL(WP), INTENT(OUT), OPTIONAL :: PR
		INTEGER, INTENT(OUT), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		LOGICAL, INTENT(IN) :: SELECT(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: ALPHA(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: BETA(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: DIF(2)
	END SUBROUTINE CTGSEN_MKL95
	SUBROUTINE ZTGSEN_MKL95(A,B,SELECT,ALPHA,BETA,IJOB,Q,Z,PL,PR,DIF,M,INFO)
		! MKL Fortran77 call:
		! ZTGSEN(IJOB,WANTQ,WANTZ,SELECT,N,A,LDA,B,LDB,ALPHA,BETA,Q,LDQ,Z,LDZ,M,PL,PR,DIF,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: IJOB
		REAL(WP), INTENT(OUT), OPTIONAL :: PL
		REAL(WP), INTENT(OUT), OPTIONAL :: PR
		INTEGER, INTENT(OUT), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		LOGICAL, INTENT(IN) :: SELECT(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: ALPHA(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: BETA(:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Z(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: DIF(2)
	END SUBROUTINE ZTGSEN_MKL95
END INTERFACE TGSEN
INTERFACE TGSYL
	SUBROUTINE STGSYL_MKL95(A,B,C,D,E,F,IJOB,TRANS,SCALE,DIF,INFO)
		! MKL Fortran77 call:
		! STGSYL(TRANS,IJOB,M,N,A,LDA,B,LDB,C,LDC,D,LDD,E,LDE,F,LDF,SCALE,DIF,WORK,LWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: IJOB
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		REAL(WP), INTENT(OUT), OPTIONAL :: SCALE
		REAL(WP), INTENT(OUT), OPTIONAL :: DIF
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(INOUT) :: C(:,:)
		REAL(WP), INTENT(IN) :: D(:,:)
		REAL(WP), INTENT(IN) :: E(:,:)
		REAL(WP), INTENT(INOUT) :: F(:,:)
	END SUBROUTINE STGSYL_MKL95
	SUBROUTINE DTGSYL_MKL95(A,B,C,D,E,F,IJOB,TRANS,SCALE,DIF,INFO)
		! MKL Fortran77 call:
		! DTGSYL(TRANS,IJOB,M,N,A,LDA,B,LDB,C,LDC,D,LDD,E,LDE,F,LDF,SCALE,DIF,WORK,LWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: IJOB
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		REAL(WP), INTENT(OUT), OPTIONAL :: SCALE
		REAL(WP), INTENT(OUT), OPTIONAL :: DIF
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(INOUT) :: C(:,:)
		REAL(WP), INTENT(IN) :: D(:,:)
		REAL(WP), INTENT(IN) :: E(:,:)
		REAL(WP), INTENT(INOUT) :: F(:,:)
	END SUBROUTINE DTGSYL_MKL95
	SUBROUTINE CTGSYL_MKL95(A,B,C,D,E,F,IJOB,TRANS,SCALE,DIF,INFO)
		! MKL Fortran77 call:
		! CTGSYL(TRANS,IJOB,M,N,A,LDA,B,LDB,C,LDC,D,LDD,E,LDE,F,LDF,SCALE,DIF,WORK,LWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: IJOB
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		REAL(WP), INTENT(OUT), OPTIONAL :: SCALE
		REAL(WP), INTENT(OUT), OPTIONAL :: DIF
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: C(:,:)
		COMPLEX(WP), INTENT(IN) :: D(:,:)
		COMPLEX(WP), INTENT(IN) :: E(:,:)
		COMPLEX(WP), INTENT(INOUT) :: F(:,:)
	END SUBROUTINE CTGSYL_MKL95
	SUBROUTINE ZTGSYL_MKL95(A,B,C,D,E,F,IJOB,TRANS,SCALE,DIF,INFO)
		! MKL Fortran77 call:
		! ZTGSYL(TRANS,IJOB,M,N,A,LDA,B,LDB,C,LDC,D,LDD,E,LDE,F,LDF,SCALE,DIF,WORK,LWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: IJOB
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		REAL(WP), INTENT(OUT), OPTIONAL :: SCALE
		REAL(WP), INTENT(OUT), OPTIONAL :: DIF
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: C(:,:)
		COMPLEX(WP), INTENT(IN) :: D(:,:)
		COMPLEX(WP), INTENT(IN) :: E(:,:)
		COMPLEX(WP), INTENT(INOUT) :: F(:,:)
	END SUBROUTINE ZTGSYL_MKL95
END INTERFACE TGSYL
INTERFACE TGSNA
	SUBROUTINE STGSNA_MKL95(A,B,S,DIF,VL,VR,SELECT,M,INFO)
		! MKL Fortran77 call:
		! STGSNA(JOB,HOWMNY,SELECT,N,A,LDA,B,LDB,VL,LDVL,VR,LDVR,S,DIF,MM,M,WORK,LWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: S(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: DIF(:)
		REAL(WP), INTENT(IN), OPTIONAL, TARGET :: VL(:,:)
		REAL(WP), INTENT(IN), OPTIONAL, TARGET :: VR(:,:)
		LOGICAL, INTENT(IN), OPTIONAL, TARGET :: SELECT(:)
	END SUBROUTINE STGSNA_MKL95
	SUBROUTINE DTGSNA_MKL95(A,B,S,DIF,VL,VR,SELECT,M,INFO)
		! MKL Fortran77 call:
		! DTGSNA(JOB,HOWMNY,SELECT,N,A,LDA,B,LDB,VL,LDVL,VR,LDVR,S,DIF,MM,M,WORK,LWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(IN) :: A(:,:)
		REAL(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: S(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: DIF(:)
		REAL(WP), INTENT(IN), OPTIONAL, TARGET :: VL(:,:)
		REAL(WP), INTENT(IN), OPTIONAL, TARGET :: VR(:,:)
		LOGICAL, INTENT(IN), OPTIONAL, TARGET :: SELECT(:)
	END SUBROUTINE DTGSNA_MKL95
	SUBROUTINE CTGSNA_MKL95(A,B,S,DIF,VL,VR,SELECT,M,INFO)
		! MKL Fortran77 call:
		! CTGSNA(JOB,HOWMNY,SELECT,N,A,LDA,B,LDB,VL,LDVL,VR,LDVR,S,DIF,MM,M,WORK,LWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: S(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: DIF(:)
		COMPLEX(WP), INTENT(IN), OPTIONAL, TARGET :: VL(:,:)
		COMPLEX(WP), INTENT(IN), OPTIONAL, TARGET :: VR(:,:)
		LOGICAL, INTENT(IN), OPTIONAL, TARGET :: SELECT(:)
	END SUBROUTINE CTGSNA_MKL95
	SUBROUTINE ZTGSNA_MKL95(A,B,S,DIF,VL,VR,SELECT,M,INFO)
		! MKL Fortran77 call:
		! ZTGSNA(JOB,HOWMNY,SELECT,N,A,LDA,B,LDB,VL,LDVL,VR,LDVR,S,DIF,MM,M,WORK,LWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: M
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(IN) :: A(:,:)
		COMPLEX(WP), INTENT(IN) :: B(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: S(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: DIF(:)
		COMPLEX(WP), INTENT(IN), OPTIONAL, TARGET :: VL(:,:)
		COMPLEX(WP), INTENT(IN), OPTIONAL, TARGET :: VR(:,:)
		LOGICAL, INTENT(IN), OPTIONAL, TARGET :: SELECT(:)
	END SUBROUTINE ZTGSNA_MKL95
END INTERFACE TGSNA
INTERFACE GGSVP
	SUBROUTINE SGGSVP_MKL95(A,B,TOLA,TOLB,K,L,U,V,Q,INFO)
		! MKL Fortran77 call:
		! SGGSVP(JOBU,JOBV,JOBQ,M,P,N,A,LDA,B,LDB,TOLA,TOLB,K,L,U,LDU,V,LDV,Q,LDQ,IWORK,TAU,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(IN) :: TOLA
		REAL(WP), INTENT(IN) :: TOLB
		INTEGER, INTENT(OUT), OPTIONAL :: K
		INTEGER, INTENT(OUT), OPTIONAL :: L
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: U(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: V(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Q(:,:)
	END SUBROUTINE SGGSVP_MKL95
	SUBROUTINE DGGSVP_MKL95(A,B,TOLA,TOLB,K,L,U,V,Q,INFO)
		! MKL Fortran77 call:
		! DGGSVP(JOBU,JOBV,JOBQ,M,P,N,A,LDA,B,LDB,TOLA,TOLB,K,L,U,LDU,V,LDV,Q,LDQ,IWORK,TAU,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(IN) :: TOLA
		REAL(WP), INTENT(IN) :: TOLB
		INTEGER, INTENT(OUT), OPTIONAL :: K
		INTEGER, INTENT(OUT), OPTIONAL :: L
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: U(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: V(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Q(:,:)
	END SUBROUTINE DGGSVP_MKL95
	SUBROUTINE CGGSVP_MKL95(A,B,TOLA,TOLB,K,L,U,V,Q,INFO)
		! MKL Fortran77 call:
		! CGGSVP(JOBU,JOBV,JOBQ,M,P,N,A,LDA,B,LDB,TOLA,TOLB,K,L,U,LDU,V,LDV,Q,LDQ,IWORK,RWORK,TAU,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(IN) :: TOLA
		REAL(WP), INTENT(IN) :: TOLB
		INTEGER, INTENT(OUT), OPTIONAL :: K
		INTEGER, INTENT(OUT), OPTIONAL :: L
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: U(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: V(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Q(:,:)
	END SUBROUTINE CGGSVP_MKL95
	SUBROUTINE ZGGSVP_MKL95(A,B,TOLA,TOLB,K,L,U,V,Q,INFO)
		! MKL Fortran77 call:
		! ZGGSVP(JOBU,JOBV,JOBQ,M,P,N,A,LDA,B,LDB,TOLA,TOLB,K,L,U,LDU,V,LDV,Q,LDQ,IWORK,RWORK,TAU,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(IN) :: TOLA
		REAL(WP), INTENT(IN) :: TOLB
		INTEGER, INTENT(OUT), OPTIONAL :: K
		INTEGER, INTENT(OUT), OPTIONAL :: L
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: U(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: V(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Q(:,:)
	END SUBROUTINE ZGGSVP_MKL95
END INTERFACE GGSVP
INTERFACE TGSJA
	SUBROUTINE STGSJA_MKL95(A,B,TOLA,TOLB,K,L,U,V,Q,JOBU,JOBV,JOBQ,ALPHA,BETA,NCYCLE,INFO)
		! MKL Fortran77 call:
		! STGSJA(JOBU,JOBV,JOBQ,M,P,N,K,L,A,LDA,B,LDB,TOLA,TOLB,ALPHA,BETA,U,LDU,V,LDV,Q,LDQ,WORK,NCYCLE,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(IN) :: TOLA
		REAL(WP), INTENT(IN) :: TOLB
		INTEGER, INTENT(IN) :: K
		INTEGER, INTENT(IN) :: L
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBU
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBV
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBQ
		INTEGER, INTENT(OUT), OPTIONAL :: NCYCLE
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: U(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: V(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: ALPHA(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BETA(:)
	END SUBROUTINE STGSJA_MKL95
	SUBROUTINE DTGSJA_MKL95(A,B,TOLA,TOLB,K,L,U,V,Q,JOBU,JOBV,JOBQ,ALPHA,BETA,NCYCLE,INFO)
		! MKL Fortran77 call:
		! DTGSJA(JOBU,JOBV,JOBQ,M,P,N,K,L,A,LDA,B,LDB,TOLA,TOLB,ALPHA,BETA,U,LDU,V,LDV,Q,LDQ,WORK,NCYCLE,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(IN) :: TOLA
		REAL(WP), INTENT(IN) :: TOLB
		INTEGER, INTENT(IN) :: K
		INTEGER, INTENT(IN) :: L
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBU
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBV
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBQ
		INTEGER, INTENT(OUT), OPTIONAL :: NCYCLE
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: U(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: V(:,:)
		REAL(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: ALPHA(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BETA(:)
	END SUBROUTINE DTGSJA_MKL95
	SUBROUTINE CTGSJA_MKL95(A,B,TOLA,TOLB,K,L,U,V,Q,JOBU,JOBV,JOBQ,ALPHA,BETA,NCYCLE,INFO)
		! MKL Fortran77 call:
		! CTGSJA(JOBU,JOBV,JOBQ,M,P,N,K,L,A,LDA,B,LDB,TOLA,TOLB,ALPHA,BETA,U,LDU,V,LDV,Q,LDQ,WORK,NCYCLE,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(IN) :: TOLA
		REAL(WP), INTENT(IN) :: TOLB
		INTEGER, INTENT(IN) :: K
		INTEGER, INTENT(IN) :: L
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBU
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBV
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBQ
		INTEGER, INTENT(OUT), OPTIONAL :: NCYCLE
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: U(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: V(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: ALPHA(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BETA(:)
	END SUBROUTINE CTGSJA_MKL95
	SUBROUTINE ZTGSJA_MKL95(A,B,TOLA,TOLB,K,L,U,V,Q,JOBU,JOBV,JOBQ,ALPHA,BETA,NCYCLE,INFO)
		! MKL Fortran77 call:
		! ZTGSJA(JOBU,JOBV,JOBQ,M,P,N,K,L,A,LDA,B,LDB,TOLA,TOLB,ALPHA,BETA,U,LDU,V,LDV,Q,LDQ,WORK,NCYCLE,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(IN) :: TOLA
		REAL(WP), INTENT(IN) :: TOLB
		INTEGER, INTENT(IN) :: K
		INTEGER, INTENT(IN) :: L
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBU
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBV
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBQ
		INTEGER, INTENT(OUT), OPTIONAL :: NCYCLE
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: U(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: V(:,:)
		COMPLEX(WP), INTENT(INOUT), OPTIONAL, TARGET :: Q(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: ALPHA(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: BETA(:)
	END SUBROUTINE ZTGSJA_MKL95
END INTERFACE TGSJA
INTERFACE GELS
	SUBROUTINE SGELS_MKL95(A,B,TRANS,INFO)
		! MKL Fortran77 call:
		! SGELS(TRANS,M,N,NRHS,A,LDA,B,LDB,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE SGELS_MKL95
	SUBROUTINE SGELS1_MKL95(A,B,TRANS,INFO)
		! MKL Fortran77 call:
		! SGELS(TRANS,M,N,NRHS,A,LDA,B,LDB,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE SGELS1_MKL95
	SUBROUTINE DGELS_MKL95(A,B,TRANS,INFO)
		! MKL Fortran77 call:
		! DGELS(TRANS,M,N,NRHS,A,LDA,B,LDB,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE DGELS_MKL95
	SUBROUTINE DGELS1_MKL95(A,B,TRANS,INFO)
		! MKL Fortran77 call:
		! DGELS(TRANS,M,N,NRHS,A,LDA,B,LDB,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE DGELS1_MKL95
	SUBROUTINE CGELS_MKL95(A,B,TRANS,INFO)
		! MKL Fortran77 call:
		! CGELS(TRANS,M,N,NRHS,A,LDA,B,LDB,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE CGELS_MKL95
	SUBROUTINE CGELS1_MKL95(A,B,TRANS,INFO)
		! MKL Fortran77 call:
		! CGELS(TRANS,M,N,NRHS,A,LDA,B,LDB,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE CGELS1_MKL95
	SUBROUTINE ZGELS_MKL95(A,B,TRANS,INFO)
		! MKL Fortran77 call:
		! ZGELS(TRANS,M,N,NRHS,A,LDA,B,LDB,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
	END SUBROUTINE ZGELS_MKL95
	SUBROUTINE ZGELS1_MKL95(A,B,TRANS,INFO)
		! MKL Fortran77 call:
		! ZGELS(TRANS,M,N,NRHS,A,LDA,B,LDB,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: TRANS
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
	END SUBROUTINE ZGELS1_MKL95
END INTERFACE GELS
INTERFACE GELSY
	SUBROUTINE SGELSY_MKL95(A,B,RANK,JPVT,RCOND,INFO)
		! MKL Fortran77 call:
		! SGELSY(M,N,NRHS,A,LDA,B,LDB,JPVT,RCOND,RANK,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: RANK
		REAL(WP), INTENT(IN), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: JPVT(:)
	END SUBROUTINE SGELSY_MKL95
	SUBROUTINE SGELSY1_MKL95(A,B,RANK,JPVT,RCOND,INFO)
		! MKL Fortran77 call:
		! SGELSY(M,N,NRHS,A,LDA,B,LDB,JPVT,RCOND,RANK,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: RANK
		REAL(WP), INTENT(IN), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: JPVT(:)
	END SUBROUTINE SGELSY1_MKL95
	SUBROUTINE DGELSY_MKL95(A,B,RANK,JPVT,RCOND,INFO)
		! MKL Fortran77 call:
		! DGELSY(M,N,NRHS,A,LDA,B,LDB,JPVT,RCOND,RANK,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: RANK
		REAL(WP), INTENT(IN), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: JPVT(:)
	END SUBROUTINE DGELSY_MKL95
	SUBROUTINE DGELSY1_MKL95(A,B,RANK,JPVT,RCOND,INFO)
		! MKL Fortran77 call:
		! DGELSY(M,N,NRHS,A,LDA,B,LDB,JPVT,RCOND,RANK,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: RANK
		REAL(WP), INTENT(IN), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: JPVT(:)
	END SUBROUTINE DGELSY1_MKL95
	SUBROUTINE CGELSY_MKL95(A,B,RANK,JPVT,RCOND,INFO)
		! MKL Fortran77 call:
		! CGELSY(M,N,NRHS,A,LDA,B,LDB,JPVT,RCOND,RANK,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: RANK
		REAL(WP), INTENT(IN), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: JPVT(:)
	END SUBROUTINE CGELSY_MKL95
	SUBROUTINE CGELSY1_MKL95(A,B,RANK,JPVT,RCOND,INFO)
		! MKL Fortran77 call:
		! CGELSY(M,N,NRHS,A,LDA,B,LDB,JPVT,RCOND,RANK,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: RANK
		REAL(WP), INTENT(IN), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: JPVT(:)
	END SUBROUTINE CGELSY1_MKL95
	SUBROUTINE ZGELSY_MKL95(A,B,RANK,JPVT,RCOND,INFO)
		! MKL Fortran77 call:
		! ZGELSY(M,N,NRHS,A,LDA,B,LDB,JPVT,RCOND,RANK,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: RANK
		REAL(WP), INTENT(IN), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: JPVT(:)
	END SUBROUTINE ZGELSY_MKL95
	SUBROUTINE ZGELSY1_MKL95(A,B,RANK,JPVT,RCOND,INFO)
		! MKL Fortran77 call:
		! ZGELSY(M,N,NRHS,A,LDA,B,LDB,JPVT,RCOND,RANK,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: RANK
		REAL(WP), INTENT(IN), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		INTEGER, INTENT(INOUT), OPTIONAL, TARGET :: JPVT(:)
	END SUBROUTINE ZGELSY1_MKL95
END INTERFACE GELSY
INTERFACE GELSS
	SUBROUTINE SGELSS_MKL95(A,B,RANK,S,RCOND,INFO)
		! MKL Fortran77 call:
		! SGELSS(M,N,NRHS,A,LDA,B,LDB,S,RCOND,RANK,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: RANK
		REAL(WP), INTENT(IN), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: S(:)
	END SUBROUTINE SGELSS_MKL95
	SUBROUTINE SGELSS1_MKL95(A,B,RANK,S,RCOND,INFO)
		! MKL Fortran77 call:
		! SGELSS(M,N,NRHS,A,LDA,B,LDB,S,RCOND,RANK,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: RANK
		REAL(WP), INTENT(IN), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: S(:)
	END SUBROUTINE SGELSS1_MKL95
	SUBROUTINE DGELSS_MKL95(A,B,RANK,S,RCOND,INFO)
		! MKL Fortran77 call:
		! DGELSS(M,N,NRHS,A,LDA,B,LDB,S,RCOND,RANK,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: RANK
		REAL(WP), INTENT(IN), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: S(:)
	END SUBROUTINE DGELSS_MKL95
	SUBROUTINE DGELSS1_MKL95(A,B,RANK,S,RCOND,INFO)
		! MKL Fortran77 call:
		! DGELSS(M,N,NRHS,A,LDA,B,LDB,S,RCOND,RANK,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: RANK
		REAL(WP), INTENT(IN), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: S(:)
	END SUBROUTINE DGELSS1_MKL95
	SUBROUTINE CGELSS_MKL95(A,B,RANK,S,RCOND,INFO)
		! MKL Fortran77 call:
		! CGELSS(M,N,NRHS,A,LDA,B,LDB,S,RCOND,RANK,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: RANK
		REAL(WP), INTENT(IN), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: S(:)
	END SUBROUTINE CGELSS_MKL95
	SUBROUTINE CGELSS1_MKL95(A,B,RANK,S,RCOND,INFO)
		! MKL Fortran77 call:
		! CGELSS(M,N,NRHS,A,LDA,B,LDB,S,RCOND,RANK,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: RANK
		REAL(WP), INTENT(IN), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: S(:)
	END SUBROUTINE CGELSS1_MKL95
	SUBROUTINE ZGELSS_MKL95(A,B,RANK,S,RCOND,INFO)
		! MKL Fortran77 call:
		! ZGELSS(M,N,NRHS,A,LDA,B,LDB,S,RCOND,RANK,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: RANK
		REAL(WP), INTENT(IN), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: S(:)
	END SUBROUTINE ZGELSS_MKL95
	SUBROUTINE ZGELSS1_MKL95(A,B,RANK,S,RCOND,INFO)
		! MKL Fortran77 call:
		! ZGELSS(M,N,NRHS,A,LDA,B,LDB,S,RCOND,RANK,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: RANK
		REAL(WP), INTENT(IN), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: S(:)
	END SUBROUTINE ZGELSS1_MKL95
END INTERFACE GELSS
INTERFACE GELSD
	SUBROUTINE SGELSD_MKL95(A,B,RANK,S,RCOND,INFO)
		! MKL Fortran77 call:
		! SGELSD(M,N,NRHS,A,LDA,B,LDB,S,RCOND,RANK,WORK,LWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: RANK
		REAL(WP), INTENT(IN), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: S(:)
	END SUBROUTINE SGELSD_MKL95
	SUBROUTINE SGELSD1_MKL95(A,B,RANK,S,RCOND,INFO)
		! MKL Fortran77 call:
		! SGELSD(M,N,NRHS,A,LDA,B,LDB,S,RCOND,RANK,WORK,LWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: RANK
		REAL(WP), INTENT(IN), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: S(:)
	END SUBROUTINE SGELSD1_MKL95
	SUBROUTINE DGELSD_MKL95(A,B,RANK,S,RCOND,INFO)
		! MKL Fortran77 call:
		! DGELSD(M,N,NRHS,A,LDA,B,LDB,S,RCOND,RANK,WORK,LWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: RANK
		REAL(WP), INTENT(IN), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: S(:)
	END SUBROUTINE DGELSD_MKL95
	SUBROUTINE DGELSD1_MKL95(A,B,RANK,S,RCOND,INFO)
		! MKL Fortran77 call:
		! DGELSD(M,N,NRHS,A,LDA,B,LDB,S,RCOND,RANK,WORK,LWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: RANK
		REAL(WP), INTENT(IN), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: S(:)
	END SUBROUTINE DGELSD1_MKL95
	SUBROUTINE CGELSD_MKL95(A,B,RANK,S,RCOND,INFO)
		! MKL Fortran77 call:
		! CGELSD(M,N,NRHS,A,LDA,B,LDB,S,RCOND,RANK,WORK,LWORK,RWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: RANK
		REAL(WP), INTENT(IN), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: S(:)
	END SUBROUTINE CGELSD_MKL95
	SUBROUTINE CGELSD1_MKL95(A,B,RANK,S,RCOND,INFO)
		! MKL Fortran77 call:
		! CGELSD(M,N,NRHS,A,LDA,B,LDB,S,RCOND,RANK,WORK,LWORK,RWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: RANK
		REAL(WP), INTENT(IN), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: S(:)
	END SUBROUTINE CGELSD1_MKL95
	SUBROUTINE ZGELSD_MKL95(A,B,RANK,S,RCOND,INFO)
		! MKL Fortran77 call:
		! ZGELSD(M,N,NRHS,A,LDA,B,LDB,S,RCOND,RANK,WORK,LWORK,RWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: RANK
		REAL(WP), INTENT(IN), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: S(:)
	END SUBROUTINE ZGELSD_MKL95
	SUBROUTINE ZGELSD1_MKL95(A,B,RANK,S,RCOND,INFO)
		! MKL Fortran77 call:
		! ZGELSD(M,N,NRHS,A,LDA,B,LDB,S,RCOND,RANK,WORK,LWORK,RWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: RANK
		REAL(WP), INTENT(IN), OPTIONAL :: RCOND
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: S(:)
	END SUBROUTINE ZGELSD1_MKL95
END INTERFACE GELSD
INTERFACE GGLSE
	SUBROUTINE SGGLSE_MKL95(A,B,C,D,X,INFO)
		! MKL Fortran77 call:
		! SGGLSE(M,N,P,A,LDA,B,LDB,C,D,X,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(INOUT) :: C(:)
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(OUT) :: X(:)
	END SUBROUTINE SGGLSE_MKL95
	SUBROUTINE DGGLSE_MKL95(A,B,C,D,X,INFO)
		! MKL Fortran77 call:
		! DGGLSE(M,N,P,A,LDA,B,LDB,C,D,X,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(INOUT) :: C(:)
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(OUT) :: X(:)
	END SUBROUTINE DGGLSE_MKL95
	SUBROUTINE CGGLSE_MKL95(A,B,C,D,X,INFO)
		! MKL Fortran77 call:
		! CGGLSE(M,N,P,A,LDA,B,LDB,C,D,X,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: C(:)
		COMPLEX(WP), INTENT(INOUT) :: D(:)
		COMPLEX(WP), INTENT(OUT) :: X(:)
	END SUBROUTINE CGGLSE_MKL95
	SUBROUTINE ZGGLSE_MKL95(A,B,C,D,X,INFO)
		! MKL Fortran77 call:
		! ZGGLSE(M,N,P,A,LDA,B,LDB,C,D,X,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: C(:)
		COMPLEX(WP), INTENT(INOUT) :: D(:)
		COMPLEX(WP), INTENT(OUT) :: X(:)
	END SUBROUTINE ZGGLSE_MKL95
END INTERFACE GGLSE
INTERFACE GGGLM
	SUBROUTINE SGGGLM_MKL95(A,B,D,X,Y,INFO)
		! MKL Fortran77 call:
		! SGGGLM(N,M,P,A,LDA,B,LDB,D,X,Y,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(OUT) :: X(:)
		REAL(WP), INTENT(OUT) :: Y(:)
	END SUBROUTINE SGGGLM_MKL95
	SUBROUTINE DGGGLM_MKL95(A,B,D,X,Y,INFO)
		! MKL Fortran77 call:
		! DGGGLM(N,M,P,A,LDA,B,LDB,D,X,Y,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(OUT) :: X(:)
		REAL(WP), INTENT(OUT) :: Y(:)
	END SUBROUTINE DGGGLM_MKL95
	SUBROUTINE CGGGLM_MKL95(A,B,D,X,Y,INFO)
		! MKL Fortran77 call:
		! CGGGLM(N,M,P,A,LDA,B,LDB,D,X,Y,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: D(:)
		COMPLEX(WP), INTENT(OUT) :: X(:)
		COMPLEX(WP), INTENT(OUT) :: Y(:)
	END SUBROUTINE CGGGLM_MKL95
	SUBROUTINE ZGGGLM_MKL95(A,B,D,X,Y,INFO)
		! MKL Fortran77 call:
		! ZGGGLM(N,M,P,A,LDA,B,LDB,D,X,Y,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(INOUT) :: D(:)
		COMPLEX(WP), INTENT(OUT) :: X(:)
		COMPLEX(WP), INTENT(OUT) :: Y(:)
	END SUBROUTINE ZGGGLM_MKL95
END INTERFACE GGGLM
INTERFACE SYEV
	SUBROUTINE SSYEV_MKL95(A,W,JOBZ,UPLO,INFO)
		! MKL Fortran77 call:
		! SSYEV(JOBZ,UPLO,N,A,LDA,W,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBZ
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
	END SUBROUTINE SSYEV_MKL95
	SUBROUTINE DSYEV_MKL95(A,W,JOBZ,UPLO,INFO)
		! MKL Fortran77 call:
		! DSYEV(JOBZ,UPLO,N,A,LDA,W,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBZ
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
	END SUBROUTINE DSYEV_MKL95
END INTERFACE SYEV
INTERFACE HEEV
	SUBROUTINE CHEEV_MKL95(A,W,JOBZ,UPLO,INFO)
		! MKL Fortran77 call:
		! CHEEV(JOBZ,UPLO,N,A,LDA,W,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBZ
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
	END SUBROUTINE CHEEV_MKL95
	SUBROUTINE ZHEEV_MKL95(A,W,JOBZ,UPLO,INFO)
		! MKL Fortran77 call:
		! ZHEEV(JOBZ,UPLO,N,A,LDA,W,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBZ
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
	END SUBROUTINE ZHEEV_MKL95
END INTERFACE HEEV
INTERFACE SYEVD
	SUBROUTINE SSYEVD_MKL95(A,W,JOBZ,UPLO,INFO)
		! MKL Fortran77 call:
		! SSYEVD(JOBZ,UPLO,N,A,LDA,W,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBZ
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
	END SUBROUTINE SSYEVD_MKL95
	SUBROUTINE DSYEVD_MKL95(A,W,JOBZ,UPLO,INFO)
		! MKL Fortran77 call:
		! DSYEVD(JOBZ,UPLO,N,A,LDA,W,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBZ
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
	END SUBROUTINE DSYEVD_MKL95
END INTERFACE SYEVD
INTERFACE HEEVD
	SUBROUTINE CHEEVD_MKL95(A,W,JOB,UPLO,INFO)
		! MKL Fortran77 call:
		! CHEEVD(JOB,UPLO,N,A,LDA,W,WORK,LWORK,RWORK,LRWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
	END SUBROUTINE CHEEVD_MKL95
	SUBROUTINE ZHEEVD_MKL95(A,W,JOB,UPLO,INFO)
		! MKL Fortran77 call:
		! ZHEEVD(JOB,UPLO,N,A,LDA,W,WORK,LWORK,RWORK,LRWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
	END SUBROUTINE ZHEEVD_MKL95
END INTERFACE HEEVD
INTERFACE SYEVX
	SUBROUTINE SSYEVX_MKL95(A,W,UPLO,Z,VL,VU,IL,IU,M,IFAIL,ABSTOL,INFO)
		! MKL Fortran77 call:
		! SSYEVX(JOBZ,RANGE,UPLO,N,A,LDA,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,WORK,LWORK,IWORK,IFAIL,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAIL(:)
	END SUBROUTINE SSYEVX_MKL95
	SUBROUTINE DSYEVX_MKL95(A,W,UPLO,Z,VL,VU,IL,IU,M,IFAIL,ABSTOL,INFO)
		! MKL Fortran77 call:
		! DSYEVX(JOBZ,RANGE,UPLO,N,A,LDA,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,WORK,LWORK,IWORK,IFAIL,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAIL(:)
	END SUBROUTINE DSYEVX_MKL95
END INTERFACE SYEVX
INTERFACE HEEVX
	SUBROUTINE CHEEVX_MKL95(A,W,UPLO,Z,VL,VU,IL,IU,M,IFAIL,ABSTOL,INFO)
		! MKL Fortran77 call:
		! CHEEVX(JOBZ,RANGE,UPLO,N,A,LDA,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,WORK,LWORK,RWORK,IWORK,IFAIL,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAIL(:)
	END SUBROUTINE CHEEVX_MKL95
	SUBROUTINE ZHEEVX_MKL95(A,W,UPLO,Z,VL,VU,IL,IU,M,IFAIL,ABSTOL,INFO)
		! MKL Fortran77 call:
		! ZHEEVX(JOBZ,RANGE,UPLO,N,A,LDA,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,WORK,LWORK,RWORK,IWORK,IFAIL,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAIL(:)
	END SUBROUTINE ZHEEVX_MKL95
END INTERFACE HEEVX
INTERFACE SYEVR
	SUBROUTINE SSYEVR_MKL95(A,W,UPLO,Z,VL,VU,IL,IU,M,ISUPPZ,ABSTOL,INFO)
		! MKL Fortran77 call:
		! SSYEVR(JOBZ,RANGE,UPLO,N,A,LDA,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,ISUPPZ,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: ISUPPZ(:)
	END SUBROUTINE SSYEVR_MKL95
	SUBROUTINE DSYEVR_MKL95(A,W,UPLO,Z,VL,VU,IL,IU,M,ISUPPZ,ABSTOL,INFO)
		! MKL Fortran77 call:
		! DSYEVR(JOBZ,RANGE,UPLO,N,A,LDA,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,ISUPPZ,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: ISUPPZ(:)
	END SUBROUTINE DSYEVR_MKL95
END INTERFACE SYEVR
INTERFACE HEEVR
	SUBROUTINE CHEEVR_MKL95(A,W,UPLO,Z,VL,VU,IL,IU,M,ISUPPZ,ABSTOL,INFO)
		! MKL Fortran77 call:
		! CHEEVR(JOBZ,RANGE,UPLO,N,A,LDA,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,ISUPPZ,WORK,LWORK,RWORK,LRWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: ISUPPZ(:)
	END SUBROUTINE CHEEVR_MKL95
	SUBROUTINE ZHEEVR_MKL95(A,W,UPLO,Z,VL,VU,IL,IU,M,ISUPPZ,ABSTOL,INFO)
		! MKL Fortran77 call:
		! ZHEEVR(JOBZ,RANGE,UPLO,N,A,LDA,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,ISUPPZ,WORK,LWORK,RWORK,LRWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: ISUPPZ(:)
	END SUBROUTINE ZHEEVR_MKL95
END INTERFACE HEEVR
INTERFACE SPEV
	SUBROUTINE SSPEV_MKL95(A,W,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! SSPEV(JOBZ,UPLO,N,A,W,Z,LDZ,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE SSPEV_MKL95
	SUBROUTINE DSPEV_MKL95(A,W,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! DSPEV(JOBZ,UPLO,N,A,W,Z,LDZ,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE DSPEV_MKL95
END INTERFACE SPEV
INTERFACE HPEV
	SUBROUTINE CHPEV_MKL95(A,W,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! CHPEV(JOBZ,UPLO,N,A,W,Z,LDZ,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE CHPEV_MKL95
	SUBROUTINE ZHPEV_MKL95(A,W,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! ZHPEV(JOBZ,UPLO,N,A,W,Z,LDZ,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE ZHPEV_MKL95
END INTERFACE HPEV
INTERFACE SPEVD
	SUBROUTINE SSPEVD_MKL95(A,W,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! SSPEVD(JOBZ,UPLO,N,A,W,Z,LDZ,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE SSPEVD_MKL95
	SUBROUTINE DSPEVD_MKL95(A,W,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! DSPEVD(JOBZ,UPLO,N,A,W,Z,LDZ,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE DSPEVD_MKL95
END INTERFACE SPEVD
INTERFACE HPEVD
	SUBROUTINE CHPEVD_MKL95(A,W,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! CHPEVD(JOB,UPLO,N,A,W,Z,LDZ,WORK,LWORK,RWORK,LRWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE CHPEVD_MKL95
	SUBROUTINE ZHPEVD_MKL95(A,W,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! ZHPEVD(JOB,UPLO,N,A,W,Z,LDZ,WORK,LWORK,RWORK,LRWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE ZHPEVD_MKL95
END INTERFACE HPEVD
INTERFACE SPEVX
	SUBROUTINE SSPEVX_MKL95(A,W,UPLO,Z,VL,VU,IL,IU,M,IFAIL,ABSTOL,INFO)
		! MKL Fortran77 call:
		! SSPEVX(JOBZ,RANGE,UPLO,N,A,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,WORK,IWORK,IFAIL,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAIL(:)
	END SUBROUTINE SSPEVX_MKL95
	SUBROUTINE DSPEVX_MKL95(A,W,UPLO,Z,VL,VU,IL,IU,M,IFAIL,ABSTOL,INFO)
		! MKL Fortran77 call:
		! DSPEVX(JOBZ,RANGE,UPLO,N,A,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,WORK,IWORK,IFAIL,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAIL(:)
	END SUBROUTINE DSPEVX_MKL95
END INTERFACE SPEVX
INTERFACE HPEVX
	SUBROUTINE CHPEVX_MKL95(A,W,UPLO,Z,VL,VU,IL,IU,M,IFAIL,ABSTOL,INFO)
		! MKL Fortran77 call:
		! CHPEVX(JOBZ,RANGE,UPLO,N,A,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,WORK,RWORK,IWORK,IFAIL,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAIL(:)
	END SUBROUTINE CHPEVX_MKL95
	SUBROUTINE ZHPEVX_MKL95(A,W,UPLO,Z,VL,VU,IL,IU,M,IFAIL,ABSTOL,INFO)
		! MKL Fortran77 call:
		! ZHPEVX(JOBZ,RANGE,UPLO,N,A,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,WORK,RWORK,IWORK,IFAIL,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAIL(:)
	END SUBROUTINE ZHPEVX_MKL95
END INTERFACE HPEVX
INTERFACE SBEV
	SUBROUTINE SSBEV_MKL95(A,W,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! SSBEV(JOBZ,UPLO,N,KD,A,LDA,W,Z,LDZ,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE SSBEV_MKL95
	SUBROUTINE DSBEV_MKL95(A,W,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! DSBEV(JOBZ,UPLO,N,KD,A,LDA,W,Z,LDZ,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE DSBEV_MKL95
END INTERFACE SBEV
INTERFACE HBEV
	SUBROUTINE CHBEV_MKL95(A,W,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! CHBEV(JOBZ,UPLO,N,KD,A,LDA,W,Z,LDZ,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE CHBEV_MKL95
	SUBROUTINE ZHBEV_MKL95(A,W,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! ZHBEV(JOBZ,UPLO,N,KD,A,LDA,W,Z,LDZ,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE ZHBEV_MKL95
END INTERFACE HBEV
INTERFACE SBEVD
	SUBROUTINE SSBEVD_MKL95(A,W,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! SSBEVD(JOBZ,UPLO,N,KD,A,LDA,W,Z,LDZ,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE SSBEVD_MKL95
	SUBROUTINE DSBEVD_MKL95(A,W,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! DSBEVD(JOBZ,UPLO,N,KD,A,LDA,W,Z,LDZ,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE DSBEVD_MKL95
END INTERFACE SBEVD
INTERFACE HBEVD
	SUBROUTINE CHBEVD_MKL95(A,W,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! CHBEVD(JOB,UPLO,N,KD,A,LDA,W,Z,LDZ,WORK,LWORK,RWORK,LRWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE CHBEVD_MKL95
	SUBROUTINE ZHBEVD_MKL95(A,W,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! ZHBEVD(JOB,UPLO,N,KD,A,LDA,W,Z,LDZ,WORK,LWORK,RWORK,LRWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE ZHBEVD_MKL95
END INTERFACE HBEVD
INTERFACE SBEVX
	SUBROUTINE SSBEVX_MKL95(A,W,UPLO,Z,VL,VU,IL,IU,M,IFAIL,Q,ABSTOL,INFO)
		! MKL Fortran77 call:
		! SSBEVX(JOBZ,RANGE,UPLO,N,KD,A,LDA,Q,LDQ,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,WORK,IWORK,IFAIL,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAIL(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Q(:,:)
	END SUBROUTINE SSBEVX_MKL95
	SUBROUTINE DSBEVX_MKL95(A,W,UPLO,Z,VL,VU,IL,IU,M,IFAIL,Q,ABSTOL,INFO)
		! MKL Fortran77 call:
		! DSBEVX(JOBZ,RANGE,UPLO,N,KD,A,LDA,Q,LDQ,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,WORK,IWORK,IFAIL,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAIL(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Q(:,:)
	END SUBROUTINE DSBEVX_MKL95
END INTERFACE SBEVX
INTERFACE HBEVX
	SUBROUTINE CHBEVX_MKL95(A,W,UPLO,Z,VL,VU,IL,IU,M,IFAIL,Q,ABSTOL,INFO)
		! MKL Fortran77 call:
		! CHBEVX(JOBZ,RANGE,UPLO,N,KD,A,LDA,Q,LDQ,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,WORK,RWORK,IWORK,IFAIL,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAIL(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Q(:,:)
	END SUBROUTINE CHBEVX_MKL95
	SUBROUTINE ZHBEVX_MKL95(A,W,UPLO,Z,VL,VU,IL,IU,M,IFAIL,Q,ABSTOL,INFO)
		! MKL Fortran77 call:
		! ZHBEVX(JOBZ,RANGE,UPLO,N,KD,A,LDA,Q,LDQ,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,WORK,RWORK,IWORK,IFAIL,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAIL(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Q(:,:)
	END SUBROUTINE ZHBEVX_MKL95
END INTERFACE HBEVX
INTERFACE STEV
	SUBROUTINE SSTEV_MKL95(D,E,Z,INFO)
		! MKL Fortran77 call:
		! SSTEV(JOBZ,N,D,E,Z,LDZ,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE SSTEV_MKL95
	SUBROUTINE DSTEV_MKL95(D,E,Z,INFO)
		! MKL Fortran77 call:
		! DSTEV(JOBZ,N,D,E,Z,LDZ,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE DSTEV_MKL95
END INTERFACE STEV
INTERFACE STEVD
	SUBROUTINE SSTEVD_MKL95(D,E,Z,INFO)
		! MKL Fortran77 call:
		! SSTEVD(JOBZ,N,D,E,Z,LDZ,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE SSTEVD_MKL95
	SUBROUTINE DSTEVD_MKL95(D,E,Z,INFO)
		! MKL Fortran77 call:
		! DSTEVD(JOBZ,N,D,E,Z,LDZ,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE DSTEVD_MKL95
END INTERFACE STEVD
INTERFACE STEVX
	SUBROUTINE SSTEVX_MKL95(D,E,W,Z,VL,VU,IL,IU,M,IFAIL,ABSTOL,INFO)
		! MKL Fortran77 call:
		! SSTEVX(JOBZ,RANGE,N,D,E,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,WORK,IWORK,IFAIL,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAIL(:)
	END SUBROUTINE SSTEVX_MKL95
	SUBROUTINE DSTEVX_MKL95(D,E,W,Z,VL,VU,IL,IU,M,IFAIL,ABSTOL,INFO)
		! MKL Fortran77 call:
		! DSTEVX(JOBZ,RANGE,N,D,E,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,WORK,IWORK,IFAIL,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAIL(:)
	END SUBROUTINE DSTEVX_MKL95
END INTERFACE STEVX
INTERFACE STEVR
	SUBROUTINE SSTEVR_MKL95(D,E,W,Z,VL,VU,IL,IU,M,ISUPPZ,ABSTOL,INFO)
		! MKL Fortran77 call:
		! SSTEVR(JOBZ,RANGE,N,D,E,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,ISUPPZ,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: ISUPPZ(:)
	END SUBROUTINE SSTEVR_MKL95
	SUBROUTINE DSTEVR_MKL95(D,E,W,Z,VL,VU,IL,IU,M,ISUPPZ,ABSTOL,INFO)
		! MKL Fortran77 call:
		! DSTEVR(JOBZ,RANGE,N,D,E,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,ISUPPZ,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: D(:)
		REAL(WP), INTENT(INOUT) :: E(:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: ISUPPZ(:)
	END SUBROUTINE DSTEVR_MKL95
END INTERFACE STEVR
INTERFACE GEES
	SUBROUTINE SGEES_MKL95(A,WR,WI,VS,SELECT,SDIM,INFO)
		! MKL Fortran77 call:
		! SGEES(JOBVS,SORT,SELECT,N,A,LDA,SDIM,WR,WI,VS,LDVS,WORK,LWORK,BWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: SDIM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: WR(:)
		REAL(WP), INTENT(OUT) :: WI(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VS(:,:)
		INTERFACE
			LOGICAL FUNCTION SELECT(WR,WI)
				USE MKL95_PRECISION, ONLY: WP => SP
				REAL(WP), INTENT(IN) :: WR,WI
			END FUNCTION SELECT
		END INTERFACE
		OPTIONAL :: SELECT
	END SUBROUTINE SGEES_MKL95
	SUBROUTINE DGEES_MKL95(A,WR,WI,VS,SELECT,SDIM,INFO)
		! MKL Fortran77 call:
		! DGEES(JOBVS,SORT,SELECT,N,A,LDA,SDIM,WR,WI,VS,LDVS,WORK,LWORK,BWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: SDIM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: WR(:)
		REAL(WP), INTENT(OUT) :: WI(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VS(:,:)
		INTERFACE
			LOGICAL FUNCTION SELECT(WR,WI)
				USE MKL95_PRECISION, ONLY: WP => DP
				REAL(WP), INTENT(IN) :: WR,WI
			END FUNCTION SELECT
		END INTERFACE
		OPTIONAL :: SELECT
	END SUBROUTINE DGEES_MKL95
	SUBROUTINE CGEES_MKL95(A,W,VS,SELECT,SDIM,INFO)
		! MKL Fortran77 call:
		! CGEES(JOBVS,SORT,SELECT,N,A,LDA,SDIM,W,VS,LDVS,WORK,LWORK,RWORK,BWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: SDIM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VS(:,:)
		INTERFACE
			LOGICAL FUNCTION SELECT(W)
				USE MKL95_PRECISION, ONLY: WP => SP
				REAL(WP), INTENT(IN) :: W
			END FUNCTION SELECT
		END INTERFACE
		OPTIONAL :: SELECT
	END SUBROUTINE CGEES_MKL95
	SUBROUTINE ZGEES_MKL95(A,W,VS,SELECT,SDIM,INFO)
		! MKL Fortran77 call:
		! ZGEES(JOBVS,SORT,SELECT,N,A,LDA,SDIM,W,VS,LDVS,WORK,LWORK,RWORK,BWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: SDIM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VS(:,:)
		INTERFACE
			LOGICAL FUNCTION SELECT(W)
				USE MKL95_PRECISION, ONLY: WP => DP
				REAL(WP), INTENT(IN) :: W
			END FUNCTION SELECT
		END INTERFACE
		OPTIONAL :: SELECT
	END SUBROUTINE ZGEES_MKL95
END INTERFACE GEES
INTERFACE GEESX
	SUBROUTINE SGEESX_MKL95(A,WR,WI,VS,SELECT,SDIM,RCONDE,RCONDV,INFO)
		! MKL Fortran77 call:
		! SGEESX(JOBVS,SORT,SELECT,SENSE,N,A,LDA,SDIM,WR,WI,VS,LDVS,RCONDE,RCONDV,WORK,LWORK,IWORK,LIWORK,BWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: SDIM
		REAL(WP), INTENT(OUT), OPTIONAL :: RCONDE
		REAL(WP), INTENT(OUT), OPTIONAL :: RCONDV
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: WR(:)
		REAL(WP), INTENT(OUT) :: WI(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VS(:,:)
		INTERFACE
			LOGICAL FUNCTION SELECT(WR,WI)
				USE MKL95_PRECISION, ONLY: WP => SP
				REAL(WP), INTENT(IN) :: WR,WI
			END FUNCTION SELECT
		END INTERFACE
		OPTIONAL :: SELECT
	END SUBROUTINE SGEESX_MKL95
	SUBROUTINE DGEESX_MKL95(A,WR,WI,VS,SELECT,SDIM,RCONDE,RCONDV,INFO)
		! MKL Fortran77 call:
		! DGEESX(JOBVS,SORT,SELECT,SENSE,N,A,LDA,SDIM,WR,WI,VS,LDVS,RCONDE,RCONDV,WORK,LWORK,IWORK,LIWORK,BWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: SDIM
		REAL(WP), INTENT(OUT), OPTIONAL :: RCONDE
		REAL(WP), INTENT(OUT), OPTIONAL :: RCONDV
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: WR(:)
		REAL(WP), INTENT(OUT) :: WI(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VS(:,:)
		INTERFACE
			LOGICAL FUNCTION SELECT(WR,WI)
				USE MKL95_PRECISION, ONLY: WP => DP
				REAL(WP), INTENT(IN) :: WR,WI
			END FUNCTION SELECT
		END INTERFACE
		OPTIONAL :: SELECT
	END SUBROUTINE DGEESX_MKL95
	SUBROUTINE CGEESX_MKL95(A,W,VS,SELECT,SDIM,RCONDE,RCONDV,INFO)
		! MKL Fortran77 call:
		! CGEESX(JOBVS,SORT,SELECT,SENSE,N,A,LDA,SDIM,W,VS,LDVS,RCONDE,RCONDV,WORK,LWORK,RWORK,BWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: SDIM
		REAL(WP), INTENT(OUT), OPTIONAL :: RCONDE
		REAL(WP), INTENT(OUT), OPTIONAL :: RCONDV
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VS(:,:)
		INTERFACE
			LOGICAL FUNCTION SELECT(W)
				USE MKL95_PRECISION, ONLY: WP => SP
				REAL(WP), INTENT(IN) :: W
			END FUNCTION SELECT
		END INTERFACE
		OPTIONAL :: SELECT
	END SUBROUTINE CGEESX_MKL95
	SUBROUTINE ZGEESX_MKL95(A,W,VS,SELECT,SDIM,RCONDE,RCONDV,INFO)
		! MKL Fortran77 call:
		! ZGEESX(JOBVS,SORT,SELECT,SENSE,N,A,LDA,SDIM,W,VS,LDVS,RCONDE,RCONDV,WORK,LWORK,RWORK,BWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: SDIM
		REAL(WP), INTENT(OUT), OPTIONAL :: RCONDE
		REAL(WP), INTENT(OUT), OPTIONAL :: RCONDV
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VS(:,:)
		INTERFACE
			LOGICAL FUNCTION SELECT(W)
				USE MKL95_PRECISION, ONLY: WP => DP
				REAL(WP), INTENT(IN) :: W
			END FUNCTION SELECT
		END INTERFACE
		OPTIONAL :: SELECT
	END SUBROUTINE ZGEESX_MKL95
END INTERFACE GEESX
INTERFACE GEEV
	SUBROUTINE SGEEV_MKL95(A,WR,WI,VL,VR,INFO)
		! MKL Fortran77 call:
		! SGEEV(JOBVL,JOBVR,N,A,LDA,WR,WI,VL,LDVL,VR,LDVR,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: WR(:)
		REAL(WP), INTENT(OUT) :: WI(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VL(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VR(:,:)
	END SUBROUTINE SGEEV_MKL95
	SUBROUTINE DGEEV_MKL95(A,WR,WI,VL,VR,INFO)
		! MKL Fortran77 call:
		! DGEEV(JOBVL,JOBVR,N,A,LDA,WR,WI,VL,LDVL,VR,LDVR,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: WR(:)
		REAL(WP), INTENT(OUT) :: WI(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VL(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VR(:,:)
	END SUBROUTINE DGEEV_MKL95
	SUBROUTINE CGEEV_MKL95(A,W,VL,VR,INFO)
		! MKL Fortran77 call:
		! CGEEV(JOBVL,JOBVR,N,A,LDA,W,VL,LDVL,VR,LDVR,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VL(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VR(:,:)
	END SUBROUTINE CGEEV_MKL95
	SUBROUTINE ZGEEV_MKL95(A,W,VL,VR,INFO)
		! MKL Fortran77 call:
		! ZGEEV(JOBVL,JOBVR,N,A,LDA,W,VL,LDVL,VR,LDVR,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VL(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VR(:,:)
	END SUBROUTINE ZGEEV_MKL95
END INTERFACE GEEV
INTERFACE GEEVX
	SUBROUTINE SGEEVX_MKL95(A,WR,WI,VL,VR,BALANC,ILO,IHI,SCALE,ABNRM,RCONDE,RCONDV,INFO)
		! MKL Fortran77 call:
		! SGEEVX(BALANC,JOBVL,JOBVR,SENSE,N,A,LDA,WR,WI,VL,LDVL,VR,LDVR,ILO,IHI,SCALE,ABNRM,RCONDE,RCONDV,WORK,LWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: BALANC
		INTEGER, INTENT(OUT), OPTIONAL :: ILO
		INTEGER, INTENT(OUT), OPTIONAL :: IHI
		REAL(WP), INTENT(OUT), OPTIONAL :: ABNRM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: WR(:)
		REAL(WP), INTENT(OUT) :: WI(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VL(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VR(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: SCALE(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RCONDE(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RCONDV(:)
	END SUBROUTINE SGEEVX_MKL95
	SUBROUTINE DGEEVX_MKL95(A,WR,WI,VL,VR,BALANC,ILO,IHI,SCALE,ABNRM,RCONDE,RCONDV,INFO)
		! MKL Fortran77 call:
		! DGEEVX(BALANC,JOBVL,JOBVR,SENSE,N,A,LDA,WR,WI,VL,LDVL,VR,LDVR,ILO,IHI,SCALE,ABNRM,RCONDE,RCONDV,WORK,LWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: BALANC
		INTEGER, INTENT(OUT), OPTIONAL :: ILO
		INTEGER, INTENT(OUT), OPTIONAL :: IHI
		REAL(WP), INTENT(OUT), OPTIONAL :: ABNRM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: WR(:)
		REAL(WP), INTENT(OUT) :: WI(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VL(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VR(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: SCALE(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RCONDE(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RCONDV(:)
	END SUBROUTINE DGEEVX_MKL95
	SUBROUTINE CGEEVX_MKL95(A,W,VL,VR,BALANC,ILO,IHI,SCALE,ABNRM,RCONDE,RCONDV,INFO)
		! MKL Fortran77 call:
		! CGEEVX(BALANC,JOBVL,JOBVR,SENSE,N,A,LDA,W,VL,LDVL,VR,LDVR,ILO,IHI,SCALE,ABNRM,RCONDE,RCONDV,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: BALANC
		INTEGER, INTENT(OUT), OPTIONAL :: ILO
		INTEGER, INTENT(OUT), OPTIONAL :: IHI
		REAL(WP), INTENT(OUT), OPTIONAL :: ABNRM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VL(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VR(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: SCALE(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RCONDE(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RCONDV(:)
	END SUBROUTINE CGEEVX_MKL95
	SUBROUTINE ZGEEVX_MKL95(A,W,VL,VR,BALANC,ILO,IHI,SCALE,ABNRM,RCONDE,RCONDV,INFO)
		! MKL Fortran77 call:
		! ZGEEVX(BALANC,JOBVL,JOBVR,SENSE,N,A,LDA,W,VL,LDVL,VR,LDVR,ILO,IHI,SCALE,ABNRM,RCONDE,RCONDV,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: BALANC
		INTEGER, INTENT(OUT), OPTIONAL :: ILO
		INTEGER, INTENT(OUT), OPTIONAL :: IHI
		REAL(WP), INTENT(OUT), OPTIONAL :: ABNRM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VL(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VR(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: SCALE(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RCONDE(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RCONDV(:)
	END SUBROUTINE ZGEEVX_MKL95
END INTERFACE GEEVX
INTERFACE GESVD
	SUBROUTINE SGESVD_MKL95(A,S,U,VT,WW,JOB,INFO)
		! MKL Fortran77 call:
		! SGESVD(JOBU,JOBVT,M,N,A,LDA,S,U,LDU,VT,LDVT,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: S(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: U(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VT(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL :: WW(:)
	END SUBROUTINE SGESVD_MKL95
	SUBROUTINE DGESVD_MKL95(A,S,U,VT,WW,JOB,INFO)
		! MKL Fortran77 call:
		! DGESVD(JOBU,JOBVT,M,N,A,LDA,S,U,LDU,VT,LDVT,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: S(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: U(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VT(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL :: WW(:)
	END SUBROUTINE DGESVD_MKL95
	SUBROUTINE CGESVD_MKL95(A,S,U,VT,WW,JOB,INFO)
		! MKL Fortran77 call:
		! CGESVD(JOBU,JOBVT,M,N,A,LDA,S,U,LDU,VT,LDVT,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: S(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: U(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VT(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL :: WW(:)
	END SUBROUTINE CGESVD_MKL95
	SUBROUTINE ZGESVD_MKL95(A,S,U,VT,WW,JOB,INFO)
		! MKL Fortran77 call:
		! ZGESVD(JOBU,JOBVT,M,N,A,LDA,S,U,LDU,VT,LDVT,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOB
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: S(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: U(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VT(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL :: WW(:)
	END SUBROUTINE ZGESVD_MKL95
END INTERFACE GESVD
INTERFACE GESDD
	SUBROUTINE SGESDD_MKL95(A,S,U,VT,JOBZ,INFO)
		! MKL Fortran77 call:
		! SGESDD(JOBZ,M,N,A,LDA,S,U,LDU,VT,LDVT,WORK,LWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBZ
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: S(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: U(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VT(:,:)
	END SUBROUTINE SGESDD_MKL95
	SUBROUTINE DGESDD_MKL95(A,S,U,VT,JOBZ,INFO)
		! MKL Fortran77 call:
		! DGESDD(JOBZ,M,N,A,LDA,S,U,LDU,VT,LDVT,WORK,LWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBZ
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: S(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: U(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VT(:,:)
	END SUBROUTINE DGESDD_MKL95
	SUBROUTINE CGESDD_MKL95(A,S,U,VT,JOBZ,INFO)
		! MKL Fortran77 call:
		! CGESDD(JOBZ,M,N,A,LDA,S,U,LDU,VT,LDVT,WORK,LWORK,RWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBZ
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: S(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: U(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VT(:,:)
	END SUBROUTINE CGESDD_MKL95
	SUBROUTINE ZGESDD_MKL95(A,S,U,VT,JOBZ,INFO)
		! MKL Fortran77 call:
		! ZGESDD(JOBZ,M,N,A,LDA,S,U,LDU,VT,LDVT,WORK,LWORK,RWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBZ
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(OUT) :: S(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: U(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VT(:,:)
	END SUBROUTINE ZGESDD_MKL95
END INTERFACE GESDD
INTERFACE GGSVD
	SUBROUTINE SGGSVD_MKL95(A,B,ALPHA,BETA,K,L,U,V,Q,IWORK,INFO)
		! MKL Fortran77 call:
		! SGGSVD(JOBU,JOBV,JOBQ,M,N,P,K,L,A,LDA,B,LDB,ALPHA,BETA,U,LDU,V,LDV,Q,LDQ,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: K
		INTEGER, INTENT(OUT), OPTIONAL :: L
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: ALPHA(:)
		REAL(WP), INTENT(OUT) :: BETA(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: U(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: V(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Q(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IWORK(:)
	END SUBROUTINE SGGSVD_MKL95
	SUBROUTINE DGGSVD_MKL95(A,B,ALPHA,BETA,K,L,U,V,Q,IWORK,INFO)
		! MKL Fortran77 call:
		! DGGSVD(JOBU,JOBV,JOBQ,M,N,P,K,L,A,LDA,B,LDB,ALPHA,BETA,U,LDU,V,LDV,Q,LDQ,WORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: K
		INTEGER, INTENT(OUT), OPTIONAL :: L
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: ALPHA(:)
		REAL(WP), INTENT(OUT) :: BETA(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: U(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: V(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Q(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IWORK(:)
	END SUBROUTINE DGGSVD_MKL95
	SUBROUTINE CGGSVD_MKL95(A,B,ALPHA,BETA,K,L,U,V,Q,IWORK,INFO)
		! MKL Fortran77 call:
		! CGGSVD(JOBU,JOBV,JOBQ,M,N,P,K,L,A,LDA,B,LDB,ALPHA,BETA,U,LDU,V,LDV,Q,LDQ,WORK,RWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: K
		INTEGER, INTENT(OUT), OPTIONAL :: L
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: ALPHA(:)
		REAL(WP), INTENT(OUT) :: BETA(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: U(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: V(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Q(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IWORK(:)
	END SUBROUTINE CGGSVD_MKL95
	SUBROUTINE ZGGSVD_MKL95(A,B,ALPHA,BETA,K,L,U,V,Q,IWORK,INFO)
		! MKL Fortran77 call:
		! ZGGSVD(JOBU,JOBV,JOBQ,M,N,P,K,L,A,LDA,B,LDB,ALPHA,BETA,U,LDU,V,LDV,Q,LDQ,WORK,RWORK,IWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: K
		INTEGER, INTENT(OUT), OPTIONAL :: L
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: ALPHA(:)
		REAL(WP), INTENT(OUT) :: BETA(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: U(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: V(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Q(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IWORK(:)
	END SUBROUTINE ZGGSVD_MKL95
END INTERFACE GGSVD
INTERFACE SYGV
	SUBROUTINE SSYGV_MKL95(A,B,W,ITYPE,JOBZ,UPLO,INFO)
		! MKL Fortran77 call:
		! SSYGV(ITYPE,JOBZ,UPLO,N,A,LDA,B,LDB,W,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBZ
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
	END SUBROUTINE SSYGV_MKL95
	SUBROUTINE DSYGV_MKL95(A,B,W,ITYPE,JOBZ,UPLO,INFO)
		! MKL Fortran77 call:
		! DSYGV(ITYPE,JOBZ,UPLO,N,A,LDA,B,LDB,W,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBZ
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
	END SUBROUTINE DSYGV_MKL95
END INTERFACE SYGV
INTERFACE HEGV
	SUBROUTINE CHEGV_MKL95(A,B,W,ITYPE,JOBZ,UPLO,INFO)
		! MKL Fortran77 call:
		! CHEGV(ITYPE,JOBZ,UPLO,N,A,LDA,B,LDB,W,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBZ
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
	END SUBROUTINE CHEGV_MKL95
	SUBROUTINE ZHEGV_MKL95(A,B,W,ITYPE,JOBZ,UPLO,INFO)
		! MKL Fortran77 call:
		! ZHEGV(ITYPE,JOBZ,UPLO,N,A,LDA,B,LDB,W,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBZ
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
	END SUBROUTINE ZHEGV_MKL95
END INTERFACE HEGV
INTERFACE SYGVD
	SUBROUTINE SSYGVD_MKL95(A,B,W,ITYPE,JOBZ,UPLO,INFO)
		! MKL Fortran77 call:
		! SSYGVD(ITYPE,JOBZ,UPLO,N,A,LDA,B,LDB,W,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBZ
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
	END SUBROUTINE SSYGVD_MKL95
	SUBROUTINE DSYGVD_MKL95(A,B,W,ITYPE,JOBZ,UPLO,INFO)
		! MKL Fortran77 call:
		! DSYGVD(ITYPE,JOBZ,UPLO,N,A,LDA,B,LDB,W,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBZ
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
	END SUBROUTINE DSYGVD_MKL95
END INTERFACE SYGVD
INTERFACE HEGVD
	SUBROUTINE CHEGVD_MKL95(A,B,W,ITYPE,JOBZ,UPLO,INFO)
		! MKL Fortran77 call:
		! CHEGVD(ITYPE,JOBZ,UPLO,N,A,LDA,B,LDB,W,WORK,LWORK,RWORK,LRWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBZ
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
	END SUBROUTINE CHEGVD_MKL95
	SUBROUTINE ZHEGVD_MKL95(A,B,W,ITYPE,JOBZ,UPLO,INFO)
		! MKL Fortran77 call:
		! ZHEGVD(ITYPE,JOBZ,UPLO,N,A,LDA,B,LDB,W,WORK,LWORK,RWORK,LRWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: JOBZ
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
	END SUBROUTINE ZHEGVD_MKL95
END INTERFACE HEGVD
INTERFACE SYGVX
	SUBROUTINE SSYGVX_MKL95(A,B,W,ITYPE,UPLO,Z,VL,VU,IL,IU,M,IFAIL,ABSTOL,INFO)
		! MKL Fortran77 call:
		! SSYGVX(ITYPE,JOBZ,RANGE,UPLO,N,A,LDA,B,LDB,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,WORK,LWORK,IWORK,IFAIL,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAIL(:)
	END SUBROUTINE SSYGVX_MKL95
	SUBROUTINE DSYGVX_MKL95(A,B,W,ITYPE,UPLO,Z,VL,VU,IL,IU,M,IFAIL,ABSTOL,INFO)
		! MKL Fortran77 call:
		! DSYGVX(ITYPE,JOBZ,RANGE,UPLO,N,A,LDA,B,LDB,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,WORK,LWORK,IWORK,IFAIL,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAIL(:)
	END SUBROUTINE DSYGVX_MKL95
END INTERFACE SYGVX
INTERFACE HEGVX
	SUBROUTINE CHEGVX_MKL95(A,B,W,ITYPE,UPLO,Z,VL,VU,IL,IU,M,IFAIL,ABSTOL,INFO)
		! MKL Fortran77 call:
		! CHEGVX(ITYPE,JOBZ,RANGE,UPLO,N,A,LDA,B,LDB,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,WORK,LWORK,RWORK,IWORK,IFAIL,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAIL(:)
	END SUBROUTINE CHEGVX_MKL95
	SUBROUTINE ZHEGVX_MKL95(A,B,W,ITYPE,UPLO,Z,VL,VU,IL,IU,M,IFAIL,ABSTOL,INFO)
		! MKL Fortran77 call:
		! ZHEGVX(ITYPE,JOBZ,RANGE,UPLO,N,A,LDA,B,LDB,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,WORK,LWORK,RWORK,IWORK,IFAIL,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAIL(:)
	END SUBROUTINE ZHEGVX_MKL95
END INTERFACE HEGVX
INTERFACE SPGV
	SUBROUTINE SSPGV_MKL95(A,B,W,ITYPE,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! SSPGV(ITYPE,JOBZ,UPLO,N,A,B,W,Z,LDZ,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE SSPGV_MKL95
	SUBROUTINE DSPGV_MKL95(A,B,W,ITYPE,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! DSPGV(ITYPE,JOBZ,UPLO,N,A,B,W,Z,LDZ,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE DSPGV_MKL95
END INTERFACE SPGV
INTERFACE HPGV
	SUBROUTINE CHPGV_MKL95(A,B,W,ITYPE,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! CHPGV(ITYPE,JOBZ,UPLO,N,A,B,W,Z,LDZ,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE CHPGV_MKL95
	SUBROUTINE ZHPGV_MKL95(A,B,W,ITYPE,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! ZHPGV(ITYPE,JOBZ,UPLO,N,A,B,W,Z,LDZ,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE ZHPGV_MKL95
END INTERFACE HPGV
INTERFACE SPGVD
	SUBROUTINE SSPGVD_MKL95(A,B,W,ITYPE,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! SSPGVD(ITYPE,JOBZ,UPLO,N,A,B,W,Z,LDZ,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE SSPGVD_MKL95
	SUBROUTINE DSPGVD_MKL95(A,B,W,ITYPE,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! DSPGVD(ITYPE,JOBZ,UPLO,N,A,B,W,Z,LDZ,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE DSPGVD_MKL95
END INTERFACE SPGVD
INTERFACE HPGVD
	SUBROUTINE CHPGVD_MKL95(A,B,W,ITYPE,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! CHPGVD(ITYPE,JOBZ,UPLO,N,A,B,W,Z,LDZ,WORK,LWORK,RWORK,LRWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE CHPGVD_MKL95
	SUBROUTINE ZHPGVD_MKL95(A,B,W,ITYPE,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! ZHPGVD(ITYPE,JOBZ,UPLO,N,A,B,W,Z,LDZ,WORK,LWORK,RWORK,LRWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE ZHPGVD_MKL95
END INTERFACE HPGVD
INTERFACE SPGVX
	SUBROUTINE SSPGVX_MKL95(A,B,W,ITYPE,UPLO,Z,VL,VU,IL,IU,M,IFAIL,ABSTOL,INFO)
		! MKL Fortran77 call:
		! SSPGVX(ITYPE,JOBZ,RANGE,UPLO,N,A,B,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,WORK,IWORK,IFAIL,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAIL(:)
	END SUBROUTINE SSPGVX_MKL95
	SUBROUTINE DSPGVX_MKL95(A,B,W,ITYPE,UPLO,Z,VL,VU,IL,IU,M,IFAIL,ABSTOL,INFO)
		! MKL Fortran77 call:
		! DSPGVX(ITYPE,JOBZ,RANGE,UPLO,N,A,B,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,WORK,IWORK,IFAIL,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:)
		REAL(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAIL(:)
	END SUBROUTINE DSPGVX_MKL95
END INTERFACE SPGVX
INTERFACE HPGVX
	SUBROUTINE CHPGVX_MKL95(A,B,W,ITYPE,UPLO,Z,VL,VU,IL,IU,M,IFAIL,ABSTOL,INFO)
		! MKL Fortran77 call:
		! CHPGVX(ITYPE,JOBZ,RANGE,UPLO,N,A,B,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,WORK,RWORK,IWORK,IFAIL,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAIL(:)
	END SUBROUTINE CHPGVX_MKL95
	SUBROUTINE ZHPGVX_MKL95(A,B,W,ITYPE,UPLO,Z,VL,VU,IL,IU,M,IFAIL,ABSTOL,INFO)
		! MKL Fortran77 call:
		! ZHPGVX(ITYPE,JOBZ,RANGE,UPLO,N,A,B,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,WORK,RWORK,IWORK,IFAIL,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(IN), OPTIONAL :: ITYPE
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:)
		COMPLEX(WP), INTENT(INOUT) :: B(:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAIL(:)
	END SUBROUTINE ZHPGVX_MKL95
END INTERFACE HPGVX
INTERFACE SBGV
	SUBROUTINE SSBGV_MKL95(A,B,W,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! SSBGV(JOBZ,UPLO,N,KA,KB,A,LDA,B,LDB,W,Z,LDZ,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE SSBGV_MKL95
	SUBROUTINE DSBGV_MKL95(A,B,W,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! DSBGV(JOBZ,UPLO,N,KA,KB,A,LDA,B,LDB,W,Z,LDZ,WORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE DSBGV_MKL95
END INTERFACE SBGV
INTERFACE HBGV
	SUBROUTINE CHBGV_MKL95(A,B,W,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! CHBGV(JOBZ,UPLO,N,KA,KB,A,LDA,B,LDB,W,Z,LDZ,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE CHBGV_MKL95
	SUBROUTINE ZHBGV_MKL95(A,B,W,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! ZHBGV(JOBZ,UPLO,N,KA,KB,A,LDA,B,LDB,W,Z,LDZ,WORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE ZHBGV_MKL95
END INTERFACE HBGV
INTERFACE SBGVD
	SUBROUTINE SSBGVD_MKL95(A,B,W,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! SSBGVD(JOBZ,UPLO,N,KA,KB,A,LDA,B,LDB,W,Z,LDZ,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE SSBGVD_MKL95
	SUBROUTINE DSBGVD_MKL95(A,B,W,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! DSBGVD(JOBZ,UPLO,N,KA,KB,A,LDA,B,LDB,W,Z,LDZ,WORK,LWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE DSBGVD_MKL95
END INTERFACE SBGVD
INTERFACE HBGVD
	SUBROUTINE CHBGVD_MKL95(A,B,W,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! CHBGVD(JOBZ,UPLO,N,KA,KB,A,LDA,B,LDB,W,Z,LDZ,WORK,LWORK,RWORK,LRWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE CHBGVD_MKL95
	SUBROUTINE ZHBGVD_MKL95(A,B,W,UPLO,Z,INFO)
		! MKL Fortran77 call:
		! ZHBGVD(JOBZ,UPLO,N,KA,KB,A,LDA,B,LDB,W,Z,LDZ,WORK,LWORK,RWORK,LRWORK,IWORK,LIWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
	END SUBROUTINE ZHBGVD_MKL95
END INTERFACE HBGVD
INTERFACE SBGVX
	SUBROUTINE SSBGVX_MKL95(A,B,W,UPLO,Z,VL,VU,IL,IU,M,IFAIL,Q,ABSTOL,INFO)
		! MKL Fortran77 call:
		! SSBGVX(JOBZ,RANGE,UPLO,N,KA,KB,A,LDA,B,LDB,Q,LDQ,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,WORK,IWORK,IFAIL,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAIL(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Q(:,:)
	END SUBROUTINE SSBGVX_MKL95
	SUBROUTINE DSBGVX_MKL95(A,B,W,UPLO,Z,VL,VU,IL,IU,M,IFAIL,Q,ABSTOL,INFO)
		! MKL Fortran77 call:
		! DSBGVX(JOBZ,RANGE,UPLO,N,KA,KB,A,LDA,B,LDB,Q,LDQ,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,WORK,IWORK,IFAIL,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAIL(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: Q(:,:)
	END SUBROUTINE DSBGVX_MKL95
END INTERFACE SBGVX
INTERFACE HBGVX
	SUBROUTINE CHBGVX_MKL95(A,B,W,UPLO,Z,VL,VU,IL,IU,M,IFAIL,Q,ABSTOL,INFO)
		! MKL Fortran77 call:
		! CHBGVX(JOBZ,RANGE,UPLO,N,KA,KB,A,LDA,B,LDB,Q,LDQ,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,WORK,RWORK,IWORK,IFAIL,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAIL(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Q(:,:)
	END SUBROUTINE CHBGVX_MKL95
	SUBROUTINE ZHBGVX_MKL95(A,B,W,UPLO,Z,VL,VU,IL,IU,M,IFAIL,Q,ABSTOL,INFO)
		! MKL Fortran77 call:
		! ZHBGVX(JOBZ,RANGE,UPLO,N,KA,KB,A,LDA,B,LDB,Q,LDQ,VL,VU,IL,IU,ABSTOL,M,W,Z,LDZ,WORK,RWORK,IWORK,IFAIL,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: UPLO
		REAL(WP), INTENT(IN), OPTIONAL :: VL
		REAL(WP), INTENT(IN), OPTIONAL :: VU
		INTEGER, INTENT(IN), OPTIONAL :: IL
		INTEGER, INTENT(IN), OPTIONAL :: IU
		INTEGER, INTENT(OUT), OPTIONAL :: M
		REAL(WP), INTENT(IN), OPTIONAL :: ABSTOL
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: W(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Z(:,:)
		INTEGER, INTENT(OUT), OPTIONAL, TARGET :: IFAIL(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: Q(:,:)
	END SUBROUTINE ZHBGVX_MKL95
END INTERFACE HBGVX
INTERFACE GGES
	SUBROUTINE SGGES_MKL95(A,B,ALPHAR,ALPHAI,BETA,VSL,VSR,SELECT,SDIM,INFO)
		! MKL Fortran77 call:
		! SGGES(JOBVSL,JOBVSR,SORT,SELECT,N,A,LDA,B,LDB,SDIM,ALPHAR,ALPHAI,BETA,VSL,LDVSL,VSR,LDVSR,WORK,LWORK,BWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: SDIM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: ALPHAR(:)
		REAL(WP), INTENT(OUT) :: ALPHAI(:)
		REAL(WP), INTENT(OUT) :: BETA(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VSL(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VSR(:,:)
		INTERFACE
			LOGICAL FUNCTION SELECT(ALPHAR,ALPHAI,BETA)
				USE MKL95_PRECISION, ONLY: WP => SP
				REAL(WP), INTENT(IN) :: ALPHAR,ALPHAI,BETA
			END FUNCTION SELECT
		END INTERFACE
		OPTIONAL :: SELECT
	END SUBROUTINE SGGES_MKL95
	SUBROUTINE DGGES_MKL95(A,B,ALPHAR,ALPHAI,BETA,VSL,VSR,SELECT,SDIM,INFO)
		! MKL Fortran77 call:
		! DGGES(JOBVSL,JOBVSR,SORT,SELECT,N,A,LDA,B,LDB,SDIM,ALPHAR,ALPHAI,BETA,VSL,LDVSL,VSR,LDVSR,WORK,LWORK,BWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: SDIM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: ALPHAR(:)
		REAL(WP), INTENT(OUT) :: ALPHAI(:)
		REAL(WP), INTENT(OUT) :: BETA(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VSL(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VSR(:,:)
		INTERFACE
			LOGICAL FUNCTION SELECT(ALPHAR,ALPHAI,BETA)
				USE MKL95_PRECISION, ONLY: WP => DP
				REAL(WP), INTENT(IN) :: ALPHAR,ALPHAI,BETA
			END FUNCTION SELECT
		END INTERFACE
		OPTIONAL :: SELECT
	END SUBROUTINE DGGES_MKL95
	SUBROUTINE CGGES_MKL95(A,B,ALPHA,BETA,VSL,VSR,SELECT,SDIM,INFO)
		! MKL Fortran77 call:
		! CGGES(JOBVSL,JOBVSR,SORT,SELECT,N,A,LDA,B,LDB,SDIM,ALPHA,BETA,VSL,LDVSL,VSR,LDVSR,WORK,LWORK,RWORK,BWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: SDIM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: ALPHA(:)
		COMPLEX(WP), INTENT(OUT) :: BETA(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VSL(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VSR(:,:)
		INTERFACE
			LOGICAL FUNCTION SELECT(ALPHA,BETA)
				USE MKL95_PRECISION, ONLY: WP => SP
				REAL(WP), INTENT(IN) :: ALPHA,BETA
			END FUNCTION SELECT
		END INTERFACE
		OPTIONAL :: SELECT
	END SUBROUTINE CGGES_MKL95
	SUBROUTINE ZGGES_MKL95(A,B,ALPHA,BETA,VSL,VSR,SELECT,SDIM,INFO)
		! MKL Fortran77 call:
		! ZGGES(JOBVSL,JOBVSR,SORT,SELECT,N,A,LDA,B,LDB,SDIM,ALPHA,BETA,VSL,LDVSL,VSR,LDVSR,WORK,LWORK,RWORK,BWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: SDIM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: ALPHA(:)
		COMPLEX(WP), INTENT(OUT) :: BETA(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VSL(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VSR(:,:)
		INTERFACE
			LOGICAL FUNCTION SELECT(ALPHA,BETA)
				USE MKL95_PRECISION, ONLY: WP => DP
				REAL(WP), INTENT(IN) :: ALPHA,BETA
			END FUNCTION SELECT
		END INTERFACE
		OPTIONAL :: SELECT
	END SUBROUTINE ZGGES_MKL95
END INTERFACE GGES
INTERFACE GGESX
	SUBROUTINE SGGESX_MKL95(A,B,ALPHAR,ALPHAI,BETA,VSL,VSR,SELECT,SDIM,RCONDE,RCONDV,INFO)
		! MKL Fortran77 call:
		! SGGESX(JOBVSL,JOBVSR,SORT,SELECT,SENSE,N,A,LDA,B,LDB,SDIM,ALPHAR,ALPHAI,BETA,VSL,LDVSL,VSR,LDVSR,RCONDE,RCONDV,WORK,LWORK,IWORK,LIWORK,BWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: SDIM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: ALPHAR(:)
		REAL(WP), INTENT(OUT) :: ALPHAI(:)
		REAL(WP), INTENT(OUT) :: BETA(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VSL(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VSR(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RCONDE(2)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RCONDV(2)
		INTERFACE
			LOGICAL FUNCTION SELECT(ALPHAR,ALPHAI,BETA)
				USE MKL95_PRECISION, ONLY: WP => SP
				REAL(WP), INTENT(IN) :: ALPHAR,ALPHAI,BETA
			END FUNCTION SELECT
		END INTERFACE
		OPTIONAL :: SELECT
	END SUBROUTINE SGGESX_MKL95
	SUBROUTINE DGGESX_MKL95(A,B,ALPHAR,ALPHAI,BETA,VSL,VSR,SELECT,SDIM,RCONDE,RCONDV,INFO)
		! MKL Fortran77 call:
		! DGGESX(JOBVSL,JOBVSR,SORT,SELECT,SENSE,N,A,LDA,B,LDB,SDIM,ALPHAR,ALPHAI,BETA,VSL,LDVSL,VSR,LDVSR,RCONDE,RCONDV,WORK,LWORK,IWORK,LIWORK,BWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: SDIM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: ALPHAR(:)
		REAL(WP), INTENT(OUT) :: ALPHAI(:)
		REAL(WP), INTENT(OUT) :: BETA(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VSL(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VSR(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RCONDE(2)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RCONDV(2)
		INTERFACE
			LOGICAL FUNCTION SELECT(ALPHAR,ALPHAI,BETA)
				USE MKL95_PRECISION, ONLY: WP => DP
				REAL(WP), INTENT(IN) :: ALPHAR,ALPHAI,BETA
			END FUNCTION SELECT
		END INTERFACE
		OPTIONAL :: SELECT
	END SUBROUTINE DGGESX_MKL95
	SUBROUTINE CGGESX_MKL95(A,B,ALPHA,BETA,VSL,VSR,SELECT,SDIM,RCONDE,RCONDV,INFO)
		! MKL Fortran77 call:
		! CGGESX(JOBVSL,JOBVSR,SORT,SELECT,SENSE,N,A,LDA,B,LDB,SDIM,ALPHA,BETA,VSL,LDVSL,VSR,LDVSR,RCONDE,RCONDV,WORK,LWORK,RWORK,IWORK,LIWORK,BWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: SDIM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: ALPHA(:)
		COMPLEX(WP), INTENT(OUT) :: BETA(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VSL(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VSR(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RCONDE(2)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RCONDV(2)
		INTERFACE
			LOGICAL FUNCTION SELECT(ALPHA,BETA)
				USE MKL95_PRECISION, ONLY: WP => SP
				REAL(WP), INTENT(IN) :: ALPHA,BETA
			END FUNCTION SELECT
		END INTERFACE
		OPTIONAL :: SELECT
	END SUBROUTINE CGGESX_MKL95
	SUBROUTINE ZGGESX_MKL95(A,B,ALPHA,BETA,VSL,VSR,SELECT,SDIM,RCONDE,RCONDV,INFO)
		! MKL Fortran77 call:
		! ZGGESX(JOBVSL,JOBVSR,SORT,SELECT,SENSE,N,A,LDA,B,LDB,SDIM,ALPHA,BETA,VSL,LDVSL,VSR,LDVSR,RCONDE,RCONDV,WORK,LWORK,RWORK,IWORK,LIWORK,BWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: SDIM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: ALPHA(:)
		COMPLEX(WP), INTENT(OUT) :: BETA(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VSL(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VSR(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RCONDE(2)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RCONDV(2)
		INTERFACE
			LOGICAL FUNCTION SELECT(ALPHA,BETA)
				USE MKL95_PRECISION, ONLY: WP => DP
				REAL(WP), INTENT(IN) :: ALPHA,BETA
			END FUNCTION SELECT
		END INTERFACE
		OPTIONAL :: SELECT
	END SUBROUTINE ZGGESX_MKL95
END INTERFACE GGESX
INTERFACE GGEV
	SUBROUTINE SGGEV_MKL95(A,B,ALPHAR,ALPHAI,BETA,VL,VR,INFO)
		! MKL Fortran77 call:
		! SGGEV(JOBVL,JOBVR,N,A,LDA,B,LDB,ALPHAR,ALPHAI,BETA,VL,LDVL,VR,LDVR,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: ALPHAR(:)
		REAL(WP), INTENT(OUT) :: ALPHAI(:)
		REAL(WP), INTENT(OUT) :: BETA(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VL(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VR(:,:)
	END SUBROUTINE SGGEV_MKL95
	SUBROUTINE DGGEV_MKL95(A,B,ALPHAR,ALPHAI,BETA,VL,VR,INFO)
		! MKL Fortran77 call:
		! DGGEV(JOBVL,JOBVR,N,A,LDA,B,LDB,ALPHAR,ALPHAI,BETA,VL,LDVL,VR,LDVR,WORK,LWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: ALPHAR(:)
		REAL(WP), INTENT(OUT) :: ALPHAI(:)
		REAL(WP), INTENT(OUT) :: BETA(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VL(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VR(:,:)
	END SUBROUTINE DGGEV_MKL95
	SUBROUTINE CGGEV_MKL95(A,B,ALPHA,BETA,VL,VR,INFO)
		! MKL Fortran77 call:
		! CGGEV(JOBVL,JOBVR,N,A,LDA,B,LDB,ALPHA,BETA,VL,LDVL,VR,LDVR,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: ALPHA(:)
		COMPLEX(WP), INTENT(OUT) :: BETA(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VL(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VR(:,:)
	END SUBROUTINE CGGEV_MKL95
	SUBROUTINE ZGGEV_MKL95(A,B,ALPHA,BETA,VL,VR,INFO)
		! MKL Fortran77 call:
		! ZGGEV(JOBVL,JOBVR,N,A,LDA,B,LDB,ALPHA,BETA,VL,LDVL,VR,LDVR,WORK,LWORK,RWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: ALPHA(:)
		COMPLEX(WP), INTENT(OUT) :: BETA(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VL(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VR(:,:)
	END SUBROUTINE ZGGEV_MKL95
END INTERFACE GGEV
INTERFACE GGEVX
	SUBROUTINE SGGEVX_MKL95(A,B,ALPHAR,ALPHAI,BETA,VL,VR,BALANC,ILO,IHI,LSCALE,RSCALE,ABNRM,BBNRM,RCONDE,RCONDV,INFO)
		! MKL Fortran77 call:
		! SGGEVX(BALANC,JOBVL,JOBVR,SENSE,N,A,LDA,B,LDB,ALPHAR,ALPHAI,BETA,VL,LDVL,VR,LDVR,ILO,IHI,LSCALE,RSCALE,ABNRM,BBNRM,RCONDE,RCONDV,WORK,LWORK,IWORK,BWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: BALANC
		INTEGER, INTENT(OUT), OPTIONAL :: ILO
		INTEGER, INTENT(OUT), OPTIONAL :: IHI
		REAL(WP), INTENT(OUT), OPTIONAL :: ABNRM
		REAL(WP), INTENT(OUT), OPTIONAL :: BBNRM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: ALPHAR(:)
		REAL(WP), INTENT(OUT) :: ALPHAI(:)
		REAL(WP), INTENT(OUT) :: BETA(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VL(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VR(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: LSCALE(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RSCALE(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RCONDE(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RCONDV(:)
	END SUBROUTINE SGGEVX_MKL95
	SUBROUTINE DGGEVX_MKL95(A,B,ALPHAR,ALPHAI,BETA,VL,VR,BALANC,ILO,IHI,LSCALE,RSCALE,ABNRM,BBNRM,RCONDE,RCONDV,INFO)
		! MKL Fortran77 call:
		! DGGEVX(BALANC,JOBVL,JOBVR,SENSE,N,A,LDA,B,LDB,ALPHAR,ALPHAI,BETA,VL,LDVL,VR,LDVR,ILO,IHI,LSCALE,RSCALE,ABNRM,BBNRM,RCONDE,RCONDV,WORK,LWORK,IWORK,BWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: BALANC
		INTEGER, INTENT(OUT), OPTIONAL :: ILO
		INTEGER, INTENT(OUT), OPTIONAL :: IHI
		REAL(WP), INTENT(OUT), OPTIONAL :: ABNRM
		REAL(WP), INTENT(OUT), OPTIONAL :: BBNRM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		REAL(WP), INTENT(INOUT) :: A(:,:)
		REAL(WP), INTENT(INOUT) :: B(:,:)
		REAL(WP), INTENT(OUT) :: ALPHAR(:)
		REAL(WP), INTENT(OUT) :: ALPHAI(:)
		REAL(WP), INTENT(OUT) :: BETA(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VL(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: VR(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: LSCALE(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RSCALE(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RCONDE(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RCONDV(:)
	END SUBROUTINE DGGEVX_MKL95
	SUBROUTINE CGGEVX_MKL95(A,B,ALPHA,BETA,VL,VR,BALANC,ILO,IHI,LSCALE,RSCALE,ABNRM,BBNRM,RCONDE,RCONDV,INFO)
		! MKL Fortran77 call:
		! CGGEVX(BALANC,JOBVL,JOBVR,SENSE,N,A,LDA,B,LDB,ALPHA,BETA,VL,LDVL,VR,LDVR,ILO,IHI,LSCALE,RSCALE,ABNRM,BBNRM,RCONDE,RCONDV,WORK,LWORK,RWORK,IWORK,BWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => SP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: BALANC
		INTEGER, INTENT(OUT), OPTIONAL :: ILO
		INTEGER, INTENT(OUT), OPTIONAL :: IHI
		REAL(WP), INTENT(OUT), OPTIONAL :: ABNRM
		REAL(WP), INTENT(OUT), OPTIONAL :: BBNRM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: ALPHA(:)
		COMPLEX(WP), INTENT(OUT) :: BETA(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VL(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VR(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: LSCALE(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RSCALE(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RCONDE(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RCONDV(:)
	END SUBROUTINE CGGEVX_MKL95
	SUBROUTINE ZGGEVX_MKL95(A,B,ALPHA,BETA,VL,VR,BALANC,ILO,IHI,LSCALE,RSCALE,ABNRM,BBNRM,RCONDE,RCONDV,INFO)
		! MKL Fortran77 call:
		! ZGGEVX(BALANC,JOBVL,JOBVR,SENSE,N,A,LDA,B,LDB,ALPHA,BETA,VL,LDVL,VR,LDVR,ILO,IHI,LSCALE,RSCALE,ABNRM,BBNRM,RCONDE,RCONDV,WORK,LWORK,RWORK,IWORK,BWORK,INFO)
		USE MKL95_PRECISION, ONLY: WP => DP
		CHARACTER(LEN=1), INTENT(IN), OPTIONAL :: BALANC
		INTEGER, INTENT(OUT), OPTIONAL :: ILO
		INTEGER, INTENT(OUT), OPTIONAL :: IHI
		REAL(WP), INTENT(OUT), OPTIONAL :: ABNRM
		REAL(WP), INTENT(OUT), OPTIONAL :: BBNRM
		INTEGER, INTENT(OUT), OPTIONAL :: INFO
		COMPLEX(WP), INTENT(INOUT) :: A(:,:)
		COMPLEX(WP), INTENT(INOUT) :: B(:,:)
		COMPLEX(WP), INTENT(OUT) :: ALPHA(:)
		COMPLEX(WP), INTENT(OUT) :: BETA(:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VL(:,:)
		COMPLEX(WP), INTENT(OUT), OPTIONAL, TARGET :: VR(:,:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: LSCALE(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RSCALE(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RCONDE(:)
		REAL(WP), INTENT(OUT), OPTIONAL, TARGET :: RCONDV(:)
	END SUBROUTINE ZGGEVX_MKL95
END INTERFACE GGEVX
END MODULE MKL95_LAPACK
